
//
// @(#)btyaccpar, based on byacc 1.8 (Berkeley)
// Parser skeleton modified for use in the Firebird project by Nickolay Samofatov
//
#define YYBTYACC 1

#include "firebird.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../dsql/Nodes.h"
#include "../dsql/AggNodes.h"
#include "../dsql/DdlNodes.h"
#include "../dsql/BoolNodes.h"
#include "../dsql/ExprNodes.h"
#include "../dsql/PackageNodes.h"
#include "../dsql/StmtNodes.h"
#include "../dsql/WinNodes.h"
#include "../jrd/RecordSourceNodes.h"
#include "../common/classes/Nullable.h"
#include "gen/parse.h"
#include "../dsql/Parser.h"

#define YYERROR_DETAILED

/*
*	PROGRAM:	Dynamic SQL runtime support
*	MODULE:		parse.y
*	DESCRIPTION:	Dynamic SQL parser
*
* The contents of this file are subject to the Interbase Public
* License Version 1.0 (the "License"); you may not use this file
* except in compliance with the License. You may obtain a copy
* of the License at http://www.Inprise.com/IPL.html
*
* Software distributed under the License is distributed on an
* "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
* or implied. See the License for the specific language governing
* rights and limitations under the License.
*
* The Original Code was created by Inprise Corporation
* and its predecessors. Portions created by Inprise Corporation are
* Copyright (C) Inprise Corporation.
*
* All Rights Reserved.
* Contributor(s): ______________________________________.
*
* 2002-02-24 Sean Leyne - Code Cleanup of old Win 3.1 port (WINDOWS_ONLY)
* 2001.05.20 Neil McCalden: Allow a udf to be used in a 'group by' clause.
* 2001.05.30 Claudio Valderrama: DROP TABLE and DROP VIEW lead now to two
*   different node types so DDL can tell which is which.
* 2001.06.13 Claudio Valderrama: SUBSTRING is being surfaced.
* 2001.06.30 Claudio valderrama: Feed (line,column) for each node. See node.h.
* 2001.07.10 Claudio Valderrama: Better (line,column) report and "--" for comments.
* 2001.07.28 John Bellardo: Changes to support parsing LIMIT and FIRST
* 2001.08.03 John Bellardo: Finalized syntax for LIMIT, change LIMIT to SKIP
* 2001.08.05 Claudio Valderrama: closed Bug #448062 and other spaces that appear
*   in rdb$*_source fields when altering domains plus one unexpected null pointer.
* 2001.08.12 Claudio Valderrama: adjust SUBSTRING's starting pos argument here
*   and not in gen.c; this closes Bug #450301.
* 2001.10.01 Claudio Valderrama: enable explicit GRANT...to ROLE role_name.
* 2001.10.06 Claudio Valderrama: Honor explicit USER keyword in GRANTs and REVOKEs.
* 2002.07.05 Mark O'Donohue: change keyword DEBUG to KW_DEBUG to avoid
*			clashes with normal DEBUG macro.
* 2002.07.30 Arno Brinkman:
* 2002.07.30 	Let IN predicate handle value_expressions
* 2002.07.30 	tokens CASE, NULLIF, COALESCE added
* 2002.07.30 	See block < CASE expression > what is added to value as case_expression
* 2002.07.30 	function is split up into aggregate_function, numeric_value_function, string_value_function, generate_value_function
* 2002.07.30 	new group_by_function and added to grp_column_elem
* 2002.07.30 	cast removed from function and added as cast_specification to value
* 2002.08.04 Claudio Valderrama: allow declaring and defining variables at the same time
* 2002.08.04 Dmitry Yemanov: ALTER VIEW
* 2002.08.06 Arno Brinkman: ordinal added to grp_column_elem for using positions in group by
* 2002.08.07 Dmitry Yemanov: INT64/LARGEINT are replaced with BIGINT and available in dialect 3 only
* 2002.08.31 Dmitry Yemanov: allowed user-defined index names for PK/FK/UK constraints
* 2002.09.01 Dmitry Yemanov: RECREATE VIEW
* 2002.09.28 Dmitry Yemanov: Reworked internal_info stuff, enhanced
*							exception handling in SPs/triggers,
*							implemented ROWS_AFFECTED system variable
* 2002.10.21 Nickolay Samofatov: Added support for explicit pessimistic locks
* 2002.10.29 Nickolay Samofatov: Added support for savepoints
* 2002.12.03 Dmitry Yemanov: Implemented ORDER BY clause in subqueries.
* 2002.12.18 Dmitry Yemanov: Added support for SQL-compliant labels and LEAVE statement
* 2002.12.28 Dmitry Yemanov: Added support for parametrized events.
* 2003.01.14 Dmitry Yemanov: Fixed bug with cursors in triggers.
* 2003.01.15 Dmitry Yemanov: Added support for runtime trigger action checks.
* 2003.02.10 Mike Nordell  : Undefined Microsoft introduced macros to get a clean compile.
* 2003.05.24 Nickolay Samofatov: Make SKIP and FIRST non-reserved keywords
* 2003.06.13 Nickolay Samofatov: Make INSERTING/UPDATING/DELETING non-reserved keywords
* 2003.07.01 Blas Rodriguez Somoza: Change DEBUG and IN to avoid conflicts in win32 build/bison
* 2003.08.11 Arno Brinkman: Changed GROUP BY to support all expressions and added "AS" support
*						   with table alias. Also removed group_by_function and ordinal.
* 2003.08.14 Arno Brinkman: Added support for derived tables.
* 2003.10.05 Dmitry Yemanov: Added support for explicit cursors in PSQL.
* 2004.01.16 Vlad Horsun: added support for default parameters and
*   EXECUTE BLOCK statement
* Adriano dos Santos Fernandes
*/

#include "firebird.h"
#include "dyn_consts.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "gen/iberror.h"
#include "../dsql/dsql.h"
#include "../jrd/ibase.h"
#include "../jrd/flags.h"
#include "../jrd/jrd.h"
#include "../jrd/DataTypeUtil.h"
#include "../dsql/errd_proto.h"
#include "../dsql/make_proto.h"
#include "../yvalve/keywords.h"
#include "../yvalve/gds_proto.h"
#include "../jrd/err_proto.h"
#include "../common/intlobj_new.h"
#include "../jrd/Attachment.h"
#include "../common/StatusArg.h"

/* This is needed here to provide backward compatibility when working with SSPI plugin*/
#include "../auth/trusted/AuthSspi.h"

/* since UNIX isn't standard, we have to define*/
/* stuff which is in <limits.h> (which isn't available on all UNIXes...*/

const long SHRT_POS_MAX			= 32767;
const long SHRT_UNSIGNED_MAX	= 65535;
const long SHRT_NEG_MAX			= 32768;
const int POSITIVE	= 0;
const int NEGATIVE	= 1;
const int UNSIGNED	= 2;

/*const int MIN_CACHE_BUFFERS	= 250;*/
/*const int DEF_CACHE_BUFFERS	= 1000;*/

#define YYSTYPE YYSTYPE
#if defined(DEBUG) || defined(DEV_BUILD)
#define YYDEBUG		1
#endif

#define YYREDUCEPOSNFUNC yyReducePosn
#define YYREDUCEPOSNFUNCARG NULL


/* ASF: Inherited attributes (aka rule parameters) are executed even when in trial mode, but action*/
/* rules ({}) are executed only when in full parse mode. NOTRIAL should be used to avoid segfaults*/
/* due to accessing invalid pointers in parameters (not yet returned from action rules).*/
#define NOTRIAL(x) (yytrial ? NULL : (x))


inline unsigned trigger_type_suffix(const unsigned slot1, const unsigned slot2, const unsigned slot3)
{
	return ((slot1 << 1) | (slot2 << 3) | (slot3 << 5));
}


#include "../dsql/chars.h"

using namespace Jrd;
using namespace Firebird;

static int yylhs[] = {                                        -1,
    0,    0,    1,    1,    1,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    4,    4,    4,    4,  552,  553,
    5,  554,  556,    6,    6,    6,    6,    6,    6,    6,
  557,    6,    6,  558,  559,    6,    7,    7,    7,    7,
    7,    7,    7,    7,    7,    7,    7,    7,    7,  555,
  555,    8,    8,  560,    8,    9,  561,    9,   10,   11,
   12,   12,   12,   12,   12,   13,   13,  562,  562,   14,
   14,   15,   15,   15,   16,   16,   17,   17,   18,   18,
   18,   19,   19,   20,   20,   21,   21,  563,  563,   22,
   22,  564,   23,  565,  566,   24,   24,   24,   24,   24,
   24,   24,  567,   24,   24,  568,   24,   24,   25,   25,
   26,   26,  569,   27,   27,  570,   27,   27,   28,   28,
   28,   28,   28,   28,   29,   29,   29,   30,   30,   31,
   32,   32,   33,   34,   35,   35,  571,  572,  573,   36,
   37,   37,   37,   38,  574,   38,  575,   38,   39,  576,
   39,   40,   41,   41,   41,   41,  577,   42,  578,   42,
   43,   43,   44,   44,   44,   44,   44,   45,   46,   46,
   47,   48,  579,  580,   48,   48,   48,   48,   48,   48,
   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,
   48,  581,   48,  582,   48,   49,   50,   50,   50,   50,
   50,   50,   50,   50,   50,   50,   50,   51,   52,   52,
   52,   52,   52,   52,   52,   52,   52,   52,   52,   53,
   54,   55,   56,   56,   57,   57,   57,  584,  585,   58,
   59,   59,   59,   60,   60,   61,   61,   62,  588,   62,
   63,   63,  590,  591,   64,   65,   65,   66,  592,   66,
   67,   67,  589,  589,   68,   69,   69,  593,   70,   71,
   72,   72,   73,   74,   74,  594,  594,   75,   76,   76,
   77,   78,   78,   79,   79,   80,   81,   81,   81,   82,
  595,  596,  597,  598,   83,   84,   84,   84,   85,   85,
   86,  599,   86,   87,   87,   87,   88,   88,   89,   89,
   90,   90,   91,   91,   92,  600,  601,  602,   93,  586,
  586,   94,   95,   95,   96,  603,   96,   97,   97,   97,
   97,   97,   97,   97,   97,   98,   98,   99,   99,  100,
  100,  100,  100,  604,  605,  101,  102,  102,  103,  606,
  103,  104,  104,  607,  607,  607,  587,  587,  587,  608,
  609,  105,  610,  106,  107,  107,  107,  108,  108,  108,
  611,  109,  612,  109,  613,  110,  110,  614,  615,  111,
  616,  111,  111,  111,  112,  113,  113,  114,  617,  617,
  618,  583,  583,  115,  115,  116,  116,  117,  117,  117,
  117,  118,  118,  118,  118,  118,  118,  119,  619,  119,
  120,  620,  120,  121,  122,  122,  122,  122,  122,  621,
  123,  124,  124,  125,  125,  125,  125,  126,  126,  127,
  127,  127,  127,  127,  128,  129,  130,  130,  130,  130,
  131,  131,  132,  133,  622,  623,  624,  134,  135,  136,
  137,  137,  137,  138,  138,  139,  139,  140,  141,  141,
  142,  143,  144,  144,  144,  145,  145,  146,  147,  625,
  148,  149,  149,  149,  150,  150,  151,  151,  152,  153,
  154,  155,  155,  156,  156,  157,  157,  158,  159,  160,
  160,  161,  161,  162,  162,  163,  163,  163,  163,  164,
  164,  165,  165,  166,  627,  628,  629,  166,  630,  166,
  167,  167,  168,  626,  626,  169,  170,  170,  170,  171,
  171,  172,  173,  173,  173,  174,  174,  175,  175,  176,
  176,  176,  176,  176,  176,  176,  176,  176,  176,  176,
  176,  176,  176,  176,  176,  176,  176,  176,  176,  177,
  177,  178,  178,  178,  178,  178,  179,  180,  180,  180,
  181,  631,  632,  182,  183,  183,  184,  184,  185,  633,
  634,  635,  186,  187,  188,  189,  636,  189,  189,  190,
  190,  191,  192,  192,  193,  194,  194,  637,  195,  195,
  196,  196,  196,  196,  196,  196,  196,  196,  197,  197,
  198,  199,  200,  201,  202,  202,  202,  202,  203,  204,
  204,  205,  205,  206,  207,  207,  208,  208,  209,  210,
  210,  638,  639,  211,  640,  212,  641,  212,  213,  213,
  213,  213,  213,  214,  214,  214,  215,  216,  642,  643,
  217,  217,  218,  218,  218,  218,  218,  218,  219,  219,
  220,  220,  220,  221,  221,  221,  644,  222,  223,  223,
  224,  224,  225,  226,  227,  228,  229,  229,  230,  230,
  230,  230,  230,  230,  231,  232,  232,  232,  233,  233,
  233,  234,  234,  234,  234,  234,  235,  235,  236,  236,
  236,  236,  236,  236,  236,  236,  236,  236,  236,  236,
  236,  236,  236,  236,  236,  236,  236,  236,  236,  236,
  236,  236,  236,  236,  236,  236,  236,  236,  236,  236,
  236,  236,  236,  236,  236,  236,  236,  236,  236,  236,
  236,  236,  236,  237,  237,  238,  238,  238,  238,  238,
  238,  238,  238,  238,  238,  238,  238,  238,  238,  238,
  239,  239,  240,  240,  241,  645,  646,  241,  241,  241,
  241,  241,  647,  241,  241,  241,  241,  241,  241,  241,
  241,  241,  241,  241,  241,  241,  648,  649,  242,  243,
  650,  243,  244,  244,  244,  244,  244,  244,  244,  244,
  244,  245,  651,  245,  246,  246,  246,  246,  246,  246,
  246,  246,  246,  246,  246,  246,  246,  246,  247,  248,
  248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
  248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
  248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
  248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
  248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
  248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
  248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
  248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
  248,  652,  652,  249,  249,  250,  250,  250,  251,  251,
  252,  253,  253,  254,  254,  255,  256,  256,  257,  257,
  653,  258,  258,  258,  258,  258,  258,  654,  258,  258,
  258,  258,  259,  259,  260,  260,  260,  261,  261,  262,
  263,  263,  263,  263,  263,  263,  263,  263,  263,  263,
  263,  263,  263,  263,  263,  263,  263,  263,  263,  263,
  263,  264,  264,  264,  265,  265,  266,  266,  267,  267,
  268,  268,  268,  269,  269,  270,  270,  271,  271,  272,
  272,  273,  273,  274,  274,  274,  274,  274,  274,  274,
  274,  274,  274,  655,  655,  656,  657,  275,  275,  275,
  275,  276,  276,  277,  277,  277,  278,  278,  279,  279,
  279,  280,  280,  280,  660,  660,  660,  659,  659,  658,
  658,  658,  281,  281,  282,  282,  282,  661,  661,  283,
  283,  283,  283,  284,  284,  285,  285,  285,  286,  287,
  288,  288,  289,  663,  663,  290,  291,  662,  662,  292,
  292,  664,  664,  665,  666,  293,  294,  294,  295,  295,
  296,  667,  296,  297,  297,  297,  297,  297,  297,  297,
  297,  297,  668,  669,  297,  298,  298,  299,  299,  299,
  300,  300,  300,  301,  301,  301,  302,  302,  302,  303,
  303,  304,  304,  305,  306,  306,  307,  307,  308,  309,
  309,  309,  309,  309,  309,  310,  311,  311,  311,  311,
  311,  311,  311,  311,  311,  311,  311,  311,  311,  312,
  313,  313,  313,  314,  314,  314,  315,  316,  316,  317,
  317,  318,  319,  319,  320,  320,  321,  321,  322,  322,
  323,  323,  323,  324,  324,  325,  326,  327,  328,  328,
  328,  329,  330,  331,  331,  331,  331,  332,  332,  332,
  333,  333,  333,  334,  334,  335,  335,  336,  336,  337,
  337,  338,  338,  672,  672,  339,  340,  340,  341,  341,
  342,  342,  342,  343,  344,  344,  345,  345,  346,  346,
  347,  347,  347,  348,  349,  350,  350,  351,  352,  353,
  353,  353,  353,  354,  354,  355,  355,  356,  357,  357,
  357,  357,  357,  673,  673,  358,  358,  359,  359,  360,
  361,  361,  362,  362,  363,  363,  675,  676,  364,  674,
  674,  674,  674,  674,  674,  365,  365,  366,  366,  367,
  367,  368,  677,  678,  368,  679,  368,  369,  369,  370,
  680,  370,  371,  371,  372,  372,  373,  374,  374,  374,
  375,  375,  376,  376,  377,  377,  378,  378,  681,  681,
  379,  379,  682,  682,  380,  380,  380,  683,  381,  381,
  381,  382,  684,  685,  383,  384,  384,  686,  384,  384,
  687,  688,  385,  689,  690,  386,  387,  387,  387,  387,
  691,  388,  692,  388,  389,  389,  390,  391,  392,  392,
  393,  394,  693,  694,  695,  395,  396,  696,  396,  397,
  397,  397,  398,  398,  399,  400,  400,  401,  402,  402,
  403,  403,  404,  405,  405,  406,  697,  406,  407,  408,
  408,  409,  409,  409,  410,  411,  411,  412,  413,  413,
  413,  413,  413,  413,  413,  414,  414,  414,  414,  414,
  414,  414,  414,  414,  414,  414,  415,  416,  416,  416,
  416,  416,  416,  416,  416,  417,  418,  418,  418,  419,
  419,  420,  420,  421,  421,  422,  422,  423,  423,  423,
  424,  424,  424,  424,  424,  424,  424,  424,  425,  425,
  426,  427,  428,  428,  428,  429,  429,  429,  429,  430,
  430,  431,  698,  699,  700,  432,  702,  433,  703,  434,
  704,  435,  701,  701,  436,  436,  437,  705,  437,  438,
  438,  438,  438,  438,  438,  438,  438,  438,  439,  439,
  440,  440,  441,  441,  706,  707,  708,  442,  709,  710,
  443,  711,  712,  444,  445,  713,  714,  715,  446,  447,
  448,  448,  449,  449,  449,  450,  450,  451,  451,  451,
  451,  451,  452,  452,  453,  453,  454,  454,  455,  455,
  456,  456,  457,  457,  457,  458,  458,  458,  458,  458,
  458,  458,  458,  458,  458,  458,  458,  458,  458,  458,
  458,  458,  458,  458,  458,  458,  458,  458,  458,  458,
  458,  458,  458,  459,  459,  460,  460,  460,  460,  460,
  461,  461,  462,  462,  463,  464,  464,  465,  465,  466,
  466,  466,  467,  467,  467,  467,  468,  468,  468,  468,
  468,  469,  469,  470,  471,  471,  472,  473,  473,  473,
  473,  473,  473,  474,  474,  475,  476,  476,  477,  478,
  479,  480,  481,  481,  482,  483,  483,  483,  484,  484,
  484,  485,  485,  485,  485,  485,  485,  485,  485,  485,
  485,  485,  485,  485,  485,  485,  485,  485,  485,  485,
  485,  485,  485,  485,  485,  485,  485,  485,  485,  485,
  486,  486,  486,  486,  486,  486,  486,  486,  486,  486,
  486,  486,  487,  487,  487,  488,  488,  489,  490,  490,
  491,  491,  492,  492,  493,  494,  494,  494,  495,  496,
  496,  497,  498,  498,  499,  499,  499,  499,  499,  499,
  499,  499,  499,  499,  499,  499,  499,  499,  499,  499,
  499,  499,  499,  499,  499,  499,  499,  499,  499,  499,
  499,  499,  499,  499,  499,  499,  499,  499,  499,  499,
  499,  499,  499,  499,  499,  499,  499,  499,  499,  499,
  499,  499,  499,  499,  499,  500,  500,  500,  500,  500,
  500,  500,  500,  501,  501,  501,  501,  502,  502,  503,
  503,  504,  504,  504,  504,  505,  505,  505,  506,  506,
  506,  506,  507,  508,  508,  509,  509,  509,  509,  509,
  510,  510,  716,  717,  718,  511,  512,  512,  513,  513,
  514,  514,  515,  515,  516,  517,  518,  519,  519,  520,
  520,  521,  521,  521,  521,  521,  521,  521,  521,  521,
  521,  671,  671,  670,  670,  522,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,  540,  541,  542,  543,  544,  545,
  546,  547,  548,  549,  550,  550,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,  551,  551,  551,  551,
  551,  551,  551,  551,  551,  551,
};
static int yylen[] = {                                         2,
    1,    2,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    0,    0,
    4,    0,    0,   10,   10,   10,   10,   10,   10,   10,
    0,    8,    8,    0,    0,    7,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    2,    1,    1,    0,
    1,    1,    2,    0,    2,    2,    0,    4,    1,    1,
    1,    1,    1,    2,    2,    2,    2,    0,    1,    2,
    4,    1,    2,    2,    2,    2,    2,    4,    1,    1,
    1,    0,    3,    0,    3,    0,    2,    2,    1,    1,
    2,    0,    4,    0,    0,   10,   10,   10,   10,   10,
   10,   10,    0,    8,    8,    0,    7,    6,    0,    3,
    0,    3,    0,    2,    2,    0,    4,    4,    2,    2,
    2,    2,    2,    2,    1,    2,    2,    2,    4,    1,
    2,    4,    1,    2,    2,    3,    0,    0,    0,   11,
    1,    1,    5,    0,    0,    2,    0,    4,    2,    0,
    4,    2,    0,    2,    2,    1,    0,    2,    0,    4,
    2,    2,    0,    2,    2,    1,    3,    9,    1,    1,
    2,    2,    0,    0,    9,    2,    2,    2,    4,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    3,    0,    3,    0,    4,    2,    2,    2,    2,    4,
    2,    2,    2,    3,    2,    2,    2,    4,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    3,    4,    2,
    2,    2,    0,    1,    1,    1,    4,    0,    0,    8,
    0,    1,    1,    0,    1,    0,    4,    0,    0,    2,
    1,    2,    0,    0,    8,    0,    2,    2,    0,    3,
    1,    1,    0,    1,    2,    0,    1,    2,    4,    3,
    0,    1,    3,    0,    3,    0,    1,    3,    1,    3,
    3,    1,    3,    0,    1,    5,    1,    1,    2,    1,
    0,    0,    0,    0,   10,    0,    2,    5,    0,    2,
    2,    0,    3,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    0,    1,    5,    0,    0,    0,    6,    0,
    1,    1,    0,    2,    2,    0,    3,    3,    2,    2,
    2,    2,    2,    3,    4,    0,    2,    2,    3,    1,
    4,    6,    3,    0,    0,    5,    0,    2,    2,    0,
    3,    3,    4,    0,    1,    2,    0,    1,    1,    0,
    0,    7,    0,    7,    0,    4,    4,    0,    3,    2,
    0,    2,    0,    4,    0,    2,    2,    0,    0,    7,
    0,    7,    3,    2,    6,    0,    3,    4,    1,    1,
    3,    2,    1,    1,    1,    0,    2,    1,    3,    6,
    1,    1,    1,    1,    1,    1,    1,    0,    0,    2,
    2,    0,    3,    3,    1,    1,    5,    2,    3,    0,
    3,    0,    2,    3,    4,    8,    1,    0,    4,    0,
    1,    1,    2,    2,    3,    3,    1,    2,    2,    2,
    1,    1,    4,    3,    0,    0,    0,    6,    1,    1,
    0,    2,    4,    0,    5,    2,    4,    3,    1,    4,
    2,    2,    0,    2,    2,    1,    1,    4,    3,    0,
    8,    0,    2,    1,    5,    3,    0,    2,    1,    1,
    5,    1,    0,    1,    2,    3,    3,    1,    1,    6,
    5,    0,    1,    1,    2,    2,    3,    2,    3,    0,
    1,    1,    2,    4,    0,    0,    0,   11,    0,   13,
    1,    1,    3,    0,    1,    7,    0,    2,    1,    1,
    1,    3,    0,    1,    2,    1,    2,    2,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    2,    1,
    2,    1,    1,    1,    1,    1,    5,    2,    3,    6,
    1,    0,    0,    8,    2,    2,    0,    1,    2,    0,
    0,    0,    7,    3,    5,    1,    0,    7,    7,    2,
    4,    3,    2,    4,    1,    0,    2,    0,    3,    2,
    5,    3,    3,    2,    2,    3,    3,    3,    8,    6,
    3,    0,    3,    2,    1,    1,    3,    3,    7,    0,
    2,    1,    2,    2,    0,    1,    0,    1,    3,    1,
    2,    0,    0,    6,    0,    2,    0,    4,    2,    2,
    2,    2,    1,    1,    1,    1,    2,    2,    0,    0,
    7,    3,    1,    1,    1,    1,    2,    2,    5,    7,
    0,    1,    3,    0,    2,    4,    0,   10,    0,    4,
    2,    4,    4,    5,    1,    1,    0,    3,    7,    6,
    9,    8,    9,    8,    1,    1,    1,    0,    2,    2,
    2,    1,    1,    2,    2,    2,    1,    3,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    3,    2,    2,    2,    3,    3,    2,
    2,    2,    3,    1,    1,    1,    1,    1,    3,    3,
    3,    3,    3,    3,    5,    5,    5,    5,    5,    5,
    0,    2,    2,    1,    2,    0,    0,    5,    2,    2,
    2,    2,    0,    4,    2,    2,    3,    2,    2,    2,
    3,    3,    2,    2,    3,    4,    0,    0,    4,    2,
    0,    3,    2,    3,    2,    2,    2,    3,    3,    2,
    2,    2,    0,    4,    3,    3,    3,    3,    4,    4,
    5,    5,    4,    5,    3,    4,    4,    3,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    2,    1,    1,    0,    1,    1,    2,    2,
    3,    0,    2,    0,    2,    5,    1,    1,    2,    3,
    0,    3,    4,    3,    2,    2,    5,    0,    5,    1,
    4,    2,    0,    2,    7,    6,    4,    2,    0,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    3,    2,
    3,    2,    2,    2,    2,    5,    2,    2,    3,    3,
    4,    0,    2,    2,    1,    1,    1,    3,    1,    1,
    3,    6,    1,    1,    1,    4,    5,    1,    3,    1,
    3,    1,    2,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    0,    0,    6,    4,    6,
    5,    0,    3,    0,    2,    2,    0,    3,    4,    1,
    5,    4,    1,    4,    1,    2,    2,    1,    1,    1,
    2,    2,    2,    2,    0,    3,    5,    1,    1,    2,
    3,    1,    2,    0,    3,    1,    1,    1,    2,    4,
    0,    1,    5,    0,    1,    3,    3,    0,    1,    0,
    2,    0,    1,    0,    0,    5,    3,    3,    0,    2,
    2,    0,    3,    2,    2,    1,    2,    1,    3,    2,
    2,    3,    0,    0,    4,    3,    1,    1,    3,    3,
    1,    2,    3,    0,    1,    2,    0,    1,    1,    1,
    1,    1,    3,    2,    0,    3,    1,    3,    4,    5,
    6,    7,    7,    6,    6,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    2,    1,    1,    1,
    1,    2,    2,    1,    2,    1,    2,    1,    3,    1,
    1,    3,    0,    3,    0,    2,    0,    2,    4,    5,
    0,    3,    2,    1,    3,    6,    1,    1,    1,    4,
    4,    1,    9,    0,    2,    1,    1,    2,    4,    2,
    2,    4,    2,    1,    1,    1,    1,    1,    3,    1,
    3,    1,    3,    0,    1,    2,    1,    3,    1,    1,
    1,    1,    3,    6,    0,    1,    0,    3,    1,    3,
    1,    1,    1,    4,    5,    5,    5,    2,    4,    4,
    2,    6,    4,    0,    3,    1,    3,    1,    0,    1,
    2,    2,    2,    0,    1,    0,    3,    1,    3,    1,
    0,    2,    0,    2,    0,    2,    0,    0,    6,    0,
    1,    2,    1,    1,    1,    1,    3,    2,    1,    1,
    2,    1,    0,    0,    6,    0,    5,    1,    3,    0,
    0,    5,    0,    3,    1,    3,    3,    0,    1,    1,
    0,    2,    1,    1,    2,    4,    0,    1,    1,    1,
    0,    3,    1,    1,    0,    5,    4,    0,    8,    5,
    4,    3,    0,    0,   11,    2,    2,    0,    3,    3,
    0,    0,    5,    0,    0,    6,    4,    6,    2,    4,
    0,    8,    0,   10,    1,    1,    8,    5,    1,    1,
    9,    6,    0,    0,    0,   15,    0,    0,    3,    0,
    2,    4,    1,    3,    4,    1,    3,    3,    1,    1,
    0,    1,    3,    1,    3,    0,    0,    2,    4,    1,
    3,    1,    3,    4,    1,    1,    3,    1,    1,    3,
    3,    2,    3,    3,    4,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    3,    1,    1,    1,
    1,    1,    1,    1,    1,    6,    1,    1,    1,    5,
    6,    5,    6,    1,    3,    3,    4,    1,    1,    2,
    3,    5,    4,    6,    4,    6,    5,    7,    3,    4,
    4,    4,    1,    1,    1,    3,    3,    4,    4,    1,
    3,    3,    0,    0,    0,    6,    0,    7,    0,    6,
    0,    7,    0,    1,    0,    2,    2,    0,    3,    2,
    2,    2,    2,    3,    3,    1,    1,    3,    0,    5,
    2,    4,    3,    2,    0,    0,    0,    5,    0,    0,
    5,    0,    0,    5,    1,    0,    0,    0,    8,    1,
    2,    2,    1,    1,    1,    1,    1,    3,    3,    3,
    2,    2,    0,    2,    3,    2,    1,    1,    1,    1,
    1,    3,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    2,    2,    3,    3,
    3,    3,    3,    3,    3,    1,    1,    1,    1,    3,
    1,    1,    1,    1,    1,    1,    2,    2,    2,    2,
    0,    3,    0,    3,    4,    0,    1,    1,    3,    1,
    2,    1,    1,    1,    1,    1,    1,    1,    2,    2,
    2,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    2,    1,    1,    2,    1,    1,
    1,    1,    1,    2,    1,    1,    1,    1,    1,    1,
    1,    4,    5,    5,    5,    5,    5,    5,    5,    5,
    5,    5,    6,    6,    4,    4,    4,    4,    6,    6,
    6,    6,    6,    6,    6,    6,    6,    6,    6,    6,
    3,    3,    3,    4,    4,    7,    8,    6,    4,    8,
    6,    4,    0,    2,    2,    1,    1,    6,    0,    3,
    0,    2,    1,    1,    6,    1,    1,    1,    4,    4,
    4,    4,    4,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    7,    8,    8,    8,   10,
    8,    6,    4,    1,    1,    4,    4,    7,    8,    0,
    2,    7,    6,    6,    4,    1,    1,    1,    4,    3,
    6,    5,    6,    1,    1,    6,    8,    6,    6,    8,
    1,    1,    0,    0,    0,    8,    4,    5,    0,    2,
    3,    5,    4,    5,    1,    1,    1,    3,    5,    4,
    6,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    0,    1,    0,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,
};
static int yydefred[] = {                                      0,
    0,    0,    0,    0,    0,    0,    0,   29,    0,    0,
  102,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    3,    4,    5,   21,   23,   19,   17,   22,   18,
  754,    9,   10,   15,   20,   11, 1026, 1027, 1028,   26,
   27,   25,   28,   24,   16,   12,    0,    0,    7,    0,
    8,    6, 1295, 1296,   13, 1299, 1300,   14,    0,    0,
  763,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  202,    0,  753, 1039,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  234,    0,    0,    0,    0,    0,    0,  202,    0,  181,
    0,    0,    0,  144,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  202,    0,  930,    0,  657,   30,    0,    0,
   30,    0,    0,    0, 1044,    0,    0, 1885, 1886, 1887,
 1888, 1889, 1890, 1891, 1892, 1893, 1894, 1895, 1896, 1897,
 1898, 1899, 1900, 1901, 1903, 1904, 1905, 1906, 1907, 1908,
 1909, 1902, 1910, 1911, 1912, 1913, 1914, 1915,    0, 1916,
 1917, 1918, 1919, 1920, 1921, 1922, 1923, 1924, 1925, 1926,
 1927, 1928, 1929, 1930, 1931, 1932, 1933, 1934, 1935, 1936,
 1937, 1938, 1939, 1940, 1941, 1942, 1943, 1775, 1777, 1778,
 1779, 1780, 1781, 1782, 1800, 1801, 1783, 1785, 1784, 1793,
 1794, 1790, 1786, 1791, 1787, 1789, 1792, 1788, 1796, 1797,
 1795, 1798, 1799, 1803, 1802, 1804, 1805, 1806, 1807, 1808,
 1809, 1810, 1811, 1812, 1813, 1814, 1815, 1816, 1817, 1818,
 1819, 1820, 1821, 1822, 1823, 1824, 1825, 1826, 1827, 1828,
 1829, 1830, 1831, 1832, 1833, 1834, 1835, 1836, 1837, 1838,
 1839, 1840, 1841, 1842, 1843, 1844, 1845, 1846, 1847, 1848,
 1849, 1850, 1851, 1852, 1853, 1854, 1855, 1856, 1857, 1858,
 1859, 1860, 1861, 1862, 1863, 1864, 1865, 1866, 1867, 1868,
 1869, 1870, 1871, 1872, 1878, 1874, 1873, 1875, 1876, 1877,
 1879, 1880, 1881, 1882, 1883, 1884, 1948, 1949, 1950, 1951,
 1952, 1953, 1955, 1956, 1957, 1959, 1960, 1958, 1944, 1961,
 1945, 1946, 1947, 1954, 1962, 1963, 1964, 1965, 1966, 1967,
 1968, 1969, 1970, 1971, 1972, 1973, 1974, 1975, 1976,    0,
 1196,    0, 1768, 1776,    0,    0, 1134,    0, 1767,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  206,
 1029, 1773,    0,    0,    2,    0,    0,    0,    0, 1139,
 1142,    0,    0,    0,    0,  317,  811,  812,  813,  814,
  815,  816,  817,  818,  819,  820,  821,  822,  823,  824,
  825,  826,  827,  828,  829,  830,  831,  839,  833,  834,
  832,  866,  887,  863,  835,  836,  837,  838,  840,  842,
  845,  841,  843,  844,  846,  847,  848,  849,  850,  851,
  852,  853,  854,  855,  856,  857,  858,  868,  881,  878,
  864,  882,  859,  865,  885,  886,  879,  880,  867,  883,
  884,  888,  889,  862,  861,  860,  869,  870,  871,  872,
  873,  874,  875,  876,  877,  890,  891,  765,  777,  810,
  755,    0, 1757,    0,  479,  466,  467,    0,  768,  470,
 1749,  773,    0, 1760,  766,    0, 1761,  449,  441,  442,
    0,  761,  445, 1765,  756, 1198,  760,    0, 1769,  770,
    0, 1770,  666,  759,    0,  769,    0, 1766,  774,  204,
 1439,  488,  762,    0, 1774,    0, 1036, 1544,    0,  194,
  316, 1546,  322,  195,    0, 1753,  182,    0,  186,  192,
    0,    0,  187, 1549,  196, 1551,    0,  188,    0,  190,
    0,  199, 1403,  191,  197,  290,  193,  198,    0, 1752,
    0,  204, 1435,    0,  200, 1249, 1250,    0,    0,  145,
    0, 1758,    0,  938, 1756,  931,    0,  937,  940,  943,
  932,  933,    0,  934,  935,    0,  936, 1772,  942,  944,
  945,  204,    0,    0,  948,    0,    0,    0,  506,    0,
 1272,    0,    0,    0, 1037,    0,    0, 1045, 1047, 1048,
    0,    0, 1165,    0,    0,    0,    0,    0,  215,  208,
  216,  207,  209,  212,  211,  217,    0,    0,  213,    0,
    0, 1096, 1097, 1102,    0, 1101,    0, 1103, 1105, 1111,
    0, 1098, 1100,    0, 1099, 1106, 1110, 1104, 1108, 1109,
    0,    0,    0,    0,    0,    0,    0, 1122,    0,    0,
    0,    0, 1147,    0,    0,    0,    0,    0,    0,  772,
    0, 1751, 1414,  771, 1409,  317,  778,  232,  767,    0,
    0,    0,    0,  148,    0,  285,    0,  899,  900,    0,
    0,  446,  757,  676,  677,    0, 1407,    0,    0, 1322,
  908,  907,    0, 1439,  775,    0,    0, 1043, 1041, 1545,
  317,  264,    0,  230,    0,    0,  272,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  202,    0,  218,  243,
  242,    0,    0,  360,    0,    0, 1404,    0,    0, 1435,
  203,    0,  201,    0,    0,  146,  147,    0,    0,    0,
    0,  939,    0,    0,  941,    0,    0,  950, 1445, 1450,
  949,    0,    0,    0,    0, 1504,    0, 1540,    0, 1746,
    0, 1501, 1650,    0,    0,    0,    0, 1665,    0, 1541,
    0, 1535, 1524,    0, 1523,    0,    0,    0, 1506,    0,
    0, 1525, 1526, 1536, 1537,    0, 1538, 1539,    0,    0,
    0, 1543, 1393, 1394, 1395,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1542, 1532, 1533, 1747, 1505,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1534, 1482,    0,    0, 1332, 1470, 1339, 1346,
 1353, 1347, 1348, 1349, 1350, 1351, 1354, 1355, 1356, 1352,
    0,    0, 1471, 1499, 1502, 1477,    0, 1527, 1479, 1480,
 1481, 1496, 1497, 1498, 1528, 1478, 1557,    0, 1607,    0,
 1558, 1559, 1613, 1614, 1616, 1617, 1618, 1561,    0, 1624,
 1560, 1684, 1685, 1486, 1483, 1484, 1704, 1705, 1711, 1712,
 1485, 1503,    0, 1335,    0,    0,    0,    0,    0,   31,
    0,  123,    0,    0,    0,    0,  103,  104,  116, 1035,
    0,    0, 1089,    0,    0,    0, 1316, 1336,    0,    0,
    0, 1197, 1135,    0, 1179, 1762,    0,    0,  214, 1032,
 1030, 1107, 1115, 1113, 1112,    0,    0,    0, 1764,    0,
    0,    0,    0,    0, 1124, 1128, 1555,    0, 1150, 1148,
    0, 1153, 1151, 1743, 1154,    0, 1155, 1145,    0,    0,
 1745,    0,    0,    0, 1129,    0,    0, 1271,    0,    0,
 1308, 1328,    0, 1404,    0,    0,  778,    0,    0,    0,
    0,    0,  502,    0,    0,    0,  469,    0,    0,    0,
  281,    0,  444,    0,  757,  735,  734,    0,    0, 1404,
    0, 1324,    0,    0,  776, 1440, 1446,    0,    0,    0,
    0,  980, 1018,  984,    0, 1019,    0,    0,  985,    0,
    0,    0, 1010, 1022,  979, 1005,  981,  982,  977,  983,
    0,  955,  956,  964,    0,  965,  974,    0,  975,  976,
  978,    0,    0,    0,    0,    0,  270,  224,    0,  480,
  220,  225,    0,  450,  219,  675,  221,  227,    0,  665,
  223,  226,  204, 1442,  489,  222,  245,    0,    0,  370,
    0,    0,    0,    0,    0,    0,  291,  189,  363,  205,
 1436,    0,    0,  148,    0,    0,  180, 1547,  179, 1750,
    0,    0, 1214,    0,    0,    0, 1298,  954,  953,    0,
  951,    0,    0,    0, 1529,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1530, 1531,    0, 1507,    0,
 1509,    0,    0,    0,    0, 1713,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1508, 1510,    0,    0,    0,
 1137, 1138,    0,    0,    0,    0,    0,  604,    0,    0,
  649,    0,    0,    0, 1378, 1361,    0,    0, 1362,    0,
 1365,    0, 1363, 1364,    0,    0,    0, 1358, 1359, 1360,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  157,  507,    0,   69,   70,
    0,    0,    0,    0,    0,   64,    0,    0,    0,    0,
 1170, 1172, 1169, 1181, 1182, 1183, 1171,    0,    0,    0,
    0,    0,    0,  123, 1033,  288,    0,  286,  287, 1553,
 1046, 1045, 1303,    0,    0,    0,    0,    0,    0, 1178,
    0,  210,    0, 1121, 1090, 1120,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1157,    0,    0, 1158, 1160,
    0,    0,    0, 1245,    0, 1141, 1140,    0,    0,    0,
 1474,    0, 1473, 1520, 1522, 1475,    0, 1130, 1313,    0,
    0, 1270,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  920,    0,  909,    0,    0,  903,    0,  901,    0,
    0,  515,    0,    0,  468,  503,  476,    0,  478,    0,
    0,  283,  277,    0,  443,  447,    0,    0,    0,    0,
    0,  928,    0,    0,    0,    0, 1323,    0,    0, 1437,
    0,    0,    0,    0,    0,  484,  318,  317,    0,  987,
 1006, 1007, 1023,    0, 1020,    0,    0, 1013, 1011, 1012,
    0,  267,  253,    0,    0,    0,  973,    0,    0,    0,
    0, 1014,  273,  231,  279,    0,    0, 1411, 1442,  228,
    0,    0,  369,  361,    0,    0,  682,  683,  680,    0,
    0,    0,    0,    0,    0,  687,  679, 1405, 1404,  292,
    0, 1437,    0,    0,    0, 1550, 1548,    0,    0,    0,
 1221, 1223,    0, 1224, 1216, 1217,    0,  946,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1732, 1733, 1734,
 1735, 1736, 1737, 1740, 1741, 1738, 1739,    0,    0,    0,
    0,    0,    0, 1721,    0, 1714,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1696, 1698, 1697,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1591,    0,    0,    0,    0,    0, 1592, 1593,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1495, 1343, 1472,  653,
    0,    0,  605,    0,  606,    0,    0,    0,    0,    0,
    0,    0, 1389, 1400,    0, 1396,    0, 1397, 1344,    0,
    0,    0,    0,    0,    0,    0, 1380,    0, 1367, 1369,
 1368,    0,    0,    0, 1491,    0,    0,    0,    0,    0,
    0,    0,    0, 1700,    0, 1500, 1333,    0,    0,  155,
    0,    0,    0,    0,    0,    0,    0,   54,   55,   59,
   50,   52,   49,   47,   48,   56,   53,   58,   51,    0,
    0,    0,    0,    0,    0,   45,  126,  138,  140,    0,
    0,    0,    0, 1200,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  120,  122,    0,    0,    0,    0,
    0,    0,  289, 1554,    0,    0, 1304, 1317,    0, 1302,
    0, 1337, 1180,    0, 1095, 1091, 1117,    0,    0, 1119,
 1094, 1149, 1152,    0,    0,    0,    0,    0,    0,    0,
    0, 1521, 1771, 1260, 1259, 1262, 1263, 1264,    0,    0,
    0,    0,    0, 1330,    0, 1405,  910,    0,    0,  915,
    0,  922,  916,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  780,  905,  509,  505,    0,    0,  511,  512,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  526,  521,
    0,    0,  520,    0,  530,  529,  552,  541,  542,  555,
    0,  539,  556,  553,  543,  536,  554,    0,  545,  546,
  544,  634,  636,  635,  537,    0,  531,  532,  535,  533,
  534,  550,  540, 1320, 1319,    0,    0,    0,  452,  155,
    0,    0,  275,    0,  793,  420,    0,    0,  792,    0,
    0,    0,    0,  752,    0,    0, 1405, 1325,    0,  664,
  906,    0, 1447,    0,    0,  481,  485,    0,    0,    0,
 1552,    0,    0,    0,    0, 1021,    0,  265,  407,  402,
  403,  404,  405,  406,  254,    0,  968,    0,    0,    0,
    0,    0,    0,    0,    0,  278, 1404,  229, 1443,    0,
  238,  371,    0,  685,  686,  684,    0,    0,    0,    0,
  708,  702,  696,  720,  693,  690,  699,  717,  705,  711,
  714,  731,  726,    0,  707,  701,  695,  719,  692,  689,
  698,  716,  704,  710,  713,  722,  730,    0,  709,  703,
  697,  721,  694,  691,  700,  718,  706,  712,  715,  723,
  732,    0,    0,    0,    0,    0,    0,  361,    0,    0,
    0,    0,    0,    0,  494,  183,  157,    0,  155,    0,
 1315, 1755, 1222,    0, 1258,    0,    0,    0,    0,    0,
 1562,    0, 1391,    0, 1686,    0,    0,    0,    0,    0,
 1683, 1392,    0,    0,    0, 1512, 1514,    0,    0,    0,
    0,    0,    0, 1715,    0,    0,    0, 1730, 1619, 1620,
 1621, 1687, 1622,    0, 1695,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1594,    0, 1595,    0, 1599,
    0, 1602, 1575, 1576, 1577, 1578,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1334,    0,
    0,    0, 1515,    0,    0,    0,    0,    0,    0, 1398,
 1399, 1345,    0,    0, 1390,    0,    0,    0,    0,    0,
    0,    0, 1623, 1699,    0,  650,    0,    0,    0,    0,
   61,    0,    0,    0,    0,    0,    0,    0,   57,   41,
   41,   67,   67,   67,    0,    0,   73,    0,   72,    0,
   71,    0,   66,   80,   87,  123,    0,    0, 1173, 1205,
 1203, 1201,    0, 1273, 1202,    0,    0,    0,    0,    0,
   41,    0,    0,    0,    0,    0,   41,    0,    0,    0,
    0, 1063,    0, 1056,    0,    0,    0, 1051, 1058, 1067,
 1068,    0,    0, 1136, 1092, 1093,    0,    0,    0,    0,
 1159, 1163, 1161, 1246,    0, 1247,    0,    0,    0,    0,
 1314,    0,    0, 1329,  315,    0,    0,    0,    0,  251,
  914,    0,    0,  918,  782,    0,    0,  785,  787,  786,
    0,    0,  783,  790,  791,  972,    0,  506,  506,    0,
    0,  504,    0,  462,    0,  960,  959,  963,    0,  519,
    0,    0,  570,    0,    0,    0,  613,  616, 1763,  637,
  638,    0,  640,  614,    0,  551,  522,  622,  527,    0,
  620,  528,    0,    0,    0,    0,  611,  475,    0,    0,
    0,  448,    0,    0,  893,    0,    0,    0,  809,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  926,    0,
    0,    0, 1441,    0,  486,  487,  319,  317,    0,    0,
    0,    0,    0,    0,    0,  325,    0,    0,  989,    0,
    0, 1025,    0, 1016,    0,    0,  966,    0,  998,    0,
    0,  999, 1002, 1004,  280,    0, 1437,  321,    0,  239,
    0,  375,    0,    0,    0,  670,  724,  688,  728,  729,
  733,    0, 1406,    0, 1426,    0, 1427,    0,    0,    0,
    0,    0,    0, 1417,    0,  293,  371, 1438,  496,    0,
    0,  498,    0,    0,    0,  491,    0,    0,  495,  184,
  155,  149,    0,    0,    0, 1218, 1297, 1568, 1567,    0,
    0,  398,  401, 1564, 1563,    0, 1572, 1571, 1570, 1569,
    0,    0, 1566, 1565,    0,    0,    0, 1723, 1722,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  656,  608,  607, 1375, 1372, 1402, 1401, 1370,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1702,
    0,  160,  660,    0,  661,  160,    0,    0,    0,    0,
    0,    0,    0,    0,  123,  123,    0,    0,    0,   83,
   84,   75,   74,    0,    0,  139,    0,    0, 1274, 1184,
    0, 1195, 1190,    0,  123,    0,    0,    0,    0,    0,
    0,    0,  113,  113,  123, 1053,    0,    0, 1057,    0,
 1054,    0, 1055, 1064,    0,    0, 1061, 1060,    0,    0,
    0,    0,    0, 1253, 1254, 1252,    0, 1267, 1269, 1331,
 1410,  913,  344,  252,    0,  921,    0,    0,  784,  788,
  789,    0,    0,  397,    0,    0,  513,    0,    0,  518,
    0,    0,    0,  571,    0,    0,  601,    0,  642,  568,
    0,  623,  621,    0,    0,    0,    0,  562,    0,    0,
  160,  453,  456,    0,    0,  794,    0,  797,  798,  420,
    0,  796, 1754,  898,  897,  795,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  808,  805,  389,    0,  390,
    0,    0,    0,    0,    0,    0,  925, 1408,  668, 1468,
 1467,    0,    0,    0,    0,    0,    0,    0,    0,  327,
    0,    0,  333,    0,  330,  329,  332,  331,  991,    0,
  995,  996,    0,    0,    0,    0,  254,  969,  971,  967,
 1001, 1405,    0,    0,    0,  373,  362,  372,    0,    0,
    0,  669,    0,    0, 1405, 1419,    0, 1423,    0,    0,
 1420, 1422, 1421,    0,  297,    0,    0,  497,  499,  490,
    0,    0,    0,  160,  152,    0,    0,  159,  151,    0,
    0,    0, 1703, 1731,    0, 1682, 1615,    0,    0,    0,
 1724,    0,    0, 1706, 1708,    0, 1693, 1694,    0,    0,
    0,    0,    0,    0,    0, 1709,    0, 1574, 1573,    0,
    0,    0, 1598,    0, 1601, 1579, 1580, 1581, 1582, 1583,
 1584, 1585, 1586, 1587, 1588, 1589, 1590, 1371, 1373,    0,
    0,    0, 1366,    0, 1608, 1701,    0,    0,    0,    0,
   33,   33,   33,   33,   33,   33,   33,    0,    0,    0,
   68,    0,   82,    0,   81,   90,   89,   91,   88,    0,
  126,    0,    0,  143,  141,  100,    0, 1176, 1185, 1274,
    0,    0, 1186, 1187,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  123,  123,    0,    0, 1066, 1059,    0,
 1075, 1070, 1069,    0, 1073, 1062,    0, 1301,    0,    0,
    0,    0, 1266,  345,  917,    0,  919,    0,    0,  507,
  464,  465,    0,  961,  958,    0,    0,    0,    0,    0,
    0,  643,  644,  646,  645,    0,    0,    0,  625,    0,
  563,    0,    0,    0,    0,  474,  471,  423,    0,    0,
    0,  384,    0,  394,    0,  395,    0,  799,    0,  800,
  392,  807,  806,  803,    0,  895,    0,    0,  749,  750,
  746,  745,  748,  747, 1465,    0,    0, 1461,    0, 1462,
 1448,    0,    0,    0,  338,  340,    0,  328,  334,  990,
    0,  988, 1017,  255,    0,    0,    0, 1444,  358,  359,
  247,  240,    0,  375,  376,  377,    0,  674,    0,  672,
 1404, 1424, 1425, 1428,    0,  294,  292,    0,  185,  236,
    0,    0,  158,  169,    0,    0,    0,    0,    0,  166,
  162,    0,    0,    0, 1229,    0, 1226,    0, 1230,    0,
  399,    0, 1688,    0,    0,    0,    0,    0,    0,    0,
 1692, 1676,    0,    0,    0,    0,    0,    0,    0, 1596,
    0,    0,    0,  662,    0,    0,  459,    0,  658,  123,
  123,  123,  123,  123,  123,  123,    0,  126,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  124,  125,  135,
    0,    0,    0,   99,    0,   46,    0,  101, 1174,    0,
    0, 1188,    0, 1192,  105,  105,  105,  105,  105,  105,
  105,    0,    0,  117, 1076,    0, 1082,    0, 1087,    0,
    0, 1208,    0, 1212, 1143,    0,  924,  269,    0,    0,
    0,    0,    0,    0,  572,    0,    0,  557,    0,    0,
    0,    0,    0,  575,    0,    0,  457,  458,  473,    0,
    0,    0,  427,  421,    0,  378,  381,  383,  801,  802,
  804,  391,    0, 1460, 1459, 1464, 1458,    0,  339,    0,
    0,  335,  993,    0,    0,  258,  262,  261, 1412,    0,
  374,  673,  671,    0,    0,    0,    0,    0,  292,    0,
  364,    0,    0,    0,    0,    0,  168,  161,    0,  164,
  165,  178,    0, 1219, 1233, 1232, 1236, 1228, 1231,    0,
 1689,    0, 1720,    0, 1716, 1707, 1677, 1678, 1679,    0,
 1710,    0, 1681, 1604, 1605, 1597, 1600,  663,  160,  455,
  461,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   42,  130,  137,  129,  132,  136,  133,  134,  131,   43,
   95,  142,   98,   97, 1275,    0,    0, 1276, 1277,    0,
  123,  123,  123,  123,  123,  123,  123,  114,  115,    0,
    0,    0, 1084,    0,    0,  346,  345,    0,    0,    0,
    0,  560,    0,    0,    0,    0,    0,  627,  633,    0,
    0,    0,    0,  626,    0,    0,    0,    0,    0,    0,
    0,    0,  379,  379,  388,    0, 1455, 1454, 1449,    0,
 1453,    0,  343,  260,  268, 1405, 1430,    0, 1431,    0,
  298,  295,  314,  292,  301,    0,    0,    0,    0,    0,
  172,    0,  176,  171,    0, 1227,    0,    0,    0, 1717,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   93,  127,  128, 1279, 1280,    0, 1281, 1189,    0,
    0,    0,    0,    0,    0,    0, 1083, 1079, 1078,    0,
 1088, 1305, 1209,    0,    0,    0,    0,  962,  516,    0,
  573,  571,    0,  641,  624,    0,  632,  631, 1759,  629,
  630,    0,    0,  566,  565,  618,  609,    0,    0,    0,
  424,    0,    0,    0, 1452, 1456, 1451, 1457,    0,    0,
 1434,    0,  303,    0,    0,    0,    0,  304,  305,  306,
    0,    0,  237,  170,    0,  174,    0,  153, 1234, 1234,
  400, 1718, 1680,  460,   34,   36,   35,   37,   38,   39,
   40, 1284, 1282,  106,  108,  107,  109,  110,  111,  112,
 1080, 1081, 1086,    0,    0,    0,    0,  349,    0,    0,
  577,    0,    0,  599,  628,  564,    0,    0,  425,    0,
    0,    0,  409,    0,    0, 1432, 1433,  307,  310,  309,
  312,  311,  308,  366,  367,  150,  177,    0,    0, 1285,
    0, 1308,    0,  351,    0,    0,    0,    0,  508,  571,
    0,    0,    0,    0,    0,    0,  590,  619,    0,    0,
    0,  380,    0,    0,  382,  342,    0, 1238,    0, 1237,
    0,    0,    0, 1283,    0, 1306,    0,  356,  352,    0,
    0,    0,    0,  589,    0,    0,    0,    0,    0,    0,
    0,    0,  429,    0,  385,    0,  411,  412,    0, 1235,
 1241,    0,    0, 1286,    0, 1289,    0, 1309,  353,  510,
  572,  578,  572,  579,  580,  583,    0,    0,    0,    0,
    0,    0,  596,  598,  597,    0,    0,    0,    0,    0,
  413,    0, 1239,    0,    0, 1291,    0,    0,    0,    0,
    0,    0,    0,    0,  426,    0,  434,    0,  433,  387,
    0,    0,    0,  416,  414,  415,    0,    0,    0, 1290,
    0,    0,  581,  584,    0,    0,    0,    0,  437,  436,
  435,    0,    0,  418, 1242, 1293,    0,    0,  440,  438,
  439,  419,    0,    0,    0,    0,    0,    0,    0,  417,
    0,    0,    0, 1292,    0, 1294,
};
static int yydgoto[] = {                                      20,
   21,   22,   23,   24,   25,  950, 1640, 1271, 1642, 1272,
 1273, 2063, 1274, 1643, 2064, 1275, 1644, 2065, 2879, 2672,
 2896, 2674,   26,  957,  958,  959, 2658, 2888, 2889, 1277,
 1648, 2394, 2675,   27,  104,  726, 2587, 1928, 2293, 2588,
 2831, 2825, 2997, 3134,  550, 1136,   28,  100,   29,  360,
   30,  709,  517, 1098,  461,  101, 2819,  525,  712, 1118,
 1861, 2802, 2129,  514, 2546, 2795, 2976, 1422, 1423, 2977,
  520,  696,  697, 1041, 1102, 1437,  472,  666,  667,   31,
 1298,  535,  538, 2276, 2816, 2988, 3125, 3218, 3219, 3220,
 3122,  650,  510,  511, 1407, 1829, 2226, 2217, 2528, 2785,
 2130, 3076, 3174, 3258,  528, 1128, 2991,  714, 2251, 2558,
 2488, 2957, 3345, 2507, 2753, 2151, 2301, 1838, 3272, 3313,
 3347, 3405, 2489, 2490, 2954, 3201, 3377, 3378, 3379, 3420,
  478,  479,  480,  481,  482, 1105, 1381, 2040, 2189, 2483,
 2866, 2867, 2374, 2457,  465,  466,  467,  468, 2747,  663,
 1037,  469, 1101,  502, 1404, 1405, 1406,  503, 1116,  723,
 1923, 1924, 1925, 1031, 1032, 1033, 1738, 1739, 1740, 2161,
 1759, 1760, 1761, 1762, 1763, 1764, 1765, 1766, 1767, 1768,
 1769, 1770, 3095, 2469, 2470, 1771, 1772, 1773, 2935, 3331,
 3365, 3332, 3366, 3181, 3262, 3307, 1774, 1775, 2467, 1776,
  884, 1574, 1777, 1778, 1779, 1780, 2167, 3195, 3194, 2180,
 2181, 2942, 3094, 1781, 1782, 1783, 1784, 2738, 1785,  885,
 1231,   33, 1267, 2037, 2375,  493, 1111,  494, 1820,  530,
 1107,  676, 1124, 1449, 1455, 1456, 1125, 1392, 1394,   34,
   78,  458, 1365, 1733, 1387, 1809, 2198,  459, 2764, 2496,
  475,  659, 1029, 1369,  496,  683, 1025, 1364, 2717,  487,
 1049,   35,  125,  735, 1081, 2154, 2155, 2156, 2157, 1083,
 1846, 1847, 1084, 2146, 1086, 2544, 2231, 1427, 1087, 2147,
 1089, 1418, 1090, 1415,   36,   37,   38,  981,   39,   40,
   41,  507,   42,   43, 1301, 1675, 2098, 2099, 2100, 2101,
 2702, 3170, 3253, 2916, 2917, 3073, 2918,   44,   45,  631,
  632,  633,  634,  635, 1319,  991, 1315, 1786,  367,  995,
  638,   47,   48,  346,  347, 1222, 1223,  369,  370,  371,
  641,  642,  643, 1006, 1327, 1328, 1329, 1330, 1695, 2107,
 1280, 1281, 1282, 2677,  598,  974, 1283, 1284, 1285, 1286,
 2683, 2684, 1287, 1662,  340,  495, 1660, 2433, 2921, 2922,
 2712,  730, 1146, 2835, 2836, 2837, 2838, 3008, 3317, 3320,
  646, 1333, 1334,  548, 2116, 2436, 1935, 1936, 1016, 1348,
 1787,   50, 1788, 2900, 3058, 3059, 3324, 3354, 1789,   53,
   54, 1790,   56,   57, 1791, 3293, 1018, 1350,  731,  966,
 1792, 1793,  679,  680, 1051,  648, 1022, 1713,  886,  887,
  969, 1143,  888,  889,  890, 1251,  891, 1602,  892,  893,
 1580,  894, 1252,  895,  896,  897,  898,  899,  900, 1583,
 1584,  532,  490,  654, 1108, 1458, 1915, 2274, 2263, 2984,
 3119,  721,  685, 1440,  738, 1056, 1057, 3109, 3110, 3207,
 2527, 2778, 2213, 2522,  901,  902, 1342,  903,  904,  905,
 1169, 1171,  906, 1493,  907, 1840,  908,  909,  910,  911,
  912,  913,  914,  915, 1316, 1137,  526, 1467,  527, 1833,
 1299, 1300,  916,  917,  918,  919, 2860,  920,  921, 2032,
 2335,  922,  923,  924,  925,  926,  927,  928,  929,  930,
  931,  932, 2599,  933, 1530,  934,  935,  936,  937,  938,
  939, 2603, 2849,  940, 1175, 2846, 1176, 1962, 2333,  941,
 1508,  942,  943,  470, 1139,  651,  539,  944, 2492, 1742,
  554,  518,  551, 3188,  521, 3318,  975, 1796,  992,  483,
 1649,  945,  486,  531, 2676, 1228,  567,  361,  946,  947,
  344,  128,  580,  951, 2042, 2870, 2385,  952, 2066, 1276,
 2387, 1623, 2897,  131, 1293, 3061, 2694, 1294, 2659, 2067,
 1134, 1038, 2583, 2294, 1620, 2647, 2826, 2993, 2290, 2581,
  501,  684, 2508, 2250, 2555, 2249, 2801, 2803,  693, 1845,
 2235, 2974, 2978, 1384, 1460, 3126, 2576, 2987, 3124,  691,
  656, 1828, 1356, 2714, 2926, 3255, 3297, 1121, 1862, 1461,
 2252, 2804, 2559, 3103, 3203, 3104, 2509, 2510, 3273, 3346,
 2194,  672, 1044, 1804,  664, 1373, 2149,  949, 1621, 2148,
 2741, 2945, 2464, 2728, 3084, 3300, 3301, 2472, 2739, 2943,
 3186, 2173, 2471,  579,  673, 1045,  376,  657, 1027, 1727,
 2193, 1810, 1719, 2447, 1091, 1410, 1834, 1092, 1093, 1094,
 1095,   80,  961,  689,  588,  964, 2088, 2424, 2704, 1012,
 1007,  594, 2071, 1476, 1934, 2591, 3137, 3288, 3138, 3384,
 1706, 1709,  373, 2399, 2680, 3056, 3243, 3291, 3290, 3321,
 3409, 3434, 1677, 2102, 3254, 1353,  649,  717, 1459, 1914,
  655, 1050, 1024, 1857, 2264,  722, 1462, 1822,  686, 1400,
 1441, 2247, 1401, 2214, 2968, 1516, 1964, 2321,
};
static int yysindex[] = {                                   6841,
37558,  -31,39391, 1553,  758,30181,  -40,    0,  858,  935,
    0,  -31,  578,95701,96094, 2678,99631,  837, 1017,    0,
 1373,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1136, 1066,    0, 1176,
    0,    0,    0,    0,    0,    0,    0,    0, 1098, 1206,
    0,89373,99631, 1292,99631,99631,99631,99631,99631,99631,
99631,99631,99631,99631, 1064,    0,99631,    0,    0, 1322,
  859,99631,99631,99631,99631, 1407,99631, 1348,99631,99631,
    0,99631,99631,99631,99631,99631, 1317,    0,100024,    0,
  123, 1487,99631,    0,99631,99631,99631, 1508,99631,99631,
99631,99631,99631, 1348,99631,99631,99631,99631,99631,99631,
99631, 1250,    0,100417,    0,99631,    0,    0,99631,99631,
    0, 1297,99631, 1558,    0,99631, 1517,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1643,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1591,
    0, 1736,    0,    0,99631, 1975,    0, 1999,    0,99631,
99631,99631,99631,99631,99631,99631,99631, 1474,100810,    0,
    0,    0,99631,38179,    0, 1622, 1656,    0,    1,    0,
    0, 1653,    0,99631, 1701,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  859,    0,99631,    0,    0,    0, 1160,    0,    0,
    0,    0, 1605,    0,    0,  946,    0,    0,    0,    0,
 1483,    0,    0,    0,    0,    0,    0, 1088,    0,    0,
 1701,    0,    0,    0, 2091,    0,   85,    0,    0,    0,
    0,    0,    0, 1889,    0, 1752,    0,    0, 1731,    0,
    0,    0,    0,    0, 1930,    0,    0,  859,    0,    0,
 1616, 5637,    0,    0,    0,    0,  735,    0, 1896,    0,
 1163,    0,    0,    0,    0,    0,    0,    0, 1918,    0,
 1833,    0,    0,99631,    0,    0,    0, 1941,99631,    0,
 1948,    0, 1862,    0,    0,    0,99631,    0,    0,    0,
    0,    0,  -50,    0,    0, 1802,    0,    0,    0,    0,
    0,    0,99631,99631,    0,77758, 2268,    0,    0,    0,
    0, 1841,  640, 1851,    0, 1937,99631,    0,    0,    0,
 2028,99631,    0,99631, 1975,99631,99631, 2108,    0,    0,
    0,    0,    0,    0,    0,    0, 1970,99631,    0, 2022,
 2013,    0,    0,    0, 2116,    0, 2021,    0,    0,    0,
 2055,    0,    0,99631,    0,    0,    0,    0,    0,    0,
 2123,99631,99631,99631,99631, 2093, 1978,    0,  167,  172,
 1309,    0,    0, 2198, 1462, -129, 1974,  541, 2458,    0,
 2098,    0,    0,    0,    0,    0,    0,    0,    0, 2202,
 2221, 1458, 2259,    0, 2088,    0, 1886,    0,    0, 2221,
 2259,    0,    0,    0,    0, 1514,    0,99631, 2279,    0,
    0,    0, 2280,    0,    0,99631, 2289,    0,    0,    0,
    0,    0,21065,    0, 2137, 1886,    0,99631,99631,99631,
99631,99631,99631,99631,99631, 1964,    0,99631,    0,    0,
    0, 2283, 1129,    0,99631, 1383,    0,99631,99631,    0,
    0,99631,    0, 2311,99631,    0,    0,86601,70828, 2199,
 1974,    0, 2247, 2264,    0, 1956,99631,    0,    0,    0,
    0, 2563, 2568, 2573, 2177,    0, 2585,    0, 2590,    0,
 2602,    0,    0, 2609, 2617,78388, 2626,    0, 2634,    0,
 2635,    0,    0,    0,    0, 2640, 2238, 2249,    0, 2664,
 2665,    0,    0,    0,    0, 2668,    0,    0,71458, 2670,
 2671,    0,    0,    0,    0, 2675, 2340, 2677, 2679, 2692,
 2695, 2702, 2703,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2704,
 2707, 2709,    0,    0,    0,    0, 2712,    0,    0,    0,
    0,    0,    0,    0, 2714,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 2715, 2720, 2721,
 2724, 2725, 2727, 2728, 2729,    0,    0,    0,    0,    0,
 2731, 2734, 2735, 2736, 2739, 2740, 2741, 2742, 2743, 2746,
 2749, 2750, 2754, 2755, 2756, 2757, 2664, 2665,82798,82798,
72088,99631,    0,    0, 2324, 2646,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
25321, 1655,    0,    0,    0,    0, 2758,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2186,
    0,    0,    0,    0,    0,    0,    0,    0, 2766,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2767,    0, 2769, 2773,    0,99631, 2780,    0,
   14,    0,85419, 2453, 2452, 2455,    0,    0,    0,    0,
99631,  226,    0,    0,99631,  272,    0,    0, 2768, 2781,
    0,    0,    0, 1741,    0,    0, 2788,99631,    0,    0,
    0,    0,    0,    0,    0, 2497,  575, 2499,    0, 2790,
 2790,    0, 2504,99631,    0,    0,    0,78388,    0,    0,
78388,    0,    0,    0,    0,72718,    0,    0,78388, 1066,
    0, 1066,78388, 1546,    0, 2339,78388,    0, 2802, 1974,
    0,    0, 2551,    0,    0, 1230,    0,  859, 2492,  984,
 2580, 2221,    0,99631,  859,  859,    0, 2815,  226, 2583,
    0, 2580,    0, 2815,    0,    0,    0, 1080, 2477,    0,
 1924,    0, 2423, 2415,    0,    0,    0,   32,    0, 2822,
 2435,    0,    0,    0, 2444,    0, 2491, 2836,    0, 2536,
 2838,   95,    0,    0,    0,    0,    0,    0,    0,    0,
 2595,    0,    0,    0, 2797,    0,    0,  253,    0,    0,
    0,  232, 2851, 2854, 2838,  226,    0,    0,  859,    0,
    0,    0, -142,    0,    0,    0,    0,    0, 1701,    0,
    0,    0,    0,    0,    0,    0,    0,  859,  859,    0,
 2855, 1088,  804, 2477, 1570,    0,    0,    0,    0,    0,
    0, 2630, 2528,    0, 2462, 2532,    0,    0,    0,    0,
 2541,72088,    0,25321,  682, 2537,    0,    0,    0,99631,
    0, 1593,78388,  882,    0, 2423,99631,78388, 1679, 1785,
 1069,78388, 2423, 1794, 3235,    0,    0, 1348,    0, 1348,
    0,78388,78388,25321,  611,    0,78388,78388,78388, 2604,
78388,78388,78388,78388,78388,68938,68308, 3235,78388, 1819,
78388, 2873,78388,78388,78388,78388,78388, 2879, 2880,78388,
78388,78388,78388,78388,78388,78388,78388,78388,78388,78388,
78388,78388,78388,78388,78388,    0,    0,82168, 1824, 1824,
    0,    0, 2881,    0, 1655, 2179, 2883,    0,    0,84631,
    0,78388,78388,83428,    0,    0, 2885,  856,    0,78388,
    0, 1270,    0,    0,78388, 2495, 2507,    0,    0,    0,
69568,78388,99631,82798,82798,82798,82798,82798,78388, 2891,
78388,73348,94129,99631,77758,    0,    0, 2547,    0,    0,
 2571, 2572, 2575,28968, 2651,    0,  242,99631,85025, 1602,
    0,    0,    0,    0,    0,    0,    0, 2904, 2899, 2689,
 2641, 2644,   14,    0,    0,    0,  926,    0,    0,    0,
    0,    0,    0,99631, 2199, 1974,78388,99631,99631,    0,
 2423,    0,  575,    0,    0,    0,  575,99631, 2619, 2625,
99631,  575, 2906, 4779, 4855,    0, 2648, 2919,    0,    0,
 2697, 2918, 2921,    0,21637,    0,    0,36941, 1980,99631,
    0, -143,    0,    0,    0,    0, -109,    0,    0,  207,
78388,    0,99631, 2457,    0, 1230, 2474, 2478,  435, 2479,
  -52,    0, 2535,    0,    0, 1334,    0,  859,    0,99631,
99631,    0,99631,87780,    0,    0,    0, 2364,    0, 2938,
 2588,    0,    0,  184,    0,    0, 2941, 1498, 2615, 2616,
 2621,    0, 1348, 1515,    0,99631,    0, 2553, 2399,    0,
 2505,99631,99631, 2696,   32,    0,    0,    0,  181,    0,
    0,    0,    0, 1348,    0, 2836,  184,    0,    0,    0,
 4144,    0,    0,  184, 2599,  184,    0, 2962, 1348, 1348,
 1348,    0,    0,    0,    0, 2566, 1886,    0,    0,    0,
99631, 2658,    0,    0, 1383,    2,    0,    0,    0, 1273,
21609, 2400,30776,31390, 2637,    0,    0,    0,    0,    0,
 2970,    0,  910,99631, 2971,    0,    0,86601,99631,25321,
    0,    0, 2659,    0,    0,    0, 2516,    0,78388,78388,
22251,78388, 2976,78388, 2979, 2977, 4900,78388,78388,78388,
78388,25935, 2982, 2758, 2983,78388,78388,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 2711, 2986, 2988,
22865, 2613,78388,    0,78388,    0,22206,22820, 2989,99631,
 4961, 5030, 6267, 6291, 6881,    0,    0,    0, 4386,73978,
    0,    0,    0,    0,36328, 2998,  248,23434,78388,78388,
36967,    0, 6905,24048, 7495, 3225, 4453,    0,    0, 7519,
 8109, 8133, 8723,24662,25276,25890,26504,27118,27732,28346,
28960,29574,30188,30802,31416, 2295,    0,    0,    0,    0,
99631,88173,    0, 2999,    0, 3000, 1533, 5024,72088, 2770,
 1655,72088,    0,    0, 2730,    0,  572,    0,    0, 2718,
83428, 2885,78388, 2633,78388,27163,    0,78388,    0,    0,
    0, 3008, 2718, 2718,    0, 2777, 1437, 1437, 1824, 1824,
25321, 2439, 3016,    0, 2257,    0,    0, 3020, 2324,    0,
 2669,    0,    0, 2650,  525,  653, 2674,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2652,
 2661, 3028, 3034, 3038, 2027,    0,    0,    0,    0, 3042,
 1644, 3045, 2717,    0, 2717, 1477,78388, 2717, 2753, 2759,
78388,    0,99631, 2779,    0,    0, 2732, 2733, 2737,28968,
 2812,  335,    0,    0,    0, 3116,    0,    0, 2537,    0,
25321,    0,    0, 3062,    0,    0,    0,  575,  575,    0,
    0,    0,    0,85419, 2672,78388,99631,86206,78388, 2622,
78388,    0,    0,    0,    0,    0,    0,    0, 2084,88173,
78388, 2278, 2291,    0,99631,    0,    0, 2771, 2772,    0,
 2774,    0,    0, 2834, 2694,99631, 1334, 1727, 1181, 1199,
99631,21065,    0,    0,    0,    0, 2841, 3056,    0,    0,
90592,  448,    0,99631,   -6,    0, 3081, 2538,78388,    0,
    0,99631,99631,99631,104347,99631,78388,99631,    0,    0,
 2819,87387,    0, 3067,    0,    0,    0,    0,    0,    0,
 2799,    0,    0,    0,    0,    0,    0,  676,    0,    0,
    0,    0,    0,    0,    0, 2800,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 3071,    0,99631,    0,    0,
90592, 2691,    0, 2669,    0,    0, 2683,96487,    0,89373,
  867,  987,   88,    0, 2221, 2259,    0,    0, 2860,    0,
    0, 2719,    0, 3080, 3082,    0,    0,    0,    0, 1530,
    0,  221, 2314, 2726, 3102,    0, 2345,    0,    0,    0,
    0,    0,    0,    0,    0, 1596,    0, 3087,99631, 1650,
 1348, 3105, 3106, 3110,  226,    0,    0,    0,    0, 3093,
    0,    0, 2477,    0,    0,    0, 2221,99631, 2259, 2751,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2667,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 2552,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2556, 1993, 2513,    0, 2654, 2847,    0, 2719,99631,
99631,  910, 2859,  962,    0,    0,    0, 2775,    0, 2861,
    0,    0,    0, 3124,    0, 1974, 8747, 9337,90985, 9361,
    0, 9951,    0,78388,    0, 9975,10565,10589,11179,79018,
    0,    0,11203,11793,78388,    0,    0,78388,74608,78388,
25321, 2864, 2761,    0,78388,78388,78388,    0,    0,    0,
    0,    0,    0,78388,    0,78388,23479, 2752,78388,78388,
78388,78388,32030,32030,78388,    0,78388,    0,78388,    0,
78388,    0,    0,    0,    0,    0,78388,78388,78388,78388,
78388,78388,78388,78388,78388,78388,78388,78388,    0, 2361,
88173,99631,    0, 3127,83428, 3131, 2375,78388, 2862,    0,
    0,    0,78388, 2911,    0, 2718,78388, 2718, 2718, 2423,
 2902, 2537,    0,    0,75238,    0, 2384,99631, 3140, 2917,
    0,99631,99631,99631,99631,99631,99631,99631,    0,    0,
    0,    0,    0,    0, 2920,    0,    0, 2923,    0, 2091,
    0, 2091,    0,    0,    0,    0,99631,    0,    0,    0,
    0,    0, 2852,    0,    0,85419,85419, 2392,99631, 2904,
    0, 2650,  636,  738, 2874, 2875,    0, 3116, 2839,   61,
 1225,    0, 2782,    0, 2673, 2680, 2685,    0,    0,    0,
    0,    0, 2516,    0,    0,    0, 3147, 1644,78388, 2872,
    0,    0,    0,    0, 1059,    0,25321, -143, 2829, 2999,
    0, 1974,99631,    0,    0, 2513,  859,  859, 2772,    0,
    0, 2806, 2691,    0,    0, 3168, 2933,    0,    0,    0,
 2867, 2871,    0,    0,    0,    0, 2944,    0,    0,99631,
  757,    0, 2849,    0,    0,    0,    0,    0, 2603,    0,
 2934,75868,    0,78388, 2804,25321,    0,    0,    0,    0,
    0, 2895,    0,    0,25321,    0,    0,    0,    0, 2794,
    0,    0,88173,  972, 3189,88173,    0,    0, 2401,99631,
 2841,    0, 1498,96880,    0,99631, 1795, 1850,    0, 1683,
    0, 2863, 2868, 2869, 2886, 2894, 2896, 2580,    0, 2513,
 2865,  -29,    0,  153,    0,    0,    0,    0, 1530, 3093,
 3093,  859, 2884,94522,94522,    0, 3192,  221,    0,86601,
 2845,    0,  184,    0,    0,  184,    0,  184,    0, 2944,
 3204,    0,    0,    0,    0,    0,    0,    0, 2691,    0,
 2402,    0, 1566, 2580, 1681,    0,    0,    0,    0,    0,
    0, 3211,    0, 2654,    0, 2810,    0,  859, 2824, 2627,
  859,  859,  859,    0,97273,    0,    0,    0,    0, 3193,
  864,    0, 3217, 1052, 2975,    0,99631,99631,    0,    0,
    0,    0, 3238,28344,  859,    0,    0,    0,    0, 2916,
 3243,    0,    0,    0,    0,11817,    0,    0,    0,    0,
72088,12407,    0,    0,12431,24093,37555,    0,    0,78388,
 2857,13021, 2409,32644,13045,13635,78388,78388,33258,37566,
33872,34486, 2428,78388, 3248, 3253,24707,13659, 4574, 4617,
14249,14273,14863,14887,15477,15501,16091,16115,16705,16729,
17319,17343,    0,    0,    0,    0,    0,    0,    0,    0,
78388, 2718,83428,78388, 2718,78388, 3256,78388, 3258,    0,
 2456,    0,    0, 2841,    0,    0, 2221, 2888, 2889, 2890,
 2893, 2897, 2898, 2901,    0,    0, 1062,  122, 1966,    0,
    0,    0,    0,  243,97666,    0,103561,85419,    0,    0,
 2134,    0,    0,    0,    0,99631,99631,99631,99631,99631,
99631,99631,    0,    0,    0,    0,   -2, 2792,    0, 2887,
    0, 2887,    0,    0, 2914, 1348,    0,    0, 2877, 1974,
85419, 3037, 2992,    0,    0,    0, 2957,    0,    0,    0,
    0,    0,    0,    0,99631,    0, 2991,78388,    0,    0,
    0, 2815, 2815,    0, 4144, 4144,    0,98059, 2985,    0,
 3018, 3291,25321,    0, 3292, 3035,    0,88173,    0,    0,
 1638,    0,    0, 2999, 2850, 2799, 3040,    0,78388, 2999,
    0,    0,    0, 2841,  145,    0,99631,    0,    0,    0,
91378,    0,    0,    0,    0,    0, 2978, 1348, 2980,99631,
 3068, 3048, 2595,91771, 2818,    0,    0,    0, 3307,    0,
 2924, 3022, 2925, 3052, 3024, 3059,    0,    0,    0,    0,
    0,99631, 2713, 3032,99631, 3032, 3044,    0,  971,    0,
 2691, 1348,    0,  859,    0,    0,    0,    0,    0, 3321,
    0,    0, 2958, 2944, 3323, 2841,    0,    0,    0,    0,
    0,    0, 2719, 1749,    0,    0,    0,    0,96880, 2221,
 2259,    0, 2221, 2259,    0,    0, 2915,    0, 2922,99631,
    0,    0,    0, 3326,    0,    0, 2463,    0,    0,    0,
85813, 2489, 3327,    0,    0, 3328, 1039,    0,    0, 3014,
94915,98452,    0,    0,    0,    0,    0,78388, 3332,78388,
    0,78388,  107,    0,    0,78388,    0,    0,17933,17957,
78388,78388,78388,78388,78388,    0,25321,    0,    0,78388,
 3063,78388,    0,78388,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2718,
78388, 2718,    0, 2758,    0,    0,99631, 3315,99631, 2580,
    0,    0,    0,    0,    0,    0,    0,  245,92950,  245,
    0, 2920,    0, 2923,    0,    0,    0,    0,    0, 2936,
    0,  246,99631,    0,    0,    0, 1999,    0,    0,    0,
78388, 3339,    0,    0,99631, 3336, 3069, 3072, 3073, 3076,
 3078, 3079, 3083,    0,    0,  165,  731,    0,    0, 2963,
    0,    0,    0,99631,    0,    0, 3355,    0, 1644,78388,
78388, 2199,    0,    0,    0,99631,    0, 3358, 3005,    0,
    0,    0,99631,    0,    0, 3361,78388,79648, 2984,87780,
 2999,    0,    0,    0,    0,78388,78388, 3089,    0,87780,
    0, 3364,99631,  757, 2789,    0,    0,    0, 1238, 3068,
 2818,    0,  596,    0, 3128,    0, 3065,    0, 3074,    0,
    0,    0,    0,    0,   45,    0, 3143,78388,    0,    0,
    0,    0,    0,    0,    0,  -32,99631,    0, 3032,    0,
    0,  971, 3137, 3077,    0,    0, 1749,    0,    0,    0,
 2981,    0,    0,    0,    0,   92, 2513,    0,    0,    0,
    0,    0, 2772,    0,    0,    0, 2580,    0, 2580,    0,
    0,    0,    0,    0,  859,    0,    0, 3051,    0,    0,
 2906, 3381,    0,    0, 3118, 5626,28344, 1348, 1479,    0,
    0,  859, 2659,    0,    0, 2494,    0,93343,    0,99631,
    0,25321,    0,18547,25321, 3009,78388,78388, 3123,18571,
    0,    0,19161,19185,19775,25321, 4682, 4749, 1473,    0,
19799,20389, 2718,    0, 3366, 2500,    0, 2841,    0,    0,
    0,    0,    0,    0,    0,    0, 3111,    0,  246,99631,
99631,99631,99631,99631,99631,103954,103954,    0,    0,    0,
  246, 3061,97666,    0, 3160,    0,97666,    0,    0, 1974,
 3002,    0,99631,    0,    0,    0,    0,    0,    0,    0,
    0,  171,  171,    0,    0, 3393,    0, 3130,    0,78388,
 3408,    0,25321,    0,    0,    0,    0,    0,90592, 2669,
 3415, 2423, 2509,72088,    0,25321,87780,    0,25321,25321,
99631,  759,  820,    0, 2895, 3179,    0,    0,    0, 3145,
 3146, 2091,    0,    0, 3212,    0,    0,    0,    0,    0,
    0,    0,20413,    0,    0,    0,    0,93736,    0, 3085,
  859,    0,    0,   92, 3148,    0,    0,    0,    0, 2772,
    0,    0,    0, 2517,98845, 3443,  859,    0,    0, 3213,
    0,78388, 5626,  859, 1348,   55,    0,    0, 3448,    0,
    0,    0,94915,    0,    0,    0,    0,    0,    0, 3444,
    0,78388,    0, 3075,    0,    0,    0,    0,    0,78388,
    0,78388,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  245,  245,  245,  245,  245,  245,  245, 3129,92950,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 3002,  490,    0,    0, 2544,
    0,    0,    0,    0,    0,    0,    0,    0,    0,99631,
 1123,99631,    0, 2548,78388,    0,    0, 2841, 3230,99631,
 3456,    0,21003,    0, 3198, 2895,87780,    0,    0,99631,
99631,  221, 3058,    0, 3205, 3239,    0,87780, 2091, 2091,
 3023, 3215,    0,    0,    0,95308,    0,    0,    0,99238,
    0,99631,    0,    0,    0,    0,    0,99631,    0, 3451,
    0,    0,    0,    0,    0,  839,  -19,21027, 3474, 3159,
    0,  533,    0,    0, 2944,    0, 3477,99631,99631,    0,
78388,25321,21617,99631,  246,  246,  246,  246,  246,  246,
  246,    0,    0,    0,    0,    0, 2959,    0,    0,  171,
  171,  171,  171,  171,  171,  171,    0,    0,    0,  766,
    0,    0,    0,    0,  817,  145, 2221,    0,    0, 3478,
    0,    0,87780,    0,    0,  820,    0,    0,    0,    0,
    0,87780, 3233,    0,    0,    0,    0, 3134, 3023,  123,
    0, 3264,    0,    0,    0,    0,    0,    0, 3021,98845,
    0,  859,    0, 2964,  431, 1035, 2947,    0,    0,    0,
 3033, 3036,    0,    0,  859,    0, 3084,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2973,  817, 3093, 3274,    0, 3277, 2580,
    0,    0, 1079,    0,    0,    0,99631,99631,    0, 3218,
 2926, 2841,    0, 2841,99631,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,99631, 3219,    0,
   22,    0, 1974,    0, 2691, 3170, 2691, 2221,    0,    0,
 1079, 3125, 3234,78388, 1567,78388,    0,    0, 2091,99631,
 3512,    0,    0, 3270,    0,    0, 2550,    0, 3515,    0,
   70,78388,  997,    0, 2458,    0, 1749,    0,    0, 2580,
 2554, 2555,78388,    0,78388,80278,25321, 3136, 3173, 3141,
25321, 3194,    0, 1616,    0, 3270,    0,    0,99631,    0,
    0,78388, 3237,    0, 3142,    0, 3164,    0,    0,    0,
    0,    0,    0,    0,    0,    0,25321, 2965,    0,25321,
 2990,25321,    0,    0,    0, 1040, 3023, 3206, 3208, 3532,
    0,   82,    0,99631, 3161,    0, 1085,99631,99631,78388,
78388,78388, 1411, 1411,    0, 3284,    0, 3156,    0,    0,
 3246,99631, 3023,    0,    0,    0, 2577, 3254,    0,    0,
 3182, 3541,    0,    0,25321,25321, 3149, 1472,    0,    0,
    0, 3023, 2091,    0,    0,    0, 3165,99631,    0,    0,
    0,    0, 3194,    0, 3555, 3541, 3023, 3172,78388,    0,
 3557, 2593,78388,    0, 2600,    0,
};
static int yyrindex[] = {                                   3200,
    0,  254,  102,    0,    0,    0,    0,    0,    0,    0,
    0,  205,    0,    0,    0,    0,    0,    0,    0,    0,
 3601,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  238,    0,    0,  498,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  975,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 3273,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  975,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,67487,    0,    0,    0, 3209,    0, 3340,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1296,76498, 1166,    0,
    0,    0,  518,    0,  395,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  174,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  840,    0,    0,
  395,    0,    0,    0, 3341,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  979,    0,    0,    0,    0,
    0,    0,    0,    0,28385,    0,    0,    0,    0,    0,
  175,    0,    0,    0,    0,    0,  633,    0, 3568,    0,
 1502,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 3346,    0,    0,    0,    0,    0,    0,
    0,    0,  334,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1263,    0,    0, 1538,    0,    0,    0,    0,
    0,    0,    0, 1610,    0,  315,    0, 5197,    0,92164,
    0,    0,92557,101203,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 3216,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 3346,    0, 1620,
    0,    0,    0,    0,    0,    0,101596,    0,    0,    0,
101989,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  512,    0,    0,    0,    0,
77128,70198,    0,    0, 3222,  199, 1721, 3200,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  563,
 3342,    0, 1764,    0,  173,    0, 1800,    0,    0, 3342,
 1764,    0,    0,    0,    0, 1021,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1800,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1778,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  369,
 1721,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,40219,    0,
    0,    0,    0,    0,    0,    0,    0,    0,40833,    0,
    0,    0,    0, 7063,    0,    0,    0,    0,    0, 5835,
 6449,    0,    0,    0,    0,37763,    0,    0,    0,41447,
42061,    0,    0,    0,    0,42675,43289,    0,    0,    0,
    0,    0,    0, 7677, 8291, 8905, 9519,10133,10747,11361,
11975,12589,13203,13817,14431,15045,15659,16273,16887,38377,
38991,43903,17501,18115,18729,19343,44517,19957,20571,21185,
21799,22413,23027,23641,39605,24255,24869,25483,26097,26711,
27325,27939,28553,29167,29781,30395,31009,31623,32237,32851,
33465,34079,34693,35307,35921,36535,37149,45131,45745,46359,
46973,47587,48201,48815,49429,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 5835, 6449,    0,    0,
 3200,    0,    0,    0, 1827,51890,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1046,56802,    0,    0,    0,    0,  351,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,50043,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,50662,    0,  658,    0,
 2557,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1342,    0,  334,    0,    0,    0,    0,
 1981,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 3282,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  208,    0,    0,    0, 1721,
    0,    0,    0,    0,  247,    0,    0,    0, 1855,102382,
    0, 3347,    0,    0,    0,    0,    0, 3224,    0,  255,
    0,    0,    0,  532,    0,    0,    0,    0,  995,    0,
    0,    0, 3200,    0,    0,    0,    0, 3322, 2090, 3908,
 2937,    0,    0,    0, 3262,    0,    0, 3527,    0,    0,
 3704,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  884,    0,    0,    0, 2329,    0,    0, 2840,    0,    0,
    0, 4015, 3369,    0, 3704,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  395,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1502,    0, 1323,    0, 1327,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 3200,    0, 2304, 3584,  210,    0,    0,    0,    0,
    0,80908,    0,80908,    0, 3200,    0,    0,80908,80908,
64591, 3587, 3200,80908,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 3195,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,80908,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 3200,53118,53732,
    0,    0,    0,35100,35714,    0,    0,    0,51276,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,81538,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 3587,    0,    0,    0,  315,    0,    0, 4212,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 3200, 3295,
    0,    0,    0,    0,    0,    0,    0,65854,    0,    0,
    0,    0, 2557,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  369, 1721,    0,    0,    0,    0,
 3200,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  992,    0,    0,    0,    0, 3317,    0,    0,
67883,    0, 1289,    0,  942,    0,    0,  214,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1861,
    0,    0,    0,    0, 1327,    0, 3294,    0,    0,    0,
    0,    0,    0,    0,  231,    0,    0,    0,    0,    0,
    0,    0,    0, 1049,    0,    0,    0,    0,    0,102775,
    0,    0,    0,    0,    0,    0, 1882,    0, 1135, 1182,
 1276,    0,    0, 1901, 1327,    0,    0, 1915,    0,    0,
    0,    0,    0,    0, 3335,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 3527,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1800,    0,    0,    0,
    0,  492,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1304,    0,    0,    0,    0,    0,
    0,    0, 3338,    0,27730,    0,    0,    0,    0,    0,
    0,    0, 3595,    0,    0,    0,  211,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2606,    0, 3598,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
88781,88792,88851,89058,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1952,    0,    0,    0,65012, 3200,    0,
57416, 3200,    0,    0,    0,    0,    0,    0,    0,62328,
    0,    0,    0,    0,    0,65433,    0,    0,    0,    0,
    0,    0,58030,58644,    0,52504,55574,56188,54346,54960,
 1108,  285,    0,    0,    0,    0,    0,    0, 1827,    0,
 3376, 5636,38755,103168,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 3275,29582, 3360,    0,    0,    0,    0,    0,    0,
 3295, 2152, 3319,    0, 3319, 3295,    0, 3319,    0,    0,
    0,66682,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2103,    0,    0,    0,  210,    0,
 1056,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  513,    0,    0,    0,    0,  474,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  982,    0,    0,    0,
    0, 3367,90199, 2180,    0, 2225, 2308,    0,    0, 2244,
 2303, 2365,    0,    0,    0, 2496,    0,    0,    0,    0,
    0, 1704,    0,    0,    0,    0,    0,    0,    0,    0,
 3596,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2348,    0,    0,    0,
    0,    0,    0, 1057,    0,    0,89786,    0,    0,    0,
    0,    0,    0,    0, 3342, 1764,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  294, 2320,    0,
    0,    0,    0, 4053,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1754,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 3226,
    0,    0, 1705,    0,    0,    0, 3342,    0, 1764,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1275,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1501,    0, 1961, 1493,    0,  403,    0,    0,    0,
    0, 3338,    0, 3356,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1721,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  679,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 3617, 3617,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,26549,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,62942,    0,59258,63556, 3200,
    0, 3619,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  765,38793,    0,  842,    0,  270,
    0,  270,    0,    0,    0,    0,    0,66272,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,65854,
    0,103168,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 2344,    0,    0,    0,    0,    0,    0,    0,
    0,  202,  211,    0,    0,    0, 1258,  244,    0, 1852,
    0,    0,    0,    0,    0,    0, 1006,    0,    0, 2075,
    0, 1721,    0,    0,    0, 1961,    0,    0,  330,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 3806,    0,    0,    0,
 3602,    0,  204,    0,  180,    0,    0,    0,    0,    0,
    0, 3608,    0,    0,    0, 3610,    0,    0,    0,    0,
    0, 3612,    0,    0, 3613,    0,    0,    0,    0, 3371,
    0,    0,    0, 3200,    0,    0,    0,    0,    0,    0,
  161,    0,    0, 1286,    0,    0, 1629,    0,    0,    0,
 1127, 1423, 1460, 1512, 1762, 1836, 1985,    0,    0, 1961,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 3226,
 3226,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 4098,    0,    0,    0, 1005,    0,    0,    0,    0, 2960,
    0,    0,    0,    0,    0, 1327,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 3280,    0,    0,    0,    0,    0,    0, 3635,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 3200,    0,    0,    0,    0, 3636,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,59872,    0,    0,64170,    0,    0,    0,    0,    0,
    0,    0,    0, 3622,    0,    0, 3342,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  194,    0,    0, 1615,    0,    0,    0,
 3295,    0,    0,67092,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1839,
    0, 1839,    0,    0, 2785,    0,    0,    0,    0, 1721,
    0,    0, 1397,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  607,    0,    0,    0,
    0, 3224,  707,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 3623,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  828,    0,    0,    0,    0,    0, 3625,
    0,    0,    0, 1329, 1086,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 3638,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2082,    0, 3373,    0, 3373,    0,  318,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 4282,    0, 2086,    0,    0,    0,    0,
    0,    0,    0,  542,  774,    0,    0,    0, 1286, 3342,
 1764,    0, 3342, 1764,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  581,    0,    0,    0,    0,
    0,    0,27771,    0,    0,    0,  278,    0,    0,    0,
 3584,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 3382,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 3647,    0,    0,    0,
 3086,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,60486,
    0,61100,    0,  455,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  200,    0,  200,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2097,    0,    0,    0,    0, 2570,    0,    0,    0,
    0,    0,    0,    0,    0, 2099,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2097,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  438,    0,
    0,  305,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1016,
 1007,    0,    0,    0,    0,    0,    0,    0,    0, 1016,
    0,    0,    0, 2607,    0,    0,    0,    0,    0, 2237,
 3337,    0, 4296,    0, 2522,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1884,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 3373,    0,    0, 3373,    0,
    0,    0,    0,    0,    0,    0, 2910,    0,    0,    0,
    0,    0,    0,    0, 1011,    0, 1961,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 2111,    0,    0,
 2118,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,83842,84236,    0,    0,    0,    0,    0,    0,
    0, 3656,    0,    0, 3281,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2629,    0,    0,    0,    0,
    0,    0,61714,    0,    0,    0,    0, 2638,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2097,    0,
    0,    0,    0,    0,    0,  378,  439,    0,    0,    0,
 2097,    0,    0,    0,    0,    0,    0,    0,    0,  237,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2097, 2097,    0,    0, 2793,    0, 3091,    0,    0,
 2018,    0, 1151,    0,    0, 1654,    0,    0,    0, 3376,
    0, 3200,    0, 3200,    0,  778, 1016,    0, 3384, 3388,
    0,    0,    0,    0, 3439,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 2124,
    0,    0,    0,    0,    0,    0, 2128,  719,    0,    0,
    0,    0,    0,    0,    0,  516,    0,    0,    0,    0,
    0,    0, 3584,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  200,  200,  200,  200,  200,  200,  200,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1177,    0,    0,    0,    0,    0,    0,  140,    0,    0,
    0,    0,    0, 1175,86994, 3612, 1016,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 3406, 1016,    0,    0,
 2005,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1333,    0,    0,    0,    0,    0,
    0, 2645,    0,    0, 2097, 2097, 2097, 2097, 2097, 2097,
 2097,    0,    0,    0,    0,    0,    0,    0,    0, 2097,
 2097, 2097, 2097, 2097, 2097, 2097,    0,    0,    0,    0,
    0,    0,    0, 2073,    0, 3442, 3342,    0,    0,    0,
    0,    0, 1016,    0,    0,    0,    0,    0,    0,    0,
    0, 1016,    0,    0,    0,    0,    0,    0, 2005, 3273,
    0,    0, 4561, 4561,    0,    0,    0,    0,  534,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  562,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  252,    0, 3226, 3265,    0,    0,    0,
    0, 1295,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 2037,    0, 2037,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 2647,    0,
    0,    0, 1721,    0,    0, 3266,    0, 3342,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  276,    0,
 4654,    0, 5014,  956,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2181,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1586,    0,    0,    0,
 1742,  268,    0,    0,    0,  956,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 2653,    0, 3676, 1875,
89212, 2275,    0,    0,    0,    0, 2005, 1844, 3607,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 6222,    0,    0,    0,    0,    0,  202,    0,
    0,  201,    0,    0, 2655, 2398,    0,    0,    0,    0,
    0, 6222, 3621,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 4136,  202,    0,  236, 6222,    0,    0,    0,
    0,    0,    0,    0,    0,    0,
};
static int yycindex[] = {                                      0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 3190,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1787,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 3070,
    0,    0,  120,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 3240,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 3665,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 3669,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 3683, 3684,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  779,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 3421,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  584,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1686,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 3681,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 3424,    0,    0, 3425,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 3687,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 3468,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 3370,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  595,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 3430,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 3220,    0,    0,    0,
    0,    0,    0,  615,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 3377,
    0, 3380,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 3706,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 3221,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 3162, 3163,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 3705,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 3493,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 3716,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,
};
static int yygindex[] = {                                      0,
    0,    0,    0,    0,    0,    0, 2096, 2475,    0, 2476,
 2480, 1384, 2481,    0, 1388, 2485,    0, 1390,-1914,    0,
 2263,  883,    0,    0,    0,    0,-1969,  741,  742, 2490,
 1716, 1371,  894,    0,    0,    0,-2474,    0, 1499,  967,
    0,    0,  796,    0,    0, 2328,    0,    0,    0,    0,
    0,    0, 3447,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  996,    0,    0,    0,  831,-1582, 1055,-1678,
  717,    0,  465, -537, 3107,    0, 3737, 1614,    0,    0,
 -893,    0,    0,    0,    0,    0,  691,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1597,    0,    0, 1036,
-1980,    0,    0,  564, 3466, 2848,    0,    0, 1548, 1023,
 1271,    0,    0,-2200,    0,-2051,    0, -163,  625,    0,
  486,    0, 1274,-2045,    0,-2184,  401,  456,  458,  444,
   94,-1724,-1660,-1181,    0,    0, -980,-1675,    0, 1096,
    0,  696,-1326, 1099,  109,-1587,-1522,-1152,  666, -446,
 -638,    0,    0,   84,    0, 2381,-1209,    0,    0, 3241,
 1923,    0, 1922, -658,    0, 2816,    0,    0,    0,    0,
-1662,-1012,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,-2586,  906, 1668, 1669,    0,    0,    0,
  467,    0,  464,    0,    0,  557,    0,    0,    0,    0,
 -926,-1429,    0,    0,    0,    0, 2105,    0,  762,    0,
 1682,    0,  677,    0,    0,    0,    0,    0, 3864, 2601,
 2250,    0,    0,    0, 1221,   96,    0,    0,    0,  -72,
    0, -340, 2425,    0, 1959,    0, 3197, 2760, -973,    0,
    0,    0,    0, 2147,    0, 1684,    0, 2065,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 2520,    0,    0,
    0,    0,    0,    0,-1735,-1668,    0,    0, -659,    0,
 2454, 1642,-2131, -648,    0,    0,    0,-2099,    0, -610,
    0, 2784,    0, 2465,    0,    0,    0,    0,    0,    0,
    0, 3750,    0,    0,    0,    0, 1798,    0, 1470,    0,
 1471,    0,    0,    0,  822,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 2903, 3260, -837,    4,    0,    0,
    0, -619,    0, 3551, 3303,-1418,    0,    0, 1370,    0,
    0,    0, 3259,    0,    0,    0, 2204, -848,    0,    0,
-1147,-1560,    0,    0, 1226,    0, 2623,    0,    0,    0,
    0,    0,    0, 1825, 1760,   97, 2251,    0,    0,  834,
    0, -806,-1188, 2765,    0,  903,    0,    0,  527,    0,
-1010,    0, 2214,-1308,    0,    0, 3268, 1812,    0,    0,
 3916,    0, 3917,    0,  865,  868,    0,    0, 3923,    0,
    0, 3925,    0,    0, 3926,    0, -722,    0, 2961,-2024,
 -587,    0,-1937,-2365, -970,-1976,  604,    0,-1102, -525,
-1197,-1133, -867,    0,    0,    0,    0,    0,    0,    0,
-1430,    0, 2688,    0,    0,    0,    0,    0,    0, 2341,
    0,    0,    0,    0,    0,-1211,    0, 1667,-1954,    0,
  722, 3214, 3251, 2498, 3199, -560, -272,    0,  833,    0,
    0,-2202,-1740,    0, 2262, -843, 2231,    0,-1057,-1287,
 3064, 3066,    0, 2681, -860, -849, 2611, -920, -928, -608,
-1279,-1271,-1269,  -70,  -59,-1701, -715,    0,  -97, 1152,
-1239, -632,    0,-1335,    0,    0,    0,    0,    0,    0,
 1971,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,-1332,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1097,    0,-1833,    0,    0,
 -880,-1267,    0, -108, 1718, -708, -115,    5, 1467,-1392,
    0,  -45, 3847,    0,   47,  -51,-1086,  535, -462,  -98,
  -53,   10,    9,  -44,  -27,-2387, 1074, -181,   31,  -14,
    0,    0, 3833,    0, 1883,  157,  166,    0,    0, 2690,
  427,    0,    0,    0,    0,  129, 1556,    0, -868,-2317,
    0, 2842,    0, -667,-1190,-2199,    0,    0,    0,    0,
   90, -344, 1391,    0,    0,-2083,-2595,    0,    0,    0,
-1957,    0,  592,    0,    0,-1340,    0,    0,    0,    0,
 -582,    0,-1661,    0,-2532,    0,    0,    0, 2060,    0,
    0,    0,    0,    0,  878,    0,    0,    0,  710,  638,
 1500,    0,    0,    0,    0,    0,    0,  340, 1268,    0,
    0,    0,    0,-2156, -953,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 2946,    0,    0, 2966,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 3983,    0,    0,    0, -707,    0,    0,    0,    0,
 1157,-1196, 1048,    0,    0,    0,    0,  767,    0,    0,
-1491,    0,    0,    0, 1320,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  711,    0,    0, -674,-1598,
 -315,    0,    0,    0,    0,    0,    0,-1315,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,
};
#define YYTABLESIZE 104996
static int yytable[] = {                                     343,
  349,  559,  362,   46,  967,  571, 1000, 1003, 1147, 1127,
  512, 1042, 1138, 1224,  562,  476,  563,  462, 1375,  497,
 1226,  513,  342, 1323,  348,  488, 1700,  576, 1020, 1385,
  999, 1002, 1043, 1082,  671, 1219, 1220, 1225, 1794, 1512,
  536, 1795, 1126,  491, 1085, 1519, 1737,  460,  463, 2138,
  471,  474,  477,  484,  343,  489,  492,  343,  498,  474,
  561,  556,  505, 1386,  533,  569,  968,  516,  463,  471,
  474,  565,  484, 1026,  343,  489, 1931,  492,  343,  498,
  474,  540, 1088, 1278,  505, 1345,  515, 1341,  552,  566,
  343,  555,  463, 1344,  552,  471,  474,  477,  484, 2179,
  343,  489,  492,  568,  498,  474,  540,  504, 1059,  505,
  341,  578,  473,  342,  343,  343, 1679, 2126,  474, 1917,
  473,  589, 2392,  564, 2393, 2429, 2318, 1575, 2192,  504,
 2227, 1651, 2191, 1839, 1697, 1477, 2531, 2532,  342, 2485,
 2550, 1841, 2010, 1716, 1803, 1382, 1919, 2143, 2444, 1842,
 1450, 1843, 1052, 1844,  575, 1714,  577,  560, 1097, 1305,
 2024, 1131, 2589, 2016, 1343,  485,  570, 2219, 1349,  988,
  990, 2441,  282,  284,  271,  677, 2649, 1837, 2278,  586,
  523,  610,  545, 1817, 1848,  529, 1848,  543,  534, 1800,
  716, 2972,  519,   94, 2526, 1827, 2282,  720, 1261,   92,
 1287,  341, 1433, 2302, 1038, 1616,  998, 1265, 2671, 1243,
 1257, 1001,  573, 1255, 2878, 2820,  282, 2119, 2210,  396,
  957, 1825, 1683,  957, 1832,  581,  341,  737, 1802,  883,
  779,  282,  284,  271,  883, 1288, 1310, 1123,  957, 1261,
  957, 1327, 1724, 1167, 1782,  733,  764, 1782, 1265, 1824,
 1711, 1307,   94, 1038, 1255, 2518, 1302, 1261,   92, 1287,
 2283, 1221, 1782, 1038, 1782, 1135, 1265,  430, 1243, 1257,
 1297, 1429, 1255, 1268, 1224, 1321, 3221, 2547, 1123, 1864,
 2120, 2743,  604, 3322, 1167, 1647, 2671, 1167, 2878,  779,
 2752, 1981, 2777,  336, 1288, 1310, 1123, 1352, 1225,  276,
  739, 1494, 1167, 1573, 1215,  764, 1536, 1537,  430, 1589,
 1307,  430, 1038, 1321,  651, 1304, 1321,  337,  163, 1321,
 1269,  163, 2648, 2780, 2501, 1609,  430, 3132, 2418,  912,
  349, 3352, 2279, 1213, 1321,  463,  471,  474,  484,  343,
  489,  343,  474, 1426,  505, 1215,  126, 1402,  362, 1355,
  652, 2996,  336, 2893,  348,  660, 1704, 2136, 3096,  652,
 2170, 2171, 2172, 1215, 1435, 1707,  233, 2136, 1215, 1419,
 1013, 1577,  644,  651, 1566, 1395,  337, 1781, 1647,   13,
  126, 1963, 2662, 2206, 2827,  681, 2697,  546,  912,  504,
 1581,  658, 1213, 3077, 1413, 1708, 2520, 2280,  233, 1865,
 1494, 1615,  296,  396, 2093, 1346,   79, 2663, 2847,  652,
 1606, 1607, 1608, 1609, 1610, 2523, 2660, 2207, 1403,  547,
 2521, 1781, 2664,  645,  396, 1278, 1794, 1215, 2894, 1795,
  233, 1420, 2744, 1398, 2894, 2686, 1781, 1168, 1956, 3323,
  727, 1436,  609,  957, 2792, 2975,  602,  694,  732,  471,
  529,  605, 1509, 1413, 1510, 2975,  957,   94, 2449,  600,
  127,  296, 1261,   92,  739, 2079, 3401, 1782, 1052, 1575,
  396, 1265, 3402, 1251,  957, 1685, 1408, 1255, 1168, 1686,
 1782, 1168, 1956, 2163, 1691,  682, 2601, 3353,  781,  957,
 1712,  246, 1345, 2419, 2794, 1610, 1168, 1956, 1782, 3184,
 1344, 1123, 3133,  396,  326, 3403, 1705, 1167, 2745, 2894,
 1261, 1125, 1213, 1782, 1251, 2400,  326, 1251, 2996, 1265,
  734, 1014, 1221, 2074,  396, 1255, 2540, 1425, 2541,  505,
 1261,  781, 1251,  341,  471,  963, 1485, 1268, 1710, 1265,
 2848,  357,  471, 1495, 3175, 1255, 2108,  326, 2786,  326,
  246, 3222, 1125, 1213, 2958, 1167,  173, 1327,  740,  505,
 3040, 1980,  902, 2282, 2961, 1167, 1167, 1782, 1215, 1123,
 1125, 1213,  477, 2965,  724, 1167, 2967,  971, 1199,  349,
  299,  349,  976, 1680, 2357, 1866, 3101,  912,  317, 2796,
  451, 1725,  341,  505, 1676, 2505,  986, 1038, 1221,  912,
  357,  970,  175,  972,  741,  348,  923, 1575,  997,  492,
 2112, 2367,  326,  997, 2964, 1167, 1215,  989,  989,  989,
  989,  902, 1831, 1038, 1167,  997, 2437, 2283, 1326, 1106,
  912,  781,  912,  317, 1287, 1261, 1215, 1321,  724,  299,
 1616, 1781,  512, 1321, 1265, 1573, 1038,  326, 2087,  781,
 1255, 1413, 1099, 1120, 1978, 1270, 1609,  326, 2022, 1650,
 1646, 1428,  524,  516, 1674,  923, 1167,  997, 2103, 1288,
 1278,  740,  163, 1133, 1123, 1109, 1215, 1814, 1167, 2670,
 1167, 2877,  781,  463,  471,  474,  484,  489,  492,  343,
  474, 1684, 1296,  505, 2589, 2589,  276, 1261, 1835, 2279,
  343, 1168, 1956,  652,  343, 1213,  729,  740,  651, 1243,
  477, 2014, 1827, 1140, 1213, 1413, 1678, 1261,  300, 1287,
 1413, 2017,  740, 1122, 2912, 2913, 1265, 1215, 1243, 1257,
  912, 3359, 1255, 3198, 3199, 1225, 2291, 1251,  504, 2284,
 1215, 1215, 1167, 1167,  652, 2891, 1103, 1581, 2524, 1168,
  430, 1103, 1138, 2474, 1288, 2754, 2480, 2895, 1321, 1168,
 1168,   15, 1167, 2895, 2280,  296,  396, 2281, 1439, 1168,
 1307, 2746, 1199,  248, 1413, 1125, 1213,  300,  968, 1295,
 1345, 1445, 1341, 1573, 1916, 1251,   94,  396, 1344,  317,
 1413, 1115,   92, 1438, 1104,  451, 1114, 1929, 2525, 1110,
 2078, 2786, 3088, 1215, 1215, 1251,  957, 1100,   90, 1168,
 2159,  537, 2798,  326, 1781, 1129, 3031, 2456, 1168,  173,
  282,  284,  271, 1215, 1213, 1830, 1610,  968,  341, 3144,
 1782,  246,  248, 1213, 1213, 1213,  576, 2679, 2895,  678,
 2043,  451, 2979, 1125, 1213,  296,  296,   43,  968, 1265,
 2105, 2106, 1213, 3157, 1288, 1687,  326,  326, 1690, 1343,
 1168, 2589, 2121, 2484,  923,  175, 2782, 1229, 1215, 2019,
 1818, 2397, 1168,  341, 1168, 1956,  923, 1413,   91, 3226,
 1859,  357,  296,  266, 1213,   91,  574, 1215, 2110, 2253,
 1421, 1227, 2595, 1213, 1167,   81,  133, 1268,  678, 1856,
 3032, 3033, 3034, 3035, 3036, 3037, 3038,  923, 2355,  923,
 1251, 2044, 1513, 1514,  241, 2020,  512, 1326, 1314,  902,
 2763,  659, 2215, 1483, 1268, 2440,  451, 3182,  296, 2401,
 1721, 2553, 2639,  484, 1268, 1213, 1168, 1168,  578,  912,
  912, 1248,  266,  292, 1326, 3368,  362, 1816, 1125, 1213,
  343, 2407, 2038, 2797, 1326, 1215, 1168,  512, 2046, 1221,
  955, 2245, 1221,  343,  512,  512, 2811, 1019, 1367,  296,
  451, 2047, 1251, 1251, 1040, 1378, 1379,   15, 1042,  516,
 1727, 1727, 1248, 1289, 3279, 1248, 1413, 2184, 1413, 1413,
  134, 1126, 1251, 2545,  751,   81, 1848,  135, 2549, 2444,
 1248, 3368,  710, 1869,  256, 1256, 3227,  923, 3280, 2420,
  257, 1213, 1213, 3013, 3269, 1332, 1471,  508,  509, 1377,
  929, 2369, 2408,  299,  299, 3263, 3176,  136, 1434,  341,
 2465, 1213, 1126, 1040, 2552, 3228, 1472, 1042, 2731, 1413,
  396,  576,  396, 2410, 1515, 1518, 1256,  512,  512, 3158,
 1126, 2455,  659,  751,   90, 1318, 2411, 3087, 1442, 1443,
  292, 1303,   34,  256, 1256,  569, 2160, 2938,  601,  257,
 2953,  105, 2145,  606, 1129,  241,  241, 2944, 1722,  929,
 1575,  302, 3089, 1575, 2354,  956, 1518,  711, 1168, 1518,
 1473, 3160, 3161, 3162, 3163, 3164, 3165, 3166,  678, 1318,
 1346,  451,   45,  678, 1518, 2323,  292, 1519,  678,  576,
 2216, 2185, 1727,  249, 1318,  454, 1138, 3145, 3146, 3147,
 3148, 3149, 3150, 3151, 1251, 3090,  574,  661, 1249, 1248,
 1250,   91, 2190,   93,  738, 1478, 3091, 1605, 1518, 2045,
 2239, 1004,  474, 3333,  472,  576, 2955,  292, 1519,  678,
 1210, 1519, 2422, 1585, 3251, 1618, 2208,  576, 2048,  266,
  266,  300,  300, 1213, 3256, 1243, 1519, 1839, 1839, 1674,
 2176, 1581, 3295,  662, 2371, 1841, 1841, 2209, 3140, 1482,
 1288,  736, 2246, 1842, 1842, 1843, 1843, 1844, 1844,  129,
   37, 1210, 3395,  738, 1210, 2517, 2202, 2395,  302, 1586,
 1519, 3214,  668, 1486, 3252, 1248, 1243, 2685, 2254, 1210,
  857,  858, 2021, 2297,  576,  971,  923,  923, 3424, 1587,
 3312,  678, 3315, 1446, 1243, 1920,  137,  576, 3257, 3092,
 2256,  422,  968,  586, 1474, 2817,  968, 3432,  540, 1576,
  736, 2562,  512, 2412,  302, 2218,  512,  266,  516,  471,
 2409,  512, 3440, 1248, 3079, 1126, 1573, 1166,  751, 1573,
 3078, 1735,  952,  498,  578, 2783,  130, 1617, 3348, 1256,
  569,  669, 1736, 1248, 3085,  737,  396, 2287, 2475, 1082,
  254,  256, 2204, 2709,  929,  302,  259,  257, 1244,  971,
 1085, 2203, 3356,  516,  976, 1127, 1921,  512, 1166, 1370,
 3348,  508,  509,  989,  751,  569,  989, 3232, 1734, 1289,
 2128, 2829, 1682,  970, 2718,  670, 1166, 1256, 3215,  422,
  454,  952, 2868, 1126,  363, 1703, 1415,  610, 1088, 1244,
  929, 1852, 1853, 1854,  737, 3393, 1127, 1256,  971, 1234,
  422, 1049, 3302, 1447,  674, 2742,  422, 1244, 2288,  472,
 1448,  523, 1235,  587, 1127,  471,  484, 2057, 1743, 1797,
  610,  662,  970, 3412, 3216, 1575,  454,  997,  254,  396,
 1371, 3342,  396,  997,  259, 1389,  997, 1741, 1248,  422,
 3410,  516, 2830,  970,  364, 1415, 1236,  471,  484,  396,
 3190, 2059, 1673, 1237, 1794,  472, 1211, 1795,  738, 2439,
 1049, 1238,  929, 3436, 1794, 3217, 1753, 1795,   15, 1390,
 1221, 2205, 1372,  675, 1210, 1131, 2484, 1518, 1239,  674,
 1240, 3357,  743,  661, 3185, 1248,  740, 1318, 1126, 1243,
 1241,  365, 3427, 1243, 1244, 3197, 1318, 1211,  571, 1518,
 1248, 1248, 1256, 1224,  738,  736, 3303,  366, 1131,  343,
 2017,  610, 2060, 1140, 1932, 1211, 3093, 3438, 3304,  744,
 1248, 2061, 1210, 2139, 3394,  368, 2784, 1225, 1119,  662,
  372, 2719, 2720, 1210,  715, 2140, 2989, 1243, 1257, 1519,
 1246,  743, 1210, 1258,  610, 3433, 2062, 1357,  675, 1318,
 1318,  736, 1416, 1421,  857,  858, 1588, 1243,  374, 1358,
 2446, 1519,  738, 2420, 1391,  474,  586, 2644,  810, 3411,
 3168,  740, 1138, 2136, 1138, 3305,  738, 3210,  744, 1581,
 3264, 1166, 1210, 2925, 1256, 3169, 1345, 1345, 3306, 3266,
 1359, 1210, 1360, 2434, 1344, 1344, 1867,  947,  725,  737,
 1591, 1573,  571, 2435, 2141,  810, 2395, 2902, 3086,  736,
 2950, 1416, 1244, 1235,  571, 1318,  516,  971,  588, 1127,
 1926,  422, 2142,  736, 2080, 1077, 1968,  681, 1004, 1166,
  740,  508,  509, 1210, 1318, 2009, 1259, 2924, 1753, 1166,
 1166, 1576,  662, 1404,  725,  737,  751, 1210, 3281, 1166,
 1339, 1728, 1248, 2421, 1592, 1288,  947,  968,  422, 2125,
 1244, 1794, 1243, 1340, 1795,  997, 1005,  464,  883, 1948,
 2821,  571, 3282,  681, 1175, 1077, 2554,  512,  512, 1031,
 1244, 1593, 2951,  396,  571, 2013,  587, 1127,  896, 1166,
 1361,  375,  751, 1753, 1729, 2529,  997, 2869, 1166, 2236,
 1868, 1046,  725,  737,  594,  725, 2422, 1404,  484, 1210,
 1210, 1047, 1404,  347, 1175, 1175,  725,  737, 1175,  500,
 1211, 2952,  588, 2423, 1243, 1243, 1247,  522, 1948, 1210,
  422,  681,  896, 1175,  588, 1045,  997, 1753, 1031,  578,
 1166, 1246,  976,  516, 1243,  681,  743,  896, 2237,  506,
  751, 1221, 1166, 2236, 1166,  971, 1257, 1255, 2162, 1256,
  540, 1258, 1617, 3404, 1045, 1332, 1404, 2708, 1211,  422,
 2706, 2134,  347, 1253,  506,  584,  516, 1254, 2650, 1576,
 1310, 1010, 1404,  744, 1289, 1244,  516,  997, 1211,  463,
 1045,  588,  743, 1045, 1730, 2144, 2715, 2169, 1932, 1932,
 1932, 2169, 2240,  971,  588, 2158,  670, 1797, 1045, 1418,
 1123, 1794, 1731, 2241, 1795, 1806, 1166, 1166, 1807, 1011,
  678,  739, 1794,  477,  727, 1795, 3430,  970, 1211,  744,
  678,  970, 1046, 3417, 1045,  740, 1166, 1211, 1815, 1310,
  997, 1310, 1047, 2188, 1309, 1732,  516, 1244, 1244,  524,
  743, 1653,  662,  516, 2982, 2201, 2983,  970, 1808,  274,
  595, 1210,  549, 1654,  743, 2158, 2561, 1244, 2564, 1404,
  727, 3418, 2197, 1418, 2200, 3431, 1243, 2868, 1418, 1211,
  739,  740,  477,  557,  662, 1655,  654,  744, 1045, 2560,
 1451, 2755, 1452, 1211,  652,  742,  745,  671, 1074, 1362,
 1363,  744, 1085,  431, 2765,  572,  970, 1794, 1045,  594,
 1795, 1206, 1004,  343,  904, 1453, 1794, 3419,  274, 1795,
 1311, 1045,  102,  103,  553, 1389, 2933, 1594,  727,  678,
 1454,  727, 1418, 1658, 3308,  662, 2255, 2220, 1175,  740,
 1088,  758,  727,  894,  431,  654,  587,  431, 1418,  582,
 1479, 3270, 1206,  740,  742, 1211, 1211, 1074, 2787, 1390,
  927, 2807,  431,  541, 2809,  471,  484, 2221, 1166, 2222,
 1206,  347,  542,  904,  667, 1211, 1653,  594, 1404, 1311,
 1404, 1404, 2808,  347,  516, 2810, 1175,  894, 1654, 1175,
 2223, 1253, 3060,  593, 2379, 1254, 1175, 1175, 1004, 1244,
  758, 1175,  894, 2303, 2563, 2380, 1175, 3024,  347, 1175,
 1655,  655, 3000,  594,  347, 2224,  347, 3025, 1653,  927,
 1429, 1656, 2501, 1175, 1397,  594,  590, 1396,  751, 1657,
 1654, 1415,  591,  667, 1175, 1418, 1488, 1288, 1288, 2225,
 1753, 3001, 1175, 2502,  741,  763, 1175,  765,  508,  509,
  662,  592, 1655,  347, 1391, 1175,  971,  516, 1658,  593,
 2382,  345, 2136, 1656,  428,  595,  524,  767,  768, 2137,
  655, 1175,  772,  773,  751,  610, 1617, 1207,  596, 1429,
 1576, 1753,  594,  516, 1753,  739, 1767,  343,  471,  484,
  505,  463,  474,  474, 2454,  594,  396, 1175,  597, 1753,
 1658, 1753, 1741,  741, 1004,  428,  636, 1211,  428, 1175,
 2378, 1175,  498, 1004,  347, 1052,  512,  512, 1207, 3074,
  607,  578,  578,  428,  349,  345, 1224, 2442, 2443, 1034,
 1035,  739,  348,  595, 1312, 2381, 1207,  396, 1004,  647,
  396, 1466, 1490, 2503, 1418,  396, 1418, 1418, 2403,  323,
 1225, 1496,  637, 2383, 2384,  396,   96, 1175,  118,  742,
 1253,  653, 1050, 1131, 1254, 1659, 1289, 1289,  971,  595,
  365,  665, 2732, 1175, 1175, 1206, 1539,  235, 1175,  431,
  431,  595, 2733,  250, 2799, 2800,  431,  313, 1339,  739,
  678,  348,  970, 1312, 2504,  540, 2985, 1416,  593,  610,
 1466, 1340, 2734,  739,  396,  742,  883, 1659,  323,  347,
 2497,  512,  687,  512,  512,   96, 3338,  118,  688, 2575,
   49, 1050, 2533, 1206, 2535, 2537, 3339, 3340,  971,  365,
   52,  971, 1074,  690, 1206,  516,  235, 3055,  595,  516,
  357, 2493,  250, 1206,  857,  858,  313, 2478, 3355,  665,
   55,  595, 1576,  692, 1741, 1576, 2536,  512, 2491,  695,
  512,  512,  512,  742,  968,  713,  593,  431, 2568,  492,
 2538, 2571, 2572, 2573,  968, 1140, 2386,  742, 3385, 1570,
  244,  244, 1259, 1206,  512, 1900, 2666, 1074,  431,  718,
 1074, 2498, 1206, 2505,  431, 2590, 1891, 1900,  561,  357,
 1900, 2494,  593, 2495, 1753, 1074, 2405, 3299,  741,  719,
 2499, 2667, 2415, 1451,  593, 1452, 3028, 2735, 2736, 2737,
  540,    4,    7,  347,  347, 1175, 2668,  431, 2500,  725,
 1901, 1074,  471,  484, 1206,  728,  393, 1891, 1453, 1753,
 1891, 1207, 1901,  548,  736, 1901, 2168, 2055, 1206, 1938,
 2168, 2721, 2722, 1454,  741, 1891,  729, 2034, 2688, 1288,
 1259, 1938,  547, 1338, 1938,   58,   61,   64, 1480, 2689,
 1484,  593, 2056,  948, 1221, 1489, 1491, 3360, 2122,  324,
 1497, 1259, 2057,  953,  593, 1074,  431, 2058,  972, 1207,
  348, 2124, 1288,  592, 2123, 1569, 1257, 1255,  960, 1256,
 1207, 1258,  348, 1071, 1338, 1074, 1540, 1338, 1783, 1207,
 1206, 1206,  741, 1905, 2229,  962, 2059, 2228, 1074,  965,
 1783,  612, 1338, 1783, 2691, 1905,  741,  348, 1905,  972,
 1206,  977,  972,  348,  745,  348, 1138, 1052,  324, 1336,
  492, 1337,  349,  578,  323, 2234,  978,  972, 2233, 1207,
  980,  343,  471,  484,  505,  463,  474,  474, 1207,  984,
 2758, 2353, 1071, 3209, 2011, 1763, 2678, 3389, 1753, 3390,
 1788,  968,  348,  982, 2687, 2359,  578, 2060, 1259,  763,
  350,  765, 1788,  615, 2373, 1788, 2061, 2372, 1289,  323,
  652,  983, 2402,  985, 2788, 1259, 1052,  317,  357, 2690,
 1207, 2482, 2557,  516, 2481, 2556,  772,  773, 1653, 2605,
  357, 2062, 1259,  971, 1207,  987,  591, 2692, 2693,  994,
 1654, 1289, 2724,  512,  996, 1052, 1169,  317, 2616,  317,
 1009, 2615, 2493,  348, 2789,  357,  516, 1576, 1169, 2388,
 2389,  357, 1655,  357,  350,  516, 1169, 2452, 2453,  516,
  317, 1052, 1017, 1656, 1052, 2756, 2646, 1021, 1023, 1259,
 1169, 2681, 1206, 2818, 2760, 1028, 2556, 2775, 2766, 1052,
 2779, 1169, 1891, 1891, 1030,  317, 1207, 1207, 3260, 1891,
  357,  955, 1036,  763, 1039,  765,  508,  509,  357, 2823,
 1658, 1891, 2584, 1040, 3004, 1052, 1207, 3003,  592,  317,
 3030, 1949, 1053, 3029,  516,  767,  768, 1054, 1169, 3082,
  772,  773, 1259, 1949,  615, 2814, 1949, 3117, 1058, 1113,
 3116,  968,  955, 2491, 1117,  955,  516, 1338,  348, 1177,
 3326, 1253,  968, 1096, 1132, 1254,  343,  516, 3442, 1145,
  955,  357, 3445, 1704, 3159,  323, 1148, 1396, 3172, 1052,
 3350, 1259,  357, 3349, 3362, 3364, 2841, 3361, 3363, 2839,
 1891, 1150, 1152, 1149,  972,  972,  592, 1153,  972, 1052,
 1177,  972, 1154, 1177,  324, 1338, 2682, 3425, 1338, 1155,
 3349, 1891, 1052,  972, 1156, 1338, 1338, 1891, 1177, 1157,
 1338, 2890,  516, 3444,  516, 1338, 1259, 1659, 1338, 3330,
 3446, 1158,  592, 1259,  492, 2898, 1518,  463, 1159, 1518,
  463, 1741, 1338, 1741,  592, 1169, 1160,  968,  492,  324,
 1891,  591, 3327, 1338, 3329, 1162,  968,  326, 1207, 1728,
  349, 1338, 1728, 1163, 1164, 1338,  357, 1071,  396, 1165,
 1166,  396,  348,  348, 1338, 1729, 1180, 1240, 1729,  343,
 1240, 1167,  972,  585, 2904,  582,  585,  326,  582,  326,
 1338, 2927, 2072, 1168, 1170, 2075, 1071, 1172,  516, 1177,
 1178,  592, 2919,  972, 1179, 1797, 1181, 1230, 1182,  972,
  326, 1338,  857,  858,  592, 1797, 1338, 2931,  516,  591,
 2999, 1183, 1071, 1705, 1184, 1071, 1232, 1338, 1338,  970,
 1338, 1185, 1186, 1187,  512,  326, 1188, 1741, 1189,  970,
 1071, 1190,  972, 1191, 1192, 2986, 1249, 1248, 1250, 1193,
 1194,  512, 2966, 1195, 1196,  591, 1197, 1198, 1199,  326,
 1200, 3042, 3002, 1201, 1202, 1203, 1071,  591, 1204, 1205,
 1206, 1207, 1208, 3044, 1072, 1209, 1338, 1891, 1210, 1211,
  357,  357, 1065, 1212, 1213, 1214, 1215,  955,  955, 1260,
  967, 1259, 1338, 1338,  955, 1261, 1262, 1338, 2871, 2872,
 2873, 2874, 2875, 2876, 1263,  324,  955,   64, 1264, 1266,
 1290, 1291, 1338,  343, 1292,  516, 1308, 1311, 1307, 1313,
 1071, 1317, 3048, 1177,  591, 1318, 1322, 1347, 3045,  997,
  967, 1351,   64, 1072, 3010, 1354, 3009,  591, 1368, 1374,
 1071, 1065,   64, 3043, 1380, 1383, 3046,   64, 1393,   15,
 1399, 1409,  968, 1071, 1411,  471,  492,  484,  489,  492,
  568,  498,  505, 1412, 1413, 1414, 1416, 1417,  492,  972,
  997, 1177,  492,  997, 1177,  955,   64, 1424,  516, 1421,
 1430, 1177, 1177, 1431, 1444, 1464, 1177, 3131,  997, 1463,
  512, 1177,  968, 1466, 1177, 1468,  955, 1469,  644,  357,
 2265, 3113,  955, 1542,  516, 1520,  512, 3049, 1177, 1548,
 1549, 1567, 1797,  512, 1582, 1598, 1932, 3123, 1571, 1177,
 1612, 1597, 1622, 2158, 3130, 3081, 1008, 1177, 1624, 1625,
 1641, 1177, 1626, 1661, 1663,  955,  970,   64, 1664, 1396,
 1177, 1688, 1665, 3111, 1338, 1666,   64, 1689, 1233,  997,
  593, 1694, 1696, 1698, 1699, 1715, 1177, 1234,  357, 1718,
 3120, 1726, 1720, 1723, 2266, 1798, 1008, 1008, 1799, 2267,
 1008,   64, 1801,  350, 1805, 1811, 1812, 1823,  343, 1819,
 1144, 1813, 1177,  351, 1821, 1008,  352, 1008, 1826, 1849,
  997, 1851, 1855,  997, 1177, 1860, 1177, 1913, 1884, 1918,
 1927, 2839, 2890, 1933, 1236, 1013, 1941, 1161,  997, 1943,
 1944, 1237, 1951, 1952, 1955,  492, 1956, 1008, 1957, 1238,
 1960, 2015, 1967, 2268, 3062, 3063, 3064, 3065, 3066, 3067,
 1174, 1979, 2011, 2018, 3187, 2012, 1239, 2030, 1240, 2269,
 1238, 2027, 1177, 1253, 2031,  343, 2033,  343, 1241, 2035,
 1242, 1243, 1244, 2039,  353,  516, 2041, 1245, 1177, 1177,
 2050, 2052, 1797, 1177, 2049,  463, 3189, 2053, 2919, 2051,
 3171, 2054, 2068, 1797, 3178, 2069, 3230, 2076, 1177, 2070,
 1085, 3111, 2081, 2077,  354, 3208,  970,  652,  355, 2082,
 2083, 2086, 2104, 3211, 2084, 2115, 2109,  970, 2132,  356,
 2127, 2128, 2133, 2131, 2152,  997,  997, 2150, 1072,  997,
 2164, 2177,  997,  477,  516, 2182, 1065, 2165, 2187,  516,
 2183, 2186,  997, 2195,  997, 2211, 2270, 2212, 2215, 2230,
 2216,  512, 2232, 3231, 2238, 2242, 2243, 1072, 1741, 1085,
 2244, 2257, 3277, 2248,  512, 1065, 2258, 2262, 2259, 3316,
 2275, 2286, 2260, 2296, 2295, 3286, 2319, 2356, 1797, 2292,
 2328, 2358, 2363, 1072, 2368, 2361, 1072, 1797, 2320, 2376,
 2377, 1065, 2390,  357, 1065, 2391, 2398, 2413, 2414, 2417,
 2431, 1072,  970, 2432, 1008, 3120, 2426, 2438, 2425, 1065,
 1008,  970, 2427,  997,  357, 2428, 2445, 2448, 2136, 1008,
 2450, 1008, 1008, 1008, 2451, 2458, 1008, 1072, 1425, 1008,
 1177, 2460, 2461, 2466,  997, 1065, 2468, 2178, 2479,  358,
  997, 1008, 2539, 2511, 2519,  997,  997, 1008, 2512, 2513,
 1008, 2534,  997, 2543, 2551, 2271, 1008, 2272, 2273,  357,
 2565, 2578, 1932,  343,  997, 2567, 2514,  357, 3343, 1324,
  540, 1009, 1325,  997, 2515, 1990, 2516, 1331, 1989, 2569,
 1335, 1072, 2570,  477, 1338, 2579, 3309, 2580, 1331, 1065,
 1008, 2584, 2592, 2593, 1249, 1248, 1250,  357, 2618,  357,
 2602, 1072,  359, 2619, 1247,  477, 2643, 3383, 2645, 1065,
 1008, 1009, 1009, 2707, 1072, 1009, 2651, 2652, 2653, 2710,
  357, 2654, 1065, 2699, 2711, 2655, 2656, 2701, 3369, 2657,
 1009, 1008, 1009,  997, 2705, 2713, 2716, 1008, 2725, 2726,
 2727, 1008, 2729, 2730,  477,  357, 1836, 1008, 2740, 2163,
 2761, 2757, 2762, 2759,  997, 2767, 2768, 2772, 2769, 2771,
  997, 2770, 1009, 2773, 2774, 1008, 2777, 2781, 2776,  357,
 1008, 2790, 2791, 2793, 2812, 2815, 2824, 2828, 1003,  477,
 2832, 2813, 2843,  971, 1703, 2865, 2859, 1836, 2903, 2878,
 1836, 2892, 2905,  997, 1008, 2906, 2907,  343, 1008, 2908,
  997, 2909, 2910, 2915, 2920, 1836, 2911,  970, 2928, 2929,
 2932, 2937, 2941, 1470, 2946,  357, 2962, 2501, 2959, 1003,
 3423, 2970, 1003,  971, 1481, 2949, 2971, 2960, 2990, 1487,
 2992, 2994, 1831, 1492, 1085, 3015, 3012, 1003,  883, 1003,
 3051, 3039, 3053, 1511, 1144, 3057, 3070,  970, 1517, 1518,
 1144, 3071, 1521, 1522, 1523, 1524, 1525, 1529, 1535, 2089,
 1538, 3075, 1541, 1085, 1543, 1544, 1545, 1546, 1547, 1003,
 3080, 1550, 1551, 1552, 1553, 1554, 1555, 1556, 1557, 1558,
 1559, 1560, 1561, 1562, 1563, 1564, 1565, 3098, 2090, 1085,
 3099, 3100, 1085, 3121, 3102, 3112, 1233, 1008, 3135, 3139,
 3127, 3115, 3141, 3177, 1578, 1234, 3179, 1085, 3152, 3183,
 3191, 1590, 3193, 3192, 2091, 3200, 1596, 2092, 1235, 3202,
  997, 3212, 1603, 1604, 3224, 3225, 3229, 3261, 3242, 1009,
 1611, 3267, 2093, 1085, 3268, 1009, 1024, 3271, 3283, 3275,
 3278, 3287, 3284, 3292, 1009, 3285, 1009, 1009, 1009, 3296,
 3298, 1009, 1236, 3336, 1009, 3328, 3310, 3319, 2094, 1237,
 3335, 3344, 2487, 3311, 3351, 3373, 1009, 1238, 3374, 3387,
 3375, 3376, 1009, 1008, 3388, 1009, 3386, 1024, 1681, 3391,
 1024, 1009, 3400, 3396, 1239, 3398, 1240, 1085, 3408, 3393,
 3394, 3422, 3428, 3426, 1304, 1024, 1241, 1024, 1242, 1243,
 1244, 3435, 3392, 3429, 3439, 1245, 3443, 1085, 3441, 1131,
    1, 1248, 2095, 1177, 1321, 1009,  432,  368, 1133, 1948,
 1085,  500, 1836, 1836, 1118, 1132,  501, 1024,  451, 1836,
 1321, 1744, 2096, 1220,  483, 1009, 1003, 1516, 1726, 1199,
 1156, 1836, 1003,  911, 1225, 2097, 1246,  482, 1517,  454,
  492, 1003,   65, 1003, 1003, 1003, 1009,  432, 1003,   86,
  432, 1003, 1009, 1204,  538,  517, 1009, 1611,  493, 1243,
  463, 1321, 1009, 1003, 1321,  432,  558,  320,  602, 1003,
  567,  549, 1003,  525,  156, 1782, 1690,  393, 1003, 1321,
 1009,  559,  396,  603, 1719, 1009, 1463, 1612, 1498, 1499,
 1500, 1501, 1502, 1503, 1504, 1505, 1691,  647, 1725, 1603,
 1836,  648,  567, 1015,  617,  472,  354,  355,    0, 1009,
   21,    9, 1003, 1009,   25,   19, 1753, 1753, 1753, 1753,
 1753, 1836, 1753,   28,   31,   40,   21, 1836,   67,   70,
   72,   76, 1003,   79,   85, 1753, 1753, 1753,   88,   97,
 1937, 1938,   97, 1940, 1015, 1942,  100, 1015,  112, 1946,
 1947, 1948, 1949, 1003,  106,  109,  115, 1953, 1954, 1003,
 1836,   21, 1015, 1003, 1015, 2085, 1753, 1667, 1668, 1003,
 2661,  103, 1669, 1670, 1961, 2665, 1144, 1671, 2669, 3054,
 3153, 3154, 2396, 1672, 1024, 2696, 3052, 1003, 3129, 2582,
 1024, 1977, 1003, 2998, 1015, 1930,  599, 1506, 2980, 1024,
 1983, 1984, 1024, 1024, 3114,  997, 1024, 2956, 3380, 1024,
  499, 1112, 1009, 2506, 3213, 2530, 1003, 2969, 3294,  603,
 1003, 1024, 1247, 1507, 2577, 1312, 2981, 1024, 3274, 2805,
 1024, 3381, 2806, 3437, 3399, 3397, 1024, 3421, 2947, 3234,
 1470, 3259, 2948, 1922, 2285, 2289,  997, 1376,  979,  997,
 3097, 2476, 2477, 3414, 2026, 3413, 2028, 3334, 3196, 2029,
 2174, 2473, 3265,   32,  997, 1619,  997, 2864, 2036, 1863,
 1024, 2261, 1048, 2135, 2199, 1717, 2486, 2548, 1432, 1850,
 1836,  585,  432,  432, 1457, 2416, 2698, 1836, 1009,  432,
 1024, 3167, 2703, 1320,  993,  595, 1321, 1321,  973, 2111,
 1008, 1652, 2899, 1321, 2404, 3136, 2073,  986, 3173, 1475,
 3407, 1024, 2114, 1015, 2430,   49,   51, 1024, 1144, 1003,
 3155, 1024,   52, 3156,   55,   58, 1306, 1024, 3358, 1595,
 2566, 3276, 2025, 1130, 1055, 1151, 1858, 1753, 3205, 2118,
 1216, 1613, 2973, 1217, 3014, 1024, 1753, 2542,  986, 1702,
 1024,  986, 1753, 2748, 2336,  558, 1753, 1331, 3047, 1753,
 1335, 1015, 2117,  583, 2406, 1645,  986, 1015,  986, 2695,
  432, 2822, 1331, 3406, 1024, 1465, 1015, 2277, 1024, 1015,
 1015, 3204, 3314, 1015, 1321, 3382, 1015, 2930, 1321, 2749,
 1388,  432, 1366, 1753,  132, 1003, 3289,  432, 1015, 2901,
 1753,    0, 3325,    0, 1015, 1321,    0, 1015, 1753,    0,
 2166, 1321,    0, 1015, 1000,    0,    0,    0, 2175,    0,
    0,    0,    0,    0,    0, 1753,    0, 1753,    0,    0,
  432,    0,    0,    0,    0,    0,    0, 1753,    0, 1753,
 1753, 1753,    0,    0, 1321,    0, 1753, 1015,    0,    0,
    0,    0,  994,    0,    0, 1000,    0,    0, 1000,    0,
    0,    0,    0,  997,    0,    0,    0, 1015,    0,  997,
    0,    0,    0, 1000,    0, 1000,    0, 1024,  997,    0,
    0,    0,  997,    0,    0,  997,    0, 1753, 1015,  432,
    0,    0,    0,  994, 1015,    0,  994,  992, 1015,    0,
  997,    0,    0, 1321, 1015, 1000,  997,    0,    0,  997,
    0,  994,    0,  994,    0,  997,    0,    0,    0,    0,
    0,    0, 1015,    0,    0,    0,    0, 1015,    0,    0,
    0,    0,    0,    0,    0,  430,    0,    0,  992,    0,
    0,  992,    0,    0,    0,    0,    0,    0,    0,  997,
    0, 1015,    0, 1024,    0, 1015,  992,    0,  992,    0,
    0,    0,    0,    0,    0,  986,    0,    0,    0,  997,
    0,  986,    0,    0,    0,    0,  430,    0,    0,  430,
    0,    0,  986,  986,  986,    0,    0,  986, 1339,    0,
  986,    0,    0,    0,  430,    0,    0,    0,    0,    0,
  997,    0,  986,    0,    0, 2306,  997,    0,  986,    0,
    0, 2312,    0,    0,    0,    0, 2315,  986,    0, 2316,
 2317, 1961,    0,    0,  997,    0, 2322,    0, 2324,    0,
    0,    0,    0,    0,    0, 2325,    0, 2326,    0,    0,
 2329, 2330, 2331, 2332,    0,    0, 2337,    0, 2338,    0,
 2339,    0, 2340,  997, 1015,    0,    0,  997, 2341, 2342,
 2343, 2344, 2345, 2346, 2347, 2348, 2349, 2350, 2351, 2352,
    0,  986, 1000, 1753,    0,    0,    0,    0, 1000, 2360,
 1771,  997,    0,    0, 2362,    0,    0, 1000, 2365,    0,
 1000, 1000,  986,    0, 1000,  266,    0, 1000,  986,    0,
    0,    0,    0,    0,    0,    0,  986,    0,  986, 1000,
  994,    0,    0,    0,    0, 1000,  994,    0, 1000,    0,
    0,  986,  997,    0, 1000,  997,  986,  994,  994,  994,
 1015,  986,  994,    0,    0,  994,  266,    0,    0,  266,
  997,    0,  997,    0,    0,    0,    0,  994,    0,    0,
    0,    0,    0,  994,  266,  992,  997,    0, 1000,  986,
    0,  992,  994,    0,    0,    0,    0,    0,    0,    0,
 1144,    0,  992,  992,  992,    0,    0,  992, 1000,    0,
  992,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  992,    0,    0,    0,    0,    0,  992, 1000,
    0,    0,    0,    0,    0, 1000,    0,  992,    0, 1000,
    0,  430,  430,    0,    0, 1000,  994,    0,  430,    0,
    0,    0,    0, 2463,    0, 1144, 1975,    0,    0,    0,
    0,    0,  997, 1000,  745,    0,    0,  994, 1000,    0,
    0,    0,    0,  994,    0, 1249, 1248, 1250,    0,    0,
    0,  994,    0,  994,    0,    0,    0,    0,  986,    0,
  748,  992, 1000,    0,    0,    0, 1000,    0,    0,    0,
    0,  994,    0,    0,    0,    0,  994,    0,    0,    0,
    0,    0,  992,    0,    0,    0,   78,  750,  992,    0,
    0,    0,    0, 1992,    0,    0, 1991,    0,  992,  430,
    0,   78,    0,    0,  994,    0,    0,    0,    0,    0,
    0,   78, 1249, 1248, 1250,    0,  992,   78,    0,    0,
  430,  992,   78,    0,    0,    0,  430,   78,    0,    0,
   78,    0,    0,    0,  986,    0,    0,    0,    0,  997,
    0,    0,    0,    0,    0,  997,    0,    0,    0,  992,
    0,    0,    0,  760,    0,    0,    0,  997,  997,  430,
  408,  997,    0,    0,  997, 1000,    0,    0,    0,  762,
    0,  266,  266,    0,    0,    0,  997,    0,  266,   62,
    0, 1961,  997,  763,    0,  765,  508,  509, 2609, 2610,
    0,  997,    0,    0,    0, 2617,    0,    0,   78,    0,
    0,  408,    0,  994,  408,  767,  768,  769,  770,  771,
  772,  773,  774,  775, 2623,    0,    0, 2622,  430,  408,
  777,  778, 2638,    0,    0, 2640,    0, 2642,   78,  782,
    0,    0,    0, 1249, 1248, 1250,    0,    0,    0,    0,
    0, 1000,    0,   78,    0,  997,    0, 1233,  992,    0,
    0,    0,    0,  386,    0,    0, 1234, 2625,    0,  266,
 2624,   78,    0,    0,    0,    0,  997,    0,    0, 1235,
    0,    0,  997,    0,    0,    0, 1249, 1248, 1250,  994,
  266,    0,  997,    0,    0,    0,  266,    0,    0,    0,
    0,    0,    0,    0,  386,    0,    0,  386,    0, 1974,
  997,    0,    0, 1236,    0,  997,    0,    0,    0, 1144,
 1237,    0,  386,    0, 1233,    0,    0,   78, 1238,  266,
   78,    0, 3021, 1234,  992, 3020,    0,    0,    0,    0,
    0,    0,    0,  997,    0, 1239, 1235, 1240,    0,    0,
 1144, 1249, 1248, 1250,    0,    0,    0, 1241,    0, 1242,
 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1236,    0,    0,    0,    0,    0,    0, 1237,    0,    0,
  856,    0,  857,  858,  859, 1238,    0,    0,    0, 3023,
    0,    0,    0,    0,    0,    0,    0, 1246,    0,    0,
    0,    0, 1239,    0, 1240,    0,    0,    0, 1249, 1248,
 1250,  877,  878,    0, 1241,    0, 1242, 1243, 1244, 1692,
    0,    0,    0, 1245,    0,    0,   78,    0,    0,    0,
    0,    0,  997,    0,    0, 1233,  409,  408, 1249, 1248,
 1250,    0,    0,  409, 1234,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1235,    0, 2842,
    0, 2844,    0, 2845, 1246,    0,    0, 2850,    0,    0,
    0,    0, 2853, 2854, 2855, 2856, 2857,    0, 1233,    0,
    0, 2858,    0, 2861,    0, 2862,    0, 1234,    0,    0,
    0, 1236,    0,    0,    0, 1693,    0,    0, 1237,    0,
 1235,    0, 2863,    0,    0,    0, 1238,    0,  997,    0,
    0,    0,    0,    0, 1249, 1248, 1250,    0,    0,    0,
    0,    0,    0, 1239,  409, 1240,    0,    0,    0,  386,
  386,    0,    0,    0, 1236, 1241,  386, 1242, 1243, 1244,
 1945, 1237, 1144, 1233, 1245,  409,    0,    0,    0, 1238,
    0,  409, 1234,    0,    0,    0,    0,    0, 2914, 1249,
 1248, 1250,    0,    0,    0, 1235, 1239,    0, 1240,    0,
    0, 2923, 1144,    0,    0,    0,    0,    0, 1241,    0,
 1242, 1243, 1244, 1247,  409, 1246,    0, 1245,    0, 2936,
    0,    0,    0,    0,    0,    0,    0, 2939, 2940, 1236,
    0, 1969,    0,    0,    0,    0, 1237,    0,    0,    0,
 1233,    0,    0,  410, 1238,    0,    0,  386,    0, 1234,
 1249, 1248, 1250,    0,    0,    0,    0,    0, 1246, 2963,
    0, 1239, 1235, 1240,    0,    0,    0,    0,  386,    0,
 1233,    0,    0, 1241,  386, 1242, 1243, 1244,    0, 1234,
 1247,    0, 1245,    0,  410,    0,    0,  410,    0,    0,
 3022,    0, 1235,    0,    0,    0, 1236,    0,    0,    0,
 1970,    0,  410, 1237,    0,    0,    0,  386,    0,    0,
    0, 1238,    0, 1249, 1248, 1250,    0,    0,    0, 1249,
 1248, 1250,    0, 1246,    0,    0, 1236,    0, 1239,    0,
 1240,    0,    0, 1237,    0,    0,    0,    0, 1961, 2845,
 1241, 1238, 1242, 1243, 1244,    0, 1233,    0,    0, 1245,
    0,    0,    0,    0,    0, 1234,    0,    0, 1239,    0,
 1240,    0,    0,    0,    0,    0,    0,    0, 1235,    0,
 1241, 3041, 1242, 1243, 1244,    0,    0,    0,    0, 1245,
    0,    0,    0, 3050,    0,    0,    0,    0,    0,    0,
 1246, 1233,    0,    0,    0,    0,    0,    0,    0,    0,
 1234, 1247, 1236,    0, 3068, 3069,    0,    0,    0, 1237,
    0,    0,    0, 1235,    0,    0,    0, 1238,    0,    0,
 1246,    0,    0,    0,    0, 3083, 1765,    0,    0,    0,
    0,    0,    0,    0, 1239,    0, 1240,    0,    0,    0,
    0,    0,    0,    0, 1247,    0, 1241, 1236, 1242, 1243,
 1244,    0, 1233,    0, 1237, 1245,    0,    0,    0,    0,
    0, 1234, 1238,    0,    0,    0, 1765, 1765,    0, 1765,
 1765, 1765, 1774,    0, 1235,    0,    0,    0,    0, 1239,
    0, 1240,    0, 3128, 1765, 1765,    0,    0,    0, 1765,
    0, 1241,    0, 1242, 1243, 1244, 1246,    0,    0,    0,
 1245,    0,    0, 1961,    0,    0,    0,    0, 1236, 1247,
    0, 3142,    0, 3143,    0, 1237,    0,    0,    0,  412,
  410, 1233,    0, 1238, 1234,    0,  412,    0,    0,    0,
 1234,    0,    0,    0,    0,    0,    0, 1235,    0,    0,
 1239, 1246, 1240, 1235,    0,    0,    0,    0,    0,    0,
    0,    0, 1241,    0, 1242, 1243, 1244,    0,    0,    0,
    0, 1245,    0,    0,    0,    0, 2923,    0,    0,    0,
    0, 1236,    0,    0,    0,    0, 1247, 1236, 1237,    0,
    0,    0,    0,    0, 1237,    0, 1238,    0,    0,    0,
    0,    0, 1238,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1246, 1239,    0, 1240, 1247,  412,    0, 1239,
    0, 1240,    0,    0,    0, 1241,    0, 1242, 1243, 1244,
    0, 1241,    0, 1242, 1243, 1244,    0,    0,  412,    0,
 1245,    0, 1961,    0,  412,    0,    0, 3235, 3236, 3237,
 3238, 3239, 3240, 3241,    0,    0,    0,    0,    0,    0,
    0,    0, 3244, 3245, 3246, 3247, 3248, 3249, 3250,    0,
    0,    0,    0,    0,    0, 1246,    0,  412,    0,    0,
    0, 1246,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1247, 1765,    0, 1765,    0,    0,    0,    0,
 1765, 1765,    0, 1765, 1765, 1765,    0,    0,    0,    0,
 1765,    0,    0,    0,    0, 1765, 1765,    0, 1765,    0,
 1765, 1765,    0, 1765,    0,    0, 1765, 1765, 1765,    0,
    0,    0,    0, 1765,    0, 1765, 1765, 1247,    0,    0,
 1765,    0, 1765,    0, 1765, 1765,    0,    0, 1765,    0,
    0, 1765,    0, 1765,    0, 1765, 1765,    0, 1765, 1765,
 1765,    0, 1765, 1765, 1765,    0,    0,    0, 1765,    0,
 1765, 1765, 1765,    0,    0,    0, 1765,    0,    0,    0,
 1765,    0, 1765, 1765, 1765, 1765,    0, 1765,    0, 1765,
 1765,    0, 1765, 1765, 1765,    0, 1765,    0, 1247,    0,
 1765,    0,    0,    0, 1765, 3337, 1765, 3341, 1765,    0,
 1765, 1765, 1765, 1765, 1765,    0, 1765, 1765, 1765,    0,
    0,    0, 1765, 1144, 1765, 1765,    0,    0, 1765, 1765,
 1765,    0,    0, 1765, 3367, 1765, 3370, 3372, 1765, 1765,
 1765, 1765,    0, 1765,    0, 1765, 1765, 1765, 1765, 1765,
 1765, 1765, 1765, 1144,    0,    0, 1765,    0, 1765, 1765,
    0, 1247, 1765,    0,    0,    0,    0, 1247,    0, 1765,
    0, 1765,    0, 1765, 1765, 1765, 1765, 1765, 1765, 1765,
 1765, 1765,    0, 1765, 1765, 1765, 1765,    0, 1765, 1765,
    0, 3367, 3415, 3416,    0,    0, 1765, 1765, 1765, 1765,
 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765,    0,
 1765, 1765, 1765, 1765, 1765,    0, 1765, 1765, 1765, 1765,
 1765, 1765, 1765, 1765,    0,    0, 1765, 1765, 1765, 1765,
 1765, 1765, 1765,    0,    0, 1765, 1765, 1765, 1765, 1765,
 1765, 1765, 1765, 1765,    0, 1765,    0, 1765, 1765, 1765,
    0, 1765, 1765,    0, 1765, 1765, 1765, 1765, 1765, 1765,
 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765,
 1765, 1765, 1765, 1765, 1765, 1765,    0, 1765, 1765, 1765,
 1765, 1765, 1765,    0, 1765, 1765, 1765, 1765,    0, 1765,
    0, 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765,
 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765,    0,
 1765, 1765, 1765, 1765,    0, 1765, 1765, 1765, 1765, 1765,
    0, 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765,
 1765, 1765, 1765, 1765,    0, 1765, 1765, 1765, 1765, 1765,
 1765,    0, 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765,
    0, 1765, 1765, 1765, 1765,    0, 1765, 1765, 1765, 1765,
 1765, 1765,    0,    0, 1765, 1765, 1765, 1765, 1765, 1765,
 1765, 1765, 1765, 1765, 1511, 1765, 1765, 1765, 1765, 1765,
 1765, 1765, 1765, 1765, 1765, 1765,    0, 1765, 1765, 1765,
 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765, 1765,
 1765, 1765, 1765, 1765, 1765, 1765,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1511, 1511, 1511, 1511, 1511,
    0, 1511,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1511, 1511, 1511, 1511, 2585,    0,    0,
 1061,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   79,    0, 2586,    0,    0,    0, 1062,    0, 1063,    0,
    0,    0,    0,    0,    0,   79,    0, 1511,    0,    0,
    0,    0,    0,    0,    0,   79,    0,    0,    0,    0,
    0,   79,  698,    0,    0,    0,   79,    0,    0,    0,
    0,   79,  699,    0,   79,  700, 1064,    0,    0,    0,
    0,    0, 1065, 1066, 1067,    0, 1068, 1069, 1070,    0,
 1071,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1072,    0, 1073,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   63, 2995,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1074,    0,    0,    0,    0,
    0,    0,   79,  701,    0,    0,    0,    0,    0,    0,
    0, 1075,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   79, 1076,    0,    0,    0,  702,    0,    0,
    0,    0,  703,    0,    0,    0,    0,   79,  704,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   79,    0, 1077, 1078,    0,
    0, 1511, 1511, 1511,    0,    0, 1511,    0, 1511, 1511,
    0,    0, 1511, 1511, 1079, 1511,    0,    0,    0,    0,
 1511, 1511,    0, 1511, 1511, 1511, 1511, 1511, 1511,    0,
    0, 1511,    0,    0, 1511,    0,    0,    0,    0,    0,
    0, 1511,    0, 1511, 1511, 1511, 1511, 1511, 1511, 1511,
 1511,   79,  705, 1511,   79,    0, 1511,    0, 1511, 1511,
    0,    0, 1511, 1511, 1511,    0, 1511, 1511, 1511, 1511,
 1511, 1511, 1511,    0,    0,    0, 1511, 1511, 1511, 1511,
 1511,    0,    0,    0, 1511,    0,    0,    0,    0,    0,
    0,    0, 1511, 1511, 1511, 1511, 1511, 1511,  706,    0,
    0, 1511, 1511,    0, 1511,    0, 1511,    0, 1511, 1511,
 1511,    0, 1511,    0, 1511, 1511, 1511,    0, 1511, 1511,
 1511, 1511, 1511,    0, 1511, 1511,    0,    0,    0, 1511,
 1511,  428, 1511, 1511,    0, 1511, 1511, 1511, 1511,    0,
    0, 1511,  707, 1511,    0, 1511, 1511, 1511, 1511, 1511,
    0, 1511,    0, 1511,    0, 1511, 1511, 1511, 1511, 1511,
   79,  708, 1511, 1511, 1511,    0, 1511, 1511, 1511,    0,
    0,    0,  428, 1080,    0,  428,    0, 1511, 1511, 1511,
    0, 1511, 1511, 1511,    0, 1511,    0,    0,    0, 1511,
  428, 1511, 1511, 1511, 1511,    0, 1511,    0, 1511, 1511,
 1511, 1511, 1511, 1511, 1511, 1511,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1511, 1511, 1971, 1511, 1511,
 1511,    0,    0,    0,    0, 1511, 1511, 1511, 1511, 1511,
    0, 1511,    0,    0, 1511, 1511, 1249, 1248, 1250, 1511,
 1511, 1972,    0, 1511, 1511, 1511, 1511, 1511, 1511, 1511,
 1511, 1511,    0, 1511,    0,    0,    0,    0,    0,    0,
 1249, 1248, 1250, 1511, 1511, 1511, 1511, 1511, 1511, 1511,
 1511, 1511, 1511, 1511, 1511, 1511, 1511, 1511, 1511, 1511,
 1511, 1511, 1511, 1511,    0, 1511, 1511, 1511, 1511, 1511,
 1511,    0, 1511, 1511, 1511, 1511,    0, 1511,    0, 1511,
 1511, 1511, 1511, 1511, 1511, 1511, 1511, 1511, 1511, 1511,
 1511, 1511, 1511, 1511, 1511, 1511, 1511,    0, 1511, 1511,
 1511, 1511,    0, 1511, 1511, 1511, 1511, 1511,    0, 1511,
 1511, 1511, 1511, 1511, 1511, 1511, 1511, 1511, 1511, 1511,
 1511, 1511, 1511, 1511, 1511, 1511, 1511, 1511, 1511,    0,
 1511, 1511, 1511, 1511, 1511, 1511, 1511, 1511, 1513, 1511,
 1511, 1511, 1511,    0, 1511, 1511, 1511, 1511, 1511, 1511,
    0,    0, 1511, 1511, 1511, 1511, 1511, 1511, 1511, 1511,
 1511,    0,    0,    0,    0,    0, 1511,    0, 1511, 1511,
 1511, 1511, 1511, 1511,    0, 1511,    0,    0,    0, 1513,
 1513, 1513, 1513, 1513,    0, 1513,    0,  428,  428,    0,
    0,    0,    0,    0,  428,    0,    0, 1513, 1513, 1513,
 1513,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1233,    0,
    0,    0,    0,    0,    0,    0,    0, 1234,    0,    0,
    0, 1513,    0,    0,    0,    0,    0,    0,    0,    0,
 1235,    0, 1233,    0,    0,    0,    0,    0,    0,    0,
    0, 1234,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1235,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1236,  428,    0,    0,    0,    0,
    0, 1237,    0,    0,    0,    0,    0,    0,    0, 1238,
    0,    0,    0,    0,    0,    0,  428,    0, 1236,    0,
    0,    0,  428,    0,    0, 1237, 1239,    0, 1240,    0,
    0,    0,    0, 1238,    0,    0,    0,    0, 1241,    0,
 1242, 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,
 1239,    0, 1240,    0,    0,  428,    0,    0,    0,    0,
    0,    0, 1241,    0, 1242, 1243, 1244,    0,    0,    0,
    0, 1245,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1246,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1246,    0,    0, 1513, 1513, 1513,    0,    0,
 1513,    0, 1513, 1513,    0,    0, 1513, 1513,    0, 1513,
    0,    0,    0,    0, 1513, 1513,    0, 1513, 1513, 1513,
 1513, 1513, 1513,    0,    0, 1513,    0,    0, 1513,    0,
    0,    0,    0,    0,    0, 1513,    0, 1513, 1513, 1513,
 1513, 1513, 1513, 1513, 1513,    0,    0, 1513,    0,    0,
 1513,    0, 1513, 1513,    0,    0, 1513, 1513, 1513,    0,
 1513, 1513, 1513, 1513, 1513, 1513, 1513,    0,    0,    0,
 1513, 1513, 1513, 1513, 1513,    0,    0,    0, 1513,    0,
    0,    0,    0,    0,    0,    0, 1513, 1513, 1513, 1513,
 1513, 1513,    0,    0,    0, 1513, 1513,    0, 1513,    0,
 1513,    0, 1513, 1513, 1513,    0, 1513,    0, 1513, 1513,
 1513,    0, 1513, 1513, 1513, 1513, 1513,    0, 1513, 1513,
    0,    0,    0, 1513, 1513,    0, 1513, 1513,    0, 1513,
 1513, 1513, 1513,    0,    0, 1513,    0, 1513,    0, 1513,
 1513, 1513, 1513, 1513,    0, 1513,    0, 1513,    0, 1513,
 1513, 1513, 1513, 1513, 1247,    0, 1513, 1513, 1513,    0,
 1513, 1513, 1513,    0,    0,    0,    0,    0,    0,    0,
    0, 1513, 1513, 1513,    0, 1513, 1513, 1513, 1247, 1513,
    0,    0,    0, 1513,    0, 1513, 1513, 1513, 1513,    0,
 1513,    0, 1513, 1513, 1513, 1513, 1513, 1513, 1513, 1513,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1513,
 1513, 1973, 1513, 1513, 1513,    0,    0,    0,    0, 1513,
 1513, 1513, 1513, 1513,    0, 1513,    0,    0, 1513, 1513,
 1249, 1248, 1250, 1513, 1513, 1986,    0, 1513, 1513, 1513,
 1513, 1513, 1513, 1513, 1513, 1513,    0, 1513,    0,    0,
    0,    0,    0,    0, 1249, 1248, 1250, 1513, 1513, 1513,
 1513, 1513, 1513, 1513, 1513, 1513, 1513, 1513, 1513, 1513,
 1513, 1513, 1513, 1513, 1513, 1513, 1513, 1513,    0, 1513,
 1513, 1513, 1513, 1513, 1513,    0, 1513, 1513, 1513, 1513,
    0, 1513,    0, 1513, 1513, 1513, 1513, 1513, 1513, 1513,
 1513, 1513, 1513, 1513, 1513, 1513, 1513, 1513, 1513, 1513,
 1513,    0, 1513, 1513, 1513, 1513,    0, 1513, 1513, 1513,
 1513, 1513,    0, 1513, 1513, 1513, 1513, 1513, 1513, 1513,
 1513, 1513, 1513, 1513, 1513, 1513, 1513, 1513, 1513, 1513,
 1513, 1513, 1513,    0, 1513, 1513, 1513, 1513, 1513, 1513,
 1513, 1513, 1775, 1513, 1513, 1513, 1513,    0, 1513, 1513,
 1513, 1513, 1513, 1513,    0,    0, 1513, 1513, 1513, 1513,
 1513, 1513, 1513, 1513, 1513,    0,    0,    0,    0,    0,
 1513,    0, 1513, 1513, 1513, 1513, 1513, 1513,    0, 1513,
    0,    1, 1748, 1775, 1775, 1775, 1775, 1775, 1775, 1775,
    0,    0,    0,    0,    0,    0,    0,    0,    2,    0,
 1775, 1775, 1775, 1775, 1775,    0,    3,    0,    0,    0,
    0,    0,    0,    0,    4,    0,    5,    0,    0,    0,
    0,    6, 1233,    0,    0,    0,    0,    7,    0,    0,
    0, 1234,    0, 1775,    0, 1775,    0,    0,    0,    0,
    0,    8,    0,    0, 1235,    0, 1233,    0,    0,    0,
    9,    0,    0,    0,    0, 1234,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1235,    0,
    0,    0,    0,    0,    0,   10,    0,    0, 1236,    0,
    0,    0,    0,    0,    0, 1237,    0,    0,    0,    0,
    0,    0,    0, 1238,    0,    0,    0,    0,    0,    0,
    0,    0, 1236,    0,    0,    0,    0,    0,    0, 1237,
 1239,    0, 1240,    0,    0,    0,   11, 1238,   12,    0,
    0,   13, 1241,    0, 1242, 1243, 1244,    0,    0,    0,
    0, 1245,    0,    0, 1239,    0, 1240,    0,    0,    0,
    0,    0,    0,    0,    0,   14, 1241,    0, 1242, 1243,
 1244,    0,    0,    0,    0, 1245,    0,   15,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1246,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   16,    0,    0, 1246,    0,    0, 1775,
    0, 1775,    0,    0, 1775,    0, 1775, 1775,   17,   18,
 1775, 1775,    0, 1775,    0,    0,    0,    0,    0, 1775,
    0, 1775, 1775, 1775, 1775,    0, 1775,    0,    0, 1775,
   19,    0, 1775,    0,    0,    0,    0,    0,    0, 1775,
    0, 1775, 1775,    0, 1775, 1775, 1775, 1775, 1775,    0,
    0, 1775,    0,    0, 1775,    0, 1775, 1775,    0,    0,
 1775, 1775, 1775,    0, 1775, 1775, 1775, 1775, 1775, 1775,
 1775,    0,    0,    0, 1775, 1775, 1775, 1775, 1775,    0,
    0,    0, 1775,    0,    0,    0,    0,    0,    0,    0,
 1775, 1775, 1775, 1775, 1775, 1775,    0,    0,    0, 1775,
 1775,    0, 1775,    0, 1775,    0, 1775, 1775, 1775,    0,
 1775,    0, 1775, 1775, 1775,    0, 1775, 1775, 1775, 1775,
 1775,    0, 1775, 1775,    0,    0,    0,    0, 1775,    0,
 1775, 1775,    0,    0, 1775, 1775, 1775,    0,    0, 1775,
    0, 1775,    0,    0, 1775, 1775, 1775, 1775,    0, 1775,
    0, 1775,    0, 1775, 1775, 1775, 1775, 1775, 1247,    0,
 1775, 1775, 1775,    0, 1775, 1775,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1775, 1775, 1775,    0, 1775,
 1775, 1775, 1247, 1775,    0,    0,    0, 1775,    0, 1775,
 1775, 1775, 1775,    0, 1775,    0, 1775, 1775, 1775, 1775,
 1775, 1775, 1775, 1775,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1775, 1775, 1988, 1775, 1775, 1775,    0,
    0,    0,    0, 1775, 1775, 1775, 1775, 1775,    0, 1775,
    0,    0, 1775, 1775, 1249, 1248, 1250, 1775, 1775, 1993,
    0, 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775,
    0, 1775,    0,    0,    0,    0,    0,    0, 1249, 1248,
 1250, 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775,
 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775,
 1775, 1775,    0, 1775, 1775, 1775, 1775, 1775, 1775,    0,
 1775, 1775, 1775, 1775,    0, 1775,    0, 1775, 1775, 1775,
 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775,
 1775, 1775, 1775, 1775, 1775,    0, 1775, 1775, 1775, 1775,
    0, 1775, 1775, 1775, 1775, 1775,    0, 1775, 1775, 1775,
 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775,
 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775,
 1775, 1775, 1775, 1775, 1775, 1775, 1812, 1775, 1775, 1775,
 1775,    0, 1775, 1775, 1775, 1775, 1775, 1775,    0,    0,
 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775, 1775,    0,
    0,    0,    0,    0, 1775,    0, 1775, 1775, 1775, 1775,
 1775, 1775,    0, 1775,    0,    0, 1625, 1812, 1812, 1812,
 1812, 1812, 1812, 1812,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1812, 1812, 1812, 1812, 1812,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1233,    0,    0,    0,
    0,    0,    0,    0,    0, 1234,    0, 1812,    0, 1812,
    0,    0,    0,    0,    0,    0,    0,    0, 1235,    0,
 1233,    0,    0,    0,    0,    0,    0,    0,    0, 1234,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1235,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1236,    0,    0,    0,    0,    0,    0, 1237,
    0,    0,    0,    0,    0,    0,    0, 1238,    0,    0,
    0,    0,    0,    0,    0,    0, 1236,    0,    0,    0,
    0,    0,    0, 1237, 1239,    0, 1240,    0,    0,    0,
    0, 1238,    0,    0,    0,    0, 1241,    0, 1242, 1243,
 1244,    0,    0,    0,    0, 1245,    0,    0, 1239,    0,
 1240,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1241,    0, 1242, 1243, 1244,    0,    0,    0,    0, 1245,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1246,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1246,    0,    0, 1812,    0, 1812,    0,    0, 1812,    0,
 1812, 1812,    0,    0, 1812, 1812,    0, 1812,    0,    0,
    0,    0,    0, 1812,    0, 1812, 1812, 1812, 1812,    0,
 1812,    0,    0, 1812,    0,    0, 1812,    0,    0,    0,
    0,    0,    0, 1812,    0, 1812, 1812,    0, 1812, 1812,
 1812, 1812, 1812,    0,    0, 1812,    0,    0, 1812,    0,
 1812, 1812,    0,    0, 1812, 1812, 1812,    0, 1812, 1812,
 1812, 1812, 1812, 1812, 1812,    0,    0,    0, 1812, 1812,
 1812, 1812, 1812,    0,    0,    0, 1812,    0,    0,    0,
    0,    0,    0,    0, 1812, 1812, 1812, 1812, 1812, 1812,
    0,    0,    0, 1812, 1812,    0, 1812,    0, 1812,    0,
 1812, 1812, 1812,    0, 1812,    0, 1812, 1812, 1812,    0,
 1812, 1812, 1812, 1812, 1812,    0, 1812, 1812,    0,    0,
    0,    0, 1812,    0, 1812, 1812,    0,    0, 1812, 1812,
 1812,    0,    0, 1812,    0, 1812,    0,    0, 1812, 1812,
 1812, 1812,    0, 1812,    0, 1812,    0, 1812, 1812, 1812,
 1812, 1812, 1247,    0, 1812, 1812, 1812,    0, 1812, 1812,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1812,
 1812, 1812,    0, 1812, 1812, 1812, 1247, 1812,    0,    0,
    0, 1812,    0, 1812, 1812, 1812, 1812,    0, 1812,    0,
 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1812, 1812, 1994,
 1812, 1812, 1812,    0,    0,    0,    0, 1812, 1812, 1812,
 1812, 1812,    0, 1812,    0,    0, 1812, 1812, 1249, 1248,
 1250, 1812, 1812, 1995,    0, 1812, 1812, 1812, 1812, 1812,
 1812, 1812, 1812, 1812,    0, 1812,    0,    0,    0,    0,
    0,    0, 1249, 1248, 1250, 1812, 1812, 1812, 1812, 1812,
 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812,
 1812, 1812, 1812, 1812, 1812, 1812,    0, 1812, 1812, 1812,
 1812, 1812, 1812,    0, 1812, 1812, 1812, 1812,    0, 1812,
    0, 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812,
 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812,    0,
 1812, 1812, 1812, 1812,    0, 1812, 1812, 1812, 1812, 1812,
    0, 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812,
 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812,
 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812, 1812,
 1814, 1812, 1812, 1812, 1812,    0, 1812, 1812, 1812, 1812,
 1812, 1812,    0,    0, 1812, 1812, 1812, 1812, 1812, 1812,
 1812, 1812, 1812,    0,    0,    0,    0,    0, 1812,    0,
 1812, 1812, 1812, 1812, 1812, 1812,    0, 1812,    0,    0,
 1626, 1814, 1814, 1814, 1814, 1814, 1814, 1814,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1814, 1814,
 1814, 1814, 1814,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1233,    0,    0,    0,    0,    0,    0,    0,    0, 1234,
    0, 1814,    0, 1814,    0,    0,    0,    0,    0,    0,
    0,    0, 1235,    0, 1233,    0,    0,    0,    0,    0,
    0,    0,    0, 1234,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1235,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1236,    0,    0,    0,
    0,    0,    0, 1237,    0,    0,    0,    0,    0,    0,
    0, 1238,    0,    0,    0,    0,    0,    0,    0,    0,
 1236,    0,    0,    0,    0,    0,    0, 1237, 1239,    0,
 1240,    0,    0,    0,    0, 1238,    0,    0,    0,    0,
 1241,    0, 1242, 1243, 1244,    0,    0,    0,    0, 1245,
    0,    0, 1239,    0, 1240,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1241,    0, 1242, 1243, 1244,    0,
    0,    0,    0, 1245,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1246,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1246,    0,    0, 1814,    0, 1814,
    0,    0, 1814,    0, 1814, 1814,    0,    0, 1814, 1814,
    0, 1814,    0,    0,    0,    0,    0, 1814,    0, 1814,
 1814, 1814, 1814,    0, 1814,    0,    0, 1814,    0,    0,
 1814,    0,    0,    0,    0,    0,    0, 1814,    0, 1814,
 1814,    0, 1814, 1814, 1814, 1814, 1814,    0,    0, 1814,
    0,    0, 1814,    0, 1814, 1814,    0,    0, 1814, 1814,
 1814,    0, 1814, 1814, 1814, 1814, 1814, 1814, 1814,    0,
    0,    0, 1814, 1814, 1814, 1814, 1814,    0,    0,    0,
 1814,    0,    0,    0,    0,    0,    0,    0, 1814, 1814,
 1814, 1814, 1814, 1814,    0,    0,    0, 1814, 1814,    0,
 1814,    0, 1814,    0, 1814, 1814, 1814,    0, 1814,    0,
 1814, 1814, 1814,    0, 1814, 1814, 1814, 1814, 1814,    0,
 1814, 1814,    0,    0,    0,    0, 1814,    0, 1814, 1814,
    0,    0, 1814, 1814, 1814,    0,    0, 1814,    0, 1814,
    0,    0, 1814, 1814, 1814, 1814,    0, 1814,    0, 1814,
    0, 1814, 1814, 1814, 1814, 1814, 1247,    0, 1814, 1814,
 1814,    0, 1814, 1814,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1814, 1814, 1814,    0, 1814, 1814, 1814,
 1247, 1814,    0,    0,    0, 1814,    0, 1814, 1814, 1814,
 1814,    0, 1814,    0, 1814, 1814, 1814, 1814, 1814, 1814,
 1814, 1814,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1814, 1814, 1996, 1814, 1814, 1814,    0,    0,    0,
    0, 1814, 1814, 1814, 1814, 1814,    0, 1814,    0,    0,
 1814, 1814, 1249, 1248, 1250, 1814, 1814, 2298,    0, 1814,
 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814,    0, 1814,
    0,    0,    0,    0,    0,    0, 1249, 1248, 1250, 1814,
 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814,
 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814,
    0, 1814, 1814, 1814, 1814, 1814, 1814,    0, 1814, 1814,
 1814, 1814,    0, 1814,    0, 1814, 1814, 1814, 1814, 1814,
 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814,
 1814, 1814, 1814,    0, 1814, 1814, 1814, 1814,    0, 1814,
 1814, 1814, 1814, 1814,    0, 1814, 1814, 1814, 1814, 1814,
 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814,
 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814, 1814,
 1814, 1814, 1814, 1814, 1816, 1814, 1814, 1814, 1814,    0,
 1814, 1814, 1814, 1814, 1814, 1814,    0,    0, 1814, 1814,
 1814, 1814, 1814, 1814, 1814, 1814, 1814,    0,    0,    0,
    0,    0, 1814,    0, 1814, 1814, 1814, 1814, 1814, 1814,
    0, 1814,    0,    0, 1628, 1816, 1816, 1816, 1816, 1816,
 1816, 1816,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1816, 1816, 1816, 1816, 1816,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1233,    0,    0,    0,    0,    0,
    0,    0,    0, 1234,    0, 1816,    0, 1816,    0,    0,
    0,    0,    0,    0,    0,    0, 1235,    0, 1233,    0,
    0,    0,    0,    0,    0,    0,    0, 1234,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1235,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1236,    0,    0,    0,    0,    0,    0, 1237,    0,    0,
    0,    0,    0,    0,    0, 1238,    0,    0,    0,    0,
    0,    0,    0,    0, 1236,    0,    0,    0,    0,    0,
    0, 1237, 1239,    0, 1240,    0,    0,    0,    0, 1238,
    0,    0,    0,    0, 1241,    0, 1242, 1243, 1244,    0,
    0,    0,    0, 1245,    0,    0, 1239,    0, 1240,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1241,    0,
 1242, 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1246,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1246,    0,
    0, 1816,    0, 1816,    0,    0, 1816,    0, 1816, 1816,
    0,    0, 1816, 1816,    0, 1816,    0,    0,    0,    0,
    0, 1816,    0, 1816, 1816, 1816, 1816,    0, 1816,    0,
    0, 1816,    0,    0, 1816,    0,    0,    0,    0,    0,
    0, 1816,    0, 1816, 1816,    0, 1816, 1816, 1816, 1816,
 1816,    0,    0, 1816,    0,    0, 1816,    0, 1816, 1816,
    0,    0, 1816, 1816, 1816,    0, 1816, 1816, 1816, 1816,
 1816, 1816, 1816,    0,    0,    0, 1816, 1816, 1816, 1816,
 1816,    0,    0,    0, 1816,    0,    0,    0,    0,    0,
    0,    0, 1816, 1816, 1816, 1816, 1816, 1816,    0,    0,
    0, 1816, 1816,    0, 1816,    0, 1816,    0, 1816, 1816,
 1816,    0, 1816,    0, 1816, 1816, 1816,    0, 1816, 1816,
 1816, 1816, 1816,    0, 1816, 1816,    0,    0,    0,    0,
 1816,    0, 1816, 1816,    0,    0, 1816, 1816, 1816,    0,
    0, 1816,    0, 1816,    0,    0, 1816, 1816, 1816, 1816,
    0, 1816,    0, 1816,    0, 1816, 1816, 1816, 1816, 1816,
 1247,    0, 1816, 1816, 1816,    0, 1816, 1816,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1816, 1816, 1816,
    0, 1816, 1816, 1816, 1247, 1816,    0,    0,    0, 1816,
    0, 1816, 1816, 1816, 1816,    0, 1816,    0, 1816, 1816,
 1816, 1816, 1816, 1816, 1816, 1816,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1816, 1816, 2299, 1816, 1816,
 1816,    0,    0,    0,    0, 1816, 1816, 1816, 1816, 1816,
    0, 1816,    0,    0, 1816, 1816, 1249, 1248, 1250, 1816,
 1816, 2304,    0, 1816, 1816, 1816, 1816, 1816, 1816, 1816,
 1816, 1816,    0, 1816,    0,    0,    0,    0,    0,    0,
 1249, 1248, 1250, 1816, 1816, 1816, 1816, 1816, 1816, 1816,
 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1816,
 1816, 1816, 1816, 1816,    0, 1816, 1816, 1816, 1816, 1816,
 1816,    0, 1816, 1816, 1816, 1816,    0, 1816,    0, 1816,
 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1816,
 1816, 1816, 1816, 1816, 1816, 1816, 1816,    0, 1816, 1816,
 1816, 1816,    0, 1816, 1816, 1816, 1816, 1816,    0, 1816,
 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1816,
 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1816,
 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1817, 1816,
 1816, 1816, 1816,    0, 1816, 1816, 1816, 1816, 1816, 1816,
    0,    0, 1816, 1816, 1816, 1816, 1816, 1816, 1816, 1816,
 1816,    0,    0,    0,    0,    0, 1816,    0, 1816, 1816,
 1816, 1816, 1816, 1816,    0, 1816,    0,    0, 1629, 1817,
 1817, 1817, 1817, 1817, 1817, 1817,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1817, 1817, 1817, 1817,
 1817,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1233,    0,
    0,    0,    0,    0,    0,    0,    0, 1234,    0, 1817,
    0, 1817,    0,    0,    0,    0,    0,    0,    0,    0,
 1235,    0, 1233,    0,    0,    0,    0,    0,    0,    0,
    0, 1234,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1235,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1236,    0,    0,    0,    0,    0,
    0, 1237,    0,    0,    0,    0,    0,    0,    0, 1238,
    0,    0,    0,    0,    0,    0,    0,    0, 1236,    0,
    0,    0,    0,    0,    0, 1237, 1239,    0, 1240,    0,
    0,    0,    0, 1238,    0,    0,    0,    0, 1241,    0,
 1242, 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,
 1239,    0, 1240,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1241,    0, 1242, 1243, 1244,    0,    0,    0,
    0, 1245,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1246,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1246,    0,    0, 1817,    0, 1817,    0,    0,
 1817,    0, 1817, 1817,    0,    0, 1817, 1817,    0, 1817,
    0,    0,    0,    0,    0, 1817,    0, 1817, 1817, 1817,
 1817,    0, 1817,    0,    0, 1817,    0,    0, 1817,    0,
    0,    0,    0,    0,    0, 1817,    0, 1817, 1817,    0,
 1817, 1817, 1817, 1817, 1817,    0,    0, 1817,    0,    0,
 1817,    0, 1817, 1817,    0,    0, 1817, 1817, 1817,    0,
 1817, 1817, 1817, 1817, 1817, 1817, 1817,    0,    0,    0,
 1817, 1817, 1817, 1817, 1817,    0,    0,    0, 1817,    0,
    0,    0,    0,    0,    0,    0, 1817, 1817, 1817, 1817,
 1817, 1817,    0,    0,    0, 1817, 1817,    0, 1817,    0,
 1817,    0, 1817, 1817, 1817,    0, 1817,    0, 1817, 1817,
 1817,    0, 1817, 1817, 1817, 1817, 1817,    0, 1817, 1817,
    0,    0,    0,    0, 1817,    0, 1817, 1817,    0,    0,
 1817, 1817, 1817,    0,    0, 1817,    0, 1817,    0,    0,
 1817, 1817, 1817, 1817,    0, 1817,    0, 1817,    0, 1817,
 1817, 1817, 1817, 1817, 1247,    0, 1817, 1817, 1817,    0,
 1817, 1817,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1817, 1817, 1817,    0, 1817, 1817, 1817, 1247, 1817,
    0,    0,    0, 1817,    0, 1817, 1817, 1817, 1817,    0,
 1817,    0, 1817, 1817, 1817, 1817, 1817, 1817, 1817, 1817,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1817,
 1817, 2305, 1817, 1817, 1817,    0,    0,    0,    0, 1817,
 1817, 1817, 1817, 1817,    0, 1817,    0,    0, 1817, 1817,
 1249, 1248, 1250, 1817, 1817, 2307,    0, 1817, 1817, 1817,
 1817, 1817, 1817, 1817, 1817, 1817,    0, 1817,    0,    0,
    0,    0,    0,    0, 1249, 1248, 1250, 1817, 1817, 1817,
 1817, 1817, 1817, 1817, 1817, 1817, 1817, 1817, 1817, 1817,
 1817, 1817, 1817, 1817, 1817, 1817, 1817, 1817,    0, 1817,
 1817, 1817, 1817, 1817, 1817,    0, 1817, 1817, 1817, 1817,
    0, 1817,    0, 1817, 1817, 1817, 1817, 1817, 1817, 1817,
 1817, 1817, 1817, 1817, 1817, 1817, 1817, 1817, 1817, 1817,
 1817,    0, 1817, 1817, 1817, 1817,    0, 1817, 1817, 1817,
 1817, 1817,    0, 1817, 1817, 1817, 1817, 1817, 1817, 1817,
 1817, 1817, 1817, 1817, 1817, 1817, 1817, 1817, 1817, 1817,
 1817, 1817, 1817, 1817, 1817, 1817, 1817, 1817, 1817, 1817,
 1817, 1817, 1818, 1817, 1817, 1817, 1817,    0, 1817, 1817,
 1817, 1817, 1817, 1817,    0,    0, 1817, 1817, 1817, 1817,
 1817, 1817, 1817, 1817, 1817,    0,    0,    0,    0,    0,
 1817,    0, 1817, 1817, 1817, 1817, 1817, 1817,    0, 1817,
    0,    0, 1630, 1818, 1818, 1818, 1818, 1818, 1818, 1818,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1818, 1818, 1818, 1818, 1818,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1233,    0,    0,    0,    0,    0,    0,    0,
    0, 1234,    0, 1818,    0, 1818,    0,    0,    0,    0,
    0,    0,    0,    0, 1235,    0, 1233,    0,    0,    0,
    0,    0,    0,    0,    0, 1234,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1235,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1236,    0,
    0,    0,    0,    0,    0, 1237,    0,    0,    0,    0,
    0,    0,    0, 1238,    0,    0,    0,    0,    0,    0,
    0,    0, 1236,    0,    0,    0,    0,    0,    0, 1237,
 1239,    0, 1240,    0,    0,    0,    0, 1238,    0,    0,
    0,    0, 1241,    0, 1242, 1243, 1244,    0,    0,    0,
    0, 1245,    0,    0, 1239,    0, 1240,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1241,    0, 1242, 1243,
 1244,    0,    0,    0,    0, 1245,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1246,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1246,    0,    0, 1818,
    0, 1818,    0,    0, 1818,    0, 1818, 1818,    0,    0,
 1818, 1818,    0, 1818,    0,    0,    0,    0,    0, 1818,
    0, 1818, 1818, 1818, 1818,    0, 1818,    0,    0, 1818,
    0,    0, 1818,    0,    0,    0,    0,    0,    0, 1818,
    0, 1818, 1818,    0, 1818, 1818, 1818, 1818, 1818,    0,
    0, 1818,    0,    0, 1818,    0, 1818, 1818,    0,    0,
 1818, 1818, 1818,    0, 1818, 1818, 1818, 1818, 1818, 1818,
 1818,    0,    0,    0, 1818, 1818, 1818, 1818, 1818,    0,
    0,    0, 1818,    0,    0,    0,    0,    0,    0,    0,
 1818, 1818, 1818, 1818, 1818, 1818,    0,    0,    0, 1818,
 1818,    0, 1818,    0, 1818,    0, 1818, 1818, 1818,    0,
 1818,    0, 1818, 1818, 1818,    0, 1818, 1818, 1818, 1818,
 1818,    0, 1818, 1818,    0,    0,    0,    0, 1818,    0,
 1818, 1818,    0,    0, 1818, 1818, 1818,    0,    0, 1818,
    0, 1818,    0,    0, 1818, 1818, 1818, 1818,    0, 1818,
    0, 1818,    0, 1818, 1818, 1818, 1818, 1818, 1247,    0,
 1818, 1818, 1818,    0, 1818, 1818,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1818, 1818, 1818,    0, 1818,
 1818, 1818, 1247, 1818,    0,    0,    0, 1818,    0, 1818,
 1818, 1818, 1818,    0, 1818,    0, 1818, 1818, 1818, 1818,
 1818, 1818, 1818, 1818,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1818, 1818, 2308, 1818, 1818, 1818,    0,
    0,    0,    0, 1818, 1818, 1818, 1818, 1818,    0, 1818,
    0,    0, 1818, 1818, 1249, 1248, 1250, 1818, 1818, 2309,
    0, 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818,
    0, 1818,    0,    0,    0,    0,    0,    0, 1249, 1248,
 1250, 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818,
 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818,
 1818, 1818,    0, 1818, 1818, 1818, 1818, 1818, 1818,    0,
 1818, 1818, 1818, 1818,    0, 1818,    0, 1818, 1818, 1818,
 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818,
 1818, 1818, 1818, 1818, 1818,    0, 1818, 1818, 1818, 1818,
    0, 1818, 1818, 1818, 1818, 1818,    0, 1818, 1818, 1818,
 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818,
 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818,
 1818, 1818, 1818, 1818, 1818, 1818, 1819, 1818, 1818, 1818,
 1818,    0, 1818, 1818, 1818, 1818, 1818, 1818,    0,    0,
 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818, 1818,    0,
    0,    0,    0,    0, 1818,    0, 1818, 1818, 1818, 1818,
 1818, 1818,    0, 1818,    0,    0, 1632, 1819, 1819, 1819,
 1819, 1819, 1819, 1819,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1819, 1819, 1819, 1819, 1819,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1233,    0,    0,    0,
    0,    0,    0,    0,    0, 1234,    0, 1819,    0, 1819,
    0,    0,    0,    0,    0,    0,    0,    0, 1235,    0,
 1233,    0,    0,    0,    0,    0,    0,    0,    0, 1234,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1235,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1236,    0,    0,    0,    0,    0,    0, 1237,
    0,    0,    0,    0,    0,    0,    0, 1238,    0,    0,
    0,    0,    0,    0,    0,    0, 1236,    0,    0,    0,
    0,    0,    0, 1237, 1239,    0, 1240,    0,    0,    0,
    0, 1238,    0,    0,    0,    0, 1241,    0, 1242, 1243,
 1244,    0,    0,    0,    0, 1245,    0,    0, 1239,    0,
 1240,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1241,    0, 1242, 1243, 1244,    0,    0,    0,    0, 1245,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1246,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1246,    0,    0, 1819,    0, 1819,    0,    0, 1819,    0,
 1819, 1819,    0,    0, 1819, 1819,    0, 1819,    0,    0,
    0,    0,    0, 1819,    0, 1819, 1819, 1819, 1819,    0,
 1819,    0,    0, 1819,    0,    0, 1819,    0,    0,    0,
    0,    0,    0, 1819,    0, 1819, 1819,    0, 1819, 1819,
 1819, 1819, 1819,    0,    0, 1819,    0,    0, 1819,    0,
 1819, 1819,    0,    0, 1819, 1819, 1819,    0, 1819, 1819,
 1819, 1819, 1819, 1819, 1819,    0,    0,    0, 1819, 1819,
 1819, 1819, 1819,    0,    0,    0, 1819,    0,    0,    0,
    0,    0,    0,    0, 1819, 1819, 1819, 1819, 1819, 1819,
    0,    0,    0, 1819, 1819,    0, 1819,    0, 1819,    0,
 1819, 1819, 1819,    0, 1819,    0, 1819, 1819, 1819,    0,
 1819, 1819, 1819, 1819, 1819,    0, 1819, 1819,    0,    0,
    0,    0, 1819,    0, 1819, 1819,    0,    0, 1819, 1819,
 1819,    0,    0, 1819,    0, 1819,    0,    0, 1819, 1819,
 1819, 1819,    0, 1819,    0, 1819,    0, 1819, 1819, 1819,
 1819, 1819, 1247,    0, 1819, 1819, 1819,    0, 1819, 1819,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1819,
 1819, 1819,    0, 1819, 1819, 1819, 1247, 1819,    0,    0,
    0, 1819,    0, 1819, 1819, 1819, 1819,    0, 1819,    0,
 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1819, 1819, 2310,
 1819, 1819, 1819,    0,    0,    0,    0, 1819, 1819, 1819,
 1819, 1819,    0, 1819,    0,    0, 1819, 1819, 1249, 1248,
 1250, 1819, 1819, 2313,    0, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819, 1819,    0, 1819,    0,    0,    0,    0,
    0,    0, 1249, 1248, 1250, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819, 1819, 1819, 1819,    0, 1819, 1819, 1819,
 1819, 1819, 1819,    0, 1819, 1819, 1819, 1819,    0, 1819,
    0, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,    0,
 1819, 1819, 1819, 1819,    0, 1819, 1819, 1819, 1819, 1819,
    0, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819, 1819,
 1820, 1819, 1819, 1819, 1819,    0, 1819, 1819, 1819, 1819,
 1819, 1819,    0,    0, 1819, 1819, 1819, 1819, 1819, 1819,
 1819, 1819, 1819,    0,    0,    0,    0,    0, 1819,    0,
 1819, 1819, 1819, 1819, 1819, 1819,    0, 1819,    0,    0,
 1633, 1820, 1820, 1820, 1820, 1820, 1820, 1820,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1820, 1820,
 1820, 1820, 1820,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1233,    0,    0,    0,    0,    0,    0,    0,    0, 1234,
    0, 1820,    0, 1820,    0,    0,    0,    0,    0,    0,
    0,    0, 1235,    0, 1233,    0,    0,    0,    0,    0,
    0,    0,    0, 1234,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1235,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1236,    0,    0,    0,
    0,    0,    0, 1237,    0,    0,    0,    0,    0,    0,
    0, 1238,    0,    0,    0,    0,    0,    0,    0,    0,
 1236,    0,    0,    0,    0,    0,    0, 1237, 1239,    0,
 1240,    0,    0,    0,    0, 1238,    0,    0,    0,    0,
 1241,    0, 1242, 1243, 1244,    0,    0,    0,    0, 1245,
    0,    0, 1239,    0, 1240,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1241,    0, 1242, 1243, 1244,    0,
    0,    0,    0, 1245,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1246,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1246,    0,    0, 1820,    0, 1820,
    0,    0, 1820,    0, 1820, 1820,    0,    0, 1820, 1820,
    0, 1820,    0,    0,    0,    0,    0, 1820,    0, 1820,
 1820, 1820, 1820,    0, 1820,    0,    0, 1820,    0,    0,
 1820,    0,    0,    0,    0,    0,    0, 1820,    0, 1820,
 1820,    0, 1820, 1820, 1820, 1820, 1820,    0,    0, 1820,
    0,    0, 1820,    0, 1820, 1820,    0,    0, 1820, 1820,
 1820,    0, 1820, 1820, 1820, 1820, 1820, 1820, 1820,    0,
    0,    0, 1820, 1820, 1820, 1820, 1820,    0,    0,    0,
 1820,    0,    0,    0,    0,    0,    0,    0, 1820, 1820,
 1820, 1820, 1820, 1820,    0,    0,    0, 1820, 1820,    0,
 1820,    0, 1820,    0, 1820, 1820, 1820,    0, 1820,    0,
 1820, 1820, 1820,    0, 1820, 1820, 1820, 1820, 1820,    0,
 1820, 1820,    0,    0,    0,    0, 1820,    0, 1820, 1820,
    0,    0, 1820, 1820, 1820,    0,    0, 1820,    0, 1820,
    0,    0, 1820, 1820, 1820, 1820,    0, 1820,    0, 1820,
    0, 1820, 1820, 1820, 1820, 1820, 1247,    0, 1820, 1820,
 1820,    0, 1820, 1820,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1820, 1820, 1820,    0, 1820, 1820, 1820,
 1247, 1820,    0,    0,    0, 1820,    0, 1820, 1820, 1820,
 1820,    0, 1820,    0, 1820, 1820, 1820, 1820, 1820, 1820,
 1820, 1820,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1820, 1820, 2314, 1820, 1820, 1820,    0,    0,    0,
    0, 1820, 1820, 1820, 1820, 1820,    0, 1820,    0,    0,
 1820, 1820, 1249, 1248, 1250, 1820, 1820, 2594,    0, 1820,
 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820,    0, 1820,
    0,    0,    0,    0,    0,    0, 1249, 1248, 1250, 1820,
 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820,
 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820,
    0, 1820, 1820, 1820, 1820, 1820, 1820,    0, 1820, 1820,
 1820, 1820,    0, 1820,    0, 1820, 1820, 1820, 1820, 1820,
 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820,
 1820, 1820, 1820,    0, 1820, 1820, 1820, 1820,    0, 1820,
 1820, 1820, 1820, 1820,    0, 1820, 1820, 1820, 1820, 1820,
 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820,
 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820, 1820,
 1820, 1820, 1820, 1820, 1821, 1820, 1820, 1820, 1820,    0,
 1820, 1820, 1820, 1820, 1820, 1820,    0,    0, 1820, 1820,
 1820, 1820, 1820, 1820, 1820, 1820, 1820,    0,    0,    0,
    0,    0, 1820,    0, 1820, 1820, 1820, 1820, 1820, 1820,
    0, 1820,    0,    0, 1635, 1821, 1821, 1821, 1821, 1821,
 1821, 1821,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1821, 1821, 1821, 1821, 1821,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1233,    0,    0,    0,    0,    0,
    0,    0,    0, 1234,    0, 1821,    0, 1821,    0,    0,
    0,    0,    0,    0,    0,    0, 1235,    0, 1233,    0,
    0,    0,    0,    0,    0,    0,    0, 1234,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1235,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1236,    0,    0,    0,    0,    0,    0, 1237,    0,    0,
    0,    0,    0,    0,    0, 1238,    0,    0,    0,    0,
    0,    0,    0,    0, 1236,    0,    0,    0,    0,    0,
    0, 1237, 1239,    0, 1240,    0,    0,    0,    0, 1238,
    0,    0,    0,    0, 1241,    0, 1242, 1243, 1244,    0,
    0,    0,    0, 1245,    0,    0, 1239,    0, 1240,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1241,    0,
 1242, 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1246,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1246,    0,
    0, 1821,    0, 1821,    0,    0, 1821,    0, 1821, 1821,
    0,    0, 1821, 1821,    0, 1821,    0,    0,    0,    0,
    0, 1821,    0, 1821, 1821, 1821, 1821,    0, 1821,    0,
    0, 1821,    0,    0, 1821,    0,    0,    0,    0,    0,
    0, 1821,    0, 1821, 1821,    0, 1821, 1821, 1821, 1821,
 1821,    0,    0, 1821,    0,    0, 1821,    0, 1821, 1821,
    0,    0, 1821, 1821, 1821,    0, 1821, 1821, 1821, 1821,
 1821, 1821, 1821,    0,    0,    0, 1821, 1821, 1821, 1821,
 1821,    0,    0,    0, 1821,    0,    0,    0,    0,    0,
    0,    0, 1821, 1821, 1821, 1821, 1821, 1821,    0,    0,
    0, 1821, 1821,    0, 1821,    0, 1821,    0, 1821, 1821,
 1821,    0, 1821,    0, 1821, 1821, 1821,    0, 1821, 1821,
 1821, 1821, 1821,    0, 1821, 1821,    0,    0,    0,    0,
 1821,    0, 1821, 1821,    0,    0, 1821, 1821, 1821,    0,
    0, 1821,    0, 1821,    0,    0, 1821, 1821, 1821, 1821,
    0, 1821,    0, 1821,    0, 1821, 1821, 1821, 1821, 1821,
 1247,    0, 1821, 1821, 1821,    0, 1821, 1821,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1821, 1821, 1821,
    0, 1821, 1821, 1821, 1247, 1821,    0,    0,    0, 1821,
    0, 1821, 1821, 1821, 1821,    0, 1821,    0, 1821, 1821,
 1821, 1821, 1821, 1821, 1821, 1821,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1821, 1821, 2596, 1821, 1821,
 1821,    0,    0,    0,    0, 1821, 1821, 1821, 1821, 1821,
    0, 1821,    0,    0, 1821, 1821, 1249, 1248, 1250, 1821,
 1821, 2597,    0, 1821, 1821, 1821, 1821, 1821, 1821, 1821,
 1821, 1821,    0, 1821,    0,    0,    0,    0,    0,    0,
 1249, 1248, 1250, 1821, 1821, 1821, 1821, 1821, 1821, 1821,
 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1821,
 1821, 1821, 1821, 1821,    0, 1821, 1821, 1821, 1821, 1821,
 1821,    0, 1821, 1821, 1821, 1821,    0, 1821,    0, 1821,
 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1821,
 1821, 1821, 1821, 1821, 1821, 1821, 1821,    0, 1821, 1821,
 1821, 1821,    0, 1821, 1821, 1821, 1821, 1821,    0, 1821,
 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1821,
 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1821,
 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1822, 1821,
 1821, 1821, 1821,    0, 1821, 1821, 1821, 1821, 1821, 1821,
    0,    0, 1821, 1821, 1821, 1821, 1821, 1821, 1821, 1821,
 1821,    0,    0,    0,    0,    0, 1821,    0, 1821, 1821,
 1821, 1821, 1821, 1821,    0, 1821,    0,    0, 1637, 1822,
 1822, 1822, 1822, 1822, 1822, 1822,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1822, 1822, 1822, 1822,
 1822,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1233,    0,
    0,    0,    0,    0,    0,    0,    0, 1234,    0, 1822,
    0, 1822,    0,    0,    0,    0,    0,    0,    0,    0,
 1235,    0, 1233,    0,    0,    0,    0,    0,    0,    0,
    0, 1234,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1235,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1236,    0,    0,    0,    0,    0,
    0, 1237,    0,    0,    0,    0,    0,    0,    0, 1238,
    0,    0,    0,    0,    0,    0,    0,    0, 1236,    0,
    0,    0,    0,    0,    0, 1237, 1239,    0, 1240,    0,
    0,    0,    0, 1238,    0,    0,    0,    0, 1241,    0,
 1242, 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,
 1239,    0, 1240,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1241,    0, 1242, 1243, 1244,    0,    0,    0,
    0, 1245,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1246,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1246,    0,    0, 1822,    0, 1822,    0,    0,
 1822,    0, 1822, 1822,    0,    0, 1822, 1822,    0, 1822,
    0,    0,    0,    0,    0, 1822,    0, 1822, 1822, 1822,
 1822,    0, 1822,    0,    0, 1822,    0,    0, 1822,    0,
    0,    0,    0,    0,    0, 1822,    0, 1822, 1822,    0,
 1822, 1822, 1822, 1822, 1822,    0,    0, 1822,    0,    0,
 1822,    0, 1822, 1822,    0,    0, 1822, 1822, 1822,    0,
 1822, 1822, 1822, 1822, 1822, 1822, 1822,    0,    0,    0,
 1822, 1822, 1822, 1822, 1822,    0,    0,    0, 1822,    0,
    0,    0,    0,    0,    0,    0, 1822, 1822, 1822, 1822,
 1822, 1822,    0,    0,    0, 1822, 1822,    0, 1822,    0,
 1822,    0, 1822, 1822, 1822,    0, 1822,    0, 1822, 1822,
 1822,    0, 1822, 1822, 1822, 1822, 1822,    0, 1822, 1822,
    0,    0,    0,    0, 1822,    0, 1822, 1822,    0,    0,
 1822, 1822, 1822,    0,    0, 1822,    0, 1822,    0,    0,
 1822, 1822, 1822, 1822,    0, 1822,    0, 1822,    0, 1822,
 1822, 1822, 1822, 1822, 1247,    0, 1822, 1822, 1822,    0,
 1822, 1822,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1822, 1822, 1822,    0, 1822, 1822, 1822, 1247, 1822,
    0,    0,    0, 1822,    0, 1822, 1822, 1822, 1822,    0,
 1822,    0, 1822, 1822, 1822, 1822, 1822, 1822, 1822, 1822,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1822,
 1822, 2604, 1822, 1822, 1822,    0,    0,    0,    0, 1822,
 1822, 1822, 1822, 1822,    0, 1822,    0,    0, 1822, 1822,
 1249, 1248, 1250, 1822, 1822, 2607,    0, 1822, 1822, 1822,
 1822, 1822, 1822, 1822, 1822, 1822,    0, 1822,    0,    0,
    0,    0,    0,    0, 1249, 1248, 1250, 1822, 1822, 1822,
 1822, 1822, 1822, 1822, 1822, 1822, 1822, 1822, 1822, 1822,
 1822, 1822, 1822, 1822, 1822, 1822, 1822, 1822,    0, 1822,
 1822, 1822, 1822, 1822, 1822,    0, 1822, 1822, 1822, 1822,
    0, 1822,    0, 1822, 1822, 1822, 1822, 1822, 1822, 1822,
 1822, 1822, 1822, 1822, 1822, 1822, 1822, 1822, 1822, 1822,
 1822,    0, 1822, 1822, 1822, 1822,    0, 1822, 1822, 1822,
 1822, 1822,    0, 1822, 1822, 1822, 1822, 1822, 1822, 1822,
 1822, 1822, 1822, 1822, 1822, 1822, 1822, 1822, 1822, 1822,
 1822, 1822, 1822, 1822, 1822, 1822, 1822, 1822, 1822, 1822,
 1822, 1822, 1823, 1822, 1822, 1822, 1822,    0, 1822, 1822,
 1822, 1822, 1822, 1822,    0,    0, 1822, 1822, 1822, 1822,
 1822, 1822, 1822, 1822, 1822,    0,    0,    0,    0,    0,
 1822,    0, 1822, 1822, 1822, 1822, 1822, 1822,    0, 1822,
    0,    0, 1638, 1823, 1823, 1823, 1823, 1823, 1823, 1823,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1823, 1823, 1823, 1823, 1823,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1233,    0,    0,    0,    0,    0,    0,    0,
    0, 1234,    0, 1823,    0, 1823,    0,    0,    0,    0,
    0,    0,    0,    0, 1235,    0, 1233,    0,    0,    0,
    0,    0,    0,    0,    0, 1234,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1235,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1236,    0,
    0,    0,    0,    0,    0, 1237,    0,    0,    0,    0,
    0,    0,    0, 1238,    0,    0,    0,    0,    0,    0,
    0,    0, 1236,    0,    0,    0,    0,    0,    0, 1237,
 1239,    0, 1240,    0,    0,    0,    0, 1238,    0,    0,
    0,    0, 1241,    0, 1242, 1243, 1244,    0,    0,    0,
    0, 1245,    0,    0, 1239,    0, 1240,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1241,    0, 1242, 1243,
 1244,    0,    0,    0,    0, 1245,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1246,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1246,    0,    0, 1823,
    0, 1823,    0,    0, 1823,    0, 1823, 1823,    0,    0,
 1823, 1823,    0, 1823,    0,    0,    0,    0,    0, 1823,
    0, 1823, 1823, 1823, 1823,    0, 1823,    0,    0, 1823,
    0,    0, 1823,    0,    0,    0,    0,    0,    0, 1823,
    0, 1823, 1823,    0, 1823, 1823, 1823, 1823, 1823,    0,
    0, 1823,    0,    0, 1823,    0, 1823, 1823,    0,    0,
 1823, 1823, 1823,    0, 1823, 1823, 1823, 1823, 1823, 1823,
 1823,    0,    0,    0, 1823, 1823, 1823, 1823, 1823,    0,
    0,    0, 1823,    0,    0,    0,    0,    0,    0,    0,
 1823, 1823, 1823, 1823, 1823, 1823,    0,    0,    0, 1823,
 1823,    0, 1823,    0, 1823,    0, 1823, 1823, 1823,    0,
 1823,    0, 1823, 1823, 1823,    0, 1823, 1823, 1823, 1823,
 1823,    0, 1823, 1823,    0,    0,    0,    0, 1823,    0,
 1823, 1823,    0,    0, 1823, 1823, 1823,    0,    0, 1823,
    0, 1823,    0,    0, 1823, 1823, 1823, 1823,    0, 1823,
    0, 1823,    0, 1823, 1823, 1823, 1823, 1823, 1247,    0,
 1823, 1823, 1823,    0, 1823, 1823,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1823, 1823, 1823,    0, 1823,
 1823, 1823, 1247, 1823,    0,    0,    0, 1823,    0, 1823,
 1823, 1823, 1823,    0, 1823,    0, 1823, 1823, 1823, 1823,
 1823, 1823, 1823, 1823,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1823, 1823, 2608, 1823, 1823, 1823,    0,
    0,    0,    0, 1823, 1823, 1823, 1823, 1823,    0, 1823,
    0,    0, 1823, 1823, 1249, 1248, 1250, 1823, 1823, 2621,
    0, 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823,
    0, 1823,    0,    0,    0,    0,    0,    0, 1249, 1248,
 1250, 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823,
 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823,
 1823, 1823,    0, 1823, 1823, 1823, 1823, 1823, 1823,    0,
 1823, 1823, 1823, 1823,    0, 1823,    0, 1823, 1823, 1823,
 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823,
 1823, 1823, 1823, 1823, 1823,    0, 1823, 1823, 1823, 1823,
    0, 1823, 1823, 1823, 1823, 1823,    0, 1823, 1823, 1823,
 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823,
 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823,
 1823, 1823, 1823, 1823, 1823, 1823, 1824, 1823, 1823, 1823,
 1823,    0, 1823, 1823, 1823, 1823, 1823, 1823,    0,    0,
 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823, 1823,    0,
    0,    0,    0,    0, 1823,    0, 1823, 1823, 1823, 1823,
 1823, 1823,    0, 1823,    0,    0, 1639, 1824, 1824, 1824,
 1824, 1824, 1824, 1824,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1824, 1824, 1824, 1824, 1824,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1233,    0,    0,    0,
    0,    0,    0,    0,    0, 1234,    0, 1824,    0, 1824,
    0,    0,    0,    0,    0,    0,    0,    0, 1235,    0,
 1233,    0,    0,    0,    0,    0,    0,    0,    0, 1234,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1235,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1236,    0,    0,    0,    0,    0,    0, 1237,
    0,    0,    0,    0,    0,    0,    0, 1238,    0,    0,
    0,    0,    0,    0,    0,    0, 1236,    0,    0,    0,
    0,    0,    0, 1237, 1239,    0, 1240,    0,    0,    0,
    0, 1238,    0,    0,    0,    0, 1241,    0, 1242, 1243,
 1244,    0,    0,    0,    0, 1245,    0,    0, 1239,    0,
 1240,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1241,    0, 1242, 1243, 1244,    0,    0,    0,    0, 1245,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1246,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1246,    0,    0, 1824,    0, 1824,    0,    0, 1824,    0,
 1824, 1824,    0,    0, 1824, 1824,    0, 1824,    0,    0,
    0,    0,    0, 1824,    0, 1824, 1824, 1824, 1824,    0,
 1824,    0,    0, 1824,    0,    0, 1824,    0,    0,    0,
    0,    0,    0, 1824,    0, 1824, 1824,    0, 1824, 1824,
 1824, 1824, 1824,    0,    0, 1824,    0,    0, 1824,    0,
 1824, 1824,    0,    0, 1824, 1824, 1824,    0, 1824, 1824,
 1824, 1824, 1824, 1824, 1824,    0,    0,    0, 1824, 1824,
 1824, 1824, 1824,    0,    0,    0, 1824,    0,    0,    0,
    0,    0,    0,    0, 1824, 1824, 1824, 1824, 1824, 1824,
    0,    0,    0, 1824, 1824,    0, 1824,    0, 1824,    0,
 1824, 1824, 1824,    0, 1824,    0, 1824, 1824, 1824,    0,
 1824, 1824, 1824, 1824, 1824,    0, 1824, 1824,    0,    0,
    0,    0, 1824,    0, 1824, 1824,    0,    0, 1824, 1824,
 1824,    0,    0, 1824,    0, 1824,    0,    0, 1824, 1824,
 1824, 1824,    0, 1824,    0, 1824,    0, 1824, 1824, 1824,
 1824, 1824, 1247,    0, 1824, 1824, 1824,    0, 1824, 1824,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1824,
 1824, 1824,    0, 1824, 1824, 1824, 1247, 1824,    0,    0,
    0, 1824,    0, 1824, 1824, 1824, 1824,    0, 1824,    0,
 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1824, 1824, 2626,
 1824, 1824, 1824,    0,    0,    0,    0, 1824, 1824, 1824,
 1824, 1824,    0, 1824,    0,    0, 1824, 1824, 1249, 1248,
 1250, 1824, 1824, 2627,    0, 1824, 1824, 1824, 1824, 1824,
 1824, 1824, 1824, 1824,    0, 1824,    0,    0,    0,    0,
    0,    0, 1249, 1248, 1250, 1824, 1824, 1824, 1824, 1824,
 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824,
 1824, 1824, 1824, 1824, 1824, 1824,    0, 1824, 1824, 1824,
 1824, 1824, 1824,    0, 1824, 1824, 1824, 1824,    0, 1824,
    0, 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824,
 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824,    0,
 1824, 1824, 1824, 1824,    0, 1824, 1824, 1824, 1824, 1824,
    0, 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824,
 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824,
 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824, 1824,
 1825, 1824, 1824, 1824, 1824,    0, 1824, 1824, 1824, 1824,
 1824, 1824,    0,    0, 1824, 1824, 1824, 1824, 1824, 1824,
 1824, 1824, 1824,    0,    0,    0,    0,    0, 1824,    0,
 1824, 1824, 1824, 1824, 1824, 1824,    0, 1824,    0,    0,
 1640, 1825, 1825, 1825, 1825, 1825, 1825, 1825,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1825, 1825,
 1825, 1825, 1825,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1233,    0,    0,    0,    0,    0,    0,    0,    0, 1234,
    0, 1825,    0, 1825,    0,    0,    0,    0,    0,    0,
    0,    0, 1235,    0, 1233,    0,    0,    0,    0,    0,
    0,    0,    0, 1234,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1235,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1236,    0,    0,    0,
    0,    0,    0, 1237,    0,    0,    0,    0,    0,    0,
    0, 1238,    0,    0,    0,    0,    0,    0,    0,    0,
 1236,    0,    0,    0,    0,    0,    0, 1237, 1239,    0,
 1240,    0,    0,    0,    0, 1238,    0,    0,    0,    0,
 1241,    0, 1242, 1243, 1244,    0,    0,    0,    0, 1245,
    0,    0, 1239,    0, 1240,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1241,    0, 1242, 1243, 1244,    0,
    0,    0,    0, 1245,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1246,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1246,    0,    0, 1825,    0, 1825,
    0,    0, 1825,    0, 1825, 1825,    0,    0, 1825, 1825,
    0, 1825,    0,    0,    0,    0,    0, 1825,    0, 1825,
 1825, 1825, 1825,    0, 1825,    0,    0, 1825,    0,    0,
 1825,    0,    0,    0,    0,    0,    0, 1825,    0, 1825,
 1825,    0, 1825, 1825, 1825, 1825, 1825,    0,    0, 1825,
    0,    0, 1825,    0, 1825, 1825,    0,    0, 1825, 1825,
 1825,    0, 1825, 1825, 1825, 1825, 1825, 1825, 1825,    0,
    0,    0, 1825, 1825, 1825, 1825, 1825,    0,    0,    0,
 1825,    0,    0,    0,    0,    0,    0,    0, 1825, 1825,
 1825, 1825, 1825, 1825,    0,    0,    0, 1825, 1825,    0,
 1825,    0, 1825,    0, 1825, 1825, 1825,    0, 1825,    0,
 1825, 1825, 1825,    0, 1825, 1825, 1825, 1825, 1825,    0,
 1825, 1825,    0,    0,    0,    0, 1825,    0, 1825, 1825,
    0,    0, 1825, 1825, 1825,    0,    0, 1825,    0, 1825,
    0,    0, 1825, 1825, 1825, 1825,    0, 1825,    0, 1825,
    0, 1825, 1825, 1825, 1825, 1825, 1247,    0, 1825, 1825,
 1825,    0, 1825, 1825,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1825, 1825, 1825,    0, 1825, 1825, 1825,
 1247, 1825,    0,    0,    0, 1825,    0, 1825, 1825, 1825,
 1825,    0, 1825,    0, 1825, 1825, 1825, 1825, 1825, 1825,
 1825, 1825,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1825, 1825, 2628, 1825, 1825, 1825,    0,    0,    0,
    0, 1825, 1825, 1825, 1825, 1825,    0, 1825,    0,    0,
 1825, 1825, 1249, 1248, 1250, 1825, 1825, 2629,    0, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,    0, 1825,
    0,    0,    0,    0,    0,    0, 1249, 1248, 1250, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
    0, 1825, 1825, 1825, 1825, 1825, 1825,    0, 1825, 1825,
 1825, 1825,    0, 1825,    0, 1825, 1825, 1825, 1825, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
 1825, 1825, 1825,    0, 1825, 1825, 1825, 1825,    0, 1825,
 1825, 1825, 1825, 1825,    0, 1825, 1825, 1825, 1825, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825, 1825,
 1825, 1825, 1825, 1825, 1826, 1825, 1825, 1825, 1825,    0,
 1825, 1825, 1825, 1825, 1825, 1825,    0,    0, 1825, 1825,
 1825, 1825, 1825, 1825, 1825, 1825, 1825,    0,    0,    0,
    0,    0, 1825,    0, 1825, 1825, 1825, 1825, 1825, 1825,
    0, 1825,    0,    0, 1641, 1826, 1826, 1826, 1826, 1826,
 1826, 1826,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1826, 1826, 1826, 1826, 1826,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1233,    0,    0,    0,    0,    0,
    0,    0,    0, 1234,    0, 1826,    0, 1826,    0,    0,
    0,    0,    0,    0,    0,    0, 1235,    0, 1233,    0,
    0,    0,    0,    0,    0,    0,    0, 1234,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1235,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1236,    0,    0,    0,    0,    0,    0, 1237,    0,    0,
    0,    0,    0,    0,    0, 1238,    0,    0,    0,    0,
    0,    0,    0,    0, 1236,    0,    0,    0,    0,    0,
    0, 1237, 1239,    0, 1240,    0,    0,    0,    0, 1238,
    0,    0,    0,    0, 1241,    0, 1242, 1243, 1244,    0,
    0,    0,    0, 1245,    0,    0, 1239,    0, 1240,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1241,    0,
 1242, 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1246,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1246,    0,
    0, 1826,    0, 1826,    0,    0, 1826,    0, 1826, 1826,
    0,    0, 1826, 1826,    0, 1826,    0,    0,    0,    0,
    0, 1826,    0, 1826, 1826, 1826, 1826,    0, 1826,    0,
    0, 1826,    0,    0, 1826,    0,    0,    0,    0,    0,
    0, 1826,    0, 1826, 1826,    0, 1826, 1826, 1826, 1826,
 1826,    0,    0, 1826,    0,    0, 1826,    0, 1826, 1826,
    0,    0, 1826, 1826, 1826,    0, 1826, 1826, 1826, 1826,
 1826, 1826, 1826,    0,    0,    0, 1826, 1826, 1826, 1826,
 1826,    0,    0,    0, 1826,    0,    0,    0,    0,    0,
    0,    0, 1826, 1826, 1826, 1826, 1826, 1826,    0,    0,
    0, 1826, 1826,    0, 1826,    0, 1826,    0, 1826, 1826,
 1826,    0, 1826,    0, 1826, 1826, 1826,    0, 1826, 1826,
 1826, 1826, 1826,    0, 1826, 1826,    0,    0,    0,    0,
 1826,    0, 1826, 1826,    0,    0, 1826, 1826, 1826,    0,
    0, 1826,    0, 1826,    0,    0, 1826, 1826, 1826, 1826,
    0, 1826,    0, 1826,    0, 1826, 1826, 1826, 1826, 1826,
 1247,    0, 1826, 1826, 1826,    0, 1826, 1826,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1826, 1826, 1826,
    0, 1826, 1826, 1826, 1247, 1826,    0,    0,    0, 1826,
    0, 1826, 1826, 1826, 1826,    0, 1826,    0, 1826, 1826,
 1826, 1826, 1826, 1826, 1826, 1826,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1826, 1826, 2630, 1826, 1826,
 1826,    0,    0,    0,    0, 1826, 1826, 1826, 1826, 1826,
    0, 1826,    0,    0, 1826, 1826, 1249, 1248, 1250, 1826,
 1826, 2631,    0, 1826, 1826, 1826, 1826, 1826, 1826, 1826,
 1826, 1826,    0, 1826,    0,    0,    0,    0,    0,    0,
 1249, 1248, 1250, 1826, 1826, 1826, 1826, 1826, 1826, 1826,
 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1826,
 1826, 1826, 1826, 1826,    0, 1826, 1826, 1826, 1826, 1826,
 1826,    0, 1826, 1826, 1826, 1826,    0, 1826,    0, 1826,
 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1826,
 1826, 1826, 1826, 1826, 1826, 1826, 1826,    0, 1826, 1826,
 1826, 1826,    0, 1826, 1826, 1826, 1826, 1826,    0, 1826,
 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1826,
 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1826,
 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1827, 1826,
 1826, 1826, 1826,    0, 1826, 1826, 1826, 1826, 1826, 1826,
    0,    0, 1826, 1826, 1826, 1826, 1826, 1826, 1826, 1826,
 1826,    0,    0,    0,    0,    0, 1826,    0, 1826, 1826,
 1826, 1826, 1826, 1826,    0, 1826,    0,    0, 1643, 1827,
 1827, 1827, 1827, 1827, 1827, 1827,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1827, 1827, 1827, 1827,
 1827,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1233,    0,
    0,    0,    0,    0,    0,    0,    0, 1234,    0, 1827,
    0, 1827,    0,    0,    0,    0,    0,    0,    0,    0,
 1235,    0, 1233,    0,    0,    0,    0,    0,    0,    0,
    0, 1234,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1235,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1236,    0,    0,    0,    0,    0,
    0, 1237,    0,    0,    0,    0,    0,    0,    0, 1238,
    0,    0,    0,    0,    0,    0,    0,    0, 1236,    0,
    0,    0,    0,    0,    0, 1237, 1239,    0, 1240,    0,
    0,    0,    0, 1238,    0,    0,    0,    0, 1241,    0,
 1242, 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,
 1239,    0, 1240,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1241,    0, 1242, 1243, 1244,    0,    0,    0,
    0, 1245,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1246,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1246,    0,    0, 1827,    0, 1827,    0,    0,
 1827,    0, 1827, 1827,    0,    0, 1827, 1827,    0, 1827,
    0,    0,    0,    0,    0, 1827,    0, 1827, 1827, 1827,
 1827,    0, 1827,    0,    0, 1827,    0,    0, 1827,    0,
    0,    0,    0,    0,    0, 1827,    0, 1827, 1827,    0,
 1827, 1827, 1827, 1827, 1827,    0,    0, 1827,    0,    0,
 1827,    0, 1827, 1827,    0,    0, 1827, 1827, 1827,    0,
 1827, 1827, 1827, 1827, 1827, 1827, 1827,    0,    0,    0,
 1827, 1827, 1827, 1827, 1827,    0,    0,    0, 1827,    0,
    0,    0,    0,    0,    0,    0, 1827, 1827, 1827, 1827,
 1827, 1827,    0,    0,    0, 1827, 1827,    0, 1827,    0,
 1827,    0, 1827, 1827, 1827,    0, 1827,    0, 1827, 1827,
 1827,    0, 1827, 1827, 1827, 1827, 1827,    0, 1827, 1827,
    0,    0,    0,    0, 1827,    0, 1827, 1827,    0,    0,
 1827, 1827, 1827,    0,    0, 1827,    0, 1827,    0,    0,
 1827, 1827, 1827, 1827,    0, 1827,    0, 1827,    0, 1827,
 1827, 1827, 1827, 1827, 1247,    0, 1827, 1827, 1827,    0,
 1827, 1827,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1827, 1827, 1827,    0, 1827, 1827, 1827, 1247, 1827,
    0,    0,    0, 1827,    0, 1827, 1827, 1827, 1827,    0,
 1827,    0, 1827, 1827, 1827, 1827, 1827, 1827, 1827, 1827,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1827,
 1827, 2632, 1827, 1827, 1827,    0,    0,    0,    0, 1827,
 1827, 1827, 1827, 1827,    0, 1827,    0,    0, 1827, 1827,
 1249, 1248, 1250, 1827, 1827, 2633,    0, 1827, 1827, 1827,
 1827, 1827, 1827, 1827, 1827, 1827,    0, 1827,    0,    0,
    0,    0,    0,    0, 1249, 1248, 1250, 1827, 1827, 1827,
 1827, 1827, 1827, 1827, 1827, 1827, 1827, 1827, 1827, 1827,
 1827, 1827, 1827, 1827, 1827, 1827, 1827, 1827,    0, 1827,
 1827, 1827, 1827, 1827, 1827,    0, 1827, 1827, 1827, 1827,
    0, 1827,    0, 1827, 1827, 1827, 1827, 1827, 1827, 1827,
 1827, 1827, 1827, 1827, 1827, 1827, 1827, 1827, 1827, 1827,
 1827,    0, 1827, 1827, 1827, 1827,    0, 1827, 1827, 1827,
 1827, 1827,    0, 1827, 1827, 1827, 1827, 1827, 1827, 1827,
 1827, 1827, 1827, 1827, 1827, 1827, 1827, 1827, 1827, 1827,
 1827, 1827, 1827, 1827, 1827, 1827, 1827, 1827, 1827, 1827,
 1827, 1827, 1828, 1827, 1827, 1827, 1827,    0, 1827, 1827,
 1827, 1827, 1827, 1827,    0,    0, 1827, 1827, 1827, 1827,
 1827, 1827, 1827, 1827, 1827,    0,    0,    0,    0,    0,
 1827,    0, 1827, 1827, 1827, 1827, 1827, 1827,    0, 1827,
    0,    0, 1644, 1828, 1828, 1828, 1828, 1828, 1828, 1828,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1828, 1828, 1828, 1828, 1828,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1233,    0,    0,    0,    0,    0,    0,    0,
    0, 1234,    0, 1828,    0, 1828,    0,    0,    0,    0,
    0,    0,    0,    0, 1235,    0, 1233,    0,    0,    0,
    0,    0,    0,    0,    0, 1234,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1235,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1236,    0,
    0,    0,    0,    0,    0, 1237,    0,    0,    0,    0,
    0,    0,    0, 1238,    0,    0,    0,    0,    0,    0,
    0,    0, 1236,    0,    0,    0,    0,    0,    0, 1237,
 1239,    0, 1240,    0,    0,    0,    0, 1238,    0,    0,
    0,    0, 1241,    0, 1242, 1243, 1244,    0,    0,    0,
    0, 1245,    0,    0, 1239,    0, 1240,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1241,    0, 1242, 1243,
 1244,    0,    0,    0,    0, 1245,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1246,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1246,    0,    0, 1828,
    0, 1828,    0,    0, 1828,    0, 1828, 1828,    0,    0,
 1828, 1828,    0, 1828,    0,    0,    0,    0,    0, 1828,
    0, 1828, 1828, 1828, 1828,    0, 1828,    0,    0, 1828,
    0,    0, 1828,    0,    0,    0,    0,    0,    0, 1828,
    0, 1828, 1828,    0, 1828, 1828, 1828, 1828, 1828,    0,
    0, 1828,    0,    0, 1828,    0, 1828, 1828,    0,    0,
 1828, 1828, 1828,    0, 1828, 1828, 1828, 1828, 1828, 1828,
 1828,    0,    0,    0, 1828, 1828, 1828, 1828, 1828,    0,
    0,    0, 1828,    0,    0,    0,    0,    0,    0,    0,
 1828, 1828, 1828, 1828, 1828, 1828,    0,    0,    0, 1828,
 1828,    0, 1828,    0, 1828,    0, 1828, 1828, 1828,    0,
 1828,    0, 1828, 1828, 1828,    0, 1828, 1828, 1828, 1828,
 1828,    0, 1828, 1828,    0,    0,    0,    0, 1828,    0,
 1828, 1828,    0,    0, 1828, 1828, 1828,    0,    0, 1828,
    0, 1828,    0,    0, 1828, 1828, 1828, 1828,    0, 1828,
    0, 1828,    0, 1828, 1828, 1828, 1828, 1828, 1247,    0,
 1828, 1828, 1828,    0, 1828, 1828,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1828, 1828, 1828,    0, 1828,
 1828, 1828, 1247, 1828,    0,    0,    0, 1828,    0, 1828,
 1828, 1828, 1828,    0, 1828,    0, 1828, 1828, 1828, 1828,
 1828, 1828, 1828, 1828,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1828, 1828, 2634, 1828, 1828, 1828,    0,
    0,    0,    0, 1828, 1828, 1828, 1828, 1828,    0, 1828,
    0,    0, 1828, 1828, 1249, 1248, 1250, 1828, 1828, 2635,
    0, 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828,
    0, 1828,    0,    0,    0,    0,    0,    0, 1249, 1248,
 1250, 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828,
 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828,
 1828, 1828,    0, 1828, 1828, 1828, 1828, 1828, 1828,    0,
 1828, 1828, 1828, 1828,    0, 1828,    0, 1828, 1828, 1828,
 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828,
 1828, 1828, 1828, 1828, 1828,    0, 1828, 1828, 1828, 1828,
    0, 1828, 1828, 1828, 1828, 1828,    0, 1828, 1828, 1828,
 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828,
 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828,
 1828, 1828, 1828, 1828, 1828, 1828, 1829, 1828, 1828, 1828,
 1828,    0, 1828, 1828, 1828, 1828, 1828, 1828,    0,    0,
 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828, 1828,    0,
    0,    0,    0,    0, 1828,    0, 1828, 1828, 1828, 1828,
 1828, 1828,    0, 1828,    0,    0, 1645, 1829, 1829, 1829,
 1829, 1829, 1829, 1829,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1829, 1829, 1829, 1829, 1829,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1233,    0,    0,    0,
    0,    0,    0,    0,    0, 1234,    0, 1829,    0, 1829,
    0,    0,    0,    0,    0,    0,    0,    0, 1235,    0,
 1233,    0,    0,    0,    0,    0,    0,    0,    0, 1234,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1235,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1236,    0,    0,    0,    0,    0,    0, 1237,
    0,    0,    0,    0,    0,    0,    0, 1238,    0,    0,
    0,    0,    0,    0,    0,    0, 1236,    0,    0,    0,
    0,    0,    0, 1237, 1239,    0, 1240,    0,    0,    0,
    0, 1238,    0,    0,    0,    0, 1241,    0, 1242, 1243,
 1244,    0,    0,    0,    0, 1245,    0,    0, 1239,    0,
 1240,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1241,    0, 1242, 1243, 1244,    0,    0,    0,    0, 1245,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1246,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1246,    0,    0, 1829,    0, 1829,    0,    0, 1829,    0,
 1829, 1829,    0,    0, 1829, 1829,    0, 1829,    0,    0,
    0,    0,    0, 1829,    0, 1829, 1829, 1829, 1829,    0,
 1829,    0,    0, 1829,    0,    0, 1829,    0,    0,    0,
    0,    0,    0, 1829,    0, 1829, 1829,    0, 1829, 1829,
 1829, 1829, 1829,    0,    0, 1829,    0,    0, 1829,    0,
 1829, 1829,    0,    0, 1829, 1829, 1829,    0, 1829, 1829,
 1829, 1829, 1829, 1829, 1829,    0,    0,    0, 1829, 1829,
 1829, 1829, 1829,    0,    0,    0, 1829,    0,    0,    0,
    0,    0,    0,    0, 1829, 1829, 1829, 1829, 1829, 1829,
    0,    0,    0, 1829, 1829,    0, 1829,    0, 1829,    0,
 1829, 1829, 1829,    0, 1829,    0, 1829, 1829, 1829,    0,
 1829, 1829, 1829, 1829, 1829,    0, 1829, 1829,    0,    0,
    0,    0, 1829,    0, 1829, 1829,    0,    0, 1829, 1829,
 1829,    0,    0, 1829,    0, 1829,    0,    0, 1829, 1829,
 1829, 1829,    0, 1829,    0, 1829,    0, 1829, 1829, 1829,
 1829, 1829, 1247,    0, 1829, 1829, 1829,    0, 1829, 1829,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1829,
 1829, 1829,    0, 1829, 1829, 1829, 1247, 1829,    0,    0,
    0, 1829,    0, 1829, 1829, 1829, 1829,    0, 1829,    0,
 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1829, 1829, 2636,
 1829, 1829, 1829,    0,    0,    0,    0, 1829, 1829, 1829,
 1829, 1829,    0, 1829,    0,    0, 1829, 1829, 1249, 1248,
 1250, 1829, 1829, 2637,    0, 1829, 1829, 1829, 1829, 1829,
 1829, 1829, 1829, 1829,    0, 1829,    0,    0,    0,    0,
    0,    0, 1249, 1248, 1250, 1829, 1829, 1829, 1829, 1829,
 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829,
 1829, 1829, 1829, 1829, 1829, 1829,    0, 1829, 1829, 1829,
 1829, 1829, 1829,    0, 1829, 1829, 1829, 1829,    0, 1829,
    0, 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829,
 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829,    0,
 1829, 1829, 1829, 1829,    0, 1829, 1829, 1829, 1829, 1829,
    0, 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829,
 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829,
 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829, 1829,
 1833, 1829, 1829, 1829, 1829,    0, 1829, 1829, 1829, 1829,
 1829, 1829,    0,    0, 1829, 1829, 1829, 1829, 1829, 1829,
 1829, 1829, 1829,    0,    0,    0,    0,    0, 1829,    0,
 1829, 1829, 1829, 1829, 1829, 1829,    0, 1829,    0,    0,
 1646, 1833, 1833, 1833, 1833, 1833, 1833, 1833,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1833, 1833,
 1833, 1833, 1833,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1233,    0,    0,    0,    0,    0,    0,    0,    0, 1234,
    0, 1833,    0, 1833,    0,    0,    0,    0,    0,    0,
    0,    0, 1235,    0, 1233,    0,    0,    0,    0,    0,
    0,    0,    0, 1234,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1235,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1236,    0,    0,    0,
    0,    0,    0, 1237,    0,    0,    0,    0,    0,    0,
    0, 1238,    0,    0,    0,    0,    0,    0,    0,    0,
 1236,    0,    0,    0,    0,    0,    0, 1237, 1239,    0,
 1240,    0,    0,    0,    0, 1238,    0,    0,    0,    0,
 1241,    0, 1242, 1243, 1244,    0,    0,    0,    0, 1245,
    0,    0, 1239,    0, 1240,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1241,    0, 1242, 1243, 1244,    0,
    0,    0,    0, 1245,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1246,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1246,    0,    0, 1833,    0, 1833,
    0,    0, 1833,    0, 1833, 1833,    0,    0, 1833, 1833,
    0, 1833,    0,    0,    0,    0,    0, 1833,    0, 1833,
 1833, 1833, 1833,    0, 1833,    0,    0, 1833,    0,    0,
 1833,    0,    0,    0,    0,    0,    0, 1833,    0, 1833,
 1833,    0, 1833, 1833, 1833, 1833, 1833,    0,    0, 1833,
    0,    0, 1833,    0, 1833, 1833,    0,    0, 1833, 1833,
 1833,    0, 1833, 1833, 1833, 1833, 1833, 1833, 1833,    0,
    0,    0, 1833, 1833, 1833, 1833, 1833,    0,    0,    0,
 1833,    0,    0,    0,    0,    0,    0,    0, 1833, 1833,
 1833, 1833, 1833, 1833,    0,    0,    0, 1833, 1833,    0,
 1833,    0, 1833,    0, 1833, 1833, 1833,    0, 1833,    0,
 1833, 1833, 1833,    0, 1833, 1833, 1833, 1833, 1833,    0,
 1833, 1833,    0,    0,    0,    0, 1833,    0, 1833, 1833,
    0,    0, 1833, 1833, 1833,    0,    0, 1833,    0, 1833,
    0,    0, 1833, 1833, 1833, 1833,    0, 1833,    0, 1833,
    0, 1833, 1833, 1833, 1833, 1833, 1247,    0, 1833, 1833,
 1833,    0, 1833, 1833,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1833, 1833, 1833,    0, 1833, 1833, 1833,
 1247, 1833,    0,    0,    0, 1833,    0, 1833, 1833, 1833,
 1833,    0, 1833,    0, 1833, 1833, 1833, 1833, 1833, 1833,
 1833, 1833,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1833, 1833, 2851, 1833, 1833, 1833,    0,    0,    0,
    0, 1833, 1833, 1833, 1833, 1833,    0, 1833,    0,    0,
 1833, 1833, 1249, 1248, 1250, 1833, 1833, 2852,    0, 1833,
 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833,    0, 1833,
    0,    0,    0,    0,    0,    0, 1249, 1248, 1250, 1833,
 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833,
 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833,
    0, 1833, 1833, 1833, 1833, 1833, 1833,    0, 1833, 1833,
 1833, 1833,    0, 1833,    0, 1833, 1833, 1833, 1833, 1833,
 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833,
 1833, 1833, 1833,    0, 1833, 1833, 1833, 1833,    0, 1833,
 1833, 1833, 1833, 1833,    0, 1833, 1833, 1833, 1833, 1833,
 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833,
 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833, 1833,
 1833, 1833, 1833, 1833, 1834, 1833, 1833, 1833, 1833,    0,
 1833, 1833, 1833, 1833, 1833, 1833,    0,    0, 1833, 1833,
 1833, 1833, 1833, 1833, 1833, 1833, 1833,    0,    0,    0,
    0,    0, 1833,    0, 1833, 1833, 1833, 1833, 1833, 1833,
    0, 1833,    0,    0, 1647, 1834, 1834, 1834, 1834, 1834,
 1834, 1834,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1834, 1834, 1834, 1834, 1834,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1233,    0,    0,    0,    0,    0,
    0,    0,    0, 1234,    0, 1834,    0, 1834,    0,    0,
    0,    0,    0,    0,    0,    0, 1235,    0, 1233,    0,
    0,    0,    0,    0,    0,    0,    0, 1234,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1235,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1236,    0,    0,    0,    0,    0,    0, 1237,    0,    0,
    0,    0,    0,    0,    0, 1238,    0,    0,    0,    0,
    0,    0,    0,    0, 1236,    0,    0,    0,    0,    0,
    0, 1237, 1239,    0, 1240,    0,    0,    0,    0, 1238,
    0,    0,    0,    0, 1241,    0, 1242, 1243, 1244,    0,
    0,    0,    0, 1245,    0,    0, 1239,    0, 1240,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1241,    0,
 1242, 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1246,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1246,    0,
    0, 1834,    0, 1834,    0,    0, 1834,    0, 1834, 1834,
    0,    0, 1834, 1834,    0, 1834,    0,    0,    0,    0,
    0, 1834,    0, 1834, 1834, 1834, 1834,    0, 1834,    0,
    0, 1834,    0,    0, 1834,    0,    0,    0,    0,    0,
    0, 1834,    0, 1834, 1834,    0, 1834, 1834, 1834, 1834,
 1834,    0,    0, 1834,    0,    0, 1834,    0, 1834, 1834,
    0,    0, 1834, 1834, 1834,    0, 1834, 1834, 1834, 1834,
 1834, 1834, 1834,    0,    0,    0, 1834, 1834, 1834, 1834,
 1834,    0,    0,    0, 1834,    0,    0,    0,    0,    0,
    0,    0, 1834, 1834, 1834, 1834, 1834, 1834,    0,    0,
    0, 1834, 1834,    0, 1834,    0, 1834,    0, 1834, 1834,
 1834,    0, 1834,    0, 1834, 1834, 1834,    0, 1834, 1834,
 1834, 1834, 1834,    0, 1834, 1834,    0,    0,    0,    0,
 1834,    0, 1834, 1834,    0,    0, 1834, 1834, 1834,    0,
    0, 1834,    0, 1834,    0,    0, 1834, 1834, 1834, 1834,
    0, 1834,    0, 1834,    0, 1834, 1834, 1834, 1834, 1834,
 1247,    0, 1834, 1834, 1834,    0, 1834, 1834,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1834, 1834, 1834,
    0, 1834, 1834, 1834, 1247, 1834,    0,    0,    0, 1834,
    0, 1834, 1834, 1834, 1834,    0, 1834,    0, 1834, 1834,
 1834, 1834, 1834, 1834, 1834, 1834,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1834, 1834, 3011, 1834, 1834,
 1834,    0,    0,    0,    0, 1834, 1834, 1834, 1834, 1834,
    0, 1834,    0,    0, 1834, 1834, 1249, 1248, 1250, 1834,
 1834, 3016,    0, 1834, 1834, 1834, 1834, 1834, 1834, 1834,
 1834, 1834,    0, 1834,    0,    0,    0,    0,    0,    0,
 1249, 1248, 1250, 1834, 1834, 1834, 1834, 1834, 1834, 1834,
 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1834,
 1834, 1834, 1834, 1834,    0, 1834, 1834, 1834, 1834, 1834,
 1834,    0, 1834, 1834, 1834, 1834,    0, 1834,    0, 1834,
 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1834,
 1834, 1834, 1834, 1834, 1834, 1834, 1834,    0, 1834, 1834,
 1834, 1834,    0, 1834, 1834, 1834, 1834, 1834,    0, 1834,
 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1834,
 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1834,
 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1835, 1834,
 1834, 1834, 1834,    0, 1834, 1834, 1834, 1834, 1834, 1834,
    0,    0, 1834, 1834, 1834, 1834, 1834, 1834, 1834, 1834,
 1834,    0,    0,    0,    0,    0, 1834,    0, 1834, 1834,
 1834, 1834, 1834, 1834,    0, 1834,    0,    0, 1648, 1835,
 1835, 1835, 1835, 1835, 1835, 1835,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1835, 1835, 1835, 1835,
 1835,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1233,    0,
    0,    0,    0,    0,    0,    0,    0, 1234,    0, 1835,
    0, 1835,    0,    0,    0,    0,    0,    0,    0,    0,
 1235,    0, 1233,    0,    0,    0,    0,    0,    0,    0,
    0, 1234,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1235,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1236,    0,    0,    0,    0,    0,
    0, 1237,    0,    0,    0,    0,    0,    0,    0, 1238,
    0,    0,    0,    0,    0,    0,    0,    0, 1236,    0,
    0,    0,    0,    0,    0, 1237, 1239,    0, 1240,    0,
    0,    0,    0, 1238,    0,    0,    0,    0, 1241,    0,
 1242, 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,
 1239,    0, 1240,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1241,    0, 1242, 1243, 1244,    0,    0,    0,
    0, 1245,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1246,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1246,    0,    0, 1835,    0, 1835,    0,    0,
 1835,    0, 1835, 1835,    0,    0, 1835, 1835,    0, 1835,
    0,    0,    0,    0,    0, 1835,    0, 1835, 1835, 1835,
 1835,    0, 1835,    0,    0, 1835,    0,    0, 1835,    0,
    0,    0,    0,    0,    0, 1835,    0, 1835, 1835,    0,
 1835, 1835, 1835, 1835, 1835,    0,    0, 1835,    0,    0,
 1835,    0, 1835, 1835,    0,    0, 1835, 1835, 1835,    0,
 1835, 1835, 1835, 1835, 1835, 1835, 1835,    0,    0,    0,
 1835, 1835, 1835, 1835, 1835,    0,    0,    0, 1835,    0,
    0,    0,    0,    0,    0,    0, 1835, 1835, 1835, 1835,
 1835, 1835,    0,    0,    0, 1835, 1835,    0, 1835,    0,
 1835,    0, 1835, 1835, 1835,    0, 1835,    0, 1835, 1835,
 1835,    0, 1835, 1835, 1835, 1835, 1835,    0, 1835, 1835,
    0,    0,    0,    0, 1835,    0, 1835, 1835,    0,    0,
 1835, 1835, 1835,    0,    0, 1835,    0, 1835,    0,    0,
 1835, 1835, 1835, 1835,    0, 1835,    0, 1835,    0, 1835,
 1835, 1835, 1835, 1835, 1247,    0, 1835, 1835, 1835,    0,
 1835, 1835,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1835, 1835, 1835,    0, 1835, 1835, 1835, 1247, 1835,
    0,    0,    0, 1835,    0, 1835, 1835, 1835, 1835,    0,
 1835,    0, 1835, 1835, 1835, 1835, 1835, 1835, 1835, 1835,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1835,
 1835, 3017, 1835, 1835, 1835,    0,    0,    0,    0, 1835,
 1835, 1835, 1835, 1835,    0, 1835,    0,    0, 1835, 1835,
 1249, 1248, 1250, 1835, 1835, 3018,    0, 1835, 1835, 1835,
 1835, 1835, 1835, 1835, 1835, 1835,    0, 1835,    0,    0,
    0,    0,    0,    0, 1249, 1248, 1250, 1835, 1835, 1835,
 1835, 1835, 1835, 1835, 1835, 1835, 1835, 1835, 1835, 1835,
 1835, 1835, 1835, 1835, 1835, 1835, 1835, 1835,    0, 1835,
 1835, 1835, 1835, 1835, 1835,    0, 1835, 1835, 1835, 1835,
    0, 1835,    0, 1835, 1835, 1835, 1835, 1835, 1835, 1835,
 1835, 1835, 1835, 1835, 1835, 1835, 1835, 1835, 1835, 1835,
 1835,    0, 1835, 1835, 1835, 1835,    0, 1835, 1835, 1835,
 1835, 1835,    0, 1835, 1835, 1835, 1835, 1835, 1835, 1835,
 1835, 1835, 1835, 1835, 1835, 1835, 1835, 1835, 1835, 1835,
 1835, 1835, 1835, 1835, 1835, 1835, 1835, 1835, 1835, 1835,
 1835, 1835, 1837, 1835, 1835, 1835, 1835,    0, 1835, 1835,
 1835, 1835, 1835, 1835,    0,    0, 1835, 1835, 1835, 1835,
 1835, 1835, 1835, 1835, 1835,    0,    0,    0,    0,    0,
 1835,    0, 1835, 1835, 1835, 1835, 1835, 1835,    0, 1835,
    0,    0, 1649, 1837, 1837, 1837, 1837, 1837, 1837, 1837,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1837, 1837, 1837, 1837, 1837,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1233,    0,    0,    0,    0,    0,    0,    0,
    0, 1234,    0, 1837,    0, 1837,    0,    0,    0,    0,
    0,    0,    0,    0, 1235,    0, 1233,    0,    0,    0,
    0,    0,    0,    0,    0, 1234,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1235,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1236,    0,
    0,    0,    0,    0,    0, 1237,    0,    0,    0,    0,
    0,    0,    0, 1238,    0,    0,    0,    0,    0,    0,
    0,    0, 1236,    0,    0,    0,    0,    0,    0, 1237,
 1239,    0, 1240,    0,    0,    0,    0, 1238,    0,    0,
    0,    0, 1241,    0, 1242, 1243, 1244,    0,    0,    0,
    0, 1245,    0,    0, 1239,    0, 1240,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1241,    0, 1242, 1243,
 1244,    0,    0,    0,    0, 1245,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1246,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1246,    0,    0, 1837,
    0, 1837,    0,    0, 1837,    0, 1837, 1837,    0,    0,
 1837, 1837,    0, 1837,    0,    0,    0,    0,    0, 1837,
    0, 1837, 1837, 1837, 1837,    0, 1837,    0,    0, 1837,
    0,    0, 1837,    0,    0,    0,    0,    0,    0, 1837,
    0, 1837, 1837,    0, 1837, 1837, 1837, 1837, 1837,    0,
    0, 1837,    0,    0, 1837,    0, 1837, 1837,    0,    0,
 1837, 1837, 1837,    0, 1837, 1837, 1837, 1837, 1837, 1837,
 1837,    0,    0,    0, 1837, 1837, 1837, 1837, 1837,    0,
    0,    0, 1837,    0,    0,    0,    0,    0,    0,    0,
 1837, 1837, 1837, 1837, 1837, 1837,    0,    0,    0, 1837,
 1837,    0, 1837,    0, 1837,    0, 1837, 1837, 1837,    0,
 1837,    0, 1837, 1837, 1837,    0, 1837, 1837, 1837, 1837,
 1837,    0, 1837, 1837,    0,    0,    0,    0, 1837,    0,
 1837, 1837,    0,    0, 1837, 1837, 1837,    0,    0, 1837,
    0, 1837,    0,    0, 1837, 1837, 1837, 1837,    0, 1837,
    0, 1837,    0, 1837, 1837, 1837, 1837, 1837, 1247,    0,
 1837, 1837, 1837,    0, 1837, 1837,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1837, 1837, 1837,    0, 1837,
 1837, 1837, 1247, 1837,    0,    0,    0, 1837,    0, 1837,
 1837, 1837, 1837,    0, 1837,    0, 1837, 1837, 1837, 1837,
 1837, 1837, 1837, 1837,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1837, 1837, 3019, 1837, 1837, 1837,    0,
    0,    0,    0, 1837, 1837, 1837, 1837, 1837,    0, 1837,
    0,    0, 1837, 1837, 1249, 1248, 1250, 1837, 1837, 3026,
    0, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837,
    0, 1837,    0,    0,    0,    0,    0,    0, 1249, 1248,
 1250, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837,
 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837,
 1837, 1837,    0, 1837, 1837, 1837, 1837, 1837, 1837,    0,
 1837, 1837, 1837, 1837,    0, 1837,    0, 1837, 1837, 1837,
 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837,
 1837, 1837, 1837, 1837, 1837,    0, 1837, 1837, 1837, 1837,
    0, 1837, 1837, 1837, 1837, 1837,    0, 1837, 1837, 1837,
 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837,
 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837,
 1837, 1837, 1837, 1837, 1837, 1837, 1839, 1837, 1837, 1837,
 1837,    0, 1837, 1837, 1837, 1837, 1837, 1837,    0,    0,
 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837,    0,
    0,    0,    0,    0, 1837,    0, 1837, 1837, 1837, 1837,
 1837, 1837,    0, 1837,    0,    0, 1651, 1839, 1839, 1839,
 1839, 1839, 1839, 1839,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1839, 1839, 1839, 1839, 1839,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1233,    0,    0,    0,
    0,    0,    0,    0,    0, 1234,    0, 1839,    0, 1839,
    0,    0,    0,    0,    0,    0,    0,    0, 1235,    0,
 1233,    0,    0,    0,    0,    0,    0,    0,    0, 1234,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1235,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1236,    0,    0,    0,    0,    0,    0, 1237,
    0,    0,    0,    0,    0,    0,    0, 1238,    0,    0,
    0,    0,    0,    0,    0,    0, 1236,    0,    0,    0,
    0,    0,    0, 1237, 1239,    0, 1240,    0,    0,    0,
    0, 1238,    0,    0,    0,    0, 1241,    0, 1242, 1243,
 1244,    0,    0,    0,    0, 1245,    0,    0, 1239,    0,
 1240,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1241,    0, 1242, 1243, 1244,    0,    0,    0,    0, 1245,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1246,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1246,    0,    0, 1839,    0, 1839,    0,    0, 1839,    0,
 1839, 1839,    0,    0, 1839, 1839,    0, 1839,    0,    0,
    0,    0,    0, 1839,    0, 1839, 1839, 1839, 1839,    0,
 1839,    0,    0, 1839,    0,    0, 1839,    0,    0,    0,
    0,    0,    0, 1839,    0, 1839, 1839,    0, 1839, 1839,
 1839, 1839, 1839,    0,    0, 1839,    0,    0, 1839,    0,
 1839, 1839,    0,    0, 1839, 1839, 1839,    0, 1839, 1839,
 1839, 1839, 1839, 1839, 1839,    0,    0,    0, 1839, 1839,
 1839, 1839, 1839,    0,    0,    0, 1839,    0,    0,    0,
    0,    0,    0,    0, 1839, 1839, 1839, 1839, 1839, 1839,
    0,    0,    0, 1839, 1839,    0, 1839,    0, 1839,    0,
 1839, 1839, 1839,    0, 1839,    0, 1839, 1839, 1839,    0,
 1839, 1839, 1839, 1839, 1839,    0, 1839, 1839,    0,    0,
    0,    0, 1839,    0, 1839, 1839,    0,    0, 1839, 1839,
 1839,    0,    0, 1839,    0, 1839,    0,    0, 1839, 1839,
 1839, 1839,    0, 1839,    0, 1839,    0, 1839, 1839, 1839,
 1839, 1839, 1247,    0, 1839, 1839, 1839,    0, 1839, 1839,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1839,
 1839, 1839,    0, 1839, 1839, 1839, 1247, 1839,    0,    0,
    0, 1839,    0, 1839, 1839, 1839, 1839,    0, 1839,    0,
 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1839, 1839, 3027,
 1839, 1839, 1839,    0,    0,    0,    0, 1839, 1839, 1839,
 1839, 1839,    0, 1839,    0,    0, 1839, 1839, 1249, 1248,
 1250, 1839, 1839, 3105,    0, 1839, 1839, 1839, 1839, 1839,
 1839, 1839, 1839, 1839,    0, 1839,    0,    0,    0,    0,
    0,    0, 1249, 1248, 1250, 1839, 1839, 1839, 1839, 1839,
 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839,
 1839, 1839, 1839, 1839, 1839, 1839,    0, 1839, 1839, 1839,
 1839, 1839, 1839,    0, 1839, 1839, 1839, 1839,    0, 1839,
    0, 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839,
 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839,    0,
 1839, 1839, 1839, 1839,    0, 1839, 1839, 1839, 1839, 1839,
    0, 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839,
 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839,
 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839, 1839,
 1840, 1839, 1839, 1839, 1839,    0, 1839, 1839, 1839, 1839,
 1839, 1839,    0,    0, 1839, 1839, 1839, 1839, 1839, 1839,
 1839, 1839, 1839,    0,    0,    0,    0,    0, 1839,    0,
 1839, 1839, 1839, 1839, 1839, 1839,    0, 1839,    0,    0,
 1652, 1840, 1840, 1840, 1840, 1840, 1840, 1840,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1840, 1840,
 1840, 1840, 1840,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1233,    0,    0,    0,    0,    0,    0,    0,    0, 1234,
    0, 1840,    0, 1840,    0,    0,    0,    0,    0,    0,
    0,    0, 1235,    0, 1233,    0,    0,    0,    0,    0,
    0,    0,    0, 1234,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1235,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1236,    0,    0,    0,
    0,    0,    0, 1237,    0,    0,    0,    0,    0,    0,
    0, 1238,    0,    0,    0,    0,    0,    0,    0,    0,
 1236,    0,    0,    0,    0,    0,    0, 1237, 1239,    0,
 1240,    0,    0,    0,    0, 1238,    0,    0,    0,    0,
 1241,    0, 1242, 1243, 1244,    0,    0,    0,    0, 1245,
    0,    0, 1239,    0, 1240,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1241,    0, 1242, 1243, 1244,    0,
    0,    0,    0, 1245,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1246,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1246,    0,    0, 1840,    0, 1840,
    0,    0, 1840,    0, 1840, 1840,    0,    0, 1840, 1840,
    0, 1840,    0,    0,    0,    0,    0, 1840,    0, 1840,
 1840, 1840, 1840,    0, 1840,    0,    0, 1840,    0,    0,
 1840,    0,    0,    0,    0,    0,    0, 1840,    0, 1840,
 1840,    0, 1840, 1840, 1840, 1840, 1840,    0,    0, 1840,
    0,    0, 1840,    0, 1840, 1840,    0,    0, 1840, 1840,
 1840,    0, 1840, 1840, 1840, 1840, 1840, 1840, 1840,    0,
    0,    0, 1840, 1840, 1840, 1840, 1840,    0,    0,    0,
 1840,    0,    0,    0,    0,    0,    0,    0, 1840, 1840,
 1840, 1840, 1840, 1840,    0,    0,    0, 1840, 1840,    0,
 1840,    0, 1840,    0, 1840, 1840, 1840,    0, 1840,    0,
 1840, 1840, 1840,    0, 1840, 1840, 1840, 1840, 1840,    0,
 1840, 1840,    0,    0,    0,    0, 1840,    0, 1840, 1840,
    0,    0, 1840, 1840, 1840,    0,    0, 1840,    0, 1840,
    0,    0, 1840, 1840, 1840, 1840,    0, 1840,    0, 1840,
    0, 1840, 1840, 1840, 1840, 1840, 1247,    0, 1840, 1840,
 1840,    0, 1840, 1840,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1840, 1840, 1840,    0, 1840, 1840, 1840,
 1247, 1840,    0,    0,    0, 1840,    0, 1840, 1840, 1840,
 1840,    0, 1840,    0, 1840, 1840, 1840, 1840, 1840, 1840,
 1840, 1840,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1840, 1840, 3180, 1840, 1840, 1840,    0,    0,    0,
    0, 1840, 1840, 1840, 1840, 1840,    0, 1840,    0,    0,
 1840, 1840, 1249, 1248, 1250, 1840, 1840, 3223,    0, 1840,
 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840,    0, 1840,
    0,    0,    0,    0,    0,    0, 1249, 1248, 1250, 1840,
 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840,
 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840,
    0, 1840, 1840, 1840, 1840, 1840, 1840,    0, 1840, 1840,
 1840, 1840,    0, 1840,    0, 1840, 1840, 1840, 1840, 1840,
 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840,
 1840, 1840, 1840,    0, 1840, 1840, 1840, 1840,    0, 1840,
 1840, 1840, 1840, 1840,    0, 1840, 1840, 1840, 1840, 1840,
 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840,
 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840, 1840,
 1840, 1840, 1840, 1840, 1841, 1840, 1840, 1840, 1840,    0,
 1840, 1840, 1840, 1840, 1840, 1840,    0,    0, 1840, 1840,
 1840, 1840, 1840, 1840, 1840, 1840, 1840,    0,    0,    0,
    0,    0, 1840,    0, 1840, 1840, 1840, 1840, 1840, 1840,
    0, 1840,    0,    0, 1653, 1841, 1841, 1841, 1841, 1841,
 1841, 1841,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1841, 1841, 1841, 1841, 1841,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1233,    0,    0,    0,    0,    0,
    0,    0,    0, 1234,    0, 1841,    0, 1841,    0,    0,
    0,    0,    0,    0,    0,    0, 1235,    0, 1233,    0,
    0,    0,    0,    0,    0,    0,    0, 1234,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1235,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1236,    0,    0,    0,    0,    0,    0, 1237,    0,    0,
    0,    0,    0,    0,    0, 1238, 1060,    0,    0, 1061,
    0,    0,    0,    0, 1236,    0,    0,    0,    0,    0,
    0, 1237, 1239,    0, 1240, 1062,    0, 1063,    0, 1238,
    0,    0,    0,    0, 1241,    0, 1242, 1243, 1244,    0,
    0,    0,    0, 1245,    0,    0, 1239,    0, 1240,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1241,    0,
 1242, 1243, 1244,    0,    0, 1064,    0, 1245,    0,    0,
    0, 1065, 1066, 1067,    0, 1068, 1069, 1070,    0, 1071,
    0,    0,    0,    0, 1246,    0,    0,    0,    0,    0,
    0,    0,    0, 1072,    0, 1073,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1246,    0,
    0, 1841,    0, 1841,    0,    0, 1841,    0, 1841, 1841,
    0,    0, 1841, 1841, 1074, 1841,    0,    0,    0,    0,
    0, 1841,    0, 1841, 1841, 1841, 1841,    0, 1841,    0,
 1075, 1841,    0,    0, 1841,    0,    0,    0,    0,    0,
    0, 1841,    0, 1841, 1841,    0, 1841, 1841, 1841, 1841,
 1841,    0, 1076, 1841,    0,    0, 1841,    0, 1841, 1841,
    0,    0, 1841, 1841, 1841,    0, 1841, 1841, 1841, 1841,
 1841, 1841, 1841,    0,    0,    0, 1841, 1841, 1841, 1841,
 1841,    0,    0,    0, 1841,    0, 1077, 1078,    0,    0,
    0,    0, 1841, 1841, 1841, 1841, 1841, 1841,    0,    0,
    0, 1841, 1841, 1079, 1841,    0, 1841,    0, 1841, 1841,
 1841,    0, 1841,    0, 1841, 1841, 1841,    0, 1841, 1841,
 1841, 1841, 1841,    0, 1841, 1841,    0,    0,    0,    0,
 1841,    0, 1841, 1841,    0,    0, 1841, 1841, 1841,    0,
    0, 1841,    0, 1841,    0,    0, 1841, 1841, 1841, 1841,
    0, 1841,    0, 1841,    0, 1841, 1841, 1841, 1841, 1841,
 1247,    0, 1841, 1841, 1841,    0, 1841, 1841,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1841, 1841, 1841,
    0, 1841, 1841, 1841, 1247, 1841,    0,    0,    0, 1841,
    0, 1841, 1841, 1841, 1841,    0, 1841,    0, 1841, 1841,
 1841, 1841, 1841, 1841, 1841, 1841,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1841, 1841, 3233, 1841, 1841,
 1841,    0,    0,    0,    0, 1841, 1841, 1841, 1841, 1841,
    0, 1841,    0,    0, 1841, 1841, 1249, 1248, 1250, 1841,
 1841,    0,    0, 1841, 1841, 1841, 1841, 1841, 1841, 1841,
 1841, 1841,    0, 1841,    0,    0, 1249, 1248, 1250,    0,
    0,    0, 1080, 1841, 1841, 1841, 1841, 1841, 1841, 1841,
 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1841,
 1841, 1841, 1841, 1841,    0, 1841, 1841, 1841, 1841, 1841,
 1841,    0, 1841, 1841, 1841, 1841,    0, 1841,    0, 1841,
 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1841,
 1841, 1841, 1841, 1841, 1841, 1841, 1841,    0, 1841, 1841,
 1841, 1841,    0, 1841, 1841, 1841, 1841, 1841,    0, 1841,
 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1841,
 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1841,
 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1842, 1841,
 1841, 1841, 1841,    0, 1841, 1841, 1841, 1841, 1841, 1841,
    0,    0, 1841, 1841, 1841, 1841, 1841, 1841, 1841, 1841,
 1841,    0,    0,    0,    0,    0, 1841,    0, 1841, 1841,
 1841, 1841, 1841, 1841,    0, 1841,    0,    0, 1654, 1842,
 1842, 1842, 1842, 1842, 1842, 1842,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1842, 1842, 1842, 1842,
 1842,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1233,    0,
    0,    0,    0, 1870,    0,    0,    0, 1234,    0, 1842,
    0, 1842,    0,    0,    0,    0,    0,    0, 1233,    0,
 1235,  546,    0,    0,    0,    0,    0, 1234, 1871,    0,
    0,    0,    0,    0, 1872,    0,    0,    0,    0,    0,
 1235,    0,    0,    0, 1873,    0,    0,    0,    0,    0,
    0,    0,    0,  547, 1236,    0,    0, 1874,    0,    0,
    0, 1237,    0,    0,    0,    0,    0,    0,    0, 1238,
    0,    0,    0,    0, 1236,    0,    0,    0,    0,    0,
    0, 1237,    0,    0,    0,    0, 1239,    0, 1240, 1238,
    0,    0,    0,    0,    0,    0,    0,    0, 1241,    0,
 1242, 1243, 1244,    0,    0,    0, 1239, 1245, 1240,    0,
    0,    0,    0,    0,    0, 1875,    0,    0, 1241,    0,
 1242, 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1876,    0,    0, 1246, 1877,
    0,    0,    0,    0, 1878,    0,    0,    0,    0,    0,
 1879,    0,    0,    0,    0,    0,    0,    0, 1246,    0,
    0,    0,    0,    0,    0, 1842,    0, 1842, 1880,    0,
 1842,    0, 1842, 1842,    0,    0, 1842, 1842,    0, 1842,
    0,    0,    0,    0,    0, 1842,    0, 1842, 1842, 1842,
 1842,    0, 1842,    0,    0, 1842,    0,    0, 1842,    0,
    0,    0,    0,    0,    0, 1842,    0, 1842, 1842,    0,
 1842, 1842, 1842, 1842, 1842,    0,    0, 1842,    0,    0,
 1842,    0, 1842, 1842, 1881,    0, 1842, 1842, 1842,    0,
 1842, 1842, 1842, 1842, 1842, 1842, 1842,    0,    0,    0,
 1842, 1842, 1842, 1842, 1842,    0,    0,    0, 1842,    0,
    0,    0,    0,    0,    0,    0, 1842, 1842, 1842, 1842,
 1842, 1842,    0,    0,    0, 1842, 1842,    0, 1842,    0,
 1842,    0, 1842, 1842, 1842,    0, 1842,    0, 1842, 1842,
 1842,    0, 1842, 1842, 1842, 1842, 1842,    0, 1842, 1842,
    0,    0,    0,    0, 1842,    0, 1842, 1842,    0,    0,
 1842, 1842, 1842,    0,    0, 1842,    0, 1842,    0,    0,
 1842, 1842, 1842, 1842, 1882, 1842,    0, 1842,    0, 1842,
 1842, 1842, 1842, 1842, 1247,    0, 1842, 1842, 1842,    0,
 1842, 1842,    0, 1883,    0,    0,    0,    0,    0,    0,
    0, 1842, 1842, 1842, 1247, 1842, 1842, 1842,    0, 1842,
    0,    0,    0, 1842,    0, 1842, 1842, 1842, 1842, 1965,
 1842,    0, 1842, 1842, 1842, 1842, 1842, 1842, 1842, 1842,
    0,    0,    0,    0,    0, 1249, 1248, 1250,    0, 1842,
 1842,    0, 1842, 1842, 1842,    0,    0,    0,    0, 1842,
 1842, 1842, 1842, 1842,    0, 1842,    0,    0, 1842, 1842,
    0,    0,    0, 1842, 1842,    0,    0, 1842, 1842, 1842,
 1842, 1842, 1842, 1842, 1842, 1842,    0, 1842,    0,    0,
 1249, 1248, 1250,    0,    0,    0,    0, 1842, 1842, 1842,
 1842, 1842, 1842, 1842, 1842, 1842, 1842, 1842, 1842, 1842,
 1842, 1842, 1842, 1842, 1842, 1842, 1842, 1842,    0, 1842,
 1842, 1842, 1842, 1842, 1842,    0, 1842, 1842, 1842, 1842,
    0, 1842,    0, 1842, 1842, 1842, 1842, 1842, 1842, 1842,
 1842, 1842, 1842, 1842, 1842, 1842, 1842, 1842, 1842, 1842,
 1842,    0, 1842, 1842, 1842, 1842,    0, 1842, 1842, 1842,
 1842, 1842,    0, 1842, 1842, 1842, 1842, 1842, 1842, 1842,
 1842, 1842, 1842, 1842, 1842, 1842, 1842, 1842, 1842, 1842,
 1842, 1842, 1842, 1842, 1842, 1842, 1842, 1842, 1842, 1842,
 1842, 1842, 1845, 1842, 1842, 1842, 1842,    0, 1842, 1842,
 1842, 1842, 1842, 1842,    0,    0, 1842, 1842, 1842, 1842,
 1842, 1842, 1842, 1842, 1842,    0,    0,    0,    0,    0,
 1842,    0, 1842, 1842, 1842, 1842, 1842, 1842,    0, 1842,
    0,    0, 1655, 1845, 1845, 1845, 1845, 1845, 1845, 1845,
    0,    0,    0,    0,    0,    0,    0, 1233,    0,    0,
 1845, 1845, 1845, 1845, 1845,    0, 1234,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1235,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1845,    0, 1845,    0,    0,    0,    0,
    0,    0, 1233,    0, 1939,    0,    0,    0,    0,    0,
    0, 1234,    0, 1236,    0,    0,    0,    0,    0,    0,
 1237,    0,    0,    0, 1235,    0,    0,    0, 1238,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1239,    0, 1240,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1241, 1236, 1242,
 1243, 1244,    0,    0,    0, 1237, 1245,    0,    0,    0,
    0,    0,    0, 1238,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1239,    0, 1240,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1241,    0, 1242, 1243, 1244, 1246,    0,    0,
    0, 1245,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1246,    0,    0,    0,    0,    0,    0, 1845,
    0, 1845,    0,    0, 1845,    0, 1845, 1845,    0,    0,
 1845, 1845,    0, 1845,    0,    0,    0,    0,    0, 1845,
    0, 1845, 1845, 1845, 1845,    0, 1845,    0,    0, 1845,
    0,    0, 1845,    0,    0,    0,    0,    0,    0, 1845,
    0, 1845, 1845,    0, 1845, 1845, 1845, 1845, 1845,    0,
    0, 1845,    0,    0, 1845,    0, 1845, 1845,    0,    0,
 1845, 1845, 1845,    0, 1845, 1845, 1845, 1845, 1845, 1845,
 1845,    0,    0,    0, 1845, 1845, 1845, 1845, 1845,    0,
    0,    0, 1845,    0,    0,    0,    0,    0,    0,    0,
 1845, 1845, 1845, 1845, 1845, 1845,    0,    0,    0, 1845,
 1845,    0, 1845,    0, 1845,    0, 1845, 1845, 1845,    0,
 1845,    0, 1845, 1845, 1845,    0, 1845, 1845, 1845, 1845,
 1845,    0, 1845, 1845,    0,    0,    0,    0, 1845,    0,
 1845, 1845,    0, 1247, 1845, 1845, 1845,    0,    0, 1845,
    0, 1845,    0,    0, 1845, 1845, 1845, 1845,    0, 1845,
    0, 1845,    0, 1845, 1845, 1845, 1845, 1845,    0,    0,
 1845, 1845, 1845,    0, 1845, 1845,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1845, 1845, 1845, 1247, 1845,
 1845, 1845,    0, 1845,    0,    0,    0, 1845,    0, 1845,
 1845, 1845, 1845, 1966, 1845,    0, 1845, 1845, 1845, 1845,
 1845, 1845, 1845, 1845,    0,    0,    0,    0,    0, 1249,
 1248, 1250,    0, 1845, 1845,    0, 1845, 1845, 1845,    0,
    0,    0,    0, 1845, 1845, 1845, 1845, 1845,    0, 1845,
    0,    0, 1845, 1845,    0,    0,    0, 1845, 1845,    0,
    0, 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845,
    0, 1845,    0,    0, 1249, 1248, 1250,    0,    0,    0,
    0, 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845,
 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845,
 1845, 1845,    0, 1845, 1845, 1845, 1845, 1845, 1845,    0,
 1845, 1845, 1845, 1845,    0, 1845,    0, 1845, 1845, 1845,
 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845,
 1845, 1845, 1845, 1845, 1845,    0, 1845, 1845, 1845, 1845,
    0, 1845, 1845, 1845, 1845, 1845,    0, 1845, 1845, 1845,
 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845,
 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845,
 1845, 1845, 1845, 1845, 1845, 1845, 1847, 1845, 1845, 1845,
 1845,    0, 1845, 1845, 1845, 1845, 1845, 1845,    0,    0,
 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845, 1845,    0,
    0,    0,    0,    0, 1845,    0, 1845, 1845, 1845, 1845,
 1845, 1845,    0, 1845,    0,    0, 1656, 1847, 1847, 1847,
 1847, 1847, 1847, 1847,    0,    0,    0,    0,    0,    0,
    0, 1233,    0,    0, 1847, 1847, 1847, 1847, 1847,    0,
 1234,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1235,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1847,    0, 1847,
    0,    0,    0,    0,    0,    0, 1233,    0,    0,    0,
    0,    0,    0,    0,    0, 1234,    0, 1236,    0,    0,
    0,    0,    0,    0, 1237,    0,    0,    0, 1235,    0,
    0,    0, 1238,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1239,
    0, 1240,    0,    0,    0,    0,    0,    0, 1958,    0,
    0, 1241, 1236, 1242, 1243, 1244,    0,    0,    0, 1237,
 1245,    0,    0,    0,    0,    0,    0, 1238,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1239,    0, 1240,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1241,    0, 1242, 1243,
 1244, 1246,    0,    0,    0, 1245,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1246,    0,    0,    0,
    0,    0,    0, 1847,    0, 1847,    0,    0, 1847,    0,
 1847, 1847,    0,    0, 1847, 1847,    0, 1847,    0,    0,
    0,    0,    0, 1847,    0, 1847, 1847, 1847, 1847,    0,
 1847,    0,    0, 1847,    0,    0, 1847,    0,    0,    0,
    0,    0,    0, 1847,    0, 1847, 1847,    0, 1847, 1847,
 1847, 1847, 1847,    0,    0, 1847,    0,    0, 1847,    0,
 1847, 1847,    0,    0, 1847, 1847, 1847,    0, 1847, 1847,
 1847, 1847, 1847, 1847, 1847,    0,    0,    0, 1847, 1847,
 1847, 1847, 1847,    0,    0,    0, 1847,    0,    0,    0,
    0,    0,    0,    0, 1847, 1847, 1847, 1847, 1847, 1847,
    0,    0,    0, 1847, 1847,    0, 1847,    0, 1847,    0,
 1847, 1847, 1847,    0, 1847,    0, 1847, 1847, 1847,    0,
 1847, 1847, 1847, 1847, 1847,    0, 1847, 1847,    0,    0,
    0,    0, 1847,    0, 1847, 1847,    0, 1247, 1847, 1847,
 1847,    0,    0, 1847,    0, 1847,    0,    0, 1847, 1847,
 1847, 1847,    0, 1847,    0, 1847,    0, 1847, 1847, 1847,
 1847, 1847,    0,    0, 1847, 1847, 1847,    0, 1847, 1847,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1847,
 1847, 1847, 1959, 1847, 1847, 1847,    0, 1847,    0,    0,
    0, 1847,    0, 1847, 1847, 1847, 1847, 1982, 1847,    0,
 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847,    0,    0,
    0,    0,    0, 1249, 1248, 1250,    0, 1847, 1847,    0,
 1847, 1847, 1847,    0,    0,    0,    0, 1847, 1847, 1847,
 1847, 1847,    0, 1847,    0,    0, 1847, 1847,    0,    0,
    0, 1847, 1847,    0,    0, 1847, 1847, 1847, 1847, 1847,
 1847, 1847, 1847, 1847,    0, 1847,    0,    0, 1249, 1248,
 1250,    0,    0,    0,    0, 1847, 1847, 1847, 1847, 1847,
 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847,
 1847, 1847, 1847, 1847, 1847, 1847,    0, 1847, 1847, 1847,
 1847, 1847, 1847,    0, 1847, 1847, 1847, 1847,    0, 1847,
    0, 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847,
 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847,    0,
 1847, 1847, 1847, 1847,    0, 1847, 1847, 1847, 1847, 1847,
    0, 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847,
 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847,
 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847, 1847,
 1848, 1847, 1847, 1847, 1847,    0, 1847, 1847, 1847, 1847,
 1847, 1847,    0,    0, 1847, 1847, 1847, 1847, 1847, 1847,
 1847, 1847, 1847,    0,    0,    0,    0,    0, 1847,    0,
 1847, 1847, 1847, 1847, 1847, 1847,    0, 1847,    0,    0,
 1657, 1848, 1848, 1848, 1848, 1848, 1848, 1848,    0,    0,
    0,    0,    0,    0,    0, 1233,    0,    0, 1848, 1848,
 1848, 1848, 1848,    0, 1234,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1235,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1848,    0, 1848,    0,    0,    0,    0,    0,    0,
 1233,    0,    0,    0,    0,    0,    0,    0,    0, 1234,
    0, 1236,    0,    0,    0,    0,    0,    0, 1237,    0,
    0,    0, 1235,    0,    0,    0, 1238,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1239,    0, 1240,    0,    0,    0,    0,
    0,    0, 2327,    0,    0, 1241, 1236, 1242, 1243, 1244,
    0,    0,    0, 1237, 1245,    0,    0,    0,    0,    0,
    0, 1238,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1239,    0,
 1240,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1241,    0, 1242, 1243, 1244, 1246,    0,    0,    0, 1245,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1246,    0,    0,    0,    0,    0,    0, 1848,    0, 1848,
    0,    0, 1848,    0, 1848, 1848,    0,    0, 1848, 1848,
    0, 1848,    0,    0,    0,    0,    0, 1848,    0, 1848,
 1848, 1848, 1848,    0, 1848,    0,    0, 1848,    0,    0,
 1848,    0,    0,    0,    0,    0,    0, 1848,    0, 1848,
 1848,    0, 1848, 1848, 1848, 1848, 1848,    0,    0, 1848,
    0,    0, 1848,    0, 1848, 1848,    0,    0, 1848, 1848,
 1848,    0, 1848, 1848, 1848, 1848, 1848, 1848, 1848,    0,
    0,    0, 1848, 1848, 1848, 1848, 1848,    0,    0,    0,
 1848,    0,    0,    0,    0,    0,    0,    0, 1848, 1848,
 1848, 1848, 1848, 1848,    0,    0,    0, 1848, 1848,    0,
 1848,    0, 1848,    0, 1848, 1848, 1848,    0, 1848,    0,
 1848, 1848, 1848,    0, 1848, 1848, 1848, 1848, 1848,    0,
 1848, 1848,    0,    0,    0,    0, 1848,    0, 1848, 1848,
    0, 1247, 1848, 1848, 1848,    0,    0, 1848,    0, 1848,
    0,    0, 1848, 1848, 1848, 1848,    0, 1848,    0, 1848,
    0, 1848, 1848, 1848, 1848, 1848,    0,    0, 1848, 1848,
 1848,    0, 1848, 1848,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1848, 1848, 1848, 1247, 1848, 1848, 1848,
    0, 1848,    0,    0,    0, 1848,    0, 1848, 1848, 1848,
 1848, 1987, 1848,    0, 1848, 1848, 1848, 1848, 1848, 1848,
 1848, 1848,    0,    0,    0,    0,    0, 1249, 1248, 1250,
    0, 1848, 1848,    0, 1848, 1848, 1848,    0,    0,    0,
    0, 1848, 1848, 1848, 1848, 1848,    0, 1848,    0,    0,
 1848, 1848,    0,    0,    0, 1848, 1848,    0,    0, 1848,
 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848,    0, 1848,
    0,    0, 1249, 1248, 1250,    0,    0,    0,    0, 1848,
 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848,
 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848,
    0, 1848, 1848, 1848, 1848, 1848, 1848,    0, 1848, 1848,
 1848, 1848,    0, 1848,    0, 1848, 1848, 1848, 1848, 1848,
 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848,
 1848, 1848, 1848,    0, 1848, 1848, 1848, 1848,    0, 1848,
 1848, 1848, 1848, 1848,    0, 1848, 1848, 1848, 1848, 1848,
 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848,
 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848, 1848,
 1848, 1848, 1848, 1848, 1851, 1848, 1848, 1848, 1848,    0,
 1848, 1848, 1848, 1848, 1848, 1848,    0,    0, 1848, 1848,
 1848, 1848, 1848, 1848, 1848, 1848, 1848,    0,    0,    0,
    0,    0, 1848,    0, 1848, 1848, 1848, 1848, 1848, 1848,
    0, 1848,    0,    0, 1658, 1851, 1851, 1851, 1851, 1851,
 1851, 1851,    0,    0,    0,    0,    0,    0,    0, 1233,
    0,    0, 1851, 1851, 1851, 1851, 1851,    0, 1234,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1235,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1851,    0, 1851,    0,    0,
    0,    0,    0,    0, 1233,    0,    0,    0,    0,    0,
    0,    0,    0, 1234,    0, 1236,    0,    0,    0,    0,
    0,    0, 1237,    0,    0,    0, 1235,    0,    0,    0,
 1238,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1239,    0, 1240,
    0,    0,    0,    0, 2598,    0,    0,    0,    0, 1241,
 1236, 1242, 1243, 1244,    0,    0,    0, 1237, 1245,    0,
    0,    0,    0,    0,    0, 1238,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1239,    0, 1240,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1241,    0, 1242, 1243, 1244, 1246,
    0,    0,    0, 1245,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1246,    0,    0,    0,    0,    0,
    0, 1851,    0, 1851,    0,    0, 1851,    0, 1851, 1851,
    0,    0, 1851, 1851,    0, 1851,    0,    0,    0,    0,
    0, 1851,    0, 1851, 1851, 1851, 1851,    0, 1851,    0,
    0, 1851,    0,    0, 1851,    0,    0,    0,    0,    0,
    0, 1851,    0, 1851, 1851,    0, 1851, 1851, 1851, 1851,
 1851,    0,    0, 1851,    0,    0, 1851,    0, 1851, 1851,
    0,    0, 1851, 1851, 1851,    0, 1851, 1851, 1851, 1851,
 1851, 1851, 1851,    0,    0,    0, 1851, 1851, 1851, 1851,
 1851,    0,    0,    0, 1851,    0,    0,    0,    0,    0,
    0,    0, 1851, 1851, 1851, 1851, 1851, 1851,    0,    0,
    0, 1851, 1851,    0, 1851,    0, 1851,    0, 1851, 1851,
 1851,    0, 1851,    0, 1851, 1851, 1851,    0, 1851, 1851,
 1851, 1851, 1851,    0, 1851, 1851,    0,    0,    0,    0,
 1851,    0, 1851, 1851,    0, 1247, 1851, 1851, 1851,    0,
    0, 1851,    0, 1851,    0,    0, 1851, 1851, 1851, 1851,
    0, 1851,    0, 1851,    0, 1851, 1851, 1851, 1851, 1851,
    0,    0, 1851, 1851, 1851,    0, 1851, 1851,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1851, 1851, 1851,
 1247, 1851, 1851, 1851,    0, 1851,    0,    0,    0, 1851,
    0, 1851, 1851, 1851, 1851, 1997, 1851,    0, 1851, 1851,
 1851, 1851, 1851, 1851, 1851, 1851,    0,    0,    0,    0,
    0, 1249, 1248, 1250,    0, 1851, 1851,    0, 1851, 1851,
 1851,    0,    0,    0,    0, 1851, 1851, 1851, 1851, 1851,
    0, 1851,    0,    0, 1851, 1851,    0,    0,    0, 1851,
 1851,    0,    0, 1851, 1851, 1851, 1851, 1851, 1851, 1851,
 1851, 1851,    0, 1851,    0,    0, 1249, 1248, 1250,    0,
    0,    0,    0, 1851, 1851, 1851, 1851, 1851, 1851, 1851,
 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1851,
 1851, 1851, 1851, 1851,    0, 1851, 1851, 1851, 1851, 1851,
 1851,    0, 1851, 1851, 1851, 1851,    0, 1851,    0, 1851,
 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1851,
 1851, 1851, 1851, 1851, 1851, 1851, 1851,    0, 1851, 1851,
 1851, 1851,    0, 1851, 1851, 1851, 1851, 1851,    0, 1851,
 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1851,
 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1851,
 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1853, 1851,
 1851, 1851, 1851,    0, 1851, 1851, 1851, 1851, 1851, 1851,
    0,    0, 1851, 1851, 1851, 1851, 1851, 1851, 1851, 1851,
 1851,    0,    0,    0,    0,    0, 1851,    0, 1851, 1851,
 1851, 1851, 1851, 1851,    0, 1851,    0,    0, 1659, 1853,
 1853, 1853, 1853, 1853, 1853, 1853,    0,    0,    0,    0,
    0,    0,    0, 1233,    0,    0, 1853, 1853, 1853, 1853,
 1853,    0, 1234,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1235,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1853,
    0, 1853,    0,    0,    0,    0,    0,    0, 1233,    0,
    0,    0,    0,    0,    0,    0,    0, 1234,    0, 1236,
    0,    0,    0,    0,    0,    0, 1237,    0,    0,    0,
 1235,    0,    0,    0, 1238,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1239,    0, 1240,    0,    0,    0,    0,    0,    0,
 2620,    0,    0, 1241, 1236, 1242, 1243, 1244,    0,    0,
    0, 1237, 1245,    0,    0,    0,    0,    0,    0, 1238,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1239,    0, 1240,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1241,    0,
 1242, 1243, 1244, 1246,    0,    0,    0, 1245,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1246,    0,
    0,    0,    0,    0,    0, 1853,    0, 1853,    0,    0,
 1853,    0, 1853, 1853,    0,    0, 1853, 1853,    0, 1853,
    0,    0,    0,    0,    0, 1853,    0, 1853, 1853, 1853,
 1853,    0, 1853,    0,    0, 1853,    0,    0, 1853,    0,
    0,    0,    0,    0,    0, 1853,    0, 1853, 1853,    0,
 1853, 1853, 1853, 1853, 1853,    0,    0, 1853,    0,    0,
 1853,    0, 1853, 1853,    0,    0, 1853, 1853, 1853,    0,
 1853, 1853, 1853, 1853, 1853, 1853, 1853,    0,    0,    0,
 1853, 1853, 1853, 1853, 1853,    0,    0,    0, 1853,    0,
    0,    0,    0,    0,    0,    0, 1853, 1853, 1853, 1853,
 1853, 1853,    0,    0,    0, 1853, 1853,    0, 1853,    0,
 1853,    0, 1853, 1853, 1853,    0, 1853,    0, 1853, 1853,
 1853,    0, 1853, 1853, 1853, 1853, 1853,    0, 1853, 1853,
    0,    0,    0,    0, 1853,    0, 1853, 1853,    0, 1247,
 1853, 1853, 1853,    0,    0, 1853,    0, 1853,    0,    0,
 1853, 1853, 1853, 1853,    0, 1853,    0, 1853,    0, 1853,
 1853, 1853, 1853, 1853,    0,    0, 1853, 1853, 1853,    0,
 1853, 1853,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1853, 1853, 1853, 1247, 1853, 1853, 1853,    0, 1853,
    0,    0,    0, 1853,    0, 1853, 1853, 1853, 1853, 1998,
 1853,    0, 1853, 1853, 1853, 1853, 1853, 1853, 1853, 1853,
    0,    0,    0,    0,    0, 1249, 1248, 1250,    0, 1853,
 1853,    0, 1853, 1853, 1853,    0,    0,    0,    0, 1853,
 1853, 1853, 1853, 1853,    0, 1853,    0,    0, 1853, 1853,
    0,    0,    0, 1853, 1853,    0,    0, 1853, 1853, 1853,
 1853, 1853, 1853, 1853, 1853, 1853,    0, 1853,    0,    0,
 1249, 1248, 1250,    0,    0,    0,    0, 1853, 1853, 1853,
 1853, 1853, 1853, 1853, 1853, 1853, 1853, 1853, 1853, 1853,
 1853, 1853, 1853, 1853, 1853, 1853, 1853, 1853,    0, 1853,
 1853, 1853, 1853, 1853, 1853,    0, 1853, 1853, 1853, 1853,
    0, 1853,    0, 1853, 1853, 1853, 1853, 1853, 1853, 1853,
 1853, 1853, 1853, 1853, 1853, 1853, 1853, 1853, 1853, 1853,
 1853,    0, 1853, 1853, 1853, 1853,    0, 1853, 1853, 1853,
 1853, 1853,    0, 1853, 1853, 1853, 1853, 1853, 1853, 1853,
 1853, 1853, 1853, 1853, 1853, 1853, 1853, 1853, 1853, 1853,
 1853, 1853, 1853, 1853, 1853, 1853, 1853, 1853, 1853, 1853,
 1853, 1853, 1855, 1853, 1853, 1853, 1853,    0, 1853, 1853,
 1853, 1853, 1853, 1853,    0,    0, 1853, 1853, 1853, 1853,
 1853, 1853, 1853, 1853, 1853,    0,    0,    0,    0,    0,
 1853,    0, 1853, 1853, 1853, 1853, 1853, 1853,    0, 1853,
    0,    0, 1660, 1855, 1855, 1855, 1855, 1855, 1855, 1855,
    0,    0,    0,    0,    0,    0,    0, 1233,    0,    0,
 1855, 1855, 1855, 1855, 1855,    0, 1234,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1235,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1855,    0, 1855,    0,    0,    0,    0,
    0,    0, 1233,    0,    0,    0,    0,    0,    0,    0,
    0, 1234,    0, 1236,    0,    0,    0,    0,    0,    0,
 1237,    0,    0,    0, 1235,    0,    0,    0, 1238,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1239,    0, 1240,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1241, 1236, 1242,
 1243, 1244,    0,    0,    0, 1237, 1245,    0,    0,    0,
    0,    0,    0, 1238,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1239,    0, 1240,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1241,    0, 1242, 1243, 1244, 1246,    0,    0,
    0, 1245,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1246,    0,    0,    0,    0,    0,    0, 1855,
    0, 1855,    0,    0, 1855,    0, 1855, 1855,    0,    0,
 1855, 1855,    0, 1855,    0,    0,    0,    0,    0, 1855,
    0, 1855, 1855, 1855, 1855,    0, 1855,    0,    0, 1855,
    0,    0, 1855,    0,    0,    0,    0,    0,    0, 1855,
    0, 1855, 1855,    0, 1855, 1855, 1855, 1855, 1855,    0,
    0, 1855,    0,    0, 1855,    0, 1855, 1855,    0,    0,
 1855, 1855, 1855,    0, 1855, 1855, 1855, 1855, 1855, 1855,
 1855,    0,    0,    0, 1855, 1855, 1855, 1855, 1855,    0,
    0,    0, 1855,    0,    0,    0,    0,    0,    0,    0,
 1855, 1855, 1855, 1855, 1855, 1855,    0,    0,    0, 1855,
 1855,    0, 1855,    0, 1855,    0, 1855, 1855, 1855,    0,
 1855,    0, 1855, 1855, 1855,    0, 1855, 1855, 1855, 1855,
 1855,    0, 1855, 1855,    0,    0,    0,    0, 1855,    0,
 1855, 1855,    0, 1247, 1855, 1855, 1855,    0,    0, 1855,
    0, 1855,    0,    0, 1855, 1855, 1855, 1855,    0, 1855,
    0, 1855,    0, 1855, 1855, 1855, 1855, 1855,    0,    0,
 1855, 1855, 1855,    0, 1855, 1855,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1855, 1855, 1855, 1247, 1855,
 1855, 1855,    0, 1855,    0,    0,    0, 1855,    0, 1855,
 1855, 1855, 1855, 1999, 1855,    0, 1855, 1855, 1855, 1855,
 1855, 1855, 1855, 1855,    0,    0,    0,    0,    0, 1249,
 1248, 1250,    0, 1855, 1855,    0, 1855, 1855, 1855,    0,
    0,    0,    0, 1855, 1855, 1855, 1855, 1855,    0, 1855,
    0,    0, 1855, 1855,    0,    0,    0, 1855, 1855,    0,
    0, 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855,
    0, 1855,    0,    0, 1249, 1248, 1250,    0,    0,    0,
    0, 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855,
 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855,
 1855, 1855,    0, 1855, 1855, 1855, 1855, 1855, 1855,    0,
 1855, 1855, 1855, 1855,    0, 1855,    0, 1855, 1855, 1855,
 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855,
 1855, 1855, 1855, 1855, 1855,    0, 1855, 1855, 1855, 1855,
    0, 1855, 1855, 1855, 1855, 1855,    0, 1855, 1855, 1855,
 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855,
 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855,
 1855, 1855, 1855, 1855, 1855, 1855, 1856, 1855, 1855, 1855,
 1855,    0, 1855, 1855, 1855, 1855, 1855, 1855,    0,    0,
 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855, 1855,    0,
    0,    0,    0,    0, 1855,    0, 1855, 1855, 1855, 1855,
 1855, 1855,    0, 1855,    0,    0, 1663, 1856, 1856, 1856,
 1856, 1856, 1856, 1856,    0,    0,    0,    0,    0,    0,
    0, 1233,    0,    0, 1856, 1856, 1856, 1856, 1856,    0,
 1234,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1235,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1856,    0, 1856,
    0,    0,    0,    0,    0,    0, 1233,    0,    0,    0,
    0,    0,    0,    0,    0, 1234,    0, 1236,    0,    0,
    0,    0,    0,    0, 1237,    0,    0,    0, 1235,    0,
    0,    0, 1238,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1239,
    0, 1240,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1241, 1236, 1242, 1243, 1244,    0,    0,    0, 1950,
 1245,    0,    0,    0,    0,    0,    0, 1238,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1239,    0, 1240,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1241,    0, 1242, 1243,
 1244, 1246,    0,    0,    0, 1245,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1246,    0,    0,    0,
    0,    0,    0, 1856,    0, 1856,    0,    0, 1856,    0,
 1856, 1856,    0,    0, 1856, 1856,    0, 1856,    0,    0,
    0,    0,    0, 1856,    0, 1856, 1856, 1856, 1856,    0,
 1856,    0,    0, 1856,    0,    0, 1856,    0,    0,    0,
    0,    0,    0, 1856,    0, 1856, 1856,    0, 1856, 1856,
 1856, 1856, 1856,    0,    0, 1856,    0,    0, 1856,    0,
 1856, 1856,    0,    0, 1856, 1856, 1856,    0, 1856, 1856,
 1856, 1856, 1856, 1856, 1856,    0,    0,    0, 1856, 1856,
 1856, 1856, 1856,    0,    0,    0, 1856,    0,    0,    0,
    0,    0,    0,    0, 1856, 1856, 1856, 1856, 1856, 1856,
    0,    0,    0, 1856, 1856,    0, 1856,    0, 1856,    0,
 1856, 1856, 1856,    0, 1856,    0, 1856, 1856, 1856,    0,
 1856, 1856, 1856, 1856, 1856,    0, 1856, 1856,    0,    0,
    0,    0, 1856,    0, 1856, 1856,    0, 1247, 1856, 1856,
 1856,    0,    0, 1856,    0, 1856,    0,    0, 1856, 1856,
 1856, 1856,    0, 1856,    0, 1856,    0, 1856, 1856, 1856,
 1856, 1856,    0,    0, 1856, 1856, 1856,    0, 1856, 1856,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1856,
 1856, 1856, 1247, 1856, 1856, 1856,    0, 1856,    0,    0,
    0, 1856,    0, 1856, 1856, 1856, 1856, 2000, 1856,    0,
 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856,    0,    0,
    0,    0,    0, 1249, 1248, 1250,    0, 1856, 1856,    0,
 1856, 1856, 1856,    0,    0,    0,    0, 1856, 1856, 1856,
 1856, 1856,    0, 1856,    0,    0, 1856, 1856,    0,    0,
    0, 1856, 1856,    0,    0, 1856, 1856, 1856, 1856, 1856,
 1856, 1856, 1856, 1856,    0, 1856,    0,    0, 1470, 1470,
 1470,    0,    0,    0,    0, 1856, 1856, 1856, 1856, 1856,
 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856,
 1856, 1856, 1856, 1856, 1856, 1856,    0, 1856, 1856, 1856,
 1856, 1856, 1856,    0, 1856, 1856, 1856, 1856,    0, 1856,
    0, 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856,
 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856,    0,
 1856, 1856, 1856, 1856,    0, 1856, 1856, 1856, 1856, 1856,
    0, 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856,
 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856,
 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856, 1856,
 1857, 1856, 1856, 1856, 1856,    0, 1856, 1856, 1856, 1856,
 1856, 1856,    0,    0, 1856, 1856, 1856, 1856, 1856, 1856,
 1856, 1856, 1856,    0,    0,    0,    0,    0, 1856,    0,
 1856, 1856, 1856, 1856, 1856, 1856,    0, 1856,    0,    0,
 1664, 1857, 1857, 1857, 1857, 1857, 1857, 1857,    0,    0,
    0,    0,    0,    0,    0, 1233,    0,    0, 1857, 1857,
 1857, 1857, 1857,    0, 1234,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1235,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1857,    0, 1857,    0,    0,    0,    0,    0,    0,
 1470,    0,    0,    0,    0,    0,    0,    0,    0, 1470,
    0, 1236,    0,    0,    0,    0,    0,    0, 1237,    0,
    0,    0, 1470,    0,    0,    0, 1238,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1239,    0, 1240,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1241, 1470, 1242, 1243, 1244,
    0,    0,    0, 1470, 1245,    0,    0,    0,    0,    0,
    0, 1470,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1470,    0,
 1470,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1470,    0, 1470, 1470, 1470, 1246,    0,    0,    0, 1470,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1470,    0,    0,    0,    0,    0,    0, 1857,    0, 1857,
    0,    0, 1857,    0, 1857, 1857,    0,    0, 1857, 1857,
    0, 1857,    0,    0,    0,    0,    0, 1857,    0, 1857,
 1857, 1857, 1857,    0, 1857,    0,    0, 1857,    0,    0,
 1857,    0,    0,    0,    0,    0,    0, 1857,    0, 1857,
 1857,    0, 1857, 1857, 1857, 1857, 1857,    0,    0, 1857,
    0,    0, 1857,    0, 1857, 1857,    0,    0, 1857, 1857,
 1857,    0, 1857, 1857, 1857, 1857, 1857, 1857, 1857,    0,
    0,    0, 1857, 1857, 1857, 1857, 1857,    0,    0,    0,
 1857,    0,    0,    0,    0,    0,    0,    0, 1857, 1857,
 1857, 1857, 1857, 1857,    0,    0,    0, 1857, 1857,    0,
 1857,    0, 1857,    0, 1857, 1857, 1857,    0, 1857,    0,
 1857, 1857, 1857,    0, 1857, 1857, 1857, 1857, 1857,    0,
 1857, 1857,    0,    0,    0,    0, 1857,    0, 1857, 1857,
    0, 1247, 1857, 1857, 1857,    0,    0, 1857,    0, 1857,
    0,    0, 1857, 1857, 1857, 1857,    0, 1857,    0, 1857,
    0, 1857, 1857, 1857, 1857, 1857,    0,    0, 1857, 1857,
 1857,    0, 1857, 1857,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1857, 1857, 1857, 1470, 1857, 1857, 1857,
    0, 1857,    0,    0,    0, 1857,    0, 1857, 1857, 1857,
 1857, 2001, 1857,    0, 1857, 1857, 1857, 1857, 1857, 1857,
 1857, 1857,    0,    0,    0,    0,    0, 1249, 1248, 1250,
    0, 1857, 1857,    0, 1857, 1857, 1857,    0,    0,    0,
    0, 1857, 1857, 1857, 1857, 1857,    0, 1857,    0,    0,
 1857, 1857,    0,    0,    0, 1857, 1857,    0,    0, 1857,
 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857,    0, 1857,
    0,    0, 1249, 1248, 1250,    0,    0,    0,    0, 1857,
 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857,
 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857,
    0, 1857, 1857, 1857, 1857, 1857, 1857,    0, 1857, 1857,
 1857, 1857,    0, 1857,    0, 1857, 1857, 1857, 1857, 1857,
 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857,
 1857, 1857, 1857,    0, 1857, 1857, 1857, 1857,    0, 1857,
 1857, 1857, 1857, 1857,    0, 1857, 1857, 1857, 1857, 1857,
 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857,
 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857, 1857,
 1857, 1857, 1857, 1857, 1858, 1857, 1857, 1857, 1857,    0,
 1857, 1857, 1857, 1857, 1857, 1857,    0,    0, 1857, 1857,
 1857, 1857, 1857, 1857, 1857, 1857, 1857,    0,    0,    0,
    0,    0, 1857,    0, 1857, 1857, 1857, 1857, 1857, 1857,
    0, 1857,    0,    0, 1666, 1858, 1858, 1858, 1858, 1858,
 1858, 1858,    0,    0,    0,    0,    0,    0,    0, 1233,
    0,    0, 1858, 1858, 1858, 1858, 1858,    0, 1234,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1235,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1858,    0, 1858,    0,    0,
    0,    0,    0,    0, 1233,    0,    0,    0,    0,    0,
    0,    0,    0, 1234,    0, 1236,    0,    0,    0,    0,
    0,    0, 1237,    0,    0,    0, 1235,    0,    0,    0,
 1238,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1239,    0, 1240,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1241,
 1236, 1242, 1243, 1244,    0,    0,    0, 1237, 1245,    0,
    0,    0,    0,    0,    0, 1238,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1239,    0, 1240,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1241,    0, 1242, 1243, 1244, 1246,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1246,    0,    0,    0,    0,    0,
    0, 1858,    0, 1858,    0,    0, 1858,    0, 1858, 1858,
    0,    0, 1858, 1858,    0, 1858,    0,    0,    0,    0,
    0, 1858,    0, 1858, 1858, 1858, 1858,    0, 1858,    0,
    0, 1858,    0,    0, 1858,    0,    0,    0,    0,    0,
    0, 1858,    0, 1858, 1858,    0, 1858, 1858, 1858, 1858,
 1858,    0,    0, 1858,    0,    0, 1858,    0, 1858, 1858,
    0,    0, 1858, 1858, 1858,    0, 1858, 1858, 1858, 1858,
 1858, 1858, 1858,    0,    0,    0, 1858, 1858, 1858, 1858,
 1858,    0,    0,    0, 1858,    0,    0,    0,    0,    0,
    0,    0, 1858, 1858, 1858, 1858, 1858, 1858,    0,    0,
    0, 1858, 1858,    0, 1858,    0, 1858,    0, 1858, 1858,
 1858,    0, 1858,    0, 1858, 1858, 1858,    0, 1858, 1858,
 1858, 1858, 1858,    0, 1858, 1858,    0,    0,    0,    0,
 1858,    0, 1858, 1858,    0, 1247, 1858, 1858, 1858,    0,
    0, 1858,    0, 1858,    0,    0, 1858, 1858, 1858, 1858,
    0, 1858,    0, 1858,    0, 1858, 1858, 1858, 1858, 1858,
    0,    0, 1858, 1858, 1858,    0, 1858, 1858,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1858, 1858, 1858,
 1247, 1858, 1858, 1858,    0, 1858,    0,    0,    0, 1858,
    0, 1858, 1858, 1858, 1858, 2002, 1858,    0, 1858, 1858,
 1858, 1858, 1858, 1858, 1858, 1858,    0,    0,    0,    0,
    0, 1249, 1248, 1250,    0, 1858, 1858,    0, 1858, 1858,
 1858,    0,    0,    0,    0, 1858, 1858, 1858, 1858, 1858,
    0, 1858,    0,    0, 1858, 1858,    0,    0,    0, 1858,
 1858,    0,    0, 1858, 1858, 1858, 1858, 1858, 1858, 1858,
 1858, 1858,    0, 1858,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1858, 1858, 1858, 1858, 1858, 1858, 1858,
 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1858,
 1858, 1858, 1858, 1858,    0, 1858, 1858, 1858, 1858, 1858,
 1858,    0, 1858, 1858, 1858, 1858,    0, 1858,    0, 1858,
 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1858,
 1858, 1858, 1858, 1858, 1858, 1858, 1858,    0, 1858, 1858,
 1858, 1858,    0, 1858, 1858, 1858, 1858, 1858,    0, 1858,
 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1858,
 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1858,
 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1859, 1858,
 1858, 1858, 1858,    0, 1858, 1858, 1858, 1858, 1858, 1858,
    0,    0, 1858, 1858, 1858, 1858, 1858, 1858, 1858, 1858,
 1858,    0,    0,    0,    0,    0, 1858,    0, 1858, 1858,
 1858, 1858, 1858, 1858,    0, 1858,    0,    0, 1667, 1859,
 1859, 1859, 1859, 1859, 1859, 1859,    0,    0,    0,    0,
    0,    0,    0, 1233,    0,    0, 1859, 1859, 1859, 1859,
 1859,  155, 1234,    0,  155,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1235,  155,    0,    0,    0,
  155,    0,  155,    0,    0,    0,    0,    0,    0, 1859,
    0, 1859,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  167,    0,    0,  167,    0,    0,    0, 1236,
    0,    0,    0,    0,    0,    0, 1237,  167,    0,    0,
  155,  167,    0,  167, 1238,    0,  155,  155,  155,    0,
  155,  155,  155,    0,  155,    0,    0,    0,    0,    0,
    0, 1239,    0, 1240,    0,    0,    0,    0,  155,    0,
  155,    0,    0, 1241,    0, 1242, 1243, 1244,    0,    0,
    0,  167, 1245,    0,    0,    0,    0,  167,  167,  167,
    0,  167,  167,  167,    0,  167,    0,    0,    0,  155,
    0,    0,    0,    0,  154,    0,    0,    0,    0,  167,
    0,  167,    0,    0,    0,  155,    0,    0,    0,    0,
    0,    0,    0, 1246,    0,    0,    0,    0,    0,  167,
    0,    0,    0,    0,    0,    0,    0,  155,    0,    0,
  167,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  167,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  155,  155,    0,    0, 1859,    0, 1859,  167,    0,
 1859,    0, 1859, 1859,    0,    0, 1859, 1859,  155, 1859,
    0,    0,    0,    0,    0, 1859,    0, 1859, 1859, 1859,
 1859,    0, 1859,    0,    0, 1859,    0,    0, 1859,    0,
    0,    0,  167,  167,    0, 1859,    0, 1859, 1859,    0,
 1859, 1859, 1859, 1859, 1859,    0,    0, 1859,    0,  167,
 1859,    0, 1859, 1859,    0,    0, 1859, 1859, 1859,    0,
 1859, 1859, 1859, 1859, 1859, 1859, 1859,    0,    0,    0,
 1859, 1859, 1859, 1859, 1859,    0,    0,    0, 1859,    0,
    0,    0,    0,    0,    0,    0, 1859, 1859, 1859, 1859,
 1859, 1859,    0,    0,    0, 1859, 1859,    0, 1859,    0,
 1859,    0, 1859, 1859, 1859,    0, 1859,    0, 1859, 1859,
 1859,    0, 1859, 1859, 1859, 1859, 1859,    0, 1859, 1859,
    0,    0,    0,    0, 1859,    0, 1859, 1859,    0, 1247,
 1859, 1859, 1859,    0,    0, 1859,    0, 1859,    0,    0,
 1859, 1859, 1859, 1859,    0, 1859,    0, 1859,    0, 1859,
 1859, 1859, 1859, 1859,    0,    0, 1859, 1859, 1859,    0,
 1859, 1859,    0,    0,    0,    0,    0,  155,    0,    0,
    0, 1859, 1859, 1859,    0, 1859, 1859, 1859,    0, 1859,
    0,    0,    0, 1859,    0, 1859, 1859, 1859, 1859, 2003,
 1859,    0, 1859, 1859, 1859, 1859, 1859, 1859, 1859, 1859,
    0,    0,    0,    0,    0, 1249, 1248, 1250,  167, 1859,
 1859,    0, 1859, 1859, 1859,    0,    0,    0,    0, 1859,
 1859, 1859, 1859, 1859,    0, 1859,    0,    0, 1859, 1859,
    0,    0,    0, 1859, 1859,    0,    0, 1859, 1859, 1859,
 1859, 1859, 1859, 1859, 1859, 1859,    0, 1859,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1859, 1859, 1859,
 1859, 1859, 1859, 1859, 1859, 1859, 1859, 1859, 1859, 1859,
 1859, 1859, 1859, 1859, 1859, 1859, 1859, 1859,    0, 1859,
 1859, 1859, 1859, 1859, 1859,    0, 1859, 1859, 1859, 1859,
    0, 1859,    0, 1859, 1859, 1859, 1859, 1859, 1859, 1859,
 1859, 1859, 1859, 1859, 1859, 1859, 1859, 1859, 1859, 1859,
 1859,    0, 1859, 1859, 1859, 1859,    0, 1859, 1859, 1859,
 1859, 1859,    0, 1859, 1859, 1859, 1859, 1859, 1859, 1859,
 1859, 1859, 1859, 1859, 1859, 1859, 1859, 1859, 1859, 1859,
 1859, 1859, 1859, 1859, 1859, 1859, 1859, 1859, 1859, 1859,
 1859, 1859, 1860, 1859, 1859, 1859, 1859,    0, 1859, 1859,
 1859, 1859, 1859, 1859,    0,    0, 1859, 1859, 1859, 1859,
 1859, 1859, 1859, 1859, 1859,    0,    0,    0,    0,    0,
 1859,    0, 1859, 1859, 1859, 1859, 1859, 1859,    0, 1859,
    0,    0, 1668, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
    0,    0,    0,    0,    0,    0,    0, 1233,    0,    0,
 1860, 1860, 1860, 1860, 1860, 2585, 1234,    0, 1061,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1235,
 2586,    0,    0,    0, 1062,    0, 1063,    0,    0,    0,
    0,    0,    0, 1860,    0, 1860,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  263,    0,    0,  263,
    0,    0,    0, 1236,    0,    0,    0,    0,    0,    0,
 1237,    0,    0,    0, 1064,  263,    0,  263, 1238,    0,
 1065, 1066, 1067,    0, 1068, 1069, 1070,    0, 1071,    0,
    0,    0,    0,    0,    0, 1239,    0, 1240,    0,    0,
    0,    0, 1072,    0, 1073,    0,    0, 1241,    0, 1242,
 1243, 1244,    0,    0,    0,  263, 1245,    0,    0,    0,
    0,  263,  263,  263,    0,  263,  263,  263,    0,  263,
    0,    0,    0, 1074,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  263,    0,  263,    0,    0,    0, 1075,
    0,    0,    0,    0,    0,    0,    0, 1246,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1076,    0,    0,  263,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  263,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1077, 1078,    0,    0, 1860,
    0, 1860,  263,    0, 1860,    0, 1860, 1860,    0,    0,
 1860, 1860, 1079, 1860,    0,    0,    0,    0,    0, 1860,
    0, 1860, 1860, 1860, 1860,    0, 1860,    0,    0, 1860,
    0,    0, 1860,    0,    0,    0,  263,  263,    0, 1860,
    0, 1860, 1860,    0, 1860, 1860, 1860, 1860, 1860,    0,
    0, 1860,    0,  263, 1860,    0, 1860, 1860,    0,    0,
 1860, 1860, 1860,    0, 1860, 1860, 1860, 1860, 1860, 1860,
 1860,    0,    0,    0, 1860, 1860, 1860, 1860, 1860,    0,
    0,    0, 1860,    0,    0,    0,    0,    0,    0,    0,
 1860, 1860, 1860, 1860, 1860, 1860,    0,    0,    0, 1860,
 1860,    0, 1860,    0, 1860,    0, 1860, 1860, 1860,    0,
 1860,    0, 1860, 1860, 1860,    0, 1860, 1860, 1860, 1860,
 1860,    0, 1860, 1860,    0,    0,    0,    0, 1860,    0,
 1860, 1860,    0, 1247, 1860, 1860, 1860,    0,    0, 1860,
    0, 1860,    0,    0, 1860, 1860, 1860, 1860,    0, 1860,
    0, 1860,    0, 1860, 1860, 1860, 1860, 1860,    0,    0,
 1860, 1860, 1860,    0, 1860, 1860,    0,    0,    0,    0,
    0, 1080,    0,    0,    0, 1860, 1860, 1860,    0, 1860,
 1860, 1860,    0, 1860,    0,    0,    0, 1860,    0, 1860,
 1860, 1860, 1860, 2004, 1860,    0, 1860, 1860, 1860, 1860,
 1860, 1860, 1860, 1860,    0,    0,    0,    0,    0, 1249,
 1248, 1250,  263, 1860, 1860,    0, 1860, 1860, 1860,    0,
    0,    0,    0, 1860, 1860, 1860, 1860, 1860,    0, 1860,
    0,    0, 1860, 1860,    0,    0,    0, 1860, 1860,    0,
    0, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
    0, 1860,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
 1860, 1860,    0, 1860, 1860, 1860, 1860, 1860, 1860,    0,
 1860, 1860, 1860, 1860,    0, 1860,    0, 1860, 1860, 1860,
 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
 1860, 1860, 1860, 1860, 1860,    0, 1860, 1860, 1860, 1860,
    0, 1860, 1860, 1860, 1860, 1860,    0, 1860, 1860, 1860,
 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,
 1860, 1860, 1860, 1860, 1860, 1860, 1861, 1860, 1860, 1860,
 1860,    0, 1860, 1860, 1860, 1860, 1860, 1860,    0,    0,
 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860,    0,
    0,    0,    0,    0, 1860,    0, 1860, 1860, 1860, 1860,
 1860, 1860,    0, 1860,    0,    0, 1669, 1861, 1861, 1861,
 1861, 1861, 1861, 1861,    0,    0,    0,    0,    0,    0,
    0, 1233,    0,    0, 1861, 1861, 1861, 1861, 1861,    0,
 1234,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1627, 1235,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1861,    0, 1861,
    0,    0,    0,    0,    0,    0,    0, 1628,    0,    0,
    0,    0,    0, 1629,    0,    0,    0, 1236, 1630,    0,
    0,    0,    0, 1631, 1237,    0, 1632,    0,    0,    0,
    0,    0, 1238,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1239,
    0, 1240,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1241,    0, 1242, 1243, 1244,    0,    0,    0,    0,
 1245,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1633,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1246,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1634,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1635,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1636,    0,    0,
    0,    0,    0, 1861,    0, 1861,    0,    0, 1861,    0,
 1861, 1861,    0,    0, 1861, 1861,    0, 1861,    0,    0,
    0,    0,    0, 1861,    0, 1861, 1861, 1861, 1861,    0,
 1861,    0,    0, 1861,    0,    0, 1861,    0,    0,    0,
    0,    0,    0, 1861,    0, 1861, 1861,    0, 1861, 1861,
 1861, 1861, 1861, 1637,    0, 1861, 1638,    0, 1861,    0,
 1861, 1861,    0,    0, 1861, 1861, 1861,    0, 1861, 1861,
 1861, 1861, 1861, 1861, 1861,    0,    0,    0, 1861, 1861,
 1861, 1861, 1861,    0,    0,    0, 1861,    0,    0,    0,
    0,    0,    0,    0, 1861, 1861, 1861, 1861, 1861, 1861,
    0,    0,    0, 1861, 1861,    0, 1861,    0, 1861,    0,
 1861, 1861, 1861,    0, 1861,    0, 1861, 1861, 1861,    0,
 1861, 1861, 1861, 1861, 1861,    0, 1861, 1861,    0,    0,
    0,    0, 1861,    0, 1861, 1861,    0, 1247, 1861, 1861,
 1861,    0,    0, 1861,    0, 1861,    0,    0, 1861, 1861,
 1861, 1861,    0, 1861,    0, 1861,    0, 1861, 1861, 1861,
 1861, 1861, 1639,    0, 1861, 1861, 1861,    0, 1861, 1861,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1861,
 1861, 1861,    0, 1861, 1861, 1861,    0, 1861,    0,    0,
    0, 1861,    0, 1861, 1861, 1861, 1861, 2005, 1861,    0,
 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861,    0,    0,
    0,    0,    0, 1249, 1248, 1250,    0, 1861, 1861,    0,
 1861, 1861, 1861,    0,    0,    0,    0, 1861, 1861, 1861,
 1861, 1861,    0, 1861,    0,    0, 1861, 1861,    0,    0,
    0, 1861, 1861,    0,    0, 1861, 1861, 1861, 1861, 1861,
 1861, 1861, 1861, 1861,    0, 1861,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1861, 1861, 1861, 1861, 1861,
 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861,
 1861, 1861, 1861, 1861, 1861, 1861,    0, 1861, 1861, 1861,
 1861, 1861, 1861,    0, 1861, 1861, 1861, 1861,    0, 1861,
    0, 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861,
 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861,    0,
 1861, 1861, 1861, 1861,    0, 1861, 1861, 1861, 1861, 1861,
    0, 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861,
 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861,
 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861, 1861,
 1862, 1861, 1861, 1861, 1861,    0, 1861, 1861, 1861, 1861,
 1861, 1861,    0,    0, 1861, 1861, 1861, 1861, 1861, 1861,
 1861, 1861, 1861,    0,    0,    0,    0,    0, 1861,    0,
 1861, 1861, 1861, 1861, 1861, 1861,    0, 1861,    0,    0,
 1670, 1862, 1862, 1862, 1862, 1862, 1862, 1862,    0,    0,
    0,    0,    0,    0,    0, 1233,    0,    0, 1862, 1862,
 1862, 1862, 1862,    0, 1234,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   77, 1235,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1862,    0, 1862,    0,    0,    0,    0,    0,    0,
    0,   77,    0,    0,    0,    0,    0,   77,    0,    0,
    0, 1236,   77,    0,    0,    0,    0,   77, 1237,    0,
   77,    0,    0,    0,    0,    0, 1238,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1239,    0, 1240,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1241,    0, 1242, 1243, 1244,
    0,    0,    0,    0, 1245,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   77,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1246,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   77,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   77,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   77,    0,    0,    0,    0,    0, 1862,    0, 1862,
    0,    0, 1862,    0, 1862, 1862,    0,    0, 1862, 1862,
    0, 1862,    0,    0,    0,    0,    0, 1862,    0, 1862,
 1862, 1862, 1862,    0, 1862,    0,    0, 1862,    0,    0,
 1862,    0,    0,    0,    0,    0,    0, 1862,    0, 1862,
 1862,    0, 1862, 1862, 1862, 1862, 1862,   77,    0, 1862,
   77,    0, 1862,    0, 1862, 1862,    0,    0, 1862, 1862,
 1862,    0, 1862, 1862, 1862, 1862, 1862, 1862, 1862,    0,
    0,    0, 1862, 1862, 1862, 1862, 1862,    0,    0,    0,
 1862,    0,    0,    0,    0,    0,    0,    0, 1862, 1862,
 1862, 1862, 1862, 1862,    0,    0,    0, 1862, 1862,    0,
 1862,    0, 1862,    0, 1862, 1862, 1862,    0, 1862,    0,
 1862, 1862, 1862,    0, 1862, 1862, 1862, 1862, 1862,    0,
 1862, 1862,    0,    0,    0,    0, 1862,    0, 1862, 1862,
    0, 1247, 1862, 1862, 1862,    0,    0, 1862,    0, 1862,
    0,    0, 1862, 1862, 1862, 1862,    0, 1862,    0, 1862,
    0, 1862, 1862, 1862, 1862, 1862,   77,    0, 1862, 1862,
 1862,    0, 1862, 1862,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1862, 1862, 1862,    0, 1862, 1862, 1862,
    0, 1862,    0,    0,    0, 1862,    0, 1862, 1862, 1862,
 1862, 2006, 1862,    0, 1862, 1862, 1862, 1862, 1862, 1862,
 1862, 1862,    0,    0,    0,    0,    0, 1249, 1248, 1250,
    0, 1862, 1862,    0, 1862, 1862, 1862,    0,    0,    0,
    0, 1862, 1862, 1862, 1862, 1862,    0, 1862,    0,    0,
 1862, 1862,    0,    0,    0, 1862, 1862,    0,    0, 1862,
 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862,    0, 1862,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1862,
 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862,
 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862,
    0, 1862, 1862, 1862, 1862, 1862, 1862,    0, 1862, 1862,
 1862, 1862,    0, 1862,    0, 1862, 1862, 1862, 1862, 1862,
 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862,
 1862, 1862, 1862,    0, 1862, 1862, 1862, 1862,    0, 1862,
 1862, 1862, 1862, 1862,    0, 1862, 1862, 1862, 1862, 1862,
 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862,
 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862, 1862,
 1862, 1862, 1862, 1862, 1864, 1862, 1862, 1862, 1862,    0,
 1862, 1862, 1862, 1862, 1862, 1862,    0,    0, 1862, 1862,
 1862, 1862, 1862, 1862, 1862, 1862, 1862,    0,    0,    0,
    0,    0, 1862,    0, 1862, 1862, 1862, 1862, 1862, 1862,
    0, 1862,    0,    0, 1671, 1864, 1864, 1864, 1864, 1864,
 1864, 1864,    0,    0,    0,    0,    0,    0,    0, 1233,
    0,    0, 1864, 1864, 1864, 1864, 1864,    0, 1234,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1235,    0,    0,    0,    0,    0,    0,    0,    0,
  106,    0,    0,    0,    0, 1864,  107, 1864,    0,    0,
  108,  109,    0,    0,    0,    0,  110,    0,    0,  111,
    0,    0,    0,    0,    0, 1236,    0,    0,    0,  112,
    0,    0, 1237,    0,    0,    0,    0,    0,    0,    0,
 1238,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1239,    0, 1240,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1241,
    0, 1242, 1243, 1244,    0,    0,    0,    0, 1245,    0,
    0,    0,    0,    0,    0,    0,    0,  113,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  114,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  115,    0, 1246,
    0,  116,    0,    0,    0,    0,  117,    0,    0,    0,
    0,    0,  118,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  119,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1864,    0, 1864,    0,    0, 1864,    0, 1864, 1864,
    0,    0, 1864, 1864,    0, 1864,    0,    0,    0,    0,
    0, 1864,    0, 1864, 1864, 1864, 1864,    0, 1864,    0,
    0, 1864,    0,    0, 1864,    0,  120,    0,    0,  121,
    0, 1864,    0, 1864, 1864,    0, 1864, 1864, 1864, 1864,
 1864,    0,    0, 1864,    0,    0, 1864,    0, 1864, 1864,
    0,    0, 1864, 1864, 1864,    0, 1864, 1864, 1864, 1864,
 1864, 1864, 1864,    0,    0,    0, 1864, 1864, 1864, 1864,
 1864,    0,  122,    0, 1864,    0,    0,    0,    0,    0,
    0,    0, 1864, 1864, 1864, 1864, 1864, 1864,    0,    0,
    0, 1864, 1864,    0, 1864,    0, 1864,    0, 1864, 1864,
 1864,    0, 1864,    0, 1864, 1864, 1864,    0, 1864, 1864,
 1864, 1864, 1864,    0, 1864, 1864,  123,    0,    0,    0,
 1864,    0, 1864, 1864,    0, 1247, 1864, 1864, 1864,    0,
    0, 1864,    0, 1864,    0,  124, 1864, 1864, 1864, 1864,
    0, 1864,    0, 1864,    0, 1864, 1864, 1864, 1864, 1864,
    0,    0, 1864, 1864, 1864,    0, 1864, 1864,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1864, 1864, 1864,
    0, 1864, 1864, 1864,    0, 1864,    0,    0,    0, 1864,
    0, 1864, 1864, 1864, 1864, 2007, 1864,    0, 1864, 1864,
 1864, 1864, 1864, 1864, 1864, 1864,    0,    0,    0,    0,
    0, 1249, 1248, 1250,    0, 1864, 1864,    0, 1864, 1864,
 1864,    0,    0,    0,    0, 1864, 1864, 1864, 1864, 1864,
    0, 1864,    0,    0, 1864, 1864,    0,    0,    0, 1864,
 1864,    0,    0, 1864, 1864, 1864, 1864, 1864, 1864, 1864,
 1864, 1864,    0, 1864,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1864, 1864, 1864, 1864, 1864, 1864, 1864,
 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1864,
 1864, 1864, 1864, 1864,    0, 1864, 1864, 1864, 1864, 1864,
 1864,    0, 1864, 1864, 1864, 1864,    0, 1864,    0, 1864,
 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1864,
 1864, 1864, 1864, 1864, 1864, 1864, 1864,    0, 1864, 1864,
 1864, 1864,    0, 1864, 1864, 1864, 1864, 1864,    0, 1864,
 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1864,
 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1864,
 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1865, 1864,
 1864, 1864, 1864,    0, 1864, 1864, 1864, 1864, 1864, 1864,
    0,    0, 1864, 1864, 1864, 1864, 1864, 1864, 1864, 1864,
 1864,    0,    0,    0,    0,    0, 1864,    0, 1864, 1864,
 1864, 1864, 1864, 1864,    0, 1864,    0,    0, 1672, 1865,
 1865, 1865, 1865, 1865, 1865, 1865,    0,    0,    0,    0,
    0,    0,    0, 1233,    0,    0, 1865, 1865, 1865, 1865,
 1865,    0, 1234,    0,    0, 1885,    0,    0,    0,    0,
    0, 1886,    0,    0,    0, 1235,    0,    0,    0,    0,
    0, 1887,    0,    0,    0,    0,    0,    0,    0, 1865,
    0, 1865,    0,    0, 1888,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1236,
    0,    0,    0,    0,    0,    0, 1237,    0,    0,    0,
    0,    0,    0,    0, 1238,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1239,    0, 1240,    0,    0,    0,    0,    0,    0,
    0,    0, 1889, 1241,    0, 1242, 1243, 1244,    0,    0,
    0,    0, 1245,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1890,    0,    0,    0, 1891,    0,    0,    0,
    0, 1892,    0,    0,    0,    0,    0, 1893,    0,    0,
    0,    0,    0, 1246,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1894,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1865,    0, 1865,    0,    0,
 1865,    0, 1865, 1865,    0,    0, 1865, 1865,    0, 1865,
    0, 1895,    0,    0, 1896, 1865,    0, 1865, 1865, 1865,
 1865,    0, 1865,    0,    0, 1865,    0,    0, 1865,    0,
    0,    0,    0,    0,    0, 1865,    0, 1865, 1865,    0,
 1865, 1865, 1865, 1865, 1865,    0,    0, 1865,    0,    0,
 1865,    0, 1865, 1865,    0,    0, 1865, 1865, 1865,    0,
 1865, 1865, 1865, 1865, 1865, 1865, 1865,    0,    0,    0,
 1865, 1865, 1865, 1865, 1865,    0,    0,    0, 1865,    0,
    0,    0,    0,    0,    0,    0, 1865, 1865, 1865, 1865,
 1865, 1865,    0,    0,    0, 1865, 1865,    0, 1865,    0,
 1865, 1897, 1865, 1865, 1865,    0, 1865,    0, 1865, 1865,
 1865,    0, 1865, 1865, 1865, 1865, 1865,    0, 1865, 1865,
 1898,    0,    0,    0, 1865,    0, 1865, 1865,    0, 1247,
 1865, 1865, 1865,    0,    0, 1865,    0, 1865,    0,    0,
 1865, 1865, 1865, 1865,    0, 1865,    0, 1865,    0, 1865,
 1865, 1865, 1865, 1865,    0,    0, 1865, 1865, 1865,    0,
 1865, 1865,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1865, 1865, 1865,    0, 1865, 1865, 1865,    0, 1865,
    0,    0,    0, 1865,    0, 1865, 1865, 1865, 1865, 2008,
 1865,    0, 1865, 1865, 1865, 1865, 1865, 1865, 1865, 1865,
    0,    0,    0,    0,    0, 1249, 1248, 1250,    0, 1865,
 1865,    0, 1865, 1865, 1865,    0,    0,    0,    0, 1865,
 1865, 1865, 1865, 1865,    0, 1865,    0,    0, 1865, 1865,
    0,    0,    0, 1865, 1865,    0,    0, 1865, 1865, 1865,
 1865, 1865, 1865, 1865, 1865, 1865,    0, 1865,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1865, 1865, 1865,
 1865, 1865, 1865, 1865, 1865, 1865, 1865, 1865, 1865, 1865,
 1865, 1865, 1865, 1865, 1865, 1865, 1865, 1865,    0, 1865,
 1865, 1865, 1865, 1865, 1865,    0, 1865, 1865, 1865, 1865,
    0, 1865,    0, 1865, 1865, 1865, 1865, 1865, 1865, 1865,
 1865, 1865, 1865, 1865, 1865, 1865, 1865, 1865, 1865, 1865,
 1865,    0, 1865, 1865, 1865, 1865,    0, 1865, 1865, 1865,
 1865, 1865,    0, 1865, 1865, 1865, 1865, 1865, 1865, 1865,
 1865, 1865, 1865, 1865, 1865, 1865, 1865, 1865, 1865, 1865,
 1865, 1865, 1865, 1865, 1865, 1865, 1865, 1865, 1865, 1865,
 1865, 1865, 1866, 1865, 1865, 1865, 1865,    0, 1865, 1865,
 1865, 1865, 1865, 1865,    0,    0, 1865, 1865, 1865, 1865,
 1865, 1865, 1865, 1865, 1865,    0,    0,    0,    0,    0,
 1865,    0, 1865, 1865, 1865, 1865, 1865, 1865,    0, 1865,
    0,    0, 1673, 1866, 1866, 1866, 1866, 1866, 1866, 1866,
    0,    0,    0,    0,    0,    0,    0, 1233,    0,    0,
 1866, 1866, 1866, 1866, 1866,    0, 1234,    0,    0, 1899,
    0,    0,    0,    0,    0, 1900,    0,    0,    0, 1235,
    0,    0,    0,    0,    0, 1901,    0,    0,    0,    0,
    0,    0,    0, 1866,    0, 1866,    0,    0, 1902,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1236,    0,    0,    0,    0,    0,    0,
 1237,    0,    0,    0,    0,    0,    0,    0, 1238,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1239,    0, 1240,    0,    0,
    0,    0,    0,    0,    0,    0, 1903, 1241,    0, 1242,
 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1904,    0,    0,    0,
 1905,    0,    0,    0,    0, 1906,    0,    0,    0,    0,
    0, 1907,    0,    0,    0,    0,    0, 1246,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1908,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1866,
    0, 1866,    0,    0, 1866,    0, 1866, 1866,    0,    0,
 1866, 1866,    0, 1866,    0, 1909,    0,    0, 1910, 1866,
    0, 1866, 1866, 1866, 1866,    0, 1866,    0,    0, 1866,
    0,    0, 1866,    0,    0,    0,    0,    0,    0, 1866,
    0, 1866, 1866,    0, 1866, 1866, 1866, 1866, 1866,    0,
    0, 1866,    0,    0, 1866,    0, 1866, 1866,    0,    0,
 1866, 1866, 1866,    0, 1866, 1866, 1866, 1866, 1866, 1866,
 1866,    0,    0,    0, 1866, 1866, 1866, 1866, 1866,    0,
    0,    0, 1866,    0,    0,    0,    0,    0,    0,    0,
 1866, 1866, 1866, 1866, 1866, 1866,    0,    0,    0, 1866,
 1866,    0, 1866,    0, 1866, 1911, 1866, 1866, 1866,    0,
 1866,    0, 1866, 1866, 1866,    0, 1866, 1866, 1866, 1866,
 1866,    0, 1866, 1866, 1912,    0,    0,    0, 1866,    0,
 1866, 1866,    0, 1247, 1866, 1866, 1866,    0,    0, 1866,
    0, 1866,    0,    0, 1866, 1866, 1866, 1866,    0, 1866,
    0, 1866,    0, 1866, 1866, 1866, 1866, 1866,    0,    0,
 1866, 1866, 1866,    0, 1866, 1866,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1866, 1866, 1866,    0, 1866,
 1866, 1866,    0, 1866,    0,    0,    0, 1866,    0, 1866,
 1866, 1866, 1866, 2334, 1866,    0, 1866, 1866, 1866, 1866,
 1866, 1866, 1866, 1866,    0,    0,    0,    0,    0, 1249,
 1248, 1250,    0, 1866, 1866,    0, 1866, 1866, 1866,    0,
    0,    0,    0, 1866, 1866, 1866, 1866, 1866,    0, 1866,
    0,    0, 1866, 1866,    0,    0,    0, 1866, 1866,    0,
    0, 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866,
    0, 1866,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866,
 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866,
 1866, 1866,    0, 1866, 1866, 1866, 1866, 1866, 1866,    0,
 1866, 1866, 1866, 1866,    0, 1866,    0, 1866, 1866, 1866,
 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866,
 1866, 1866, 1866, 1866, 1866,    0, 1866, 1866, 1866, 1866,
    0, 1866, 1866, 1866, 1866, 1866,    0, 1866, 1866, 1866,
 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866,
 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866,
 1866, 1866, 1866, 1866, 1866, 1866, 1868, 1866, 1866, 1866,
 1866,    0, 1866, 1866, 1866, 1866, 1866, 1866,    0,    0,
 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866, 1866,    0,
    0,    0,    0,    0, 1866,    0, 1866, 1866, 1866, 1866,
 1866, 1866,    0, 1866,    0,    0, 1674, 1868, 1868, 1868,
 1868, 1868, 1868, 1868,    0,    0,    0,    0,    0,    0,
    0, 1233,    0,    0, 1868, 1868, 1868, 1868, 1868,    0,
 1234,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1235,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1868,    0, 1868,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1236,    0,    0,
    0,    0,    0,    0, 1237,    0,    0,    0,    0,    0,
    0,    0, 1238,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1239,
    0, 1240,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1241,    0, 1242, 1243, 1244,    0,    0,    0,    0,
 1245,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1246,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1868,    0, 1868,    0,    0, 1868,    0,
 1868, 1868,    0,    0, 1868, 1868,    0, 1868,    0,    0,
    0,    0,    0, 1868,    0, 1868, 1868, 1868, 1868,    0,
 1868,    0,    0, 1868,    0,    0, 1868,    0,    0,    0,
    0,    0,    0, 1868,    0, 1868, 1868,    0, 1868, 1868,
 1868, 1868, 1868,    0,    0, 1868,    0,    0, 1868,    0,
 1868, 1868,    0,    0, 1868, 1868, 1868,    0, 1868, 1868,
 1868, 1868, 1868, 1868, 1868,    0,    0,    0, 1868, 1868,
 1868, 1868, 1868,    0,    0,    0, 1868,    0,    0,    0,
    0,    0,    0,    0, 1868, 1868, 1868, 1868, 1868, 1868,
    0,    0,    0, 1868, 1868,    0, 1868,    0, 1868,    0,
 1868, 1868, 1868,    0, 1868,    0, 1868, 1868, 1868,    0,
 1868, 1868, 1868, 1868, 1868,    0, 1868, 1868,    0,    0,
    0,    0, 1868,    0, 1868, 1868,    0, 1247, 1868, 1868,
 1868,    0,    0, 1868,    0, 1868,    0,    0, 1868, 1868,
 1868, 1868,    0, 1868,    0, 1868,    0, 1868, 1868, 1868,
 1868, 1868,    0,    0, 1868, 1868, 1868,    0, 1868, 1868,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1868,
 1868, 1868,    0, 1868, 1868, 1868,    0, 1868,    0,    0,
    0, 1868,    0, 1868, 1868, 1868, 1868, 2606, 1868,    0,
 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868,    0,    0,
    0,    0,    0, 1249, 1248, 1250,    0, 1868, 1868,    0,
 1868, 1868, 1868,    0,    0,    0,    0, 1868, 1868, 1868,
 1868, 1868,    0, 1868,    0,    0, 1868, 1868,    0,    0,
    0, 1868, 1868,    0,    0, 1868, 1868, 1868, 1868, 1868,
 1868, 1868, 1868, 1868,    0, 1868,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1868, 1868, 1868, 1868, 1868,
 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868,
 1868, 1868, 1868, 1868, 1868, 1868,    0, 1868, 1868, 1868,
 1868, 1868, 1868,    0, 1868, 1868, 1868, 1868,    0, 1868,
    0, 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868,
 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868,    0,
 1868, 1868, 1868, 1868,    0, 1868, 1868, 1868, 1868, 1868,
    0, 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868,
 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868,
 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868, 1868,
 1871, 1868, 1868, 1868, 1868,    0, 1868, 1868, 1868, 1868,
 1868, 1868,    0,    0, 1868, 1868, 1868, 1868, 1868, 1868,
 1868, 1868, 1868,    0,    0,    0,    0,    0, 1868,    0,
 1868, 1868, 1868, 1868, 1868, 1868,    0, 1868,    0,    0,
 1642, 1871, 1871, 1871, 1871, 1871, 1871, 1871,    0,    0,
    0,    0,    0,    0,    0, 1233,    0,    0, 1871, 1871,
 1871, 1871, 1871,    0, 1234,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1235,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1871,    0, 1871,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1236,    0,    0,    0,    0,    0,    0, 1237,    0,
    0,    0,    0,    0,    0,    0, 1238,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1239,    0, 1240,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1241,    0, 1242, 1243, 1244,
    0,    0,    0,    0, 1245,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1246,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1871,    0, 1871,
    0,    0, 1871,    0, 1871, 1871,    0,    0, 1871, 1871,
    0, 1871,    0,    0,    0,    0,    0, 1871,    0, 1871,
 1871, 1871, 1871,    0, 1871,    0,    0, 1871,    0,    0,
 1871,    0,    0,    0,    0,    0,    0, 1871,    0, 1871,
 1871,    0, 1871, 1871, 1871, 1871, 1871,    0,    0, 1871,
    0,    0, 1871,    0, 1871, 1871,    0,    0, 1871, 1871,
 1871,    0, 1871, 1871, 1871, 1871, 1871, 1871, 1871,    0,
    0,    0, 1871, 1871, 1871, 1871, 1871,    0,    0,    0,
 1871,    0,    0,    0,    0,    0,    0,    0, 1871, 1871,
 1871, 1871, 1871, 1871,    0,    0,    0, 1871, 1871,    0,
 1871,    0, 1871,    0, 1871, 1871, 1871,    0, 1871,    0,
 1871, 1871, 1871,    0, 1871, 1871, 1871, 1871, 1871,    0,
 1871, 1871,    0,    0,    0,    0, 1871,    0, 1871, 1871,
    0, 1247, 1871, 1871, 1871,    0,    0, 1871,    0, 1871,
    0,    0, 1871, 1871, 1871, 1871,    0, 1871,    0, 1871,
    0, 1871, 1871, 1871, 1871, 1871,    0,    0, 1871, 1871,
 1871,    0, 1871, 1871,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1871, 1871, 1871,    0, 1871, 1871, 1871,
    0, 1871,    0,    0,    0, 1871,    0, 1871, 1871, 1871,
 1871, 2611, 1871,    0, 1871, 1871, 1871, 1871, 1871, 1871,
 1871, 1871,    0,    0,    0,    0,    0, 1249, 1248, 1250,
    0, 1871, 1871,    0, 1871, 1871, 1871,    0,    0,    0,
    0, 1871, 1871, 1871, 1871, 1871,    0, 1871,    0,    0,
 1871, 1871,    0,    0,    0, 1871, 1871,    0,    0, 1871,
 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871,    0, 1871,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1871,
 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871,
 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871,
    0, 1871, 1871, 1871, 1871, 1871, 1871,    0, 1871, 1871,
 1871, 1871,    0, 1871,    0, 1871, 1871, 1871, 1871, 1871,
 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871,
 1871, 1871, 1871,    0, 1871, 1871, 1871, 1871,    0, 1871,
 1871, 1871, 1871, 1871,    0, 1871, 1871, 1871, 1871, 1871,
 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871,
 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871, 1871,
 1871, 1871, 1871, 1871, 1877, 1871, 1871, 1871, 1871,    0,
 1871, 1871, 1871, 1871, 1871, 1871,    0,    0, 1871, 1871,
 1871, 1871, 1871, 1871, 1871, 1871, 1871,    0,    0,    0,
    0,    0, 1871,    0, 1871, 1871, 1871, 1871, 1871, 1871,
    0, 1871,    0,    0, 1675, 1877, 1877, 1877, 1877, 1877,
 1877, 1877,    0,    0,    0,    0,    0,    0,    0, 1233,
    0,    0, 1877, 1877, 1877, 1877, 1877,    0, 1234,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1235,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1877,    0, 1877,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1236,    0,    0,    0,    0,
    0,    0, 1237,    0,    0,    0,    0,    0,    0,    0,
 1238,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1239,    0, 1240,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1241,
    0, 1242, 1243, 1244,    0,    0,    0,    0, 1245,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1246,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1877,    0, 1877,    0,    0, 1877,    0, 1877, 1877,
    0,    0, 1877, 1877,    0, 1877,    0,    0,    0,    0,
    0, 1877,    0, 1877, 1877, 1877, 1877,    0, 1877,    0,
    0, 1877,    0,    0, 1877,    0,    0,    0,    0,    0,
    0, 1877,    0, 1877, 1877,    0, 1877, 1877, 1877, 1877,
 1877,    0,    0, 1877,    0,    0, 1877,    0, 1877, 1877,
    0,    0, 1877, 1877, 1877,    0, 1877, 1877, 1877, 1877,
 1877, 1877, 1877,    0,    0,    0, 1877, 1877, 1877, 1877,
 1877,    0,    0,    0, 1877,    0,    0,    0,    0,    0,
    0,    0, 1877, 1877, 1877, 1877, 1877, 1877,    0,    0,
    0, 1877, 1877,    0, 1877,    0, 1877,    0, 1877, 1877,
 1877,    0, 1877,    0, 1877, 1877, 1877,    0, 1877, 1877,
 1877, 1877, 1877,    0, 1877, 1877,    0,    0,    0,    0,
 1877,    0, 1877, 1877,    0, 1247, 1877, 1877, 1877,    0,
    0, 1877,    0, 1877,    0,    0, 1877, 1877, 1877, 1877,
    0, 1877,    0, 1877,    0, 1877, 1877, 1877, 1877, 1877,
    0,    0, 1877, 1877, 1877,    0, 1877, 1877,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1877, 1877, 1877,
    0, 1877, 1877, 1877,    0, 1877,    0,    0,    0, 1877,
    0, 1877, 1877, 1877, 1877, 2613, 1877,    0, 1877, 1877,
 1877, 1877, 1877, 1877, 1877, 1877,    0,    0,    0,    0,
    0, 1249, 1248, 1250,    0, 1877, 1877,    0, 1877, 1877,
 1877,    0,    0,    0,    0, 1877, 1877, 1877, 1877, 1877,
    0, 1877,    0,    0, 1877, 1877,    0,    0,    0, 1877,
 1877,    0,    0, 1877, 1877, 1877, 1877, 1877, 1877, 1877,
 1877, 1877,    0, 1877,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1877, 1877, 1877, 1877, 1877, 1877, 1877,
 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1877,
 1877, 1877, 1877, 1877,    0, 1877, 1877, 1877, 1877, 1877,
 1877,    0, 1877, 1877, 1877, 1877,    0, 1877,    0, 1877,
 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1877,
 1877, 1877, 1877, 1877, 1877, 1877, 1877,    0, 1877, 1877,
 1877, 1877,    0, 1877, 1877, 1877, 1877, 1877,    0, 1877,
 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1877,
 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1877,
 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1884, 1877,
 1877, 1877, 1877,    0, 1877, 1877, 1877, 1877, 1877, 1877,
    0,    0, 1877, 1877, 1877, 1877, 1877, 1877, 1877, 1877,
 1877,    0,    0,    0,    0,    0, 1877,    0, 1877, 1877,
 1877, 1877, 1877, 1877,    0, 1877,    0,    0, 1636, 1884,
 1884, 1884, 1884, 1884, 1884, 1884,    0,    0,    0,    0,
    0,    0,    0, 1233,    0,    0, 1884, 1884, 1884, 1884,
 1884,    0, 1234,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1235,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1884,
    0, 1884,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1236,
    0,    0,    0,    0,    0,    0, 1237,    0,    0,    0,
    0,    0,    0,    0, 1238,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1239,    0, 1240,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1241,    0, 1242, 1243, 1244,    0,    0,
    0,    0, 1245,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1246,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1884,    0, 1884,    0,    0,
 1884,    0, 1884, 1884,    0,    0, 1884, 1884,    0, 1884,
    0,    0,    0,    0,    0, 1884,    0, 1884, 1884, 1884,
 1884,    0, 1884,    0,    0, 1884,    0,    0, 1884,    0,
    0,    0,    0,    0,    0, 1884,    0, 1884, 1884,    0,
 1884, 1884, 1884, 1884, 1884,    0,    0, 1884,    0,    0,
 1884,    0, 1884, 1884,    0,    0, 1884, 1884, 1884,    0,
 1884, 1884, 1884, 1884, 1884, 1884, 1884,    0,    0,    0,
 1884, 1884, 1884, 1884, 1884,    0,    0,    0, 1884,    0,
    0,    0,    0,    0,    0,    0, 1884, 1884, 1884, 1884,
 1884, 1884,    0,    0,    0, 1884, 1884,    0, 1884,    0,
 1884,    0, 1884, 1884, 1884,    0, 1884,    0, 1884, 1884,
 1884,    0, 1884, 1884, 1884, 1884, 1884,    0, 1884, 1884,
    0,    0,    0,    0, 1884,    0, 1884, 1884,    0, 1247,
 1884, 1884, 1884,    0,    0, 1884,    0, 1884,    0,    0,
 1884, 1884, 1884, 1884,    0, 1884,    0, 1884,    0, 1884,
 1884, 1884, 1884, 1884,    0,    0, 1884, 1884, 1884,    0,
 1884, 1884,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1884, 1884, 1884,    0, 1884, 1884, 1884,    0, 1884,
    0,    0,    0, 1884,    0, 1884, 1884, 1884, 1884, 2614,
 1884,    0, 1884, 1884, 1884, 1884, 1884, 1884, 1884, 1884,
    0,    0,    0,    0,    0, 1249, 1248, 1250,    0, 1884,
 1884,    0, 1884, 1884, 1884,    0,    0,    0,    0, 1884,
 1884, 1884, 1884, 1884,    0, 1884,    0,    0, 1884, 1884,
    0,    0,    0, 1884, 1884,    0,    0, 1884, 1884, 1884,
 1884, 1884, 1884, 1884, 1884, 1884,    0, 1884,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1884, 1884, 1884,
 1884, 1884, 1884, 1884, 1884, 1884, 1884, 1884, 1884, 1884,
 1884, 1884, 1884, 1884, 1884, 1884, 1884, 1884,    0, 1884,
 1884, 1884, 1884, 1884, 1884,    0, 1884, 1884, 1884, 1884,
    0, 1884,    0, 1884, 1884, 1884, 1884, 1884, 1884, 1884,
 1884, 1884, 1884, 1884, 1884, 1884, 1884, 1884, 1884, 1884,
 1884,    0, 1884, 1884, 1884, 1884,    0, 1884, 1884, 1884,
 1884, 1884,    0, 1884, 1884, 1884, 1884, 1884, 1884, 1884,
 1884, 1884, 1884, 1884, 1884, 1884, 1884, 1884, 1884, 1884,
 1884, 1884, 1884, 1884, 1884, 1884, 1884, 1884, 1884, 1884,
 1884, 1884, 1959, 1884, 1884, 1884, 1884,    0, 1884, 1884,
 1884, 1884, 1884, 1884,    0,    0, 1884, 1884, 1884, 1884,
 1884, 1884, 1884, 1884, 1884,    0,    0,    0,    0,    0,
 1884,    0, 1884, 1884, 1884, 1884, 1884, 1884,    0, 1884,
    0,    0, 1661, 1959, 1959, 1959, 1959, 1959, 1959, 1959,
    0,    0,    0,    0,    0,    0,    0, 1233,    0,    0,
 1959, 1959, 1959, 1959, 1959,    0, 1234,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1235,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1959,    0, 1959,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1236,    0,    0,    0,    0,    0,    0,
 1237,    0,    0,    0,    0,    0,    0,    0, 1238,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1239,    0, 1240,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1241,    0, 1242,
 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1246,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1959,
    0, 1959,    0,    0, 1959,    0, 1959, 1959,    0,    0,
 1959, 1959,    0, 1959,    0,    0,    0,    0,    0, 1959,
    0, 1959, 1959, 1959, 1959,    0, 1959,    0,    0, 1959,
    0,    0, 1959,    0,    0,    0,    0,    0,    0, 1959,
    0, 1959, 1959,    0, 1959, 1959, 1959, 1959, 1959,    0,
    0, 1959,    0,    0, 1959,    0, 1959, 1959,    0,    0,
 1959, 1959, 1959,    0, 1959, 1959, 1959, 1959, 1959, 1959,
 1959,    0,    0,    0, 1959, 1959, 1959, 1959, 1959,    0,
    0,    0, 1959,    0,    0,    0,    0,    0,    0,    0,
 1959, 1959, 1959, 1959, 1959, 1959,    0,    0,    0, 1959,
 1959,    0, 1959,    0, 1959,    0, 1959, 1959, 1959,    0,
 1959,    0, 1959, 1959, 1959,    0, 1959, 1959, 1959, 1959,
 1959,    0, 1959, 1959,    0,    0,    0,    0, 1959,    0,
 1959, 1959,    0, 1247, 1959, 1959, 1959,    0,    0, 1959,
    0, 1959,    0,    0, 1959, 1959, 1959, 1959,    0, 1959,
    0, 1959,    0, 1959, 1959, 1959, 1959, 1959,    0,    0,
 1959, 1959, 1959,    0, 1959, 1959,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1959, 1959, 1959,    0, 1959,
 1959, 1959,    0, 1959,    0,    0,    0, 1959,    0, 1959,
 1959, 1959, 1959, 1470, 1959,    0, 1959, 1959, 1959, 1959,
 1959, 1959, 1959, 1959,    0,    0,    0,    0,    0, 1470,
 1470, 1470,    0, 1959, 1959,    0, 1959, 1959, 1959,    0,
    0,    0,    0, 1959, 1959, 1959, 1959, 1959,    0, 1959,
    0,    0, 1959, 1959,    0,    0,    0, 1959, 1959,    0,
    0, 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959,
    0, 1959,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959,
 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959,
 1959, 1959,    0, 1959, 1959, 1959, 1959, 1959, 1959,    0,
 1959, 1959, 1959, 1959,    0, 1959,    0, 1959, 1959, 1959,
 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959,
 1959, 1959, 1959, 1959, 1959,    0, 1959, 1959, 1959, 1959,
    0, 1959, 1959, 1959, 1959, 1959,    0, 1959, 1959, 1959,
 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959,
 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959,
 1959, 1959, 1959, 1959, 1959, 1959, 1960, 1959, 1959, 1959,
 1959,    0, 1959, 1959, 1959, 1959, 1959, 1959,    0,    0,
 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959, 1959,    0,
    0,    0,    0,    0, 1959,    0, 1959, 1959, 1959, 1959,
 1959, 1959,    0, 1959,    0,    0, 1662, 1960, 1960, 1960,
 1960, 1960, 1960, 1960,    0,    0,    0,    0,    0,    0,
    0, 1470,    0,    0, 1960, 1960, 1960, 1960, 1960,    0,
 1470,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1470,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1960,    0, 1960,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1470,    0,    0,
    0,    0,    0,    0, 1470,    0,    0,    0,    0,    0,
    0,    0, 1470,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1470,
    0, 1470,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1470,    0, 1470, 1470, 1470,    0,    0,    0,    0,
 1470,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1470,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1960,    0, 1960,    0,    0, 1960,    0,
 1960, 1960,    0,    0, 1960, 1960,    0, 1960,    0,    0,
    0,    0,    0, 1960,    0, 1960, 1960, 1960, 1960,    0,
 1960,    0,    0, 1960,    0,    0, 1960,    0,    0,    0,
    0,    0,    0, 1960,    0, 1960, 1960,    0, 1960, 1960,
 1960, 1960, 1960,    0,    0, 1960,    0,    0, 1960,    0,
 1960, 1960,    0,    0, 1960, 1960, 1960,    0, 1960, 1960,
 1960, 1960, 1960, 1960, 1960,    0,    0,    0, 1960, 1960,
 1960, 1960, 1960,    0,    0,    0, 1960,    0,    0,    0,
    0,    0,    0,    0, 1960, 1960, 1960, 1960, 1960, 1960,
    0,    0,    0, 1960, 1960,    0, 1960,    0, 1960,    0,
 1960, 1960, 1960,    0, 1960,    0, 1960, 1960, 1960,    0,
 1960, 1960, 1960, 1960, 1960,    0, 1960, 1960,    0,    0,
    0,    0, 1960,    0, 1960, 1960,    0, 1470, 1960, 1960,
 1960,    0,    0, 1960,    0, 1960,    0,    0, 1960, 1960,
 1960, 1960,    0, 1960,    0, 1960,    0, 1960, 1960, 1960,
 1960, 1960,    0,    0, 1960, 1960, 1960,    0, 1960, 1960,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1960,
 1960, 1960,    0, 1960, 1960, 1960,    0, 1960,    0,    0,
    0, 1960,    0, 1960, 1960, 1960, 1960, 1469, 1960,    0,
 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960,    0,    0,
    0,    0,    0, 1469, 1469, 1469,    0, 1960, 1960,    0,
 1960, 1960, 1960,    0,    0,    0,    0, 1960, 1960, 1960,
 1960, 1960,    0, 1960,    0,    0, 1960, 1960,    0,    0,
    0, 1960, 1960,    0,    0, 1960, 1960, 1960, 1960, 1960,
 1960, 1960, 1960, 1960,    0, 1960,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1960, 1960, 1960, 1960, 1960,
 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960,
 1960, 1960, 1960, 1960, 1960, 1960,    0, 1960, 1960, 1960,
 1960, 1960, 1960,    0, 1960, 1960, 1960, 1960,    0, 1960,
    0, 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960,
 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960,    0,
 1960, 1960, 1960, 1960,    0, 1960, 1960, 1960, 1960, 1960,
    0, 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960,
 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960,
 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960, 1960,
 1945, 1960, 1960, 1960, 1960,    0, 1960, 1960, 1960, 1960,
 1960, 1960,    0,    0, 1960, 1960, 1960, 1960, 1960, 1960,
 1960, 1960, 1960,    0,    0,    0,    0,    0, 1960,    0,
 1960, 1960, 1960, 1960, 1960, 1960,    0, 1960,    0,    0,
 1627, 1945, 1945, 1945, 1945, 1945, 1945, 1945,    0,    0,
    0,    0,    0,    0,    0, 1469,    0,    0, 1945, 1945,
 1945, 1945, 1945,    0, 1469,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1469,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1945,    0, 1945,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1469,    0,    0,    0,    0,    0,    0, 1469,    0,
    0,    0,    0,    0,    0,    0, 1469,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1469,    0, 1469,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1469,    0, 1469, 1469, 1469,
    0,    0,    0,    0, 1469,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1469,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1945,    0, 1945,
    0,    0, 1945,    0, 1945, 1945,    0,    0, 1945, 1945,
    0, 1945,    0,    0,    0,    0,    0, 1945,    0, 1945,
 1945, 1945, 1945,    0, 1945,    0,    0, 1945,    0,    0,
 1945,    0,    0,    0,    0,    0,    0, 1945,    0, 1945,
 1945,    0, 1945, 1945, 1945, 1945, 1945,    0,    0, 1945,
    0,    0, 1945,    0, 1945, 1945,    0,    0, 1945, 1945,
 1945,    0, 1945, 1945, 1945, 1945, 1945, 1945, 1945,    0,
    0,    0, 1945, 1945, 1945, 1945, 1945,    0,    0,    0,
 1945,    0,    0,    0,    0,    0,    0,    0, 1945, 1945,
 1945, 1945, 1945, 1945,    0,    0,    0, 1945, 1945,    0,
 1945,    0, 1945,    0, 1945, 1945, 1945,    0, 1945,    0,
 1945, 1945, 1945,    0, 1945, 1945, 1945, 1945, 1945,    0,
 1945, 1945,    0,    0,    0,    0, 1945,    0, 1945, 1945,
    0, 1469, 1945, 1945, 1945,    0,    0, 1945,    0, 1945,
    0,    0, 1945, 1945, 1945, 1945,    0, 1945,    0, 1945,
    0, 1945, 1945, 1945, 1945, 1945,    0,    0, 1945, 1945,
 1945,    0, 1945, 1945,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1945, 1945, 1945,    0, 1945, 1945, 1945,
    0, 1945,    0,    0,    0, 1945,    0, 1945, 1945, 1945,
 1945,    0, 1945,    0, 1945, 1945, 1945, 1945, 1945, 1945,
 1945, 1945,    0,    0,    0,    0,    0, 1249, 1248, 1250,
    0, 1945, 1945,    0, 1945, 1945, 1945,    0,    0,    0,
    0, 1945, 1945, 1945, 1945, 1945,    0, 1945,    0,    0,
 1945, 1945,    0,    0,    0, 1945, 1945,    0,    0, 1945,
 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945,    0, 1945,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1945,
 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945,
 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945,
    0, 1945, 1945, 1945, 1945, 1945, 1945,    0, 1945, 1945,
 1945, 1945,    0, 1945,    0, 1945, 1945, 1945, 1945, 1945,
 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945,
 1945, 1945, 1945,    0, 1945, 1945, 1945, 1945,    0, 1945,
 1945, 1945, 1945, 1945,    0, 1945, 1945, 1945, 1945, 1945,
 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945,
 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945, 1945,
 1945, 1945, 1945, 1945, 1946, 1945, 1945, 1945, 1945,    0,
 1945, 1945, 1945, 1945, 1945, 1945,    0,    0, 1945, 1945,
 1945, 1945, 1945, 1945, 1945, 1945, 1945,    0,    0,    0,
    0,    0, 1945,    0, 1945, 1945, 1945, 1945, 1945, 1945,
    0, 1945,    0,    0, 1631, 1946, 1946, 1946, 1946, 1946,
 1946, 1946,    0,    0,    0,    0,    0,    0,    0, 1233,
    0,    0, 1946, 1946, 1946, 1946, 1946,    0, 1234,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1235,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1946,    0, 1946,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1236,    0,    0,    0,    0,
    0,    0, 1237,    0,    0,    0,    0,    0,    0,    0,
 1238,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1239,    0, 1240,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1241,
    0, 1242, 1243, 1244,    0,    0,    0,    0, 1245,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1246,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1498, 1499, 1500, 1501, 1502, 1503, 1504, 1505,    0,
    0, 1946,    0, 1946,    0,    0, 1946,    0, 1946, 1946,
    0,    0, 1946, 1946,    0, 1946,    0,    0,    0,    0,
    0, 1946,    0, 1946, 1946, 1946, 1946,    0, 1946,    0,
    0, 1946,    0,    0, 1946,    0,    0,    0,    0,    0,
    0, 1946,    0, 1946, 1946,    0, 1946, 1946, 1946, 1946,
 1946,    0,    0, 1946,    0,    0, 1946,    0, 1946, 1946,
    0,    0, 1946, 1946, 1946,    0, 1946, 1946, 1946, 1946,
 1946, 1946, 1946,    0,    0,    0, 1946, 1946, 1946, 1946,
 1946,    0,    0,    0, 1946,    0,    0,    0,    0,    0,
    0,    0, 1946, 1946, 1946, 1946, 1946, 1946,    0,    0,
 1506, 1946, 1946,    0, 1946,    0, 1946,    0, 1946, 1946,
 1946,    0, 1946,    0, 1946, 1946, 1946,    0, 1946, 1946,
 1946, 1946, 1946,    0, 1946, 1946, 1507,    0,    0,    0,
 1946,    0, 1946, 1946,    0, 1247, 1946, 1946, 1946,    0,
    0, 1946,    0, 1946,    0,    0, 1946, 1946, 1946, 1946,
    0, 1946,    0, 1946,    0, 1946, 1946, 1946, 1946, 1946,
    0,    0, 1946, 1946, 1946,    0, 1946, 1946,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1946, 1946, 1946,
    0, 1946, 1946, 1946,    0, 1946,    0,    0,    0, 1946,
    0, 1946, 1946, 1946, 1946,    0, 1946,    0, 1946, 1946,
 1946, 1946, 1946, 1946, 1946, 1946,    0,    0,    0,    0,
 1249, 1248, 1250,    0,    0, 1946, 1946,    0, 1946, 1946,
 1946,    0,    0,    0,    0, 1946, 1946, 1946, 1946, 1946,
    0, 1946,    0,    0, 1946, 1946, 1249, 1248, 1250, 1946,
 1946,    0,    0, 1946, 1946, 1946, 1946, 1946, 1946, 1946,
 1946, 1946,    0, 1946,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1946, 1946, 1946, 1946, 1946, 1946, 1946,
 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1946,
 1946, 1946, 1946, 1946,    0, 1946, 1946, 1946, 1946, 1946,
 1946,    0, 1946, 1946, 1946, 1946,    0, 1946,    0, 1946,
 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1946,
 1946, 1946, 1946, 1946, 1946, 1946, 1946,    0, 1946, 1946,
 1946, 1946,    0, 1946, 1946, 1946, 1946, 1946,    0, 1946,
 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1946,
 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1946,
 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1947, 1946,
 1946, 1946, 1946,    0, 1946, 1946, 1946, 1946, 1946, 1946,
    0,    0, 1946, 1946, 1946, 1946, 1946, 1946, 1946, 1946,
 1946,    0,    0,    0,    0,    0, 1946,    0, 1946, 1946,
 1946, 1946, 1946, 1946,    0, 1946,    0,    0, 1634, 1947,
 1947, 1947, 1947, 1947, 1947, 1947,    0,    0,    0,    0,
    0,    0, 1233,    0,    0,    0, 1947, 1947, 1947, 1947,
 1947, 1234,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1235,    0,    0,    0, 1233,    0,
    0,    0,    0,    0,    0,    0,    0, 1234,    0, 1947,
    0, 1947,    0,    0,    0,    0,    0,    0,    0,    0,
 1235,    0,    0,    0,    0,    0,    0,    0, 1236,    0,
    0,    0,    0,    0,    0, 1237,    0,    0,    0,    0,
    0,    0,    0, 1238,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1236,    0,    0,    0,    0,    0,
 1239, 1237, 1240,    0,    0,    0,    0,    0,    0, 1238,
    0,    0, 1241,    0, 1242, 1243, 1244,    0,    0,    0,
    0, 1245,    0,    0,    0,    0, 1239,    0, 1240,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1241,    0,
 1242, 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1246,    0,    0,    0,    0,    0,    0, 1701,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1246,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1947,    0, 1947,    0,    0,
 1947,    0, 1947, 1947,    0,    0, 1947, 1947,    0, 1947,
    0,    0,    0,    0,    0, 1947,    0, 1947, 1947, 1947,
 1947,    0, 1947,    0,    0, 1947,    0,    0, 1947,    0,
    0,    0,    0,    0,    0, 1947,    0, 1947, 1947,    0,
 1947, 1947, 1947, 1947, 1947,    0,    0, 1947,    0,    0,
 1947,    0, 1947, 1947,    0,    0, 1947, 1947, 1947,    0,
 1947, 1947, 1947, 1947, 1947, 1947, 1947,    0,    0,    0,
 1947, 1947, 1947, 1947, 1947,    0,    0,    0, 1947,    0,
    0,    0,    0,    0,    0,    0, 1947, 1947, 1947, 1947,
 1947, 1947,    0,    0,    0, 1947, 1947,    0, 1947,    0,
 1947,    0, 1947, 1947, 1947,    0, 1947,    0, 1947, 1947,
 1947,    0, 1947, 1947, 1947, 1947, 1947,    0, 1947, 1947,
    0,    0,    0,    0, 1947, 1985, 1947, 1947, 1247,    0,
 1947, 1947, 1947,    0,    0, 1947,    0, 1947,    0,    0,
 1947, 1947, 1947, 1947,    0, 1947,    0, 1947,    0, 1947,
 1947, 1947, 1947, 1947, 1247,    0, 1947, 1947, 1947,    0,
 1947, 1947,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1947, 1947, 1947,    0, 1947, 1947, 1947,    0, 1947,
    0,    0,    0, 1947,    0, 1947, 1947, 1947, 1947,    0,
 1947,    0, 1947, 1947, 1947, 1947, 1947, 1947, 1947, 1947,
    0,    0,    0,    0, 1249, 1248, 1250,    0,    0, 1947,
 1947,    0, 1947, 1947, 1947, 1249, 1248, 1250,    0, 1947,
 1947, 1947, 1947, 1947,    0, 1947,    0,    0, 1947, 1947,
    0,    0,    0, 1947, 1947,    0,    0, 1947, 1947, 1947,
 1947, 1947, 1947, 1947, 1947, 1947,    0, 1947,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1947, 1947, 1947,
 1947, 1947, 1947, 1947, 1947, 1947, 1947, 1947, 1947, 1947,
 1947, 1947, 1947, 1947, 1947, 1947, 1947, 1947,    0, 1947,
 1947, 1947, 1947, 1947, 1947,    0, 1947, 1947, 1947, 1947,
    0, 1947,    0, 1947, 1947, 1947, 1947, 1947, 1947, 1947,
 1947, 1947, 1947, 1947, 1947, 1947, 1947, 1947, 1947, 1947,
 1947,    0, 1947, 1947, 1947, 1947,    0, 1947, 1947, 1947,
 1947, 1947,    0, 1947, 1947, 1947, 1947, 1947, 1947, 1947,
 1947, 1947, 1947, 1947, 1947, 1947, 1947, 1947, 1947, 1947,
 1947, 1947, 1947, 1947, 1947, 1947, 1947, 1947, 1947, 1947,
 1947, 1947, 1785, 1947, 1947, 1947, 1947,    0, 1947, 1947,
 1947, 1947, 1947, 1947,    0,    0, 1947, 1947, 1947, 1947,
 1947, 1947, 1947, 1947, 1947,    0,    0,    0,    0,    0,
 1947,    0, 1947, 1947, 1947, 1947, 1947, 1947,    0, 1947,
    0,    0,    0, 1785, 1785, 1785, 1785, 1785, 1785, 1785,
    0,    0,    0,    0,    0,    0, 1233,    0,    0,    0,
 1785, 1785, 1785, 1785, 1785, 1234,    0, 1233,    0,    0,
    0,    0,   59,    0,    0,    0, 1234,    0, 1235,    0,
    0,    0,    0,    0,    0,   60,    0,   61,    0, 1235,
    0,    0,    0, 1785,    0, 1785,    0,   62,    0, 2600,
    0,    0,    0,   63,    0,    0,    0,   64,    0,    0,
    0,    0, 1236,   65,    0,    0,   66,    0,    0, 1237,
    0,    0,    0, 1236,    0,    0,   67, 1238,    0,    0,
 1237,    0,    0,    0,    0,    0,    0,    0, 1238,    0,
    0,    0,    0,    0, 1239,    0, 1240,    0,    0,    0,
    0,    0,    0,    0,    0, 1239, 1241, 1240, 1242, 1243,
 1244,    0,    0,    0,    0, 1245,    0, 1241,    0, 1242,
 1243, 1244,    0,    0,    0,    0, 1245,    0,    0,    0,
    0,    0,    0,    0,   68,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1246,    0,    0,    0,
    0,    0,    0,    0,   69,    0,    0, 1246,   70,    0,
    0,    0,    0,   71, 2612,    0,    0,    0,    0,   72,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   73,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1785,
    0, 1785,    0,    0, 1785,    0, 1785, 1785,    0,    0,
 1785, 1785,    0, 1785,    0,    0,    0,    0,    0, 1785,
    0, 1785, 1785, 1785, 1785,    0, 1785,    0,    0, 1785,
    0,    0, 1785,    0,    0,    0,    0,    0,    0, 1785,
    0, 1785, 1785,   74, 1785, 1785, 1785, 1785, 1785,    0,
    0, 1785,    0,    0, 1785,    0, 1785, 1785,    0,    0,
 1785, 1785, 1785,    0, 1785, 1785, 1785, 1785, 1785, 1785,
 1785,    0,    0,    0, 1785, 1785, 1785, 1785, 1785,    0,
    0,    0, 1785,    0,    0,    0,    0,    0,    0,   75,
 1785, 1785, 1785, 1785, 1785, 1785,    0,    0,    0, 1785,
 1785,    0, 1785,    0, 1785,    0, 1785, 1785, 1785,    0,
 1785,    0, 1785, 1785, 1785,    0, 1785, 1785, 1785, 1785,
 1785,    0, 1785, 1785,    0,    0,    0,    0, 1785,    0,
 1785, 1785, 1247,   76, 1785, 1785, 1785,    0,    0, 1785,
    0, 1785,    0, 1247, 1785, 1785, 1785, 1785,    0, 1785,
    0, 1785,   77, 1785, 1785, 1785, 1785, 1785,    0,    0,
 1785, 1785, 1785,    0, 1785, 1785,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1785, 1785, 1785,    0, 1785,
 1785, 1785,    0, 1785,    0,    0,    0, 1785,    0, 1785,
 1785, 1785, 1785,    0, 1785,    0, 1785, 1785, 1785, 1785,
 1785, 1785, 1785, 1785,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1785, 1785,    0, 1785, 1785, 1785,    0,
    0,    0,    0, 1785, 1785, 1785, 1785, 1785,    0, 1785,
    0,    0, 1785, 1785,    0,    0,    0, 1785, 1785,    0,
    0, 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785,
    0, 1785,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785,
 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785,
 1785, 1785,    0, 1785, 1785, 1785, 1785, 1785, 1785,    0,
 1785, 1785, 1785, 1785,    0, 1785,    0, 1785, 1785, 1785,
 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785,
 1785, 1785, 1785, 1785, 1785,    0, 1785, 1785, 1785, 1785,
    0, 1785, 1785, 1785, 1785, 1785,    0, 1785, 1785, 1785,
 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785,
 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785,
 1785, 1785, 1785, 1785, 1785, 1785, 1830, 1785, 1785, 1785,
 1785,    0, 1785, 1785, 1785, 1785, 1785, 1785,    0,    0,
 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785, 1785,    0,
    0,    0,    0,    0, 1785,    0, 1785, 1785, 1785, 1785,
 1785, 1785,    0, 1785,    0,    0,    0, 1830, 1830, 1830,
 1830, 1830, 1830, 1830,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1830, 1830, 1830, 1830, 1830,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  611,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1830,  612, 1830,
    0,    0,    0,    0,    0,    0,    0,    0,  613,    0,
    0,    0,    0,    0,  614,    0,    0,    0,  615,  616,
    0,    0,    0,    0,  617,    0,    0,  618,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  619,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  620,    0,    0,
    0,    0,    0,    0,    0,  621,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  622,    0,    0,    0,  623,
    0,    0,    0,    0,  624,    0,    0,    0,    0,    0,
  625,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  626,  627,
    0,    0,    0, 1830,    0, 1830,    0,    0, 1830,    0,
 1830, 1830,    0,    0, 1830, 1830,    0, 1830,    0,    0,
    0,    0,    0, 1830,    0, 1830, 1830, 1830, 1830,    0,
 1830,    0,    0, 1830,    0,    0, 1830,    0,    0,    0,
    0,    0,    0, 1830,    0, 1830, 1830,    0, 1830, 1830,
 1830, 1830, 1830,    0,  628, 1830,    0,  629, 1830,    0,
 1830, 1830,    0,    0, 1830, 1830, 1830,    0, 1830, 1830,
 1830, 1830, 1830, 1830, 1830,    0,    0,    0, 1830, 1830,
 1830, 1830, 1830,    0,    0,    0, 1830,    0,    0,    0,
    0,    0,    0,    0, 1830, 1830, 1830, 1830, 1830, 1830,
    0,    0,    0, 1830, 1830,    0, 1830,    0, 1830,    0,
 1830, 1830, 1830,    0, 1830,    0, 1830, 1830, 1830,    0,
 1830, 1830, 1830, 1830, 1830,    0, 1830, 1830,    0,    0,
    0,    0, 1830,    0, 1830, 1830,    0,    0, 1830, 1830,
 1830,    0,    0, 1830,    0, 1830,    0,    0, 1830, 1830,
 1830, 1830,    0, 1830,    0, 1830,    0, 1830, 1830, 1830,
 1830, 1830,    0,  630, 1830, 1830, 1830,    0, 1830, 1830,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1830,
 1830, 1830,    0, 1830, 1830, 1830,    0, 1830,    0,    0,
    0, 1830,    0, 1830, 1830, 1830, 1830,    0, 1830,    0,
 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1830, 1830,    0,
 1830, 1830, 1830,    0,    0,    0,    0, 1830, 1830, 1830,
 1830, 1830,    0, 1830,    0,    0, 1830, 1830,    0,    0,
    0, 1830, 1830,    0,    0, 1830, 1830, 1830, 1830, 1830,
 1830, 1830, 1830, 1830,    0, 1830,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1830, 1830, 1830, 1830, 1830,
 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830,
 1830, 1830, 1830, 1830, 1830, 1830,    0, 1830, 1830, 1830,
 1830, 1830, 1830,    0, 1830, 1830, 1830, 1830,    0, 1830,
    0, 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830,
 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830,    0,
 1830, 1830, 1830, 1830,    0, 1830, 1830, 1830, 1830, 1830,
    0, 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830,
 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830,
 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830, 1830,
 1831, 1830, 1830, 1830, 1830,    0, 1830, 1830, 1830, 1830,
 1830, 1830,    0,    0, 1830, 1830, 1830, 1830, 1830, 1830,
 1830, 1830, 1830,    0,    0,    0,    0,    0, 1830,    0,
 1830, 1830, 1830, 1830, 1830, 1830,    0, 1830,    0,   76,
    0, 1831, 1831, 1831, 1831, 1831, 1831, 1831,    0,    0,
    0,    0,    0,    0,   85,    0,    0,    0, 1831, 1831,
 1831, 1831, 1831,    0,   76,    0,    0,    0,    0,    0,
   76,    0,    0,    0,    0,   76,    0,   82,    0,    0,
   76,    0,    0,   76,    0,    0,    0,    0,    0,    0,
    0, 1831,   89, 1831,    0,    0,    0,    0,    0,    0,
    0,    0,   82,    0,    0,    0,    0,    0,   82,    0,
    0,    0,    0,   82,    0,    0,    0,    0,   82,    0,
    0,   82,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   76,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   76,    0,    0,    0,    0,    0,    0,    0,   82,
    0,    0,    0,    0,    0,    0,   76,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   76,    0,    0,    0,    0,   82,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   82,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   82,    0,    0,    0,    0, 1831,    0, 1831,
    0,    0, 1831,    0, 1831, 1831,    0,    0, 1831, 1831,
   76, 1831,    0,   76,    0,    0,    0, 1831,    0, 1831,
 1831, 1831, 1831,    0, 1831,    0,    0, 1831,    0,    0,
 1831,    0,    0,    0,    0,    0,    0, 1831,    0, 1831,
 1831,    0, 1831, 1831, 1831, 1831, 1831,    0,   82, 1831,
    0,   82, 1831,    0, 1831, 1831,    0,    0, 1831, 1831,
 1831,    0, 1831, 1831, 1831, 1831, 1831, 1831, 1831,    0,
    0,    0, 1831, 1831, 1831, 1831, 1831,    0,    0,    0,
 1831,    0,    0,    0,    0,    0,    0,    0, 1831, 1831,
 1831, 1831, 1831, 1831,    0,    0,    0, 1831, 1831,    0,
 1831,    0, 1831,    0, 1831, 1831, 1831,    0, 1831,    0,
 1831, 1831, 1831,    0, 1831, 1831, 1831, 1831, 1831,   76,
 1831, 1831,    0,    0,    0,    0, 1831,    0, 1831, 1831,
    0,    0, 1831, 1831, 1831,    0,    0, 1831,    0, 1831,
    0,    0, 1831, 1831, 1831, 1831,    0, 1831,    0, 1831,
    0, 1831, 1831, 1831, 1831, 1831,    0,   82, 1831, 1831,
 1831,    0, 1831, 1831,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1831, 1831, 1831,    0, 1831, 1831, 1831,
    0, 1831,    0,    0,    0, 1831,    0, 1831, 1831, 1831,
 1831,    0, 1831,    0, 1831, 1831, 1831, 1831, 1831, 1831,
 1831, 1831,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1831, 1831,    0, 1831, 1831, 1831,    0,    0,    0,
    0, 1831, 1831, 1831, 1831, 1831,    0, 1831,    0,    0,
 1831, 1831,    0,    0,    0, 1831, 1831,    0,    0, 1831,
 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831,    0, 1831,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1831,
 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831,
 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831,
    0, 1831, 1831, 1831, 1831, 1831, 1831,    0, 1831, 1831,
 1831, 1831,    0, 1831,    0, 1831, 1831, 1831, 1831, 1831,
 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831,
 1831, 1831, 1831,    0, 1831, 1831, 1831, 1831,    0, 1831,
 1831, 1831, 1831, 1831,    0, 1831, 1831, 1831, 1831, 1831,
 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831,
 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831, 1831,
 1831, 1831, 1831, 1831, 1849, 1831, 1831, 1831, 1831,    0,
 1831, 1831, 1831, 1831, 1831, 1831,    0,    0, 1831, 1831,
 1831, 1831, 1831, 1831, 1831, 1831, 1831,    0,    0,    0,
    0,    0, 1831,    0, 1831, 1831, 1831, 1831, 1831, 1831,
    0, 1831,    0,    0,    0, 1849, 1849, 1849, 1849, 1849,
 1849, 1849,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1849, 1849, 1849, 1849, 1849,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   81,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   82,    0,    0,    0,    0, 1849,   83, 1849,    0,    0,
    0,    0,    0,    0,    0,    0,   84,    0,    0,   85,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   86,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   87,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   88,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   89,    0,    0,
    0,   90,    0,    0,   91,    0,   92,    0,    0,    0,
    0,    0,   93,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   94,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1849,    0, 1849,    0,    0, 1849,    0, 1849, 1849,
    0,    0, 1849, 1849,    0, 1849,    0,    0,    0,    0,
    0, 1849,    0, 1849, 1849, 1849, 1849,    0, 1849,    0,
    0, 1849,    0,    0, 1849,    0,   95,    0,    0,   96,
    0, 1849,    0, 1849, 1849,    0, 1849, 1849, 1849, 1849,
 1849,    0,    0, 1849,    0,    0, 1849,    0, 1849, 1849,
    0,    0, 1849, 1849, 1849,    0, 1849, 1849, 1849, 1849,
 1849, 1849, 1849,    0,    0,    0, 1849, 1849, 1849, 1849,
 1849,    0,   97,    0, 1849,    0,    0,    0,    0,    0,
    0,    0, 1849, 1849, 1849, 1849, 1849, 1849,    0,    0,
    0, 1849, 1849,    0, 1849,    0, 1849,    0, 1849, 1849,
 1849,    0, 1849,    0, 1849, 1849, 1849,    0, 1849, 1849,
 1849, 1849, 1849,    0, 1849, 1849,   98,    0,    0,    0,
 1849,    0, 1849, 1849,    0,    0, 1849, 1849, 1849,    0,
    0, 1849,    0, 1849,    0,   99, 1849, 1849, 1849, 1849,
    0, 1849,    0, 1849,    0, 1849, 1849, 1849, 1849, 1849,
    0,    0, 1849, 1849, 1849,    0, 1849, 1849,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1849, 1849, 1849,
    0, 1849, 1849, 1849,    0, 1849,    0,    0,    0, 1849,
    0, 1849, 1849, 1849, 1849,    0, 1849,    0, 1849, 1849,
 1849, 1849, 1849, 1849, 1849, 1849,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1849, 1849,    0, 1849, 1849,
 1849,    0,    0,    0,    0, 1849, 1849, 1849, 1849, 1849,
    0, 1849,    0,    0, 1849, 1849,    0,    0,    0, 1849,
 1849,    0,    0, 1849, 1849, 1849, 1849, 1849, 1849, 1849,
 1849, 1849,    0, 1849,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1849, 1849, 1849, 1849, 1849, 1849, 1849,
 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1849,
 1849, 1849, 1849, 1849,    0, 1849, 1849, 1849, 1849, 1849,
 1849,    0, 1849, 1849, 1849, 1849,    0, 1849,    0, 1849,
 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1849,
 1849, 1849, 1849, 1849, 1849, 1849, 1849,    0, 1849, 1849,
 1849, 1849,    0, 1849, 1849, 1849, 1849, 1849,    0, 1849,
 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1849,
 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1849,
 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1904, 1849,
 1849, 1849, 1849,    0, 1849, 1849, 1849, 1849, 1849, 1849,
    0,    0, 1849, 1849, 1849, 1849, 1849, 1849, 1849, 1849,
 1849,    0,    0,    0,    0,    0, 1849,    0, 1849, 1849,
 1849, 1849, 1849, 1849,    0, 1849,    0,    0,    0, 1904,
 1904, 1904, 1904, 1904, 1904, 1904,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1904, 1904, 1904,
 1904,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1904,
    0, 1904,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1904,    0, 1904,    0,    0,
 1904,    0, 1904, 1904,    0,    0, 1904, 1904,    0, 1904,
    0,    0,    0,    0,    0, 1904,    0, 1904, 1904, 1904,
 1904,    0, 1904,    0,    0, 1904,    0,    0, 1904,    0,
    0,    0,    0,    0,    0, 1904,    0, 1904, 1904,    0,
 1904, 1904, 1904, 1904, 1904,    0,    0, 1904,    0,    0,
 1904,    0, 1904, 1904,    0,    0, 1904, 1904, 1904,    0,
 1904, 1904, 1904, 1904, 1904, 1904, 1904,    0,    0,    0,
 1904, 1904, 1904, 1904, 1904,    0,    0,    0, 1904,    0,
    0,    0,    0,    0,    0,    0, 1904, 1904, 1904, 1904,
 1904, 1904,    0,    0,    0, 1904, 1904,    0, 1904,    0,
 1904,    0, 1904, 1904, 1904,    0, 1904,    0, 1904, 1904,
 1904,    0, 1904, 1904, 1904, 1904, 1904,    0, 1904, 1904,
    0,    0,    0,    0, 1904,    0, 1904, 1904,    0,    0,
 1904, 1904, 1904,    0,    0, 1904,    0, 1904,    0,    0,
 1904, 1904, 1904, 1904,    0, 1904,    0, 1904,    0, 1904,
 1904, 1904, 1904, 1904,    0,    0, 1904, 1904, 1904,    0,
 1904, 1904,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1904, 1904, 1904,    0, 1904, 1904, 1904,    0, 1904,
    0,    0,    0, 1904,    0, 1904, 1904, 1904, 1904,    0,
 1904,    0, 1904, 1904, 1904, 1904, 1904, 1904, 1904, 1904,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1904,
 1904,    0, 1904, 1904, 1904,    0,    0,    0,    0, 1904,
 1904, 1904, 1904, 1904,    0, 1904,    0,    0, 1904, 1904,
    0,    0,    0, 1904, 1904,    0,    0, 1904, 1904, 1904,
 1904, 1904, 1904, 1904, 1904, 1904,    0, 1904,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1904, 1904, 1904,
 1904, 1904, 1904, 1904, 1904, 1904, 1904, 1904, 1904, 1904,
 1904, 1904, 1904, 1904, 1904, 1904, 1904, 1904,    0, 1904,
 1904, 1904, 1904, 1904, 1904,    0, 1904, 1904, 1904, 1904,
    0, 1904,    0, 1904, 1904, 1904, 1904, 1904, 1904, 1904,
 1904, 1904, 1904, 1904, 1904, 1904, 1904, 1904, 1904, 1904,
 1904,    0, 1904, 1904, 1904, 1904,    0, 1904, 1904, 1904,
 1904, 1904,    0, 1904, 1904, 1904, 1904, 1904, 1904, 1904,
 1904, 1904, 1904, 1904, 1904, 1904, 1904, 1904, 1904, 1904,
 1904, 1904, 1904, 1904, 1904, 1904, 1904, 1904, 1904, 1904,
 1904, 1904, 1930, 1904, 1904, 1904, 1904,    0, 1904, 1904,
 1904, 1904, 1904, 1904,    0,    0, 1904, 1904, 1904, 1904,
 1904, 1904, 1904, 1904, 1904,    0,    0,    0,    0,    0,
 1904,    0, 1904, 1904, 1904, 1904, 1904, 1904,    0, 1904,
    0,    0,    0, 1930, 1930, 1930, 1930, 1930, 1930, 1930,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1930, 1930, 1930, 1930,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1930,    0, 1930,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1930,
    0, 1930,    0,    0, 1930,    0, 1930, 1930,    0,    0,
 1930, 1930,    0, 1930,    0,    0,    0,    0,    0, 1930,
    0, 1930, 1930, 1930, 1930,    0, 1930,    0,    0, 1930,
    0,    0, 1930,    0,    0,    0,    0,    0,    0, 1930,
    0, 1930, 1930,    0, 1930, 1930, 1930, 1930, 1930,    0,
    0, 1930,    0,    0, 1930,    0, 1930, 1930,    0,    0,
 1930, 1930, 1930,    0, 1930, 1930, 1930, 1930, 1930, 1930,
 1930,    0,    0,    0, 1930, 1930, 1930, 1930, 1930,    0,
    0,    0, 1930,    0,    0,    0,    0,    0,    0,    0,
 1930, 1930, 1930, 1930, 1930, 1930,    0,    0,    0, 1930,
 1930,    0, 1930,    0, 1930,    0, 1930, 1930, 1930,    0,
 1930,    0, 1930, 1930, 1930,    0, 1930, 1930, 1930, 1930,
 1930,    0, 1930, 1930,    0,    0,    0,    0, 1930,    0,
 1930, 1930,    0,    0, 1930, 1930, 1930,    0,    0, 1930,
    0, 1930,    0,    0, 1930, 1930, 1930, 1930,    0, 1930,
    0, 1930,    0, 1930, 1930, 1930, 1930, 1930,    0,    0,
 1930, 1930, 1930,    0, 1930, 1930,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1930, 1930, 1930,    0, 1930,
 1930, 1930,    0, 1930,    0,    0,    0, 1930,    0, 1930,
 1930, 1930, 1930,    0, 1930,    0, 1930, 1930, 1930, 1930,
 1930, 1930, 1930, 1930,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1930, 1930,    0, 1930, 1930, 1930,    0,
    0,    0,    0, 1930, 1930, 1930, 1930, 1930,    0, 1930,
    0,    0, 1930, 1930,    0,    0,    0, 1930, 1930,    0,
    0, 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930,
    0, 1930,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930,
 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930,
 1930, 1930,    0, 1930, 1930, 1930, 1930, 1930, 1930,    0,
 1930, 1930, 1930, 1930,    0, 1930,    0, 1930, 1930, 1930,
 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930,
 1930, 1930, 1930, 1930, 1930,    0, 1930, 1930, 1930, 1930,
    0, 1930, 1930, 1930, 1930, 1930,    0, 1930, 1930, 1930,
 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930,
 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930,
 1930, 1930, 1930, 1930, 1930, 1930, 1790, 1930, 1930, 1930,
 1930,    0, 1930, 1930, 1930, 1930, 1930, 1930,    0,    0,
 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930, 1930,    0,
    0,    0,    0,    0, 1930,    0, 1930, 1930, 1930, 1930,
 1930, 1930,    0, 1930,    0,    0,    0, 1790, 1790, 1790,
 1790, 1790, 1790, 1790,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1790, 1790, 1790, 1790,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1790,    0, 1790,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1790,    0, 1790,    0,    0, 1790,    0,
 1790, 1790,    0,    0, 1790, 1790,    0, 1790,    0,    0,
    0,    0,    0, 1790,    0, 1790, 1790, 1790, 1790,    0,
 1790,    0,    0, 1790,    0,    0, 1790,    0,    0,    0,
    0,    0,    0, 1790,    0, 1790, 1790,    0, 1790, 1790,
 1790, 1790, 1790,    0,    0, 1790,    0,    0, 1790,    0,
 1790, 1790,    0,    0, 1790, 1790, 1790,    0, 1790, 1790,
 1790, 1790, 1790, 1790, 1790,    0,    0,    0, 1790, 1790,
 1790, 1790, 1790,    0,    0,    0, 1790,    0,    0,    0,
    0,    0,    0,    0, 1790, 1790, 1790, 1790, 1790, 1790,
    0,    0,    0, 1790, 1790,    0, 1790,    0, 1790,    0,
 1790, 1790, 1790,    0, 1790,    0, 1790, 1790, 1790,    0,
 1790, 1790, 1790, 1790, 1790,    0, 1790, 1790,    0,    0,
    0,    0, 1790,    0, 1790, 1790,    0,    0, 1790, 1790,
 1790,    0,    0, 1790,    0, 1790,    0,    0, 1790, 1790,
 1790, 1790,    0, 1790,    0, 1790,    0, 1790, 1790, 1790,
 1790, 1790,    0,    0, 1790, 1790, 1790,    0, 1790, 1790,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1790,
 1790, 1790,    0, 1790, 1790, 1790,    0, 1790,    0,    0,
    0, 1790,    0, 1790, 1790, 1790, 1790,    0, 1790,    0,
 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1790, 1790,    0,
 1790, 1790, 1790,    0,    0,    0,    0, 1790, 1790, 1790,
 1790, 1790,    0, 1790,    0,    0, 1790, 1790,    0,    0,
    0, 1790, 1790,    0,    0, 1790, 1790, 1790, 1790, 1790,
 1790, 1790, 1790, 1790,    0, 1790,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1790, 1790, 1790, 1790, 1790,
 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790,
 1790, 1790, 1790, 1790, 1790, 1790,    0, 1790, 1790, 1790,
 1790, 1790, 1790,    0, 1790, 1790, 1790, 1790,    0, 1790,
    0, 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790,
 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790,    0,
 1790, 1790, 1790, 1790,    0, 1790, 1790, 1790, 1790, 1790,
    0, 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790,
 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790,
 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790, 1790,
 1786, 1790, 1790, 1790, 1790,    0, 1790, 1790, 1790, 1790,
 1790, 1790,    0,    0, 1790, 1790, 1790, 1790, 1790, 1790,
 1790, 1790, 1790,    0,    0,    0,    0,    0, 1790,    0,
 1790, 1790, 1790, 1790, 1790, 1790,    0, 1790,    0,    0,
    0, 1786, 1786, 1786, 1786, 1786, 1786, 1786,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1786,
 1786, 1786, 1786,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1786,    0, 1786,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1786,    0, 1786,
    0,    0, 1786,    0, 1786, 1786,    0,    0, 1786, 1786,
    0, 1786,    0,    0,    0,    0,    0, 1786,    0, 1786,
 1786, 1786, 1786,    0, 1786,    0,    0, 1786,    0,    0,
 1786,    0,    0,    0,    0,    0,    0, 1786,    0, 1786,
 1786,    0, 1786, 1786, 1786, 1786, 1786,    0,    0, 1786,
    0,    0, 1786,    0, 1786, 1786,    0,    0, 1786, 1786,
 1786,    0, 1786, 1786, 1786, 1786, 1786, 1786, 1786,    0,
    0,    0, 1786, 1786, 1786, 1786, 1786,    0,    0,    0,
 1786,    0,    0,    0,    0,    0,    0,    0, 1786, 1786,
 1786, 1786, 1786, 1786,    0,    0,    0, 1786, 1786,    0,
 1786,    0, 1786,    0, 1786, 1786, 1786,    0, 1786,    0,
 1786, 1786, 1786,    0, 1786, 1786, 1786, 1786, 1786,    0,
 1786, 1786,    0,    0,    0,    0, 1786,    0, 1786, 1786,
    0,    0, 1786, 1786, 1786,    0,    0, 1786,    0, 1786,
    0,    0, 1786, 1786, 1786, 1786,    0, 1786,    0, 1786,
    0, 1786, 1786, 1786, 1786, 1786,    0,    0, 1786, 1786,
 1786,    0, 1786, 1786,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1786, 1786, 1786,    0, 1786, 1786, 1786,
    0, 1786,    0,    0,    0, 1786,    0, 1786, 1786, 1786,
 1786,    0, 1786,    0, 1786, 1786, 1786, 1786, 1786, 1786,
 1786, 1786,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1786, 1786,    0, 1786, 1786, 1786,    0,    0,    0,
    0, 1786, 1786, 1786, 1786, 1786,    0, 1786,    0,    0,
 1786, 1786,    0,    0,    0, 1786, 1786,    0,    0, 1786,
 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786,    0, 1786,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1786,
 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786,
 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786,
    0, 1786, 1786, 1786, 1786, 1786, 1786,    0, 1786, 1786,
 1786, 1786,    0, 1786,    0, 1786, 1786, 1786, 1786, 1786,
 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786,
 1786, 1786, 1786,    0, 1786, 1786, 1786, 1786,    0, 1786,
 1786, 1786, 1786, 1786,    0, 1786, 1786, 1786, 1786, 1786,
 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786,
 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786, 1786,
 1786, 1786, 1786, 1786, 1798, 1786, 1786, 1786, 1786,    0,
 1786, 1786, 1786, 1786, 1786, 1786,    0,    0, 1786, 1786,
 1786, 1786, 1786, 1786, 1786, 1786, 1786,    0,    0,    0,
    0,    0, 1786,    0, 1786, 1786, 1786, 1786, 1786, 1786,
    0, 1786,    0,    0,    0, 1798, 1798, 1798, 1798, 1798,
 1798, 1798,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1798, 1798, 1798, 1798,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1798,    0, 1798,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1798,    0, 1798,    0,    0, 1798,    0, 1798, 1798,
    0,    0, 1798, 1798,    0, 1798,    0,    0,    0,    0,
    0, 1798,    0, 1798, 1798, 1798, 1798,    0, 1798,    0,
    0, 1798,    0,    0, 1798,    0,    0,    0,    0,    0,
    0, 1798,    0, 1798, 1798,    0, 1798, 1798, 1798, 1798,
 1798,    0,    0, 1798,    0,    0, 1798,    0, 1798, 1798,
    0,    0, 1798, 1798, 1798,    0, 1798, 1798, 1798, 1798,
 1798, 1798, 1798,    0,    0,    0, 1798, 1798, 1798, 1798,
 1798,    0,    0,    0, 1798,    0,    0,    0,    0,    0,
    0,    0, 1798, 1798, 1798, 1798, 1798, 1798,    0,    0,
    0, 1798, 1798,    0, 1798,    0, 1798,    0, 1798, 1798,
 1798,    0, 1798,    0, 1798, 1798, 1798,    0, 1798, 1798,
 1798, 1798, 1798,    0, 1798, 1798,    0,    0,    0,    0,
 1798,    0, 1798, 1798,    0,    0, 1798, 1798, 1798,    0,
    0, 1798,    0, 1798,    0,    0, 1798, 1798, 1798, 1798,
    0, 1798,    0, 1798,    0, 1798, 1798, 1798, 1798, 1798,
    0,    0, 1798, 1798, 1798,    0, 1798, 1798,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1798, 1798, 1798,
    0, 1798, 1798, 1798,    0, 1798,    0,    0,    0, 1798,
    0, 1798, 1798, 1798, 1798,    0, 1798,    0, 1798, 1798,
 1798, 1798, 1798, 1798, 1798, 1798,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1798, 1798,    0, 1798, 1798,
 1798,    0,    0,    0,    0, 1798, 1798, 1798, 1798, 1798,
    0, 1798,    0,    0, 1798, 1798,    0,    0,    0, 1798,
 1798,    0,    0, 1798, 1798, 1798, 1798, 1798, 1798, 1798,
 1798, 1798,    0, 1798,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1798, 1798, 1798, 1798, 1798, 1798, 1798,
 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1798,
 1798, 1798, 1798, 1798,    0, 1798, 1798, 1798, 1798, 1798,
 1798,    0, 1798, 1798, 1798, 1798,    0, 1798,    0, 1798,
 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1798,
 1798, 1798, 1798, 1798, 1798, 1798, 1798,    0, 1798, 1798,
 1798, 1798,    0, 1798, 1798, 1798, 1798, 1798,    0, 1798,
 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1798,
 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1798,
 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1803, 1798,
 1798, 1798, 1798,    0, 1798, 1798, 1798, 1798, 1798, 1798,
    0,    0, 1798, 1798, 1798, 1798, 1798, 1798, 1798, 1798,
 1798,    0,    0,    0,    0,    0, 1798,    0, 1798, 1798,
 1798, 1798, 1798, 1798,    0, 1798,    0,    0,    0, 1803,
 1803, 1803, 1803, 1803, 1803, 1803,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1803, 1803, 1803,
 1803,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1803,
    0, 1803,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1803,    0, 1803,    0,    0,
 1803,    0, 1803, 1803,    0,    0, 1803, 1803,    0, 1803,
    0,    0,    0,    0,    0, 1803,    0, 1803, 1803, 1803,
 1803,    0, 1803,    0,    0, 1803,    0,    0, 1803,    0,
    0,    0,    0,    0,    0, 1803,    0, 1803, 1803,    0,
 1803, 1803, 1803, 1803, 1803,    0,    0, 1803,    0,    0,
 1803,    0, 1803, 1803,    0,    0, 1803, 1803, 1803,    0,
 1803, 1803, 1803, 1803, 1803, 1803, 1803,    0,    0,    0,
 1803, 1803, 1803, 1803, 1803,    0,    0,    0, 1803,    0,
    0,    0,    0,    0,    0,    0, 1803, 1803, 1803, 1803,
 1803, 1803,    0,    0,    0, 1803, 1803,    0, 1803,    0,
 1803,    0, 1803, 1803, 1803,    0, 1803,    0, 1803, 1803,
 1803,    0, 1803, 1803, 1803, 1803, 1803,    0, 1803, 1803,
    0,    0,    0,    0, 1803,    0, 1803, 1803,    0,    0,
 1803, 1803, 1803,    0,    0, 1803,    0, 1803,    0,    0,
 1803, 1803, 1803, 1803,    0, 1803,    0, 1803,    0, 1803,
 1803, 1803, 1803, 1803,    0,    0, 1803, 1803, 1803,    0,
 1803, 1803,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1803, 1803, 1803,    0, 1803, 1803, 1803,    0, 1803,
    0,    0,    0, 1803,    0, 1803, 1803, 1803, 1803,    0,
 1803,    0, 1803, 1803, 1803, 1803, 1803, 1803, 1803, 1803,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1803,
 1803,    0, 1803, 1803, 1803,    0,    0,    0,    0, 1803,
 1803, 1803, 1803, 1803,    0, 1803,    0,    0, 1803, 1803,
    0,    0,    0, 1803, 1803,    0,    0, 1803, 1803, 1803,
 1803, 1803, 1803, 1803, 1803, 1803,    0, 1803,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1803, 1803, 1803,
 1803, 1803, 1803, 1803, 1803, 1803, 1803, 1803, 1803, 1803,
 1803, 1803, 1803, 1803, 1803, 1803, 1803, 1803,    0, 1803,
 1803, 1803, 1803, 1803, 1803,    0, 1803, 1803, 1803, 1803,
    0, 1803,    0, 1803, 1803, 1803, 1803, 1803, 1803, 1803,
 1803, 1803, 1803, 1803, 1803, 1803, 1803, 1803, 1803, 1803,
 1803,    0, 1803, 1803, 1803, 1803,    0, 1803, 1803, 1803,
 1803, 1803,    0, 1803, 1803, 1803, 1803, 1803, 1803, 1803,
 1803, 1803, 1803, 1803, 1803, 1803, 1803, 1803, 1803, 1803,
 1803, 1803, 1803, 1803, 1803, 1803, 1803, 1803, 1803, 1803,
 1803, 1803, 1832, 1803, 1803, 1803, 1803,    0, 1803, 1803,
 1803, 1803, 1803, 1803,    0,    0, 1803, 1803, 1803, 1803,
 1803, 1803, 1803, 1803, 1803,    0,    0,    0,    0,    0,
 1803,    0, 1803, 1803, 1803, 1803, 1803, 1803,    0, 1803,
    0,    0,    0, 1832, 1832, 1832, 1832, 1832, 1832, 1832,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1832, 1832, 1832, 1832,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1832,    0, 1832,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1832,
    0, 1832,    0,    0, 1832,    0, 1832, 1832,    0,    0,
 1832, 1832,    0, 1832,    0,    0,    0,    0,    0, 1832,
    0, 1832, 1832, 1832, 1832,    0, 1832,    0,    0, 1832,
    0,    0, 1832,    0,    0,    0,    0,    0,    0, 1832,
    0, 1832, 1832,    0, 1832, 1832, 1832, 1832, 1832,    0,
    0, 1832,    0,    0, 1832,    0, 1832, 1832,    0,    0,
 1832, 1832, 1832,    0, 1832, 1832, 1832, 1832, 1832, 1832,
 1832,    0,    0,    0, 1832, 1832, 1832, 1832, 1832,    0,
    0,    0, 1832,    0,    0,    0,    0,    0,    0,    0,
 1832, 1832, 1832, 1832, 1832, 1832,    0,    0,    0, 1832,
 1832,    0, 1832,    0, 1832,    0, 1832, 1832, 1832,    0,
 1832,    0, 1832, 1832, 1832,    0, 1832, 1832, 1832, 1832,
 1832,    0, 1832, 1832,    0,    0,    0,    0, 1832,    0,
 1832, 1832,    0,    0, 1832, 1832, 1832,    0,    0, 1832,
    0, 1832,    0,    0, 1832, 1832, 1832, 1832,    0, 1832,
    0, 1832,    0, 1832, 1832, 1832, 1832, 1832,    0,    0,
 1832, 1832, 1832,    0, 1832, 1832,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1832, 1832, 1832,    0, 1832,
 1832, 1832,    0, 1832,    0,    0,    0, 1832,    0, 1832,
 1832, 1832, 1832,    0, 1832,    0, 1832, 1832, 1832, 1832,
 1832, 1832, 1832, 1832,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1832, 1832,    0, 1832, 1832, 1832,    0,
    0,    0,    0, 1832, 1832, 1832, 1832, 1832,    0, 1832,
    0,    0, 1832, 1832,    0,    0,    0, 1832, 1832,    0,
    0, 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832,
    0, 1832,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832,
 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832,
 1832, 1832,    0, 1832, 1832, 1832, 1832, 1832, 1832,    0,
 1832, 1832, 1832, 1832,    0, 1832,    0, 1832, 1832, 1832,
 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832,
 1832, 1832, 1832, 1832, 1832,    0, 1832, 1832, 1832, 1832,
    0, 1832, 1832, 1832, 1832, 1832,    0, 1832, 1832, 1832,
 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832,
 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832,
 1832, 1832, 1832, 1832, 1832, 1832, 1838, 1832, 1832, 1832,
 1832,    0, 1832, 1832, 1832, 1832, 1832, 1832,    0,    0,
 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832, 1832,    0,
    0,    0,    0,    0, 1832,    0, 1832, 1832, 1832, 1832,
 1832, 1832,    0, 1832,    0,    0,    0, 1838, 1838, 1838,
 1838, 1838, 1838, 1838,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1838, 1838, 1838, 1838,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1838,    0, 1838,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1838,    0, 1838,    0,    0, 1838,    0,
 1838, 1838,    0,    0, 1838, 1838,    0, 1838,    0,    0,
    0,    0,    0, 1838,    0, 1838, 1838, 1838, 1838,    0,
 1838,    0,    0, 1838,    0,    0, 1838,    0,    0,    0,
    0,    0,    0, 1838,    0, 1838, 1838,    0, 1838, 1838,
 1838, 1838, 1838,    0,    0, 1838,    0,    0, 1838,    0,
 1838, 1838,    0,    0, 1838, 1838, 1838,    0, 1838, 1838,
 1838, 1838, 1838, 1838, 1838,    0,    0,    0, 1838, 1838,
 1838, 1838, 1838,    0,    0,    0, 1838,    0,    0,    0,
    0,    0,    0,    0, 1838, 1838, 1838, 1838, 1838, 1838,
    0,    0,    0, 1838, 1838,    0, 1838,    0, 1838,    0,
 1838, 1838, 1838,    0, 1838,    0, 1838, 1838, 1838,    0,
 1838, 1838, 1838, 1838, 1838,    0, 1838, 1838,    0,    0,
    0,    0, 1838,    0, 1838, 1838,    0,    0, 1838, 1838,
 1838,    0,    0, 1838,    0, 1838,    0,    0, 1838, 1838,
 1838, 1838,    0, 1838,    0, 1838,    0, 1838, 1838, 1838,
 1838, 1838,    0,    0, 1838, 1838, 1838,    0, 1838, 1838,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1838,
 1838, 1838,    0, 1838, 1838, 1838,    0, 1838,    0,    0,
    0, 1838,    0, 1838, 1838, 1838, 1838,    0, 1838,    0,
 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1838, 1838,    0,
 1838, 1838, 1838,    0,    0,    0,    0, 1838, 1838, 1838,
 1838, 1838,    0, 1838,    0,    0, 1838, 1838,    0,    0,
    0, 1838, 1838,    0,    0, 1838, 1838, 1838, 1838, 1838,
 1838, 1838, 1838, 1838,    0, 1838,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1838, 1838, 1838, 1838, 1838,
 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838,
 1838, 1838, 1838, 1838, 1838, 1838,    0, 1838, 1838, 1838,
 1838, 1838, 1838,    0, 1838, 1838, 1838, 1838,    0, 1838,
    0, 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838,
 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838,    0,
 1838, 1838, 1838, 1838,    0, 1838, 1838, 1838, 1838, 1838,
    0, 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838,
 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838,
 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838, 1838,
 1962, 1838, 1838, 1838, 1838,    0, 1838, 1838, 1838, 1838,
 1838, 1838,    0,    0, 1838, 1838, 1838, 1838, 1838, 1838,
 1838, 1838, 1838,    0,    0,    0,    0,    0, 1838,    0,
 1838, 1838, 1838, 1838, 1838, 1838,    0, 1838,    0,    0,
    0, 1962, 1962, 1962, 1962, 1962, 1962, 1962,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1962,
 1962, 1962, 1962,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1962,    0, 1962,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1962,    0, 1962,
    0,    0, 1962,    0, 1962, 1962,    0,    0, 1962, 1962,
    0, 1962,    0,    0,    0,    0,    0, 1962,    0, 1962,
 1962, 1962, 1962,    0, 1962,    0,    0, 1962,    0,    0,
 1962,    0,    0,    0,    0,    0,    0, 1962,    0, 1962,
 1962,    0, 1962, 1962, 1962, 1962, 1962,    0,    0, 1962,
    0,    0, 1962,    0, 1962, 1962,    0,    0, 1962, 1962,
 1962,    0, 1962, 1962, 1962, 1962, 1962, 1962, 1962,    0,
    0,    0, 1962, 1962, 1962, 1962, 1962,    0,    0,    0,
 1962,    0,    0,    0,    0,    0,    0,    0, 1962, 1962,
 1962, 1962, 1962, 1962,    0,    0,    0, 1962, 1962,    0,
 1962,    0, 1962,    0, 1962, 1962, 1962,    0, 1962,    0,
 1962, 1962, 1962,    0, 1962, 1962, 1962, 1962, 1962,    0,
 1962, 1962,    0,    0,    0,    0, 1962,    0, 1962, 1962,
    0,    0, 1962, 1962, 1962,    0,    0, 1962,    0, 1962,
    0,    0, 1962, 1962, 1962, 1962,    0, 1962,    0, 1962,
    0, 1962, 1962, 1962, 1962, 1962,    0,    0, 1962, 1962,
 1962,    0, 1962, 1962,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1962, 1962, 1962,    0, 1962, 1962, 1962,
    0, 1962,    0,    0,    0, 1962,    0, 1962, 1962, 1962,
 1962,    0, 1962,    0, 1962, 1962, 1962, 1962, 1962, 1962,
 1962, 1962,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1962, 1962,    0, 1962, 1962, 1962,    0,    0,    0,
    0, 1962, 1962, 1962, 1962, 1962,    0, 1962,    0,    0,
 1962, 1962,    0,    0,    0, 1962, 1962,    0,    0, 1962,
 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962,    0, 1962,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1962,
 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962,
 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962,
    0, 1962, 1962, 1962, 1962, 1962, 1962,    0, 1962, 1962,
 1962, 1962,    0, 1962,    0, 1962, 1962, 1962, 1962, 1962,
 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962,
 1962, 1962, 1962,    0, 1962, 1962, 1962, 1962,    0, 1962,
 1962, 1962, 1962, 1962,    0, 1962, 1962, 1962, 1962, 1962,
 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962,
 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962, 1962,
 1962, 1962, 1962, 1962, 1963, 1962, 1962, 1962, 1962,    0,
 1962, 1962, 1962, 1962, 1962, 1962,    0,    0, 1962, 1962,
 1962, 1962, 1962, 1962, 1962, 1962, 1962,    0,    0,    0,
    0,    0, 1962,    0, 1962, 1962, 1962, 1962, 1962, 1962,
    0, 1962,    0,    0,    0, 1963, 1963, 1963, 1963, 1963,
 1963, 1963,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1963, 1963, 1963, 1963,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1963,    0, 1963,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1963,    0, 1963,    0,    0, 1963,    0, 1963, 1963,
    0,    0, 1963, 1963,    0, 1963,    0,    0,    0,    0,
    0, 1963,    0, 1963, 1963, 1963, 1963,    0, 1963,    0,
    0, 1963,    0,    0, 1963,    0,    0,    0,    0,    0,
    0, 1963,    0, 1963, 1963,    0, 1963, 1963, 1963, 1963,
 1963,    0,    0, 1963,    0,    0, 1963,    0, 1963, 1963,
    0,    0, 1963, 1963, 1963,    0, 1963, 1963, 1963, 1963,
 1963, 1963, 1963,    0,    0,    0, 1963, 1963, 1963, 1963,
 1963,    0,    0,    0, 1963,    0,    0,    0,    0,    0,
    0,    0, 1963, 1963, 1963, 1963, 1963, 1963,    0,    0,
    0, 1963, 1963,    0, 1963,    0, 1963,    0, 1963, 1963,
 1963,    0, 1963,    0, 1963, 1963, 1963,    0, 1963, 1963,
 1963, 1963, 1963,    0, 1963, 1963,    0,    0,    0,    0,
 1963,    0, 1963, 1963,    0,    0, 1963, 1963, 1963,    0,
    0, 1963,    0, 1963,    0,    0, 1963, 1963, 1963, 1963,
    0, 1963,    0, 1963,    0, 1963, 1963, 1963, 1963, 1963,
    0,    0, 1963, 1963, 1963,    0, 1963, 1963,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1963, 1963, 1963,
    0, 1963, 1963, 1963,    0, 1963,    0,    0,    0, 1963,
    0, 1963, 1963, 1963, 1963,    0, 1963,    0, 1963, 1963,
 1963, 1963, 1963, 1963, 1963, 1963,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1963, 1963,    0, 1963, 1963,
 1963,    0,    0,    0,    0, 1963, 1963, 1963, 1963, 1963,
    0, 1963,    0,    0, 1963, 1963,    0,    0,    0, 1963,
 1963,    0,    0, 1963, 1963, 1963, 1963, 1963, 1963, 1963,
 1963, 1963,    0, 1963,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1963, 1963, 1963, 1963, 1963, 1963, 1963,
 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1963,
 1963, 1963, 1963, 1963,    0, 1963, 1963, 1963, 1963, 1963,
 1963,    0, 1963, 1963, 1963, 1963,    0, 1963,    0, 1963,
 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1963,
 1963, 1963, 1963, 1963, 1963, 1963, 1963,    0, 1963, 1963,
 1963, 1963,    0, 1963, 1963, 1963, 1963, 1963,    0, 1963,
 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1963,
 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1963,
 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1964, 1963,
 1963, 1963, 1963,    0, 1963, 1963, 1963, 1963, 1963, 1963,
    0,    0, 1963, 1963, 1963, 1963, 1963, 1963, 1963, 1963,
 1963,    0,    0,    0,    0,    0, 1963,    0, 1963, 1963,
 1963, 1963, 1963, 1963,    0, 1963,    0,    0,    0, 1964,
 1964, 1964, 1964, 1964, 1964, 1964,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1964, 1964, 1964,
 1964,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1964,
    0, 1964,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1964,    0, 1964,    0,    0,
 1964,    0, 1964, 1964,    0,    0, 1964, 1964,    0, 1964,
    0,    0,    0,    0,    0, 1964,    0, 1964, 1964, 1964,
 1964,    0, 1964,    0,    0, 1964,    0,    0, 1964,    0,
    0,    0,    0,    0,    0, 1964,    0, 1964, 1964,    0,
 1964, 1964, 1964, 1964, 1964,    0,    0, 1964,    0,    0,
 1964,    0, 1964, 1964,    0,    0, 1964, 1964, 1964,    0,
 1964, 1964, 1964, 1964, 1964, 1964, 1964,    0,    0,    0,
 1964, 1964, 1964, 1964, 1964,    0,    0,    0, 1964,    0,
    0,    0,    0,    0,    0,    0, 1964, 1964, 1964, 1964,
 1964, 1964,    0,    0,    0, 1964, 1964,    0, 1964,    0,
 1964,    0, 1964, 1964, 1964,    0, 1964,    0, 1964, 1964,
 1964,    0, 1964, 1964, 1964, 1964, 1964,    0, 1964, 1964,
    0,    0,    0,    0, 1964,    0, 1964, 1964,    0,    0,
 1964, 1964, 1964,    0,    0, 1964,    0, 1964,    0,    0,
 1964, 1964, 1964, 1964,    0, 1964,    0, 1964,    0, 1964,
 1964, 1964, 1964, 1964,    0,    0, 1964, 1964, 1964,    0,
 1964, 1964,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1964, 1964, 1964,    0, 1964, 1964, 1964,    0, 1964,
    0,    0,    0, 1964,    0, 1964, 1964, 1964, 1964,    0,
 1964,    0, 1964, 1964, 1964, 1964, 1964, 1964, 1964, 1964,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1964,
 1964,    0, 1964, 1964, 1964,    0,    0,    0,    0, 1964,
 1964, 1964, 1964, 1964,    0, 1964,    0,    0, 1964, 1964,
    0,    0,    0, 1964, 1964,    0,    0, 1964, 1964, 1964,
 1964, 1964, 1964, 1964, 1964, 1964,    0, 1964,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1964, 1964, 1964,
 1964, 1964, 1964, 1964, 1964, 1964, 1964, 1964, 1964, 1964,
 1964, 1964, 1964, 1964, 1964, 1964, 1964, 1964,    0, 1964,
 1964, 1964, 1964, 1964, 1964,    0, 1964, 1964, 1964, 1964,
    0, 1964,    0, 1964, 1964, 1964, 1964, 1964, 1964, 1964,
 1964, 1964, 1964, 1964, 1964, 1964, 1964, 1964, 1964, 1964,
 1964,    0, 1964, 1964, 1964, 1964,    0, 1964, 1964, 1964,
 1964, 1964,    0, 1964, 1964, 1964, 1964, 1964, 1964, 1964,
 1964, 1964, 1964, 1964, 1964, 1964, 1964, 1964, 1964, 1964,
 1964, 1964, 1964, 1964, 1964, 1964, 1964, 1964, 1964, 1964,
 1964, 1964, 1965, 1964, 1964, 1964, 1964,    0, 1964, 1964,
 1964, 1964, 1964, 1964,    0,    0, 1964, 1964, 1964, 1964,
 1964, 1964, 1964, 1964, 1964,    0,    0,    0,    0,    0,
 1964,    0, 1964, 1964, 1964, 1964, 1964, 1964,    0, 1964,
    0,    0,    0, 1965, 1965, 1965, 1965, 1965, 1965, 1965,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1965, 1965, 1965, 1965,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1965,    0, 1965,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1965,
    0, 1965,    0,    0, 1965,    0, 1965, 1965,    0,    0,
 1965, 1965,    0, 1965,    0,    0,    0,    0,    0, 1965,
    0, 1965, 1965, 1965, 1965,    0, 1965,    0,    0, 1965,
    0,    0, 1965,    0,    0,    0,    0,    0,    0, 1965,
    0, 1965, 1965,    0, 1965, 1965, 1965, 1965, 1965,    0,
    0, 1965,    0,    0, 1965,    0, 1965, 1965,    0,    0,
 1965, 1965, 1965,    0, 1965, 1965, 1965, 1965, 1965, 1965,
 1965,    0,    0,    0, 1965, 1965, 1965, 1965, 1965,    0,
    0,    0, 1965,    0,    0,    0,    0,    0,    0,    0,
 1965, 1965, 1965, 1965, 1965, 1965,    0,    0,    0, 1965,
 1965,    0, 1965,    0, 1965,    0, 1965, 1965, 1965,    0,
 1965,    0, 1965, 1965, 1965,    0, 1965, 1965, 1965, 1965,
 1965,    0, 1965, 1965,    0,    0,    0,    0, 1965,    0,
 1965, 1965,    0,    0, 1965, 1965, 1965,    0,    0, 1965,
    0, 1965,    0,    0, 1965, 1965, 1965, 1965,    0, 1965,
    0, 1965,    0, 1965, 1965, 1965, 1965, 1965,    0,    0,
 1965, 1965, 1965,    0, 1965, 1965,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1965, 1965, 1965,    0, 1965,
 1965, 1965,    0, 1965,    0,    0,    0, 1965,    0, 1965,
 1965, 1965, 1965,    0, 1965,    0, 1965, 1965, 1965, 1965,
 1965, 1965, 1965, 1965,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1965, 1965,    0, 1965, 1965, 1965,    0,
    0,    0,    0, 1965, 1965, 1965, 1965, 1965,    0, 1965,
    0,    0, 1965, 1965,    0,    0,    0, 1965, 1965,    0,
    0, 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965,
    0, 1965,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965,
 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965,
 1965, 1965,    0, 1965, 1965, 1965, 1965, 1965, 1965,    0,
 1965, 1965, 1965, 1965,    0, 1965,    0, 1965, 1965, 1965,
 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965,
 1965, 1965, 1965, 1965, 1965,    0, 1965, 1965, 1965, 1965,
    0, 1965, 1965, 1965, 1965, 1965,    0, 1965, 1965, 1965,
 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965,
 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965,
 1965, 1965, 1965, 1965, 1965, 1965, 1966, 1965, 1965, 1965,
 1965,    0, 1965, 1965, 1965, 1965, 1965, 1965,    0,    0,
 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965, 1965,    0,
    0,    0,    0,    0, 1965,    0, 1965, 1965, 1965, 1965,
 1965, 1965,    0, 1965,    0,    0,    0, 1966, 1966, 1966,
 1966, 1966, 1966, 1966,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1966, 1966, 1966, 1966,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1966,    0, 1966,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1966,    0, 1966,    0,    0, 1966,    0,
 1966, 1966,    0,    0, 1966, 1966,    0, 1966,    0,    0,
    0,    0,    0, 1966,    0, 1966, 1966, 1966, 1966,    0,
 1966,    0,    0, 1966,    0,    0, 1966,    0,    0,    0,
    0,    0,    0, 1966,    0, 1966, 1966,    0, 1966, 1966,
 1966, 1966, 1966,    0,    0, 1966,    0,    0, 1966,    0,
 1966, 1966,    0,    0, 1966, 1966, 1966,    0, 1966, 1966,
 1966, 1966, 1966, 1966, 1966,    0,    0,    0, 1966, 1966,
 1966, 1966, 1966,    0,    0,    0, 1966,    0,    0,    0,
    0,    0,    0,    0, 1966, 1966, 1966, 1966, 1966, 1966,
    0,    0,    0, 1966, 1966,    0, 1966,    0, 1966,    0,
 1966, 1966, 1966,    0, 1966,    0, 1966, 1966, 1966,    0,
 1966, 1966, 1966, 1966, 1966,    0, 1966, 1966,    0,    0,
    0,    0, 1966,    0, 1966, 1966,    0,    0, 1966, 1966,
 1966,    0,    0, 1966,    0, 1966,    0,    0, 1966, 1966,
 1966, 1966,    0, 1966,    0, 1966,    0, 1966, 1966, 1966,
 1966, 1966,    0,    0, 1966, 1966, 1966,    0, 1966, 1966,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1966,
 1966, 1966,    0, 1966, 1966, 1966,    0, 1966,    0,    0,
    0, 1966,    0, 1966, 1966, 1966, 1966,    0, 1966,    0,
 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1966, 1966,    0,
 1966, 1966, 1966,    0,    0,    0,    0, 1966, 1966, 1966,
 1966, 1966,    0, 1966,    0,    0, 1966, 1966,    0,    0,
    0, 1966, 1966,    0,    0, 1966, 1966, 1966, 1966, 1966,
 1966, 1966, 1966, 1966,    0, 1966,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1966, 1966, 1966, 1966, 1966,
 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966,
 1966, 1966, 1966, 1966, 1966, 1966,    0, 1966, 1966, 1966,
 1966, 1966, 1966,    0, 1966, 1966, 1966, 1966,    0, 1966,
    0, 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966,
 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966,    0,
 1966, 1966, 1966, 1966,    0, 1966, 1966, 1966, 1966, 1966,
    0, 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966,
 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966,
 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966, 1966,
 1967, 1966, 1966, 1966, 1966,    0, 1966, 1966, 1966, 1966,
 1966, 1966,    0,    0, 1966, 1966, 1966, 1966, 1966, 1966,
 1966, 1966, 1966,    0,    0,    0,    0,    0, 1966,    0,
 1966, 1966, 1966, 1966, 1966, 1966,    0, 1966,    0,    0,
    0, 1967, 1967, 1967, 1967, 1967, 1967, 1967,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1967,
 1967, 1967, 1967,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1967,    0, 1967,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1967,    0, 1967,
    0,    0, 1967,    0, 1967, 1967,    0,    0, 1967, 1967,
    0, 1967,    0,    0,    0,    0,    0, 1967,    0, 1967,
 1967, 1967, 1967,    0, 1967,    0,    0, 1967,    0,    0,
 1967,    0,    0,    0,    0,    0,    0, 1967,    0, 1967,
 1967,    0, 1967, 1967, 1967, 1967, 1967,    0,    0, 1967,
    0,    0, 1967,    0, 1967, 1967,    0,    0, 1967, 1967,
 1967,    0, 1967, 1967, 1967, 1967, 1967, 1967, 1967,    0,
    0,    0, 1967, 1967, 1967, 1967, 1967,    0,    0,    0,
 1967,    0,    0,    0,    0,    0,    0,    0, 1967, 1967,
 1967, 1967, 1967, 1967,    0,    0,    0, 1967, 1967,    0,
 1967,    0, 1967,    0, 1967, 1967, 1967,    0, 1967,    0,
 1967, 1967, 1967,    0, 1967, 1967, 1967, 1967, 1967,    0,
 1967, 1967,    0,    0,    0,    0, 1967,    0, 1967, 1967,
    0,    0, 1967, 1967, 1967,    0,    0, 1967,    0, 1967,
    0,    0, 1967, 1967, 1967, 1967,    0, 1967,    0, 1967,
    0, 1967, 1967, 1967, 1967, 1967,    0,    0, 1967, 1967,
 1967,    0, 1967, 1967,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1967, 1967, 1967,    0, 1967, 1967, 1967,
    0, 1967,    0,    0,    0, 1967,    0, 1967, 1967, 1967,
 1967,    0, 1967,    0, 1967, 1967, 1967, 1967, 1967, 1967,
 1967, 1967,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1967, 1967,    0, 1967, 1967, 1967,    0,    0,    0,
    0, 1967, 1967, 1967, 1967, 1967,    0, 1967,    0,    0,
 1967, 1967,    0,    0,    0, 1967, 1967,    0,    0, 1967,
 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967,    0, 1967,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1967,
 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967,
 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967,
    0, 1967, 1967, 1967, 1967, 1967, 1967,    0, 1967, 1967,
 1967, 1967,    0, 1967,    0, 1967, 1967, 1967, 1967, 1967,
 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967,
 1967, 1967, 1967,    0, 1967, 1967, 1967, 1967,    0, 1967,
 1967, 1967, 1967, 1967,    0, 1967, 1967, 1967, 1967, 1967,
 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967,
 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967, 1967,
 1967, 1967, 1967, 1967, 1968, 1967, 1967, 1967, 1967,    0,
 1967, 1967, 1967, 1967, 1967, 1967,    0,    0, 1967, 1967,
 1967, 1967, 1967, 1967, 1967, 1967, 1967,    0,    0,    0,
    0,    0, 1967,    0, 1967, 1967, 1967, 1967, 1967, 1967,
    0, 1967,    0,    0,    0, 1968, 1968, 1968, 1968, 1968,
 1968, 1968,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1968, 1968, 1968, 1968,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1968,    0, 1968,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1968,    0, 1968,    0,    0, 1968,    0, 1968, 1968,
    0,    0, 1968, 1968,    0, 1968,    0,    0,    0,    0,
    0, 1968,    0, 1968, 1968, 1968, 1968,    0, 1968,    0,
    0, 1968,    0,    0, 1968,    0,    0,    0,    0,    0,
    0, 1968,    0, 1968, 1968,    0, 1968, 1968, 1968, 1968,
 1968,    0,    0, 1968,    0,    0, 1968,    0, 1968, 1968,
    0,    0, 1968, 1968, 1968,    0, 1968, 1968, 1968, 1968,
 1968, 1968, 1968,    0,    0,    0, 1968, 1968, 1968, 1968,
 1968,    0,    0,    0, 1968,    0,    0,    0,    0,    0,
    0,    0, 1968, 1968, 1968, 1968, 1968, 1968,    0,    0,
    0, 1968, 1968,    0, 1968,    0, 1968,    0, 1968, 1968,
 1968,    0, 1968,    0, 1968, 1968, 1968,    0, 1968, 1968,
 1968, 1968, 1968,    0, 1968, 1968,    0,    0,    0,    0,
 1968,    0, 1968, 1968,    0,    0, 1968, 1968, 1968,    0,
    0, 1968,    0, 1968,    0,    0, 1968, 1968, 1968, 1968,
    0, 1968,    0, 1968,    0, 1968, 1968, 1968, 1968, 1968,
    0,    0, 1968, 1968, 1968,    0, 1968, 1968,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1968, 1968, 1968,
    0, 1968, 1968, 1968,    0, 1968,    0,    0,    0, 1968,
    0, 1968, 1968, 1968, 1968,    0, 1968,    0, 1968, 1968,
 1968, 1968, 1968, 1968, 1968, 1968,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1968, 1968,    0, 1968, 1968,
 1968,    0,    0,    0,    0, 1968, 1968, 1968, 1968, 1968,
    0, 1968,    0,    0, 1968, 1968,    0,    0,    0, 1968,
 1968,    0,    0, 1968, 1968, 1968, 1968, 1968, 1968, 1968,
 1968, 1968,    0, 1968,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1968, 1968, 1968, 1968, 1968, 1968, 1968,
 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1968,
 1968, 1968, 1968, 1968,    0, 1968, 1968, 1968, 1968, 1968,
 1968,    0, 1968, 1968, 1968, 1968,    0, 1968,    0, 1968,
 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1968,
 1968, 1968, 1968, 1968, 1968, 1968, 1968,    0, 1968, 1968,
 1968, 1968,    0, 1968, 1968, 1968, 1968, 1968,    0, 1968,
 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1968,
 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1968,
 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1969, 1968,
 1968, 1968, 1968,    0, 1968, 1968, 1968, 1968, 1968, 1968,
    0,    0, 1968, 1968, 1968, 1968, 1968, 1968, 1968, 1968,
 1968,    0,    0,    0,    0,    0, 1968,    0, 1968, 1968,
 1968, 1968, 1968, 1968,    0, 1968,    0,    0,    0, 1969,
 1969, 1969, 1969, 1969, 1969, 1969,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1969, 1969, 1969,
 1969,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1969,
    0, 1969,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1969,    0, 1969,    0,    0,
 1969,    0, 1969, 1969,    0,    0, 1969, 1969,    0, 1969,
    0,    0,    0,    0,    0, 1969,    0, 1969, 1969, 1969,
 1969,    0, 1969,    0,    0, 1969,    0,    0, 1969,    0,
    0,    0,    0,    0,    0, 1969,    0, 1969, 1969,    0,
 1969, 1969, 1969, 1969, 1969,    0,    0, 1969,    0,    0,
 1969,    0, 1969, 1969,    0,    0, 1969, 1969, 1969,    0,
 1969, 1969, 1969, 1969, 1969, 1969, 1969,    0,    0,    0,
 1969, 1969, 1969, 1969, 1969,    0,    0,    0, 1969,    0,
    0,    0,    0,    0,    0,    0, 1969, 1969, 1969, 1969,
 1969, 1969,    0,    0,    0, 1969, 1969,    0, 1969,    0,
 1969,    0, 1969, 1969, 1969,    0, 1969,    0, 1969, 1969,
 1969,    0, 1969, 1969, 1969, 1969, 1969,    0, 1969, 1969,
    0,    0,    0,    0, 1969,    0, 1969, 1969,    0,    0,
 1969, 1969, 1969,    0,    0, 1969,    0, 1969,    0,    0,
 1969, 1969, 1969, 1969,    0, 1969,    0, 1969,    0, 1969,
 1969, 1969, 1969, 1969,    0,    0, 1969, 1969, 1969,    0,
 1969, 1969,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1969, 1969, 1969,    0, 1969, 1969, 1969,    0, 1969,
    0,    0,    0, 1969,    0, 1969, 1969, 1969, 1969,    0,
 1969,    0, 1969, 1969, 1969, 1969, 1969, 1969, 1969, 1969,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1969,
 1969,    0, 1969, 1969, 1969,    0,    0,    0,    0, 1969,
 1969, 1969, 1969, 1969,    0, 1969,    0,    0, 1969, 1969,
    0,    0,    0, 1969, 1969,    0,    0, 1969, 1969, 1969,
 1969, 1969, 1969, 1969, 1969, 1969,    0, 1969,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1969, 1969, 1969,
 1969, 1969, 1969, 1969, 1969, 1969, 1969, 1969, 1969, 1969,
 1969, 1969, 1969, 1969, 1969, 1969, 1969, 1969,    0, 1969,
 1969, 1969, 1969, 1969, 1969,    0, 1969, 1969, 1969, 1969,
    0, 1969,    0, 1969, 1969, 1969, 1969, 1969, 1969, 1969,
 1969, 1969, 1969, 1969, 1969, 1969, 1969, 1969, 1969, 1969,
 1969,    0, 1969, 1969, 1969, 1969,    0, 1969, 1969, 1969,
 1969, 1969,    0, 1969, 1969, 1969, 1969, 1969, 1969, 1969,
 1969, 1969, 1969, 1969, 1969, 1969, 1969, 1969, 1969, 1969,
 1969, 1969, 1969, 1969, 1969, 1969, 1969, 1969, 1969, 1969,
 1969, 1969, 1556, 1969, 1969, 1969, 1969,    0, 1969, 1969,
 1969, 1969, 1969, 1969,    0,    0, 1969, 1969, 1969, 1969,
 1969, 1969, 1969, 1969, 1969,    0,    0,    0,    0,    0,
 1969,    0, 1969, 1969, 1969, 1969, 1969, 1969,    0, 1969,
    0,    0,    0, 1556, 1556, 1556, 1556, 1556,    0, 1556,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1556, 1556, 1556, 1556,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1556,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1556,
    0, 1556,    0,    0, 1556,    0, 1556, 1556,    0,    0,
 1556, 1556,    0, 1556,    0,    0,    0,    0,    0, 1556,
    0, 1556, 1556, 1556, 1556,    0, 1556,    0,    0, 1556,
    0,    0, 1556,    0,    0,    0,    0,    0,    0, 1556,
    0, 1556, 1556,    0, 1556, 1556, 1556, 1556, 1556,    0,
    0, 1556,    0,    0, 1556,    0, 1556, 1556,    0,    0,
 1556, 1556, 1556,    0, 1556, 1556, 1556, 1556, 1556, 1556,
 1556,    0,    0,    0, 1556, 1556, 1556, 1556, 1556,    0,
    0,    0, 1556,    0,    0,    0,    0,    0,    0,    0,
 1556, 1556, 1556, 1556, 1556, 1556,    0,    0,    0, 1556,
 1556,    0, 1556,    0, 1556,    0, 1556, 1556, 1556,    0,
 1556,    0, 1556, 1556, 1556,    0, 1556, 1556, 1556, 1556,
 1556,    0, 1556, 1556,    0,    0,    0,    0, 1556,    0,
 1556, 1556,    0,    0, 1556, 1556, 1556,    0,    0, 1556,
    0, 1556,    0,    0, 1556, 1556, 1556, 1556,    0, 1556,
    0, 1556,    0, 1556, 1556, 1556, 1556, 1556,    0,    0,
 1556, 1556, 1556,    0, 1556, 1556,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1556, 1556, 1556,    0, 1556,
 1556, 1556,    0, 1556,    0,    0,    0, 1556,    0, 1556,
 1556, 1556, 1556,    0, 1556,    0, 1556, 1556, 1556, 1556,
 1556, 1556, 1556, 1556,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1556, 1556,    0, 1556, 1556, 1556,    0,
    0,    0,    0, 1556, 1556, 1556, 1556, 1556,    0, 1556,
    0,    0, 1556, 1556,    0,    0,    0, 1556, 1556,    0,
    0, 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556,
    0, 1556,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556,
 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556,
 1556, 1556,    0, 1556, 1556, 1556, 1556, 1556, 1556,    0,
 1556, 1556, 1556, 1556,    0, 1556,    0, 1556, 1556, 1556,
 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556,
 1556, 1556, 1556, 1556, 1556,    0, 1556, 1556, 1556, 1556,
    0, 1556, 1556, 1556, 1556, 1556,    0, 1556, 1556, 1556,
 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556,
 1556, 1556, 1556, 1556, 1556, 1556, 1556,    0, 1556, 1556,
 1556, 1556, 1556, 1556, 1556, 1556, 1606, 1556, 1556, 1556,
 1556, 1753, 1556, 1556, 1556, 1556, 1556, 1556,    0,    0,
 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556, 1556,    0,
    0,    0,    0,    0, 1556,    0, 1556, 1556, 1556, 1556,
 1556, 1556,    0, 1556,    0,    0,    0,    0,    0,    0,
    0,    0, 1753, 1753, 1753, 1753, 1753,    0, 1753,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1753, 1753, 1753, 1753,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1753,    0, 1753,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1753,    0,
 1753,    0,    0, 1753,    0, 1753, 1753,    0,    0, 1753,
 1753,    0, 1753,    0,    0,    0,    0,    0, 1753,    0,
 1753, 1753, 1753, 1753,    0, 1753,    0,    0, 1753,    0,
    0, 1753,    0,    0,    0,    0,    0,    0, 1753,    0,
 1753, 1753,    0, 1753, 1753, 1753, 1753, 1753,    0,    0,
 1753,    0,    0, 1753,    0, 1753, 1753,    0,    0, 1753,
 1753, 1753,    0, 1753, 1753, 1753, 1753, 1753, 1753, 1753,
    0,    0,    0, 1753, 1753, 1753, 1753, 1753,    0,    0,
    0, 1753,    0,    0,    0,    0,    0,    0,    0, 1753,
 1753, 1753, 1753, 1753, 1753,    0,    0,    0, 1753, 1753,
    0, 1753,    0, 1753,    0, 1753, 1753, 1753,    0, 1753,
    0, 1753, 1753, 1753,    0, 1753, 1753, 1753, 1753, 1753,
    0, 1753, 1753,    0,    0,    0,    0, 1753,    0, 1753,
 1753,    0,    0, 1753, 1753, 1753,    0,    0, 1753,    0,
 1753,    0,    0, 1753, 1753, 1753, 1753,    0, 1753,    0,
 1753,    0, 1753, 1753, 1753, 1753, 1753,    0,    0, 1753,
 1753, 1753,    0, 1753, 1753,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1753, 1753, 1753,    0, 1753, 1753,
 1753,    0, 1753,    0,    0,    0, 1753,    0, 1753, 1753,
 1753, 1753,    0, 1753,    0, 1753, 1753, 1753, 1753, 1753,
 1753, 1753, 1753,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1753, 1753,    0, 1753, 1753, 1753,    0,    0,
    0,    0, 1753, 1753, 1753, 1753, 1753,    0, 1753,    0,
    0, 1753, 1753,    0,    0,    0, 1753, 1753,    0,    0,
 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753,    0,
 1753,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753,
 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753,
 1753,    0, 1753, 1753, 1753, 1753, 1753, 1753,    0, 1753,
 1753, 1753, 1753,    0, 1753,    0, 1753, 1753, 1753, 1753,
 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753,
 1753, 1753, 1753, 1753,    0, 1753, 1753, 1753, 1753,    0,
 1753, 1753, 1753, 1753, 1753,    0, 1753, 1753, 1753, 1753,
 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753,
 1753, 1753, 1753, 1753, 1753, 1753,    0, 1753, 1753, 1753,
 1753, 1753, 1753, 1753, 1753, 1771, 1753, 1753, 1753, 1753,
    0, 1753, 1753, 1753, 1753, 1753, 1753,    0,    0, 1753,
 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753,    0,    0,
    0,    0,    0, 1753,    0, 1753, 1753, 1753, 1753, 1753,
 1753,    0, 1753,    0,    0,    0, 1771, 1771, 1771, 1771,
 1771, 1767, 1771,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1771, 1771, 1771, 1771,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1771,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1771,    0, 1771,    0,    0, 1771,    0, 1771,
 1771,    0,    0, 1771, 1771,    0, 1771,    0,    0,    0,
    0,    0, 1771,    0, 1771, 1771, 1771, 1771,    0, 1771,
    0,    0, 1771,    0,    0, 1771,    0,    0,    0,    0,
    0,    0, 1771,    0, 1771, 1771,    0, 1771, 1771, 1771,
 1771, 1771,    0,    0, 1771,    0,    0, 1771,    0, 1771,
 1771,    0,    0, 1771, 1771, 1771,    0, 1771, 1771, 1771,
 1771, 1771, 1771, 1771,    0,    0,    0, 1771, 1771, 1771,
 1771, 1771,    0,    0,    0, 1771,    0,    0,    0,    0,
    0,    0,    0, 1771, 1771, 1771, 1771, 1771, 1771,    0,
    0,    0, 1771, 1771,    0, 1771,    0, 1771,    0, 1771,
 1771, 1771,    0, 1771,    0, 1771, 1771, 1771,    0, 1771,
 1771, 1771, 1771, 1771,    0, 1771, 1771,    0,    0,    0,
    0, 1771,    0, 1771, 1771,    0,    0, 1771, 1771, 1771,
    0,    0, 1771,    0, 1771,    0,    0, 1771, 1771, 1771,
 1771,    0, 1771,    0, 1771,    0, 1771, 1771, 1771, 1771,
 1771,    0,    0, 1771, 1771, 1771,    0, 1771, 1771,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1771, 1771,
 1771,    0, 1771, 1771, 1771,    0, 1771,    0,    0,    0,
 1771,    0, 1771, 1771, 1771, 1771,    0, 1771,    0, 1771,
 1771, 1771, 1771, 1771, 1771, 1771, 1771,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1771, 1771,    0, 1771,
 1771, 1771,    0,    0,    0,    0, 1771, 1771, 1771, 1771,
 1771,    0, 1771,    0,    0, 1771, 1771,    0,    0,    0,
 1771, 1771,    0,    0, 1771, 1771, 1771, 1771, 1771, 1771,
 1771, 1771, 1771,    0, 1771,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1771, 1771, 1771, 1771, 1771, 1771,
 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1771,
 1771, 1771, 1771, 1771, 1771,    0, 1771, 1771, 1771, 1771,
 1771, 1771,    0, 1771, 1771, 1771, 1771,    0, 1771,    0,
 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1771,
 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1771,    0, 1771,
 1771, 1771, 1771,    0, 1771, 1771, 1771, 1771, 1771,    0,
 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1771,
 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1771,
    0, 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1771, 1476,
 1771, 1771, 1771, 1771,    0, 1771, 1771, 1771, 1771, 1771,
 1771,    0,    0, 1771, 1771, 1771, 1771, 1771, 1771, 1771,
 1771, 1771,    0,    0,    0,    0,    0, 1771,    0, 1771,
 1771, 1771, 1771, 1771, 1771,    0, 1771,    0,    0,    0,
 1476, 1476, 1476, 1476, 1476,    0, 1476,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1476, 1476,
 1476, 1476,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1476,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1476,    0, 1476,    0,
    0, 1476,    0, 1476, 1476,    0,    0, 1476, 1476,    0,
 1476,    0,    0,    0,    0,    0, 1476,    0, 1476, 1476,
 1476, 1476,    0, 1476,    0,    0, 1476,    0,    0, 1476,
    0,    0,    0,    0,    0,    0, 1476,    0, 1476, 1476,
    0, 1476, 1476, 1476, 1476, 1476,    0,    0, 1476,    0,
    0, 1476,    0, 1476, 1476,    0,    0, 1476, 1476, 1476,
    0, 1476, 1476, 1476, 1476, 1476, 1476, 1476,    0,    0,
    0, 1476, 1476, 1476, 1476, 1476,    0,    0,    0, 1476,
    0,    0,    0,    0,    0,    0,    0, 1476, 1476, 1476,
 1476, 1476, 1476,    0,    0,    0, 1476, 1476,    0, 1476,
    0, 1476,    0, 1476, 1476, 1476,    0, 1476,    0, 1476,
 1476, 1476,    0, 1476, 1476, 1476, 1476, 1476,    0, 1476,
 1476,    0,    0,    0,    0, 1476,    0, 1476, 1476,    0,
    0, 1476, 1476, 1476,    0,    0, 1476,    0, 1476,    0,
    0, 1476, 1476, 1476, 1476,    0, 1476,    0, 1476,    0,
 1476, 1476, 1476, 1476, 1476,    0,    0, 1476, 1476, 1476,
    0, 1476, 1476,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1476, 1476, 1476,    0, 1476, 1476, 1476,    0,
 1476,    0,    0,    0, 1476,    0, 1476, 1476, 1476, 1476,
    0, 1476,    0, 1476, 1476, 1476, 1476, 1476, 1476, 1476,
 1476,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1476, 1476,    0, 1476, 1476, 1476,    0,    0,    0,    0,
 1476, 1476, 1476, 1476, 1476,    0, 1476,    0,    0, 1476,
 1476,    0,    0,    0, 1476, 1476,    0,    0, 1476, 1476,
 1476, 1476, 1476, 1476, 1476, 1476, 1476,    0, 1476,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1476, 1476,
 1476, 1476, 1476, 1476, 1476, 1476, 1476, 1476, 1476, 1476,
 1476, 1476, 1476, 1476, 1476, 1476, 1476, 1476, 1476,    0,
 1476, 1476, 1476, 1476, 1476, 1476,    0, 1476, 1476, 1476,
 1476,    0, 1476,    0, 1476, 1476, 1476, 1476, 1476, 1476,
 1476, 1476, 1476, 1476, 1476, 1476, 1476, 1476, 1476, 1476,
 1476, 1476,    0, 1476, 1476, 1476, 1476,    0, 1476, 1476,
 1476, 1476, 1476,    0, 1476, 1476, 1476, 1476, 1476, 1476,
 1476, 1476, 1476, 1476, 1476, 1476, 1476, 1476, 1476, 1476,
 1476, 1476, 1476, 1476,    0, 1476, 1476, 1476, 1476, 1476,
 1476, 1476, 1476, 1490, 1476, 1476, 1476, 1476,    0, 1476,
 1476, 1476, 1476, 1476, 1476,    0,    0, 1476, 1476, 1476,
 1476, 1476, 1476, 1476, 1476, 1476,    0,    0,    0,    0,
    0, 1476,    0, 1476, 1476, 1476, 1476, 1476, 1476,    0,
 1476,    0,    0,    0, 1490, 1490, 1490, 1490, 1490,    0,
 1490,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1490, 1490, 1490, 1490,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1490,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1490,    0, 1490,    0,    0, 1490,    0, 1490, 1490,    0,
    0, 1490, 1490,    0, 1490,    0,    0,    0,    0,    0,
    0,    0, 1490, 1490, 1490, 1490,    0, 1490,    0,    0,
 1490,    0,    0, 1490,    0,    0,    0,    0,    0,    0,
 1490,    0, 1490, 1490,    0, 1490, 1490, 1490, 1490, 1490,
    0,    0, 1490,    0,    0, 1490,    0, 1490, 1490,    0,
    0, 1490, 1490, 1490,    0, 1490, 1490, 1490, 1490, 1490,
 1490, 1490,    0,    0,    0, 1490, 1490, 1490, 1490, 1490,
    0,    0,    0, 1490,    0,    0,    0,    0,    0,    0,
    0, 1490, 1490, 1490, 1490, 1490, 1490,    0,    0,    0,
 1490, 1490,    0, 1490,    0, 1490,    0, 1490, 1490, 1490,
    0, 1490,    0, 1490, 1490, 1490,    0, 1490, 1490, 1490,
 1490, 1490,    0, 1490, 1490,    0,    0,    0,    0, 1490,
    0, 1490, 1490,    0,    0, 1490, 1490, 1490,    0,    0,
 1490,    0, 1490,    0,    0, 1490, 1490, 1490, 1490,    0,
 1490,    0, 1490,    0, 1490, 1490, 1490, 1490, 1490,    0,
    0, 1490, 1490, 1490,    0, 1490, 1490,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1490, 1490, 1490,    0,
 1490, 1490, 1490,    0, 1490,    0,    0,    0, 1490,    0,
 1490, 1490, 1490, 1490,    0, 1490,    0, 1490, 1490, 1490,
 1490, 1490, 1490, 1490, 1490,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1490, 1490,    0, 1490, 1490, 1490,
    0,    0,    0,    0, 1490, 1490, 1490, 1490, 1490,    0,
 1490,    0,    0, 1490, 1490,    0,    0,    0, 1490, 1490,
    0,    0, 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1490,
 1490,    0, 1490,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1490,
 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1490,
 1490, 1490, 1490,    0, 1490, 1490, 1490, 1490, 1490, 1490,
    0, 1490, 1490, 1490, 1490,    0, 1490,    0, 1490, 1490,
 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1490,
 1490, 1490, 1490, 1490, 1490, 1490,    0, 1490, 1490, 1490,
 1490,    0, 1490, 1490, 1490, 1490, 1490,    0, 1490, 1490,
 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1490,
 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1490,    0, 1490,
 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1488, 1490, 1490,
 1490, 1490,    0, 1490, 1490, 1490, 1490, 1490, 1490,    0,
    0, 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1490, 1490,
    0,    0,    0,    0,    0, 1490,    0, 1490, 1490, 1490,
 1490, 1490, 1490,    0, 1490,    0,    0,    0, 1488, 1488,
 1488, 1488, 1488,    0, 1488,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1488, 1488, 1488, 1488,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1488,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1488,    0, 1488,    0,    0, 1488,
    0, 1488, 1488,    0,    0, 1488, 1488,    0, 1488,    0,
    0,    0,    0,    0,    0,    0, 1488, 1488,    0, 1488,
    0, 1488,    0,    0, 1488,    0,    0, 1488,    0,    0,
    0,    0,    0,    0, 1488,    0, 1488, 1488,    0, 1488,
 1488, 1488, 1488, 1488,    0,    0, 1488,    0,    0, 1488,
    0, 1488, 1488,    0,    0, 1488, 1488, 1488,    0, 1488,
 1488, 1488, 1488, 1488, 1488, 1488,    0,    0,    0, 1488,
 1488, 1488, 1488, 1488,    0,    0,    0, 1488,    0,    0,
    0,    0,    0,    0,    0, 1488, 1488, 1488, 1488, 1488,
 1488,    0,    0,    0, 1488, 1488,    0, 1488,    0, 1488,
    0, 1488, 1488, 1488,    0, 1488,    0, 1488, 1488, 1488,
    0, 1488, 1488, 1488, 1488, 1488,    0, 1488, 1488,    0,
    0,    0,    0, 1488,    0, 1488, 1488,    0,    0, 1488,
 1488, 1488,    0,    0, 1488,    0, 1488,    0,    0, 1488,
 1488, 1488, 1488,    0, 1488,    0, 1488,    0, 1488, 1488,
 1488, 1488, 1488,    0,    0, 1488, 1488, 1488,    0, 1488,
 1488,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1488, 1488, 1488,    0, 1488, 1488, 1488,    0, 1488,    0,
    0,    0, 1488,    0, 1488, 1488, 1488, 1488,    0, 1488,
    0, 1488, 1488, 1488, 1488, 1488, 1488, 1488, 1488,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1488, 1488,
    0, 1488, 1488, 1488,    0,    0,    0,    0, 1488, 1488,
 1488, 1488, 1488,    0, 1488,    0,    0, 1488, 1488,    0,
    0,    0, 1488, 1488,    0,    0, 1488, 1488, 1488, 1488,
 1488, 1488, 1488, 1488, 1488,    0, 1488,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1488, 1488, 1488, 1488,
 1488, 1488, 1488, 1488, 1488, 1488, 1488, 1488, 1488, 1488,
 1488, 1488, 1488, 1488, 1488, 1488, 1488,    0, 1488, 1488,
 1488, 1488, 1488, 1488,    0, 1488, 1488, 1488, 1488,    0,
 1488,    0, 1488, 1488, 1488, 1488, 1488, 1488, 1488, 1488,
 1488, 1488, 1488, 1488, 1488, 1488, 1488, 1488, 1488, 1488,
    0, 1488, 1488, 1488, 1488,    0, 1488, 1488, 1488, 1488,
 1488,    0, 1488, 1488, 1488, 1488, 1488, 1488, 1488, 1488,
 1488, 1488, 1488, 1488, 1488, 1488, 1488, 1488, 1488, 1488,
 1488, 1488,    0, 1488, 1488, 1488, 1488, 1488, 1488, 1488,
 1488, 1487, 1488, 1488, 1488, 1488,    0, 1488, 1488, 1488,
 1488, 1488, 1488,    0,    0, 1488, 1488, 1488, 1488, 1488,
 1488, 1488, 1488, 1488,    0,    0,    0,    0,    0, 1488,
    0, 1488, 1488, 1488, 1488, 1488, 1488,    0, 1488,    0,
    0,    0, 1487, 1487, 1487, 1487, 1487,    0, 1487,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1487, 1487, 1487, 1487,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1487,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1487,    0,
 1487,    0,    0, 1487,    0, 1487, 1487,    0,    0, 1487,
 1487,    0, 1487,    0,    0,    0,    0,    0,    0,    0,
 1487, 1487,    0, 1487,    0, 1487,    0,    0, 1487,    0,
    0, 1487,    0,    0,    0,    0,    0,    0, 1487,    0,
 1487, 1487,    0, 1487, 1487, 1487, 1487, 1487,    0,    0,
 1487,    0,    0, 1487,    0, 1487, 1487,    0,    0, 1487,
 1487, 1487,    0, 1487, 1487, 1487, 1487, 1487, 1487, 1487,
    0,    0,    0, 1487, 1487, 1487, 1487, 1487,    0,    0,
    0, 1487,    0,    0,    0,    0,    0,    0,    0, 1487,
 1487, 1487, 1487, 1487, 1487,    0,    0,    0, 1487, 1487,
    0, 1487,    0, 1487,    0, 1487, 1487, 1487,    0, 1487,
    0, 1487, 1487, 1487,    0, 1487, 1487, 1487, 1487, 1487,
    0, 1487, 1487,    0,    0,    0,    0, 1487,    0, 1487,
 1487,    0,    0, 1487, 1487, 1487,    0,    0, 1487,    0,
 1487,    0,    0, 1487, 1487, 1487, 1487,    0, 1487,    0,
 1487,    0, 1487, 1487, 1487, 1487, 1487,    0,    0, 1487,
 1487, 1487,    0, 1487, 1487,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1487, 1487, 1487,    0, 1487, 1487,
 1487,    0, 1487,    0,    0,    0, 1487,    0, 1487, 1487,
 1487, 1487,    0, 1487,    0, 1487, 1487, 1487, 1487, 1487,
 1487, 1487, 1487,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1487, 1487,    0, 1487, 1487, 1487,    0,    0,
    0,    0, 1487, 1487, 1487, 1487, 1487,    0, 1487,    0,
    0, 1487, 1487,    0,    0,    0, 1487, 1487,    0,    0,
 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487,    0,
 1487,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487,
 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487,
 1487,    0, 1487, 1487, 1487, 1487, 1487, 1487,    0, 1487,
 1487, 1487, 1487,    0, 1487,    0, 1487, 1487, 1487, 1487,
 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487,
 1487, 1487, 1487, 1487,    0, 1487, 1487, 1487, 1487,    0,
 1487, 1487, 1487, 1487, 1487,    0, 1487, 1487, 1487, 1487,
 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487,
 1487, 1487, 1487, 1487, 1487, 1487,    0, 1487, 1487, 1487,
 1487, 1487, 1487, 1487, 1487, 1493, 1487, 1487, 1487, 1487,
    0, 1487, 1487, 1487, 1487, 1487, 1487,    0,    0, 1487,
 1487, 1487, 1487, 1487, 1487, 1487, 1487, 1487,    0,    0,
    0,    0,    0, 1487,    0, 1487, 1487, 1487, 1487, 1487,
 1487,    0, 1487,    0,    0,    0, 1493, 1493, 1493, 1493,
 1493,    0, 1493,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1493, 1493, 1493, 1493,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1493,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1493,    0, 1493,    0,    0, 1493,    0, 1493,
 1493,    0,    0, 1493, 1493,    0, 1493,    0,    0,    0,
    0,    0,    0,    0, 1493, 1493,    0, 1493,    0, 1493,
    0,    0, 1493,    0,    0, 1493,    0,    0,    0,    0,
    0,    0, 1493,    0, 1493, 1493,    0, 1493, 1493, 1493,
 1493, 1493,    0,    0, 1493,    0,    0, 1493,    0, 1493,
 1493,    0,    0, 1493, 1493, 1493,    0, 1493, 1493, 1493,
 1493, 1493, 1493, 1493,    0,    0,    0, 1493, 1493, 1493,
 1493, 1493,    0,    0,    0, 1493,    0,    0,    0,    0,
    0,    0,    0, 1493, 1493, 1493, 1493, 1493, 1493,    0,
    0,    0, 1493, 1493,    0, 1493,    0, 1493,    0, 1493,
 1493, 1493,    0, 1493,    0, 1493, 1493, 1493,    0, 1493,
 1493, 1493, 1493, 1493,    0, 1493, 1493,    0,    0,    0,
    0, 1493,    0, 1493, 1493,    0,    0, 1493, 1493, 1493,
    0,    0, 1493,    0, 1493,    0,    0, 1493, 1493, 1493,
 1493,    0, 1493,    0, 1493,    0, 1493, 1493, 1493, 1493,
 1493,    0,    0, 1493, 1493, 1493,    0, 1493, 1493,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1493, 1493,
 1493,    0, 1493, 1493, 1493,    0, 1493,    0,    0,    0,
 1493,    0, 1493, 1493, 1493, 1493,    0, 1493,    0, 1493,
 1493, 1493, 1493, 1493, 1493, 1493, 1493,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1493, 1493,    0, 1493,
 1493, 1493,    0,    0,    0,    0, 1493, 1493, 1493, 1493,
 1493,    0, 1493,    0,    0, 1493, 1493,    0,    0,    0,
 1493, 1493,    0,    0, 1493, 1493, 1493, 1493, 1493, 1493,
 1493, 1493, 1493,    0, 1493,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1493, 1493, 1493, 1493, 1493, 1493,
 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1493,
 1493, 1493, 1493, 1493, 1493,    0, 1493, 1493, 1493, 1493,
 1493, 1493,    0, 1493, 1493, 1493, 1493,    0, 1493,    0,
 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1493,
 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1493,    0, 1493,
 1493, 1493, 1493,    0, 1493, 1493, 1493, 1493, 1493,    0,
 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1493,
 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1493,
    0, 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1493, 1494,
 1493, 1493, 1493, 1493,    0, 1493, 1493, 1493, 1493, 1493,
 1493,    0,    0, 1493, 1493, 1493, 1493, 1493, 1493, 1493,
 1493, 1493,    0,    0,    0,    0,    0, 1493,    0, 1493,
 1493, 1493, 1493, 1493, 1493,    0, 1493,    0,    0,    0,
 1494, 1494, 1494, 1494, 1494,    0, 1494,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1494, 1494,
 1494, 1494,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1494,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1494,    0, 1494,    0,
    0, 1494,    0, 1494, 1494,    0,    0, 1494, 1494,    0,
 1494,    0,    0,    0,    0,    0,    0,    0, 1494, 1494,
    0, 1494,    0, 1494,    0,    0, 1494,    0,    0, 1494,
    0,    0,    0,    0,    0,    0, 1494,    0, 1494, 1494,
    0, 1494, 1494, 1494, 1494, 1494,    0,    0, 1494,    0,
    0, 1494,    0, 1494, 1494,    0,    0, 1494, 1494, 1494,
    0, 1494, 1494, 1494, 1494, 1494, 1494, 1494,    0,    0,
    0, 1494, 1494, 1494, 1494, 1494,    0,    0,    0, 1494,
    0,    0,    0,    0,    0,    0,    0, 1494, 1494, 1494,
 1494, 1494, 1494,    0,    0,    0, 1494, 1494,    0, 1494,
    0, 1494,    0, 1494, 1494, 1494,    0, 1494,    0, 1494,
 1494, 1494,    0, 1494, 1494, 1494, 1494, 1494,    0, 1494,
 1494,    0,    0,    0,    0, 1494,    0, 1494, 1494,    0,
    0, 1494, 1494, 1494,    0,    0, 1494,    0, 1494,    0,
    0, 1494, 1494, 1494, 1494,    0, 1494,    0, 1494,    0,
 1494, 1494, 1494, 1494, 1494,    0,    0, 1494, 1494, 1494,
    0, 1494, 1494,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1494, 1494, 1494,    0, 1494, 1494, 1494,    0,
 1494,    0,    0,    0, 1494,    0, 1494, 1494, 1494, 1494,
    0, 1494,    0, 1494, 1494, 1494, 1494, 1494, 1494, 1494,
 1494,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1494, 1494,    0, 1494, 1494, 1494,    0,    0,    0,    0,
 1494, 1494, 1494, 1494, 1494,    0, 1494,    0,    0, 1494,
 1494,    0,    0,    0, 1494, 1494,    0,    0, 1494, 1494,
 1494, 1494, 1494, 1494, 1494, 1494, 1494,    0, 1494,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1494, 1494,
 1494, 1494, 1494, 1494, 1494, 1494, 1494, 1494, 1494, 1494,
 1494, 1494, 1494, 1494, 1494, 1494, 1494, 1494, 1494,    0,
 1494, 1494, 1494, 1494, 1494, 1494,    0, 1494, 1494, 1494,
 1494,    0, 1494,    0, 1494, 1494, 1494, 1494, 1494, 1494,
 1494, 1494, 1494, 1494, 1494, 1494, 1494, 1494, 1494, 1494,
 1494, 1494,    0, 1494, 1494, 1494, 1494,    0, 1494, 1494,
 1494, 1494, 1494,    0, 1494, 1494, 1494, 1494, 1494, 1494,
 1494, 1494, 1494, 1494, 1494, 1494, 1494, 1494, 1494, 1494,
 1494, 1494, 1494, 1494,    0, 1494, 1494, 1494, 1494, 1494,
 1494, 1494, 1494, 1489, 1494, 1494, 1494, 1494,    0, 1494,
 1494, 1494, 1494, 1494, 1494,    0,    0, 1494, 1494, 1494,
 1494, 1494, 1494, 1494, 1494, 1494,    0,    0,    0,    0,
    0, 1494,    0, 1494, 1494, 1494, 1494, 1494, 1494,    0,
 1494,    0,    0,    0, 1489,    0, 1489, 1489, 1489,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1489, 1489, 1489, 1489,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1489,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1489,    0, 1489,    0,    0, 1489,    0, 1489, 1489,    0,
    0, 1489, 1489,    0, 1489,    0,    0,    0,    0,    0,
    0,    0, 1489, 1489,    0, 1489,    0, 1489,    0,    0,
 1489,    0,    0, 1489,    0,    0,    0,    0,    0,    0,
 1489,    0, 1489, 1489,    0, 1489, 1489, 1489, 1489, 1489,
    0,    0, 1489,    0,    0, 1489,    0, 1489, 1489,    0,
    0, 1489, 1489, 1489,    0, 1489, 1489, 1489, 1489, 1489,
 1489, 1489,    0,    0,    0, 1489, 1489, 1489, 1489, 1489,
    0,    0,    0, 1489,    0,    0,    0,    0,    0,    0,
    0, 1489, 1489, 1489, 1489, 1489, 1489,    0,    0,    0,
 1489, 1489,    0, 1489,    0, 1489,    0, 1489, 1489, 1489,
    0, 1489,    0, 1489, 1489, 1489,    0, 1489, 1489, 1489,
 1489, 1489,    0, 1489, 1489,    0,    0,    0,    0, 1489,
    0, 1489, 1489,    0,    0, 1489, 1489, 1489,    0,    0,
 1489,    0, 1489,    0,    0, 1489, 1489, 1489, 1489,    0,
 1489,    0, 1489,    0, 1489, 1489, 1489, 1489, 1489,    0,
    0, 1489, 1489, 1489,    0, 1489, 1489,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1489, 1489, 1489,    0,
 1489, 1489, 1489,    0, 1489,    0,    0,    0, 1489,    0,
 1489, 1489, 1489, 1489,    0, 1489,    0, 1489, 1489, 1489,
 1489, 1489, 1489, 1489, 1489,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1489, 1489,    0, 1489, 1489, 1489,
    0,    0,    0,    0, 1489, 1489, 1489, 1489, 1489,    0,
 1489,    0,    0, 1489, 1489,    0,    0,    0, 1489, 1489,
    0,    0, 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1489,
 1489,    0, 1489,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1489,
 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1489,
 1489, 1489, 1489,    0, 1489, 1489, 1489, 1489, 1489, 1489,
    0, 1489, 1489, 1489, 1489,    0, 1489,    0, 1489, 1489,
 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1489,
 1489, 1489, 1489, 1489, 1489, 1489,    0, 1489, 1489, 1489,
 1489,    0, 1489, 1489, 1489, 1489, 1489,    0, 1489, 1489,
 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1489,
 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1489,    0, 1489,
 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1492, 1489, 1489,
 1489, 1489,    0, 1489, 1489, 1489, 1489, 1489, 1489,    0,
    0, 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1489, 1489,
    0,    0,    0,    0,    0, 1489,    0, 1489, 1489, 1489,
 1489, 1489, 1489,    0, 1489,    0,    0,    0, 1492,    0,
 1492, 1492, 1492,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1492, 1492, 1492, 1492,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1492,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1492,    0, 1492,    0,    0, 1492,
    0, 1492, 1492,    0,    0, 1492, 1492,    0, 1492,    0,
    0,    0,    0,    0,    0,    0, 1492, 1492,    0, 1492,
    0, 1492,    0,    0, 1492,    0,    0, 1492,    0,    0,
    0,    0,    0,    0, 1492,    0, 1492, 1492,    0, 1492,
 1492, 1492, 1492, 1492,    0,    0, 1492,    0,    0, 1492,
    0, 1492, 1492,    0,    0, 1492, 1492, 1492,    0, 1492,
 1492, 1492, 1492, 1492, 1492, 1492,    0,    0,    0, 1492,
 1492, 1492, 1492, 1492,    0,    0,    0, 1492,    0,    0,
    0,    0,    0,    0,    0, 1492, 1492, 1492, 1492, 1492,
 1492,    0,    0,    0, 1492, 1492,    0, 1492,    0, 1492,
    0, 1492, 1492, 1492,    0, 1492,    0, 1492, 1492, 1492,
    0, 1492, 1492, 1492, 1492, 1492,    0, 1492, 1492,    0,
    0,    0,    0, 1492,    0, 1492, 1492,    0,    0, 1492,
 1492, 1492,    0,    0, 1492,    0, 1492,    0,    0, 1492,
 1492, 1492, 1492,    0, 1492,    0, 1492,    0, 1492, 1492,
 1492, 1492, 1492,    0,    0, 1492, 1492, 1492,    0, 1492,
 1492,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1492, 1492, 1492,    0, 1492, 1492, 1492,    0, 1492,    0,
    0,    0, 1492,    0, 1492, 1492, 1492, 1492,    0, 1492,
    0, 1492, 1492, 1492, 1492, 1492, 1492, 1492, 1492,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1492, 1492,
    0, 1492, 1492, 1492,    0,    0,    0,    0, 1492, 1492,
 1492, 1492, 1492,    0, 1492,    0,    0, 1492, 1492,    0,
    0,    0, 1492, 1492,    0,    0, 1492, 1492, 1492, 1492,
 1492, 1492, 1492, 1492, 1492,    0, 1492,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1492, 1492, 1492, 1492,
 1492, 1492, 1492, 1492, 1492, 1492, 1492, 1492, 1492, 1492,
 1492, 1492, 1492, 1492, 1492, 1492, 1492,    0, 1492, 1492,
 1492, 1492, 1492, 1492,    0, 1492, 1492, 1492, 1492,    0,
 1492,    0, 1492, 1492, 1492, 1492, 1492, 1492, 1492, 1492,
 1492, 1492, 1492, 1492, 1492, 1492, 1492, 1492, 1492, 1492,
    0, 1492, 1492, 1492, 1492,    0, 1492, 1492, 1492, 1492,
 1492,    0, 1492, 1492, 1492, 1492, 1492, 1492, 1492, 1492,
 1492, 1492, 1492, 1492, 1492, 1492, 1492, 1492, 1492, 1492,
 1492, 1492,    0, 1492, 1492, 1492, 1492, 1492, 1492, 1492,
 1492, 1469, 1492, 1492, 1492, 1492,    0, 1492, 1492, 1492,
 1492, 1492, 1492,    0,    0, 1492, 1492, 1492, 1492, 1492,
 1492, 1492, 1492, 1492,    0,    0,    0,    0,    0, 1492,
    0, 1492, 1492, 1492, 1492, 1492, 1492,    0, 1492,    0,
    0,    0, 1469,    0,    0, 1469,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1469, 1469, 1469, 1469,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1469,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1469,    0,
 1469,    0,    0, 1469,    0, 1469, 1469,    0,    0, 1469,
 1469,    0, 1469,    0,    0,    0,    0,    0,    0,    0,
 1469, 1469,    0, 1469,    0, 1469,    0,    0, 1469,    0,
    0, 1469,    0,    0,    0,    0,    0,    0, 1469,    0,
 1469, 1469,    0, 1469, 1469, 1469, 1469, 1469,    0,    0,
 1469,    0,    0, 1469,    0, 1469, 1469,    0,    0, 1469,
 1469, 1469,    0, 1469, 1469, 1469, 1469, 1469, 1469, 1469,
    0,    0,    0, 1469, 1469, 1469, 1469, 1469,    0,    0,
    0, 1469,    0,    0,    0,    0,    0,    0,    0, 1469,
 1469, 1469, 1469, 1469, 1469,    0,    0,    0, 1469, 1469,
    0, 1469,    0, 1469,    0, 1469, 1469, 1469,    0, 1469,
    0, 1469, 1469, 1469,    0, 1469, 1469, 1469, 1469, 1469,
    0, 1469, 1469,    0,    0,    0,    0, 1469,    0, 1469,
 1469,    0,    0, 1469, 1469, 1469,    0,    0, 1469,    0,
 1469,    0,    0, 1469, 1469, 1469, 1469,    0, 1469,    0,
 1469,    0, 1469, 1469, 1469, 1469, 1469,    0,    0, 1469,
 1469, 1469,    0, 1469, 1469,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1469, 1469, 1469,    0, 1469, 1469,
 1469,    0, 1469,    0,    0,    0, 1469,    0, 1469, 1469,
 1469, 1469,    0, 1469,    0, 1469, 1469, 1469, 1469, 1469,
 1469, 1469, 1469,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1469, 1469,    0, 1469, 1469, 1469,    0,    0,
    0,    0, 1469, 1469, 1469, 1469, 1469,    0, 1469,    0,
    0, 1469, 1469,    0,    0,    0, 1469, 1469,    0,    0,
 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469,    0,
 1469,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469,
 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469,
 1469,    0, 1469, 1469, 1469, 1469, 1469, 1469,    0, 1469,
 1469, 1469, 1469,    0, 1469,    0, 1469, 1469, 1469, 1469,
 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469,
 1469, 1469, 1469, 1469,    0, 1469, 1469, 1469, 1469,    0,
 1469, 1469, 1469, 1469, 1469,    0, 1469, 1469, 1469, 1469,
 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469,
 1469, 1469, 1469, 1469, 1469, 1469,    0, 1469, 1469, 1469,
 1469, 1469, 1469, 1469, 1469, 1374, 1469, 1469, 1469, 1469,
    0, 1469, 1469, 1469, 1469, 1469, 1469,    0,    0, 1469,
 1469, 1469, 1469, 1469, 1469, 1469, 1469, 1469,    0,    0,
    0,    0,    0, 1469,    0, 1469, 1469, 1469, 1469, 1469,
 1469,    0, 1469,    0,    0,    0, 1374,    0,    0, 1374,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1374, 1374, 1374, 1374,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1374,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1374,    0, 1374,    0,    0, 1374,    0, 1374,
 1374,    0,    0, 1374, 1374,    0, 1374,    0,    0,    0,
    0,    0,    0,    0, 1374, 1374,    0, 1374,    0, 1374,
    0,    0, 1374,    0,    0, 1374,    0,    0,    0,    0,
    0,    0, 1374,    0, 1374, 1374,    0, 1374, 1374, 1374,
 1374, 1374,    0,    0, 1374,    0,    0, 1374,    0, 1374,
 1374,    0,    0, 1374, 1374, 1374,    0, 1374, 1374, 1374,
 1374, 1374, 1374, 1374,    0,    0,    0, 1374, 1374, 1374,
 1374, 1374,    0,    0,    0, 1374,    0,    0,    0,    0,
    0,    0,    0, 1374, 1374, 1374, 1374, 1374, 1374,    0,
    0,    0, 1374, 1374,    0, 1374,    0, 1374,    0, 1374,
 1374, 1374,    0, 1374,    0, 1374, 1374, 1374,    0, 1374,
 1374, 1374, 1374, 1374,    0, 1374, 1374,    0,    0,    0,
    0, 1374,    0, 1374, 1374,    0,    0, 1374, 1374, 1374,
    0,    0, 1374,    0, 1374,    0,    0, 1374, 1374, 1374,
 1374,    0, 1374,    0, 1374,    0, 1374, 1374, 1374, 1374,
 1374,    0,    0, 1374, 1374, 1374,    0, 1374, 1374,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1374, 1374,
 1374,    0, 1374, 1374, 1374,    0, 1374,    0,    0,    0,
 1374,    0, 1374, 1374, 1374, 1374,    0, 1374,    0, 1374,
 1374, 1374, 1374, 1374, 1374, 1374, 1374,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1374, 1374,    0, 1374,
 1374, 1374,    0,    0,    0,    0, 1374, 1374, 1374, 1374,
 1374,    0, 1374,    0,    0, 1374, 1374,    0,    0,    0,
 1374, 1374,    0,    0, 1374, 1374, 1374, 1374, 1374, 1374,
 1374, 1374, 1374,    0, 1374,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1374, 1374, 1374, 1374, 1374, 1374,
 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1374,
 1374, 1374, 1374, 1374, 1374,    0, 1374, 1374, 1374, 1374,
 1374, 1374,    0, 1374, 1374, 1374, 1374,    0, 1374,    0,
 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1374,
 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1374,    0, 1374,
 1374, 1374, 1374,    0, 1374, 1374, 1374, 1374, 1374,    0,
 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1374,
 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1374,
    0, 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1374, 1357,
 1374, 1374, 1374, 1374,    0, 1374, 1374, 1374, 1374, 1374,
 1374,    0,    0, 1374, 1374, 1374, 1374, 1374, 1374, 1374,
 1374, 1374,    0,    0,    0,    0,    0, 1374,    0, 1374,
 1374, 1374, 1374, 1374, 1374,    0, 1374,    0,    0,    0,
 1357,    0,    0, 1357,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1357, 1357,
 1357, 1357,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1357,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1357,    0, 1357,    0,
    0, 1357,    0, 1357, 1357,    0,    0, 1357, 1357,    0,
 1357,    0,    0,    0,    0,    0,    0,    0, 1357, 1357,
    0, 1357,    0, 1357,    0,    0, 1357,    0,    0, 1357,
    0,    0,    0,    0,    0,    0, 1357,    0, 1357, 1357,
    0, 1357, 1357, 1357, 1357, 1357,    0,    0, 1357,    0,
    0, 1357,    0, 1357, 1357,    0,    0, 1357, 1357, 1357,
    0, 1357, 1357, 1357, 1357, 1357, 1357, 1357,    0,    0,
    0, 1357,    0, 1357, 1357, 1357,    0,    0,    0, 1357,
    0,    0,    0,    0,    0,    0,    0, 1357, 1357, 1357,
 1357, 1357, 1357,    0,    0,    0, 1357, 1357,    0, 1357,
    0, 1357,    0, 1357, 1357, 1357,    0, 1357,    0, 1357,
 1357, 1357,    0, 1357, 1357, 1357, 1357, 1357,    0, 1357,
 1357,    0,    0,    0,    0, 1357,    0, 1357, 1357,    0,
    0, 1357, 1357, 1357,    0,    0, 1357,    0, 1357,    0,
    0, 1357, 1357, 1357, 1357,    0, 1357,    0, 1357,    0,
 1357, 1357, 1357, 1357, 1357,    0,    0, 1357, 1357, 1357,
    0, 1357, 1357,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1357, 1357, 1357,    0, 1357, 1357, 1357,    0,
 1357,    0,    0,    0, 1357,    0, 1357, 1357, 1357, 1357,
    0, 1357,    0, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
 1357,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1357, 1357,    0, 1357, 1357, 1357,    0,    0,    0,    0,
 1357, 1357, 1357, 1357, 1357,    0, 1357,    0,    0, 1357,
 1357,    0,    0,    0, 1357, 1357,    0,    0, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357,    0, 1357,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,    0,
 1357, 1357, 1357, 1357, 1357, 1357,    0, 1357, 1357, 1357,
 1357,    0, 1357,    0, 1357, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
 1357, 1357,    0, 1357, 1357, 1357, 1357,    0, 1357, 1357,
 1357, 1357, 1357,    0, 1357, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1357,    0, 1357, 1357, 1357, 1357, 1357,
 1357, 1357, 1357, 1376, 1357, 1357, 1357, 1357,    0, 1357,
 1357, 1357, 1357, 1357, 1357,    0,    0, 1357, 1357, 1357,
 1357, 1357, 1357, 1357, 1357, 1357,    0,    0,    0,    0,
    0, 1357,    0, 1357, 1357, 1357, 1357, 1357, 1357,    0,
 1357,    0,    0,    0, 1376,    0,    0, 1376,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1376, 1376, 1376, 1376,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1376,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1376,    0, 1376,    0,    0, 1376,    0, 1376, 1376,    0,
    0, 1376, 1376,    0, 1376,    0,    0,    0,    0,    0,
    0,    0, 1376, 1376,    0, 1376,    0, 1376,    0,    0,
 1376,    0,    0, 1376,    0,    0,    0,    0,    0,    0,
 1376,    0, 1376, 1376,    0, 1376, 1376, 1376, 1376, 1376,
    0,    0, 1376,    0,    0, 1376,    0, 1376, 1376,    0,
    0, 1376, 1376, 1376,    0, 1376, 1376, 1376, 1376, 1376,
 1376, 1376,    0,    0,    0, 1376,    0, 1376, 1376, 1376,
    0,    0,    0, 1376,    0,    0,    0,    0,    0,    0,
    0, 1376, 1376, 1376, 1376, 1376, 1376,    0,    0,    0,
 1376, 1376,    0, 1376,    0, 1376,    0, 1376, 1376, 1376,
    0, 1376,    0, 1376, 1376, 1376,    0, 1376, 1376, 1376,
 1376, 1376,    0, 1376, 1376,    0,    0,    0,    0, 1376,
    0, 1376, 1376,    0,    0, 1376, 1376, 1376,    0,    0,
 1376,    0, 1376,    0,    0, 1376, 1376, 1376, 1376,    0,
 1376,    0, 1376,    0, 1376, 1376, 1376, 1376, 1376,    0,
    0, 1376, 1376, 1376,    0, 1376, 1376,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1376, 1376, 1376,    0,
 1376, 1376, 1376,    0, 1376,    0,    0,    0, 1376,    0,
 1376, 1376, 1376, 1376,    0, 1376,    0, 1376, 1376, 1376,
 1376, 1376, 1376, 1376, 1376,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1376, 1376,    0, 1376, 1376, 1376,
    0,    0,    0,    0, 1376, 1376, 1376, 1376, 1376,    0,
 1376,    0,    0, 1376, 1376,    0,    0,    0, 1376, 1376,
    0,    0, 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1376,
 1376,    0, 1376,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1376,
 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1376,
 1376, 1376, 1376,    0, 1376, 1376, 1376, 1376, 1376, 1376,
    0, 1376, 1376, 1376, 1376,    0, 1376,    0, 1376, 1376,
 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1376,
 1376, 1376, 1376, 1376, 1376, 1376,    0, 1376, 1376, 1376,
 1376,    0, 1376, 1376, 1376, 1376, 1376,    0, 1376, 1376,
 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1376,
 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1376,    0, 1376,
 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1377, 1376, 1376,
 1376, 1376,    0, 1376, 1376, 1376, 1376, 1376, 1376,    0,
    0, 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1376, 1376,
    0,    0,    0,    0,    0, 1376,    0, 1376, 1376, 1376,
 1376, 1376, 1376,    0, 1376,    0,    0,    0, 1377,    0,
    0, 1377,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1377, 1377, 1377, 1377,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1377,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1377,    0, 1377,    0,    0, 1377,
    0, 1377, 1377,    0,    0, 1377, 1377,    0, 1377,    0,
    0,    0,    0,    0,    0,    0, 1377, 1377,    0, 1377,
    0, 1377,    0,    0, 1377,    0,    0, 1377,    0,    0,
    0,    0,    0,    0, 1377,    0, 1377, 1377,    0, 1377,
 1377, 1377, 1377, 1377,    0,    0, 1377,    0,    0, 1377,
    0, 1377, 1377,    0,    0, 1377, 1377, 1377,    0, 1377,
 1377, 1377, 1377, 1377, 1377, 1377,    0,    0,    0, 1377,
    0, 1377, 1377, 1377,    0,    0,    0, 1377,    0,    0,
    0,    0,    0,    0,    0, 1377, 1377, 1377, 1377, 1377,
 1377,    0,    0,    0, 1377, 1377,    0, 1377,    0, 1377,
    0, 1377, 1377, 1377,    0, 1377,    0, 1377, 1377, 1377,
    0, 1377, 1377, 1377, 1377, 1377,    0, 1377, 1377,    0,
    0,    0,    0, 1377,    0, 1377, 1377,    0,    0, 1377,
 1377, 1377,    0,    0, 1377,    0, 1377,    0,    0, 1377,
 1377, 1377, 1377,    0, 1377,    0, 1377,    0, 1377, 1377,
 1377, 1377, 1377,    0,    0, 1377, 1377, 1377,    0, 1377,
 1377,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1377, 1377, 1377,    0, 1377, 1377, 1377,    0, 1377,    0,
    0,    0, 1377,    0, 1377, 1377, 1377, 1377,    0, 1377,
    0, 1377, 1377, 1377, 1377, 1377, 1377, 1377, 1377,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1377, 1377,
    0, 1377, 1377, 1377,    0,    0,    0,    0, 1377, 1377,
 1377, 1377, 1377,    0, 1377,    0,    0, 1377, 1377,    0,
    0,    0, 1377, 1377,    0,    0, 1377, 1377, 1377, 1377,
 1377, 1377, 1377, 1377, 1377,    0, 1377,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1377, 1377, 1377, 1377,
 1377, 1377, 1377, 1377, 1377, 1377, 1377, 1377, 1377, 1377,
 1377, 1377, 1377, 1377, 1377, 1377, 1377,    0, 1377, 1377,
 1377, 1377, 1377, 1377,    0, 1377, 1377, 1377, 1377,    0,
 1377,    0, 1377, 1377, 1377, 1377, 1377, 1377, 1377, 1377,
 1377, 1377, 1377, 1377, 1377, 1377, 1377, 1377, 1377, 1377,
    0, 1377, 1377, 1377, 1377,    0, 1377, 1377, 1377, 1377,
 1377,    0, 1377, 1377, 1377, 1377, 1377, 1377, 1377, 1377,
 1377, 1377, 1377, 1377, 1377, 1377, 1377, 1377, 1377, 1377,
 1377, 1377,    0, 1377, 1377, 1377, 1377, 1377, 1377, 1377,
 1377, 1382, 1377, 1377, 1377, 1377,    0, 1377, 1377, 1377,
 1377, 1377, 1377,    0,    0, 1377, 1377, 1377, 1377, 1377,
 1377, 1377, 1377, 1377,    0,    0,    0,    0,    0, 1377,
    0, 1377, 1377, 1377, 1377, 1377, 1377,    0, 1377,    0,
    0,    0, 1382,    0,    0, 1382,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1382, 1382, 1382, 1382,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1382,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1382,    0,
 1382,    0,    0, 1382,    0, 1382, 1382,    0,    0, 1382,
 1382,    0, 1382,    0,    0,    0,    0,    0,    0,    0,
 1382, 1382,    0, 1382,    0, 1382,    0,    0, 1382,    0,
    0, 1382,    0,    0,    0,    0,    0,    0, 1382,    0,
 1382, 1382,    0, 1382, 1382, 1382, 1382, 1382,    0,    0,
 1382,    0,    0, 1382,    0, 1382, 1382,    0,    0, 1382,
 1382, 1382,    0, 1382, 1382, 1382, 1382, 1382, 1382, 1382,
    0,    0,    0, 1382,    0, 1382, 1382, 1382,    0,    0,
    0, 1382,    0,    0,    0,    0,    0,    0,    0, 1382,
 1382, 1382, 1382, 1382, 1382,    0,    0,    0, 1382, 1382,
    0, 1382,    0, 1382,    0, 1382, 1382, 1382,    0, 1382,
    0, 1382, 1382, 1382,    0, 1382, 1382, 1382, 1382, 1382,
    0, 1382, 1382,    0,    0,    0,    0, 1382,    0, 1382,
 1382,    0,    0, 1382, 1382, 1382,    0,    0, 1382,    0,
 1382,    0,    0, 1382, 1382, 1382, 1382,    0, 1382,    0,
 1382,    0, 1382, 1382, 1382, 1382, 1382,    0,    0, 1382,
 1382, 1382,    0, 1382, 1382,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1382, 1382, 1382,    0, 1382, 1382,
 1382,    0, 1382,    0,    0,    0, 1382,    0, 1382, 1382,
 1382, 1382,    0, 1382,    0, 1382, 1382, 1382, 1382, 1382,
 1382, 1382, 1382,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1382, 1382,    0, 1382, 1382, 1382,    0,    0,
    0,    0, 1382, 1382, 1382, 1382, 1382,    0, 1382,    0,
    0, 1382, 1382,    0,    0,    0, 1382, 1382,    0,    0,
 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382,    0,
 1382,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382,
 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382,
 1382,    0, 1382, 1382, 1382, 1382, 1382, 1382,    0, 1382,
 1382, 1382, 1382,    0, 1382,    0, 1382, 1382, 1382, 1382,
 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382,
 1382, 1382, 1382, 1382,    0, 1382, 1382, 1382, 1382,    0,
 1382, 1382, 1382, 1382, 1382,    0, 1382, 1382, 1382, 1382,
 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382,
 1382, 1382, 1382, 1382, 1382, 1382,    0, 1382, 1382, 1382,
 1382, 1382, 1382, 1382, 1382, 1384, 1382, 1382, 1382, 1382,
    0, 1382, 1382, 1382, 1382, 1382, 1382,    0,    0, 1382,
 1382, 1382, 1382, 1382, 1382, 1382, 1382, 1382,    0,    0,
    0,    0,    0, 1382,    0, 1382, 1382, 1382, 1382, 1382,
 1382,    0, 1382,    0,    0,    0, 1384,    0,    0, 1384,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1384, 1384, 1384, 1384,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1384,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1384,    0, 1384,    0,    0, 1384,    0, 1384,
 1384,    0,    0, 1384, 1384,    0, 1384,    0,    0,    0,
    0,    0,    0,    0, 1384, 1384,    0, 1384,    0, 1384,
    0,    0, 1384,    0,    0, 1384,    0,    0,    0,    0,
    0,    0, 1384,    0, 1384, 1384,    0, 1384, 1384, 1384,
 1384, 1384,    0,    0, 1384,    0,    0, 1384,    0, 1384,
 1384,    0,    0, 1384, 1384, 1384,    0, 1384, 1384, 1384,
 1384, 1384, 1384, 1384,    0,    0,    0, 1384,    0, 1384,
 1384, 1384,    0,    0,    0, 1384,    0,    0,    0,    0,
    0,    0,    0, 1384, 1384, 1384, 1384, 1384, 1384,    0,
    0,    0, 1384, 1384,    0, 1384,    0, 1384,    0, 1384,
 1384, 1384,    0, 1384,    0, 1384, 1384, 1384,    0, 1384,
 1384, 1384, 1384, 1384,    0, 1384, 1384,    0,    0,    0,
    0, 1384,    0, 1384, 1384,    0,    0, 1384, 1384, 1384,
    0,    0, 1384,    0, 1384,    0,    0, 1384, 1384, 1384,
 1384,    0, 1384,    0, 1384,    0, 1384, 1384, 1384, 1384,
 1384,    0,    0, 1384, 1384, 1384,    0, 1384, 1384,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1384, 1384,
 1384,    0, 1384, 1384, 1384,    0, 1384,    0,    0,    0,
 1384,    0, 1384, 1384, 1384, 1384,    0, 1384,    0, 1384,
 1384, 1384, 1384, 1384, 1384, 1384, 1384,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1384, 1384,    0, 1384,
 1384, 1384,    0,    0,    0,    0, 1384, 1384, 1384, 1384,
 1384,    0, 1384,    0,    0, 1384, 1384,    0,    0,    0,
 1384, 1384,    0,    0, 1384, 1384, 1384, 1384, 1384, 1384,
 1384, 1384, 1384,    0, 1384,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1384, 1384, 1384, 1384, 1384, 1384,
 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1384,
 1384, 1384, 1384, 1384, 1384,    0, 1384, 1384, 1384, 1384,
 1384, 1384,    0, 1384, 1384, 1384, 1384,    0, 1384,    0,
 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1384,
 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1384,    0, 1384,
 1384, 1384, 1384,    0, 1384, 1384, 1384, 1384, 1384,    0,
 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1384,
 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1384,
    0, 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1384, 1386,
 1384, 1384, 1384, 1384,    0, 1384, 1384, 1384, 1384, 1384,
 1384,    0,    0, 1384, 1384, 1384, 1384, 1384, 1384, 1384,
 1384, 1384,    0,    0,    0,    0,    0, 1384,    0, 1384,
 1384, 1384, 1384, 1384, 1384,    0, 1384,    0,    0,    0,
 1386,    0,    0, 1386,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1386, 1386,
 1386, 1386,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1386,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1386,    0, 1386,    0,
    0, 1386,    0, 1386, 1386,    0,    0, 1386, 1386,    0,
 1386,    0,    0,    0,    0,    0,    0,    0, 1386, 1386,
    0, 1386,    0, 1386,    0,    0, 1386,    0,    0, 1386,
    0,    0,    0,    0,    0,    0, 1386,    0, 1386, 1386,
    0, 1386, 1386, 1386, 1386, 1386,    0,    0, 1386,    0,
    0, 1386,    0, 1386, 1386,    0,    0, 1386, 1386, 1386,
    0, 1386, 1386, 1386, 1386, 1386, 1386, 1386,    0,    0,
    0, 1386,    0, 1386, 1386, 1386,    0,    0,    0, 1386,
    0,    0,    0,    0,    0,    0,    0, 1386, 1386, 1386,
 1386, 1386, 1386,    0,    0,    0, 1386, 1386,    0, 1386,
    0, 1386,    0, 1386, 1386, 1386,    0, 1386,    0, 1386,
 1386, 1386,    0, 1386, 1386, 1386, 1386, 1386,    0, 1386,
 1386,    0,    0,    0,    0, 1386,    0, 1386, 1386,    0,
    0, 1386, 1386, 1386,    0,    0, 1386,    0, 1386,    0,
    0, 1386, 1386, 1386, 1386,    0, 1386,    0, 1386,    0,
 1386, 1386, 1386, 1386, 1386,    0,    0, 1386, 1386, 1386,
    0, 1386, 1386,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1386, 1386, 1386,    0, 1386, 1386, 1386,    0,
 1386,    0,    0,    0, 1386,    0, 1386, 1386, 1386, 1386,
    0, 1386,    0, 1386, 1386, 1386, 1386, 1386, 1386, 1386,
 1386,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1386, 1386,    0, 1386, 1386, 1386,    0,    0,    0,    0,
 1386, 1386, 1386, 1386, 1386,    0, 1386,    0,    0, 1386,
 1386,    0,    0,    0, 1386, 1386,    0,    0, 1386, 1386,
 1386, 1386, 1386, 1386, 1386, 1386, 1386,    0, 1386,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1386, 1386,
 1386, 1386, 1386, 1386, 1386, 1386, 1386, 1386, 1386, 1386,
 1386, 1386, 1386, 1386, 1386, 1386, 1386, 1386, 1386,    0,
 1386, 1386, 1386, 1386, 1386, 1386,    0, 1386, 1386, 1386,
 1386,    0, 1386,    0, 1386, 1386, 1386, 1386, 1386, 1386,
 1386, 1386, 1386, 1386, 1386, 1386, 1386, 1386, 1386, 1386,
 1386, 1386,    0, 1386, 1386, 1386, 1386,    0, 1386, 1386,
 1386, 1386, 1386,    0, 1386, 1386, 1386, 1386, 1386, 1386,
 1386, 1386, 1386, 1386, 1386, 1386, 1386, 1386, 1386, 1386,
 1386, 1386, 1386, 1386,    0, 1386, 1386, 1386, 1386, 1386,
 1386, 1386, 1386, 1388, 1386, 1386, 1386, 1386,    0, 1386,
 1386, 1386, 1386, 1386, 1386,    0,    0, 1386, 1386, 1386,
 1386, 1386, 1386, 1386, 1386, 1386,    0,    0,    0,    0,
    0, 1386,    0, 1386, 1386, 1386, 1386, 1386, 1386,    0,
 1386,    0,    0,    0, 1388,    0,    0, 1388,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1388, 1388, 1388, 1388,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1388,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1388,    0, 1388,    0,    0, 1388,    0, 1388, 1388,    0,
    0, 1388, 1388,    0, 1388,    0,    0,    0,    0,    0,
    0,    0, 1388, 1388,    0, 1388,    0, 1388,    0,    0,
 1388,    0,    0, 1388,    0,    0,    0,    0,    0,    0,
 1388,    0, 1388, 1388,    0, 1388, 1388, 1388, 1388, 1388,
    0,    0, 1388,    0,    0, 1388,    0, 1388, 1388,    0,
    0, 1388, 1388, 1388,    0, 1388, 1388, 1388, 1388, 1388,
 1388, 1388,    0,    0,    0, 1388,    0, 1388, 1388, 1388,
    0,    0,    0, 1388,    0,    0,    0,    0,    0,    0,
    0, 1388, 1388, 1388, 1388, 1388, 1388,    0,    0,    0,
 1388, 1388,    0, 1388,    0, 1388,    0, 1388, 1388, 1388,
    0, 1388,    0, 1388, 1388, 1388,    0, 1388, 1388, 1388,
 1388, 1388,    0, 1388, 1388,    0,    0,    0,    0, 1388,
    0, 1388, 1388,    0,    0, 1388, 1388, 1388,    0,    0,
 1388,    0, 1388,    0,    0, 1388, 1388, 1388, 1388,    0,
 1388,    0, 1388,    0, 1388, 1388, 1388, 1388, 1388,    0,
    0, 1388, 1388, 1388,    0, 1388, 1388,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1388, 1388, 1388,    0,
 1388, 1388, 1388,    0, 1388,    0,    0,    0, 1388,    0,
 1388, 1388, 1388, 1388,    0, 1388,    0, 1388, 1388, 1388,
 1388, 1388, 1388, 1388, 1388,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1388, 1388,    0, 1388, 1388, 1388,
    0,    0,    0,    0, 1388, 1388, 1388, 1388, 1388,    0,
 1388,    0,    0, 1388, 1388,    0,    0,    0, 1388, 1388,
    0,    0, 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1388,
 1388,    0, 1388,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1388,
 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1388,
 1388, 1388, 1388,    0, 1388, 1388, 1388, 1388, 1388, 1388,
    0, 1388, 1388, 1388, 1388,    0, 1388,    0, 1388, 1388,
 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1388,
 1388, 1388, 1388, 1388, 1388, 1388,    0, 1388, 1388, 1388,
 1388,    0, 1388, 1388, 1388, 1388, 1388,    0, 1388, 1388,
 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1388,
 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1388,    0, 1388,
 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1381, 1388, 1388,
 1388, 1388,    0, 1388, 1388, 1388, 1388, 1388, 1388,    0,
    0, 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1388, 1388,
    0,    0,    0,    0,    0, 1388,    0, 1388, 1388, 1388,
 1388, 1388, 1388,    0, 1388,    0,    0,    0, 1381,    0,
    0, 1381,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1381, 1381, 1381, 1381,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1381,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1381,    0, 1381,    0,    0, 1381,
    0, 1381, 1381,    0,    0, 1381, 1381,    0, 1381,    0,
    0,    0,    0,    0,    0,    0, 1381, 1381,    0, 1381,
    0, 1381,    0,    0, 1381,    0,    0, 1381,    0,    0,
    0,    0,    0,    0, 1381,    0, 1381, 1381,    0, 1381,
 1381, 1381,    0, 1381,    0,    0, 1381,    0,    0, 1381,
    0, 1381, 1381,    0,    0, 1381, 1381, 1381,    0, 1381,
 1381, 1381, 1381, 1381, 1381, 1381,    0,    0,    0, 1381,
    0, 1381, 1381, 1381,    0,    0,    0, 1381,    0,    0,
    0,    0,    0,    0,    0, 1381, 1381, 1381, 1381, 1381,
 1381,    0,    0,    0, 1381, 1381,    0, 1381,    0, 1381,
    0, 1381, 1381, 1381,    0, 1381,    0, 1381, 1381, 1381,
    0, 1381, 1381, 1381, 1381, 1381,    0, 1381, 1381,    0,
    0,    0,    0, 1381,    0, 1381, 1381,    0,    0, 1381,
 1381, 1381,    0,    0, 1381,    0, 1381,    0,    0, 1381,
 1381, 1381, 1381,    0, 1381,    0, 1381,    0, 1381, 1381,
 1381, 1381, 1381,    0,    0, 1381, 1381, 1381,    0, 1381,
 1381,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1381, 1381, 1381,    0, 1381, 1381, 1381,    0, 1381,    0,
    0,    0, 1381,    0, 1381, 1381, 1381, 1381,    0, 1381,
    0, 1381, 1381, 1381, 1381, 1381, 1381, 1381, 1381,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1381, 1381,
    0, 1381, 1381, 1381,    0,    0,    0,    0, 1381, 1381,
 1381, 1381, 1381,    0, 1381,    0,    0, 1381, 1381,    0,
    0,    0, 1381, 1381,    0,    0, 1381, 1381, 1381, 1381,
 1381, 1381, 1381, 1381, 1381,    0, 1381,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1381, 1381, 1381, 1381,
 1381, 1381, 1381, 1381, 1381, 1381, 1381, 1381, 1381, 1381,
 1381, 1381, 1381, 1381, 1381, 1381, 1381,    0, 1381, 1381,
 1381, 1381, 1381, 1381,    0, 1381, 1381, 1381, 1381,    0,
 1381,    0, 1381, 1381, 1381, 1381, 1381, 1381, 1381, 1381,
 1381, 1381, 1381, 1381, 1381, 1381, 1381, 1381, 1381, 1381,
    0, 1381, 1381, 1381, 1381,    0, 1381, 1381, 1381, 1381,
 1381,    0, 1381, 1381, 1381, 1381, 1381, 1381, 1381, 1381,
 1381, 1381, 1381, 1381, 1381, 1381, 1381, 1381, 1381, 1381,
 1381, 1381,    0, 1381, 1381, 1381, 1381, 1381, 1381, 1381,
 1381, 1383, 1381, 1381, 1381, 1381,    0, 1381, 1381, 1381,
 1381, 1381, 1381,    0,    0, 1381, 1381, 1381, 1381, 1381,
 1381, 1381, 1381, 1381,    0,    0,    0,    0,    0, 1381,
    0, 1381, 1381, 1381, 1381, 1381, 1381,    0, 1381,    0,
    0,    0, 1383,    0,    0, 1383,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1383, 1383, 1383, 1383,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1383,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1383,    0,
 1383,    0,    0, 1383,    0, 1383, 1383,    0,    0, 1383,
 1383,    0, 1383,    0,    0,    0,    0,    0,    0,    0,
 1383, 1383,    0, 1383,    0, 1383,    0,    0, 1383,    0,
    0, 1383,    0,    0,    0,    0,    0,    0, 1383,    0,
 1383, 1383,    0, 1383, 1383, 1383,    0, 1383,    0,    0,
 1383,    0,    0, 1383,    0, 1383, 1383,    0,    0, 1383,
 1383, 1383,    0, 1383, 1383, 1383, 1383, 1383, 1383, 1383,
    0,    0,    0, 1383,    0, 1383, 1383, 1383,    0,    0,
    0, 1383,    0,    0,    0,    0,    0,    0,    0, 1383,
 1383, 1383, 1383, 1383, 1383,    0,    0,    0, 1383, 1383,
    0, 1383,    0, 1383,    0, 1383, 1383, 1383,    0, 1383,
    0, 1383, 1383, 1383,    0, 1383, 1383, 1383, 1383, 1383,
    0, 1383, 1383,    0,    0,    0,    0, 1383,    0, 1383,
 1383,    0,    0, 1383, 1383, 1383,    0,    0, 1383,    0,
 1383,    0,    0, 1383, 1383, 1383, 1383,    0, 1383,    0,
 1383,    0, 1383, 1383, 1383, 1383, 1383,    0,    0, 1383,
 1383, 1383,    0, 1383, 1383,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1383, 1383, 1383,    0, 1383, 1383,
 1383,    0, 1383,    0,    0,    0, 1383,    0, 1383, 1383,
 1383, 1383,    0, 1383,    0, 1383, 1383, 1383, 1383, 1383,
 1383, 1383, 1383,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1383, 1383,    0, 1383, 1383, 1383,    0,    0,
    0,    0, 1383, 1383, 1383, 1383, 1383,    0, 1383,    0,
    0, 1383, 1383,    0,    0,    0, 1383, 1383,    0,    0,
 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383,    0,
 1383,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383,
 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383,
 1383,    0, 1383, 1383, 1383, 1383, 1383, 1383,    0, 1383,
 1383, 1383, 1383,    0, 1383,    0, 1383, 1383, 1383, 1383,
 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383,
 1383, 1383, 1383, 1383,    0, 1383, 1383, 1383, 1383,    0,
 1383, 1383, 1383, 1383, 1383,    0, 1383, 1383, 1383, 1383,
 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383,
 1383, 1383, 1383, 1383, 1383, 1383,    0, 1383, 1383, 1383,
 1383, 1383, 1383, 1383, 1383, 1385, 1383, 1383, 1383, 1383,
    0, 1383, 1383, 1383, 1383, 1383, 1383,    0,    0, 1383,
 1383, 1383, 1383, 1383, 1383, 1383, 1383, 1383,    0,    0,
    0,    0,    0, 1383,    0, 1383, 1383, 1383, 1383, 1383,
 1383,    0, 1383,    0,    0,    0, 1385,    0,    0, 1385,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1385, 1385, 1385, 1385,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1385,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1385,    0, 1385,    0,    0, 1385,    0, 1385,
 1385,    0,    0, 1385, 1385,    0, 1385,    0,    0,    0,
    0,    0,    0,    0, 1385, 1385,    0, 1385,    0, 1385,
    0,    0, 1385,    0,    0, 1385,    0,    0,    0,    0,
    0,    0, 1385,    0, 1385, 1385,    0, 1385, 1385, 1385,
    0, 1385,    0,    0, 1385,    0,    0, 1385,    0, 1385,
 1385,    0,    0, 1385, 1385, 1385,    0, 1385, 1385, 1385,
 1385, 1385, 1385, 1385,    0,    0,    0, 1385,    0, 1385,
 1385, 1385,    0,    0,    0, 1385,    0,    0,    0,    0,
    0,    0,    0, 1385, 1385, 1385, 1385, 1385, 1385,    0,
    0,    0, 1385, 1385,    0, 1385,    0, 1385,    0, 1385,
 1385, 1385,    0, 1385,    0, 1385, 1385, 1385,    0, 1385,
 1385, 1385, 1385, 1385,    0, 1385, 1385,    0,    0,    0,
    0, 1385,    0, 1385, 1385,    0,    0, 1385, 1385, 1385,
    0,    0, 1385,    0, 1385,    0,    0, 1385, 1385, 1385,
 1385,    0, 1385,    0, 1385,    0, 1385, 1385, 1385, 1385,
 1385,    0,    0, 1385, 1385, 1385,    0, 1385, 1385,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1385, 1385,
 1385,    0, 1385, 1385, 1385,    0, 1385,    0,    0,    0,
 1385,    0, 1385, 1385, 1385, 1385,    0, 1385,    0, 1385,
 1385, 1385, 1385, 1385, 1385, 1385, 1385,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1385, 1385,    0, 1385,
 1385, 1385,    0,    0,    0,    0, 1385, 1385, 1385, 1385,
 1385,    0, 1385,    0,    0, 1385, 1385,    0,    0,    0,
 1385, 1385,    0,    0, 1385, 1385, 1385, 1385, 1385, 1385,
 1385, 1385, 1385,    0, 1385,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1385, 1385, 1385, 1385, 1385, 1385,
 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1385,
 1385, 1385, 1385, 1385, 1385,    0, 1385, 1385, 1385, 1385,
 1385, 1385,    0, 1385, 1385, 1385, 1385,    0, 1385,    0,
 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1385,
 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1385,    0, 1385,
 1385, 1385, 1385,    0, 1385, 1385, 1385, 1385, 1385,    0,
 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1385,
 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1385,
    0, 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1385, 1387,
 1385, 1385, 1385, 1385,    0, 1385, 1385, 1385, 1385, 1385,
 1385,    0,    0, 1385, 1385, 1385, 1385, 1385, 1385, 1385,
 1385, 1385,    0,    0,    0,    0,    0, 1385,    0, 1385,
 1385, 1385, 1385, 1385, 1385,    0, 1385,    0,    0,    0,
 1387,    0,    0, 1387,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1387, 1387,
 1387, 1387,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1387,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1387,    0, 1387,    0,
    0, 1387,    0, 1387, 1387,    0,    0, 1387, 1387,    0,
 1387,    0,    0,    0,    0,    0,    0,    0, 1387, 1387,
    0, 1387,    0, 1387,    0,    0, 1387,    0,    0, 1387,
    0,    0,    0,    0,    0,    0, 1387,    0, 1387, 1387,
    0, 1387, 1387, 1387,    0, 1387,    0,    0, 1387,    0,
    0, 1387,    0, 1387, 1387,    0,    0, 1387, 1387, 1387,
    0, 1387, 1387, 1387, 1387, 1387, 1387, 1387,    0,    0,
    0, 1387,    0, 1387, 1387, 1387,    0,    0,    0, 1387,
    0,    0,    0,    0,    0,    0,    0, 1387, 1387, 1387,
 1387, 1387, 1387,    0,    0,    0, 1387, 1387,    0, 1387,
    0, 1387,    0, 1387, 1387, 1387,    0, 1387,    0, 1387,
 1387, 1387,    0, 1387, 1387, 1387, 1387, 1387,    0, 1387,
 1387,    0,    0,    0,    0, 1387,    0, 1387, 1387,    0,
    0, 1387, 1387, 1387,    0,    0, 1387,    0, 1387,    0,
    0, 1387, 1387, 1387, 1387,    0, 1387,    0, 1387,    0,
 1387, 1387, 1387, 1387, 1387,    0,    0, 1387, 1387, 1387,
 1342, 1387, 1387,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1387, 1387, 1387,    0, 1387, 1387, 1387,    0,
 1387,    0,    0,    0, 1387,    0, 1387, 1387, 1387, 1387,
    0, 1387,    0, 1387, 1387, 1387, 1387, 1387, 1387, 1387,
 1387, 1342,    0,    0, 1342,    0,    0,    0,    0,    0,
 1387, 1387,    0, 1387, 1387, 1387,    0,    0,    0, 1342,
 1387, 1387, 1387, 1387, 1387,    0, 1387,    0,    0, 1387,
 1387,    0,    0,    0, 1387, 1387,    0,    0, 1387, 1387,
 1387, 1387, 1387, 1387, 1387, 1387, 1387,    0, 1387,    0,
    0,    0,    0, 1342,    0,    0,    0,    0, 1387, 1387,
 1387, 1387, 1387, 1387, 1387, 1387, 1387, 1387, 1387, 1387,
 1387, 1387, 1387, 1387, 1387, 1387, 1387, 1387, 1387,    0,
 1387, 1387, 1387, 1387, 1387, 1387,    0, 1387, 1387, 1387,
 1387,    0, 1387,    0, 1387, 1387, 1387, 1387, 1387, 1387,
 1387, 1387, 1387, 1387, 1387, 1387, 1387, 1387, 1387, 1387,
 1387, 1387,    0, 1387, 1387, 1387, 1387,    0, 1387, 1387,
 1387, 1387, 1387,    0, 1387, 1387, 1387, 1387, 1387, 1387,
 1387, 1387, 1387, 1387, 1387, 1387, 1387, 1387, 1387, 1387,
 1387, 1387, 1387, 1387,    0, 1387, 1387, 1387, 1387, 1387,
 1387, 1387, 1387,    0, 1387, 1387, 1387, 1387,    0, 1387,
 1387, 1387, 1387, 1387, 1387,    0,    0, 1387, 1387, 1387,
 1387, 1387, 1387, 1387, 1387, 1387,    0,    0,    0,    0,
    0, 1387,    0, 1387, 1387, 1387, 1387, 1387, 1387,    0,
 1387,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1342,    0, 1342,
    0,    0, 1342,    0, 1342, 1342,    0,    0, 1342, 1342,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1342,
 1342,    0, 1342,    0,    0,    0,    0, 1342,    0,    0,
 1342,    0,    0,    0,    0,    0,    0, 1342,    0, 1342,
 1342,    0, 1342, 1342, 1342, 1342, 1342,    0,    0, 1342,
    0,    0, 1342,    0, 1342, 1342,    0,    0,    0, 1342,
 1342,    0, 1342, 1342, 1342,    0, 1342, 1342, 1342,    0,
    0,    0, 1342,    0, 1342, 1342, 1342,    0,    0,    0,
 1342,    0,    0,    0,    0,    0,    0,    0, 1342, 1342,
    0, 1342,    0, 1342,    0,    0,    0, 1342, 1342,    0,
 1342,    0,    0,    0, 1342,    0,    0,    0, 1342,    0,
 1342, 1342, 1342,    0, 1342, 1342, 1342, 1342, 1342,    0,
 1342, 1342,    0,    0,    0,    0, 1342,    0, 1342, 1342,
    0,    0, 1342, 1342, 1342,    0,    0, 1342,    0, 1342,
    0,    0, 1342, 1342, 1342, 1342,    0, 1342,    0, 1342,
    0, 1342,    0, 1342, 1342, 1342,    0,    0, 1342, 1342,
 1342, 1341, 1342, 1342,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1342, 1342, 1342,    0, 1342, 1342, 1342,
    0, 1342,    0,    0,    0, 1342,    0, 1342, 1342, 1342,
 1342,    0, 1342,    0, 1342, 1342, 1342, 1342, 1342, 1342,
 1342, 1342, 1341,    0,    0, 1341,    0,    0,    0,    0,
    0, 1342, 1342,    0, 1342, 1342, 1342,    0,    0,    0,
 1341, 1342, 1342, 1342, 1342, 1342,    0, 1342,    0,    0,
 1342, 1342,    0,    0,    0, 1342, 1342,    0,    0, 1342,
 1342, 1342, 1342, 1342, 1342, 1342, 1342, 1342,    0, 1342,
    0,    0,    0,    0, 1341,    0,    0,    0,    0, 1342,
 1342, 1342, 1342, 1342, 1342, 1342, 1342, 1342, 1342, 1342,
 1342, 1342, 1342, 1342, 1342, 1342, 1342, 1342, 1342, 1342,
    0, 1342, 1342, 1342, 1342, 1342, 1342,    0, 1342, 1342,
 1342, 1342,    0, 1342,    0, 1342, 1342, 1342, 1342, 1342,
 1342, 1342, 1342, 1342, 1342, 1342, 1342, 1342, 1342, 1342,
 1342, 1342, 1342,    0, 1342, 1342, 1342, 1342,    0, 1342,
 1342, 1342, 1342, 1342,    0, 1342, 1342, 1342, 1342, 1342,
 1342, 1342, 1342, 1342, 1342, 1342, 1342, 1342,    0, 1342,
 1342, 1342, 1342, 1342, 1342,    0, 1342, 1342, 1342, 1342,
 1342, 1342, 1342, 1342,    0, 1342, 1342, 1342, 1342,    0,
 1342, 1342, 1342, 1342, 1342, 1342,    0,    0, 1342, 1342,
 1342, 1342, 1342, 1342, 1342, 1342, 1342,    0,    0,    0,
    0,    0, 1342,    0, 1342, 1342, 1342, 1342, 1342, 1342,
    0, 1342,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1341,    0,
 1341,    0,    0, 1341,    0, 1341, 1341,    0,    0, 1341,
 1341,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1341, 1341,    0, 1341,    0,    0,    0,    0, 1341,    0,
    0, 1341,    0,    0,    0,    0,    0,    0, 1341,    0,
 1341, 1341,    0, 1341, 1341, 1341, 1341, 1341,    0,    0,
 1341,    0,    0, 1341,    0, 1341, 1341,    0,    0,    0,
 1341, 1341,    0, 1341, 1341, 1341,    0, 1341, 1341, 1341,
    0,    0,    0, 1341,    0, 1341, 1341, 1341,    0,    0,
    0, 1341,    0,    0,    0,    0,    0,    0,    0, 1341,
 1341,    0, 1341,    0, 1341,    0,    0,    0, 1341, 1341,
    0, 1341,    0,    0,    0,    0,    0,    0,    0, 1341,
    0, 1341, 1341, 1341,    0, 1341, 1341, 1341, 1341, 1341,
    0, 1341, 1341,    0,    0,    0,    0, 1341,    0, 1341,
 1341,    0,    0, 1341, 1341, 1341,    0,    0, 1341,    0,
 1341,    0,    0, 1341, 1341, 1341, 1341,    0, 1341,    0,
 1341,    0, 1341,    0, 1341, 1341, 1341,    0,    0, 1341,
 1341, 1341, 1340, 1341, 1341,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1341, 1341, 1341,    0, 1341, 1341,
 1341,    0, 1341,    0,    0,    0, 1341,    0, 1341, 1341,
 1341, 1341,    0, 1341,    0, 1341, 1341, 1341, 1341, 1341,
 1341, 1341, 1341, 1340,    0,    0, 1340,    0,    0,    0,
    0,    0, 1341, 1341,    0, 1341, 1341, 1341,    0,    0,
    0, 1340, 1341, 1341, 1341, 1341, 1341,    0, 1341,    0,
    0, 1341, 1341,    0,    0,    0, 1341, 1341,    0,    0,
 1341, 1341, 1341, 1341, 1341, 1341, 1341, 1341, 1341,    0,
 1341,    0,    0,    0,    0, 1340,    0,    0,    0,    0,
 1341, 1341, 1341, 1341, 1341, 1341, 1341, 1341, 1341, 1341,
 1341, 1341, 1341, 1341, 1341, 1341, 1341, 1341, 1341, 1341,
 1341,    0, 1341, 1341, 1341, 1341, 1341, 1341,    0, 1341,
 1341, 1341, 1341,    0, 1341,    0, 1341, 1341, 1341, 1341,
 1341, 1341, 1341, 1341, 1341, 1341, 1341, 1341, 1341, 1341,
 1341, 1341, 1341, 1341,    0, 1341, 1341, 1341, 1341,    0,
 1341, 1341, 1341, 1341, 1341,    0, 1341, 1341, 1341, 1341,
 1341, 1341, 1341, 1341, 1341, 1341, 1341, 1341, 1341,    0,
 1341, 1341, 1341, 1341, 1341, 1341,    0, 1341, 1341, 1341,
 1341, 1341, 1341, 1341, 1341,    0, 1341, 1341, 1341, 1341,
    0, 1341, 1341, 1341, 1341, 1341, 1341,    0,    0, 1341,
 1341, 1341, 1341, 1341, 1341, 1341, 1341, 1341,    0,    0,
    0,    0,    0, 1341,    0, 1341, 1341, 1341, 1341, 1341,
 1341,    0, 1341,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1340,
    0, 1340,    0,    0,    0,    0, 1340, 1340,    0,    0,
 1340, 1340,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1340, 1340,    0, 1340,    0,    0,    0,    0, 1340,
    0,    0, 1340,    0,    0,    0,    0,    0,    0, 1340,
    0, 1340, 1340,    0, 1340, 1340, 1340, 1340, 1340,    0,
    0, 1340,    0,    0, 1340,    0, 1340, 1340,    0,    0,
    0, 1340, 1340,    0, 1340, 1340, 1340,    0, 1340, 1340,
 1340,    0,    0,    0, 1340,    0, 1340, 1340, 1340,    0,
    0,    0, 1340,    0,    0,    0,    0,    0,    0,    0,
 1340, 1340,    0, 1340,    0, 1340,    0,    0,    0, 1340,
 1340,    0, 1340,    0,    0,    0,    0,    0,    0,    0,
 1340,    0, 1340, 1340, 1340,    0, 1340, 1340, 1340, 1340,
 1340,    0, 1340, 1340,    0,    0,    0,    0, 1340,    0,
 1340, 1340,    0,    0, 1340, 1340, 1340,    0,    0, 1340,
    0, 1340,    0,    0, 1340, 1340, 1340, 1340,    0, 1340,
    0, 1340,    0, 1340,    0, 1340, 1340, 1340,    0,    0,
 1340, 1340, 1340, 1194, 1340, 1340,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1340, 1340, 1340,    0, 1340,
 1340, 1340,    0, 1340,    0,    0,    0, 1340,    0, 1340,
 1340, 1340, 1340,    0, 1340,    0, 1340, 1340, 1340, 1340,
 1340, 1340, 1340, 1340, 1194,    0,    0, 1194,    0,    0,
    0,    0,    0, 1340, 1340,    0, 1340, 1340, 1340,    0,
    0,    0, 1194, 1340, 1340, 1340, 1340, 1340,    0, 1340,
    0,    0, 1340, 1340,    0,    0,    0, 1340, 1340,    0,
    0, 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340,
    0, 1340,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340,
 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340,
 1340, 1340,    0, 1340, 1340, 1340, 1340, 1340, 1340,    0,
 1340, 1340, 1340, 1340,    0, 1340,    0, 1340, 1340, 1340,
 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340,
 1340, 1340, 1340, 1340, 1340,    0, 1340, 1340, 1340, 1340,
    0, 1340, 1340, 1340, 1340, 1340,    0, 1340, 1340, 1340,
 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340,
    0, 1340, 1340, 1340, 1340, 1340, 1340,    0, 1340, 1340,
 1340, 1340, 1340, 1340, 1340, 1340,    0, 1340, 1340, 1340,
 1340,    0, 1340, 1340, 1340, 1340, 1340, 1340,    0,    0,
 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340, 1340,    0,
    0,    0,    0,    0, 1340,    0, 1340, 1340, 1340, 1340,
 1340, 1340,    0, 1340,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1194,    0, 1194,    0,    0,    0,    0, 1194, 1194,    0,
    0, 1194, 1194,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1194, 1194,    0, 1194,    0, 1194,    0,    0,
 1194,    0,    0, 1194,    0,    0,    0,    0,    0,    0,
 1194,    0, 1194, 1194,    0,    0,    0, 1194,    0, 1194,
    0,    0, 1194,    0,    0, 1194,    0,    0, 1194,    0,
    0,    0, 1194, 1194,    0, 1194, 1194, 1194,    0, 1194,
 1194, 1194,    0,    0,    0, 1194,    0, 1194, 1194, 1194,
    0,    0,    0, 1194,    0,    0,    0,    0,    0,    0,
    0, 1194, 1194,    0, 1194,    0, 1194,    0,    0,    0,
 1194, 1194,    0, 1194,    0,    0,    0,    0,    0,    0,
    0, 1194,    0, 1194,    0, 1194,    0, 1194, 1194, 1194,
 1194, 1194,    0, 1194, 1194,    0,    0,    0,    0, 1194,
    0, 1194, 1194,    0,    0, 1194, 1194,    0,    0,    0,
 1194,    0, 1194,    0,    0, 1194, 1194, 1194, 1194,    0,
 1194,    0, 1194,    0, 1194, 1194, 1194, 1194, 1194,    0,
    0, 1164,    0, 1194,    0, 1194, 1194,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1194,    0, 1194,    0,
 1194, 1194, 1194,    0, 1194,    0,    0,    0, 1194,    0,
 1194, 1194, 1194, 1194,    0, 1194,    0,    0,    0,    0,
    0, 1164, 1164, 1194, 1194, 1164,    0,    0,    0,    0,
    0,    0,    0,    0, 1194, 1194,    0, 1194, 1194, 1194,
 1164,    0,    0,    0, 1194, 1194, 1194, 1194, 1194,    0,
 1194,    0,    0, 1194, 1194,    0,    0,    0, 1194, 1194,
    0,    0, 1194, 1194, 1194, 1194, 1194, 1194, 1194, 1194,
 1194,    0, 1194,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1194, 1194, 1194, 1194, 1194, 1194, 1194, 1194,
 1194, 1194, 1194, 1194, 1194, 1194, 1194, 1194, 1194, 1194,
 1194, 1194, 1194,    0, 1194, 1194, 1194, 1194, 1194, 1194,
    0, 1194, 1194, 1194, 1194,    0, 1194,    0, 1194, 1194,
 1194, 1194, 1194, 1194, 1194, 1194, 1194, 1194, 1194, 1194,
 1194, 1194, 1194, 1194, 1194, 1194,    0, 1194, 1194, 1194,
 1194,    0, 1194, 1194, 1194, 1194, 1194,    0, 1194, 1194,
 1194, 1194, 1194, 1194, 1194, 1194, 1194, 1194, 1194, 1194,
 1194,    0, 1194, 1194, 1194, 1194, 1194, 1194,    0, 1194,
 1194, 1194, 1194, 1194, 1194, 1194, 1194,    0, 1194, 1194,
 1194, 1194,    0, 1194, 1194, 1194, 1194, 1194, 1194,    0,
    0, 1194, 1194, 1194, 1194, 1194, 1194, 1194, 1194, 1194,
    0,    0,    0,    0,    0, 1194,    0, 1194, 1194, 1194,
 1194, 1194, 1194,    0, 1194,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1164,    0,
 1164,    0,    0,    0,    0,    0, 1164,    0,    0, 1164,
 1164,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1164, 1164,    0, 1164,    0, 1164,    0,    0, 1164,    0,
    0, 1164,    0,    0,    0,    0,    0,    0, 1164,    0,
 1164, 1164,    0,    0,    0, 1164,    0, 1164,    0,    0,
 1164,    0,    0, 1164,    0,    0, 1164,    0,    0,    0,
 1164, 1164,    0, 1164, 1164, 1164,    0, 1164, 1164, 1164,
    0,    0,    0, 1164,    0, 1164, 1164, 1164,    0,    0,
    0, 1164,    0,    0,    0,    0,    0,    0,    0, 1164,
 1164,    0, 1164,    0, 1164,    0,    0,    0, 1164, 1164,
    0, 1164,    0,    0,    0,    0,    0,    0,    0, 1164,
    0, 1164,    0, 1164,    0, 1164, 1164, 1164, 1164, 1164,
    0, 1164, 1164,    0,    0,    0,    0, 1164,    0, 1164,
 1164,    0,    0, 1164, 1164,    0,    0,    0, 1164,    0,
 1164,    0,    0, 1164, 1164, 1164, 1164,    0, 1164,    0,
 1164, 1191, 1164, 1164, 1164, 1164, 1164,    0,    0,    0,
    0, 1164,    0, 1164, 1164,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1164,    0, 1164,    0, 1164, 1164,
 1164,    0, 1164,    0,    0,    0, 1164,    0, 1164, 1164,
 1164, 1164, 1191, 1164,    0, 1191,    0,    0,    0,    0,
    0, 1164, 1164,    0,    0,    0,    0,    0,    0,    0,
 1191,    0, 1164, 1164,    0, 1164, 1164, 1164,    0,    0,
    0,    0, 1164, 1164, 1164, 1164, 1164,    0, 1164,    0,
    0, 1164, 1164,    0,    0,    0, 1164, 1164,    0,    0,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,    0,
 1164,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164,    0, 1164, 1164, 1164, 1164, 1164, 1164,    0, 1164,
 1164, 1164, 1164,    0, 1164,    0, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164, 1164,    0,
 1164, 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,    0,
 1164, 1164, 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164, 1164,
    0, 1164, 1164, 1164, 1164, 1164, 1164,    0,    0, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,    0,    0,
    0,    0,    0, 1164,    0, 1164, 1164, 1164, 1164, 1164,
 1164,    0, 1164,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1164,    0,
 1164,    0,    0,    0,    0,    0, 1164,    0,    0, 1164,
 1164,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1164, 1164,    0, 1164,    0, 1164,    0,    0, 1164,    0,
    0, 1164,    0,    0,    0,    0,    0,    0, 1164,    0,
 1164, 1164,    0,    0,    0, 1164,    0, 1164,    0,    0,
 1164,    0,    0, 1191,    0,    0, 1191,    0,    0,    0,
 1164, 1164,    0, 1191, 1191, 1164,    0, 1164, 1191, 1164,
    0,    0,    0, 1191,    0, 1164, 1191, 1164,    0,    0,
    0, 1164,    0,    0,    0,    0,    0,    0,    0, 1164,
 1191,    0, 1164,    0, 1164,    0,    0,    0, 1164, 1164,
    0, 1191,    0,    0,    0,    0,    0,    0,    0, 1191,
    0, 1164,    0, 1191,    0, 1164, 1164, 1164, 1164, 1164,
    0, 1164, 1191,    0,    0,    0,    0, 1164,    0, 1164,
 1164,    0,    0, 1164, 1164,    0,    0,    0, 1191,    0,
 1164,    0,    0, 1164, 1164, 1164, 1164,    0, 1164,    0,
 1164, 1193, 1164, 1164, 1164, 1164, 1164,    0,    0,    0,
    0, 1164,    0, 1164, 1191,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1164,    0, 1191,    0, 1191, 1164,
 1164,    0, 1164,    0,    0,    0, 1164,    0, 1164, 1164,
 1164, 1164, 1193, 1164,    0, 1193,    0,    0,    0,    0,
    0, 1164, 1164,    0,    0,    0,    0,    0,    0,    0,
 1193,    0, 1164, 1164,    0, 1164, 1164, 1164,    0,    0,
    0,    0, 1164, 1164, 1191, 1164, 1164,    0, 1164,    0,
    0, 1164, 1164,    0,    0,    0, 1164, 1164,    0,    0,
 1191, 1191, 1164, 1164, 1164, 1191, 1164, 1164, 1164,    0,
 1164,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164,    0, 1164, 1164, 1164, 1164, 1164, 1164,    0, 1164,
 1164, 1164, 1164,    0, 1164,    0, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164, 1164,    0,
 1164, 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,    0,
 1164, 1164, 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164, 1164,
    0, 1164, 1164, 1164, 1164, 1164, 1164,    0,    0, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,    0,    0,
    0,    0,    0, 1164,    0, 1164, 1164, 1164, 1164, 1164,
 1164,    0, 1191,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1164,    0,
 1164,    0,    0,    0,    0,    0, 1164,    0,    0, 1164,
 1164,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1164, 1164,    0, 1164,    0, 1164,    0,    0, 1164,    0,
    0, 1164,    0,    0,    0,    0,    0,    0, 1164,    0,
 1164, 1164,    0,    0,    0, 1164,    0, 1164,    0,    0,
 1164,    0,    0, 1193,    0,    0, 1193,    0,    0,    0,
 1164, 1164,    0, 1193, 1193, 1164,    0, 1164, 1193, 1164,
    0,    0,    0, 1193,    0, 1164, 1193, 1164,    0,    0,
    0, 1164,    0,    0,    0,    0,    0,    0,    0, 1164,
 1193,    0, 1164,    0, 1164,    0,    0,    0, 1164, 1164,
    0, 1193,    0,    0,    0,    0,    0,    0,    0, 1193,
    0, 1164,    0, 1193,    0, 1164, 1164, 1164, 1164, 1164,
    0, 1164, 1193,    0,    0,    0,    0, 1164,    0, 1164,
 1164,    0,    0, 1164, 1164,    0, 1198,    0, 1193,    0,
 1164,    0,    0, 1164, 1164, 1164, 1164,    0, 1164,    0,
 1164,    0, 1164, 1164, 1164, 1164, 1164,    0,    0,    0,
    0, 1164,    0, 1164, 1193,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1164,    0, 1193,    0, 1193, 1164,
 1164,    0, 1164,    0,    0,    0, 1164,    0, 1164, 1164,
 1164, 1164,    0, 1164,    0, 1198,    0,    0,    0,    0,
    0, 1164, 1164,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1164, 1164,    0, 1164, 1164, 1164,    0,    0,
    0,    0, 1164, 1164, 1193, 1164, 1164,    0, 1164,    0,
    0, 1164, 1164,    0,    0,    0, 1164, 1164,    0,    0,
 1193, 1193, 1164, 1164, 1164, 1193, 1164, 1164, 1164,    0,
 1164,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164,    0, 1164, 1164, 1164, 1164, 1164, 1164,    0, 1164,
 1164, 1164, 1164,    0, 1164,    0, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164, 1164,    0,
 1164, 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,    0,
 1164, 1164, 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164, 1164,
    0, 1164, 1164, 1164, 1164, 1164, 1164,    0,    0, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,    0,    0,
    0,    0,    0, 1164,    0, 1164, 1164, 1164, 1164, 1164,
 1164,    0, 1193, 1164,    0, 1164,    0,    0,    0,    0,
    0, 1164,    0,    0, 1164, 1164,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1164, 1164,    0, 1164,    0,
 1164,    0,    0, 1164,    0,    0, 1164,    0,    0,    0,
    0,    0,    0, 1164,    0, 1164, 1164,    0,    0,    0,
 1164,    0, 1164,    0,    0, 1164,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1164, 1164,    0,    0,    0,
 1164,    0, 1164,    0, 1164,    0,    0,    0,    0,    0,
 1164,    0, 1164,    0,    0,    0, 1164,    0,    0,    0,
    0,    0,    0,    0, 1164,    0,    0, 1164,    0, 1164,
    0,    0,    0, 1164, 1164,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1164,    0, 1198,    0,
 1164, 1164, 1164, 1164, 1164,    0, 1164, 1198,    0,    0,
    0,    0, 1164,    0, 1164, 1164,    0,    0, 1164, 1164,
    0,    0, 1162,    0,    0, 1164,    0, 1198, 1164, 1164,
 1164, 1164,    0, 1164,    0, 1164,    0, 1164, 1164, 1164,
 1164, 1164,    0,    0,    0,    0, 1164,    0, 1164,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1164,
    0, 1198,    0,    0, 1164, 1164, 1162, 1164,    0,    0,
    0, 1164,    0, 1164, 1164, 1164, 1164,    0, 1164,    0,
    0, 1162,    0,    0,    0,    0, 1164, 1164,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1164, 1164,    0,
 1164, 1164, 1164,    0,    0,    0,    0, 1164, 1164, 1198,
 1164, 1164,    0, 1164,    0,    0, 1164, 1164,    0,    0,
    0, 1164, 1164,    0,    0,    0, 1198, 1164, 1164, 1164,
    0, 1164, 1164, 1164,    0, 1164,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164,
 1164, 1164, 1164,    0, 1164, 1164, 1164, 1164,    0, 1164,
    0, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,    0,
 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164, 1164, 1164,
    0, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164, 1164, 1164,
 1164,    0, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
    0, 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164, 1164,
 1164, 1164,    0,    0, 1164, 1164, 1164, 1164, 1164, 1164,
 1164, 1164, 1164,    0,    0,    0,    0,    0, 1164,    0,
 1164, 1164, 1164, 1164, 1164, 1164,    0,    0,    0, 1164,
    0, 1164,    0,    0,    0,    0,    0, 1164,    0,    0,
 1164, 1164,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1164, 1164,    0, 1164,    0,    0,    0,    0, 1164,
    0,    0, 1164,    0,    0,    0,    0,    0,    0, 1164,
    0, 1164, 1164,    0,    0,    0, 1164,    0, 1164,    0,
    0, 1164,    0,    0,    0,    0, 1162,    0,    0,    0,
    0, 1164, 1164,    0,    0,    0, 1164,    0, 1164,    0,
 1164,    0,    0,    0, 1162,    0, 1164,    0, 1164,    0,
    0,    0, 1164,    0,    0,    0,    0,    0,    0,    0,
 1164,    0,    0, 1164,    0, 1164,    0,    0,    0, 1164,
 1164,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1164,    0,    0,    0, 1164, 1164, 1164, 1164,
 1164,    0, 1164,    0,    0,    0,    0,    0, 1164,    0,
 1164, 1164,    0,    0, 1164, 1164,    0,    0,    0,    0,
    0, 1164,    0,    0, 1164, 1164, 1164, 1164,    0, 1164,
    0, 1164,    0, 1164,    0, 1164, 1164, 1164,    0,    0,
    0,    0, 1164,    0, 1164,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1164,    0,    0,    0,    0,
 1164, 1164,    0, 1164,    0,    0,    0, 1164,    0, 1164,
 1164, 1164, 1164,    0, 1164,    0,    0,    0,    0,    0,
    0,    0, 1164, 1164,    0,    0,    0, 1142,    0,    0,
  879,    0,  880, 1164, 1164,    0, 1164, 1164, 1164,    0,
    0,    0,    0, 1164, 1164,  882, 1164, 1164,    0, 1164,
  883,    0, 1164, 1164,    0,    0,    0, 1164, 1164,    0,
    0,    0,    0, 1164, 1164, 1164,    0, 1164, 1164, 1164,
    0, 1164,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164, 1164,    0, 1164, 1164, 1164, 1164, 1164, 1164,    0,
 1164, 1164, 1164, 1164,    0, 1164,    0, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164, 1164,
    0, 1164, 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,
    0, 1164, 1164, 1164, 1164, 1164, 1164,    0, 1164, 1164,
 1164, 1164, 1164, 1164, 1164, 1164,    0, 1164, 1164, 1164,
 1164,    0, 1164, 1164, 1164, 1164, 1164, 1164,    0,    0,
 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164, 1164,    0,
    0,    0,    0,    0, 1164,    0, 1164, 1164, 1164, 1164,
 1164, 1164,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766, 1498, 1499, 1500, 1501, 1502, 1503, 1531, 1532,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
 1533,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839, 1534,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1142,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226, 1526,  227,    0,  788,  789,
  790, 1527,  791,  792, 1528,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1142,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139, 1599,    0,    0,
 1600,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186, 1601,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1146,    0, 1146,
 1146,    0, 1146,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1146,    0,    0,    0,    0,
 1146,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1146,    0, 1146, 1146,    0,    0,
    0,    0, 1146,    0, 1146, 1146, 1146,    0,    0,    0,
    0, 1146,    0,    0,    0,    0, 1146, 1146,    0, 1146,
    0, 1146, 1146,    0, 1146,    0,    0, 1146, 1146, 1146,
    0,    0,    0,    0, 1146, 1146, 1146, 1146,    0,    0,
    0, 1146,    0, 1146,    0, 1146, 1146,    0,    0,    0,
    0,    0,    0,    0, 1146,    0, 1146, 1146,    0,    0,
    0, 1146,    0, 1146,    0, 1146,    0,    0,    0,    0,
    0, 1146,    0, 1146,    0,    0,    0, 1146,    0,    0,
    0, 1146,    0, 1146, 1146, 1146, 1146,    0, 1146,    0,
 1146, 1146,    0, 1146, 1146, 1146,    0,    0,    0,    0,
    0, 1146,    0,    0,    0,    0,    0, 1146,    0,    0,
    0, 1146, 1146, 1146, 1146, 1146,    0, 1146,    0, 1146,
    0,    0,    0, 1146,    0, 1146, 1146,    0,    0, 1146,
 1146,    0,    0,    0, 1146,    0, 1146,    0,    0, 1146,
 1146, 1146, 1146,    0, 1146,    0, 1146, 1146, 1146, 1146,
 1146, 1146, 1146, 1146,    0,    0,    0, 1146,    0, 1146,
    0,    0,    0, 1146,    0,    0,    0,    0,    0,    0,
 1146,    0,    0,    0,    0, 1146, 1146, 1146, 1146, 1146,
 1146, 1146, 1146,    0, 1146, 1146, 1146, 1146,    0, 1146,
 1146,    0,    0,    0,    0,    0,    0, 1146, 1146, 1146,
 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1146,
    0, 1146, 1146,    0, 1146, 1146,    0, 1146, 1146, 1146,
    0, 1146, 1146, 1146, 1146,    0,    0, 1146, 1146, 1146,
 1146, 1146, 1146, 1146,    0,    0,    0,    0, 1146, 1146,
 1146,    0, 1146, 1146, 1146,    0, 1146,    0, 1146, 1146,
 1146,    0, 1146, 1146,    0, 1146, 1146, 1146, 1146, 1146,
 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1146,
 1146, 1146, 1146, 1146, 1146, 1146, 1146,    0, 1146, 1146,
 1146, 1146, 1146, 1146,    0, 1146, 1146, 1146, 1146,    0,
 1146,    0, 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1146,
 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1146,
    0, 1146, 1146, 1146, 1146,    0, 1146, 1146, 1146, 1146,
 1146,    0, 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1146,
 1146, 1146, 1146, 1146, 1146,    0, 1146, 1146, 1146, 1146,
 1146, 1146,    0, 1146, 1146, 1146, 1146, 1146, 1146, 1146,
 1146,    0, 1146, 1146, 1146, 1146,    0, 1146, 1146, 1146,
 1146, 1146, 1146,    0,    0, 1146, 1146, 1146, 1146, 1146,
 1146, 1146, 1146, 1146, 1146,    0, 1146, 1146, 1146, 1146,
 1146, 1146, 1146, 1146, 1146, 1146, 1146,    0,    0, 1146,
 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1146,
 1146, 1146, 1146, 1146, 1146, 1146, 1146, 1142,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147, 1141,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1142,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195, 1173,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1142,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,   15,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1142,    0, 1326,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1142, 1614,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1142,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0, 1976,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1142,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0, 1598,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1142, 2370,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1142,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
 2462,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1144,    0, 1144,
 1144,    0, 1144,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1144,    0,    0,    0,    0,
 1144,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1144,    0, 1144, 1144,    0,    0,
    0,    0, 1144,    0, 1144, 1144, 1144,    0,    0,    0,
    0, 1144,    0,    0,    0,    0, 1144, 1144,    0, 1144,
    0, 1144, 1144,    0, 1144,    0,    0, 1144, 1144, 1144,
    0,    0,    0,    0, 1144, 1144, 1144, 1144,    0,    0,
    0, 1144,    0, 1144,    0, 1144, 1144,    0,    0,    0,
    0,    0,    0,    0, 1144,    0, 1144, 1144,    0,    0,
    0, 1144,    0, 1144,    0, 1144,    0,    0,    0,    0,
    0, 1144,    0, 1144,    0,    0,    0, 1144,    0,    0,
    0, 1144,    0, 1144, 1144, 1144, 1144,    0, 1144,    0,
 1144, 1144,    0, 1144, 1144, 1144,    0,    0,    0,    0,
    0, 1144,    0,    0,    0,    0,    0, 1144,    0,    0,
    0, 1144, 1144, 1144, 1144, 1144,    0, 1144,    0, 1144,
    0,    0,    0, 1144,    0, 1144, 1144,    0,    0, 1144,
 1144,    0,    0,    0, 1144,    0, 1144,    0,    0, 1144,
 1144, 1144, 1144,    0, 1144,    0, 1144, 1144, 1144, 1144,
 1144, 1144, 1144, 1144,    0,    0,    0, 1144,    0, 1144,
    0,    0,    0, 1144,    0,    0,    0,    0,    0,    0,
 1144,    0,    0,    0,    0, 1144, 1144, 1144, 1144, 1144,
 1144, 1144, 1144,    0, 1144, 1144, 1144, 1144,    0, 1144,
 1144,    0,    0,    0,    0,    0,    0, 1144, 1144, 1144,
 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1144,
    0, 1144,    0,    0, 1144, 1144,    0, 1144, 1144, 1144,
    0, 1144, 1144, 1144, 1144,    0,    0, 1144, 1144, 1144,
 1144, 1144, 1144, 1144,    0,    0,    0,    0, 1144, 1144,
 1144,    0, 1144, 1144, 1144,    0, 1144,    0, 1144, 1144,
 1144,    0, 1144, 1144,    0, 1144, 1144, 1144, 1144, 1144,
 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1144,
 1144, 1144, 1144, 1144, 1144, 1144, 1144,    0, 1144, 1144,
 1144, 1144, 1144, 1144,    0, 1144, 1144, 1144, 1144,    0,
 1144,    0, 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1144,
 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1144,
    0, 1144, 1144, 1144, 1144,    0, 1144, 1144, 1144, 1144,
 1144,    0, 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1144,
 1144, 1144, 1144, 1144, 1144,    0, 1144, 1144, 1144, 1144,
 1144, 1144,    0, 1144, 1144, 1144, 1144, 1144, 1144, 1144,
 1144,    0, 1144, 1144, 1144, 1144,    0, 1144, 1144, 1144,
 1144, 1144, 1144,    0,    0, 1144, 1144, 1144, 1144, 1144,
 1144, 1144, 1144, 1144, 1144,    0, 1144, 1144, 1144, 1144,
 1144, 1144, 1144, 1144, 1144, 1144, 1144,    0,    0, 1144,
 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1144,
 1144, 1144, 1144, 1144, 1144, 1144, 1144, 1742,    0, 1742,
 1742,    0, 1742,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1742,    0,    0,    0,    0,
 1742,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1742,    0, 1742,    0,    0,    0,
    0,    0, 1742,    0, 1742, 1742, 1742,    0,    0,    0,
    0, 1742,    0,    0,    0,    0, 1742, 1742,    0, 1742,
    0, 1742, 1742,    0, 1742,    0,    0, 1742, 1742, 1742,
    0,    0,    0,    0, 1742,    0, 1742, 1742,    0,    0,
    0, 1742,    0, 1742,    0, 1742, 1742,    0,    0,    0,
    0,    0,    0,    0, 1742,    0, 1742, 1742,    0,    0,
    0, 1742,    0, 1742,    0, 1742,    0,    0,    0,    0,
    0, 1742,    0, 1742,    0,    0,    0, 1742,    0,    0,
    0, 1742,    0, 1742, 1742, 1742, 1742,    0, 1742,    0,
 1742, 1742,    0, 1742, 1742, 1742,    0,    0,    0,    0,
    0, 1742,    0,    0,    0,    0,    0, 1742,    0,    0,
    0, 1742, 1742, 1742, 1742, 1742,    0, 1742,    0, 1742,
    0,    0,    0, 1742,    0, 1742, 1742,    0,    0, 1742,
 1742,    0,    0,    0, 1742,    0, 1742,    0,    0, 1742,
 1742, 1742, 1742,    0, 1742,    0, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742,    0,    0,    0, 1742,    0, 1742,
    0,    0,    0, 1742,    0,    0,    0,    0,    0,    0,
 1742,    0,    0,    0,    0, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742,    0, 1742, 1742, 1742, 1742,    0, 1742,
 1742,    0,    0,    0,    0,    0,    0, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
    0, 1742, 1742, 1742, 1742, 1742,    0, 1742, 1742, 1742,
    0, 1742, 1742, 1742, 1742,    0,    0, 1742, 1742, 1742,
 1742, 1742, 1742, 1742,    0,    0,    0,    0, 1742, 1742,
 1742,    0, 1742, 1742, 1742,    0, 1742,    0, 1742, 1742,
 1742,    0, 1742, 1742,    0, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742,    0, 1742, 1742,
 1742, 1742, 1742, 1742,    0, 1742, 1742, 1742, 1742,    0,
 1742,    0, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
    0, 1742, 1742, 1742, 1742,    0, 1742, 1742, 1742, 1742,
 1742,    0, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742,    0, 1742, 1742, 1742, 1742,
 1742, 1742,    0, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742,    0, 1742, 1742, 1742, 1742,    0, 1742, 1742, 1742,
 1742, 1742, 1742,    0,    0, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742,    0, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742,    0,    0, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742,  881,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1142,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 2311,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 2934,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1142,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,  747,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,  756,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  759,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,  783,
  784,  785,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302, 3371,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1742,    0,    0,
 1742,    0, 1742,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1742,    0,    0,    0,    0,
 1742,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1742,    0, 1742,    0,    0,    0,
    0,    0, 1742,    0, 1742, 1742, 1742,    0,    0,    0,
    0, 1742,    0,    0,    0,    0, 1742, 1742,    0, 1742,
    0, 1742, 1742,    0, 1742,    0,    0, 1742, 1742, 1742,
    0,    0,    0,    0, 1742,    0, 1742, 1742,    0,    0,
    0, 1742,    0, 1742,    0, 1742, 1742,    0,    0,    0,
    0,    0,    0,    0, 1742,    0, 1742, 1742,    0,    0,
    0, 1742,    0, 1742,    0, 1742,    0,    0,    0,    0,
    0, 1742,    0, 1742,    0,    0,    0, 1742,    0,    0,
    0, 1742,    0, 1742, 1742, 1742, 1742,    0, 1742,    0,
 1742, 1742,    0, 1742, 1742, 1742,    0,    0,    0,    0,
    0, 1742,    0,    0,    0,    0,    0, 1742,    0,    0,
    0, 1742, 1742, 1742, 1742, 1742,    0, 1742,    0, 1742,
    0,    0,    0, 1742,    0, 1742, 1742,    0,    0, 1742,
 1742,    0,    0,    0, 1742,    0, 1742,    0,    0, 1742,
 1742, 1742, 1742,    0, 1742,    0, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742,    0,    0,    0, 1742,    0, 1742,
    0,    0,    0, 1742,    0,    0,    0,    0,    0,    0,
 1742,    0,    0,    0,    0, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742,    0, 1742, 1742, 1742, 1742,    0, 1742,
 1742,    0,    0,    0,    0,    0,    0, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
    0, 1742, 1742, 1742, 1742, 1742,    0, 1742, 1742, 1742,
    0, 1742, 1742, 1742, 1742,    0,    0, 1742, 1742, 1742,
 1742, 1742, 1742, 1742,    0,    0,    0,    0, 1742, 1742,
 1742,    0, 1742, 1742, 1742,    0, 1742,    0, 1742, 1742,
 1742,    0, 1742, 1742,    0, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742,    0, 1742, 1742,
 1742, 1742, 1742, 1742,    0, 1742, 1742, 1742, 1742,    0,
 1742,    0, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
    0, 1742, 1742, 1742, 1742,    0, 1742, 1742, 1742, 1742,
 1742,    0, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742,    0, 1742, 1742, 1742, 1742,
 1742, 1742,    0, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742,    0, 1742, 1742, 1742, 1742,    0, 1742, 1742, 1742,
 1742, 1742, 1742,    0,    0, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742,    0, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742,    0,    0, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1742,
 1742, 1742, 1742, 1742, 1742, 1742, 1742, 1379,    0,    0,
 1379,    0, 1379,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1379,    0,    0,    0,    0,
 1379,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1379,    0, 1379,    0,    0,    0,
    0,    0, 1379,    0, 1379, 1379, 1379,    0,    0,    0,
    0, 1379,    0,    0,    0,    0, 1379, 1379,    0, 1379,
    0, 1379, 1379,    0, 1379,    0,    0, 1379, 1379, 1379,
    0,    0,    0,    0, 1379,    0, 1379, 1379,    0,    0,
    0, 1379,    0, 1379,    0, 1379, 1379,    0,    0,    0,
    0,    0,    0,    0, 1379,    0, 1379, 1379,    0,    0,
    0, 1379,    0, 1379,    0, 1379,    0,    0,    0,    0,
    0, 1379,    0, 1379,    0,    0,    0, 1379,    0,    0,
    0, 1379,    0, 1379, 1379, 1379, 1379,    0, 1379,    0,
 1379, 1379,    0, 1379, 1379, 1379,    0,    0,    0,    0,
    0, 1379,    0,    0,    0,    0,    0, 1379,    0,    0,
    0, 1379, 1379, 1379, 1379, 1379,    0, 1379,    0, 1379,
    0,    0,    0, 1379,    0, 1379, 1379,    0,    0, 1379,
 1379,    0,    0,    0, 1379,    0, 1379,    0,    0, 1379,
 1379, 1379, 1379,    0, 1379,    0, 1379, 1379, 1379, 1379,
 1379, 1379, 1379, 1379,    0,    0,    0, 1379,    0, 1379,
    0,    0,    0, 1379,    0,    0,    0,    0,    0,    0,
 1379,    0,    0,    0,    0, 1379, 1379, 1379, 1379, 1379,
 1379, 1379, 1379,    0, 1379, 1379, 1379, 1379,    0, 1379,
 1379,    0,    0,    0,    0,    0,    0, 1379, 1379, 1379,
 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1379,
    0, 1379, 1379, 1379, 1379, 1379,    0, 1379, 1379, 1379,
    0, 1379, 1379, 1379, 1379,    0,    0, 1379, 1379, 1379,
 1379, 1379, 1379, 1379,    0,    0,    0,    0, 1379, 1379,
 1379,    0, 1379, 1379, 1379,    0, 1379,    0, 1379, 1379,
 1379,    0, 1379, 1379,    0, 1379, 1379, 1379, 1379, 1379,
 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1379,
 1379, 1379, 1379, 1379, 1379, 1379, 1379,    0, 1379, 1379,
 1379, 1379, 1379, 1379,    0, 1379, 1379, 1379, 1379,    0,
 1379,    0, 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1379,
 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1379,
    0, 1379, 1379, 1379, 1379,    0, 1379, 1379, 1379, 1379,
 1379,    0, 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1379,
 1379, 1379, 1379, 1379, 1379,    0, 1379, 1379, 1379, 1379,
 1379, 1379,    0, 1379, 1379, 1379, 1379, 1379, 1379, 1379,
 1379,    0, 1379, 1379, 1379, 1379,    0, 1379, 1379, 1379,
 1379, 1379, 1379,    0,    0, 1379, 1379, 1379, 1379, 1379,
 1379, 1379, 1379, 1379, 1379,    0, 1379, 1379, 1379, 1379,
 1379, 1379, 1379, 1379, 1379, 1379, 1379,    0,    0, 1379,
 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1379,
 1379, 1379, 1379, 1379, 1379, 1379, 1379, 1218,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,    0,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  184,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,   15,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,    0,
    0,    0,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1218,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,    0,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  184,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766,    0,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,    0,
    0,    0,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878, 1579,    0,    0,
  879,    0,  880,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  882,    0,    0,    0,    0,
  883,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,  742,  141,  142,    0,    0,    0,
    0,  743,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,  744,    0,  147,    0,    0,  148,  745,  746,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,    0,  154,    0,    0,    0,
    0,    0,    0,    0,  748,    0,  155,  749,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,  750,    0,  751,  752,  163,  753,    0,  164,    0,
  165,  754,    0,  755,  166,  167,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,  757,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,  758,    0,  181,    0,    0,  182,
  183,  184,  185,    0,  186,    0,  187,  760,  188,  189,
  190,  191,  192,  761,    0,    0,    0,  193,    0,  194,
    0,    0,    0,  762,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,  763,  764,  765,
  508,  509,  199,    0,  200,  201,  202,  203,    0,  204,
  766, 1225,    0,    0,    0,    0,    0,  205,  206,  767,
  768,  769,  770,  771,  772,  773,  774,  775,  207,  776,
    0,  209,  210,  211,  777,  778,    0,  779,  780,  781,
    0,  214,  215,  782,  216,    0,    0,  217,  218,    0,
    0,    0,  219,  220,    0,    0,    0,    0,  221,  786,
  223,    0,  787,  225,  226,    0,  227,    0,  788,  789,
  790,    0,  791,  792,    0,  793,  228,  229,  230,  231,
  232,  233,  794,  235,  795,  237,  796,  797,  798,  799,
  800,  801,  802,  803,  804,  805,  806,    0,  807,  808,
  809,  810,  811,  812,    0,  813,  814,  815,  258,    0,
  816,    0,  817,  818,  819,  820,  821,  265,  266,  822,
  268,  823,  824,  825,  272,  826,  274,  827,  276,  828,
    0,  829,  830,  831,  832,    0,  833,  834,  835,  285,
  836,    0,  837,  838,  289,  839,  291,  292,  840,  294,
  295,  296,  297,  298,  299,    0,  841,  301,  302,  303,
  304,  305,    0,  842,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  843,  844,    0,  318,  319,  320,
  845,  846,  847,    0,    0,  324,  848,  849,  850,  851,
  852,  853,  854,  855,  856,    0,  857,  858,  859,  333,
  860,  334,  335,  336,  337,  338,  339,    0,    0,  861,
  862,  863,  864,  865,  866,  867,  868,  869,  870,  871,
  872,  873,  874,  875,  876,  877,  878,    0, 1933,    0,
 1933,    0,    0,    0,    0,    0, 1933,    0,    0, 1933,
 1933,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1933, 1933,    0, 1933,    0, 1933,    0,    0, 1933,    0,
    0, 1933,    0,    0,    0,    0,    0,    0, 1933,    0,
 1933, 1933,    0,    0,    0, 1933,    0, 1933,    0,    0,
 1933,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1933, 1933,    0,    0,    0, 1933,    0, 1933,    0, 1933,
 1933,    0,    0,    0,    0, 1933,    0, 1933,    0,    0,
    0, 1933,    0,    0,    0,    0,    0,    0,    0, 1933,
    0,    0, 1933,    0, 1933,    0,    0,    0, 1933, 1933,
    0, 1933,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1933,    0, 1933,    0, 1933, 1933, 1933, 1933, 1933,
    0, 1933,    0,    0,    0,    0,    0, 1933,    0, 1933,
 1933,    0,    0, 1933, 1933,    0,    0,    0,    0,    0,
 1933,    0,    0, 1933, 1933, 1933, 1933,    0, 1933,    0,
 1933,    0, 1933, 1933, 1933, 1933, 1933,    0,    0,    0,
    0, 1933,    0, 1933,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1933, 1224,    0,    0,    0, 1933,
 1933,    0, 1933,    0,    0,    0, 1933,    0, 1933, 1933,
 1933, 1933,    0, 1933,    0,    0,    0,    0,    0,    0,
    0, 1933, 1933,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1933, 1933,    0, 1933, 1933, 1933,    0,    0,
    0,    0, 1933, 1933,    0, 1933, 1933,    0, 1933,    0,
    0, 1933, 1933,    0,    0,    0, 1933, 1933,    0,    0,
    0,    0, 1933, 1933, 1933,    0, 1933, 1933, 1933,    0,
 1933,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1933, 1933, 1933, 1933, 1933, 1933, 1933, 1933, 1933, 1933,
 1933, 1933, 1933, 1933, 1933, 1933, 1933, 1933, 1933, 1933,
 1933,    0, 1933, 1933, 1933, 1933, 1933, 1933,    0, 1933,
 1933, 1933, 1933,    0, 1933,    0, 1933, 1933, 1933, 1933,
 1933, 1933, 1933, 1933, 1933, 1933, 1933, 1933, 1933, 1933,
 1933, 1933, 1933, 1933,    0, 1933, 1933, 1933, 1933,    0,
 1933, 1933, 1933, 1933, 1933,    0, 1933, 1933, 1933, 1933,
 1933, 1933, 1933, 1933, 1933, 1933, 1933, 1933, 1933,    0,
 1933, 1933, 1933, 1933, 1933, 1933,    0, 1933, 1933, 1933,
 1933, 1933, 1933, 1933, 1933,    0, 1933, 1933, 1933, 1933,
    0, 1933, 1933, 1933, 1933, 1933, 1933,    0,    0, 1933,
 1933, 1933, 1933, 1933, 1933, 1933, 1933, 1933,    0,    0,
    0,    0,    0, 1933,    0, 1933, 1933, 1933, 1933, 1933,
 1933,    0, 1837,    0, 1837,    0,    0,    0,    0,    0,
 1837,    0,    0, 1837, 1837,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1837, 1837,    0, 1837,    0, 1837,
    0,    0, 1837,    0,    0, 1837,    0,    0,    0,    0,
    0,    0, 1837,    0, 1837, 1837,    0,    0,    0, 1837,
    0, 1837,    0,    0, 1837,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1837, 1837,    0,    0,    0, 1837,
    0, 1837,    0, 1837, 1837,    0,    0,    0,    0, 1837,
    0, 1837,    0,    0,    0, 1837,    0,    0,    0,    0,
    0,    0,    0, 1837,    0,    0, 1837,    0, 1837,    0,
    0,    0, 1837, 1837,    0, 1837,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1837,    0, 1837,    0, 1837,
 1837, 1837, 1837, 1837,    0, 1837,    0,    0,    0,    0,
    0, 1837,    0, 1837, 1837,    0,    0, 1837, 1837,    0,
    0,    0,    0,    0, 1837,    0,    0, 1837, 1837, 1837,
 1837,    0, 1837,    0, 1837,    0, 1837, 1837, 1837, 1837,
 1837,    0,    0,    0,    0, 1837,    0, 1837,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1837,    0,
 1572,    0,    0, 1837, 1837,    0, 1837,    0,    0,    0,
 1837,    0, 1837, 1837, 1837, 1837,    0, 1837,  882,    0,
    0,    0,    0,    0,    0, 1837, 1837,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1837, 1837,    0, 1837,
 1837, 1837,    0,    0,    0,    0, 1837, 1837,    0, 1837,
 1837,    0, 1837,    0,    0, 1837, 1837,    0,    0,    0,
 1837, 1837,    0,    0,    0,    0, 1837, 1837, 1837,    0,
 1837, 1837, 1837,    0, 1837,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1837, 1837, 1837, 1837, 1837, 1837,
 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837,
 1837, 1837, 1837, 1837, 1837,    0, 1837, 1837, 1837, 1837,
 1837, 1837,    0, 1837, 1837, 1837, 1837,    0, 1837,    0,
 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837,
 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837,    0, 1837,
 1837, 1837, 1837,    0, 1837, 1837, 1837, 1837, 1837,    0,
 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837,
 1837, 1837, 1837,    0, 1837, 1837, 1837, 1837, 1837, 1837,
    0, 1837, 1837, 1837, 1837, 1837, 1837, 1837, 1837,    0,
 1837, 1837, 1837, 1837,    0, 1837, 1837, 1837, 1837, 1837,
 1837,    0,    0, 1837, 1837, 1837, 1837, 1837, 1837, 1837,
 1837, 1837,    0,    0,    0,    0,    0, 1837,    0, 1837,
 1837, 1837, 1837, 1837, 1837,    0,    0,  138,    0,  139,
    0,    0,    0,    0,    0,  140,    0,    0,  141,  142,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  143,
  144,    0,  145,    0,  146,    0,    0,  147,    0,    0,
  148,    0,    0,    0,    0,    0,    0,  149,    0,  150,
  151,    0,    0,    0,  152,    0,  153,    0,    0,  154,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  155,
  156,    0,    0,    0,  157,    0,  158,    0,  159,    0,
    0,    0,    0,    0,  160,    0,  161,    0,    0,    0,
  162,    0,    0,    0,    0,    0,    0,    0,  163,    0,
    0,  164,    0,  165,    0,    0,    0,  166,  167,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  168,    0,    0,    0,  170,  171,  172,  173,  174,    0,
  175,    0,    0,    0,    0,    0,  176,    0,  177,  178,
    0,    0,  179,  180,    0,    0,    0,    0,    0,  181,
    0,    0,  182,  183,  184,  185,    0,  186,    0,  187,
    0,  188,  189,  190,  191,  192,    0,    0,    0,    0,
  193,    0,  194,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  195, 1279,    0,    0,    0,  196,  197,
    0,  198,    0,    0,    0,  199,    0,  200,  201,  202,
  203,    0,  204,    0,    0,    0,    0,    0,    0,    0,
  205,  206,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  207,  208,    0,  209,  210,  211,    0,    0,    0,
    0,  212,  213,    0,  214,  215,    0,  216,    0,    0,
  217,  218,    0,    0,    0,  219,  220,    0,    0,    0,
    0,  221,  222,  223,    0,  224,  225,  226,    0,  227,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  228,
  229,  230,  231,  232,  233,  234,  235,  236,  237,  238,
  239,  240,  241,  242,  243,  244,  245,  246,  247,  248,
    0,  249,  250,  251,  252,  253,  254,    0,  255,  256,
  257,  258,    0,  259,    0,  260,  261,  262,  263,  264,
  265,  266,  267,  268,  269,  270,  271,  272,  273,  274,
  275,  276,  277,    0,  278,  279,  280,  281,    0,  282,
  283,  284,  285,  286,    0,  287,  288,  289,  290,  291,
  292,  293,  294,  295,  296,  297,  298,  299,    0,  300,
  301,  302,  303,  304,  305,    0,  306,  307,  308,  309,
  310,  311,  312,  313,    0,  314,  315,  316,  317,    0,
  318,  319,  320,  321,  322,  323,    0,    0,  324,  325,
  326,  327,  328,  329,  330,  331,  332,    0,    0,    0,
    0,    0,  333,    0,  334,  335,  336,  337,  338,  339,
    0,  138,    0,  139,    0,    0,    0,    0,    0,  140,
    0,    0,  141,  142,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  143,  144,    0,  145,    0,  146,    0,
    0,  147,    0,    0,  148,    0,    0,    0,    0,    0,
    0,  149,    0,  150,  151,    0,    0,    0,  152,    0,
  153,    0,    0,  154,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  155,  156,    0,    0,    0,  157,    0,
  158,    0,  159,    0,    0,    0,    0,    0,  160,    0,
  161,    0,    0,    0,  162,    0,    0,    0,    0,    0,
    0,    0,  163,    0,    0,  164,    0,  165,    0,    0,
    0,  166,  167,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  168,    0,    0,    0,  170,  171,
  172,  173,  174,    0,  175,    0,    0,    0,    0,    0,
  176,    0,  177,  178,    0,    0,  179,  180,    0,    0,
    0,    0,    0,  181,    0,    0,  182,  183,  184,  185,
    0,  186,    0,  187,    0,  188,  189,  190,  191,  192,
    0,    0,    0,    0,  193,    0,  194,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  195, 1279,    0,
    0,   15,  196,  197,    0,  198,    0,    0,    0,  199,
    0,  200,  201,  202,  203,    0,  204,    0,    0,    0,
    0,    0,    0,    0,  205,  206,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  207,  208,    0,  209,  210,
  211,    0,    0,    0,    0,  212,  213,    0,  214,  215,
    0,  216,    0,    0,  217,  218,    0,    0,    0,  219,
  220,    0,    0,    0,    0,  221,  222,  223,    0,  224,
  225,  226,    0,  227,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  228,  229,  230,  231,  232,  233,  234,
  235,  236,  237,  238,  239,  240,  241,  242,  243,  244,
  245,  246,  247,  248,    0,  249,  250,  251,  252,  253,
  254,    0,  255,  256,  257,  258,    0,  259,    0,  260,
  261,  262,  263,  264,  265,  266,  267,  268,  269,  270,
  271,  272,  273,  274,  275,  276,  277,    0,  278,  279,
  280,  281,    0,  282,  283,  284,  285,  286,    0,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,  297,
  298,  299,    0,  300,  301,  302,  303,  304,  305,    0,
  306,  307,  308,  309,  310,  311,  312,  313,    0,  314,
  315,  316,  317,    0,  318,  319,  320,  321,  322,  323,
    0,    0,  324,  325,  326,  327,  328,  329,  330,  331,
  332,    0,    0,    0,    0,    0,  333,    0,  334,  335,
  336,  337,  338,  339,    0,  138,    0,  139,    0,    0,
    0,    0,    0,  140,    0,    0,  141,  142,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  143,  144,    0,
  145,    0,  146,    0,    0,  147,    0,    0,  148,    0,
    0,    0,    0,    0,    0,  149,    0,  150,  151,    0,
    0,    0,  152,    0,  153,    0,    0,  154,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  155,  156,    0,
    0,    0,  157,    0,  158,    0,  159,    0,    0,    0,
    0,    0,  160,    0,  161,    0,    0,    0,  162,    0,
    0,    0,    0,    0,    0,    0,  163,    0,    0,  164,
    0,  165,    0,    0,    0,  166,  167,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  168,    0,
    0,    0,  170,  171,  172,  173,  174,    0,  175,    0,
    0,    0,    0,    0,  176,    0,  177,  178,    0,    0,
  179,  180,    0,    0,    0,    0,    0,  181,    0,    0,
  182,  183,  184,  185,    0,  186,    0,  187,    0,  188,
  189,  190,  191,  192,    0,    0,    0,    0,  193,    0,
  194,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  195,  678,    0,    0,    0,  196,  197,    0,  198,
    0,    0,    0,  199,    0,  200,  201,  202,  203,    0,
  204,    0,    0,    0,    0,    0,    0,    0,  205,  206,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  207,
  208,    0,  209,  210,  211,    0,    0,    0,    0,  212,
  213,    0,  214,  215,    0,  216,    0,    0,  217,  218,
    0,    0,    0,  219,  220,    0,    0,    0,    0,  221,
  222,  223,    0,  224,  225,  226,    0,  227,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  228,  229,  230,
  231,  232,  233,  234,  235,  236,  237,  238,  239,  240,
  241,  242,  243,  244,  245,  246,  247,  248,    0,  249,
  250,  251,  252,  253,  254,    0,  255,  256,  257,  258,
    0,  259,    0,  260,  261,  262,  263,  264,  265,  266,
  267,  268,  269,  270,  271,  272,  273,  274,  275,  276,
  277,    0,  278,  279,  280,  281,    0,  282,  283,  284,
  285,  286,    0,  287,  288,  289,  290,  291,  292,  293,
  294,  295,  296,  297,  298,  299,    0,  300,  301,  302,
  303,  304,  305,    0,  306,  307,  308,  309,  310,  311,
  312,  313,    0,  314,  315,  316,  317,    0,  318,  319,
  320,  321,  322,  323,    0,    0,  324,  325,  326,  327,
  328,  329,  330,  331,  332,    0,    0,    0,    0,    0,
  333,    0,  334,  335,  336,  337,  338,  339,    0,  138,
    0,  139,    0,    0,    0,    0,    0,  140,    0,    0,
  141,  142,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  143, 2750,    0,  145,    0,  146,    0,    0,  147,
    0,    0,  148,    0,    0,    0,    0,    0,    0,  149,
    0,  150,  151,    0,    0,    0,  152,    0,  153,    0,
    0,  154,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  155,  156,    0,    0,    0,  157,    0,  158,    0,
  159,    0,    0,    0,    0,    0,  160,    0,  161,    0,
    0,    0,  162,    0,    0,    0,    0,    0,    0,    0,
  163,    0,    0,  164,    0,  165,    0,    0,    0,  166,
  167,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  168,    0,    0,    0,  170,  171,  172,  173,
  174,    0,  175,    0,    0,    0,    0,    0,  176,    0,
  177,  178,    0,    0,  179,  180,    0,    0,    0,    0,
    0,  181,    0,    0,  182,  183,  184,  185,    0,  186,
    0,  187,    0,  188,  189,  190,  191,  192,    0,    0,
    0,    0,  193,    0,  194,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  195,    0, 2113,    0,    0,
  196,  197,    0,  198,    0,    0,    0,  199,    0,  200,
  201,  202,  203,    0,  204,    0,    0,    0,    0,    0,
    0,    0,  205,  206,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  207,  208,    0,  209,  210,  211,    0,
    0,    0,    0,  212,  213,    0,  214,  215,    0,  216,
    0,    0,  217,  218,    0,    0,    0,  219,  220,    0,
    0,    0,    0,  221,  222,  223,    0,  224,  225,  226,
    0,  227,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  228,  229,  230,  231,  232,  233,  234,  235,  236,
  237,  238,  239,  240,  241,  242,  243,  244,  245,  246,
  247,  248,    0,  249,  250,  251,  252,  253,  254,    0,
  255,  256,  257,  258,    0,  259,    0,  260,  261,  262,
  263,  264,  265,  266,  267,  268,  269,  270,  271,  272,
  273,  274,  275,  276,  277,    0,  278,  279,  280,  281,
    0,  282,  283,  284,  285,  286,    0,  287,  288,  289,
  290,  291,  292,  293,  294,  295,  296,  297,  298,  299,
    0,  300,  301,  302,  303,  304,  305,    0,  306,  307,
  308,  309,  310,  311,  312,  313,    0,  314,  315,  316,
  317,    0,  318,  319,  320,  321,  322,  323,    0,    0,
  324,  325,  326,  327,  328,  329,  330,  331,  332,    0,
    0,    0,    0,    0,  333,    0,  334,  335,  336,  337,
  338,  339,  138,    0,  139,    0,    0,    0,    0,    0,
  140,    0,    0,  141,  142,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  143,  144,    0,  145,    0,  146,
    0,    0,  147,    0,    0,  148,    0,  746,    0,    0,
    0,    0,  149,    0,  150,  151,    0,    0,    0,  152,
    0,  153,    0,    0,  154,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  155,  156,    0,    0,    0,  157,
    0,  158,    0,  159,    0,    0,    0,    0,    0,  160,
    0,  161,    0,    0,    0,  162,    0,    0,    0,    0,
    0,    0,    0,  163,    0,    0,  164,    0,  165,    0,
    0,    0,  166,  167,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  168,    0,    0,    0,  170,
  171,  172,  173,  174,    0,  175,    0,    0,    0,    0,
    0,  176,    0,  177,  178,    0,    0,  179,  180,    0,
    0,    0,    0,    0,  181,    0,    0,  182,  183,  184,
  185,    0,  186,    0,  187,    0,  188,  189,  190,  191,
  192,    0,    0,    0,    0,  193,    0,  194,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  195,    0,
    0,    0,    0,  196,  197, 1135,  198,    0,    0,    0,
  199,    0,  200,  201,  202,  203,    0,  204,    0,    0,
    0,    0,    0,    0,    0,  205,  206,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  207,  208,    0,  209,
  210,  211,    0,    0,    0,    0,  212,  213,    0,  214,
  215,    0,  216,    0,    0,  217,  218,    0,    0,    0,
  219,  220,    0,    0,    0,    0,  221,  222,  223,    0,
  224,  225,  226,    0,  227,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  228,  229,  230,  231,  232,  233,
  234,  235,  236,  237,  238,  239,  240,  241,  242,  243,
  244,  245,  246,  247,  248,    0,  249,  250,  251,  252,
  253,  254,    0,  255,  256,  257,  258,    0,  259,    0,
  260,  261,  262,  263,  264,  265,  266,  267,  268,  269,
  270,  271,  272,  273,  274,  275,  276,  277,    0,  278,
  279,  280,  281,    0,  282,  283,  284,  285,  286,    0,
  287,  288,  289,  290,  291,  292,  293,  294,  295,  296,
  297,  298,  299,    0,  300,  301,  302,  303,  304,  305,
    0,  306,  307,  308,  309,  310,  311,  312,  313,    0,
  314,  315,  316,  317,    0,  318,  319,  320,  321,  322,
  323,    0,    0,  324,  325,  326,  327,  328,  329,  330,
  331,  332,    0,    0,    0,    0,    0,  333,  860,  334,
  335,  336,  337,  338,  339,    0,    0,  138,    0,  139,
    0,    0,    0,    0,    0,  140,    0,    0,  141,  142,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  143,
  144,    0,  145,    0,  146,    0,    0,  147,    0,    0,
  148,    0,    0,    0,    0,    0,    0,  149,    0,  150,
  151,    0,    0,    0,  152,    0,  153,    0,    0,  154,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  155,
  156,    0,    0,    0,  157,    0,  158,    0,  159,    0,
    0,    0,    0,    0,  160,    0,  161,    0,    0,    0,
  162,    0,    0,    0,    0,    0,    0,    0,  163,    0,
    0,  164,    0,  165,    0,    0,    0,  166,  167,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  168,    0,    0,    0,  170,  171,  172,  173,  174,    0,
  175,    0,    0,    0,    0,    0,  176,    0,  177,  178,
    0,    0,  179,  180,    0,    0,    0,    0,    0,  181,
    0,    0,  182,  183,  184,  185,    0,  186,    0,  187,
    0,  188,  189,  190,  191,  192,    0,    0,    0,    0,
  193,    0,  194,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  195,    0,    0,    0,    0,  196,  197,
    0,  198,  524,    0,    0,  199,    0,  200,  201,  202,
  203,  600,  204,    0,    0,    0,    0,    0,    0,    0,
  205,  206,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  207,  208,    0,  209,  210,  211,    0,    0,    0,
    0,  212,  213,    0,  214,  215,    0,  216,    0,    0,
  217,  218,    0,    0,    0,  219,  220,    0,    0,    0,
    0,  221,  222,  223,    0,  224,  225,  226,    0,  227,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  228,
  229,  230,  231,  232,  233,  234,  235,  236,  237,  238,
  239,  240,  241,  242,  243,  244,  245,  246,  247,  248,
    0,  249,  250,  251,  252,  253,  254,    0,  255,  256,
  257,  258,    0,  259,    0,  260,  261,  262,  263,  264,
  265,  266,  267,  268,  269,  270,  271,  272,  273,  274,
  275,  276,  277,    0,  278,  279,  280,  281,    0,  282,
  283,  284,  285,  286,    0,  287,  288,  289,  290,  291,
  292,  293,  294,  295,  296,  297,  298,  299,    0,  300,
  301,  302,  303,  304,  305,    0,  306,  307,  308,  309,
  310,  311,  312,  313,    0,  314,  315,  316,  317,    0,
  318,  319,  320,  321,  322,  323,    0,    0,  324,  325,
  326,  327,  328,  329,  330,  331,  332,    0,    0,    0,
    0,    0,  333,    0,  334,  335,  336,  337,  338,  339,
  600,    0,  600,    0,    0,    0,    0,    0,  600,    0,
    0,  600,  600,  600,    0,    0,    0,    0,    0,    0,
    0,    0,  600,  600,    0,  600,    0,  600,    0,    0,
  600,    0,    0,  600,    0,    0,    0,    0,    0,  600,
  600,    0,  600,  600,    0,    0,  600,  600,    0,  600,
  600,    0,  600,    0,    0,  600,    0,    0,    0,    0,
    0,    0,  600,  600,    0,    0,    0,  600,  600,  600,
    0,  600,    0,  600,    0,    0,    0,  600,    0,  600,
    0,    0,    0,  600,    0,    0,    0,    0,    0,  600,
    0,  600,  600,    0,  600,    0,  600,    0,  600,    0,
  600,  600,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  600,    0,    0,    0,  600,  600,  600,
  600,  600,    0,  600,    0,  600,  600,    0,    0,  600,
    0,  600,  600,    0,    0,  600,  600,    0,    0,    0,
  600,    0,  600,  600,    0,  600,  600,  600,  600,    0,
  600,    0,  600,    0,  600,  600,  600,  600,  600,    0,
    0,    0,    0,  600,    0,  600,    0,    0,  600,    0,
    0,    0,    0,    0,    0,    0,  600,  600,    0,  600,
  600,  600,  600,    0,  600,    0,    0,    0,  600,    0,
  600,  600,  600,  600, 1758,  600,  600,    0,    0,    0,
    0,    0,    0,  600,  600,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  600,  600,    0,  600,  600,  600,
    0,    0,    0,    0,  600,  600,    0,  600,  600,    0,
  600,    0,    0,  600,  600,    0,    0,    0,  600,  600,
  600,  600,  600,    0,  600,  600,  600,    0,  600,  600,
  600,    0,  600,    0,  600,  600,  600,    0,  600,  600,
    0,  600,  600,  600,  600,  600,  600,  600,  600,  600,
  600,  600,  600,  600,  600,  600,  600,  600,  600,  600,
  600,  600,  600,    0,  600,  600,  600,  600,  600,  600,
    0,  600,  600,  600,  600,    0,  600,    0,  600,  600,
  600,  600,  600,  600,  600,  600,  600,  600,  600,  600,
  600,  600,  600,  600,  600,  600,    0,  600,  600,  600,
  600,    0,  600,  600,  600,  600,  600,    0,  600,  600,
  600,  600,  600,  600,  600,  600,  600,  600,  600,  600,
  600,    0,  600,  600,  600,  600,  600,  600,    0,  600,
  600,  600,  600,  600,  600,  600,  600,    0,  600,  600,
  600,  600,    0,  600,  600,  600,  600,  600,  600,  600,
    0,  600,  600,  600,  600,  600,  600,  600,  600,  600,
    0,    0,    0,    0,    0,  600,    0,  600,  600,  600,
  600,  600,  600,  138,    0,  139,    0,    0,    0,    0,
    0,  140,    0,    0,  141,  142, 1374,    0,    0,    0,
    0,    0,    0,    0,    0,  143,  144,    0,  145,    0,
  146,    0,    0,  147,    0,    0,  148,    0,    0,    0,
    0,    0,    5,  149,    0,  150,  151,    0,    0,    0,
  152,    0, 1744, 1745,    0, 1746,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  155,  156,    0,    0,    0,
 1747, 1748,  158,    0,  159,    0,    9,    0,    0,    0,
  160,    0,  161,    0,    0,    0,  162,    0,    0,    0,
    0,    0,  751,    0,  163,  753,    0,  164,    0,  165,
    0,   10,    0,  166,  167,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  168,    0,    0,    0,
  170,  171,  172,  173,  174,    0,  175,    0,  757, 1749,
    0,    0,  176,    0,  177,  178,    0,    0,  179,  180,
    0,    0,    0,  758,    0,  181,    0,    0,  182,  183,
  184,  185,    0,  186,    0,  187,    0,  188,  189,  190,
  191, 1750,    0,    0,    0,    0,  193,    0,  194,    0,
    0,   14,    0,    0,    0,    0,    0,    0,    0,  195,
 2178,    0,    0,   15,  196,  197,    0,  764,    0,    0,
    0,  199,    0,  200,  201,  202,  203, 1758,  204,  766,
    0,    0,    0,    0,    0,    0,  205,  206,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1751,  776,    0,
  209,  210,  211,    0,    0,    0,    0,  212,  213,    0,
  214,  215,    0,  216,    0,    0,  217, 1752,    0,    0,
    0,  219,  220, 1753, 1754, 1755,    0,  221,  222,  223,
    0,  224,  225,  226,    0,  227,    0,  788,  789,  790,
    0,  791,  792,    0,  793,  228,  229,  230,  231,  232,
  233,  794,  235,  795,  237,  796,  797,  798,  799,  800,
  801,  802,  803,  804,  805,  806,    0,  807,  808,  809,
  810,  811,  254,    0,  813,  814,  815,  258,    0,  816,
    0,  260,  818,  819,  820,  821,  265,  266,  822,  268,
  823,  824,  825,  272,  826,  274,  827,  276,  828,    0,
  829,  830,  831,  832,    0,  833,  834,  835,  285,  836,
    0,  837,  838,  289,  839,  291,  292,  840,  294,  295,
  296,  297,  298,  299,    0,  841,  301,  302,  303,  304,
  305,    0,  842,  307, 1756,  309,  310,  311,  312,  313,
    0,  314,  315,  843,  844,    0,  318,  319,  320,  845,
  846,  847, 1757,    0,  324,  325,  326,  327,  328,  329,
  330,  331,  332,    0,    0,    0,    0,    0,  333,    0,
  334,  335,  336,  337,  338,  339,  138,    0,  139,    0,
    0,    0,    0,    0,  140,    0,    0,  141,  142, 1374,
    0,    0,    0,    0,    0,    0,    0,    0,  143,  144,
    0,  145,    0,  146,    0,    0,  147,    0,    0,  148,
    0,    0,    0,    0,    0,    5,  149,    0,  150,  151,
    0,    0,    0,  152,    0, 1744, 1745,    0, 1746,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  155,  156,
    0,    0,    0, 1747, 1748,  158,    0,  159,    0,    9,
    0,    0,    0,  160,    0,  161,    0,    0,    0,  162,
    0,    0,    0,    0,    0,  751,    0,  163,  753,    0,
  164,    0,  165,    0,   10,    0,  166,  167,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  168,
    0,    0,    0,  170,  171,  172,  173,  174,    0,  175,
    0,  757, 1749,    0,    0,  176,    0,  177,  178,    0,
    0,  179,  180,    0,    0,    0,  758,    0,  181,    0,
    0,  182,  183,  184,  185,    0,  186,    0,  187,    0,
  188,  189,  190,  191, 1750,    0,    0,    0,    0,  193,
    0,  194,    0,    0,   14,    0,    0,    0,    0,    0,
    0,    0,  195,    0,    0,    0,   15,  196,  197,    0,
  764,    0,    0,    0,  199,    0,  200,  201,  202,  203,
  882,  204,  766,    0,    0,    0,    0,    0,    0,  205,
  206,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1751,  776,    0,  209,  210,  211,    0,    0,    0,    0,
  212,  213,    0,  214,  215,    0,  216,    0,    0,  217,
 1752,    0,    0,    0,  219,  220, 1753, 1754, 1755,    0,
  221,  222,  223,    0,  224,  225,  226,    0,  227,    0,
  788,  789,  790,    0,  791,  792,    0,  793,  228,  229,
  230,  231,  232,  233,  794,  235,  795,  237,  796,  797,
  798,  799,  800,  801,  802,  803,  804,  805,  806,    0,
  807,  808,  809,  810,  811,  254,    0,  813,  814,  815,
  258,    0,  816,    0,  260,  818,  819,  820,  821,  265,
  266,  822,  268,  823,  824,  825,  272,  826,  274,  827,
  276,  828,    0,  829,  830,  831,  832,    0,  833,  834,
  835,  285,  836,    0,  837,  838,  289,  839,  291,  292,
  840,  294,  295,  296,  297,  298,  299,    0,  841,  301,
  302,  303,  304,  305,    0,  842,  307, 1756,  309,  310,
  311,  312,  313,    0,  314,  315,  843,  844,    0,  318,
  319,  320,  845,  846,  847, 1757,    0,  324,  325,  326,
  327,  328,  329,  330,  331,  332,    0,    0,    0,    0,
    0,  333,    0,  334,  335,  336,  337,  338,  339,  138,
    0,  139,    0,    0,    0,    0,    0,  140,    0,    0,
  141,  142,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  143,  144,    0,  145,    0,  146,    0,    0,  147,
    0,    0,  148,    0,    0,    0,    0,    0,    0,  149,
    0,  150,  151,    0,    0,    0,  152,    0,  153,    0,
    0,  154,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  155,  156,    0,    0,    0,  157,    0,  158,    0,
  159,    0,    0,    0,    0,    0,  160,    0,  161,    0,
    0,    0,  162,    0,    0,    0,    0,    0,    0,    0,
  163,    0,    0,  164,    0,  165,    0,    0,    0,  166,
  167,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  168,    0,    0,    0,  170,  171,  172,  173,
  174,    0,  175,    0,    0,    0,    0,    0,  176,    0,
  177,  178,    0,    0,  179,  180,    0,    0,    0,    0,
    0,  181,    0,    0,  182,  183,  184,  185,    0,  186,
    0,  187,    0,  188,  189,  190,  191,  192,    0,    0,
    0,    0,  193,    0,  194,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  195,    0,    0,    0,    0,
  196,  197,    0,  198,    0,    0,    0,  199,    0,  200,
  201,  202,  203,    0,  204,    0,    0,    0,    0,    0,
    0,    0,  205,  206,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  207,  208,    0,  209,  210,  211,    0,
    0,    0,    0,  212,  213,    0,  214,  215,    0,  216,
    0,    0,  217,  218,    0,    0,    0,  219,  220,    0,
    0,    0,    0,  221,  222,  223,    0,  224,  225,  226,
    0,  227,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  228,  229,  230,  231,  232,  233,  234,  235,  236,
  237,  238,  239,  240,  241,  242,  243,  244,  245,  246,
  247,  248,    0,  249,  250,  251,  252,  253,  254,    0,
  255,  256,  257,  258,    0,  259,    0,  260,  261,  262,
  263,  264,  265,  266,  267,  268,  269,  270,  271,  272,
  273,  274,  275,  276,  277,    0,  278,  279,  280,  281,
    0,  282,  283,  284,  285,  286,    0,  287,  288,  289,
  290,  291,  292,  293,  294,  295,  296,  297,  298,  299,
    0,  300,  301,  302,  303,  304,  305,    0,  306,  307,
  308,  309,  310,  311,  312,  313,    0,  314,  315,  316,
  317,    0,  318,  319,  320,  321,  322,  323,    0,    0,
  324,  325,  326,  327,  328,  329,  330,  331,  332,    0,
    0,    0,    0,    0,  333,    0,  334,  335,  336,  337,
  338,  339, 1800, 1800, 1740, 1800, 1800, 1800,    0,    0,
    0,    0,    0, 1801, 1801, 1741, 1801, 1801, 1801,    0,
 1800, 1800, 1800,    0,    0,    0,    0,    0,    0,    0,
    0, 1801, 1801, 1801,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1800,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1801,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1846, 1846, 1738, 1846, 1846, 1846,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1846, 1846, 1846,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1846,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1800,    0,    0,    0,    0,    0,    0,    0,
    0, 1800,    0, 1801,    0,    0,    0, 1800,    0,    0,
    0, 1800, 1801,    0, 1800,    0,    0,    0, 1801,    0,
    0,    0, 1801,    0,    0, 1801,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1800, 1869,
 1869, 1739, 1869, 1869, 1869, 1800,    0,    0,    0, 1801,
    0,    0, 1846, 1800,    0,    0, 1801, 1869, 1869, 1869,
    0, 1846,    0,    0, 1801,    0,    0, 1846,    0,    0,
 1800, 1846, 1800,    0, 1846,    0,    0,    0,    0,    0,
    0, 1801, 1800, 1801, 1800, 1800, 1800,    0, 1869,    0,
    0, 1800,    0, 1801,    0, 1801, 1801, 1801,    0,    0,
    0,    0, 1801,    0,    0,    0,    0,    0, 1846,    0,
    0,    0,    0,    0,    0, 1846,    0,    0,    0,    0,
    0,    0,    0, 1846,    0,    0,    0,    0,    0,    0,
    0,    0, 1800,    0,    0,    0,    0,    0,    0,    0,
 1846,    0, 1846, 1801,    0,    0,    0,    0,    0,    0,
    0,    0, 1846,    0, 1846, 1846, 1846,    0,    0,    0,
    0, 1846,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1800, 1800, 1800, 1800, 1800, 1800,
 1800, 1800,    0,    0,    0, 1801, 1801, 1801, 1801, 1801,
 1801, 1801, 1801, 1881, 1881,    0, 1881, 1881, 1881,    0,
    0,    0, 1846,    0,    0,    0,    0,    0,    0,    0,
 1881, 1881, 1881, 1881,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1881,    0, 1846, 1846, 1846, 1846, 1846, 1846,
 1846, 1846,    0,    0,    0,    0,    0,    0,    0, 1869,
    0,    0,    0,    0,    0,    0,    0,    0, 1869,    0,
    0,    0,    0,    0, 1869,    0,    0,    0, 1869,    0,
    0, 1869,    0, 1800,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1801,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1800,
    0,    0,    0,    0,    0, 1869,    0,    0, 1800,    0,
 1801,    0, 1869,    0,    0,    0,    0,    0,    0, 1801,
 1869,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1869,    0, 1869,
    0,    0,    0, 1846,    0,    0,    0,    0,    0, 1869,
    0, 1869, 1869, 1869,    0,    0,    0,    0, 1869,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1846,
    0,    0,    0,    0,    0,    0,    0,    0, 1846,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1869,
    0,    0,    0, 1881,    0, 1881,    0,    0,    0,    0,
    0,    0, 1881,    0,    0,    0,    0,    0, 1881,    0,
    0,    0, 1881,    0,    0, 1881,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1869, 1869, 1869, 1869, 1869, 1869, 1869, 1869,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1881,
    0,    0,    0,    0,    0,    0, 1881,    0,    0,    0,
    0,    0,    0, 1881, 1881,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1881,    0, 1881,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1881,    0, 1881, 1881, 1881,    0, 1881,
    0,    0, 1881,    0,    0,    0,    0,    0,    0,    0,
    0, 1881,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1869,    0,    0, 1881,    0,    0,    0,    0,    0,  138,
    0,  139,    0,    0,    0,    0,    0,  140,    0,    0,
  141,  142,    0,    0,    0,    0, 1869,    0, 1881,    0,
    0,  143,  144,    0,  145, 1869,  146,    0,    0,  147,
    0, 1881,  148,    0,    0,    0,    0,    0,    0,  149,
    0,  150,  151,    0,    0,    0,  152,    0,  153,    0,
    0,  154,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  155,  156,    0,    0,    0,  157,    0,  158,    0,
  159,    0,    0,    0,    0,    0,  160,    0,  161,    0,
    0,    0,  162,    0,    0,    0,    0,    0,    0,    0,
  163,    0,    0,  164,    0,  165,    0,    0,    0,  166,
  167,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  168,    0,    0,    0,  170,  171,  172,  173,
  174,    0,  175,    0,    0,    0,    0,    0,  176,    0,
  177,  178,    0,    0,  179,  180,    0,    0,    0,    0,
    0,  181,    0,    0,  182,  183,  184,  185,    0,  186,
    0,  187,    0,  188,  189,  190,  191,  192,    0,    0,
    0,    0,  193,    0,  194,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  195,    0,    0,    0, 1881,
  196,  197,    0,  198,    0,    0,    0,  199,  377,  200,
  201,  202,  203,  378,  204,  379,  380,  381,  382,  383,
  384,  385,  205,  206,  386,  387,  388,  389,  390,    0,
    0,  391,  392,  207,  208,  393,  209,  210,  211,  394,
  395,  396,  397,  212,  213,  398,  214,  215,  399,  216,
  400,  401,  217,  218,  402,  403,  404,  219,  220,  405,
  406,  407,  408,  221,  222,  223,  409,  224,  225,  226,
  410,  227,  411,  412,  413,  414,  415,  416,  417,  418,
  419,  228,  229,  230,  231,  232,  233,  234,  235,  236,
  237,  238,  239,  240,  241,  242,  243,  244,  245,  246,
  247,  248,  420,  249,  250,  251,  252,  253,  254,  421,
  255,  256,  257,  258,  422,  259,  423,  260,  261,  262,
  263,  264,  265,  266,  267,  268,  269,  270,  271,  272,
  273,  274,  275,  276,  277,  424,  278,  279,  280,  281,
  425,  282,  283,  284,  285,  286,  426,  287,  288,  289,
  290,  291,  292,  293,  294,  295,  296,  297,  298,  299,
  427,  300,  301,  302,  303,  304,  305,    0,  306,  307,
  308,  309,  310,  311,  312,  313,  428,  314,  315,  316,
  317,  429,  318,  319,  320,  321,  322,  323,  430,  431,
  324,  325,  326,  327,  328,  329,  330,  331,  332,  432,
  433,  434,  435,  436,  333,  437,  334,  335,  336,  337,
  338,  339,  438,  439,  440,  441,  442,  443,  444,  445,
  446,  447,  448,  449,  450,  451,  452,  453,  454,  455,
  456,  457,  892,    0,  892,    0,    0,    0,    0,    0,
  892,    0,    0,  892,  892,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  892,  892,    0,  892,    0,  892,
    0,    0,  892,    0,    0,  892,    0,    0,    0,    0,
    0,    0,  892,    0,  892,  892,    0,    0,    0,  892,
    0,  892,    0,    0,  892,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  892,  892,    0,    0,    0,  892,
    0,  892,    0,  892,    0,    0,    0,    0,    0,  892,
    0,  892,    0,    0,    0,  892,    0,    0,    0,    0,
    0,    0,    0,  892,    0,    0,  892,    0,  892,    0,
    0,    0,  892,  892,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  892,    0,    0,    0,  892,
  892,  892,  892,  892,    0,  892,    0,    0,    0,    0,
    0,  892,    0,  892,  892,    0,    0,  892,  892,    0,
    0,    0,    0,    0,  892,    0,    0,  892,  892,  892,
  892,    0,  892,    0,  892,    0,  892,  892,  892,  892,
  892,    0,    0,    0,    0,  892,    0,  892,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  892,    0,
    0,    0,    0,  892,  892,    0,  892,    0,    0,    0,
  892,  892,  892,  892,  892,  892,    0,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,    0,    0,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
    0,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892,  892,  892,  892,  892,  892,
  892,  892,  892,  892,  892, 1753,    0, 1753,    0,    0,
    0,    0,    0, 1753,    0,    0, 1753, 1753,    0,    0,
 1753,    0,    0, 1753,    0,    0,    0, 1753, 1753,    0,
 1753,    0, 1753,    0,    0, 1753,    0, 1755, 1753, 1753,
    0, 1753,    0,    0,    0, 1753,    0, 1753, 1753,    0,
    0,    0, 1753,    0, 1753,    0,    0, 1753,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1753, 1753,    0,
    0,    0, 1753,    0, 1753,    0, 1753,    0,    0, 1753,
    0,    0, 1753,    0, 1753, 1753, 1753, 1753, 1753, 1753,
 1753, 1753,    0, 1753,    0,    0, 1753,    0,    0, 1753,
    0, 1753,    0,    0,    0, 1753, 1753, 1753,    0, 1753,
    0, 1755,    0,    0,    0,    0,    0,    0, 1753,    0,
    0,    0, 1753, 1753, 1753, 1753, 1753,    0, 1753,    0,
    0,    0,    0,    0, 1753,    0, 1753, 1753, 1753,    0,
 1753, 1753,    0,    0,    0,    0,    0, 1753,    0,    0,
 1753, 1753, 1753, 1753, 1753, 1753,    0, 1753,    0, 1753,
 1753, 1753, 1753, 1753,    0,    0,    0,    0, 1753,    0,
 1753,    0,    0,    0,    0,    0, 1753,    0,    0,    0,
    0, 1753,    0,    0,    0,    0, 1753, 1753,    0, 1753,
    0,    0,    0, 1753,    0, 1753, 1753, 1753, 1753,    0,
 1753,    0,    0,    0,    0,    0,    0,    0, 1753, 1753,
 1753, 1753,    0,    0,    0,    0,    0,    0,    0, 1753,
 1753,    0, 1753, 1753, 1753,    0,    0, 1753,    0, 1753,
 1753,    0, 1753, 1753,    0, 1753,    0,    0, 1753, 1753,
    0,    0,    0, 1753, 1753,    0,    0,    0,    0, 1753,
 1753, 1753,    0, 1753, 1753, 1753,    0, 1753,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1753, 1753, 1753,
 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753,
 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753,    0, 1753,
 1753, 1753, 1753, 1753, 1753,    0, 1753, 1753, 1753, 1753,
    0, 1753,    0, 1753, 1753, 1753, 1753, 1753, 1753, 1753,
 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753, 1753,
 1753,    0, 1753, 1753, 1753, 1753,    0, 1753, 1753, 1753,
 1753, 1753,    0, 1753, 1753, 1753, 1753, 1753, 1753, 1753,
 1753, 1753, 1753, 1753, 1753, 1753,    0, 1753, 1753, 1753,
 1753, 1753, 1753,    0, 1753, 1753, 1753, 1753, 1753, 1753,
 1753, 1753,    0, 1753, 1753, 1753, 1753, 1755, 1753, 1753,
 1753, 1753, 1753, 1753,    0,    0, 1753, 1753, 1753, 1753,
 1753, 1753, 1753, 1753, 1753,    0, 1753,    0,    0,    0,
 1753,    0, 1753, 1753, 1753, 1753, 1753, 1753,  138,    0,
  139,    0,    0,    0,    0,    0,  140,    0,    0,  141,
  142,    0,    0, 1060,    0,    0, 1061,    0,    0,    0,
  143,  144,    0,  145,    0,  146,    0,    0,  147,    0,
    0,  148, 1062,    0, 1063,    0,    0,    0,  149,    0,
  150,  151,    0,    0,    0,  152,    0,  153,    0,    0,
  154,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  155,  156,    0,    0,    0,  157,    0,  158,    0,  159,
    0,    0, 1064,    0,    0,  160,    0,  161, 1065, 1066,
 1067,  162, 1068, 1069, 1070,    0, 1071,    0,    0,  163,
    0,    0,  164,    0,  165,    0,    0,    0,  166,  167,
 1072,    0, 1073,    0,    0,    0,    0,    0,    0,    0,
    0,  168,    0,    0,    0,  170,  171,  172,  173,  174,
    0,  175,    0,    0,    0,    0,    0,  176,    0,  177,
  178, 1074,    0,  179,  180,    0,    0,    0,    0,    0,
  181,    0,    0,  182,  183,  184,  185, 1075,  186,    0,
  187,    0,  188,  189,  190,  191,  192,    0,    0,    0,
    0,  193,    0,  194,    0,    0,    0,    0,    0, 1076,
    0,    0,    0,    0,  195,    0,    0,    0,    0,  196,
  197,    0,  198,    0,    0,    0,  199,    0,  200,  201,
  202,  203,    0, 2153,    0,    0,    0,    0,    0,    0,
    0,  205,  206, 1077, 1078,    0,    0,    0,    0,    0,
    0,    0,  207,  208,    0,  209,  210,  211,    0,    0,
 1079,    0,  212,  213,    0,  214,  215,    0,  216,    0,
    0,  217,  218,    0,    0,    0,  219,  220,    0,    0,
    0,    0,  221,  222,  223,    0,  224,  225,  226,    0,
  227,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  228,  229,  230,  231,  232,  233,  234,  235,  236,  237,
  238,  239,  240,  241,  242,  243,  244,  245,  246,  247,
  248,    0,  249,  250,  251,  252,  253,  254,    0,  255,
  256,  257,  258,    0,  259,    0,  260,  261,  262,  263,
  264,  265,  266,  267,  268,  269,  270,  271,  272,  273,
  274,  275,  276,  277,    0,  278,  279,  280,  281,    0,
  282,  283,  284,  285,  286,    0,  287,  288,  289,  290,
  291,  292,  293,  294,  295,  296,  297,  298,  299,    0,
  300,  301,  302,  303,  304,  305,    0,  306,  307,  308,
  309,  310,  311,  312,  313,    0,  314,  315,  316,  317,
    0,  318,  319,  320,  321,  322,  323,    0,    0,  324,
  325,  326,  327,  328,  329,  330,  331,  332,    0, 1080,
    0,    0,    0,  333,    0,  334,  335,  336,  337,  338,
  339,  138,    0,  139,    0,    0,    0,    0,    0,  140,
    0,    0,  141,  142,    0,    0, 1060,    0,    0, 1061,
    0,    0,    0,  143,  144,    0,  145,    0,  146,    0,
    0,  147,    0,    0,  148, 1062,    0, 1063,    0,    0,
    0,  149,    0,  150,  151,    0,    0,    0,  152,    0,
  153,    0,    0,  154,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  155,  156,    0,    0,    0,  157,    0,
  158,    0,  159,    0,    0, 1064,    0,    0,  160,    0,
  161, 1065, 1066, 1067,  162, 1068, 1069, 1070,    0, 1071,
    0,    0,  163,    0,    0,  164,    0,  165,    0,    0,
    0,  166,  167, 1072,    0, 1073,    0,    0,    0,    0,
    0,    0,    0,    0,  168,    0,    0,    0,  170,  171,
  172,  173,  174,    0,  175,    0,    0,    0,    0,    0,
  176,    0,  177,  178, 1074,    0,  179,  180,    0,    0,
    0,    0,    0,  181,    0,    0,  182,  183,  184,  185,
 1075,  186,    0,  187,    0,  188,  189,  190,  191,  192,
    0,    0,    0,    0,  193,    0,  194,    0,    0,    0,
    0,    0, 1076,    0,    0,    0,    0,  195,    0,    0,
    0,    0,  196,  197,    0,  198,    0,    0,    0,  199,
    0,  200,  201,  202,  203,    0, 2300,    0,    0,    0,
    0,    0,    0,    0,  205,  206, 1077, 1078,    0,    0,
    0,    0,    0,    0,    0,  207,  208,    0,  209,  210,
  211,    0,    0, 1079,    0,  212,  213,    0,  214,  215,
    0,  216,    0,    0,  217,  218,    0,    0,    0,  219,
  220,    0,    0,    0,    0,  221,  222,  223,    0,  224,
  225,  226,    0,  227,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  228,  229,  230,  231,  232,  233,  234,
  235,  236,  237,  238,  239,  240,  241,  242,  243,  244,
  245,  246,  247,  248,    0,  249,  250,  251,  252,  253,
  254,    0,  255,  256,  257,  258,    0,  259,    0,  260,
  261,  262,  263,  264,  265,  266,  267,  268,  269,  270,
  271,  272,  273,  274,  275,  276,  277,    0,  278,  279,
  280,  281,    0,  282,  283,  284,  285,  286,    0,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,  297,
  298,  299,    0,  300,  301,  302,  303,  304,  305,    0,
  306,  307,  308,  309,  310,  311,  312,  313,    0,  314,
  315,  316,  317,    0,  318,  319,  320,  321,  322,  323,
    0,    0,  324,  325,  326,  327,  328,  329,  330,  331,
  332,    0, 1080,    0,    0,    0,  333,    0,  334,  335,
  336,  337,  338,  339,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,    0,  141,  142,    0,    0, 1060,
    0,    0, 1061,    0,    0,    0,  143, 2750,    0,  145,
    0,  146,    0,    0,  147,    0,    0,  148, 1062,    0,
 1063,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,    0,  154,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  155,  156,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0, 1064,    0,
    0,  160,    0,  161, 1065, 1066, 1067,  162, 1068, 1069,
 1070,    0, 1071,    0,    0,  163,    0,    0,  164,    0,
  165,    0,    0,    0,  166,  167, 1072,    0, 1073,    0,
    0,    0,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,    0,
    0,    0,    0,  176,    0,  177,  178, 1074,    0,  179,
  180,    0,    0,    0,    0,    0,  181,    0,    0,  182,
  183,  184,  185, 1075,  186,    0,  187,    0,  188,  189,
  190,  191,  192,    0,    0,    0,    0,  193,    0,  194,
    0,    0,    0,    0,    0, 1076,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,    0,  198,    0,
    0,    0,  199,    0,  200,  201,  202,  203,    0,  204,
    0,    0,    0,    0,    0,    0,    0,  205,  206, 1077,
 1078,    0,    0,    0,    0,    0,    0,    0,  207,  208,
    0,  209,  210,  211,    0,    0, 1079,    0,  212,  213,
    0,  214,  215,    0,  216,    0,    0,  217,  218,    0,
    0,    0,  219,  220,    0,    0,    0,    0,  221,  222,
  223,    0,  224,  225,  226,    0,  227,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  228,  229,  230,  231,
  232,  233,  234,  235,  236,  237,  238,  239,  240,  241,
  242,  243,  244,  245,  246,  247,  248,    0,  249,  250,
  251,  252,  253,  254,    0,  255,  256,  257, 2751,    0,
  259,    0,  260,  261,  262,  263,  264,  265,  266,  267,
  268,  269,  270,  271,  272,  273,  274,  275,  276,  277,
    0,  278,  279,  280,  281,    0,  282,  283,  284,  285,
  286,    0,  287,  288,  289,  290,  291,  292,  293,  294,
  295,  296,  297,  298,  299,    0,  300,  301,  302,  303,
  304,  305,    0,  306,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  316,  317,    0,  318,  319,  320,
  321,  322,  323,    0,    0,  324,  325,  326,  327,  328,
  329,  330,  331,  332,    0, 1080,    0,    0,    0,  333,
    0,  334,  335,  336,  337,  338,  339,  138,    0,  139,
    0,    0,    0,    0,    0,  140,    0,    0,  141,  142,
    0,    0, 1060,    0,    0, 1061,    0,    0,    0,  143,
  144,    0,  145,    0,  146,    0,    0,  147,    0,    0,
  148, 1062,    0, 1063,    0,    0,    0,  149,    0,  150,
  151,    0,    0,    0,  152,    0,  153,    0,    0,  154,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  155,
  156,    0,    0,    0,  157,    0,  158,    0,  159,    0,
    0, 1064,    0,    0,  160,    0,  161, 1065, 1066, 1067,
  162, 1068, 1069, 1070,    0, 1071,    0,    0,  163,    0,
    0,  164,    0,  165,    0,    0,    0,  166,  167, 1072,
    0, 1073,    0,    0,    0,    0,    0,    0,    0,    0,
  168,    0,    0,    0,  170,  171,  172,  173,  174,    0,
  175,    0,    0,    0,    0,    0,  176,    0,  177,  178,
 1074,    0,  179,  180,    0,    0,    0,    0,    0,  181,
    0,    0,  182,  183,  184,  185, 1075,  186,    0,  187,
    0,  188,  189,  190,  191,  192,    0,    0,    0,    0,
  193,    0,  194,    0,    0,    0,    0,    0, 1076,    0,
    0,    0,    0,  195,    0,    0,    0,    0,  196,  197,
    0,  198,    0,    0,    0,  199,    0,  200,  201,  202,
  203,    0,  204,    0,    0,    0,    0,    0,    0,    0,
  205,  206, 1077, 1078,    0,    0,    0,    0,    0,    0,
    0,  207,  208,    0,  209,  210,  211,    0,    0, 1079,
    0,  212,  213,    0,  214,  215,    0,  216,    0,    0,
  217,  218,    0,    0,    0,  219,  220,    0,    0,    0,
    0,  221,  222,  223,    0,  224,  225,  226,    0,  227,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  228,
  229,  230,  231,  232,  233,  234,  235,  236,  237,  238,
  239,  240,  241,  242,  243,  244,  245,  246,  247,  248,
    0,  249,  250,  251,  252,  253,  254,    0,  255,  256,
  257,  258,    0,  259,    0,  260,  261,  262,  263,  264,
  265,  266,  267,  268,  269,  270,  271,  272,  273,  274,
  275,  276,  277,    0,  278,  279,  280,  281,    0,  282,
  283,  284,  285,  286,    0,  287,  288,  289,  290,  291,
  292,  293,  294,  295,  296,  297,  298,  299,    0,  300,
  301,  302,  303,  304,  305,    0,  306,  307,  308,  309,
  310,  311,  312,  313,    0,  314,  315,  316,  317,    0,
  318,  319,  320,  321,  322,  323,    0,    0,  324,  325,
  326,  327,  328,  329,  330,  331,  332,    0, 1080,    0,
    0,    0,  333,    0,  334,  335,  336,  337,  338,  339,
   44,    0,   44,   32,   32,    0,    0,    0,   44,    0,
    0,   44,   44,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   44,   44,    0,   44,    0,   44,    0,   32,
   44,    0,    0,   44,    0,    0,    0,    0,    0,   32,
   44,    0,   44,   44,   32,    0,    0,   44,    0,   44,
   32,    0,   44,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   44,   44,    0,    0,    0,   44,    0,   44,
    0,   44,    0,   32,    0,    0,    0,   44,    0,   44,
    0,    0,    0,   44,    0,    0,    0,    0,    0,    0,
    0,   44,    0,    0,   44,    0,   44,    0,    0,    0,
   44,   44,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   44,    0,    0,    0,   44,   44,   44,
   44,   44,    0,   44,    0,    0,    0,    0,    0,   44,
    0,   44,   44,    0,   32,   44,   44,    0,    0,    0,
    0,    0,   44,   32,    0,   44,   44,   44,   44,    0,
   44,    0,   44,    0,   44,   44,   44,   44,   44,    0,
    0,    0,    0,   44,    0,   44,    0,    0,   32,    0,
    0,    0,    0,    0,    0,    0,   44,    0,    0,    0,
    0,   44,   44,    0,   44,    0,    0,    0,   44,    0,
   44,   44,   44,   44,    0,   44,    0,    0,    0,    0,
    0,    0,    0,   44,   44,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   44,   44,    0,   44,   44,   44,
    0,    0,    0,    0,   44,   44,    0,   44,   44,    0,
   44,    0,    0,   44,   44,    0,    0,    0,   44,   44,
    0,    0,    0,    0,   44,   44,   44,    0,   44,   44,
   44,    0,   44,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   44,   44,   44,   44,   44,   44,   44,   44,
   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
   44,   44,   44,    0,   44,   44,   44,   44,   44,   44,
    0,   44,   44,   44,   44,    0,   44,    0,   44,   44,
   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
   44,   44,   44,   44,   44,   44,    0,   44,   44,   44,
   44,    0,   44,   44,   44,   44,   44,    0,   44,   44,
   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
   44,    0,   44,   44,   44,   44,   44,   44,    0,   44,
   44,   44,   44,   44,   44,   44,   44,    0,   44,   44,
   44,   44,    0,   44,   44,   44,   44,   44,   44,    0,
    0,   44,   44,   44,   44,   44,   44,   44,   44,   44,
    0,    0,    0,    0,    0,    0,    0,   44,   44,   44,
   44,   44,   44,  121,    0,  121,    0,  119,    0,    0,
    0,  121,    0,    0,  121,  121,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  121,  121,    0,  121,    0,
  121,    0,  119,  121,    0,    0,  121,    0,    0,    0,
    0,    0,  119,  121,    0,  121,  121,  119,    0,    0,
  121,    0,  121,  119,    0,  121,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  121,  121,    0,    0,    0,
  121,    0,  121,    0,  121,    0,  119,    0,    0,    0,
  121,    0,  121,    0,    0,    0,  121,    0,    0,    0,
    0,    0,    0,    0,  121,    0,    0,  121,    0,  121,
    0,    0,    0,  121,  121,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  121,    0,    0,    0,
  121,  121,  121,  121,  121,    0,  121,    0,    0,    0,
    0,    0,  121,    0,  121,  121,    0,  119,  121,  121,
    0,    0,    0,    0,    0,  121,  119,    0,  121,  121,
  121,  121,    0,  121,    0,  121,    0,  121,  121,  121,
  121,  121,    0,    0,    0,    0,  121,    0,  121,    0,
    0,  119,    0,    0,    0,    0,    0,    0,    0,  121,
    0,    0,    0,    0,  121,  121,    0,  121,    0,    0,
    0,  121,    0,  121,  121,  121,  121,    0,  121,    0,
    0,    0,    0,    0,    0,    0,  121,  121,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  121,  121,    0,
  121,  121,  121,    0,    0,    0,    0,  121,  121,    0,
  121,  121,    0,  121,    0,    0,  121,  121,    0,    0,
    0,  121,  121,    0,    0,    0,    0,  121,  121,  121,
    0,  121,  121,  121,    0,  121,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  121,  121,  121,  121,  121,
  121,  121,  121,  121,  121,  121,  121,  121,  121,  121,
  121,  121,  121,  121,  121,  121,    0,  121,  121,  121,
  121,  121,  121,    0,  121,  121,  121,  121,    0,  121,
    0,  121,  121,  121,  121,  121,  121,  121,  121,  121,
  121,  121,  121,  121,  121,  121,  121,  121,  121,    0,
  121,  121,  121,  121,    0,  121,  121,  121,  121,  121,
    0,  121,  121,  121,  121,  121,  121,  121,  121,  121,
  121,  121,  121,  121,    0,  121,  121,  121,  121,  121,
  121,    0,  121,  121,  121,  121,  121,  121,  121,  121,
    0,  121,  121,  121,  121,    0,  121,  121,  121,  121,
  121,  121,    0,    0,  121,  121,  121,  121,  121,  121,
  121,  121,  121,    0,    0,    0,    0,    0,    0,    0,
  121,  121,  121,  121,  121,  121,  138,    0,  139,    0,
    0,    0,    0,    0,  140,    0,    0,  141,  142,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  143,  144,
    0,  145,    0,  146,    0,    0,  147,    0,    0,  148,
    0,    0,    0,    0,    0,    0,  149,    0,  150,  151,
    0,    0,    0,  152,    0,  153,    0,    0,  154,    0,
    0,    0,    0,    0,    0, 2880,    0,    0,  155,  156,
    0, 2881,    0,  157,    0,  158,    0,  159,    0,    0,
    0,    0,    0,  160,    0,  161,    0,    0,    0,  162,
    0,    0,    0,    0,    0,    0,    0,  163,    0,    0,
  164,    0,  165,    0,    0,    0,  166,  167,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  168,
    0,    0,    0,  170,  171,  172,  173,  174,    0,  175,
    0,    0,    0,    0,    0,  176, 2882,  177,  178,    0,
    0,  179,  180,    0,    0,    0,    0,    0,  181,    0,
    0,  182,  183,  184,  185,    0,  186,    0,  187,    0,
  188,  189,  190,  191,  192,    0,    0,    0,    0,  193,
 2883,  194,    0,    0,    0, 2884,    0,    0,    0,    0,
    0, 2885,  195,    0,    0,    0,    0,  196,  197,    0,
  198,    0,    0,    0,  199,    0,  200,  201,  202, 2886,
    0,  204,    0,    0,    0,    0,    0,    0,    0,  205,
  206,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  207,  208,    0,  209,  210,  211,    0,    0,    0,    0,
  212,  213,    0,  214,  215,    0,  216,    0,    0,  217,
  218,    0,    0,    0,  219,  220,    0,    0,    0,    0,
  221,  222,  223,    0,  224,  225,  226,    0,  227,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  228,  229,
  230,  231,  232,  233,  234,  235,  236,  237,  238,  239,
  240,  241,  242,  243,  244,  245,  246,  247,  248,    0,
  249,  250,  251,  252,  253,  254,    0,  255,  256,  257,
  258,    0,  259,    0,  260,  261,  262,  263,  264,  265,
  266,  267,  268,  269,  270,  271,  272,  273,  274,  275,
  276,  277,    0,  278,  279,  280,  281,    0,  282,  283,
  284,  285,  286,    0,  287,  288,  289,  290,  291,  292,
  293,  294,  295,  296,  297,  298,  299,    0,  300,  301,
  302,  303,  304,  305,    0,  306,  307,  308,  309,  310,
  311,  312,  313,    0, 2887,  315,  316,  317,    0,  318,
  319,  320,  321,  322,  323,    0,    0,  324,  325,  326,
  327,  328,  329,  330,  331,  332,    0,    0,    0,    0,
    0,  333,    0,  334,  335,  336,  337,  338,  339,  138,
    0,  139,    0,    0,    0,    0,    0,  140,    0,    0,
  141,  142,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  143,  144,    0,  145,    0,  146,    0,    0,  147,
    0,    0,  148,    0,    0,    0,    0,    0,    0,  149,
    0,  150,  151,    0,    0,    0,  152,    0,  153,    0,
    0,  154,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  155,  156,    0,    0,    0,  157,    0,  158,    0,
  159, 3005,    0,    0,    0,    0,  160,    0,  161,    0,
    0,    0,  162,    0,    0,    0,    0,    0,    0,    0,
  163,    0,    0,  164,    0,  165,    0,    0,    0,  166,
  167,    0, 3006,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  168,    0, 3007,    0,  170,  171,  172,  173,
  174,    0,  175,    0,    0,    0,    0,    0,  176,    0,
  177,  178,    0,    0,  179,  180,    0,    0,    0,    0,
    0,  181,    0,    0,  182,  183,  184,  185,    0,  186,
    0,  187,    0,  188,  189,  190,  191,  192,    0,    0,
    0,    0,  193,    0,  194,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  195,    0,    0,    0,    0,
  196,  197,    0,  198,    0,    0,    0,  199,    0,  200,
  201,  202,  203,    0,  204,    0,    0,    0,    0,    0,
    0,    0,  205,  206,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  207,  208,    0,  209,  210,  211,    0,
    0,    0,    0,  212,  213,    0,  214,  215,    0,  216,
    0,    0,  217,  218,    0,    0,    0,  219,  220,    0,
    0,    0,    0,  221,  222,  223,    0,  224,  225,  226,
    0,  227,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  228,  229,  230,  231,  232,  233,  234,  235,  236,
  237,  238,  239,  240,  241,  242,  243,  244,  245,  246,
  247,  248,    0,  249,  250,  251,  252,  253,  254,    0,
  255,  256,  257,  258,    0,  259,    0,  260,  261,  262,
  263,  264,  265,  266,  267,  268,  269,  270,  271,  272,
  273,  274,  275,  276,  277,    0,  278,  279,  280,  281,
    0,  282,  283,  284,  285,  286,    0,  287,  288,  289,
  290,  291,  292,  293,  294,  295,  296,  297,  298,  299,
    0,  300,  301,  302,  303,  304,  305,    0,  306,  307,
  308,  309,  310,  311,  312,  313,    0,  314,  315,  316,
  317,    0,  318,  319,  320,  321,  322,  323,    0,    0,
  324,  325,  326,  327,  328,  329,  330,  331,  332,    0,
    0,    0,    0,    0,  333,    0,  334,  335,  336,  337,
  338,  339,  138,    0,  139,    0,    0,    0, 3106,    0,
  140,    0,    0,  141,  142,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  143,  144,    0,  145,    0,  146,
    0,    0,  147,    0,    0,  148,    0,    0,    0,    0,
    0,    0,  149,    0,  150,  151,    0,    0,    0,  152,
    0,  153,    0,    0,  154,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  155,  156,    0, 3107,    0,  157,
    0,  158,    0,  159,    0,    0,    0,    0,    0,  160,
    0,  161,    0,    0,    0,  162,    0,    0,    0,    0,
    0,    0,    0,  163,    0,    0,  164,    0,  165,    0,
    0,    0,  166,  167,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  168,    0,    0,    0,  170,
  171,  172,  173,  174,    0,  175,    0,    0,    0,    0,
    0,  176,    0,  177,  178,    0,    0,  179,  180,    0,
    0,    0,    0,    0,  181,    0,    0,  182,  183,  184,
  185,    0,  186,    0,  187,    0,  188,  189,  190,  191,
  192,    0,    0,    0,    0,  193,    0,  194,    0,    0,
    0, 3108,    0,    0,    0,    0,    0,    0,  195,    0,
    0,    0,    0,  196,  197,    0,  198,    0,    0,    0,
  199,    0,  200,  201,  202,  203,    0,  204,    0,    0,
    0,    0,    0,    0,    0,  205,  206,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  207,  208,    0,  209,
  210,  211,    0,    0,    0,    0,  212,  213,    0,  214,
  215,    0,  216,    0,    0,  217,  218,    0,    0,    0,
  219,  220,    0,    0,    0,    0,  221,  222,  223,    0,
  224,  225,  226,    0,  227,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  228,  229,  230,  231,  232,  233,
  234,  235,  236,  237,  238,  239,  240,  241,  242,  243,
  244,  245,  246,  247,  248,    0,  249,  250,  251,  252,
  253,  254,    0,  255,  256,  257,  258,    0,  259,    0,
  260,  261,  262,  263,  264,  265,  266,  267,  268,  269,
  270,  271,  272,  273,  274,  275,  276,  277,    0,  278,
  279,  280,  281,    0,  282,  283,  284,  285,  286,    0,
  287,  288,  289,  290,  291,  292,  293,  294,  295,  296,
  297,  298,  299,    0,  300,  301,  302,  303,  304,  305,
    0,  306,  307,  308,  309,  310,  311,  312,  313,    0,
  314,  315,  316,  317,    0,  318,  319,  320,  321,  322,
  323,    0,    0,  324,  325,  326,  327,  328,  329,  330,
  331,  332,    0,    0,    0,    0,    0,  333,    0,  334,
  335,  336,  337,  338,  339,  138,    0,  139,    0,    0,
    0,    0,    0,  140,    0,    0,  141,  142,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  143,  144,    0,
  145,    0,  146,    0,    0,  147,    0,    0,  148,    0,
  746,    0,    0,    0,    0,  149,    0,  150,  151,    0,
    0,    0,  152,    0,  153,    0,    0,  154,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  155,  156,    0,
    0,    0,  157,    0,  158,    0,  159,    0,    0,    0,
    0,    0,  160,    0,  161,    0,    0,    0,  162,    0,
    0,    0,    0,    0,    0,    0,  163,    0,    0,  164,
    0,  165,    0,    0,    0,  166,  167,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  168,    0,
    0,    0,  170,  171,  172,  173,  174,    0,  175,    0,
    0,    0,    0,    0,  176,    0,  177,  178,    0,    0,
  179,  180,    0,    0,    0,    0,    0,  181,    0,    0,
  182,  183,  184,  185,    0,  186,    0,  187,    0,  188,
  189,  190,  191,  192,    0,    0,    0,    0,  193,    0,
  194,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  195,    0,    0,    0,    0,  196,  197,    0,  198,
    0,    0,    0,  199,    0,  200,  201,  202,  203,    0,
  204,    0,    0,    0,    0,    0,    0,    0,  205,  206,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  207,
  208,    0,  209,  210,  211,    0,    0,    0,    0,  212,
  213,    0,  214,  215,    0,  216,    0,    0,  217,  218,
    0,    0,    0,  219,  220,    0,    0,    0,    0,  221,
  222,  223,    0,  224,  225,  226,    0,  227,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  228,  229,  230,
  231,  232,  233,  234,  235,  236,  237,  238,  239,  240,
  241,  242,  243,  244,  245,  246,  247,  248,    0,  249,
  250,  251,  252,  253,  254,    0,  255,  256,  257,  258,
    0,  259,    0,  260,  261,  262,  263,  264,  265,  266,
  267,  268,  269,  270,  271,  272,  273,  274,  275,  276,
  277,    0,  278,  279,  280,  281,    0,  282,  283,  284,
  285,  286,    0,  287,  288,  289,  290,  291,  292,  293,
  294,  295,  296,  297,  298,  299,    0,  300,  301,  302,
  303,  304,  305,    0,  306,  307,  308,  309,  310,  311,
  312,  313,    0,  314,  315,  316,  317,    0,  318,  319,
  320,  321,  322,  323,    0,    0,  324,  325,  326,  327,
  328,  329,  330,  331,  332,    0,    0,    0,    0,    0,
  333,  860,  334,  335,  336,  337,  338,  339,  138,    0,
  139,    0,    0,    0,    0,    0,  140,    0,    0,  141,
  142,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  143,  144,    0,  145,    0,  146,    0,    0,  147,    0,
    0,  148,    0,    0,    0,    0,    0,    0,  149,    0,
  150,  151,    0,    0,    0,  152,    0,  153,    0,    0,
  154,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  155,  156,    0,    0,    0,  157,    0,  158,    0,  159,
    0,    0,    0,    0,    0,  160,    0,  161,    0,    0,
    0,  162,    0,    0,    0,    0,    0,    0,    0,  163,
    0,    0,  164,    0,  165,    0,    0,    0,  166,  167,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  168,    0,    0,    0,  170,  171,  172,  173,  174,
    0,  175,    0,    0,    0,    0,    0,  176,    0,  177,
  178,    0,    0,  179,  180,    0,    0,    0,    0,    0,
  181,    0,    0,  182,  183,  184,  185,    0,  186,    0,
  187,    0,  188,  189,  190,  191,  192,    0,    0,    0,
    0,  193,    0,  194,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  195,    0,    0,    0,    0,  196,
  197,    0,  198,    0,  508,  509,  199,    0,  200,  201,
  202,  203,    0,  204,    0,    0,    0,    0,    0,    0,
    0,  205,  206,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  207,  208,    0,  209,  210,  211,    0,    0,
    0,    0,  212,  213,    0,  214,  215,    0,  216,    0,
    0,  217,  218,    0,    0,    0,  219,  220,    0,    0,
    0,    0,  221,  222,  223,    0,  224,  225,  226,    0,
  227,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  228,  229,  230,  231,  232,  233,  234,  235,  236,  237,
  238,  239,  240,  241,  242,  243,  244,  245,  246,  247,
  248,    0,  249,  250,  251,  252,  253,  254,    0,  255,
  256,  257,  258,    0,  259,    0,  260,  261,  262,  263,
  264,  265,  266,  267,  268,  269,  270,  271,  272,  273,
  274,  275,  276,  277,    0,  278,  279,  280,  281,    0,
  282,  283,  284,  285,  286,    0,  287,  288,  289,  290,
  291,  292,  293,  294,  295,  296,  297,  298,  299,    0,
  300,  301,  302,  303,  304,  305,    0,  306,  307,  308,
  309,  310,  311,  312,  313,    0,  314,  315,  316,  317,
    0,  318,  319,  320,  321,  322,  323,    0,    0,  324,
  325,  326,  327,  328,  329,  330,  331,  332,    0,    0,
    0,    0,    0,  333,    0,  334,  335,  336,  337,  338,
  339,  138,    0,  139,    0,    0,    0,    0,    0,  140,
    0,    0,  141,  142,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  143,  144,    0,  145,    0,  146,    0,
    0,  147,    0,    0,  148,    0,    0,    0,    0,    0,
    0,  149,    0,  150,  151,    0,    0,    0,  152,    0,
  153,    0,    0,  154,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  155,  156,    0,    0,    0,  157,    0,
  158,    0,  159,    0,    0,    0,    0,    0,  160, 1471,
  161,    0,    0,    0,  162,    0,    0,    0,    0,    0,
    0,    0,  163,    0,    0,  164,    0,  165,    0, 1472,
    0,  166,  167,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  168,    0,    0,    0,  170,  171,
  172,  173,  174,    0,  175,    0,    0,    0,    0,    0,
  176,    0,  177,  178,    0,    0,  179,  180,    0,    0,
    0,    0,    0,  181,    0,    0,  182,  183,  184,  185,
    0,  186,    0, 2833,    0,  188,  189,  190,  191,  192,
    0,    0,    0,    0,  193,    0,  194,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  195,    0,    0,
    0,    0,  196,  197,    0,  198,    0,    0,    0,  199,
    0,  200,  201,  202,  203,    0,  204,    0,    0,    0,
    0,    0,    0,    0,  205,  206,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  207,  208,    0,  209,  210,
  211,    0,    0,    0,    0,  212,  213,    0,  214,  215,
    0,  216,    0,    0,  217,  218,    0,    0,    0,  219,
  220,    0,    0,    0,    0,  221,  222,  223,    0,  224,
  225,  226,    0,  227,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  228,  229,  230,  231,  232,  233,  234,
  235,  236,  237,  238,  239,  240,  241,  242,  243,  244,
  245,  246,  247,  248,    0,  249,  250,  251,  252,  253,
  254,    0,  255,  256,  257,  258,    0, 2834,    0,  260,
  261,  262,  263,  264,  265,  266,  267,  268,  269,  270,
  271,  272,  273,  274,  275,  276,  277,    0,  278,  279,
  280,  281,    0,  282,  283,  284,  285,  286,    0,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,  297,
  298,  299,    0,  300,  301,  302,  303,  304,  305,    0,
  306,  307,  308,  309,  310,  311,  312,  313,    0,  314,
  315,  316,  317,    0,  318,  319,  320,  321,  322,  323,
    0,    0,  324,  325,  326,  327,  328,  329,  330,  331,
  332,    0,    0,    0,    0,    0,  333,    0,  334,  335,
  336,  337,  338,  339,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,    0,  141,  142,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,    0,    0,  147,    0,    0,  148,    0,    0,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,    0,  154,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  155,  156,    0, 3107,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,    0,    0,    0,    0,  163,    0,    0,  164,    0,
  165,    0,    0,    0,  166,  167,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,    0,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,    0,    0,  181,    0,    0,  182,
  183,  184,  185,    0,  186,    0,  187,    0,  188,  189,
  190,  191,  192,    0,    0,    0,    0,  193,    0,  194,
    0,    0,    0, 3108,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,    0,  198,    0,
    0,    0,  199,    0,  200,  201,  202,  203,    0,  204,
    0,    0,    0,    0,    0,    0,    0,  205,  206,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  207,  208,
    0,  209,  210,  211,    0,    0,    0,    0,  212,  213,
    0,  214,  215,    0,  216,    0,    0,  217,  218,    0,
    0,    0,  219,  220,    0,    0,    0,    0,  221,  222,
  223,    0,  224,  225,  226,    0,  227,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  228,  229,  230,  231,
  232,  233,  234,  235,  236,  237,  238,  239,  240,  241,
  242,  243,  244,  245,  246,  247,  248,    0,  249,  250,
  251,  252,  253,  254,    0,  255,  256,  257,  258,    0,
  259,    0,  260,  261,  262,  263,  264,  265,  266,  267,
  268,  269,  270,  271,  272,  273,  274,  275,  276,  277,
    0,  278,  279,  280,  281,    0,  282,  283,  284,  285,
  286,    0,  287,  288,  289,  290,  291,  292,  293,  294,
  295,  296,  297,  298,  299,    0,  300,  301,  302,  303,
  304,  305,    0,  306,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  316,  317,    0,  318,  319,  320,
  321,  322,  323,    0,    0,  324,  325,  326,  327,  328,
  329,  330,  331,  332,    0,    0,    0,    0,    0,  333,
    0,  334,  335,  336,  337,  338,  339,  138,    0,  139,
    0,    0,    0,    0,    0,  140,    0,    0,  141,  142,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  143,
  144,    0,  145,    0,  146,    0,    0,  147,    0,    0,
  148,    0,    0,    0,    0,    0,    0,  149,    0,  150,
  151,    0,    0,    0,  152,    0,  153,    0,    0,  154,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  155,
  156,    0,    0,    0,  157,    0,  158,    0,  159,    0,
    0,    0,    0,    0,  160,    0,  161,    0,    0,    0,
  162,    0,    0,    0,    0,    0,    0,    0,  163,    0,
    0,  164,    0,  165,    0,    0,    0,  166,  167,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  168,  169,    0,    0,  170,  171,  172,  173,  174,    0,
  175,    0,    0,    0,    0,    0,  176,    0,  177,  178,
    0,    0,  179,  180,    0,    0,    0,    0,    0,  181,
    0,    0,  182,  183,  184,  185,    0,  186,    0,  187,
    0,  188,  189,  190,  191,  192,    0,    0,    0,    0,
  193,    0,  194,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  195,    0,    0,    0,    0,  196,  197,
    0,  198,    0,    0,    0,  199,    0,  200,  201,  202,
  203,    0,  204,    0,    0,    0,    0,    0,    0,    0,
  205,  206,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  207,  208,    0,  209,  210,  211,    0,    0,    0,
    0,  212,  213,    0,  214,  215,    0,  216,    0,    0,
  217,  218,    0,    0,    0,  219,  220,    0,    0,    0,
    0,  221,  222,  223,    0,  224,  225,  226,    0,  227,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  228,
  229,  230,  231,  232,  233,  234,  235,  236,  237,  238,
  239,  240,  241,  242,  243,  244,  245,  246,  247,  248,
    0,  249,  250,  251,  252,  253,  254,    0,  255,  256,
  257,  258,    0,  259,    0,  260,  261,  262,  263,  264,
  265,  266,  267,  268,  269,  270,  271,  272,  273,  274,
  275,  276,  277,    0,  278,  279,  280,  281,    0,  282,
  283,  284,  285,  286,    0,  287,  288,  289,  290,  291,
  292,  293,  294,  295,  296,  297,  298,  299,    0,  300,
  301,  302,  303,  304,  305,    0,  306,  307,  308,  309,
  310,  311,  312,  313,    0,  314,  315,  316,  317,    0,
  318,  319,  320,  321,  322,  323,    0,    0,  324,  325,
  326,  327,  328,  329,  330,  331,  332,    0,    0,    0,
    0,    0,  333,    0,  334,  335,  336,  337,  338,  339,
  138,    0,  139,    0,    0,    0,    0,    0,  140,    0,
    0,  141,  142,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  143,  144,    0,  145,    0,  146,    0,    0,
  147,    0,    0,  148,    0,    0,    0,    0,    0,    0,
  149,    0,  150,  151,    0,    0,    0,  152,    0,  153,
    0,    0,  154,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  155,  156,    0,    0,    0,  157,    0,  158,
    0,  159,    0,    0,    0,    0,    0,  160,    0,  161,
    0,    0,    0,  162,    0,    0,    0,    0,    0,    0,
    0,  163,    0,    0,  164,    0,  165,    0,    0,    0,
  166,  167,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  168,    0,    0,    0,  170,  171,  172,
  173,  174,    0,  175,    0,    0,    0,    0,    0,  176,
    0,  177,  178,    0,    0,  179,  180,    0,    0,    0,
    0,    0,  181,    0,    0,  182,  183,  184,  185,    0,
  186,    0,  187,    0,  188,  189,  190,  191,  192,    0,
    0,    0,    0,  193,    0,  194,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  195,    0,    0,    0,
    0,  196,  197,    0,  198,    0,    0,    0,  199,    0,
  200,  201,  202,  203,    0,  204,    0,    0,    0,    0,
    0,    0,    0,  205,  206,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  207,  208,    0,  209,  210,  211,
    0,    0,    0,    0,  212,  213,    0,  214,  215,    0,
  216,    0,    0,  217,  218,    0,    0,    0,  219,  220,
    0,    0,    0,    0,  221,  222,  223,    0,  224,  225,
  226,    0,  227,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  228,  229,  230,  231,  232,  233,  234,  235,
  236,  237,  238,  239,  240,  241,  242,  243,  244,  245,
  246,  247,  248,    0,  249,  250,  251,  252,  253,  254,
    0,  255,  256,  257,  258,    0,  259,    0,  260,  261,
  262,  263,  264,  265,  266,  267,  268,  269,  270,  271,
  272,  273,  274,  275,  276,  277,  345,  278,  279,  280,
  281,    0,  282,  283,  284,  285,  286,    0,  287,  288,
  289,  290,  291,  292,  293,  294,  295,  296,  297,  298,
  299,    0,  300,  301,  302,  303,  304,  305,    0,  306,
  307,  308,  309,  310,  311,  312,  313,    0,  314,  315,
  316,  317,    0,  318,  319,  320,  321,  322,  323,    0,
    0,  324,  325,  326,  327,  328,  329,  330,  331,  332,
    0,    0,    0,    0,    0,  333,    0,  334,  335,  336,
  337,  338,  339,  138,    0,  139,    0,    0,    0,    0,
    0,  140,    0,    0,  141,  142,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  143,  144,    0,  145, 2196,
  146,    0,    0,  147,    0,    0,  148,    0,    0,    0,
    0,    0,    0,  149,    0,  150,  151,    0,    0,    0,
  152,    0,  153,    0,    0,  154,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  155,  156,    0,    0,    0,
  157,    0,  158,    0,  159,    0,    0,    0,    0,    0,
  160,    0,  161,    0,    0,    0,  162,    0,    0,    0,
    0,    0,    0,    0,  163,    0,    0,  164,    0,  165,
    0,    0,    0,  166,  167,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  168,    0,    0,    0,
  170,  171,  172,  173,  174,    0,  175,    0,    0,    0,
    0,    0,  176,    0,  177,  178,    0,    0,  179,  180,
    0,    0,    0,    0,    0,  181,    0,    0,  182,  183,
  184,  185,    0,  186,    0,  187,    0,  188,  189,  190,
  191,  192,    0,    0,    0,    0,  193,    0,  194,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  195,
    0,    0,    0,    0,  196,  197,    0,  198,    0,    0,
    0,  199,    0,  200,  201,  202,  203,    0,  204,    0,
    0,    0,    0,    0,    0,    0,  205,  206,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  207,  208,    0,
  209,  210,  211,    0,    0,    0,    0,  212,  213,    0,
  214,  215,    0,  216,    0,    0,  217,  218,    0,    0,
    0,  219,  220,    0,    0,    0,    0,  221,  222,  223,
    0,  224,  225,  226,    0,  227,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  228,  229,  230,  231,  232,
  233,  234,  235,  236,  237,  238,  239,  240,  241,  242,
  243,  244,  245,  246,  247,  248,    0,  249,  250,  251,
  252,  253,  254,    0,  255,  256,  257,  258,    0,  259,
    0,  260,  261,  262,  263,  264,  265,  266,  267,  268,
  269,  270,  271,  272,  273,  274,  275,  276,  277,    0,
  278,  279,  280,  281,    0,  282,  283,  284,  285,  286,
    0,  287,  288,  289,  290,  291,  292,  293,  294,  295,
  296,  297,  298,  299,    0,  300,  301,  302,  303,  304,
  305,    0,  306,  307,  308,  309,  310,  311,  312,  313,
    0,  314,  315,  316,  317,    0,  318,  319,  320,  321,
  322,  323,    0,    0,  324,  325,  326,  327,  328,  329,
  330,  331,  332,    0,    0,    0,    0,    0,  333,    0,
  334,  335,  336,  337,  338,  339,  138,    0,  139,    0,
    0,    0,    0,    0,  140,    0,    0,  141,  142,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  143,  144,
    0,  145, 2487,  146,    0,    0,  147,    0,    0,  148,
    0,    0,    0,    0,    0,    0,  149,    0,  150,  151,
    0,    0,    0,  152,    0,  153,    0,    0,  154,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  155,  156,
    0,    0,    0,  157,    0,  158,    0,  159,    0,    0,
    0,    0,    0,  160,    0,  161,    0,    0,    0,  162,
    0,    0,    0,    0,    0,    0,    0,  163,    0,    0,
  164,    0,  165,    0,    0,    0,  166,  167,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  168,
    0,    0,    0,  170,  171,  172,  173,  174,    0,  175,
    0,    0,    0,    0,    0,  176,    0,  177,  178,    0,
    0,  179,  180,    0,    0,    0,    0,    0,  181,    0,
    0,  182,  183,  184,  185,    0,  186,    0,  187,    0,
  188,  189,  190,  191,  192,    0,    0,    0,    0,  193,
    0,  194,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  195,    0,    0,    0,    0,  196,  197,    0,
  198,    0,    0,    0,  199,    0,  200,  201,  202,  203,
    0,  204,    0,    0,    0,    0,    0,    0,    0,  205,
  206,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  207,  208,    0,  209,  210,  211,    0,    0,    0,    0,
  212,  213,    0,  214,  215,    0,  216,    0,    0,  217,
  218,    0,    0,    0,  219,  220,    0,    0,    0,    0,
  221,  222,  223,    0,  224,  225,  226,    0,  227,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  228,  229,
  230,  231,  232,  233,  234,  235,  236,  237,  238,  239,
  240,  241,  242,  243,  244,  245,  246,  247,  248,    0,
  249,  250,  251,  252,  253,  254,    0,  255,  256,  257,
  258,    0,  259,    0,  260,  261,  262,  263,  264,  265,
  266,  267,  268,  269,  270,  271,  272,  273,  274,  275,
  276,  277,    0,  278,  279,  280,  281,    0,  282,  283,
  284,  285,  286,    0,  287,  288,  289,  290,  291,  292,
  293,  294,  295,  296,  297,  298,  299,    0,  300,  301,
  302,  303,  304,  305,    0,  306,  307,  308,  309,  310,
  311,  312,  313,    0,  314,  315,  316,  317,    0,  318,
  319,  320,  321,  322,  323,    0,    0,  324,  325,  326,
  327,  328,  329,  330,  331,  332,    0,    0,    0,    0,
    0,  333,    0,  334,  335,  336,  337,  338,  339,  138,
    0,  139,    0,    0,    0,    0,    0,  140,    0,    0,
  141,  142,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  143,  144,    0,  145,    0,  146,    0,    0,  147,
    0,    0,  148,    0,    0,    0,    0,    0,    0,  149,
    0,  150,  151,    0,    0,    0,  152,    0,  153,    0,
    0,  154, 2574,    0,    0,    0,    0,    0,    0,    0,
    0,  155,  156,    0,    0,    0,  157,    0,  158,    0,
  159,    0,    0,    0,    0,    0,  160,    0,  161,    0,
    0,    0,  162,    0,    0,    0,    0,    0,    0,    0,
  163,    0,    0,  164,    0,  165,    0,    0,    0,  166,
  167,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  168,    0,    0,    0,  170,  171,  172,  173,
  174,    0,  175,    0,    0,    0,    0,    0,  176,    0,
  177,  178,    0,    0,  179,  180,    0,    0,    0,    0,
    0,  181,    0,    0,  182,  183,  184,  185,    0,  186,
    0,  187,    0,  188,  189,  190,  191,  192,    0,    0,
    0,    0,  193,    0,  194,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  195,    0,    0,    0,    0,
  196,  197,    0,  198,    0,    0,    0,  199,    0,  200,
  201,  202,  203,    0,  204,    0,    0,    0,    0,    0,
    0,    0,  205,  206,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  207,  208,    0,  209,  210,  211,    0,
    0,    0,    0,  212,  213,    0,  214,  215,    0,  216,
    0,    0,  217,  218,    0,    0,    0,  219,  220,    0,
    0,    0,    0,  221,  222,  223,    0,  224,  225,  226,
    0,  227,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  228,  229,  230,  231,  232,  233,  234,  235,  236,
  237,  238,  239,  240,  241,  242,  243,  244,  245,  246,
  247,  248,    0,  249,  250,  251,  252,  253,  254,    0,
  255,  256,  257,  258,    0,  259,    0,  260,  261,  262,
  263,  264,  265,  266,  267,  268,  269,  270,  271,  272,
  273,  274,  275,  276,  277,    0,  278,  279,  280,  281,
    0,  282,  283,  284,  285,  286,    0,  287,  288,  289,
  290,  291,  292,  293,  294,  295,  296,  297,  298,  299,
    0,  300,  301,  302,  303,  304,  305,    0,  306,  307,
  308,  309,  310,  311,  312,  313,    0,  314,  315,  316,
  317,    0,  318,  319,  320,  321,  322,  323,    0,    0,
  324,  325,  326,  327,  328,  329,  330,  331,  332,    0,
    0,    0,    0,    0,  333,    0,  334,  335,  336,  337,
  338,  339,  138,    0,  139,    0,    0,    0,    0,    0,
  140,    0,    0,  141,  142,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  143,  144,    0,  145,    0,  146,
    0,    0,  147,    0,    0,  148,    0,    0,    0,    0,
    0,    0,  149,    0,  150,  151,    0,    0,    0,  152,
    0,  153,    0,    0,  154,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  155,  156,    0,    0,    0,  157,
    0,  158,    0,  159,    0,    0,    0,    0,    0,  160,
    0,  161,    0,    0,    0,  162,    0,    0,    0,    0,
    0,    0,    0,  163,    0,    0,  164,    0,  165,    0,
    0,    0,  166,  167,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  168,    0,    0,    0,  170,
  171,  172,  173,  174,    0,  175,    0,    0,    0,    0,
    0,  176,    0,  177,  178,    0,    0,  179,  180,    0,
    0,    0,    0,    0,  181,    0,    0,  182,  183,  184,
  185,    0,  186,    0,  187,    0,  188,  189,  190,  191,
  192,    0,    0,    0,    0,  193,    0,  194,    0,    0,
    0, 2673,    0,    0,    0,    0,    0,    0,  195,    0,
    0,    0,    0,  196,  197,    0,  198,    0,    0,    0,
  199,    0,  200,  201,  202,  203,    0,  204,    0,    0,
    0,    0,    0,    0,    0,  205,  206,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  207,  208,    0,  209,
  210,  211,    0,    0,    0,    0,  212,  213,    0,  214,
  215,    0,  216,    0,    0,  217,  218,    0,    0,    0,
  219,  220,    0,    0,    0,    0,  221,  222,  223,    0,
  224,  225,  226,    0,  227,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  228,  229,  230,  231,  232,  233,
  234,  235,  236,  237,  238,  239,  240,  241,  242,  243,
  244,  245,  246,  247,  248,    0,  249,  250,  251,  252,
  253,  254,    0,  255,  256,  257,  258,    0,  259,    0,
  260,  261,  262,  263,  264,  265,  266,  267,  268,  269,
  270,  271,  272,  273,  274,  275,  276,  277,    0,  278,
  279,  280,  281,    0,  282,  283,  284,  285,  286,    0,
  287,  288,  289,  290,  291,  292,  293,  294,  295,  296,
  297,  298,  299,    0,  300,  301,  302,  303,  304,  305,
    0,  306,  307,  308,  309,  310,  311,  312,  313,    0,
  314,  315,  316,  317,    0,  318,  319,  320,  321,  322,
  323,    0,    0,  324,  325,  326,  327,  328,  329,  330,
  331,  332,    0,    0,    0,    0,    0,  333,    0,  334,
  335,  336,  337,  338,  339,  138,    0,  139,    0,    0,
    0,    0,    0,  140,    0,    0,  141,  142,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  143,  144,    0,
  145,    0,  146,    0,    0,  147,    0,    0,  148,    0,
    0,    0,    0,    0,    0,  149,    0,  150,  151,    0,
    0,    0,  152,    0,  153,    0,    0,  154,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  155,  156,    0,
    0,    0,  157,    0,  158,    0,  159,    0,    0,    0,
    0,    0,  160,    0,  161,    0,    0,    0,  162,    0,
    0,    0,    0,    0,    0,    0,  163,    0,    0,  164,
    0,  165,    0,    0,    0,  166,  167,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  168,    0,
    0,    0,  170,  171,  172,  173,  174,    0,  175,    0,
    0,    0,    0,    0,  176,    0,  177,  178,    0,    0,
  179,  180,    0,    0,    0,    0,    0,  181,    0,    0,
  182,  183,  184,  185,    0,  186,    0,  187,    0,  188,
  189,  190,  191,  192,    0,    0,    0,    0,  193,    0,
  194,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  195,    0,    0,    0,    0,  196,  197,    0,  198,
    0,    0,    0,  199,    0,  200,  201,  202,  203, 2723,
  204,    0,    0,    0,    0,    0,    0,    0,  205,  206,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  207,
  208,    0,  209,  210,  211,    0,    0,    0,    0,  212,
  213,    0,  214,  215,    0,  216,    0,    0,  217,  218,
    0,    0,    0,  219,  220,    0,    0,    0,    0,  221,
  222,  223,    0,  224,  225,  226,    0,  227,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  228,  229,  230,
  231,  232,  233,  234,  235,  236,  237,  238,  239,  240,
  241,  242,  243,  244,  245,  246,  247,  248,    0,  249,
  250,  251,  252,  253,  254,    0,  255,  256,  257,  258,
    0,  259,    0,  260,  261,  262,  263,  264,  265,  266,
  267,  268,  269,  270,  271,  272,  273,  274,  275,  276,
  277,    0,  278,  279,  280,  281,    0,  282,  283,  284,
  285,  286,    0,  287,  288,  289,  290,  291,  292,  293,
  294,  295,  296,  297,  298,  299,    0,  300,  301,  302,
  303,  304,  305,    0,  306,  307,  308,  309,  310,  311,
  312,  313,    0,  314,  315,  316,  317,    0,  318,  319,
  320,  321,  322,  323,    0,    0,  324,  325,  326,  327,
  328,  329,  330,  331,  332,    0,    0,    0,    0,    0,
  333,    0,  334,  335,  336,  337,  338,  339,  138,    0,
  139,    0,    0,    0,    0,    0,  140,    0,    0,  141,
  142,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  143,  144,    0,  145,    0,  146,    0,    0,  147,    0,
    0,  148,    0,    0,    0,    0,    0,    0,  149,    0,
  150,  151,    0,    0,    0,  152,    0,  153,    0,    0,
  154,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  155,  156,    0,    0,    0,  157,    0,  158,    0,  159,
    0,    0,    0,    0,    0,  160,    0,  161,    0,    0,
    0,  162,    0,    0,    0,    0,    0,    0,    0,  163,
    0,    0,  164,    0,  165,    0,    0,    0,  166,  167,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  168,    0,    0,    0,  170,  171,  172,  173,  174,
    0,  175,    0,    0,    0,    0,    0,  176,    0,  177,
  178,    0,    0,  179,  180,    0,    0,    0,    0,    0,
  181,    0,    0,  182,  183,  184,  185,    0,  186,    0,
  187,    0,  188,  189,  190,  191,  192,    0,    0,    0,
    0,  193,    0,  194,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  195,    0,    0,    0,    0,  196,
  197,    0,  198,    0,    0,    0,  199,    0,  200,  201,
  202,  203, 2840,  204,    0,    0,    0,    0,    0,    0,
    0,  205,  206,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  207,  208,    0,  209,  210,  211,    0,    0,
    0,    0,  212,  213,    0,  214,  215,    0,  216,    0,
    0,  217,  218,    0,    0,    0,  219,  220,    0,    0,
    0,    0,  221,  222,  223,    0,  224,  225,  226,    0,
  227,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  228,  229,  230,  231,  232,  233,  234,  235,  236,  237,
  238,  239,  240,  241,  242,  243,  244,  245,  246,  247,
  248,    0,  249,  250,  251,  252,  253,  254,    0,  255,
  256,  257,  258,    0,  259,    0,  260,  261,  262,  263,
  264,  265,  266,  267,  268,  269,  270,  271,  272,  273,
  274,  275,  276,  277,    0,  278,  279,  280,  281,    0,
  282,  283,  284,  285,  286,    0,  287,  288,  289,  290,
  291,  292,  293,  294,  295,  296,  297,  298,  299,    0,
  300,  301,  302,  303,  304,  305,    0,  306,  307,  308,
  309,  310,  311,  312,  313,    0,  314,  315,  316,  317,
    0,  318,  319,  320,  321,  322,  323,    0,    0,  324,
  325,  326,  327,  328,  329,  330,  331,  332,    0,    0,
    0,    0,    0,  333,    0,  334,  335,  336,  337,  338,
  339,  138,    0,  139,    0,    0,    0,    0,    0,  140,
    0,    0,  141,  142,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  143,  144,    0,  145,    0,  146,    0,
    0,  147,    0,    0,  148,    0,    0,    0,    0,    0,
    0,  149,    0,  150,  151, 3118,    0,    0,  152,    0,
  153,    0,    0,  154,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  155,  156,    0,    0,    0,  157,    0,
  158,    0,  159,    0,    0,    0,    0,    0,  160,    0,
  161,    0,    0,    0,  162,    0,    0,    0,    0,    0,
    0,    0,  163,    0,    0,  164,    0,  165,    0,    0,
    0,  166,  167,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  168,    0,    0,    0,  170,  171,
  172,  173,  174,    0,  175,    0,    0,    0,    0,    0,
  176,    0,  177,  178,    0,    0,  179,  180,    0,    0,
    0,    0,    0,  181,    0,    0,  182,  183,  184,  185,
    0,  186,    0,  187,    0,  188,  189,  190,  191,  192,
    0,    0,    0,    0,  193,    0,  194,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  195,    0,    0,
    0,    0,  196,  197,    0,  198,    0,    0,    0,  199,
    0,  200,  201,  202,  203,    0,  204,    0,    0,    0,
    0,    0,    0,    0,  205,  206,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  207,  208,    0,  209,  210,
  211,    0,    0,    0,    0,  212,  213,    0,  214,  215,
    0,  216,    0,    0,  217,  218,    0,    0,    0,  219,
  220,    0,    0,    0,    0,  221,  222,  223,    0,  224,
  225,  226,    0,  227,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  228,  229,  230,  231,  232,  233,  234,
  235,  236,  237,  238,  239,  240,  241,  242,  243,  244,
  245,  246,  247,  248,    0,  249,  250,  251,  252,  253,
  254,    0,  255,  256,  257,  258,    0,  259,    0,  260,
  261,  262,  263,  264,  265,  266,  267,  268,  269,  270,
  271,  272,  273,  274,  275,  276,  277,    0,  278,  279,
  280,  281,    0,  282,  283,  284,  285,  286,    0,  287,
  288,  289,  290,  291,  292,  293,  294,  295,  296,  297,
  298,  299,    0,  300,  301,  302,  303,  304,  305,    0,
  306,  307,  308,  309,  310,  311,  312,  313,    0,  314,
  315,  316,  317,    0,  318,  319,  320,  321,  322,  323,
    0,    0,  324,  325,  326,  327,  328,  329,  330,  331,
  332,    0,    0,    0,    0,    0,  333,    0,  334,  335,
  336,  337,  338,  339,  138,    0,  139,    0,    0,    0,
    0,    0,  140,    0,    0,  141,  142,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  143,  144,    0,  145,
    0,  146,    0,    0,  147,    0,    0,  148,    0,    0,
    0,    0,    0,    0,  149,    0,  150,  151,    0,    0,
    0,  152,    0,  153,    0,    0,  154,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  155,  156,    0,    0,
    0,  157,    0,  158,    0,  159,    0,    0,    0,    0,
    0,  160,    0,  161,    0,    0,    0,  162,    0,    0,
    0,    0,    0,    0,    0,  163,    0,    0,  164,    0,
  165,    0,    0,    0,  166,  167,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  168,    0,    0,
    0,  170,  171,  172,  173,  174,    0,  175,    0,    0,
    0,    0,    0,  176,    0,  177,  178,    0,    0,  179,
  180,    0,    0,    0,    0,    0,  181,    0,    0,  182,
  183,  184,  185,    0,  186,    0,  187,    0,  188,  189,
  190,  191,  192,    0,    0,    0,    0,  193,    0,  194,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  195,    0,    0,    0,    0,  196,  197,    0,  198,    0,
 3206,    0,  199,    0,  200,  201,  202,  203,    0,  204,
    0,    0,    0,    0,    0,    0,    0,  205,  206,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  207,  208,
    0,  209,  210,  211,    0,    0,    0,    0,  212,  213,
    0,  214,  215,    0,  216,    0,    0,  217,  218,    0,
    0,    0,  219,  220,    0,    0,    0,    0,  221,  222,
  223,    0,  224,  225,  226,    0,  227,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  228,  229,  230,  231,
  232,  233,  234,  235,  236,  237,  238,  239,  240,  241,
  242,  243,  244,  245,  246,  247,  248,    0,  249,  250,
  251,  252,  253,  254,    0,  255,  256,  257,  258,    0,
  259,    0,  260,  261,  262,  263,  264,  265,  266,  267,
  268,  269,  270,  271,  272,  273,  274,  275,  276,  277,
    0,  278,  279,  280,  281,    0,  282,  283,  284,  285,
  286,    0,  287,  288,  289,  290,  291,  292,  293,  294,
  295,  296,  297,  298,  299,    0,  300,  301,  302,  303,
  304,  305,    0,  306,  307,  308,  309,  310,  311,  312,
  313,    0,  314,  315,  316,  317,    0,  318,  319,  320,
  321,  322,  323,    0,    0,  324,  325,  326,  327,  328,
  329,  330,  331,  332,    0,    0,    0,    0,    0,  333,
    0,  334,  335,  336,  337,  338,  339,  138,    0,  139,
    0,    0,    0,    0,    0,  140,    0,    0,  141,  142,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  143,
  144,    0,  145,    0,  146,    0,    0,  147,    0,    0,
  148,    0,    0,    0,    0,    0,    0,  149,    0,  150,
  151,    0,    0,    0,  152,    0,  153,    0,    0,  154,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  155,
  156,    0,    0,    0,  157,    0,  158,    0,  159,    0,
    0,    0,    0,    0,  160,    0,  161,    0,    0,    0,
  162,    0,    0,    0,    0,    0,    0,    0,  163,    0,
    0,  164,    0,  165,    0,    0,    0,  166,  167,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  168,    0,    0,    0,  170,  171,  172,  173,  174,    0,
  175,    0,    0,    0,    0,    0,  176,    0,  177,  178,
    0,    0,  179,  180,    0,    0,    0,    0,    0,  181,
    0,    0,  182,  183,  184,  185,    0,  186,    0,  187,
    0,  188,  189,  190,  191,  192,    0,    0,    0,    0,
  193,    0,  194,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  195,    0,    0,    0,    0,  196,  197,
    0,  198,    0,    0,    0,  199,    0,  200,  201,  202,
  203,    0,  204,    0,    0,    0,    0,    0,    0,    0,
  205,  206,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  207,  208,    0,  209,  210,  211,    0,    0,    0,
    0,  212,  213,    0,  214,  215,    0,  216,    0,    0,
  217,  218,    0,    0,    0,  219,  220,    0,    0,    0,
    0,  221,  222,  223,    0,  224,  225,  226,    0,  227,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  228,
  229,  230,  231,  232,  233,  234,  235,  236,  237,  238,
  239,  240,  241,  242,  243,  244,  245,  246,  247,  248,
    0,  249,  250,  251,  252,  253,  254,    0,  255,  256,
  257,  258,    0,  259,    0,  260,  261,  262,  263,  264,
  265,  266,  267,  268,  269,  270,  271,  272,  273,  274,
  275,  276,  277,    0,  278,  279,  280,  281,    0,  282,
  283,  284,  285,  286,    0,  287,  288,  289,  290,  291,
  292,  293,  294,  295,  296,  297,  298,  299,    0,  300,
  301,  302,  303,  304,  305,    0,  306,  307,  308,  309,
  310,  311,  312,  313,    0,  314,  315,  316,  317,    0,
  318,  319,  320,  321,  322,  323,    0,    0,  324,  325,
  326,  327,  328,  329,  330,  331,  332,    0,    0,    0,
    0,    0,  333,    0,  334,  335,  336,  337,  338,  339,
  138,    0,  139,    0,    0,    0,    0,    0,  140,    0,
    0,  141,  142,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  143,  144,    0,  145,    0,  146,    0,    0,
  147,    0,    0,  148,    0,    0,    0,    0,    0,    0,
  149,    0,  150,  151,    0,    0,    0,  152,    0,  153,
    0,    0,  154,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  155,  156,    0,    0,    0,  157,    0,  158,
    0,  159,    0,    0,    0,    0,    0,  160,    0,  161,
    0,    0,    0,  162,    0,    0,    0,    0,    0,    0,
    0,  163,    0,    0,  164,    0,  165,    0,    0,    0,
  166,  167,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  168,    0,    0,    0,  170,  171,  172,
  173,  174,    0,  175,    0,    0,    0,    0,    0,  176,
    0,  177,  178,    0,    0,  179,  180,    0,    0,    0,
    0,    0,  181,    0,    0,  182,  183,  184,  185,    0,
  186,    0,  187,    0,  188,  189,  190,  191,  192,    0,
    0,    0,    0,  193,    0,  194,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  195,    0,    0,    0,
    0,  196,  197,    0,  198,    0,    0,    0,  199,    0,
  200,  201,  202,  203,    0,  204,    0,    0,    0,    0,
    0,    0,    0,  205,  206,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  207,  208,    0,  209,  210,  211,
    0,    0,    0,    0,  212,  213,    0,  214,  215,    0,
  216,    0,    0,  217,  218,    0,    0,    0,  219,  220,
    0,    0,    0,    0,  221,  222,  223,    0,  224,  225,
  226,    0,  227,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  228,  229,  230,  231,  232,  233,  234,  235,
  236,  237,  238,  239,  240,  241,  242,  243,  244,  245,
  246,  247,  248,    0,  249,  250,  251,  252,  253,  254,
    0,  255,  256,  257,  258,    0,  259,    0,  260,  261,
  262,  263,  264,  265,  266,  267,  268,  269,  270,  271,
  272,  273,  274,  275,  276,  277,    0,  278,  279,  280,
  281,    0,  282,  283,  284,  285,  286,    0,  287,  288,
  289,  290,  291,  292,  293,  294,  295,  296,  297,  298,
  299,    0,  300,  301,  302,  303,  304,  305,    0,  306,
  544,  308,  309,  310,  311,  312,  313,    0,  314,  315,
  316,  317,    0,  318,  319,  320,  321,  322,  323,    0,
    0,  324,  325,  326,  327,  328,  329,  330,  331,  332,
    0,    0,    0,    0,    0,  333,    0,  334,  335,  336,
  337,  338,  339,  138,    0,  139,    0,    0,    0,    0,
    0,  140,    0,    0,  141,  142,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  143,  144,    0,  145,    0,
  146,    0,    0,  147,    0,    0,  148,    0,    0,    0,
    0,    0,    0,  149,    0,  150,  151,    0,    0,    0,
  152,    0,  153,    0,    0,  154,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  155,  156,    0,    0,    0,
  157,    0,  158,    0,  159,    0,    0,    0,    0,    0,
  160,    0,  161,    0,    0,    0,  162,    0,    0,    0,
    0,    0,    0,    0,  163,    0,    0,  164,    0,  165,
    0,    0,    0,  166,  167,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  168,    0,    0,    0,
  170,  171,  172,  173,  174,    0,  175,    0,    0,    0,
    0,    0,  176,    0,  177,  178,    0,    0,  179,  180,
    0,    0,    0,    0,    0,  181,    0,    0,  182,  183,
  184,  185,    0,  186,    0,  187,    0,  188,  189,  190,
  191,  192,    0,    0,    0,    0,  193,    0,  194,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  195,
    0,    0,    0,    0,  196,  197,    0,  198,    0,    0,
    0,  199,    0,  200,  201,  202,  203,    0,  204,    0,
    0,    0,    0,    0,    0,    0,  205,  206,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  207,  208,    0,
  209,  210,  211,    0,    0,    0,    0,  212,  213,    0,
  214,  215,    0,  216,    0,    0,  217,  218,    0,    0,
    0,  219,  220,    0,    0,    0,    0,  221,  222,  223,
    0,  224,  225,  226,    0,  227,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  228,  229,  230,  231,  232,
  233,  234,  235,  236,  237,  238,  239,  240,  241,  242,
  243,  244,  245,  246,  247,  248,    0,  249,  250,  251,
  252,  253,  254,    0,  255,  256,  257,  258,    0,  259,
    0,  260,  261,  262,  263,  264,  265,  266,  267,  268,
  269,  270,  271,  272,  273,  274,  275,  276,  277,    0,
  278,  279,  280,  281,    0,  282,  283,  284,  285,  286,
    0,  287,  288,  289,  290,  291,  292,  293,  294,  295,
  296,  297,  298,  299,    0,  300,  301,  302,  303,  304,
  305,    0,  306,  574,  308,  309,  310,  311,  312,  313,
    0,  314,  315,  316,  317,    0,  318,  319,  320,  321,
  322,  323,    0,    0,  324,  325,  326,  327,  328,  329,
  330,  331,  332,    0,    0,    0,    0,    0,  333,    0,
  334,  335,  336,  337,  338,  339,  138,    0,  139,    0,
    0,    0,    0,    0,  140,    0,    0,  141,  142,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  143,  144,
    0,  145,    0,  146,    0,    0,  147,    0,    0,  148,
    0,    0,    0,    0,    0,    0,  149,    0,  150,  151,
    0,    0,    0,  152,    0,  153,    0,    0,  154,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  155,  156,
    0,    0,    0,  157,    0,  158,    0,  159,    0,    0,
    0,    0,    0,  160,    0,  161,    0,    0,    0,  162,
    0,    0,    0,    0,    0,    0,    0,  163,    0,    0,
  164,    0,  165,    0,    0,    0,  166,  167,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  168,
    0,    0,    0,  170,  171,  172,  173,  174,    0,  175,
    0,    0,    0,    0,    0,  176,    0,  177,  178,    0,
    0,  179,  180,    0,    0,    0,    0,    0,  181,    0,
    0,  182,  183,  184,  185,    0,  186,    0,  187,    0,
  188,  189,  190,  191,  192,    0,    0,    0,    0,  193,
    0,  194,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  195,    0,    0,    0,    0,  196,  197,    0,
  198,    0,    0,    0,  199,    0,  200,  201,  202,  203,
    0,  204,    0,    0,    0,    0,    0,    0,    0,  205,
  206,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  207,  208,    0,  209,  210,  211,    0,    0,    0,    0,
  212,  213,    0,  214,  215,    0,  216,    0,    0,  217,
  218,    0,    0,    0,  219,  220,    0,    0,    0,    0,
  221,  222,  223,    0,  224,  225,  226,    0,  227,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  228,  229,
  230,  231,  232,  233,  234,  235,  236,  237,  238,  239,
  240,  241,  242,  243,  244,  245,  246,  247,  248,    0,
  249,  250,  251,  252,  253,  254,    0,  255,  256,  257,
  258,    0,  259,    0,  260,  261,  262,  263,  264,  265,
  266,  267,  268,  269,  270,  271,  272,  273,  274,  275,
  276,  277,    0,  278,  279,  280,  281,    0,  282,  283,
  284,  285,  286,    0,  287,  288,  289,  290,  291,  292,
  293,  294,  295,  296,  297,  298,  299,    0,  300,  301,
  302,  303,  304,  305,    0,  306,  608,  308,  309,  310,
  311,  312,  313,    0,  314,  315,  316,  317,    0,  318,
  319,  320,  321,  322,  323,    0,    0,  324,  325,  326,
  327,  328,  329,  330,  331,  332,    0,    0,    0,    0,
    0,  333,    0,  334,  335,  336,  337,  338,  339, 1034,
    0, 1034,    0,    0,    0,    0,    0, 1034,    0,    0,
 1034, 1034,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1034, 1034,    0, 1034,    0, 1034,    0,    0, 1034,
    0,    0, 1034,    0,    0,    0,    0,    0,    0, 1034,
    0, 1034, 1034,    0,    0,    0, 1034,    0, 1034,    0,
    0, 1034,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1034, 1034,    0,    0,    0, 1034,    0, 1034,    0,
 1034,    0,    0,    0,    0,    0, 1034,    0, 1034,    0,
    0,    0, 1034,    0,    0,    0,    0,    0,    0,    0,
 1034,    0,    0, 1034,    0, 1034,    0,    0,    0, 1034,
 1034,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1034,    0,    0,    0, 1034, 1034, 1034, 1034,
 1034,    0, 1034,    0,    0,    0,    0,    0, 1034,    0,
 1034, 1034,    0,    0, 1034, 1034,    0,    0,    0,    0,
    0, 1034,    0,    0, 1034, 1034, 1034, 1034,    0, 1034,
    0, 1034,    0, 1034, 1034, 1034, 1034, 1034,    0,    0,
    0,    0, 1034,    0, 1034,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1034,    0,    0,    0,    0,
 1034, 1034,    0, 1034,    0,    0,    0, 1034,    0, 1034,
 1034, 1034, 1034,    0, 1034,    0,    0,    0,    0,    0,
    0,    0, 1034, 1034,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 1034, 1034,    0, 1034, 1034, 1034,    0,
    0,    0,    0, 1034, 1034,    0, 1034, 1034,    0, 1034,
    0,    0, 1034, 1034,    0,    0,    0, 1034, 1034,    0,
    0,    0,    0, 1034, 1034, 1034,    0, 1034, 1034, 1034,
    0, 1034,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1034, 1034, 1034, 1034, 1034, 1034, 1034, 1034, 1034,
 1034, 1034, 1034, 1034, 1034, 1034, 1034, 1034, 1034, 1034,
 1034, 1034,    0, 1034, 1034, 1034, 1034, 1034, 1034,    0,
 1034, 1034, 1034, 1034,    0, 1034,    0, 1034, 1034, 1034,
 1034, 1034, 1034, 1034, 1034, 1034, 1034, 1034, 1034, 1034,
 1034, 1034, 1034, 1034, 1034,    0, 1034, 1034, 1034, 1034,
    0, 1034, 1034, 1034, 1034, 1034,    0, 1034, 1034, 1034,
 1034, 1034, 1034, 1034, 1034, 1034, 1034, 1034, 1034, 1034,
    0, 1034, 1034, 1034, 1034, 1034, 1034,    0, 1034, 1034,
 1034, 1034, 1034, 1034, 1034, 1034,    0, 1034, 1034, 1034,
 1034,    0, 1034, 1034, 1034, 1034, 1034, 1034,    0,    0,
 1034, 1034, 1034, 1034, 1034, 1034, 1034, 1034, 1034,    0,
    0,    0,    0,    0, 1034,    0, 1034, 1034, 1034, 1034,
 1034, 1034, 1116,    0, 1116,    0,    0,    0,    0,    0,
 1116,    0,    0, 1116, 1116,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1116, 1116,    0, 1116,    0, 1116,
    0,    0, 1116,    0,    0, 1116,    0,    0,    0,    0,
    0,    0, 1116,    0, 1116, 1116,    0,    0,    0, 1116,
    0, 1116,    0,    0, 1116,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1116, 1116,    0,    0,    0, 1116,
    0, 1116,    0, 1116,    0,    0,    0,    0,    0, 1116,
    0, 1116,    0,    0,    0, 1116,    0,    0,    0,    0,
    0,    0,    0, 1116,    0,    0, 1116,    0, 1116,    0,
    0,    0, 1116, 1116,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, 1116,    0,    0,    0, 1116,
 1116, 1116, 1116, 1116,    0, 1116,    0,    0,    0,    0,
    0, 1116,    0, 1116, 1116,    0,    0, 1116, 1116,    0,
    0,    0,    0,    0, 1116,    0,    0, 1116, 1116, 1116,
 1116,    0, 1116,    0, 1116,    0, 1116, 1116, 1116, 1116,
 1116,    0,    0,    0,    0, 1116,    0, 1116,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1116,    0,
    0,    0,    0, 1116, 1116,    0, 1116,    0,    0,    0,
 1116,    0, 1116, 1116, 1116, 1116,    0, 1116,    0,    0,
    0,    0,    0,    0,    0, 1116, 1116,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1116, 1116,    0, 1116,
 1116, 1116,    0,    0,    0,    0, 1116, 1116,    0, 1116,
 1116,    0, 1116,    0,    0, 1116, 1116,    0,    0,    0,
 1116, 1116,    0,    0,    0,    0, 1116, 1116, 1116,    0,
 1116, 1116, 1116,    0, 1116,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1116, 1116, 1116, 1116, 1116, 1116,
 1116, 1116, 1116, 1116, 1116, 1116, 1116, 1116, 1116, 1116,
 1116, 1116, 1116, 1116, 1116,    0, 1116, 1116, 1116, 1116,
 1116, 1116,    0, 1116, 1116, 1116, 1116,    0, 1116,    0,
 1116, 1116, 1116, 1116, 1116, 1116, 1116, 1116, 1116, 1116,
 1116, 1116, 1116, 1116, 1116, 1116, 1116, 1116,    0, 1116,
 1116, 1116, 1116,    0, 1116, 1116, 1116, 1116, 1116,    0,
 1116, 1116, 1116, 1116, 1116, 1116, 1116, 1116, 1116, 1116,
 1116, 1116, 1116,    0, 1116, 1116, 1116, 1116, 1116, 1116,
    0, 1116, 1116, 1116, 1116, 1116, 1116, 1116, 1116,    0,
 1116, 1116, 1116, 1116,    0, 1116, 1116, 1116, 1116, 1116,
 1116,    0,    0, 1116, 1116, 1116, 1116, 1116, 1116, 1116,
 1116, 1116,    0,    0,    0,    0,    0, 1116,    0, 1116,
 1116, 1116, 1116, 1116, 1116, 1114,    0, 1114,    0,    0,
    0,    0,    0, 1114,    0,    0, 1114, 1114,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1114, 1114,    0,
 1114,    0, 1114,    0,    0, 1114,    0,    0, 1114,    0,
    0,    0,    0,    0,    0, 1114,    0, 1114, 1114,    0,
    0,    0, 1114,    0, 1114,    0,    0, 1114,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1114, 1114,    0,
    0,    0, 1114,    0, 1114,    0, 1114,    0,    0,    0,
    0,    0, 1114,    0, 1114,    0,    0,    0, 1114,    0,
    0,    0,    0,    0,    0,    0, 1114,    0,    0, 1114,
    0, 1114,    0,    0,    0, 1114, 1114,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1114,    0,
    0,    0, 1114, 1114, 1114, 1114, 1114,    0, 1114,    0,
    0,    0,    0,    0, 1114,    0, 1114, 1114,    0,    0,
 1114, 1114,    0,    0,    0,    0,    0, 1114,    0,    0,
 1114, 1114, 1114, 1114,    0, 1114,    0, 1114,    0, 1114,
 1114, 1114, 1114, 1114,    0,    0,    0,    0, 1114,    0,
 1114,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 1114,    0,    0,    0,    0, 1114, 1114,    0, 1114,
    0,    0,    0, 1114,    0, 1114, 1114, 1114, 1114,    0,
 1114,    0,    0,    0,    0,    0,    0,    0, 1114, 1114,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1114,
 1114,    0, 1114, 1114, 1114,    0,    0,    0,    0, 1114,
 1114,    0, 1114, 1114,    0, 1114,    0,    0, 1114, 1114,
    0,    0,    0, 1114, 1114,    0,    0,    0,    0, 1114,
 1114, 1114,    0, 1114, 1114, 1114,    0, 1114,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1114, 1114, 1114,
 1114, 1114, 1114, 1114, 1114, 1114, 1114, 1114, 1114, 1114,
 1114, 1114, 1114, 1114, 1114, 1114, 1114, 1114,    0, 1114,
 1114, 1114, 1114, 1114, 1114,    0, 1114, 1114, 1114, 1114,
    0, 1114,    0, 1114, 1114, 1114, 1114, 1114, 1114, 1114,
 1114, 1114, 1114, 1114, 1114, 1114, 1114, 1114, 1114, 1114,
 1114,    0, 1114, 1114, 1114, 1114,    0, 1114, 1114, 1114,
 1114, 1114,    0, 1114, 1114, 1114, 1114, 1114, 1114, 1114,
 1114, 1114, 1114, 1114, 1114, 1114,    0, 1114, 1114, 1114,
 1114, 1114, 1114,    0, 1114, 1114, 1114, 1114, 1114, 1114,
 1114, 1114,    0, 1114, 1114, 1114, 1114,    0, 1114, 1114,
 1114, 1114, 1114, 1114,    0,    0, 1114, 1114, 1114, 1114,
 1114, 1114, 1114, 1114, 1114,    0,    0,    0,    0,    0,
 1114,    0, 1114, 1114, 1114, 1114, 1114, 1114,  514,    0,
  514,    0,    0,    0,    0,    0,  514,    0,    0,  514,
  514,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  514,  514,    0,  514,    0,  514,    0,    0,  514,    0,
    0,  514,    0,    0,    0,    0,    0,    0,  514,    0,
  514,  514,    0,    0,    0,  514,    0,  514,    0,    0,
  514,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  514,  514,    0,    0,    0,  514,    0,  514,    0,  514,
    0,    0,    0,    0,    0,  514,    0,  514,    0,    0,
    0,  514,    0,    0,    0,    0,    0,    0,    0,  514,
    0,    0,  514,    0,  514,    0,    0,    0,  514,  514,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  514,    0,    0,    0,  514,  514,  514,  514,  514,
    0,  514,    0,    0,    0,    0,    0,  514,    0,  514,
  514,    0,    0,  514,  514,    0,    0,    0,    0,    0,
  514,    0,    0,  514,  514,  514,  514,    0,  514,    0,
  514,    0,  514,  514,  514,  514,  514,    0,    0,    0,
    0,  514,    0,  514,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  514,    0,    0,    0,    0,  514,
  514,    0,  514,    0,    0,    0,  514,    0,  514,  514,
  514,  514,    0,  514,    0,    0,    0,    0,    0,    0,
    0,  514,  514,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  514,  514,    0,  514,  514,  514,    0,    0,
    0,    0,  514,  514,    0,  514,  514,    0,  514,    0,
    0,  514,  514,    0,    0,    0,  514,  514,    0,    0,
    0,    0,  514,  514,  514,    0,  514,  514,  514,    0,
  514,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
  514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
  514,    0,  514,  514,  514,  514,  514,  514,    0,  514,
  514,  514,  514,    0,  514,    0,  514,  514,  514,  514,
  514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
  514,  514,  514,  514,    0,  514,  514,  514,  514,    0,
  514,  514,  514,  514,  514,    0,  514,  514,  514,  514,
  514,  514,  514,  514,  514,  514,  514,  514,  514,    0,
  514,  514,  514,  514,  514,  514,    0,  514,  514,  514,
  514,  514,  514,  514,  514,    0,  514,  514,  514,  514,
    0,  514,  514,  514,  514,  514,  514,    0,    0,  514,
  514,  514,  514,  514,  514,  514,  514,  514,    0,    0,
    0,    0,    0,  514,    0,  514,  514,  514,  514,  514,
  514,  157,    0,  157,    0,    0,    0,    0,    0,  157,
    0,    0,  157,  157,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  157,  157,    0,  157,    0,  157,    0,
    0,  157,    0,    0,  157,    0,    0,    0,    0,    0,
    0,  157,    0,  157,  157,    0,    0,    0,  157,    0,
  157,    0,    0,  157,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  157,  157,    0,    0,    0,  157,    0,
  157,    0,  157,    0,    0,    0,    0,    0,  157,    0,
  157,    0,    0,    0,  157,    0,    0,    0,    0,    0,
    0,    0,  157,    0,    0,  157,    0,  157,    0,    0,
    0,  157,  157,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  157,    0,    0,    0,  157,  157,
  157,  157,  157,    0,  157,    0,    0,    0,    0,    0,
  157,    0,  157,  157,    0,    0,  157,  157,    0,    0,
    0,    0,    0,  157,    0,    0,  157,  157,  157,  157,
    0,  157,    0,  157,    0,  157,  157,  157,  157,  157,
    0,    0,    0,    0,  157,    0,  157,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  157,    0,    0,
    0,    0,  157,  157,    0,  157,    0,    0,    0,  157,
    0,  157,  157,  157,  157,    0,  157,    0,    0,    0,
    0,    0,    0,    0,  157,  157,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  157,  157,    0,  157,  157,
  157,    0,    0,    0,    0,  157,  157,    0,  157,  157,
    0,  157,    0,    0,  157,  157,    0,    0,    0,  157,
  157,    0,    0,    0,    0,  157,  157,  157,    0,  157,
  157,  157,    0,  157,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  157,  157,  157,  157,  157,  157,  157,
  157,  157,  157,  157,  157,  157,  157,  157,  157,  157,
  157,  157,  157,  157,    0,  157,  157,  157,  157,  157,
  157,    0,  157,  157,  157,  157,    0,  157,    0,  157,
  157,  157,  157,  157,  157,  157,  157,  157,  157,  157,
  157,  157,  157,  157,  157,  157,  157,    0,  157,  157,
  157,  157,    0,  157,  157,  157,  157,  157,    0,  157,
  157,  157,  157,  157,  157,  157,  157,  157,  157,  157,
  157,  157,    0,  157,  157,  157,  157,  157,  157,    0,
  157,  157,  157,  157,  157,  157,  157,  157,    0,  157,
  157,  157,  157,    0,  157,  157,  157,  157,  157,  157,
    0,    0,  157,  157,  157,  157,  157,  157,  157,  157,
  157,    0,    0,    0,    0,    0,  157,    0,  157,  157,
  157,  157,  157,  157,   60,    0,   60,    0,    0,    0,
    0,    0,   60,    0,    0,   60,   60,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   60,   60,    0,   60,
    0,   60,    0,    0,   60,    0,    0,   60,    0,    0,
    0,    0,    0,    0,   60,    0,   60,   60,    0,    0,
    0,   60,    0,   60,    0,    0,   60,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   60,   60,    0,    0,
    0,   60,    0,   60,    0,   60,    0,    0,    0,    0,
    0,   60,    0,   60,    0,    0,    0,   60,    0,    0,
    0,    0,    0,    0,    0,   60,    0,    0,   60,    0,
   60,    0,    0,    0,   60,   60,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   60,    0,    0,
    0,   60,   60,   60,   60,   60,    0,   60,    0,    0,
    0,    0,    0,   60,    0,   60,   60,    0,    0,   60,
   60,    0,    0,    0,    0,    0,   60,    0,    0,   60,
   60,   60,   60,    0,   60,    0,   60,    0,   60,   60,
   60,   60,   60,    0,    0,    0,    0,   60,    0,   60,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   60,    0,    0,    0,    0,   60,   60,    0,   60,    0,
    0,    0,   60,    0,   60,   60,   60,   60,    0,   60,
    0,    0,    0,    0,    0,    0,    0,   60,   60,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   60,   60,
    0,   60,   60,   60,    0,    0,    0,    0,   60,   60,
    0,   60,   60,    0,   60,    0,    0,   60,   60,    0,
    0,    0,   60,   60,    0,    0,    0,    0,   60,   60,
   60,    0,   60,   60,   60,    0,   60,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   60,   60,   60,   60,
   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,
   60,   60,   60,   60,   60,   60,   60,    0,   60,   60,
   60,   60,   60,   60,    0,   60,   60,   60,   60,    0,
   60,    0,   60,   60,   60,   60,   60,   60,   60,   60,
   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,
    0,   60,   60,   60,   60,    0,   60,   60,   60,   60,
   60,    0,   60,   60,   60,   60,   60,   60,   60,   60,
   60,   60,   60,   60,   60,    0,   60,   60,   60,   60,
   60,   60,    0,   60,   60,   60,   60,   60,   60,   60,
   60,    0,   60,   60,   60,   60,    0,   60,   60,   60,
   60,   60,   60,    0,    0,   60,   60,   60,   60,   60,
   60,   60,   60,   60,    0,    0,    0,    0,    0,   60,
    0,   60,   60,   60,   60,   60,   60,  138,    0,  139,
    0,    0,    0,    0,    0,  140,    0,    0,  141,  142,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  143,
  144,    0,  145,    0,  146,    0,    0,  147,    0,    0,
  148,    0,    0,    0,    0,    0,    0,  149,    0,  150,
  151,    0,    0,    0,  152,    0,  153,    0,    0,  154,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  155,
  156,    0,    0,    0,  157,    0,  158,    0,  159,    0,
    0,    0,    0,    0,  160,    0,  161,    0,    0,    0,
  162,    0,    0,    0,    0,    0,    0,    0,  163,    0,
    0,  164,    0,  165,    0,    0,    0,  166,  167,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  168,    0,    0,    0,  170,  171,  172,  173,  174,    0,
  175,    0,    0,    0,    0,    0,  176,    0,  177,  178,
    0,    0,  179,  180,    0,    0,    0,    0,    0,  181,
    0,    0,  182,  183,  184,  185,    0,  186,    0,  187,
    0,  188,  189,  190,  191,  192,    0,    0,    0,    0,
  193,    0,  194,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  195,    0,    0,    0,    0,  196,  197,
    0,  198,    0,    0,    0,  199,    0,  200,  201,  202,
  203,    0,  204,    0,    0,    0,    0,    0,    0,    0,
  205,  206,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  207,  208,    0,  209,  210,  211,    0,    0,    0,
    0,  212,  213,    0,  214,  215,    0,  216,    0,    0,
  217,  218,    0,    0,    0,  219,  220,    0,    0,    0,
    0,  221,  222,  223,    0,  224,  225,  226,    0,  227,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  229,  230,  231,  232,  233,  234,  235,  236,  237,  238,
  239,  240,  241,  242,  243,  244,  245,  246,  247,  248,
    0,  249,  250,  251,  252,  253,  254,    0,  255,  256,
  257,  258,    0,  259,    0,  260,  261,  262,  263,  264,
  265,  266,  267,  268,  269,  270,  271,  272,  273,  274,
  275,  276,  277,    0,  278,  279,  280,  281,    0,  282,
  283,  284,  285,  286,    0,  287,  288,  289,  290,  291,
  292,  293,  294,  295,  296,  297,  298,  299,    0,  300,
  301,  302,  303,  304,  305,    0,  306,  307,  308,  309,
  310,  311,  312,  313,    0,  314,  315,  316,  317,    0,
  318,  319,  320,  321,  322,  323,    0,    0,  324,  325,
  326,  327,  328,  329,  330,  331,  332,    0,    0,    0,
    0,    0,  333,    0,  334,  335,  336,  337,  338,  339,
  138,    0,  139,    0,    0,    0,    0,    0,  140,    0,
    0,  141,  142,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  143,  144,    0,  145,    0,  146,    0,    0,
  147,    0,    0,  148,    0,    0,    0,    0,    0,    0,
  149,    0,  150,  151,    0,    0,    0,  152,    0,  153,
    0,    0,  154,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  155,  156,    0,    0,    0,  157,    0,  158,
    0,  159,    0,    0,    0,    0,    0,  160,    0,  161,
    0,    0,    0,  162,    0,    0,    0,    0,    0,    0,
    0,  163,    0,    0,  164,    0,  165,    0,    0,    0,
  166,  167,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  168,    0,    0,    0,  170,  171,  172,
  173,  174,    0,  175,    0,    0,    0,    0,    0,  176,
    0,  177,  178,    0,    0,  179,  180,    0,    0,    0,
    0,    0,  181,    0,    0,  182,  183,  184,  185,    0,
  186,    0,  187,    0,  188,  189,  190,  191,  192,    0,
    0,    0,    0,  193,    0,  194,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  195,    0,    0,    0,
    0,  196,  197,    0,  198,    0,    0,    0,  199,    0,
  200,  201,  202,  203,    0,  204,    0,    0,    0,    0,
    0,    0,    0,  205,  206,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  207,  208,    0,  209,  210,  211,
    0,    0,    0,    0,  212,  213,    0,  214,  215,    0,
  216,    0,    0,  217,  218,    0,    0,    0,  219,  220,
    0,    0,    0,    0,  221,  222,  223,    0,  224,  225,
  226,    0,  227,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  228,  229,  230,  231,  232,  233,  234,  235,
  236,  237,  238,  239,  240,  241,  242,  243,  244,  245,
  246,  247,  248,    0,  249,  250,  251,  252,  253,  254,
    0,  255,  256,  257,  258,    0,  259,    0,  260,  261,
  262,  263,  264,  265,  266,  267,  268,  269,  270,  271,
  272,  273,  274,  275,  276,  277,    0,  278,  279,  280,
  281,    0,  282,  283,  284,  285,  286,    0,  287,  288,
  289,  290,  291,  292,  293,  294,    0,  296,  297,  298,
  299,    0,  300,  301,  302,  303,  304,  305,    0,  306,
  307,  308,  309,  310,  311,  312,  313,    0,  314,  315,
  316,  317,    0,  318,  319,  320,  321,  322,  323,    0,
    0,  324,  325,  326,  327,  328,  329,  330,  331,  332,
    0,    0,    0,    0,    0,  333,    0,  334,  335,  336,
  337,  338,  339,  138,    0,  139,    0,    0,    0,    0,
    0,  140,    0,    0,  141,  142,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  143,  144,    0,  145,    0,
  146,    0,    0,  147,    0,    0,  148,    0,    0,    0,
    0,    0,    0,  149,    0,  150,  151,    0,    0,    0,
  152,    0,  153,    0,    0,  154,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  155,  156,    0,    0,    0,
  157,    0,  158,    0,  159,    0,    0,    0,    0,    0,
  160,    0,  161,    0,    0,    0,  162,    0,    0,    0,
    0,    0,    0,    0,  163,    0,    0,  164,    0,  165,
    0,    0,    0,  166,  167,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  168,    0,    0,    0,
  170,  171,  172,  173,  174,    0,  175,    0,    0,    0,
    0,    0,  176,    0,  177,  178,    0,    0,  179,  180,
    0,    0,    0,    0,    0,  181,    0,    0,  182,  183,
  184,  185,    0,  186,    0,  187,    0,  188,  189,  190,
  191,  192,    0,    0,    0,    0,  193,    0,  194,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  195,
    0,    0,    0,    0,  196,  197,    0,  198,    0,    0,
    0,  199,    0,  200,  201,  202,  203,    0,  204,    0,
    0,    0,    0,    0,    0,    0,  205,  206,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  207,  208,    0,
  209,    0,  211,    0,    0,    0,    0,  212,  213,    0,
  214,    0,    0,  216,    0,    0,  217,  218,    0,    0,
    0,  219,  220,    0,    0,    0,    0,  221,  222,  223,
    0,    0,  225,  226,    0,  227,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  228,  229,  230,  231,  232,
  233,  234,  235,  236,  237,  238,  239,  240,  241,  242,
  243,  244,  245,  246,  247,  248,    0,  249,  250,  251,
  252,  253,  254,    0,  255,  256,  257,  258,    0,  259,
    0,  260,  261,  262,  263,  264,  265,  266,  267,  268,
  269,  270,  271,  272,  273,  274,  275,  276,  277,    0,
  278,  279,  280,  281,    0,  282,  283,  284,  285,  286,
    0,  287,  288,  289,  290,  291,  292,  293,  294,  295,
  296,  297,  298,  299,    0,  300,  301,  302,  303,  304,
  305,    0,  306,  307,  308,  309,  310,  311,  312,  313,
    0,  314,  315,  316,  317,    0,    0,    0,    0,  321,
  322,  323,    0,    0,  324,  325,  326,  327,  328,  329,
  330,  331,  332,    0,    0,    0,    0,    0,  333,    0,
  334,  335,  336,  337,  338,  339,
};
static int yycheck[] = {                                      14,
   15,  110,   17,    0,  592,  121,  639,  640,  731,  718,
   81,  670,  728,  881,  113,   67,  114,   63, 1031,   73,
  881,   81,   14,  994,   15,   70, 1335,  126,  648, 1042,
  639,  640,  671,  693,  481,  879,  880,  881, 1374, 1173,
   94, 1374,  717,   71,  693, 1179, 1373,   62,   63, 1728,
   65,   66,   67,   68,   69,   70,   71,   72,   73,   74,
  112,  107,   77, 1044,   92,  119,  592,   82,   83,   84,
   85,  116,   87,  656,   89,   90, 1469,   92,   93,   94,
   95,   96,  693,  952,   99, 1014,   82, 1014,  103,  117,
  105,  106,  107, 1014,  109,  110,  111,  112,  113, 1762,
  115,  116,  117,  118,  119,  120,  121,   77,  691,  124,
   14,  126,   66,  105,  129,  130, 1305, 1716,  133, 1460,
   74,  136, 2060,  115, 2062, 2102, 1960, 1230, 1804,   99,
 1832, 1279, 1801, 1421, 1331, 1146, 2220, 2221,  130, 2191,
 2240, 1421, 1572, 1355, 1384, 1039, 1462, 1730, 2129, 1421,
 1124, 1421,  678, 1421,  124, 1353,  126,  111,  696,  966,
 1591,  722, 2294, 1582, 1014,   69,  120, 1829, 1017,  632,
  633, 2126,    0,    0,    0,  491, 2376, 1417, 1919,  133,
   87,  363,   99, 1395, 1424,   89, 1426,   98,   93, 1380,
  531, 2787,   84,    0,   42, 1405, 1921,  542,    0,    0,
    0,  105, 1096, 1939,    0, 1263,   40,    0,   44,    0,
    0,   40,  123,    0,   44, 2581,   44, 1709, 1817,   59,
   41, 1403, 1309,   44,   44,  129,  130,  572,   45,   63,
    0,   59,   59,   59,   63,    0,    0,    0,   59,   41,
   61,   40,  295,    0,   41,  296,    0,   44,   41, 1402,
   44,    0,   59,    0,   41, 2210,  964,   59,   59,   59,
 1921,  881,   59,   59,   61,   45,   59,    0,   59,   59,
   45,   40,   59,  260, 1142,    0,  296, 2235,   41,  278,
 1710, 2481,  355,  262,   41,   44,   44,   44,   44,   59,
 2491,   44,  325,    0,   59,   59,   59, 1020, 1142,   45,
  573, 1162,   59, 1230,    0,   59, 1187, 1188,   41, 1238,
   59,   44,   59,   44,    0,   44,   41,    0,   41,   44,
  307,   44, 2374, 2526,  280,   41,   59,  273,  268,    0,
  345,  262, 1920,    0,   59,  350,  351,  352,  353,  354,
  355,  356,  357,   91,  359,   41,  387,  316,  363, 1024,
    0, 2826,   59, 2671,  345,  464,  500,  276, 2945,  374,
 1753, 1754, 1755,   59,  507,  475,  265,  276,    0,  275,
  500, 1232,  372,   59, 1218, 1050,   59,    0,   44,  260,
  387, 1515,  261,  296, 2584,  301,  389,  265,   59,  359,
 1234,  462,   59, 2926,    0,  505,  426, 1920,  297,  398,
 1261, 1262,    0,  264,  407, 1014,  438,  286,  302,   59,
 1254, 1255, 1256, 1257, 1258,  263, 2386,  330,  387,  297,
  450,   44,  301,  423,  264, 1294, 1762,   59,  264, 1762,
  329,  337, 2484, 1053,  264, 2405,   59,    0,    0,  418,
  549,  584,  359,  264, 2544,  364,  353,  518,  557,  464,
  354,  356, 1168,   59, 1170,  364,  277,  264, 2137,  351,
  501,   59,  264,  264,  737, 1662,  385,  264,  994, 1572,
  310,  264,  391,    0,  295, 1313, 1059,  264,   41, 1317,
  277,   44,   44,  490, 1322,  401, 2320,  418,  258,  310,
 1351,    0, 1421,  433, 2546,   41,   59,   59,  295, 3086,
 1421,  264,  448,  364,  258,  424,  650,  264,  364,  264,
  312,    0,    0,  310,   41, 2076,  270,   44, 2993,  312,
  571,  651, 1142, 1657,  364,  312, 2228,  275, 2230,  544,
  332,  301,   59,    0,  549,  587, 1156,   40,  332,  332,
  434,    0,  557, 1163, 3077,  332, 1694,  301, 2529,  303,
   59,  571,   41,   41, 2755,  312,   41,   40,  573,  574,
 2878,  314,    0, 2288, 2765,  322,  323,  364,  264,  332,
   59,   59,  587, 2776,  544,  332, 2779,  592,  335,  594,
    0,  596,  597, 1306, 2015,  584, 2952,  258,  295, 2547,
   59,  644,   59,  608, 1302,  551,  624,  393, 1218,  270,
   59,  592,   41,  594,  574,  596,    0, 1710,  442,  624,
 1697, 2030,  295,  442,  647,  372,  312,  632,  633,  634,
  635,   59,  442,  419,  381,  442, 2118, 2288,  427,  702,
  301,  401,  303,  340,  434,  437,  332,  368,  608,   59,
 1698,  264,  713,  368,  437, 1572,  393,  401,  314,  419,
  437,  257,  698,  713, 1535,  642,  372,  340, 1587, 1279,
  419,  430,  442,  678, 1297,   59,  423,  442, 1679,  434,
  434,  686,  395,  725,  437,  703,  372, 1393,  435,  437,
  437,  437,  452,  698,  699,  700,  701,  702,  703,  704,
  705, 1311,  467,  708, 2826, 2827,  442,  499, 1414, 2287,
  715,  264,  264,  718,  719,  372,  435,  722,  394,  500,
  725, 1579, 1922,  728,  381,  321, 1304,  519,    0,  519,
  326, 1582,  737,  715, 2694, 2695,  519,  423,  519,  519,
  401, 3327,  519, 3099, 3100, 1579, 1927,  264,  708, 1921,
  372,  437,  499,  500,  394, 2660,  700, 1591,  596,  312,
  483,  705, 1468, 2183,  519, 2491, 2186,  593,  483,  322,
  323,  642,  519,  593, 2287,  363,  627, 1920, 1113,  332,
  519,  627,  335,    0,  380,  264,  264,   59, 1304,  961,
 1709, 1122, 1709, 1710, 1459,  312,  593,  627, 1709,  496,
  396,  708,  593, 1109,  701,  264,  707, 1465,  646,  704,
 1661, 2782,   44,  499,  500,  332,  627,  699,   44,  372,
  363,   95, 2553,  496,  437,  719, 2868,   61,  381,  304,
  648,  648,  648,  519,  312, 1408,  372, 1353,  295, 3029,
  627,  340,   59,  500,  322,  323,   59, 2398,  593,    0,
  316,  310, 2797,  332,  332,  443,  444,  264, 1374,  948,
 1688, 1689,  519,  364,  953, 1318,  610,  611, 1321, 1709,
  423, 2993, 1711, 2190,  258,  304, 2528,  882,  500,  298,
 1396, 2068,  435,  340,  437,  437,  270,  483,  264,  347,
 1441,  340,  480,    0,  372,   44,   59,  519, 1695, 1863,
  295,  882, 2311,  381,  651,  301,  319,  400,   59, 1437,
 2870, 2871, 2872, 2873, 2874, 2875, 2876,  301, 2011,  303,
  437,  387,  302,  303,  282,  344,  987,  400,  344,  357,
 2503,  264,   59,   42,  427, 2123,  395, 3084,  526, 2077,
  496, 2247, 2363,  948,  437,  423,  499,  500,  953,  610,
  611,    0,   59,  363,  427, 3333,  961, 1394,  437,  437,
  965,  316, 1620, 2552,  437,  651,  519, 1028,  306, 1579,
  321, 1855, 1582,  978, 1035, 1036, 2565,  427, 1028,  567,
  264,  319,  499,  500,    0, 1035, 1036,  437,    0,  994,
  302,  303,   41,  953,  554,   44,  592,  312,  594,  595,
  413,    0,  519, 2233,    0,  401, 2236,  420, 2238, 2980,
   59, 3389,  268, 1450,    0,    0,  474,  401,  578,  279,
    0,  499,  500, 2847, 3199, 1006,  335,  443,  444, 1034,
    0, 2032,  387,  443,  444, 3182, 3078,  450, 1099,  496,
 2164,  519,   41,   59, 2246, 3135,  355,   59, 2468,  645,
   59,  264,   61,  306,  434,    0,   41, 1118, 1119,  560,
   59,  295,  395,   59,  290,    0,  319,  299, 1118, 1119,
  480,  965,  284,   59,   59,   59,  619, 2730,  352,   59,
 2749,  314, 1732,  357,  978,  443,  444, 2740,  644,   59,
 2183,  363,  263, 2186, 2011,  446,   41,  353,  651,   44,
  409, 3061, 3062, 3063, 3064, 3065, 3066, 3067,  259,   44,
 1709,  395,  519,  264,   59, 1966,  526,    0,  269,  332,
   59,  436,  434,  340,   59,   59, 1832, 3032, 3033, 3034,
 3035, 3036, 3037, 3038,  651,  306,  299,  264,   60,   61,
   62,  290, 1800,  519,    0, 1150,  317, 1253,   93,  615,
 1849,  260, 1157, 3300,   59,  368,  551,  567,   41,  310,
    0,   44,  422,  298,  389, 1264, 1815,  380,  506,  276,
  277,  443,  444,  651,  348,    0,   59, 2455, 2456, 1802,
 1758, 2015, 3256,  310, 2035, 2455, 2456, 1816, 3012,  298,
 1279,    0, 1857, 2455, 2456, 2455, 2456, 2455, 2456,  332,
  412,   41, 3377,   59,   44, 2208,  330, 2066,  480,  344,
   93,  363,  257, 1157,  439,  264,   41, 2404, 1867,   59,
  639,  640,  641, 1936,  437, 1230,  610,  611, 3403,  364,
 3272,  382, 3274,  420,   59,  316,  649,  450,  412,  410,
 1869,  276, 1758,   59,  553, 2576, 1762, 3422, 1253, 1230,
   59, 2254, 1313,  506,  526, 1828, 1317,  364, 1263, 1264,
  615, 1322, 3437,  312, 2930,  264, 2183,    0,  264, 2186,
 2929, 1370,    0, 1278, 1279,  295,  332, 1263, 3314,  264,
  264,  326, 1371,  332, 2937,    0,  295,  316,  307, 1939,
  276,  277,  296, 2431,  264,  567,  276,  277,    0, 1304,
 1939,  425,  296, 1308, 1309,    0,  387, 1368,   41,  316,
 3346,  443,  444, 1318,  310,  299, 1321, 3141, 1368, 1279,
  340,  273, 1308, 1304, 2448,  264,   59,  312,  480,  364,
  264,   59, 2649,  332,  488, 1340,    0,  312, 1939,   41,
  310, 1429, 1430, 1431,   59,  296,   41,  332, 1353,  271,
  385,    0,  264,  540,  257, 2479,  391,   59,  387,  264,
  547,  303,  284,   59,   59, 1370, 1371,  296, 1373, 1374,
  312,  310, 1353, 3388,  526, 2468,  310,  442,  364,   41,
  387, 3309,   44,   41,  364,  296,   44, 1373,  437,  424,
  296, 1396,  344, 1374,  368,   59,  318, 1402, 1403,   61,
 3092,  330,  467,  325, 2730,  310,    0, 2730,  264, 2122,
   59,  333,  382, 3428, 2740,  567,  280, 2740,  437,  330,
 2030,  425,  429,  326,  264,  400, 2743,  372,  350,  257,
  352,  425,    0,  264, 3087,  484, 1441,  372,  437,  264,
  362,   59, 3409,  365,  366, 3098,  381,   41,  264,  394,
  499,  500,  437, 2311,  310,  264,  368,  312,  400, 1464,
 2311,  436,  391, 1468, 1469,   59,  637, 3434,  380,    0,
  519,  400,  312,  283,  425,  400,  496, 2311,  340,  310,
  295, 2452, 2453,  323,  312,  295, 2817,  312,   42,  372,
  412,   59,  332,   47,  436, 3423,  425,  258,  326,  434,
  435,  310,    0,  295,  639,  640,  641,  332,  401,  270,
 2133,  394,  368,  279,  425, 1520,  332, 2368,  382,  425,
  388,    0, 2228,  276, 2230,  437,  382, 3116,   59, 2363,
 3183,  264,  372, 2712,  519,  403, 2455, 2456,  450, 3192,
  301,  381,  303,  475, 2455, 2456,  264,    0,  264,  264,
  271, 2468,  368,  485,  364,  419, 2415, 2681, 2941,  368,
  313,   59,  264,  284,  380,  500, 1571, 1572,  264,  264,
 1464,  276,  364,  382, 1663,  389, 1520,  264,  260,  312,
   59,  443,  444,  423,  519, 1571,   44, 2711,  452,  322,
  323, 1572,  310,  257,  310,  310,  264,  437,  554,  332,
   45,  258,  651,  369,  325, 1694,   59, 2123,  313, 1715,
  312, 2937,  437,   58, 2937,  273,  298,  316,   63,    0,
 2581,  437,  578,  310,    0,  439, 2249, 1688, 1689,    0,
  332,  352,  385,  295,  450,   93,  332,  332,    0,  372,
  401,  426,  310,  507,  301, 2218,  304, 2650,  381,   44,
  368,  259,  368,  368,   59,  371,  422,  321, 1663,  499,
  500,  269,  326,    0,   40,   41,  382,  382,   44,  596,
  264,  424,  368,  439,  499,  500,  598,  261,   59,  519,
  385,  368,   44,   59,  380,  334,  344,  551,   59, 1694,
  423,  412, 1697, 1698,  519,  382,  264,   59,   93,  393,
  368, 2311,  435,   44,  437, 1710,   42,   43, 1744,   45,
 1715,   47, 1698, 3382,  363, 1696,  380, 2430,  312,  424,
 2426, 1726,   59,  277,  393,  419, 1731,  281, 2377, 1710,
    0,  260,  396,  264, 1694,  437, 1741,  395,  332, 1744,
  389,  437,  310,  392,  401, 1731, 2445, 1752, 1753, 1754,
 1755, 1756,   93, 1758,  450, 1741,  264, 1762,  407,  257,
  368, 3087,  419, 1851, 3087,  258,  499,  500,  261,  298,
  259,    0, 3098,    0,  264, 3098,  295, 1758,  372,  310,
  269, 1762,  259,  363,  433,  264,  519,  381,  264,   59,
  448,   41,  269, 1798,   44,  452, 1801,  499,  500,  442,
  368,  315,  310, 1808, 2807, 1810, 2809,   44,  301,    0,
   59,  651,  316,  327,  382, 1801, 2253,  519, 2255,  483,
  310,  401, 1808,  321, 1810,  344,  651, 3144,  326,  423,
   59,  310,   59,  316,  310,  349,    0,  368,  487,  264,
  261, 2491,  263,  437, 1849,    0,  291, 2284,    0,  610,
  611,  382, 2491,    0, 2504,  596,   93, 3183,  507,  264,
 3183,    0,  260, 1868,    0,  286, 3192,  447,   59, 3192,
    0,  520,  310,  311,  105,  296, 2727,  598,  368,  368,
  301,  371,  380,  397, 3267,  310, 1868,  348,  264,  368,
 2491,    0,  382,    0,   41,   59,  329,   44,  396,  130,
  298, 3200,   41,  382,   59,  499,  500,   59, 2531,  330,
    0, 2560,   59,  587, 2563, 1920, 1921,  378,  651,  380,
   59,  258,  596,   59,    0,  519,  315,  332,  592,   59,
  594,  595, 2561,  270, 1939, 2564,  312,   44,  327,  315,
  401,  277, 2903,   59, 2043,  281,  322,  323,  260,  651,
   59,  327,   59, 1939,  264, 2044,  332,  475,  295,  335,
  349,    0,  474,  368,  301,  426,  303,  485,  315,   59,
    0,  360,  280,  349,   41,  380,  450,   44,  264,  368,
  327,  645,  330,   59,  360,  483,  298, 2076, 2077,  450,
    0,  503,  368,  301,    0,  440,  372,  442,  443,  444,
  310,  401,  349,  340,  425,  381, 2011, 2012,  397,  264,
 2046,  348,  276,  360,    0,  264,  303,  462,  463,  283,
   59,  397,  467,  468,  310,  312, 2012,    0,   44,   59,
 2011,   41,  437, 2038,   44,  264,   46, 2042, 2043, 2044,
 2045, 2046, 2047, 2048, 2150,  450,    0,  423,   40,   59,
  397,   61, 2038,   59,  260,   41,  425,  651,   44,  435,
 2042,  437, 2067,  260,  401, 2581, 2127, 2128,   41, 2920,
  587, 2076, 2077,   59, 2079,  412, 2934, 2127, 2128,  612,
  613,  310,    0,  332,    0, 2045,   59,   41,  260,  427,
   44,    0,  298,  401,  592,    0,  594,  595, 2079,    0,
 2934,  298,  437, 2047, 2048,   59,    0,  483,    0,  264,
  277,  401,    0,  400,  281,  504, 2076, 2077, 2123,  368,
    0,  507,  475,  499,  500,  264,  298,    0,  504,  276,
  277,  380,  485,    0,  376,  377,  283,    0,   45,  368,
   40,   59, 2123,   59,  452, 2150, 2811,  645,  264,  436,
   59,   58,  505,  382,   59,  310,   63,  504,   59,  496,
  301, 2222,  264, 2224, 2225,   59,  590,   59,  407, 2275,
  475,   59, 2222,  312, 2224, 2225,  600,  601, 2183,   59,
  485, 2186,  334,  443,  323, 2190,   59, 2900,  437, 2194,
    0, 2196,   59,  332,  639,  640,   59, 2184, 3322,  507,
  505,  450, 2183,  264, 2190, 2186, 2224, 2268, 2194,  584,
 2271, 2272, 2273,  368, 2730,  310,  332,  364, 2268, 2224,
 2225, 2271, 2272, 2273, 2740, 2230, 2051,  382, 3352,   41,
  443,  444,   44,  372, 2295,   46,  261,  389,  385,  312,
  392,  382,  381,  551,  391, 2295,    0,   58,   59,   59,
   61,  447,  368,  449,  264,  407, 2081, 3260,  264,  417,
  401,  286, 2087,  261,  380,  263, 2865,  620,  621,  622,
 2275,  475,  476,  610,  611,  651,  301,  424,  419,  329,
   46,  433, 2287, 2288,  423,  328,   40,   41,  286,  299,
   44,  264,   58,   59,  483,   61, 1752,  261,  437,   46,
 1756, 2455, 2456,  301,  310,   59,  435,   41, 2407, 2398,
   44,   58,   59,    0,   61,  620,  621,  622, 1152, 2408,
 1154,  437,  286,   46, 2934, 1159, 1160, 3330,   41,    0,
 1164,   44,  296,  483,  450,  487,  483,  301,    0,  312,
  258,   41, 2431,   59,   44,   41,   42,   43,  488,   45,
  323,   47,  270,    0,   41,  507, 1190,   44,   46,  332,
  499,  500,  368,   46,   41,  419,  330,   44,  520,  332,
   58,   59,   59,   61, 2410,   58,  382,  295,   61,   41,
  519,  264,   44,  301,  291,  303, 3092, 2903,   59, 1010,
 2395, 1012, 2397, 2398,  295,   41,  417,   59,   44,  372,
  369, 2406, 2407, 2408, 2409, 2410, 2411, 2412,  381,  379,
 2498,   41,   59, 3112,   44,   58, 2397, 3361,   61, 3363,
   46, 2937,  340,  401, 2406,   41, 2431,  391,   44,  440,
  348,  442,   58,   59,   41,   61,  400,   44, 2398,  340,
 2445,  316,   41,  379, 2532,   44,  334,  348,  258, 2409,
  423,   41,   41, 2458,   44,   44,  467,  468,  315,   41,
  270,  425,   44, 2468,  437,  333,   59, 2411, 2412,  367,
  327, 2431, 2458, 2534,  487,  363,  315,  378,   41,  380,
  273,   44, 2487,  401, 2534,  295, 2491, 2468,  327, 2053,
 2054,  301,  349,  303,  412, 2500,  335, 2148, 2149, 2504,
  401,  389,  519,  360,  392, 2491,   41,   40,  401,   44,
  349,  368,  651,   41, 2500,  304,   44, 2522, 2504,  407,
 2525,  360,  276,  277,  294,  426,  499,  500, 3177,  283,
  340,    0,  264,  440,  437,  442,  443,  444,  348,   41,
  397,  295,   44,  648,   41,  433,  519,   44,  264,  450,
   41,   46,  264,   44, 2559,  462,  463,  268,  397,   41,
  467,  468,   44,   58,   59, 2570,   61,   41,  270,  596,
   44, 3087,   41, 2559,  282,   44, 2581,  264,  496,    0,
 3293,  277, 3098,  437,  264,  281, 2591, 2592, 3439,  381,
   59,  401, 3443,  500,   41,  496,  340,   44,   41,  487,
   41,   44,  412,   44,   41,   41, 2592,   44,   44, 2591,
  364,  646,   40,  340,  276,  277,  332,   40,  280,  507,
   41,  283,   40,   44,  295,  312,  483,   41,  315,  443,
   44,  385,  520,  295,   40,  322,  323,  391,   59,   40,
  327, 2659, 2647,   41, 2649,  332,   44,  504,  335, 3298,
   41,   40,  368,   44, 2659, 2673,   41,   41,   40,   44,
   44, 2647,  349, 2649,  380,  504,   40, 3183, 2673,  340,
  424,  264, 3295,  360, 3297,   40, 3192,  348,  651,   41,
 2685,  368,   44,   40,   40,  372,  496,  334,   41,   40,
  443,   44,  610,  611,  381,   41,  347,   41,   44, 2704,
   44,  443,  364,   41, 2685,   41,   44,  378,   44,  380,
  397, 2716, 1655,   40,   40, 1658,  363,   40, 2723,   40,
   40,  437, 2704,  385,   40, 2730,   40,  394,   40,  391,
  401,  418,  639,  640,  450, 2740,  423, 2723, 2743,  332,
 2828,   40,  389,  650,   40,  392,   91,  434,  435, 2730,
  437,   40,   40,   40, 2815,  426,   40, 2743,   40, 2740,
  407,   40,  424,   40,   40, 2815,   60,   61,   62,   40,
   40, 2832, 2777,   40,   40,  368,   40,   40,   40,  450,
   40, 2880, 2832,   40,   40,   40,  433,  380,   40,   40,
   40,   40,   40, 2882,    0,   40,  483,  551,   40,   40,
  610,  611,    0,   40,   40,   40,   40,  276,  277,  614,
 3388,   44,  499,  500,  283,   40,   40,  504, 2652, 2653,
 2654, 2655, 2656, 2657,   46,  496,  295,  261,   46,   40,
  368,  370,  519, 2838,  370, 2840,   46,   40,   61,  333,
  487,  333, 2886,  264,  437,   46,  333,  499, 2883,    0,
 3428,   40,  286,   59, 2840,  295, 2838,  450,  357,  270,
  507,   59,  296, 2881,   40,  273, 2884,  301,  382,  437,
  446,   40, 3388,  520,  430, 2880, 2881, 2882, 2883, 2884,
 2885, 2886, 2887,  430,  384,   40,  341,   40, 2893,  551,
   41,  312, 2897,   44,  315,  364,  330,   91, 2903,  295,
   40,  322,  323,   40,   40,  368,  327, 2995,   59,  270,
 2971,  332, 3428,  442,  335,  374,  385,  367,  372,    0,
  257, 2971,  391,   41, 2929,  312, 2987, 2887,  349,   41,
   41,   41, 2937, 2994,   40,  419, 2941, 2987,   46,  360,
   40,  437,  386, 2929, 2994, 2932,    0,  368,  368,  368,
  290,  372,  368,   40,   46,  424, 2937,  391,  260,   44,
  381,  333,  312, 2968,  651,  312,  400,  333,  262,    0,
  264,  314,   44,   46,   44,  509,  397,  271,   59,  496,
 2985,  437,  495,  495,  321,  612,   40,   41,   41,  326,
   44,  425,  395,  306,   44,  371,  371,  483, 3003,  437,
  729,  371,  423,  316,  596,   59,  319,   61,  303,  401,
   41,   40,  437,   44,  435,  348,  437,  371,  609,   40,
   40, 3003, 3040,  355,  318,  500,   41,  756,   59,   41,
   44,  325,   41,   41,  314, 3040,   41,   91,   41,  333,
  418,  262,   44,  380, 2906, 2907, 2908, 2909, 2910, 2911,
  779,   44,   44,  314, 3090,   46,  350,   40,  352,  396,
  333,  419,  483,  277,  616, 3070,   41, 3072,  362,   40,
  364,  365,  366,  395,  387, 3080,  417,  371,  499,  500,
  419,   44, 3087,  504,  401, 3090, 3091,   44, 3070,  419,
 3072,   44,   41, 3098, 3080,   41, 3138,  335,  519,  373,
    0, 3106,  314,  335,  417, 3110, 3087, 3112,  421,  368,
  368,  290,   41, 3118,  368,  484,  435, 3098,  275,  432,
  340,  340,  419,  340,   59,  276,  277,  277,  334,  280,
   40,  303,  283, 3138, 3139,   59,  334,  590,   58, 3144,
  332,  332,  442,  451,  295,  276,  483,  419,   59,  414,
   59, 3212,   41, 3139,   58,   41,   41,  363, 3144,   59,
   41,  401, 3212,   61, 3225,  363,  490,  645,  607, 3275,
  314,  303,  607,   40,  304, 3225,  303,   41, 3183,  395,
  419,   41,  262,  389,  273,  314,  392, 3192,  418,   40,
  264,  389,  263,  506,  392,  263,  335,  314,  314,  351,
   44,  407, 3183,  322,  258, 3210,  524,  369,  417,  407,
  264, 3192,  523,  364,  295,  521,  401,   40,  276,  273,
  344,  275,  276,  277,  344,  367,  280,  433,  275,  283,
  651,  619,  289,  420,  385,  433,  332,  434,   40,  552,
  391,  295,   41,  371,  370,  276,  277,  301,  371,  371,
  304,  358,  283,  399,   41,  592,  310,  594,  595,  340,
   40,   59, 3267, 3268,  295,  446,  371,  348, 3310,  998,
 3275,    0, 1001,  424,  371,   41,  371, 1006,   44,  446,
 1009,  487,  646, 3288, 1013,   59, 3268,  303, 1017,  487,
  344,   44,  367,   41,   60,   61,   62,  378,   41,  380,
  434,  507,  615,   41,  598, 3310,   41, 3349,   41,  507,
  364,   40,   41,  427,  520,   44,  419,  419,  419,  273,
  401,  419,  520,  522,  323,  419,  419,  431, 3333,  419,
   59,  385,   61,  364,  411,  369,  336,  391,  344,  312,
   40,  395,   41,  299, 3349,  426,    0,  401,  299,  490,
  273,  364,  295,  364,  385,  528,   40,  296,  425,  425,
  391,  330,   91,  330,  296,  419,  325,  314,  646,  450,
  424,   41,  405,   41,  450,   40,   40,   40,    0, 3384,
  357,  450,   41, 3388, 3389,   61,  314,   41,   40,   44,
   44,  446,  314,  424,  448,  314,  314, 3402,  452,  314,
  551,  314,  314,  431,   40,   59,  314, 3388,   41,  395,
   40,  418,  314, 1142,   41,  496,  264,  280,  344,   41,
 3402,  275,   44, 3428, 1153,  627,  340,  344,  368, 1158,
   40,  304,  442, 1162,  334,  303,  418,   59,   63,   61,
  370,  321,  273, 1172, 1173,  434,   44, 3428, 1177, 1178,
 1179,  312, 1181, 1182, 1183, 1184, 1185, 1186, 1187,  334,
 1189,   44, 1191,  363, 1193, 1194, 1195, 1196, 1197,   91,
   46, 1200, 1201, 1202, 1203, 1204, 1205, 1206, 1207, 1208,
 1209, 1210, 1211, 1212, 1213, 1214, 1215,  299,  363,  389,
  336,  336,  392,   41,  273,  401,  262,  551,   41,   46,
  278,  344,  418,  264, 1233,  271,   41,  407,  370,  302,
  443, 1240,  264,  299,  389,  483, 1245,  392,  284,  295,
  551,   61, 1251, 1252,   41,  357,   40,   40,  560,  258,
 1259,  289,  407,  433,  391,  264,    0,  264,  582,  509,
  567,  448,  500,  561,  273,  500,  275,  276,  277,  266,
  264,  280,  318,  310,  283,  376,  329,  329,  433,  325,
  426,   40,  283,  628,   40,  420,  295,  333,  386,  418,
  420,  368,  301,  627,  401,  304,  330,   41, 1307,  605,
   44,  310,   41,  368,  350,  368,  352,  487,  418,  296,
  425,  336,  401,  330,   44,   59,  362,   61,  364,  365,
  366,  427,  603,  445,   40,  371,   40,  507,  427,  400,
    0,  329,  487,  264,  264,  344,    0,   40,  400,  264,
  520,  270,  276,  277,  333,  400,  270,   91,  395,  283,
    0,  400,  507,   40,  303,  364,  258,   41,  434,  335,
  314,  295,  264,  340,   40,  520,  412,  303,   41,  264,
  303,  273,  368,  275,  276,  277,  385,   41,  280,  290,
   44,  283,  391,  335,   59,  289,  395,   41,  303,   41,
   59,   41,  401,  295,   44,   59,   59,  442,   59,  301,
   59,   59,  304,  303,  395,   41,   41,   40,  310,   59,
  419,   59,   61,   59,  303,  424,  314,   41,  454,  455,
  456,  457,  458,  459,  460,  461,   41,  314,  418,  614,
  364,  314,  264,    0,  299,  264,  442,  442,  519,  448,
   46,  642,  344,  452,   46,  476,   41,   42,   43,   44,
   45,  385,   47,   41,   41,  305,   46,  391,  305,  305,
   44,  264,  364,  364,  305,   60,   61,   62,  519,  363,
 1479, 1480,  363, 1482,   41, 1484,   41,   44,   44, 1488,
 1489, 1490, 1491,  385,  593,  593,  264, 1496, 1497,  391,
  424,   46,   59,  395,   61, 1670,   91, 1293, 1293,  401,
 2387,  551, 1293, 1293, 1513, 2388, 1515, 1293, 2389, 2897,
 3040, 3040, 2067, 1294,  258, 2415, 2893,  419, 2993, 2291,
  264, 1530,  424, 2827,   91, 1468,  350,  563, 2803,  273,
 1539, 1540,  276,  277, 2974,    0,  280, 2753, 3344,  283,
   74,  705,  551, 2200, 3124, 2219,  448, 2782, 3255,  354,
  452,  295,  598,  589, 2277,  978, 2804,  301, 3204, 2559,
  304, 3346, 2559, 3433, 3379, 3378,  310, 3394, 2743, 3144,
 1579, 3176, 2744, 1463, 1922, 1924,   41, 1032,  608,   44,
 2945, 2184, 2184, 3390, 1593, 3389, 1595, 3301, 3097, 1598,
 1756, 2180, 3186,    0,   59, 1265,   61, 2647, 1619, 1445,
  344, 1913,  676, 1727, 1810, 1356, 2193, 2236, 1095, 1426,
 1416,  132,  276,  277, 1125, 2088, 2417,  551,  627,  283,
  364, 3070, 2422,  991,  635,  345,  276,  277,  596, 1696,
  642, 1279, 2677,  283, 2080, 3003, 1656,    0, 3075, 1145,
 3384,  385, 1699,  646, 2103,    0,    0,  391, 1657,  551,
 3056,  395,    0, 3056,    0,    0,  966,  401, 3325, 1242,
 2264, 3210, 1592,  720,  684,  737, 1439,  262, 3106, 1709,
  877, 1261, 2791,  878, 2848,  419,  271, 2230,   41, 1339,
  424,   44,  277, 2487, 1984,  109,  281, 1696, 2885,  284,
 1699,  258, 1701,  131, 2082, 1276,   59,  264,   61, 2414,
  364, 2581, 1711, 3382,  448, 1134,  273, 1918,  452,  276,
  277, 3104, 3273,  280,  364, 3348,  283, 2720,  368, 2490,
 1045,  385, 1027,  318,   12,  627, 3230,  391,  295, 2680,
  325,   -1, 3292,   -1,  301,  385,   -1,  304,  333,   -1,
 1749,  391,   -1,  310,    0,   -1,   -1,   -1, 1757,   -1,
   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,
  424,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,
  365,  366,   -1,   -1,  424,   -1,  371,  344,   -1,   -1,
   -1,   -1,    0,   -1,   -1,   41,   -1,   -1,   44,   -1,
   -1,   -1,   -1,  258,   -1,   -1,   -1,  364,   -1,  264,
   -1,   -1,   -1,   59,   -1,   61,   -1,  551,  273,   -1,
   -1,   -1,  277,   -1,   -1,  280,   -1,  412,  385,  483,
   -1,   -1,   -1,   41,  391,   -1,   44,    0,  395,   -1,
  295,   -1,   -1,  483,  401,   91,  301,   -1,   -1,  304,
   -1,   59,   -1,   61,   -1,  310,   -1,   -1,   -1,   -1,
   -1,   -1,  419,   -1,   -1,   -1,   -1,  424,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   41,   -1,
   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  344,
   -1,  448,   -1,  627,   -1,  452,   59,   -1,   61,   -1,
   -1,   -1,   -1,   -1,   -1,  258,   -1,   -1,   -1,  364,
   -1,  264,   -1,   -1,   -1,   -1,   41,   -1,   -1,   44,
   -1,   -1,  275,  276,  277,   -1,   -1,  280,   45,   -1,
  283,   -1,   -1,   -1,   59,   -1,   -1,   -1,   -1,   -1,
  395,   -1,  295,   -1,   -1, 1944,  401,   -1,  301,   -1,
   -1, 1950,   -1,   -1,   -1,   -1, 1955,  310,   -1, 1958,
 1959, 1960,   -1,   -1,  419,   -1, 1965,   -1, 1967,   -1,
   -1,   -1,   -1,   -1,   -1, 1974,   -1, 1976,   -1,   -1,
 1979, 1980, 1981, 1982,   -1,   -1, 1985,   -1, 1987,   -1,
 1989,   -1, 1991,  448,  551,   -1,   -1,  452, 1997, 1998,
 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
   -1,  364,  258,  598,   -1,   -1,   -1,   -1,  264, 2018,
  605,    0,   -1,   -1, 2023,   -1,   -1,  273, 2027,   -1,
  276,  277,  385,   -1,  280,    0,   -1,  283,  391,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  399,   -1,  401,  295,
  258,   -1,   -1,   -1,   -1,  301,  264,   -1,  304,   -1,
   -1,  414,   41,   -1,  310,   44,  419,  275,  276,  277,
  627,  424,  280,   -1,   -1,  283,   41,   -1,   -1,   44,
   59,   -1,   61,   -1,   -1,   -1,   -1,  295,   -1,   -1,
   -1,   -1,   -1,  301,   59,  258,  551,   -1,  344,  452,
   -1,  264,  310,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
 2109,   -1,  275,  276,  277,   -1,   -1,  280,  364,   -1,
  283,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  295,   -1,   -1,   -1,   -1,   -1,  301,  385,
   -1,   -1,   -1,   -1,   -1,  391,   -1,  310,   -1,  395,
   -1,  276,  277,   -1,   -1,  401,  364,   -1,  283,   -1,
   -1,   -1,   -1, 2162,   -1, 2164,   41,   -1,   -1,   -1,
   -1,   -1,  627,  419,  291,   -1,   -1,  385,  424,   -1,
   -1,   -1,   -1,  391,   -1,   60,   61,   62,   -1,   -1,
   -1,  399,   -1,  401,   -1,   -1,   -1,   -1,  551,   -1,
  317,  364,  448,   -1,   -1,   -1,  452,   -1,   -1,   -1,
   -1,  419,   -1,   -1,   -1,   -1,  424,   -1,   -1,   -1,
   -1,   -1,  385,   -1,   -1,   -1,  275,  344,  391,   -1,
   -1,   -1,   -1,   41,   -1,   -1,   44,   -1,  401,  364,
   -1,  290,   -1,   -1,  452,   -1,   -1,   -1,   -1,   -1,
   -1,  300,   60,   61,   62,   -1,  419,  306,   -1,   -1,
  385,  424,  311,   -1,   -1,   -1,  391,  316,   -1,   -1,
  319,   -1,   -1,   -1,  627,   -1,   -1,   -1,   -1,  258,
   -1,   -1,   -1,   -1,   -1,  264,   -1,   -1,   -1,  452,
   -1,   -1,   -1,  410,   -1,   -1,   -1,  276,  277,  424,
    0,  280,   -1,   -1,  283,  551,   -1,   -1,   -1,  426,
   -1,  276,  277,   -1,   -1,   -1,  295,   -1,  283,  368,
   -1, 2320,  301,  440,   -1,  442,  443,  444, 2327, 2328,
   -1,  310,   -1,   -1,   -1, 2334,   -1,   -1,  387,   -1,
   -1,   41,   -1,  551,   44,  462,  463,  464,  465,  466,
  467,  468,  469,  470,   41,   -1,   -1,   44,  483,   59,
  477,  478, 2361,   -1,   -1, 2364,   -1, 2366,  417,  486,
   -1,   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,  627,   -1,  432,   -1,  364,   -1,  262,  551,   -1,
   -1,   -1,   -1,    0,   -1,   -1,  271,   41,   -1,  364,
   44,  450,   -1,   -1,   -1,   -1,  385,   -1,   -1,  284,
   -1,   -1,  391,   -1,   -1,   -1,   60,   61,   62,  627,
  385,   -1,  401,   -1,   -1,   -1,  391,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   41,   -1,   -1,   44,   -1,  314,
  419,   -1,   -1,  318,   -1,  424,   -1,   -1,   -1, 2448,
  325,   -1,   59,   -1,  262,   -1,   -1,  506,  333,  424,
  509,   -1,   41,  271,  627,   44,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  452,   -1,  350,  284,  352,   -1,   -1,
 2479,   60,   61,   62,   -1,   -1,   -1,  362,   -1,  364,
  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
  637,   -1,  639,  640,  641,  333,   -1,   -1,   -1,   41,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   60,   61,
   62,  668,  669,   -1,  362,   -1,  364,  365,  366,   41,
   -1,   -1,   -1,  371,   -1,   -1,  615,   -1,   -1,   -1,
   -1,   -1,  551,   -1,   -1,  262,  276,  277,   60,   61,
   62,   -1,   -1,  283,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1, 2598,
   -1, 2600,   -1, 2602,  412,   -1,   -1, 2606,   -1,   -1,
   -1,   -1, 2611, 2612, 2613, 2614, 2615,   -1,  262,   -1,
   -1, 2620,   -1, 2622,   -1, 2624,   -1,  271,   -1,   -1,
   -1,  318,   -1,   -1,   -1,   41,   -1,   -1,  325,   -1,
  284,   -1, 2641,   -1,   -1,   -1,  333,   -1,  627,   -1,
   -1,   -1,   -1,   -1,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,  350,  364,  352,   -1,   -1,   -1,  276,
  277,   -1,   -1,   -1,  318,  362,  283,  364,  365,  366,
   41,  325, 2681,  262,  371,  385,   -1,   -1,   -1,  333,
   -1,  391,  271,   -1,   -1,   -1,   -1,   -1, 2696,   60,
   61,   62,   -1,   -1,   -1,  284,  350,   -1,  352,   -1,
   -1, 2710, 2711,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,  598,  424,  412,   -1,  371,   -1, 2728,
   -1,   -1,   -1,   -1,   -1,   -1,   -1, 2736, 2737,  318,
   -1,   41,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,
  262,   -1,   -1,    0,  333,   -1,   -1,  364,   -1,  271,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,  412, 2768,
   -1,  350,  284,  352,   -1,   -1,   -1,   -1,  385,   -1,
  262,   -1,   -1,  362,  391,  364,  365,  366,   -1,  271,
  598,   -1,  371,   -1,   41,   -1,   -1,   44,   -1,   -1,
  312,   -1,  284,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   41,   -1,   59,  325,   -1,   -1,   -1,  424,   -1,   -1,
   -1,  333,   -1,   60,   61,   62,   -1,   -1,   -1,   60,
   61,   62,   -1,  412,   -1,   -1,  318,   -1,  350,   -1,
  352,   -1,   -1,  325,   -1,   -1,   -1,   -1, 2847, 2848,
  362,  333,  364,  365,  366,   -1,  262,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
  362, 2879,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,   -1, 2891,   -1,   -1,   -1,   -1,   -1,   -1,
  412,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  271,  598,  318,   -1, 2912, 2913,   -1,   -1,   -1,  325,
   -1,   -1,   -1,  284,   -1,   -1,   -1,  333,   -1,   -1,
  412,   -1,   -1,   -1,   -1, 2934,    0,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  598,   -1,  362,  318,  364,  365,
  366,   -1,  262,   -1,  325,  371,   -1,   -1,   -1,   -1,
   -1,  271,  333,   -1,   -1,   -1,   40,   41,   -1,   43,
   44,   45,   46,   -1,  284,   -1,   -1,   -1,   -1,  350,
   -1,  352,   -1, 2992,   58,   59,   -1,   -1,   -1,   63,
   -1,  362,   -1,  364,  365,  366,  412,   -1,   -1,   -1,
  371,   -1,   -1, 3012,   -1,   -1,   -1,   -1,  318,  598,
   -1, 3020,   -1, 3022,   -1,  325,   -1,   -1,   -1,  276,
  277,  262,   -1,  333,  271,   -1,  283,   -1,   -1,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,
  350,  412,  352,  284,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1, 3075,   -1,   -1,   -1,
   -1,  318,   -1,   -1,   -1,   -1,  598,  318,  325,   -1,
   -1,   -1,   -1,   -1,  325,   -1,  333,   -1,   -1,   -1,
   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,  350,   -1,  352,  598,  364,   -1,  350,
   -1,  352,   -1,   -1,   -1,  362,   -1,  364,  365,  366,
   -1,  362,   -1,  364,  365,  366,   -1,   -1,  385,   -1,
  371,   -1, 3141,   -1,  391,   -1,   -1, 3145, 3146, 3147,
 3148, 3149, 3150, 3151,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1, 3160, 3161, 3162, 3163, 3164, 3165, 3166,   -1,
   -1,   -1,   -1,   -1,   -1,  412,   -1,  424,   -1,   -1,
   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  598,  257,   -1,  259,   -1,   -1,   -1,   -1,
  264,  265,   -1,  267,  268,  269,   -1,   -1,   -1,   -1,
  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,  598,   -1,   -1,
  304,   -1,  306,   -1,  308,  309,   -1,   -1,  312,   -1,
   -1,  315,   -1,  317,   -1,  319,  320,   -1,  322,  323,
  324,   -1,  326,  327,  328,   -1,   -1,   -1,  332,   -1,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,  353,
  354,   -1,  356,  357,  358,   -1,  360,   -1,  598,   -1,
  364,   -1,   -1,   -1,  368, 3304,  370, 3306,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,  382,   -1,
   -1,   -1,  386, 3322,  388,  389,   -1,   -1,  392,  393,
  394,   -1,   -1,  397, 3333,  399, 3335, 3336,  402,  403,
  404,  405,   -1,  407,   -1,  409,  410,  411,  412,  413,
  414,  415,  416, 3352,   -1,   -1,  420,   -1,  422,  423,
   -1,  598,  426,   -1,   -1,   -1,   -1,  598,   -1,  433,
   -1,  435,   -1,  437,  438,  439,  440,  441,  442,  443,
  444,  445,   -1,  447,  448,  449,  450,   -1,  452,  453,
   -1, 3390, 3391, 3392,   -1,   -1,  460,  461,  462,  463,
  464,  465,  466,  467,  468,  469,  470,  471,  472,   -1,
  474,  475,  476,  477,  478,   -1,  480,  481,  482,  483,
  484,  485,  486,  487,   -1,   -1,  490,  491,  492,  493,
  494,  495,  496,   -1,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,  511,  512,  513,
   -1,  515,  516,   -1,  518,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,   -1,  599,  600,  601,  602,  603,
  604,   -1,  606,  607,  608,  609,  610,  611,  612,  613,
   -1,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,  637,    0,  639,  640,  641,  642,  643,
  644,  645,  646,  647,  648,  649,   -1,  651,  652,  653,
  654,  655,  656,  657,  658,  659,  660,  661,  662,  663,
  664,  665,  666,  667,  668,  669,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   41,   42,   43,   44,   45,
   -1,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   59,   60,   61,   62,  272,   -1,   -1,
  275,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  275,   -1,  287,   -1,   -1,   -1,  291,   -1,  293,   -1,
   -1,   -1,   -1,   -1,   -1,  290,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  300,   -1,   -1,   -1,   -1,
   -1,  306,  306,   -1,   -1,   -1,  311,   -1,   -1,   -1,
   -1,  316,  316,   -1,  319,  319,  331,   -1,   -1,   -1,
   -1,   -1,  337,  338,  339,   -1,  341,  342,  343,   -1,
  345,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  359,   -1,  361,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  368,  379,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  390,   -1,   -1,   -1,   -1,
   -1,   -1,  387,  387,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  406,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  417,  428,   -1,   -1,   -1,  421,   -1,   -1,
   -1,   -1,  426,   -1,   -1,   -1,   -1,  432,  432,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  450,   -1,  462,  463,   -1,
   -1,  257,  258,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,  479,  271,   -1,   -1,   -1,   -1,
  276,  277,   -1,  279,  280,  281,  282,  283,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,  301,  302,  303,  304,  305,
  306,  506,  506,  309,  509,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,  552,   -1,
   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,  385,
  386,    0,  388,  389,   -1,  391,  392,  393,  394,   -1,
   -1,  397,  596,  399,   -1,  401,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
  615,  615,  418,  419,  420,   -1,  422,  423,  424,   -1,
   -1,   -1,   41,  638,   -1,   44,   -1,  433,  434,  435,
   -1,  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   59,  447,  448,  449,  450,   -1,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   41,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   60,   61,   62,  495,
  496,   41,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,   -1,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   -1,   41,
   42,   43,   44,   45,   -1,   47,   -1,  276,  277,   -1,
   -1,   -1,   -1,   -1,  283,   -1,   -1,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,  364,   -1,   -1,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,  385,   -1,  318,   -1,
   -1,   -1,  391,   -1,   -1,  325,  350,   -1,  352,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
  350,   -1,  352,   -1,   -1,  424,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,  257,  258,  259,   -1,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,  276,  277,   -1,  279,  280,  281,
  282,  283,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,  301,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,  385,  386,   -1,  388,  389,   -1,  391,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,  401,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,  598,   -1,  418,  419,  420,   -1,
  422,  423,  424,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,  598,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,
  472,   41,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   60,   61,   62,  495,  496,   41,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   61,   62,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,   -1,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,  261,   40,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  278,   -1,
   58,   59,   60,   61,   62,   -1,  286,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  294,   -1,  296,   -1,   -1,   -1,
   -1,  301,  262,   -1,   -1,   -1,   -1,  307,   -1,   -1,
   -1,  271,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,
   -1,  321,   -1,   -1,  284,   -1,  262,   -1,   -1,   -1,
  330,   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
   -1,   -1,   -1,   -1,   -1,  355,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
  350,   -1,  352,   -1,   -1,   -1,  396,  333,  398,   -1,
   -1,  401,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  425,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,  437,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  473,   -1,   -1,  412,   -1,   -1,  257,
   -1,  259,   -1,   -1,  262,   -1,  264,  265,  488,  489,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
  510,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,  598,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,
  438,  439,  598,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   41,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   60,   61,   62,  495,  496,   41,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,
   62,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,  609,  610,  611,  612,  613,    0,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   40,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   58,   59,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   91,   -1,   93,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,  350,   -1,  352,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  412,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,
  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,
  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,
  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,
  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,
   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,
  364,  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,  598,   -1,  418,  419,  420,   -1,  422,  423,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,  435,   -1,  437,  438,  439,  598,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,
  454,  455,  456,  457,  458,  459,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   41,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   60,   61,
   62,  495,  496,   41,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,
  604,  605,  606,  607,  608,  609,  610,  611,  612,  613,
    0,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,
   40,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,  350,   -1,
  352,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  412,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,
  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,
  320,   -1,  322,  323,  324,  325,  326,  327,  328,   -1,
   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,  598,   -1,  418,  419,
  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,
  598,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   41,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   60,   61,   62,  495,  496,   41,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,    0,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   40,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   91,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,  350,   -1,  352,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,
   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,
   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,  305,
  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,
   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,
   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
  598,   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,
   -1,  437,  438,  439,  598,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   41,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   60,   61,   62,  495,
  496,   41,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,  605,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,  350,   -1,  352,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,  598,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,  598,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,
  472,   41,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   60,   61,   62,  495,  496,   41,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   61,   62,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,   -1,   40,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
  350,   -1,  352,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,  257,
   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,  598,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,
  438,  439,  598,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   41,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   60,   61,   62,  495,  496,   41,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,
   62,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,  609,  610,  611,  612,  613,    0,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   40,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   58,   59,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   91,   -1,   93,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,  350,   -1,  352,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  412,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,
  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,
  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,
  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,
  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,
   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,
  364,  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,  598,   -1,  418,  419,  420,   -1,  422,  423,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,  435,   -1,  437,  438,  439,  598,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,
  454,  455,  456,  457,  458,  459,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   41,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   60,   61,
   62,  495,  496,   41,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,
  604,  605,  606,  607,  608,  609,  610,  611,  612,  613,
    0,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,
   40,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,  350,   -1,
  352,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  412,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,
  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,
  320,   -1,  322,  323,  324,  325,  326,  327,  328,   -1,
   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,  598,   -1,  418,  419,
  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,
  598,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   41,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   60,   61,   62,  495,  496,   41,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,    0,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   40,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   91,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,  350,   -1,  352,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,
   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,
   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,  305,
  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,
   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,
   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
  598,   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,
   -1,  437,  438,  439,  598,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   41,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   60,   61,   62,  495,
  496,   41,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,  605,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,  350,   -1,  352,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,  598,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,  598,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,
  472,   41,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   60,   61,   62,  495,  496,   41,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   61,   62,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,   -1,   40,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
  350,   -1,  352,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,  257,
   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,  598,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,
  438,  439,  598,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   41,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   60,   61,   62,  495,  496,   41,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,
   62,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,  609,  610,  611,  612,  613,    0,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   40,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   58,   59,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   91,   -1,   93,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,  350,   -1,  352,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  412,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,
  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,
  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,
  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,
  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,
   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,
  364,  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,  598,   -1,  418,  419,  420,   -1,  422,  423,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,  435,   -1,  437,  438,  439,  598,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,
  454,  455,  456,  457,  458,  459,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   41,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   60,   61,
   62,  495,  496,   41,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,
  604,  605,  606,  607,  608,  609,  610,  611,  612,  613,
    0,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,
   40,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,  350,   -1,
  352,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  412,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,
  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,
  320,   -1,  322,  323,  324,  325,  326,  327,  328,   -1,
   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,  598,   -1,  418,  419,
  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,
  598,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   41,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   60,   61,   62,  495,  496,   41,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,    0,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   40,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   91,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,  350,   -1,  352,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,
   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,
   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,  305,
  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,
   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,
   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
  598,   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,
   -1,  437,  438,  439,  598,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   41,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   60,   61,   62,  495,
  496,   41,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,  605,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,  350,   -1,  352,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,  598,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,  598,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,
  472,   41,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   60,   61,   62,  495,  496,   41,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   61,   62,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,   -1,   40,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
  350,   -1,  352,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,  257,
   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,  598,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,
  438,  439,  598,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   41,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   60,   61,   62,  495,  496,   41,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,
   62,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,  609,  610,  611,  612,  613,    0,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   40,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   58,   59,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   91,   -1,   93,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,  350,   -1,  352,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  412,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,
  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,
  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,
  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,
  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,
   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,
  364,  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,  598,   -1,  418,  419,  420,   -1,  422,  423,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,  435,   -1,  437,  438,  439,  598,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,
  454,  455,  456,  457,  458,  459,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   41,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   60,   61,
   62,  495,  496,   41,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,
  604,  605,  606,  607,  608,  609,  610,  611,  612,  613,
    0,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,
   40,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,  350,   -1,
  352,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  412,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,
  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,
  320,   -1,  322,  323,  324,  325,  326,  327,  328,   -1,
   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,  598,   -1,  418,  419,
  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,
  598,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   41,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   60,   61,   62,  495,  496,   41,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,    0,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   40,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   91,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,  350,   -1,  352,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,
   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,
   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,  305,
  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,
   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,
   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
  598,   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,
   -1,  437,  438,  439,  598,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   41,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   60,   61,   62,  495,
  496,   41,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,  605,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,  350,   -1,  352,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,  598,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,  598,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,
  472,   41,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   60,   61,   62,  495,  496,   41,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   61,   62,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,   -1,   40,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   58,   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
  350,   -1,  352,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,  257,
   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,  598,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,
  438,  439,  598,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   41,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   60,   61,   62,  495,  496,   41,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,
   62,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,  609,  610,  611,  612,  613,    0,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   40,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   58,   59,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,   91,   -1,   93,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,  350,   -1,  352,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  412,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,
  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,
  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,
  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,
  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,
   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,
  364,  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,  598,   -1,  418,  419,  420,   -1,  422,  423,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,  435,   -1,  437,  438,  439,  598,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,
  454,  455,  456,  457,  458,  459,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   41,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   60,   61,
   62,  495,  496,   41,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   61,   62,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,
  604,  605,  606,  607,  608,  609,  610,  611,  612,  613,
    0,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,
   40,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  284,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,
   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,  350,   -1,
  352,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  412,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,
  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,
  320,   -1,  322,  323,  324,  325,  326,  327,  328,   -1,
   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,  598,   -1,  418,  419,
  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,
  598,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   41,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   60,   61,   62,  495,  496,   41,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,    0,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   40,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,   91,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  333,  272,   -1,   -1,  275,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,  350,   -1,  352,  291,   -1,  293,   -1,  333,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,
   -1,   -1,   -1,  371,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,  331,   -1,  371,   -1,   -1,
   -1,  337,  338,  339,   -1,  341,  342,  343,   -1,  345,
   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  359,   -1,  361,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,
   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,  390,  271,   -1,   -1,   -1,   -1,
   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,
  406,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,  305,
  306,   -1,  428,  309,   -1,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,   -1,   -1,  340,   -1,  462,  463,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,
   -1,  357,  358,  479,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,
   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
  598,   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,
   -1,  437,  438,  439,  598,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   41,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   60,   61,   62,  495,
  496,   -1,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   60,   61,   62,   -1,
   -1,   -1,  638,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,  605,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,  275,   -1,   -1,   -1,  271,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
  284,  265,   -1,   -1,   -1,   -1,   -1,  271,  300,   -1,
   -1,   -1,   -1,   -1,  306,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,  316,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  297,  318,   -1,   -1,  329,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
   -1,  325,   -1,   -1,   -1,   -1,  350,   -1,  352,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,  350,  371,  352,   -1,
   -1,   -1,   -1,   -1,   -1,  387,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  417,   -1,   -1,  412,  421,
   -1,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
  432,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,  450,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,  506,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,  596,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,  598,   -1,  418,  419,  420,   -1,
  422,  423,   -1,  615,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,  598,  437,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   44,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   -1,   60,   61,   62,   -1,  471,
  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   60,   61,   62,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,   -1,   40,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,
   58,   59,   60,   61,   62,   -1,  271,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,  264,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,
  325,   -1,   -1,   -1,  284,   -1,   -1,   -1,  333,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,  318,  364,
  365,  366,   -1,   -1,   -1,  325,  371,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,  412,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,  257,
   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,  598,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,  598,  437,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   44,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   60,
   61,   62,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   60,   61,   62,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,  609,  610,  611,  612,  613,    0,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   40,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  262,   -1,   -1,   58,   59,   60,   61,   62,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,  318,   -1,   -1,
   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,  284,   -1,
   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,
   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,  314,   -1,
   -1,  362,  318,  364,  365,  366,   -1,   -1,   -1,  325,
  371,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,  412,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,
   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,
  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,
  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,
  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,
  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,
   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,
  364,  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,  598,  392,  393,
  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,  435,  598,  437,  438,  439,   -1,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   44,  452,   -1,
  454,  455,  456,  457,  458,  459,  460,  461,   -1,   -1,
   -1,   -1,   -1,   60,   61,   62,   -1,  471,  472,   -1,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,   -1,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,   -1,   60,   61,
   62,   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,
  604,  605,  606,  607,  608,  609,  610,  611,  612,  613,
    0,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,
   40,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   58,   59,
   60,   61,   62,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,
   -1,   -1,  284,   -1,   -1,   -1,  333,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,
   -1,   -1,  314,   -1,   -1,  362,  318,  364,  365,  366,
   -1,   -1,   -1,  325,  371,   -1,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,  412,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  412,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,
  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,
  320,   -1,  322,  323,  324,  325,  326,  327,  328,   -1,
   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,  598,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,  419,
  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,  598,  437,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   44,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,    0,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   40,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   58,   59,   60,   61,   62,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,  284,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,  312,   -1,   -1,   -1,   -1,  362,
  318,  364,  365,  366,   -1,   -1,   -1,  325,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,  412,
   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,
   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,
   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,  305,
  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,
   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,  598,  392,  393,  394,   -1,
   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,
  598,  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   44,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   -1,   60,   61,   62,   -1,  471,  472,   -1,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,
  496,   -1,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   60,   61,   62,   -1,
   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,  605,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  262,   -1,   -1,   58,   59,   60,   61,
   62,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,  318,
   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,
  284,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,
  314,   -1,   -1,  362,  318,  364,  365,  366,   -1,   -1,
   -1,  325,  371,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,  412,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,  598,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,  598,  437,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   44,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   -1,   60,   61,   62,   -1,  471,
  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   60,   61,   62,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,   -1,   40,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,
   58,   59,   60,   61,   62,   -1,  271,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,
  325,   -1,   -1,   -1,  284,   -1,   -1,   -1,  333,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,  318,  364,
  365,  366,   -1,   -1,   -1,  325,  371,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,  412,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,  257,
   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,  598,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,  598,  437,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   44,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   60,
   61,   62,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   60,   61,   62,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,  609,  610,  611,  612,  613,    0,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   40,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  262,   -1,   -1,   58,   59,   60,   61,   62,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  271,   -1,  318,   -1,   -1,
   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,  284,   -1,
   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,
   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  362,  318,  364,  365,  366,   -1,   -1,   -1,  325,
  371,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,
  366,  412,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,
   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,
  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,
  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,
  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,
  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,
   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,
  364,  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,  598,  392,  393,
  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,  435,  598,  437,  438,  439,   -1,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   44,  452,   -1,
  454,  455,  456,  457,  458,  459,  460,  461,   -1,   -1,
   -1,   -1,   -1,   60,   61,   62,   -1,  471,  472,   -1,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,   -1,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,   -1,   60,   61,
   62,   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,
  604,  605,  606,  607,  608,  609,  610,  611,  612,  613,
    0,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,
   40,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   58,   59,
   60,   61,   62,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,
   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,
   -1,   -1,  284,   -1,   -1,   -1,  333,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  362,  318,  364,  365,  366,
   -1,   -1,   -1,  325,  371,   -1,   -1,   -1,   -1,   -1,
   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,
  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  362,   -1,  364,  365,  366,  412,   -1,   -1,   -1,  371,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  412,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,
  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,
  320,   -1,  322,  323,  324,  325,  326,  327,  328,   -1,
   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,  598,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,  419,
  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,  598,  437,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   44,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   60,   61,   62,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,    0,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   40,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   58,   59,   60,   61,   62,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  271,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,  284,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
  318,  364,  365,  366,   -1,   -1,   -1,  325,  371,   -1,
   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,  412,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,
   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,
   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,  305,
  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,
   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,  598,  392,  393,  394,   -1,
   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,
  598,  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   44,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   -1,   60,   61,   62,   -1,  471,  472,   -1,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,
  496,   -1,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,  605,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  262,   -1,   -1,   58,   59,   60,   61,
   62,  272,  271,   -1,  275,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  284,  287,   -1,   -1,   -1,
  291,   -1,  293,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  272,   -1,   -1,  275,   -1,   -1,   -1,  318,
   -1,   -1,   -1,   -1,   -1,   -1,  325,  287,   -1,   -1,
  331,  291,   -1,  293,  333,   -1,  337,  338,  339,   -1,
  341,  342,  343,   -1,  345,   -1,   -1,   -1,   -1,   -1,
   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,  359,   -1,
  361,   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,
   -1,  331,  371,   -1,   -1,   -1,   -1,  337,  338,  339,
   -1,  341,  342,  343,   -1,  345,   -1,   -1,   -1,  390,
   -1,   -1,   -1,   -1,  395,   -1,   -1,   -1,   -1,  359,
   -1,  361,   -1,   -1,   -1,  406,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,  379,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  428,   -1,   -1,
  390,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  406,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  462,  463,   -1,   -1,  257,   -1,  259,  428,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,  479,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,  462,  463,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,  479,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,  598,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,  638,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   44,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   -1,   60,   61,   62,  638,  471,
  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,   -1,   40,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,
   58,   59,   60,   61,   62,  272,  271,   -1,  275,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,
  287,   -1,   -1,   -1,  291,   -1,  293,   -1,   -1,   -1,
   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  272,   -1,   -1,  275,
   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,
  325,   -1,   -1,   -1,  331,  291,   -1,  293,  333,   -1,
  337,  338,  339,   -1,  341,  342,  343,   -1,  345,   -1,
   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,
   -1,   -1,  359,   -1,  361,   -1,   -1,  362,   -1,  364,
  365,  366,   -1,   -1,   -1,  331,  371,   -1,   -1,   -1,
   -1,  337,  338,  339,   -1,  341,  342,  343,   -1,  345,
   -1,   -1,   -1,  390,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  359,   -1,  361,   -1,   -1,   -1,  406,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  428,   -1,   -1,  390,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  406,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  462,  463,   -1,   -1,  257,
   -1,  259,  428,   -1,  262,   -1,  264,  265,   -1,   -1,
  268,  269,  479,  271,   -1,   -1,   -1,   -1,   -1,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,  462,  463,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,  479,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,  598,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,  638,   -1,   -1,   -1,  433,  434,  435,   -1,  437,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   44,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   60,
   61,   62,  638,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,  609,  610,  611,  612,  613,    0,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   40,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  262,   -1,   -1,   58,   59,   60,   61,   62,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  275,  284,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  300,   -1,   -1,
   -1,   -1,   -1,  306,   -1,   -1,   -1,  318,  311,   -1,
   -1,   -1,   -1,  316,  325,   -1,  319,   -1,   -1,   -1,
   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,
   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,
  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  387,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  417,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  432,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  450,   -1,   -1,
   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,
  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,
  304,  305,  306,  506,   -1,  309,  509,   -1,  312,   -1,
  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,
  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,
   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,
  364,  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,  598,  392,  393,
  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,  615,   -1,  418,  419,  420,   -1,  422,  423,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,  435,   -1,  437,  438,  439,   -1,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   44,  452,   -1,
  454,  455,  456,  457,  458,  459,  460,  461,   -1,   -1,
   -1,   -1,   -1,   60,   61,   62,   -1,  471,  472,   -1,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,   -1,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,
  604,  605,  606,  607,  608,  609,  610,  611,  612,  613,
    0,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,
   40,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   58,   59,
   60,   61,   62,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  275,  284,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  300,   -1,   -1,   -1,   -1,   -1,  306,   -1,   -1,
   -1,  318,  311,   -1,   -1,   -1,   -1,  316,  325,   -1,
  319,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,
   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  387,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  417,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  432,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  450,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,
  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,  506,   -1,  309,
  509,   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,
  320,   -1,  322,  323,  324,  325,  326,  327,  328,   -1,
   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,  598,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,  615,   -1,  418,  419,
  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   44,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,    0,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   40,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   58,   59,   60,   61,   62,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  300,   -1,   -1,   -1,   -1,   91,  306,   93,   -1,   -1,
  310,  311,   -1,   -1,   -1,   -1,  316,   -1,   -1,  319,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,  329,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  387,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  402,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  417,   -1,  412,
   -1,  421,   -1,   -1,   -1,   -1,  426,   -1,   -1,   -1,
   -1,   -1,  432,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  450,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,
   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,  506,   -1,   -1,  509,
   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,  305,
  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,  552,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,
   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,  596,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,  598,  392,  393,  394,   -1,
   -1,  397,   -1,  399,   -1,  615,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,
   -1,  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   44,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   -1,   60,   61,   62,   -1,  471,  472,   -1,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,
  496,   -1,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,  605,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  262,   -1,   -1,   58,   59,   60,   61,
   62,   -1,  271,   -1,   -1,  300,   -1,   -1,   -1,   -1,
   -1,  306,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,
   -1,  316,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,  329,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,
   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  387,  362,   -1,  364,  365,  366,   -1,   -1,
   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  417,   -1,   -1,   -1,  421,   -1,   -1,   -1,
   -1,  426,   -1,   -1,   -1,   -1,   -1,  432,   -1,   -1,
   -1,   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  450,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,
   -1,  506,   -1,   -1,  509,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,  596,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
  615,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,  598,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   44,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   -1,   60,   61,   62,   -1,  471,
  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,   -1,   40,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,
   58,   59,   60,   61,   62,   -1,  271,   -1,   -1,  300,
   -1,   -1,   -1,   -1,   -1,  306,   -1,   -1,   -1,  284,
   -1,   -1,   -1,   -1,   -1,  316,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,  329,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,
  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  387,  362,   -1,  364,
  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  417,   -1,   -1,   -1,
  421,   -1,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,
   -1,  432,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  450,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,
   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,
  268,  269,   -1,  271,   -1,  506,   -1,   -1,  509,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,  596,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,  615,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,  598,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   44,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   60,
   61,   62,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,  609,  610,  611,  612,  613,    0,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   40,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  262,   -1,   -1,   58,   59,   60,   61,   62,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,
   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,
   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,
  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,
  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,
  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,
  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,
  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,
   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,
  364,  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,  598,  392,  393,
  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,  435,   -1,  437,  438,  439,   -1,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   44,  452,   -1,
  454,  455,  456,  457,  458,  459,  460,  461,   -1,   -1,
   -1,   -1,   -1,   60,   61,   62,   -1,  471,  472,   -1,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,   -1,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,
  604,  605,  606,  607,  608,  609,  610,  611,  612,  613,
    0,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,
   40,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   58,   59,
   60,   61,   62,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,
   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,
  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,
  320,   -1,  322,  323,  324,  325,  326,  327,  328,   -1,
   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,  598,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,  419,
  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   44,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,    0,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   40,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   58,   59,   60,   61,   62,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,
   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,  305,
  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,
   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,  598,  392,  393,  394,   -1,
   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,
   -1,  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   44,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   -1,   60,   61,   62,   -1,  471,  472,   -1,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,
  496,   -1,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,  605,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  262,   -1,   -1,   58,   59,   60,   61,
   62,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,
   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,
   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,  598,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   44,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   -1,   60,   61,   62,   -1,  471,
  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,   -1,   40,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,
   58,   59,   60,   61,   62,   -1,  271,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,
  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,
  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,
   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,  598,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   44,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   60,
   61,   62,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,  609,  610,  611,  612,  613,    0,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   40,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  262,   -1,   -1,   58,   59,   60,   61,   62,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,
   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,
   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,   -1,
  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,
  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,
  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,
  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,
  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,
   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,
  364,  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,  598,  392,  393,
  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,  435,   -1,  437,  438,  439,   -1,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   44,  452,   -1,
  454,  455,  456,  457,  458,  459,  460,  461,   -1,   -1,
   -1,   -1,   -1,   60,   61,   62,   -1,  471,  472,   -1,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,   -1,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,
  604,  605,  606,  607,  608,  609,  610,  611,  612,  613,
    0,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,
   40,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   58,   59,
   60,   61,   62,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  284,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  318,   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  362,   -1,  364,  365,  366,
   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,
  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,
  320,   -1,  322,  323,  324,  325,  326,  327,  328,   -1,
   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,  598,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,  419,
  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   60,   61,   62,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,    0,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   40,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   58,   59,   60,   61,   62,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,
   -1,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  454,  455,  456,  457,  458,  459,  460,  461,   -1,
   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,
   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,  305,
  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,
  563,  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,  589,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,  598,  392,  393,  394,   -1,
   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,
   -1,  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   60,   61,   62,   -1,   -1,  471,  472,   -1,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   60,   61,   62,  495,
  496,   -1,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,  605,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   58,   59,   60,   61,
   62,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  284,   -1,   -1,   -1,  262,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,   -1,   -1,   -1,   -1,   -1,
  350,  325,  352,   -1,   -1,   -1,   -1,   -1,   -1,  333,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  362,   -1,
  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,  419,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,  569,  388,  389,  598,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,  598,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   60,   61,   62,   -1,   -1,  471,
  472,   -1,  474,  475,  476,   60,   61,   62,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,   -1,   -1,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   58,   59,   60,   61,   62,  271,   -1,  262,   -1,   -1,
   -1,   -1,  275,   -1,   -1,   -1,  271,   -1,  284,   -1,
   -1,   -1,   -1,   -1,   -1,  288,   -1,  290,   -1,  284,
   -1,   -1,   -1,   91,   -1,   93,   -1,  300,   -1,  305,
   -1,   -1,   -1,  306,   -1,   -1,   -1,  310,   -1,   -1,
   -1,   -1,  318,  316,   -1,   -1,  319,   -1,   -1,  325,
   -1,   -1,   -1,  318,   -1,   -1,  329,  333,   -1,   -1,
  325,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,
   -1,   -1,   -1,   -1,  350,   -1,  352,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  350,  362,  352,  364,  365,
  366,   -1,   -1,   -1,   -1,  371,   -1,  362,   -1,  364,
  365,  366,   -1,   -1,   -1,   -1,  371,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  387,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  412,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  417,   -1,   -1,  412,  421,   -1,
   -1,   -1,   -1,  426,  419,   -1,   -1,   -1,   -1,  432,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  450,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,
   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,  506,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,  552,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,  598,  596,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,  598,  402,  403,  404,  405,   -1,  407,
   -1,  409,  615,  411,  412,  413,  414,  415,   -1,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,  609,  610,  611,  612,  613,    0,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   -1,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   58,   59,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  275,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,  290,   93,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  300,   -1,
   -1,   -1,   -1,   -1,  306,   -1,   -1,   -1,  310,  311,
   -1,   -1,   -1,   -1,  316,   -1,   -1,  319,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  329,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  379,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  387,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  417,   -1,   -1,   -1,  421,
   -1,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
  432,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  450,  451,
   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,
  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,
  304,  305,  306,   -1,  506,  309,   -1,  509,  312,   -1,
  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,
  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,
   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,
  364,  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,  615,  418,  419,  420,   -1,  422,  423,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,  435,   -1,  437,  438,  439,   -1,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,
  454,  455,  456,  457,  458,  459,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,   -1,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,
  604,  605,  606,  607,  608,  609,  610,  611,  612,  613,
    0,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,  651,   -1,  275,
   -1,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,  290,   -1,   -1,   -1,   58,   59,
   60,   61,   62,   -1,  300,   -1,   -1,   -1,   -1,   -1,
  306,   -1,   -1,   -1,   -1,  311,   -1,  275,   -1,   -1,
  316,   -1,   -1,  319,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   91,  290,   93,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  300,   -1,   -1,   -1,   -1,   -1,  306,   -1,
   -1,   -1,   -1,  311,   -1,   -1,   -1,   -1,  316,   -1,
   -1,  319,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  387,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  417,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  387,
   -1,   -1,   -1,   -1,   -1,   -1,  432,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  450,   -1,   -1,   -1,   -1,  417,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  432,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  450,   -1,   -1,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
  506,  271,   -1,  509,   -1,   -1,   -1,  277,   -1,  279,
  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,   -1,  506,  309,
   -1,  509,  312,   -1,  314,  315,   -1,   -1,  318,  319,
  320,   -1,  322,  323,  324,  325,  326,  327,  328,   -1,
   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,  615,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,  615,  418,  419,
  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,    0,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   -1,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   58,   59,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  300,   -1,   -1,   -1,   -1,   91,  306,   93,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  316,   -1,   -1,  319,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  387,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  402,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  417,   -1,   -1,
   -1,  421,   -1,   -1,  424,   -1,  426,   -1,   -1,   -1,
   -1,   -1,  432,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  450,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,
   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,  506,   -1,   -1,  509,
   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,  305,
  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,  552,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,
   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,  596,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,
   -1,  397,   -1,  399,   -1,  615,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,
   -1,  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,
  496,   -1,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,  605,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   -1,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,
  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,   -1,   -1,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,
   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,  609,  610,  611,  612,  613,    0,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   -1,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,
  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,
  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,
  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,
  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,
   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,
  364,  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,  435,   -1,  437,  438,  439,   -1,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,
  454,  455,  456,  457,  458,  459,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,   -1,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,
  604,  605,  606,  607,  608,  609,  610,  611,  612,  613,
    0,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,
   -1,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,
  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,
  320,   -1,  322,  323,  324,  325,  326,  327,  328,   -1,
   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,  419,
  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,    0,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   -1,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,
   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,  305,
  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,
   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,
   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,
   -1,  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,
  496,   -1,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,  605,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   -1,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,
  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,   -1,   -1,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,
   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,  609,  610,  611,  612,  613,    0,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   -1,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,
  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,
  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,
  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,
  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,
   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,
  364,  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,  435,   -1,  437,  438,  439,   -1,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,
  454,  455,  456,  457,  458,  459,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,   -1,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,
  604,  605,  606,  607,  608,  609,  610,  611,  612,  613,
    0,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,
   -1,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,
  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,
  320,   -1,  322,  323,  324,  325,  326,  327,  328,   -1,
   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,  419,
  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,    0,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   -1,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,
   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,  305,
  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,
   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,
   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,
   -1,  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,
  496,   -1,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,  605,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   -1,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,
  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,   -1,   -1,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,
   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,  605,  606,  607,
  608,  609,  610,  611,  612,  613,    0,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   -1,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,
  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,
  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,
  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,
  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,
  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,
   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,
  364,  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,  435,   -1,  437,  438,  439,   -1,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,
  454,  455,  456,  457,  458,  459,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,   -1,   -1,  499,  500,  501,  502,  503,
  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,
  604,  605,  606,  607,  608,  609,  610,  611,  612,  613,
    0,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,
   -1,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,
  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,
  320,   -1,  322,  323,  324,  325,  326,  327,  328,   -1,
   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,  419,
  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,    0,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   -1,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,
   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,
   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,  305,
  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,
   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,  325,
  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,
   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,
  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,
  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,
   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,
   -1,  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,  455,
  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,
  496,   -1,   -1,  499,  500,  501,  502,  503,  504,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,  598,  599,  600,  601,  602,  603,  604,  605,
  606,  607,  608,  609,  610,  611,  612,  613,    0,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  651,   -1,   -1,   -1,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,
   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,
  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,  281,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,
  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,
  322,  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,
  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,
  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,
  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,
  372,   -1,  374,  375,  376,  377,  378,   -1,  380,  381,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,
  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,
  452,   -1,  454,  455,  456,  457,  458,  459,  460,  461,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,
  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,  501,
  502,  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,
  602,  603,  604,  605,  606,  607,  608,  609,  610,  611,
  612,  613,    0,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  651,
   -1,   -1,   -1,   41,   42,   43,   44,   45,   -1,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,
   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,
   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
  318,  319,  320,   -1,  322,  323,  324,  325,  326,  327,
  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,   -1,  606,  607,
  608,  609,  610,  611,  612,  613,  614,  615,  616,  617,
  618,    0,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   41,   42,   43,   44,   45,   -1,   47,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,
  279,  280,  281,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,
  449,  450,   -1,  452,   -1,  454,  455,  456,  457,  458,
  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,
   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,
   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,
  499,  500,  501,  502,  503,  504,  505,  506,  507,   -1,
  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  519,  520,  521,  522,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,   -1,  541,  542,  543,  544,  545,  546,   -1,  548,
  549,  550,  551,   -1,  553,   -1,  555,  556,  557,  558,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,   -1,  574,  575,  576,  577,   -1,
  579,  580,  581,  582,  583,   -1,  585,  586,  587,  588,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,   -1,  606,  607,  608,
  609,  610,  611,  612,  613,    0,  615,  616,  617,  618,
   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,  628,
  629,  630,  631,  632,  633,  634,  635,  636,   -1,   -1,
   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,
  649,   -1,  651,   -1,   -1,   -1,   41,   42,   43,   44,
   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,  277,   -1,  279,  280,  281,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,  441,   -1,   -1,   -1,
  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,
  455,  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,
  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,
  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,
  495,  496,   -1,   -1,  499,  500,  501,  502,  503,  504,
  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,   -1,  541,  542,  543,  544,
  545,  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,
  555,  556,  557,  558,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,   -1,  574,
  575,  576,  577,   -1,  579,  580,  581,  582,  583,   -1,
  585,  586,  587,  588,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
   -1,  606,  607,  608,  609,  610,  611,  612,  613,    0,
  615,  616,  617,  618,   -1,  620,  621,  622,  623,  624,
  625,   -1,   -1,  628,  629,  630,  631,  632,  633,  634,
  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,
  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,   -1,
   41,   42,   43,   44,   45,   -1,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,  279,  280,
  281,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,
   -1,  452,   -1,  454,  455,  456,  457,  458,  459,  460,
  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,
  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,
  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,
  501,  502,  503,  504,  505,  506,  507,   -1,  509,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,
  521,  522,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,   -1,
  541,  542,  543,  544,  545,  546,   -1,  548,  549,  550,
  551,   -1,  553,   -1,  555,  556,  557,  558,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,   -1,  574,  575,  576,  577,   -1,  579,  580,
  581,  582,  583,   -1,  585,  586,  587,  588,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,   -1,  606,  607,  608,  609,  610,
  611,  612,  613,    0,  615,  616,  617,  618,   -1,  620,
  621,  622,  623,  624,  625,   -1,   -1,  628,  629,  630,
  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,
   -1,  642,   -1,  644,  645,  646,  647,  648,  649,   -1,
  651,   -1,   -1,   -1,   41,   42,   43,   44,   45,   -1,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,  281,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,
  447,  448,  449,  450,   -1,  452,   -1,  454,  455,  456,
  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,
   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,
  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,
   -1,   -1,  499,  500,  501,  502,  503,  504,  505,  506,
  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  519,  520,  521,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,   -1,  541,  542,  543,  544,  545,  546,
   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,  556,
  557,  558,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,   -1,  574,  575,  576,
  577,   -1,  579,  580,  581,  582,  583,   -1,  585,  586,
  587,  588,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,   -1,  606,
  607,  608,  609,  610,  611,  612,  613,    0,  615,  616,
  617,  618,   -1,  620,  621,  622,  623,  624,  625,   -1,
   -1,  628,  629,  630,  631,  632,  633,  634,  635,  636,
   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,
  647,  648,  649,   -1,  651,   -1,   -1,   -1,   41,   42,
   43,   44,   45,   -1,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,  441,   -1,
   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,
   -1,  454,  455,  456,  457,  458,  459,  460,  461,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,
   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,
  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,
   -1,   -1,  495,  496,   -1,   -1,  499,  500,  501,  502,
  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,    0,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,
   -1,  644,  645,  646,  647,  648,  649,   -1,  651,   -1,
   -1,   -1,   41,   42,   43,   44,   45,   -1,   47,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,
  449,  450,   -1,  452,   -1,  454,  455,  456,  457,  458,
  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,
   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,
   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,
  499,  500,  501,  502,  503,  504,  505,  506,  507,   -1,
  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  519,  520,  521,  522,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,   -1,  541,  542,  543,  544,  545,  546,   -1,  548,
  549,  550,  551,   -1,  553,   -1,  555,  556,  557,  558,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,   -1,  574,  575,  576,  577,   -1,
  579,  580,  581,  582,  583,   -1,  585,  586,  587,  588,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,   -1,  606,  607,  608,
  609,  610,  611,  612,  613,    0,  615,  616,  617,  618,
   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,  628,
  629,  630,  631,  632,  633,  634,  635,  636,   -1,   -1,
   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,
  649,   -1,  651,   -1,   -1,   -1,   41,   42,   43,   44,
   45,   -1,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,  441,   -1,   -1,   -1,
  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,
  455,  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,
  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,
  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,
  495,  496,   -1,   -1,  499,  500,  501,  502,  503,  504,
  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,   -1,  541,  542,  543,  544,
  545,  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,
  555,  556,  557,  558,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,   -1,  574,
  575,  576,  577,   -1,  579,  580,  581,  582,  583,   -1,
  585,  586,  587,  588,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
   -1,  606,  607,  608,  609,  610,  611,  612,  613,    0,
  615,  616,  617,  618,   -1,  620,  621,  622,  623,  624,
  625,   -1,   -1,  628,  629,  630,  631,  632,  633,  634,
  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,
  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,   -1,
   41,   42,   43,   44,   45,   -1,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,
   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,  333,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,
   -1,  452,   -1,  454,  455,  456,  457,  458,  459,  460,
  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,
  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,
  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,
  501,  502,  503,  504,  505,  506,  507,   -1,  509,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,
  521,  522,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,   -1,
  541,  542,  543,  544,  545,  546,   -1,  548,  549,  550,
  551,   -1,  553,   -1,  555,  556,  557,  558,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,   -1,  574,  575,  576,  577,   -1,  579,  580,
  581,  582,  583,   -1,  585,  586,  587,  588,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,   -1,  606,  607,  608,  609,  610,
  611,  612,  613,    0,  615,  616,  617,  618,   -1,  620,
  621,  622,  623,  624,  625,   -1,   -1,  628,  629,  630,
  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,
   -1,  642,   -1,  644,  645,  646,  647,  648,  649,   -1,
  651,   -1,   -1,   -1,   41,   -1,   43,   44,   45,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,  333,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,
  447,  448,  449,  450,   -1,  452,   -1,  454,  455,  456,
  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,
   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,
  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,
   -1,   -1,  499,  500,  501,  502,  503,  504,  505,  506,
  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  519,  520,  521,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,   -1,  541,  542,  543,  544,  545,  546,
   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,  556,
  557,  558,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,   -1,  574,  575,  576,
  577,   -1,  579,  580,  581,  582,  583,   -1,  585,  586,
  587,  588,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,   -1,  606,
  607,  608,  609,  610,  611,  612,  613,    0,  615,  616,
  617,  618,   -1,  620,  621,  622,  623,  624,  625,   -1,
   -1,  628,  629,  630,  631,  632,  633,  634,  635,  636,
   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,
  647,  648,  649,   -1,  651,   -1,   -1,   -1,   41,   -1,
   43,   44,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
  333,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,  441,   -1,
   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,
   -1,  454,  455,  456,  457,  458,  459,  460,  461,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,
   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,
  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,
   -1,   -1,  495,  496,   -1,   -1,  499,  500,  501,  502,
  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,    0,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,
   -1,  644,  645,  646,  647,  648,  649,   -1,  651,   -1,
   -1,   -1,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,  333,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,
  449,  450,   -1,  452,   -1,  454,  455,  456,  457,  458,
  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,
   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,
   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,
  499,  500,  501,  502,  503,  504,  505,  506,  507,   -1,
  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  519,  520,  521,  522,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,   -1,  541,  542,  543,  544,  545,  546,   -1,  548,
  549,  550,  551,   -1,  553,   -1,  555,  556,  557,  558,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,   -1,  574,  575,  576,  577,   -1,
  579,  580,  581,  582,  583,   -1,  585,  586,  587,  588,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,   -1,  606,  607,  608,
  609,  610,  611,  612,  613,    0,  615,  616,  617,  618,
   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,  628,
  629,  630,  631,  632,  633,  634,  635,  636,   -1,   -1,
   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,
  649,   -1,  651,   -1,   -1,   -1,   41,   -1,   -1,   44,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,  333,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,  441,   -1,   -1,   -1,
  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,
  455,  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,
  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,
  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,
  495,  496,   -1,   -1,  499,  500,  501,  502,  503,  504,
  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,   -1,  541,  542,  543,  544,
  545,  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,
  555,  556,  557,  558,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,   -1,  574,
  575,  576,  577,   -1,  579,  580,  581,  582,  583,   -1,
  585,  586,  587,  588,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
   -1,  606,  607,  608,  609,  610,  611,  612,  613,    0,
  615,  616,  617,  618,   -1,  620,  621,  622,  623,  624,
  625,   -1,   -1,  628,  629,  630,  631,  632,  633,  634,
  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,
  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,   -1,
   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,
   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,   -1,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,
   -1,  452,   -1,  454,  455,  456,  457,  458,  459,  460,
  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,
  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,
  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,
  501,  502,  503,  504,  505,  506,  507,   -1,  509,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,
  521,  522,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,   -1,
  541,  542,  543,  544,  545,  546,   -1,  548,  549,  550,
  551,   -1,  553,   -1,  555,  556,  557,  558,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,   -1,  574,  575,  576,  577,   -1,  579,  580,
  581,  582,  583,   -1,  585,  586,  587,  588,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,   -1,  606,  607,  608,  609,  610,
  611,  612,  613,    0,  615,  616,  617,  618,   -1,  620,
  621,  622,  623,  624,  625,   -1,   -1,  628,  629,  630,
  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,
   -1,  642,   -1,  644,  645,  646,  647,  648,  649,   -1,
  651,   -1,   -1,   -1,   41,   -1,   -1,   44,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,   -1,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,
  447,  448,  449,  450,   -1,  452,   -1,  454,  455,  456,
  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,
   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,
  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,
   -1,   -1,  499,  500,  501,  502,  503,  504,  505,  506,
  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  519,  520,  521,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,   -1,  541,  542,  543,  544,  545,  546,
   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,  556,
  557,  558,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,   -1,  574,  575,  576,
  577,   -1,  579,  580,  581,  582,  583,   -1,  585,  586,
  587,  588,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,   -1,  606,
  607,  608,  609,  610,  611,  612,  613,    0,  615,  616,
  617,  618,   -1,  620,  621,  622,  623,  624,  625,   -1,
   -1,  628,  629,  630,  631,  632,  633,  634,  635,  636,
   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,
  647,  648,  649,   -1,  651,   -1,   -1,   -1,   41,   -1,
   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,  305,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
   -1,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,  441,   -1,
   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,
   -1,  454,  455,  456,  457,  458,  459,  460,  461,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,
   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,
  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,
   -1,   -1,  495,  496,   -1,   -1,  499,  500,  501,  502,
  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,    0,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,
   -1,  644,  645,  646,  647,  648,  649,   -1,  651,   -1,
   -1,   -1,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,   -1,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,
  449,  450,   -1,  452,   -1,  454,  455,  456,  457,  458,
  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,
   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,
   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,
  499,  500,  501,  502,  503,  504,  505,  506,  507,   -1,
  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  519,  520,  521,  522,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,   -1,  541,  542,  543,  544,  545,  546,   -1,  548,
  549,  550,  551,   -1,  553,   -1,  555,  556,  557,  558,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,   -1,  574,  575,  576,  577,   -1,
  579,  580,  581,  582,  583,   -1,  585,  586,  587,  588,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,   -1,  606,  607,  608,
  609,  610,  611,  612,  613,    0,  615,  616,  617,  618,
   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,  628,
  629,  630,  631,  632,  633,  634,  635,  636,   -1,   -1,
   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,
  649,   -1,  651,   -1,   -1,   -1,   41,   -1,   -1,   44,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
  305,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,   -1,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,  441,   -1,   -1,   -1,
  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,
  455,  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,
  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,
  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,
  495,  496,   -1,   -1,  499,  500,  501,  502,  503,  504,
  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,   -1,  541,  542,  543,  544,
  545,  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,
  555,  556,  557,  558,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,   -1,  574,
  575,  576,  577,   -1,  579,  580,  581,  582,  583,   -1,
  585,  586,  587,  588,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
   -1,  606,  607,  608,  609,  610,  611,  612,  613,    0,
  615,  616,  617,  618,   -1,  620,  621,  622,  623,  624,
  625,   -1,   -1,  628,  629,  630,  631,  632,  633,  634,
  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,
  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,   -1,
   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,
   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,   -1,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,
   -1,  452,   -1,  454,  455,  456,  457,  458,  459,  460,
  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,
  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,
  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,
  501,  502,  503,  504,  505,  506,  507,   -1,  509,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,
  521,  522,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,   -1,
  541,  542,  543,  544,  545,  546,   -1,  548,  549,  550,
  551,   -1,  553,   -1,  555,  556,  557,  558,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,   -1,  574,  575,  576,  577,   -1,  579,  580,
  581,  582,  583,   -1,  585,  586,  587,  588,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,   -1,  606,  607,  608,  609,  610,
  611,  612,  613,    0,  615,  616,  617,  618,   -1,  620,
  621,  622,  623,  624,  625,   -1,   -1,  628,  629,  630,
  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,
   -1,  642,   -1,  644,  645,  646,  647,  648,  649,   -1,
  651,   -1,   -1,   -1,   41,   -1,   -1,   44,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,  262,   -1,  264,  265,   -1,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,  302,  303,  304,  305,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,
   -1,  318,  319,  320,   -1,  322,  323,  324,  325,  326,
  327,  328,   -1,   -1,   -1,  332,   -1,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,  350,  351,  352,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,  362,   -1,  364,  365,  366,
   -1,  368,   -1,  370,  371,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,  418,  419,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,
  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,
  447,  448,  449,  450,   -1,  452,   -1,  454,  455,  456,
  457,  458,  459,  460,  461,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,
   -1,   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,
  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,
   -1,   -1,  499,  500,  501,  502,  503,  504,  505,  506,
  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  519,  520,  521,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,   -1,  541,  542,  543,  544,  545,  546,
   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,  556,
  557,  558,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,   -1,  574,  575,  576,
  577,   -1,  579,  580,  581,  582,  583,   -1,  585,  586,
  587,  588,  589,  590,  591,  592,  593,  594,  595,  596,
  597,  598,  599,  600,  601,  602,  603,  604,   -1,  606,
  607,  608,  609,  610,  611,  612,  613,    0,  615,  616,
  617,  618,   -1,  620,  621,  622,  623,  624,  625,   -1,
   -1,  628,  629,  630,  631,  632,  633,  634,  635,  636,
   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,
  647,  648,  649,   -1,  651,   -1,   -1,   -1,   41,   -1,
   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,  262,
   -1,  264,  265,   -1,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,  302,
  303,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,  312,
   -1,  314,  315,   -1,   -1,  318,  319,  320,   -1,  322,
  323,  324,  325,  326,  327,  328,   -1,   -1,   -1,  332,
   -1,  334,  335,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,  349,  350,  351,  352,
  353,   -1,   -1,   -1,  357,  358,   -1,  360,   -1,  362,
   -1,  364,  365,  366,   -1,  368,   -1,  370,  371,  372,
   -1,  374,  375,  376,  377,  378,   -1,  380,  381,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,  394,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,  418,  419,  420,   -1,  422,
  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,  435,   -1,  437,  438,  439,   -1,  441,   -1,
   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,
   -1,  454,  455,  456,  457,  458,  459,  460,  461,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,
   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,
  483,  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,
   -1,   -1,  495,  496,   -1,   -1,  499,  500,  501,  502,
  503,  504,  505,  506,  507,   -1,  509,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,    0,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,
   -1,  644,  645,  646,  647,  648,  649,   -1,  651,   -1,
   -1,   -1,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,   -1,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,  318,
  319,  320,   -1,  322,  323,  324,  325,  326,  327,  328,
   -1,   -1,   -1,  332,   -1,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,  350,  351,  352,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,  418,
  419,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,
  449,  450,   -1,  452,   -1,  454,  455,  456,  457,  458,
  459,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,
   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,
   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,
  499,  500,  501,  502,  503,  504,  505,  506,  507,   -1,
  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  519,  520,  521,  522,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,   -1,  541,  542,  543,  544,  545,  546,   -1,  548,
  549,  550,  551,   -1,  553,   -1,  555,  556,  557,  558,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,   -1,  574,  575,  576,  577,   -1,
  579,  580,  581,  582,  583,   -1,  585,  586,  587,  588,
  589,  590,  591,  592,  593,  594,  595,  596,  597,  598,
  599,  600,  601,  602,  603,  604,   -1,  606,  607,  608,
  609,  610,  611,  612,  613,    0,  615,  616,  617,  618,
   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,  628,
  629,  630,  631,  632,  633,  634,  635,  636,   -1,   -1,
   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,
  649,   -1,  651,   -1,   -1,   -1,   41,   -1,   -1,   44,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  257,   -1,  259,   -1,   -1,  262,   -1,  264,
  265,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,  302,  303,  304,
   -1,  306,   -1,   -1,  309,   -1,   -1,  312,   -1,  314,
  315,   -1,   -1,  318,  319,  320,   -1,  322,  323,  324,
  325,  326,  327,  328,   -1,   -1,   -1,  332,   -1,  334,
  335,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,  362,   -1,  364,
  365,  366,   -1,  368,   -1,  370,  371,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,  394,
   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,  418,  419,  420,   -1,  422,  423,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,
  435,   -1,  437,  438,  439,   -1,  441,   -1,   -1,   -1,
  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,  454,
  455,  456,  457,  458,  459,  460,  461,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,
  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,  484,
  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,
  495,  496,   -1,   -1,  499,  500,  501,  502,  503,  504,
  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,   -1,  541,  542,  543,  544,
  545,  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,
  555,  556,  557,  558,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,   -1,  574,
  575,  576,  577,   -1,  579,  580,  581,  582,  583,   -1,
  585,  586,  587,  588,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
   -1,  606,  607,  608,  609,  610,  611,  612,  613,    0,
  615,  616,  617,  618,   -1,  620,  621,  622,  623,  624,
  625,   -1,   -1,  628,  629,  630,  631,  632,  633,  634,
  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,
  645,  646,  647,  648,  649,   -1,  651,   -1,   -1,   -1,
   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,
   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,
   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,
   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,  302,  303,  304,   -1,  306,   -1,   -1,  309,   -1,
   -1,  312,   -1,  314,  315,   -1,   -1,  318,  319,  320,
   -1,  322,  323,  324,  325,  326,  327,  328,   -1,   -1,
   -1,  332,   -1,  334,  335,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,  350,
  351,  352,  353,   -1,   -1,   -1,  357,  358,   -1,  360,
   -1,  362,   -1,  364,  365,  366,   -1,  368,   -1,  370,
  371,  372,   -1,  374,  375,  376,  377,  378,   -1,  380,
  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,  418,  419,  420,
    0,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,   -1,
  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,
   -1,  452,   -1,  454,  455,  456,  457,  458,  459,  460,
  461,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,
  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,   59,
  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,  490,
  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,  500,
  501,  502,  503,  504,  505,  506,  507,   -1,  509,   -1,
   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,  519,  520,
  521,  522,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,   -1,
  541,  542,  543,  544,  545,  546,   -1,  548,  549,  550,
  551,   -1,  553,   -1,  555,  556,  557,  558,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,   -1,  574,  575,  576,  577,   -1,  579,  580,
  581,  582,  583,   -1,  585,  586,  587,  588,  589,  590,
  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,
  601,  602,  603,  604,   -1,  606,  607,  608,  609,  610,
  611,  612,  613,   -1,  615,  616,  617,  618,   -1,  620,
  621,  622,  623,  624,  625,   -1,   -1,  628,  629,  630,
  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,
   -1,  642,   -1,  644,  645,  646,  647,  648,  649,   -1,
  651,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,  259,
   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,   -1,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,  309,
   -1,   -1,  312,   -1,  314,  315,   -1,   -1,   -1,  319,
  320,   -1,  322,  323,  324,   -1,  326,  327,  328,   -1,
   -1,   -1,  332,   -1,  334,  335,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,  349,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
  360,   -1,   -1,   -1,  364,   -1,   -1,   -1,  368,   -1,
  370,  371,  372,   -1,  374,  375,  376,  377,  378,   -1,
  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,   -1,  413,  414,  415,   -1,   -1,  418,  419,
  420,    0,  422,  423,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   59,  481,  482,  483,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,  499,
  500,  501,  502,  503,  504,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,   -1,  599,
  600,  601,  602,  603,  604,   -1,  606,  607,  608,  609,
  610,  611,  612,  613,   -1,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  651,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,  262,   -1,  264,  265,   -1,   -1,  268,
  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,   -1,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,  302,  303,  304,  305,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,   -1,
  319,  320,   -1,  322,  323,  324,   -1,  326,  327,  328,
   -1,   -1,   -1,  332,   -1,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  368,
   -1,  370,  371,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,   -1,  413,  414,  415,   -1,   -1,  418,
  419,  420,    0,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,  438,
  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,
  449,  450,   -1,  452,   -1,  454,  455,  456,  457,  458,
  459,  460,  461,   41,   -1,   -1,   44,   -1,   -1,   -1,
   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,
   -1,   59,  481,  482,  483,  484,  485,   -1,  487,   -1,
   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,
  499,  500,  501,  502,  503,  504,  505,  506,  507,   -1,
  509,   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,
  519,  520,  521,  522,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,   -1,  541,  542,  543,  544,  545,  546,   -1,  548,
  549,  550,  551,   -1,  553,   -1,  555,  556,  557,  558,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,   -1,  574,  575,  576,  577,   -1,
  579,  580,  581,  582,  583,   -1,  585,  586,  587,  588,
  589,  590,  591,  592,  593,  594,  595,  596,  597,   -1,
  599,  600,  601,  602,  603,  604,   -1,  606,  607,  608,
  609,  610,  611,  612,  613,   -1,  615,  616,  617,  618,
   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,  628,
  629,  630,  631,  632,  633,  634,  635,  636,   -1,   -1,
   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,
  649,   -1,  651,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,
   -1,  259,   -1,   -1,   -1,   -1,  264,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,   -1,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,  302,  303,  304,  305,  306,   -1,
   -1,  309,   -1,   -1,  312,   -1,  314,  315,   -1,   -1,
   -1,  319,  320,   -1,  322,  323,  324,   -1,  326,  327,
  328,   -1,   -1,   -1,  332,   -1,  334,  335,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,  349,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  368,   -1,  370,  371,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,  394,   -1,   -1,  397,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,   -1,  413,  414,  415,   -1,   -1,
  418,  419,  420,    0,  422,  423,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,  434,  435,   -1,  437,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,  454,  455,  456,  457,
  458,  459,  460,  461,   41,   -1,   -1,   44,   -1,   -1,
   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   59,  481,  482,  483,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,  499,  500,  501,  502,  503,  504,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
   -1,  599,  600,  601,  602,  603,  604,   -1,  606,  607,
  608,  609,  610,  611,  612,  613,   -1,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,  651,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,   -1,  259,   -1,   -1,   -1,   -1,  264,  265,   -1,
   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,   -1,  309,   -1,   -1,  312,   -1,   -1,  315,   -1,
   -1,   -1,  319,  320,   -1,  322,  323,  324,   -1,  326,
  327,  328,   -1,   -1,   -1,  332,   -1,  334,  335,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,  349,   -1,  351,   -1,  353,   -1,   -1,   -1,
  357,  358,   -1,  360,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  368,   -1,  370,   -1,  372,   -1,  374,  375,  376,
  377,  378,   -1,  380,  381,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,    0,   -1,  420,   -1,  422,  423,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,  435,   -1,
  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,
  447,  448,  449,  450,   -1,  452,   -1,   -1,   -1,   -1,
   -1,   40,   41,  460,  461,   44,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,
   59,   -1,   -1,   -1,  481,  482,  483,  484,  485,   -1,
  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,
   -1,   -1,  499,  500,  501,  502,  503,  504,  505,  506,
  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  519,  520,  521,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,   -1,  541,  542,  543,  544,  545,  546,
   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,  556,
  557,  558,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,   -1,  574,  575,  576,
  577,   -1,  579,  580,  581,  582,  583,   -1,  585,  586,
  587,  588,  589,  590,  591,  592,  593,  594,  595,  596,
  597,   -1,  599,  600,  601,  602,  603,  604,   -1,  606,
  607,  608,  609,  610,  611,  612,  613,   -1,  615,  616,
  617,  618,   -1,  620,  621,  622,  623,  624,  625,   -1,
   -1,  628,  629,  630,  631,  632,  633,  634,  635,  636,
   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,
  647,  648,  649,   -1,  651,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,
  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,   -1,  315,   -1,   -1,   -1,
  319,  320,   -1,  322,  323,  324,   -1,  326,  327,  328,
   -1,   -1,   -1,  332,   -1,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  368,
   -1,  370,   -1,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,   -1,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,    0,  411,  412,  413,  414,  415,   -1,   -1,   -1,
   -1,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,   -1,  435,   -1,  437,  438,
  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,
  449,  450,   41,  452,   -1,   44,   -1,   -1,   -1,   -1,
   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,
   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,
   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,
  499,  500,  501,  502,  503,  504,  505,  506,  507,   -1,
  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  519,  520,  521,  522,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,   -1,  541,  542,  543,  544,  545,  546,   -1,  548,
  549,  550,  551,   -1,  553,   -1,  555,  556,  557,  558,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,   -1,  574,  575,  576,  577,   -1,
  579,  580,  581,  582,  583,   -1,  585,  586,  587,  588,
  589,  590,  591,  592,  593,  594,  595,  596,  597,   -1,
  599,  600,  601,  602,  603,  604,   -1,  606,  607,  608,
  609,  610,  611,  612,  613,   -1,  615,  616,  617,  618,
   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,  628,
  629,  630,  631,  632,  633,  634,  635,  636,   -1,   -1,
   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,
  649,   -1,  651,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,
  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,   -1,  315,   -1,   -1,   -1,
  319,  320,   -1,  322,  323,  324,   -1,  326,  327,  328,
   -1,   -1,   -1,  332,   -1,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  368,
   -1,  370,   -1,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,   -1,   -1,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,    0,  411,  412,  413,  414,  415,   -1,   -1,   -1,
   -1,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,   -1,  435,   -1,  437,  438,
  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,
  449,  450,   41,  452,   -1,   44,   -1,   -1,   -1,   -1,
   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,
   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,
   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,
  499,  500,  501,  502,  503,  504,  505,  506,  507,   -1,
  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  520,  521,  522,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,   -1,  541,  542,  543,  544,  545,  546,   -1,  548,
  549,  550,  551,   -1,  553,   -1,  555,  556,  557,  558,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,   -1,  574,  575,  576,  577,   -1,
  579,  580,  581,  582,  583,   -1,  585,  586,  587,  588,
  589,  590,  591,  592,  593,  594,  595,  596,  597,   -1,
  599,  600,  601,  602,  603,  604,   -1,  606,  607,  608,
  609,  610,  611,  612,  613,   -1,  615,  616,  617,  618,
   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,  628,
  629,  630,  631,  632,  633,  634,  635,  636,   -1,   -1,
   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,
  649,   -1,  651,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,   -1,
  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,
  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,
  309,   -1,   -1,  312,   -1,   -1,  315,   -1,   -1,   -1,
  319,  320,   -1,  322,  323,  324,   -1,  326,  327,  328,
   -1,   -1,   -1,  332,   -1,  334,  335,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
  349,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  368,
   -1,  370,   -1,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,  381,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,   -1,    0,   -1,  397,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,
   -1,  420,   -1,  422,  423,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,   -1,  435,   -1,  437,  438,
  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,
  449,  450,   -1,  452,   -1,   59,   -1,   -1,   -1,   -1,
   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,
   -1,   -1,  481,  482,  483,  484,  485,   -1,  487,   -1,
   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,
  499,  500,  501,  502,  503,  504,  505,  506,  507,   -1,
  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  520,  521,  522,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,   -1,  541,  542,  543,  544,  545,  546,   -1,  548,
  549,  550,  551,   -1,  553,   -1,  555,  556,  557,  558,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,   -1,  574,  575,  576,  577,   -1,
  579,  580,  581,  582,  583,   -1,  585,  586,  587,  588,
  589,  590,  591,  592,  593,  594,  595,  596,  597,   -1,
  599,  600,  601,  602,  603,  604,   -1,  606,  607,  608,
  609,  610,  611,  612,  613,   -1,  615,  616,  617,  618,
   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,  628,
  629,  630,  631,  632,  633,  634,  635,  636,   -1,   -1,
   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,
  649,   -1,  651,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,
  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,  372,   -1,
  374,  375,  376,  377,  378,   -1,  380,  381,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
   -1,   -1,    0,   -1,   -1,  399,   -1,  401,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,  435,   -1,   -1,  438,  439,   44,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,
   -1,   59,   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,  483,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,   -1,   -1,   -1,  500,  501,  502,  503,
   -1,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,   -1,  599,  600,  601,  602,  603,
  604,   -1,  606,  607,  608,  609,  610,  611,  612,  613,
   -1,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,   -1,   -1,   -1,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,   -1,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,  314,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,  332,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,   -1,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  460,  461,   -1,   -1,   -1,   40,   -1,   -1,
   43,   -1,   45,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,   58,  484,  485,   -1,  487,
   63,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,   -1,   -1,  501,  502,  503,   -1,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
   -1,  599,  600,  601,  602,  603,  604,   -1,  606,  607,
  608,  609,  610,  611,  612,  613,   -1,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,  454,  455,  456,  457,  458,  459,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,  508,  509,   -1,  511,  512,
  513,  514,  515,  516,  517,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,  260,   -1,   -1,
  263,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,  408,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   42,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,  260,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,  298,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,   -1,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,  288,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,  434,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,  437,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   42,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   41,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,  314,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,  419,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   41,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
  483,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   42,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,  260,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,  298,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,   -1,   -1,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   42,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,  308,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,  364,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,  492,
  493,  494,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,  437,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,   -1,
   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,   -1,
   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   40,   -1,   -1,
   43,   -1,   45,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   -1,   -1,   -1,   -1,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,  267,  268,  269,   -1,   -1,   -1,
   -1,  274,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,  285,   -1,  287,   -1,   -1,  290,  291,  292,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  317,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,  344,   -1,  346,  347,  348,  349,   -1,  351,   -1,
  353,  354,   -1,  356,  357,  358,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,  382,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,  410,  411,  412,
  413,  414,  415,  416,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,  440,  441,  442,
  443,  444,  445,   -1,  447,  448,  449,  450,   -1,  452,
  453,   40,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,
   -1,  474,  475,  476,  477,  478,   -1,  480,  481,  482,
   -1,  484,  485,  486,  487,   -1,   -1,  490,  491,   -1,
   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,
  513,   -1,  515,  516,   -1,  518,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,  637,   -1,  639,  640,  641,  642,
  643,  644,  645,  646,  647,  648,  649,   -1,   -1,  652,
  653,  654,  655,  656,  657,  658,  659,  660,  661,  662,
  663,  664,  665,  666,  667,  668,  669,   -1,  257,   -1,
  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,
  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,
  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,
  329,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,
   -1,  360,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  370,   -1,  372,   -1,  374,  375,  376,  377,  378,
   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,
   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,   40,   -1,   -1,   -1,  438,
  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,
  449,  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,
   -1,   -1,  481,  482,   -1,  484,  485,   -1,  487,   -1,
   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,
   -1,   -1,  501,  502,  503,   -1,  505,  506,  507,   -1,
  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  519,  520,  521,  522,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,   -1,  541,  542,  543,  544,  545,  546,   -1,  548,
  549,  550,  551,   -1,  553,   -1,  555,  556,  557,  558,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,   -1,  574,  575,  576,  577,   -1,
  579,  580,  581,  582,  583,   -1,  585,  586,  587,  588,
  589,  590,  591,  592,  593,  594,  595,  596,  597,   -1,
  599,  600,  601,  602,  603,  604,   -1,  606,  607,  608,
  609,  610,  611,  612,  613,   -1,  615,  616,  617,  618,
   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,  628,
  629,  630,  631,  632,  633,  634,  635,  636,   -1,   -1,
   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,
  649,   -1,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,
  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,
   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,
   -1,  326,   -1,  328,  329,   -1,   -1,   -1,   -1,  334,
   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,
   -1,   -1,  357,  358,   -1,  360,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  370,   -1,  372,   -1,  374,
  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,
   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,
   40,   -1,   -1,  438,  439,   -1,  441,   -1,   -1,   -1,
  445,   -1,  447,  448,  449,  450,   -1,  452,   58,   -1,
   -1,   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,
  475,  476,   -1,   -1,   -1,   -1,  481,  482,   -1,  484,
  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,
  495,  496,   -1,   -1,   -1,   -1,  501,  502,  503,   -1,
  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,   -1,  541,  542,  543,  544,
  545,  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,
  555,  556,  557,  558,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,   -1,  574,
  575,  576,  577,   -1,  579,  580,  581,  582,  583,   -1,
  585,  586,  587,  588,  589,  590,  591,  592,  593,  594,
  595,  596,  597,   -1,  599,  600,  601,  602,  603,  604,
   -1,  606,  607,  608,  609,  610,  611,  612,  613,   -1,
  615,  616,  617,  618,   -1,  620,  621,  622,  623,  624,
  625,   -1,   -1,  628,  629,  630,  631,  632,  633,  634,
  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,
  645,  646,  647,  648,  649,   -1,   -1,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,   40,   -1,   -1,   -1,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,   -1,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,   -1,
   -1,  501,  502,  503,   -1,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,   -1,  599,
  600,  601,  602,  603,  604,   -1,  606,  607,  608,  609,
  610,  611,  612,  613,   -1,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
   -1,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,
   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,
   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   40,   -1,
   -1,  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,   -1,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,
  496,   -1,   -1,   -1,   -1,  501,  502,  503,   -1,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,   -1,  599,  600,  601,  602,  603,  604,   -1,
  606,  607,  608,  609,  610,  611,  612,  613,   -1,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,   -1,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,
   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   40,   -1,   -1,   -1,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,
  452,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,
  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,   -1,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,  501,
  502,  503,   -1,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,   -1,  599,  600,  601,
  602,  603,  604,   -1,  606,  607,  608,  609,  610,  611,
  612,  613,   -1,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,   -1,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   42,   -1,   -1,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,   -1,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,   -1,   -1,  501,  502,  503,   -1,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
   -1,  599,  600,  601,  602,  603,  604,   -1,  606,  607,
  608,  609,  610,  611,  612,  613,   -1,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,
  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,  292,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,
   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,
   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,
   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,
   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,
  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,
   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,
   -1,   -1,   -1,  438,  439,   45,  441,   -1,   -1,   -1,
  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,
  475,  476,   -1,   -1,   -1,   -1,  481,  482,   -1,  484,
  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,
  495,  496,   -1,   -1,   -1,   -1,  501,  502,  503,   -1,
  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,   -1,  541,  542,  543,  544,
  545,  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,
  555,  556,  557,  558,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,   -1,  574,
  575,  576,  577,   -1,  579,  580,  581,  582,  583,   -1,
  585,  586,  587,  588,  589,  590,  591,  592,  593,  594,
  595,  596,  597,   -1,  599,  600,  601,  602,  603,  604,
   -1,  606,  607,  608,  609,  610,  611,  612,  613,   -1,
  615,  616,  617,  618,   -1,  620,  621,  622,  623,  624,
  625,   -1,   -1,  628,  629,  630,  631,  632,  633,  634,
  635,  636,   -1,   -1,   -1,   -1,   -1,  642,  643,  644,
  645,  646,  647,  648,  649,   -1,   -1,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,  441,  442,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   58,  452,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,   -1,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,   -1,
   -1,  501,  502,  503,   -1,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,   -1,  599,
  600,  601,  602,  603,  604,   -1,  606,  607,  608,  609,
  610,  611,  612,  613,   -1,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
   -1,  268,  269,  270,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,  296,
  297,   -1,  299,  300,   -1,   -1,  303,  304,   -1,  306,
  307,   -1,  309,   -1,   -1,  312,   -1,   -1,   -1,   -1,
   -1,   -1,  319,  320,   -1,   -1,   -1,  324,  325,  326,
   -1,  328,   -1,  330,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,  346,
   -1,  348,  349,   -1,  351,   -1,  353,   -1,  355,   -1,
  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,  382,  383,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
  397,   -1,  399,  400,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,  425,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,   -1,  436,
  437,  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,
  447,  448,  449,  450,   58,  452,  453,   -1,   -1,   -1,
   -1,   -1,   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,
   -1,   -1,   -1,   -1,  481,  482,   -1,  484,  485,   -1,
  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,
  497,  498,  499,   -1,  501,  502,  503,   -1,  505,  506,
  507,   -1,  509,   -1,  511,  512,  513,   -1,  515,  516,
   -1,  518,  519,  520,  521,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,   -1,  541,  542,  543,  544,  545,  546,
   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,  556,
  557,  558,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,   -1,  574,  575,  576,
  577,   -1,  579,  580,  581,  582,  583,   -1,  585,  586,
  587,  588,  589,  590,  591,  592,  593,  594,  595,  596,
  597,   -1,  599,  600,  601,  602,  603,  604,   -1,  606,
  607,  608,  609,  610,  611,  612,  613,   -1,  615,  616,
  617,  618,   -1,  620,  621,  622,  623,  624,  625,  626,
   -1,  628,  629,  630,  631,  632,  633,  634,  635,  636,
   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,
  647,  648,  649,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,  270,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,  296,  297,   -1,  299,  300,   -1,   -1,   -1,
  304,   -1,  306,  307,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,  325,  326,   -1,  328,   -1,  330,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,  346,   -1,  348,  349,   -1,  351,   -1,  353,
   -1,  355,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,  382,  383,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
   -1,   -1,   -1,  397,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,  425,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
  434,   -1,   -1,  437,  438,  439,   -1,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   58,  452,  453,
   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,   -1,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,  497,  498,  499,   -1,  501,  502,  503,
   -1,  505,  506,  507,   -1,  509,   -1,  511,  512,  513,
   -1,  515,  516,   -1,  518,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,   -1,  599,  600,  601,  602,  603,
  604,   -1,  606,  607,  608,  609,  610,  611,  612,  613,
   -1,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,  626,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,  257,   -1,  259,   -1,
   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,  270,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,
   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,  296,  297,   -1,  299,  300,
   -1,   -1,   -1,  304,   -1,  306,  307,   -1,  309,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,
   -1,   -1,   -1,  324,  325,  326,   -1,  328,   -1,  330,
   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,  346,   -1,  348,  349,   -1,
  351,   -1,  353,   -1,  355,   -1,  357,  358,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,
   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,
   -1,  382,  383,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,   -1,   -1,   -1,  397,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,
   -1,  422,   -1,   -1,  425,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,   -1,   -1,   -1,  437,  438,  439,   -1,
  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,
   58,  452,  453,   -1,   -1,   -1,   -1,   -1,   -1,  460,
  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,
  481,  482,   -1,  484,  485,   -1,  487,   -1,   -1,  490,
  491,   -1,   -1,   -1,  495,  496,  497,  498,  499,   -1,
  501,  502,  503,   -1,  505,  506,  507,   -1,  509,   -1,
  511,  512,  513,   -1,  515,  516,   -1,  518,  519,  520,
  521,  522,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,   -1,
  541,  542,  543,  544,  545,  546,   -1,  548,  549,  550,
  551,   -1,  553,   -1,  555,  556,  557,  558,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,   -1,  574,  575,  576,  577,   -1,  579,  580,
  581,  582,  583,   -1,  585,  586,  587,  588,  589,  590,
  591,  592,  593,  594,  595,  596,  597,   -1,  599,  600,
  601,  602,  603,  604,   -1,  606,  607,  608,  609,  610,
  611,  612,  613,   -1,  615,  616,  617,  618,   -1,  620,
  621,  622,  623,  624,  625,  626,   -1,  628,  629,  630,
  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,
   -1,  642,   -1,  644,  645,  646,  647,  648,  649,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,   -1,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,   -1,   -1,  501,  502,  503,   -1,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
   -1,  599,  600,  601,  602,  603,  604,   -1,  606,  607,
  608,  609,  610,  611,  612,  613,   -1,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   42,   43,   44,   45,   46,   47,   -1,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  271,   -1,  262,   -1,   -1,   -1,  277,   -1,   -1,
   -1,  281,  271,   -1,  284,   -1,   -1,   -1,  277,   -1,
   -1,   -1,  281,   -1,   -1,  284,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,   42,
   43,   44,   45,   46,   47,  325,   -1,   -1,   -1,  318,
   -1,   -1,  262,  333,   -1,   -1,  325,   60,   61,   62,
   -1,  271,   -1,   -1,  333,   -1,   -1,  277,   -1,   -1,
  350,  281,  352,   -1,  284,   -1,   -1,   -1,   -1,   -1,
   -1,  350,  362,  352,  364,  365,  366,   -1,   91,   -1,
   -1,  371,   -1,  362,   -1,  364,  365,  366,   -1,   -1,
   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,  318,   -1,
   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  333,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  350,   -1,  352,  412,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  362,   -1,  364,  365,  366,   -1,   -1,   -1,
   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,  454,  455,  456,  457,  458,
  459,  460,  461,   42,   43,   -1,   45,   46,   47,   -1,
   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,   61,   62,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   91,   -1,  454,  455,  456,  457,  458,  459,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  271,   -1,
   -1,   -1,   -1,   -1,  277,   -1,   -1,   -1,  281,   -1,
   -1,  284,   -1,  563,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  563,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  589,
   -1,   -1,   -1,   -1,   -1,  318,   -1,   -1,  598,   -1,
  589,   -1,  325,   -1,   -1,   -1,   -1,   -1,   -1,  598,
  333,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  350,   -1,  352,
   -1,   -1,   -1,  563,   -1,   -1,   -1,   -1,   -1,  362,
   -1,  364,  365,  366,   -1,   -1,   -1,   -1,  371,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  589,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  598,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,
   -1,   -1,   -1,  262,   -1,  264,   -1,   -1,   -1,   -1,
   -1,   -1,  271,   -1,   -1,   -1,   -1,   -1,  277,   -1,
   -1,   -1,  281,   -1,   -1,  284,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  454,  455,  456,  457,  458,  459,  460,  461,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,
   -1,   -1,   -1,   -1,   -1,   -1,  325,   -1,   -1,   -1,
   -1,   -1,   -1,  332,  333,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  350,   -1,  352,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  362,   -1,  364,  365,  366,   -1,  368,
   -1,   -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  380,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  563,   -1,   -1,  412,   -1,   -1,   -1,   -1,   -1,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,  589,   -1,  437,   -1,
   -1,  279,  280,   -1,  282,  598,  284,   -1,   -1,  287,
   -1,  450,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,  598,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,  446,  447,
  448,  449,  450,  451,  452,  453,  454,  455,  456,  457,
  458,  459,  460,  461,  462,  463,  464,  465,  466,   -1,
   -1,  469,  470,  471,  472,  473,  474,  475,  476,  477,
  478,  479,  480,  481,  482,  483,  484,  485,  486,  487,
  488,  489,  490,  491,  492,  493,  494,  495,  496,  497,
  498,  499,  500,  501,  502,  503,  504,  505,  506,  507,
  508,  509,  510,  511,  512,  513,  514,  515,  516,  517,
  518,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,  540,  541,  542,  543,  544,  545,  546,  547,
  548,  549,  550,  551,  552,  553,  554,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,  573,  574,  575,  576,  577,
  578,  579,  580,  581,  582,  583,  584,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
  598,  599,  600,  601,  602,  603,  604,   -1,  606,  607,
  608,  609,  610,  611,  612,  613,  614,  615,  616,  617,
  618,  619,  620,  621,  622,  623,  624,  625,  626,  627,
  628,  629,  630,  631,  632,  633,  634,  635,  636,  637,
  638,  639,  640,  641,  642,  643,  644,  645,  646,  647,
  648,  649,  650,  651,  652,  653,  654,  655,  656,  657,
  658,  659,  660,  661,  662,  663,  664,  665,  666,  667,
  668,  669,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,
  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,
   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,
   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,
   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,
   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,
  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,
   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,
   -1,   -1,   -1,  438,  439,   -1,  441,   -1,   -1,   -1,
  445,  446,  447,  448,  449,  450,   -1,  452,  453,  454,
  455,  456,  457,  458,  459,  460,  461,  462,  463,  464,
  465,  466,   -1,   -1,  469,  470,  471,  472,  473,  474,
  475,  476,  477,  478,  479,  480,  481,  482,  483,  484,
  485,  486,  487,  488,  489,  490,  491,  492,  493,  494,
  495,  496,  497,  498,  499,  500,  501,  502,  503,  504,
  505,  506,  507,  508,  509,  510,  511,  512,  513,  514,
  515,  516,  517,  518,  519,  520,  521,  522,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,  540,  541,  542,  543,  544,
  545,  546,  547,  548,  549,  550,  551,  552,  553,  554,
  555,  556,  557,  558,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,  573,  574,
  575,  576,  577,  578,  579,  580,  581,  582,  583,  584,
  585,  586,  587,  588,  589,  590,  591,  592,  593,  594,
  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,
   -1,  606,  607,  608,  609,  610,  611,  612,  613,  614,
  615,  616,  617,  618,  619,  620,  621,  622,  623,  624,
  625,  626,  627,  628,  629,  630,  631,  632,  633,  634,
  635,  636,  637,  638,  639,  640,  641,  642,  643,  644,
  645,  646,  647,  648,  649,  650,  651,  652,  653,  654,
  655,  656,  657,  658,  659,  660,  661,  662,  663,  664,
  665,  666,  667,  668,  669,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,
  272,   -1,   -1,  275,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,   -1,   -1,  287,   -1,  289,  290,  291,
   -1,  293,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,  331,
   -1,   -1,  334,   -1,  336,  337,  338,  339,  340,  341,
  342,  343,   -1,  345,   -1,   -1,  348,   -1,   -1,  351,
   -1,  353,   -1,   -1,   -1,  357,  358,  359,   -1,  361,
   -1,  363,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,  390,   -1,
  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,
  402,  403,  404,  405,  406,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,   -1,   -1,  428,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,
  452,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,
  462,  463,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,
  472,   -1,  474,  475,  476,   -1,   -1,  479,   -1,  481,
  482,   -1,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,  501,
  502,  503,   -1,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,   -1,  599,  600,  601,
  602,  603,  604,   -1,  606,  607,  608,  609,  610,  611,
  612,  613,   -1,  615,  616,  617,  618,  619,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,  638,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,  257,   -1,
  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,
  269,   -1,   -1,  272,   -1,   -1,  275,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,  291,   -1,  293,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,
  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,
   -1,   -1,  331,   -1,   -1,  334,   -1,  336,  337,  338,
  339,  340,  341,  342,  343,   -1,  345,   -1,   -1,  348,
   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,
  359,   -1,  361,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,
   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,  390,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,
  399,   -1,   -1,  402,  403,  404,  405,  406,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,
   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,  428,
   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,
  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,
  449,  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  460,  461,  462,  463,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,
  479,   -1,  481,  482,   -1,  484,  485,   -1,  487,   -1,
   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,
   -1,   -1,  501,  502,  503,   -1,  505,  506,  507,   -1,
  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  519,  520,  521,  522,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,   -1,  541,  542,  543,  544,  545,  546,   -1,  548,
  549,  550,  551,   -1,  553,   -1,  555,  556,  557,  558,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,   -1,  574,  575,  576,  577,   -1,
  579,  580,  581,  582,  583,   -1,  585,  586,  587,  588,
  589,  590,  591,  592,  593,  594,  595,  596,  597,   -1,
  599,  600,  601,  602,  603,  604,   -1,  606,  607,  608,
  609,  610,  611,  612,  613,   -1,  615,  616,  617,  618,
   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,  628,
  629,  630,  631,  632,  633,  634,  635,  636,   -1,  638,
   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,
  649,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,   -1,   -1,  272,   -1,   -1,  275,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,  291,   -1,  293,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,  331,   -1,   -1,  334,   -1,
  336,  337,  338,  339,  340,  341,  342,  343,   -1,  345,
   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,
   -1,  357,  358,  359,   -1,  361,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,  390,   -1,  392,  393,   -1,   -1,
   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,
  406,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,
   -1,   -1,  428,   -1,   -1,   -1,   -1,  433,   -1,   -1,
   -1,   -1,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  460,  461,  462,  463,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,
  476,   -1,   -1,  479,   -1,  481,  482,   -1,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,
  496,   -1,   -1,   -1,   -1,  501,  502,  503,   -1,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,   -1,  599,  600,  601,  602,  603,  604,   -1,
  606,  607,  608,  609,  610,  611,  612,  613,   -1,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,  638,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,  272,
   -1,   -1,  275,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,  291,   -1,
  293,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,  331,   -1,
   -1,  334,   -1,  336,  337,  338,  339,  340,  341,  342,
  343,   -1,  345,   -1,   -1,  348,   -1,   -1,  351,   -1,
  353,   -1,   -1,   -1,  357,  358,  359,   -1,  361,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,  390,   -1,  392,
  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,
  403,  404,  405,  406,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,   -1,   -1,  428,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,   -1,  441,   -1,
   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,  462,
  463,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,
   -1,  474,  475,  476,   -1,   -1,  479,   -1,  481,  482,
   -1,  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,
   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,   -1,  638,   -1,   -1,   -1,  642,
   -1,  644,  645,  646,  647,  648,  649,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,  272,   -1,   -1,  275,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,  291,   -1,  293,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,  331,   -1,   -1,  334,   -1,  336,  337,  338,  339,
  340,  341,  342,  343,   -1,  345,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,  359,
   -1,  361,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
  390,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,  406,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,  428,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  460,  461,  462,  463,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,  479,
   -1,  481,  482,   -1,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,   -1,
   -1,  501,  502,  503,   -1,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,   -1,  599,
  600,  601,  602,  603,  604,   -1,  606,  607,  608,  609,
  610,  611,  612,  613,   -1,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,  638,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
  257,   -1,  259,  260,  261,   -1,   -1,   -1,  265,   -1,
   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,  286,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,  296,
  297,   -1,  299,  300,  301,   -1,   -1,  304,   -1,  306,
  307,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,  330,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,
  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,  391,  392,  393,   -1,   -1,   -1,
   -1,   -1,  399,  400,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,  425,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,
  447,  448,  449,  450,   -1,  452,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,
   -1,   -1,   -1,   -1,  481,  482,   -1,  484,  485,   -1,
  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,
   -1,   -1,   -1,   -1,  501,  502,  503,   -1,  505,  506,
  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  519,  520,  521,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,   -1,  541,  542,  543,  544,  545,  546,
   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,  556,
  557,  558,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,   -1,  574,  575,  576,
  577,   -1,  579,  580,  581,  582,  583,   -1,  585,  586,
  587,  588,  589,  590,  591,  592,  593,  594,  595,  596,
  597,   -1,  599,  600,  601,  602,  603,  604,   -1,  606,
  607,  608,  609,  610,  611,  612,  613,   -1,  615,  616,
  617,  618,   -1,  620,  621,  622,  623,  624,  625,   -1,
   -1,  628,  629,  630,  631,  632,  633,  634,  635,  636,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  644,  645,  646,
  647,  648,  649,  257,   -1,  259,   -1,  261,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,  286,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,  296,  297,   -1,  299,  300,  301,   -1,   -1,
  304,   -1,  306,  307,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,  330,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,  391,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,  400,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,  425,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,   -1,  438,  439,   -1,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,   -1,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,   -1,   -1,   -1,   -1,  501,  502,  503,
   -1,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,   -1,  599,  600,  601,  602,  603,
  604,   -1,  606,  607,  608,  609,  610,  611,  612,  613,
   -1,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  644,  645,  646,  647,  648,  649,  257,   -1,  259,   -1,
   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,
   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,
   -1,   -1,   -1,   -1,   -1,  316,   -1,   -1,  319,  320,
   -1,  322,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,
   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,
  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,
   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,
   -1,   -1,   -1,   -1,   -1,  386,  387,  388,  389,   -1,
   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,
  421,  422,   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,
   -1,  432,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,
  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,
   -1,  452,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  460,
  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,
  481,  482,   -1,  484,  485,   -1,  487,   -1,   -1,  490,
  491,   -1,   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,
  501,  502,  503,   -1,  505,  506,  507,   -1,  509,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,
  521,  522,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,   -1,
  541,  542,  543,  544,  545,  546,   -1,  548,  549,  550,
  551,   -1,  553,   -1,  555,  556,  557,  558,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,   -1,  574,  575,  576,  577,   -1,  579,  580,
  581,  582,  583,   -1,  585,  586,  587,  588,  589,  590,
  591,  592,  593,  594,  595,  596,  597,   -1,  599,  600,
  601,  602,  603,  604,   -1,  606,  607,  608,  609,  610,
  611,  612,  613,   -1,  615,  616,  617,  618,   -1,  620,
  621,  622,  623,  624,  625,   -1,   -1,  628,  629,  630,
  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,
   -1,  642,   -1,  644,  645,  646,  647,  648,  649,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,  329,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,  360,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,  372,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,   -1,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,   -1,   -1,  501,  502,  503,   -1,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
   -1,  599,  600,  601,  602,  603,  604,   -1,  606,  607,
  608,  609,  610,  611,  612,  613,   -1,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,  257,   -1,  259,   -1,   -1,   -1,  263,   -1,
  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,
   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  319,  320,   -1,  322,   -1,  324,
   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,
   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,
   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,
  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,
   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,
   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,
   -1,   -1,   -1,  438,  439,   -1,  441,   -1,   -1,   -1,
  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,
  475,  476,   -1,   -1,   -1,   -1,  481,  482,   -1,  484,
  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,
  495,  496,   -1,   -1,   -1,   -1,  501,  502,  503,   -1,
  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,   -1,  541,  542,  543,  544,
  545,  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,
  555,  556,  557,  558,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,   -1,  574,
  575,  576,  577,   -1,  579,  580,  581,  582,  583,   -1,
  585,  586,  587,  588,  589,  590,  591,  592,  593,  594,
  595,  596,  597,   -1,  599,  600,  601,  602,  603,  604,
   -1,  606,  607,  608,  609,  610,  611,  612,  613,   -1,
  615,  616,  617,  618,   -1,  620,  621,  622,  623,  624,
  625,   -1,   -1,  628,  629,  630,  631,  632,  633,  634,
  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,
  645,  646,  647,  648,  649,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
  292,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,
   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,
  452,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,
  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,   -1,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,  501,
  502,  503,   -1,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,   -1,  599,  600,  601,
  602,  603,  604,   -1,  606,  607,  608,  609,  610,  611,
  612,  613,   -1,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,  643,  644,  645,  646,  647,  648,  649,  257,   -1,
  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,
  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,
  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,
   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,
   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,
   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,
  439,   -1,  441,   -1,  443,  444,  445,   -1,  447,  448,
  449,  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,
   -1,   -1,  481,  482,   -1,  484,  485,   -1,  487,   -1,
   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,
   -1,   -1,  501,  502,  503,   -1,  505,  506,  507,   -1,
  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  519,  520,  521,  522,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,   -1,  541,  542,  543,  544,  545,  546,   -1,  548,
  549,  550,  551,   -1,  553,   -1,  555,  556,  557,  558,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,   -1,  574,  575,  576,  577,   -1,
  579,  580,  581,  582,  583,   -1,  585,  586,  587,  588,
  589,  590,  591,  592,  593,  594,  595,  596,  597,   -1,
  599,  600,  601,  602,  603,  604,   -1,  606,  607,  608,
  609,  610,  611,  612,  613,   -1,  615,  616,  617,  618,
   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,  628,
  629,  630,  631,  632,  633,  634,  635,  636,   -1,   -1,
   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,
  649,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,  335,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,  355,
   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,
   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,
   -1,   -1,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,   -1,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,
  496,   -1,   -1,   -1,   -1,  501,  502,  503,   -1,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,   -1,  599,  600,  601,  602,  603,  604,   -1,
  606,  607,  608,  609,  610,  611,  612,  613,   -1,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,  322,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,
  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,   -1,  441,   -1,
   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,
   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,
   -1,  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,
   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,
   -1,  644,  645,  646,  647,  648,  649,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,  371,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,   -1,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,   -1,
   -1,  501,  502,  503,   -1,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,   -1,  599,
  600,  601,  602,  603,  604,   -1,  606,  607,  608,  609,
  610,  611,  612,  613,   -1,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,
  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,
  447,  448,  449,  450,   -1,  452,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,
   -1,   -1,   -1,   -1,  481,  482,   -1,  484,  485,   -1,
  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,
   -1,   -1,   -1,   -1,  501,  502,  503,   -1,  505,  506,
  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  519,  520,  521,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,   -1,  541,  542,  543,  544,  545,  546,
   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,  556,
  557,  558,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,  573,  574,  575,  576,
  577,   -1,  579,  580,  581,  582,  583,   -1,  585,  586,
  587,  588,  589,  590,  591,  592,  593,  594,  595,  596,
  597,   -1,  599,  600,  601,  602,  603,  604,   -1,  606,
  607,  608,  609,  610,  611,  612,  613,   -1,  615,  616,
  617,  618,   -1,  620,  621,  622,  623,  624,  625,   -1,
   -1,  628,  629,  630,  631,  632,  633,  634,  635,  636,
   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,
  647,  648,  649,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,  283,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,
  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,   -1,  438,  439,   -1,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,   -1,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,   -1,   -1,   -1,   -1,  501,  502,  503,
   -1,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,   -1,  599,  600,  601,  602,  603,
  604,   -1,  606,  607,  608,  609,  610,  611,  612,  613,
   -1,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,  257,   -1,  259,   -1,
   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,
   -1,  282,  283,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,
   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,
   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,
  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,
   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,
   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,
   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,
  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,
   -1,  452,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  460,
  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,
  481,  482,   -1,  484,  485,   -1,  487,   -1,   -1,  490,
  491,   -1,   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,
  501,  502,  503,   -1,  505,  506,  507,   -1,  509,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,
  521,  522,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,   -1,
  541,  542,  543,  544,  545,  546,   -1,  548,  549,  550,
  551,   -1,  553,   -1,  555,  556,  557,  558,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,   -1,  574,  575,  576,  577,   -1,  579,  580,
  581,  582,  583,   -1,  585,  586,  587,  588,  589,  590,
  591,  592,  593,  594,  595,  596,  597,   -1,  599,  600,
  601,  602,  603,  604,   -1,  606,  607,  608,  609,  610,
  611,  612,  613,   -1,  615,  616,  617,  618,   -1,  620,
  621,  622,  623,  624,  625,   -1,   -1,  628,  629,  630,
  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,
   -1,  642,   -1,  644,  645,  646,  647,  648,  649,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,  310,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,   -1,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,   -1,   -1,  501,  502,  503,   -1,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
   -1,  599,  600,  601,  602,  603,  604,   -1,  606,  607,
  608,  609,  610,  611,  612,  613,   -1,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,
  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,
   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,
   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,
   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,
   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,
  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,
   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,
   -1,  426,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,
   -1,   -1,   -1,  438,  439,   -1,  441,   -1,   -1,   -1,
  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,
  475,  476,   -1,   -1,   -1,   -1,  481,  482,   -1,  484,
  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,
  495,  496,   -1,   -1,   -1,   -1,  501,  502,  503,   -1,
  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,   -1,  541,  542,  543,  544,
  545,  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,
  555,  556,  557,  558,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,   -1,  574,
  575,  576,  577,   -1,  579,  580,  581,  582,  583,   -1,
  585,  586,  587,  588,  589,  590,  591,  592,  593,  594,
  595,  596,  597,   -1,  599,  600,  601,  602,  603,  604,
   -1,  606,  607,  608,  609,  610,  611,  612,  613,   -1,
  615,  616,  617,  618,   -1,  620,  621,  622,  623,  624,
  625,   -1,   -1,  628,  629,  630,  631,  632,  633,  634,
  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,
  645,  646,  647,  648,  649,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,
   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,  451,
  452,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,
  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,   -1,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,  501,
  502,  503,   -1,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,   -1,  599,  600,  601,
  602,  603,  604,   -1,  606,  607,  608,  609,  610,  611,
  612,  613,   -1,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,  257,   -1,
  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,
  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,
  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,
   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,
   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,
   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,
  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,
  449,  450,  451,  452,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,
   -1,   -1,  481,  482,   -1,  484,  485,   -1,  487,   -1,
   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,
   -1,   -1,  501,  502,  503,   -1,  505,  506,  507,   -1,
  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  519,  520,  521,  522,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,   -1,  541,  542,  543,  544,  545,  546,   -1,  548,
  549,  550,  551,   -1,  553,   -1,  555,  556,  557,  558,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,   -1,  574,  575,  576,  577,   -1,
  579,  580,  581,  582,  583,   -1,  585,  586,  587,  588,
  589,  590,  591,  592,  593,  594,  595,  596,  597,   -1,
  599,  600,  601,  602,  603,  604,   -1,  606,  607,  608,
  609,  610,  611,  612,  613,   -1,  615,  616,  617,  618,
   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,  628,
  629,  630,  631,  632,  633,  634,  635,  636,   -1,   -1,
   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,
  649,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,  301,   -1,   -1,  304,   -1,
  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,
   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,
   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,
   -1,   -1,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,   -1,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,
  496,   -1,   -1,   -1,   -1,  501,  502,  503,   -1,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,   -1,  599,  600,  601,  602,  603,  604,   -1,
  606,  607,  608,  609,  610,  611,  612,  613,   -1,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,
  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,   -1,  441,   -1,
  443,   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,
   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,
   -1,  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,
   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,
   -1,  644,  645,  646,  647,  648,  649,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,   -1,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,   -1,
   -1,  501,  502,  503,   -1,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,   -1,  599,
  600,  601,  602,  603,  604,   -1,  606,  607,  608,  609,
  610,  611,  612,  613,   -1,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,
  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,
  447,  448,  449,  450,   -1,  452,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,
   -1,   -1,   -1,   -1,  481,  482,   -1,  484,  485,   -1,
  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,
   -1,   -1,   -1,   -1,  501,  502,  503,   -1,  505,  506,
  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  519,  520,  521,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,   -1,  541,  542,  543,  544,  545,  546,
   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,  556,
  557,  558,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,   -1,  574,  575,  576,
  577,   -1,  579,  580,  581,  582,  583,   -1,  585,  586,
  587,  588,  589,  590,  591,  592,  593,  594,  595,  596,
  597,   -1,  599,  600,  601,  602,  603,  604,   -1,  606,
  607,  608,  609,  610,  611,  612,  613,   -1,  615,  616,
  617,  618,   -1,  620,  621,  622,  623,  624,  625,   -1,
   -1,  628,  629,  630,  631,  632,  633,  634,  635,  636,
   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,
  647,  648,  649,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,
  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,   -1,  438,  439,   -1,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,
  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,   -1,
  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,   -1,   -1,   -1,   -1,  501,  502,  503,
   -1,  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,   -1,  599,  600,  601,  602,  603,
  604,   -1,  606,  607,  608,  609,  610,  611,  612,  613,
   -1,  615,  616,  617,  618,   -1,  620,  621,  622,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,  257,   -1,  259,   -1,
   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,
   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,
   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,
   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,
   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,
   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,
  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,
   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,
   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,
   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,
   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,
  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,
   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,
  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,
   -1,  452,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  460,
  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,
  481,  482,   -1,  484,  485,   -1,  487,   -1,   -1,  490,
  491,   -1,   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,
  501,  502,  503,   -1,  505,  506,  507,   -1,  509,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,
  521,  522,  523,  524,  525,  526,  527,  528,  529,  530,
  531,  532,  533,  534,  535,  536,  537,  538,  539,   -1,
  541,  542,  543,  544,  545,  546,   -1,  548,  549,  550,
  551,   -1,  553,   -1,  555,  556,  557,  558,  559,  560,
  561,  562,  563,  564,  565,  566,  567,  568,  569,  570,
  571,  572,   -1,  574,  575,  576,  577,   -1,  579,  580,
  581,  582,  583,   -1,  585,  586,  587,  588,  589,  590,
  591,  592,  593,  594,  595,  596,  597,   -1,  599,  600,
  601,  602,  603,  604,   -1,  606,  607,  608,  609,  610,
  611,  612,  613,   -1,  615,  616,  617,  618,   -1,  620,
  621,  622,  623,  624,  625,   -1,   -1,  628,  629,  630,
  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,
   -1,  642,   -1,  644,  645,  646,  647,  648,  649,  257,
   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,
  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,
   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,
   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,
   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,
  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,
   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,
  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,
  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,
  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,
   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,
   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,
   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,
  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,
  448,  449,  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,
   -1,   -1,   -1,  481,  482,   -1,  484,  485,   -1,  487,
   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,
   -1,   -1,   -1,  501,  502,  503,   -1,  505,  506,  507,
   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  519,  520,  521,  522,  523,  524,  525,  526,  527,
  528,  529,  530,  531,  532,  533,  534,  535,  536,  537,
  538,  539,   -1,  541,  542,  543,  544,  545,  546,   -1,
  548,  549,  550,  551,   -1,  553,   -1,  555,  556,  557,
  558,  559,  560,  561,  562,  563,  564,  565,  566,  567,
  568,  569,  570,  571,  572,   -1,  574,  575,  576,  577,
   -1,  579,  580,  581,  582,  583,   -1,  585,  586,  587,
  588,  589,  590,  591,  592,  593,  594,  595,  596,  597,
   -1,  599,  600,  601,  602,  603,  604,   -1,  606,  607,
  608,  609,  610,  611,  612,  613,   -1,  615,  616,  617,
  618,   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,
  628,  629,  630,  631,  632,  633,  634,  635,  636,   -1,
   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,
  648,  649,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,
  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,
   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,
   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,
   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,
   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,
   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,
   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,
  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,
   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,
   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,
  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,
  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,
   -1,   -1,   -1,  438,  439,   -1,  441,   -1,   -1,   -1,
  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,
  475,  476,   -1,   -1,   -1,   -1,  481,  482,   -1,  484,
  485,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,
  495,  496,   -1,   -1,   -1,   -1,  501,  502,  503,   -1,
  505,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,
  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,
  535,  536,  537,  538,  539,   -1,  541,  542,  543,  544,
  545,  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,
  555,  556,  557,  558,  559,  560,  561,  562,  563,  564,
  565,  566,  567,  568,  569,  570,  571,  572,   -1,  574,
  575,  576,  577,   -1,  579,  580,  581,  582,  583,   -1,
  585,  586,  587,  588,  589,  590,  591,  592,  593,  594,
  595,  596,  597,   -1,  599,  600,  601,  602,  603,  604,
   -1,  606,  607,  608,  609,  610,  611,  612,  613,   -1,
  615,  616,  617,  618,   -1,  620,  621,  622,  623,  624,
  625,   -1,   -1,  628,  629,  630,  631,  632,  633,  634,
  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,
  645,  646,  647,  648,  649,  257,   -1,  259,   -1,   -1,
   -1,   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,
  282,   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,
   -1,   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,
   -1,   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,
   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,
   -1,  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
   -1,   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,
   -1,   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,
  392,  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,
  402,  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,
  412,  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,
  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,  438,  439,   -1,  441,
   -1,   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,
  452,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,
  472,   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,
  482,   -1,  484,  485,   -1,  487,   -1,   -1,  490,  491,
   -1,   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,  501,
  502,  503,   -1,  505,  506,  507,   -1,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,
  522,  523,  524,  525,  526,  527,  528,  529,  530,  531,
  532,  533,  534,  535,  536,  537,  538,  539,   -1,  541,
  542,  543,  544,  545,  546,   -1,  548,  549,  550,  551,
   -1,  553,   -1,  555,  556,  557,  558,  559,  560,  561,
  562,  563,  564,  565,  566,  567,  568,  569,  570,  571,
  572,   -1,  574,  575,  576,  577,   -1,  579,  580,  581,
  582,  583,   -1,  585,  586,  587,  588,  589,  590,  591,
  592,  593,  594,  595,  596,  597,   -1,  599,  600,  601,
  602,  603,  604,   -1,  606,  607,  608,  609,  610,  611,
  612,  613,   -1,  615,  616,  617,  618,   -1,  620,  621,
  622,  623,  624,  625,   -1,   -1,  628,  629,  630,  631,
  632,  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,
  642,   -1,  644,  645,  646,  647,  648,  649,  257,   -1,
  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,
  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,
   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
  299,  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,
  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  319,  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,
   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,
   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,
   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,
   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,
  389,   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,
  399,   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,
  409,   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,
   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,
  439,   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,
  449,  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,
   -1,   -1,  481,  482,   -1,  484,  485,   -1,  487,   -1,
   -1,  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,
   -1,   -1,  501,  502,  503,   -1,  505,  506,  507,   -1,
  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  519,  520,  521,  522,  523,  524,  525,  526,  527,  528,
  529,  530,  531,  532,  533,  534,  535,  536,  537,  538,
  539,   -1,  541,  542,  543,  544,  545,  546,   -1,  548,
  549,  550,  551,   -1,  553,   -1,  555,  556,  557,  558,
  559,  560,  561,  562,  563,  564,  565,  566,  567,  568,
  569,  570,  571,  572,   -1,  574,  575,  576,  577,   -1,
  579,  580,  581,  582,  583,   -1,  585,  586,  587,  588,
  589,  590,  591,  592,  593,  594,  595,  596,  597,   -1,
  599,  600,  601,  602,  603,  604,   -1,  606,  607,  608,
  609,  610,  611,  612,  613,   -1,  615,  616,  617,  618,
   -1,  620,  621,  622,  623,  624,  625,   -1,   -1,  628,
  629,  630,  631,  632,  633,  634,  635,  636,   -1,   -1,
   -1,   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,
  649,  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,
   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,
   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,
   -1,  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,
  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,
  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,
  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,
   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,
  376,  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,
  386,   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,
   -1,   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,
   -1,  407,   -1,  409,   -1,  411,  412,  413,  414,  415,
   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,
   -1,   -1,  438,  439,   -1,  441,   -1,   -1,   -1,  445,
   -1,  447,  448,  449,  450,   -1,  452,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,
  476,   -1,   -1,   -1,   -1,  481,  482,   -1,  484,  485,
   -1,  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,
  496,   -1,   -1,   -1,   -1,  501,  502,  503,   -1,  505,
  506,  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  519,  520,  521,  522,  523,  524,  525,
  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,  539,   -1,  541,  542,  543,  544,  545,
  546,   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,
  556,  557,  558,  559,  560,  561,  562,  563,  564,  565,
  566,  567,  568,  569,  570,  571,  572,   -1,  574,  575,
  576,  577,   -1,  579,  580,  581,  582,  583,   -1,  585,
  586,  587,  588,  589,  590,  591,  592,  593,  594,  595,
  596,  597,   -1,  599,  600,  601,  602,  603,  604,   -1,
  606,  607,  608,  609,  610,  611,  612,  613,   -1,  615,
  616,  617,  618,   -1,  620,  621,  622,  623,  624,  625,
   -1,   -1,  628,  629,  630,  631,  632,  633,  634,  635,
  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,
  646,  647,  648,  649,  257,   -1,  259,   -1,   -1,   -1,
   -1,   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,
   -1,  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,
   -1,   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,
   -1,  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,
   -1,  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,
   -1,  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,
  353,   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,
   -1,  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,
   -1,   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,
  393,   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,
  403,  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,
  413,  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  433,   -1,   -1,   -1,   -1,  438,  439,   -1,  441,   -1,
   -1,   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,
   -1,  474,  475,  476,   -1,   -1,   -1,   -1,  481,  482,
   -1,  484,  485,   -1,  487,   -1,   -1,  490,  491,   -1,
   -1,   -1,  495,  496,   -1,   -1,   -1,   -1,  501,  502,
  503,   -1,  505,  506,  507,   -1,  509,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,
  523,  524,  525,  526,  527,  528,  529,  530,  531,  532,
  533,  534,  535,  536,  537,  538,  539,   -1,  541,  542,
  543,  544,  545,  546,   -1,  548,  549,  550,  551,   -1,
  553,   -1,  555,  556,  557,  558,  559,  560,  561,  562,
  563,  564,  565,  566,  567,  568,  569,  570,  571,  572,
   -1,  574,  575,  576,  577,   -1,  579,  580,  581,  582,
  583,   -1,  585,  586,  587,  588,  589,  590,  591,  592,
  593,  594,  595,  596,  597,   -1,  599,  600,  601,  602,
  603,  604,   -1,  606,  607,  608,  609,  610,  611,  612,
  613,   -1,  615,  616,  617,  618,   -1,  620,  621,  622,
  623,  624,  625,   -1,   -1,  628,  629,  630,  631,  632,
  633,  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,
   -1,  644,  645,  646,  647,  648,  649,  257,   -1,  259,
   -1,   -1,   -1,   -1,   -1,  265,   -1,   -1,  268,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,
  280,   -1,  282,   -1,  284,   -1,   -1,  287,   -1,   -1,
  290,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,  299,
  300,   -1,   -1,   -1,  304,   -1,  306,   -1,   -1,  309,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,   -1,   -1,   -1,  324,   -1,  326,   -1,  328,   -1,
   -1,   -1,   -1,   -1,  334,   -1,  336,   -1,   -1,   -1,
  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  348,   -1,
   -1,  351,   -1,  353,   -1,   -1,   -1,  357,  358,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  370,   -1,   -1,   -1,  374,  375,  376,  377,  378,   -1,
  380,   -1,   -1,   -1,   -1,   -1,  386,   -1,  388,  389,
   -1,   -1,  392,  393,   -1,   -1,   -1,   -1,   -1,  399,
   -1,   -1,  402,  403,  404,  405,   -1,  407,   -1,  409,
   -1,  411,  412,  413,  414,  415,   -1,   -1,   -1,   -1,
  420,   -1,  422,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,  438,  439,
   -1,  441,   -1,   -1,   -1,  445,   -1,  447,  448,  449,
  450,   -1,  452,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  460,  461,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  471,  472,   -1,  474,  475,  476,   -1,   -1,   -1,
   -1,  481,  482,   -1,  484,  485,   -1,  487,   -1,   -1,
  490,  491,   -1,   -1,   -1,  495,  496,   -1,   -1,   -1,
   -1,  501,  502,  503,   -1,  505,  506,  507,   -1,  509,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  520,  521,  522,  523,  524,  525,  526,  527,  528,  529,
  530,  531,  532,  533,  534,  535,  536,  537,  538,  539,
   -1,  541,  542,  543,  544,  545,  546,   -1,  548,  549,
  550,  551,   -1,  553,   -1,  555,  556,  557,  558,  559,
  560,  561,  562,  563,  564,  565,  566,  567,  568,  569,
  570,  571,  572,   -1,  574,  575,  576,  577,   -1,  579,
  580,  581,  582,  583,   -1,  585,  586,  587,  588,  589,
  590,  591,  592,  593,  594,  595,  596,  597,   -1,  599,
  600,  601,  602,  603,  604,   -1,  606,  607,  608,  609,
  610,  611,  612,  613,   -1,  615,  616,  617,  618,   -1,
  620,  621,  622,  623,  624,  625,   -1,   -1,  628,  629,
  630,  631,  632,  633,  634,  635,  636,   -1,   -1,   -1,
   -1,   -1,  642,   -1,  644,  645,  646,  647,  648,  649,
  257,   -1,  259,   -1,   -1,   -1,   -1,   -1,  265,   -1,
   -1,  268,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,  282,   -1,  284,   -1,   -1,
  287,   -1,   -1,  290,   -1,   -1,   -1,   -1,   -1,   -1,
  297,   -1,  299,  300,   -1,   -1,   -1,  304,   -1,  306,
   -1,   -1,  309,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  319,  320,   -1,   -1,   -1,  324,   -1,  326,
   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,   -1,  336,
   -1,   -1,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  348,   -1,   -1,  351,   -1,  353,   -1,   -1,   -1,
  357,  358,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  370,   -1,   -1,   -1,  374,  375,  376,
  377,  378,   -1,  380,   -1,   -1,   -1,   -1,   -1,  386,
   -1,  388,  389,   -1,   -1,  392,  393,   -1,   -1,   -1,
   -1,   -1,  399,   -1,   -1,  402,  403,  404,  405,   -1,
  407,   -1,  409,   -1,  411,  412,  413,  414,  415,   -1,
   -1,   -1,   -1,  420,   -1,  422,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,
   -1,  438,  439,   -1,  441,   -1,   -1,   -1,  445,   -1,
  447,  448,  449,  450,   -1,  452,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  460,  461,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  471,  472,   -1,  474,  475,  476,
   -1,   -1,   -1,   -1,  481,  482,   -1,  484,  485,   -1,
  487,   -1,   -1,  490,  491,   -1,   -1,   -1,  495,  496,
   -1,   -1,   -1,   -1,  501,  502,  503,   -1,  505,  506,
  507,   -1,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  519,  520,  521,  522,  523,  524,  525,  526,
  527,  528,  529,  530,  531,  532,  533,  534,  535,  536,
  537,  538,  539,   -1,  541,  542,  543,  544,  545,  546,
   -1,  548,  549,  550,  551,   -1,  553,   -1,  555,  556,
  557,  558,  559,  560,  561,  562,  563,  564,  565,  566,
  567,  568,  569,  570,  571,  572,   -1,  574,  575,  576,
  577,   -1,  579,  580,  581,  582,  583,   -1,  585,  586,
  587,  588,  589,  590,  591,  592,   -1,  594,  595,  596,
  597,   -1,  599,  600,  601,  602,  603,  604,   -1,  606,
  607,  608,  609,  610,  611,  612,  613,   -1,  615,  616,
  617,  618,   -1,  620,  621,  622,  623,  624,  625,   -1,
   -1,  628,  629,  630,  631,  632,  633,  634,  635,  636,
   -1,   -1,   -1,   -1,   -1,  642,   -1,  644,  645,  646,
  647,  648,  649,  257,   -1,  259,   -1,   -1,   -1,   -1,
   -1,  265,   -1,   -1,  268,  269,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,
  284,   -1,   -1,  287,   -1,   -1,  290,   -1,   -1,   -1,
   -1,   -1,   -1,  297,   -1,  299,  300,   -1,   -1,   -1,
  304,   -1,  306,   -1,   -1,  309,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  319,  320,   -1,   -1,   -1,
  324,   -1,  326,   -1,  328,   -1,   -1,   -1,   -1,   -1,
  334,   -1,  336,   -1,   -1,   -1,  340,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  348,   -1,   -1,  351,   -1,  353,
   -1,   -1,   -1,  357,  358,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,   -1,   -1,
  374,  375,  376,  377,  378,   -1,  380,   -1,   -1,   -1,
   -1,   -1,  386,   -1,  388,  389,   -1,   -1,  392,  393,
   -1,   -1,   -1,   -1,   -1,  399,   -1,   -1,  402,  403,
  404,  405,   -1,  407,   -1,  409,   -1,  411,  412,  413,
  414,  415,   -1,   -1,   -1,   -1,  420,   -1,  422,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,
   -1,   -1,   -1,   -1,  438,  439,   -1,  441,   -1,   -1,
   -1,  445,   -1,  447,  448,  449,  450,   -1,  452,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  460,  461,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  471,  472,   -1,
  474,   -1,  476,   -1,   -1,   -1,   -1,  481,  482,   -1,
  484,   -1,   -1,  487,   -1,   -1,  490,  491,   -1,   -1,
   -1,  495,  496,   -1,   -1,   -1,   -1,  501,  502,  503,
   -1,   -1,  506,  507,   -1,  509,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  519,  520,  521,  522,  523,
  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,
  534,  535,  536,  537,  538,  539,   -1,  541,  542,  543,
  544,  545,  546,   -1,  548,  549,  550,  551,   -1,  553,
   -1,  555,  556,  557,  558,  559,  560,  561,  562,  563,
  564,  565,  566,  567,  568,  569,  570,  571,  572,   -1,
  574,  575,  576,  577,   -1,  579,  580,  581,  582,  583,
   -1,  585,  586,  587,  588,  589,  590,  591,  592,  593,
  594,  595,  596,  597,   -1,  599,  600,  601,  602,  603,
  604,   -1,  606,  607,  608,  609,  610,  611,  612,  613,
   -1,  615,  616,  617,  618,   -1,   -1,   -1,   -1,  623,
  624,  625,   -1,   -1,  628,  629,  630,  631,  632,  633,
  634,  635,  636,   -1,   -1,   -1,   -1,   -1,  642,   -1,
  644,  645,  646,  647,  648,  649,
};
static int yyctable[] = {                                     -1,
 1164, 1198,   -1,  639, 1144,   -1,  640, 1144,   -1,   32,
   44,   -1,  954,  119,   -1,  119,  121,   -1,  640, 1146,
   -1, 1767, 1774,   -1, 1321, 1118,   -1, 1568, 1470,   -1,
 1569, 1469,   -1, 1235, 1164,   -1, 1246, 1164,   -1, 2023,
 1381,   -1,  593, 1191,   -1, 1164, 1191,   -1,  210,  639,
   -1,  215,  639,   -1,  224,  639,   -1,  318,  639,   -1,
  319,  639,   -1,  320,  639,   -1, 2364, 1383,   -1, 2366,
 1385,   -1,   82,   89,   -1,  593, 1164,   -1, 2459,  957,
   -1,  810, 1753,   -1, 2641, 1387,   -1,  228, 1175,   -1,
  593, 1193,   -1, 1164, 1193,   -1, 2700, 1074,   -1, 2358,
 1138,   -1, 2505,  955,   -1,  295, 1781,   -1,  295, 1956,
   -1, 3072, 1085,   -1, 3193,  568,   -1,
};
#define YYFINAL 20
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 671
#if YYDEBUG
static const char *yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,"'('","')'","'*'","'+'","','","'-'","'.'","'/'",0,0,0,0,0,0,0,0,0,0,
"':'","';'","'<'","'='","'>'","'?'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,"'['",0,"']'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"ACTIVE","ADD","AFTER","ALL","ALTER","AND","ANY",
"AS","ASC","AT","AVG","AUTO","BEFORE","BEGIN","BETWEEN","BLOB","BY","CAST",
"CHARACTER","CHECK","COLLATE","COMMIT","COMMITTED","COMPUTED","CONCATENATE",
"CONDITIONAL","CONSTRAINT","CONTAINING","COUNT","CREATE","CSTRING","CURRENT",
"CURSOR","DATABASE","DATE","DB_KEY","DECIMAL","DECLARE","DEFAULT","KW_DELETE",
"DESC","DISTINCT","DO","KW_DOMAIN","DROP","ELSE","END","ENTRY_POINT","ESCAPE",
"EXCEPTION","EXECUTE","EXISTS","EXIT","EXTERNAL","FILTER","FOR","FOREIGN",
"FROM","FULL","FUNCTION","GDSCODE","GEQ","GENERATOR","GEN_ID","GRANT","GROUP",
"HAVING","IF","KW_IN","INACTIVE","INNER","INPUT_TYPE","INDEX","INSERT",
"INTEGER","INTO","IS","ISOLATION","JOIN","KEY","KW_CHAR","KW_DEC","KW_DOUBLE",
"KW_FILE","KW_FLOAT","KW_INT","KW_LONG","KW_NULL","KW_NUMERIC","KW_UPPER",
"KW_VALUE","LENGTH","LEFT","LEQ","LEVEL","LIKE","MANUAL","MAXIMUM","MERGE",
"MINIMUM","MODULE_NAME","NAMES","NATIONAL","NATURAL","NCHAR","NEQ","NO","NOT",
"NOT_GTR","NOT_LSS","OF","ON","ONLY","OPTION","OR","ORDER","OUTER",
"OUTPUT_TYPE","KW_OVERFLOW","PAGE","PAGES","KW_PAGE_SIZE","PARAMETER",
"PASSWORD","PLAN","POSITION","POST_EVENT","PRECISION","PRIMARY","PRIVILEGES",
"PROCEDURE","PROTECTED","READ","REAL","REFERENCES","RESERVING","RETAIN",
"RETURNING_VALUES","RETURNS","REVOKE","RIGHT","ROLLBACK","SEGMENT","SELECT",
"SET","SHADOW","KW_SHARED","SINGULAR","KW_SIZE","SMALLINT","SNAPSHOT","SOME",
"SORT","SQLCODE","STABILITY","STARTING","STATISTICS","SUB_TYPE","SUSPEND","SUM",
"TABLE","THEN","TO","TRANSACTION","TRIGGER","UNCOMMITTED","UNION","UNIQUE",
"UPDATE","USER","VALUES","VARCHAR","VARIABLE","VARYING","VERSION","VIEW","WAIT",
"WHEN","WHERE","WHILE","WITH","WORK","WRITE","FLOAT_NUMBER","SYMBOL","NUMBER",
"STRING","INTRODUCER","ACTION","ADMIN","CASCADE","FREE_IT","RESTRICT","ROLE",
"COLUMN","KW_TYPE","EXTRACT","YEAR","MONTH","DAY","HOUR","MINUTE","SECOND",
"WEEKDAY","YEARDAY","TIME","TIMESTAMP","CURRENT_DATE","CURRENT_TIME",
"CURRENT_TIMESTAMP","NUMBER64BIT","SCALEDINT","CURRENT_USER","CURRENT_ROLE",
"KW_BREAK","SUBSTRING","RECREATE","KW_DESCRIPTOR","FIRST","SKIP",
"CURRENT_CONNECTION","CURRENT_TRANSACTION","BIGINT","CASE","NULLIF","COALESCE",
"USING","NULLS","LAST","ROW_COUNT","LOCK","SAVEPOINT","RELEASE","STATEMENT",
"LEAVE","INSERTING","UPDATING","DELETING","BACKUP","KW_DIFFERENCE","OPEN",
"CLOSE","FETCH","ROWS","BLOCK","IIF","SCALAR_ARRAY","CROSS","NEXT","SEQUENCE",
"RESTART","BOTH","COLLATION","COMMENT","BIT_LENGTH","CHAR_LENGTH",
"CHARACTER_LENGTH","LEADING","KW_LOWER","OCTET_LENGTH","TRAILING","TRIM",
"RETURNING","KW_IGNORE","LIMBO","UNDO","REQUESTS","TIMEOUT","ABS","ACCENT",
"ACOS","ALWAYS","ASCII_CHAR","ASCII_VAL","ASIN","ATAN","ATAN2","BIN_AND",
"BIN_OR","BIN_SHL","BIN_SHR","BIN_XOR","CEIL","CONNECT","COS","COSH","COT",
"DATEADD","DATEDIFF","DECODE","DISCONNECT","EXP","FLOOR","GEN_UUID","GENERATED",
"GLOBAL","HASH","INSENSITIVE","LIST","LN","LOG","LOG10","LPAD","MATCHED",
"MATCHING","MAXVALUE","MILLISECOND","MINVALUE","MOD","OVERLAY","PAD","PI",
"PLACING","POWER","PRESERVE","RAND","RECURSIVE","REPLACE","REVERSE","ROUND",
"RPAD","SENSITIVE","SIGN","SIN","SINH","SPACE","SQRT","START","TAN","TANH",
"TEMPORARY","TRUNC","WEEK","AUTONOMOUS","CHAR_TO_UUID","FIRSTNAME","GRANTED",
"LASTNAME","MIDDLENAME","MAPPING","OS_NAME","SIMILAR","UUID_TO_CHAR","CALLER",
"COMMON","DATA","SOURCE","TWO_PHASE","BIND_PARAM","BIN_NOT","BODY","CONTINUE",
"DDL","DECRYPT","ENCRYPT","ENGINE","NAME","OVER","PACKAGE","PARTITION",
"RDB_GET_CONTEXT","RDB_SET_CONTEXT","SCROLL","PRIOR","KW_ABSOLUTE",
"KW_RELATIVE","ACOSH","ASINH","ATANH","RETURN","DETERMINISTIC","IDENTITY",
"DENSE_RANK","FIRST_VALUE","NTH_VALUE","LAST_VALUE","LAG","LEAD","RANK",
"ROW_NUMBER","SQLSTATE","KW_BOOLEAN","KW_FALSE","KW_TRUE","UNKNOWN","USAGE",
"RDB_RECORD_VERSION","LINGER","TAGS","PLUGIN","SERVERWIDE","INCREMENT",
"TRUSTED","ROW","OFFSET","STDDEV_SAMP","STDDEV_POP","VAR_SAMP","VAR_POP",
"COVAR_SAMP","COVAR_POP","CORR","REGR_AVGX","REGR_AVGY","REGR_COUNT",
"REGR_INTERCEPT","REGR_R2","REGR_SLOPE","REGR_SXX","REGR_SXY","REGR_SYY",
"LOCALTIME","LOCALTIMESTAMP","UMINUS","UPLUS",
};
static const char *yyrule[] = {
"$accept : top",
"top : statement",
"top : statement ';'",
"statement : dml_statement",
"statement : ddl_statement",
"statement : tra_statement",
"dml_statement : delete",
"dml_statement : insert",
"dml_statement : merge",
"dml_statement : exec_procedure",
"dml_statement : exec_block",
"dml_statement : savepoint",
"dml_statement : select",
"dml_statement : update",
"dml_statement : update_or_insert",
"ddl_statement : alter",
"ddl_statement : comment",
"ddl_statement : create",
"ddl_statement : create_or_alter",
"ddl_statement : declare",
"ddl_statement : drop",
"ddl_statement : grant",
"ddl_statement : recreate",
"ddl_statement : revoke",
"ddl_statement : set_statistics",
"tra_statement : set_transaction",
"tra_statement : commit",
"tra_statement : rollback",
"tra_statement : set_role",
"$$1 :",
"$$2 :",
"grant : GRANT $$1 $$2 grant0",
"$$3 :",
"$$4 :",
"grant0 : $$3 privileges ON table_noise symbol_table_name TO $$4 non_role_grantee_list grant_option granted_by",
"grant0 : $$3 execute_privilege ON PROCEDURE symbol_procedure_name TO $$4 non_role_grantee_list grant_option granted_by",
"grant0 : $$3 execute_privilege ON FUNCTION symbol_UDF_name TO $$4 non_role_grantee_list grant_option granted_by",
"grant0 : $$3 execute_privilege ON PACKAGE symbol_package_name TO $$4 non_role_grantee_list grant_option granted_by",
"grant0 : $$3 usage_privilege ON EXCEPTION symbol_exception_name TO $$4 non_role_grantee_list grant_option granted_by",
"grant0 : $$3 usage_privilege ON GENERATOR symbol_generator_name TO $$4 non_role_grantee_list grant_option granted_by",
"grant0 : $$3 usage_privilege ON SEQUENCE symbol_generator_name TO $$4 non_role_grantee_list grant_option granted_by",
"$$5 :",
"grant0 : $$3 ddl_privileges object TO $$5 non_role_grantee_list grant_option granted_by",
"grant0 : $$3 db_ddl_privileges DATABASE TO $$5 non_role_grantee_list grant_option granted_by",
"$$6 :",
"$$7 :",
"grant0 : $$6 role_name_list TO $$7 role_grantee_list role_admin_option granted_by",
"object : TABLE",
"object : VIEW",
"object : PROCEDURE",
"object : FUNCTION",
"object : PACKAGE",
"object : GENERATOR",
"object : SEQUENCE",
"object : KW_DOMAIN",
"object : EXCEPTION",
"object : ROLE",
"object : CHARACTER SET",
"object : COLLATION",
"object : FILTER",
"table_noise :",
"table_noise : TABLE",
"privileges : ALL",
"privileges : ALL PRIVILEGES",
"$$8 :",
"privileges : $$8 privilege_list",
"privilege_list : $$8 privilege",
"$$9 :",
"privilege_list : privilege_list ',' $$9 privilege",
"execute_privilege : EXECUTE",
"usage_privilege : USAGE",
"privilege : SELECT",
"privilege : INSERT",
"privilege : KW_DELETE",
"privilege : UPDATE column_parens_opt",
"privilege : REFERENCES column_parens_opt",
"ddl_privileges : ALL privileges_opt",
"ddl_privileges : $$8 ddl_privilege_list",
"privileges_opt :",
"privileges_opt : PRIVILEGES",
"ddl_privilege_list : $$8 ddl_privilege",
"ddl_privilege_list : ddl_privilege_list ',' $$9 ddl_privilege",
"ddl_privilege : CREATE",
"ddl_privilege : ALTER ANY",
"ddl_privilege : DROP ANY",
"db_ddl_privileges : ALL privileges_opt",
"db_ddl_privileges : $$8 db_ddl_privilege_list",
"db_ddl_privilege_list : $$8 db_ddl_privilege",
"db_ddl_privilege_list : db_ddl_privilege_list ',' $$9 db_ddl_privilege",
"db_ddl_privilege : CREATE",
"db_ddl_privilege : ALTER",
"db_ddl_privilege : DROP",
"grant_option :",
"grant_option : WITH GRANT OPTION",
"role_admin_option :",
"role_admin_option : WITH ADMIN OPTION",
"granted_by :",
"granted_by : granted_by_text grantor",
"granted_by_text : GRANTED BY",
"granted_by_text : AS",
"grantor : symbol_user_name",
"grantor : USER symbol_user_name",
"$$10 :",
"revoke : REVOKE $$10 $$2 revoke0",
"$$11 :",
"$$12 :",
"revoke0 : rev_grant_option $$11 privileges ON table_noise symbol_table_name FROM $$12 non_role_grantee_list granted_by",
"revoke0 : rev_grant_option $$11 execute_privilege ON PROCEDURE symbol_procedure_name FROM $$12 non_role_grantee_list granted_by",
"revoke0 : rev_grant_option $$11 execute_privilege ON FUNCTION symbol_UDF_name FROM $$12 non_role_grantee_list granted_by",
"revoke0 : rev_grant_option $$11 execute_privilege ON PACKAGE symbol_package_name FROM $$12 non_role_grantee_list granted_by",
"revoke0 : rev_grant_option $$11 usage_privilege ON EXCEPTION symbol_exception_name FROM $$12 non_role_grantee_list granted_by",
"revoke0 : rev_grant_option $$11 usage_privilege ON GENERATOR symbol_generator_name FROM $$12 non_role_grantee_list granted_by",
"revoke0 : rev_grant_option $$11 usage_privilege ON SEQUENCE symbol_generator_name FROM $$12 non_role_grantee_list granted_by",
"$$13 :",
"revoke0 : rev_grant_option $$11 ddl_privileges object FROM $$13 non_role_grantee_list granted_by",
"revoke0 : rev_grant_option $$11 db_ddl_privileges DATABASE FROM $$13 non_role_grantee_list granted_by",
"$$14 :",
"revoke0 : rev_admin_option $$14 role_name_list FROM $$5 role_grantee_list granted_by",
"revoke0 : ALL ON ALL FROM $$5 non_role_grantee_list",
"rev_grant_option :",
"rev_grant_option : GRANT OPTION FOR",
"rev_admin_option :",
"rev_admin_option : ADMIN OPTION FOR",
"$$15 :",
"non_role_grantee_list : $$15 grantee",
"non_role_grantee_list : $$15 user_grantee",
"$$16 :",
"non_role_grantee_list : non_role_grantee_list ',' $$16 grantee",
"non_role_grantee_list : non_role_grantee_list ',' $$16 user_grantee",
"grantee : PROCEDURE symbol_procedure_name",
"grantee : FUNCTION symbol_UDF_name",
"grantee : PACKAGE symbol_package_name",
"grantee : TRIGGER symbol_trigger_name",
"grantee : VIEW symbol_view_name",
"grantee : ROLE symbol_role_name",
"user_grantee : symbol_user_name",
"user_grantee : USER symbol_user_name",
"user_grantee : GROUP symbol_user_name",
"role_name_list : $$15 role_name",
"role_name_list : role_name_list ',' $$16 role_name",
"role_name : symbol_role_name",
"role_grantee_list : $$15 role_grantee",
"role_grantee_list : role_grantee_list ',' $$16 role_grantee",
"role_grantee : grantor",
"declare : DECLARE declare_clause",
"declare_clause : FILTER filter_decl_clause",
"declare_clause : EXTERNAL FUNCTION udf_decl_clause",
"$$17 :",
"$$18 :",
"$$19 :",
"udf_decl_clause : symbol_UDF_name $$17 $$18 arg_desc_list1 RETURNS $$19 return_value1 ENTRY_POINT utf_string MODULE_NAME utf_string",
"udf_data_type : simple_type",
"udf_data_type : BLOB",
"udf_data_type : CSTRING '(' pos_short_integer ')' charset_clause",
"arg_desc_list1 :",
"$$20 :",
"arg_desc_list1 : $$20 arg_desc_list",
"$$21 :",
"arg_desc_list1 : '(' $$21 arg_desc_list ')'",
"arg_desc_list : $$20 arg_desc",
"$$22 :",
"arg_desc_list : arg_desc_list ',' $$22 arg_desc",
"arg_desc : udf_data_type param_mechanism",
"param_mechanism :",
"param_mechanism : BY KW_DESCRIPTOR",
"param_mechanism : BY SCALAR_ARRAY",
"param_mechanism : KW_NULL",
"$$23 :",
"return_value1 : $$23 return_value",
"$$24 :",
"return_value1 : '(' $$24 return_value ')'",
"return_value : udf_data_type return_mechanism",
"return_value : PARAMETER pos_short_integer",
"return_mechanism :",
"return_mechanism : BY KW_VALUE",
"return_mechanism : BY KW_DESCRIPTOR",
"return_mechanism : FREE_IT",
"return_mechanism : BY KW_DESCRIPTOR FREE_IT",
"filter_decl_clause : symbol_filter_name INPUT_TYPE blob_filter_subtype OUTPUT_TYPE blob_filter_subtype ENTRY_POINT utf_string MODULE_NAME utf_string",
"blob_filter_subtype : symbol_blob_subtype_name",
"blob_filter_subtype : signed_short_integer",
"create : CREATE create_clause",
"create_clause : EXCEPTION exception_clause",
"$$25 :",
"$$26 :",
"create_clause : unique_opt order_direction INDEX symbol_index_name ON simple_table_name $$25 $$26 index_definition",
"create_clause : FUNCTION function_clause",
"create_clause : PROCEDURE procedure_clause",
"create_clause : TABLE table_clause",
"create_clause : GLOBAL TEMPORARY TABLE gtt_table_clause",
"create_clause : TRIGGER trigger_clause",
"create_clause : VIEW view_clause",
"create_clause : GENERATOR generator_clause",
"create_clause : SEQUENCE generator_clause",
"create_clause : DATABASE db_clause",
"create_clause : KW_DOMAIN domain_clause",
"create_clause : SHADOW shadow_clause",
"create_clause : ROLE role_clause",
"create_clause : COLLATION collation_clause",
"create_clause : USER create_user_clause",
"create_clause : PACKAGE package_clause",
"create_clause : PACKAGE BODY package_body_clause",
"$$27 :",
"create_clause : MAPPING $$27 create_map_clause",
"$$28 :",
"create_clause : GLOBAL MAPPING $$28 create_map_clause",
"recreate : RECREATE recreate_clause",
"recreate_clause : PROCEDURE procedure_clause",
"recreate_clause : FUNCTION function_clause",
"recreate_clause : TABLE table_clause",
"recreate_clause : GLOBAL TEMPORARY TABLE gtt_table_clause",
"recreate_clause : VIEW view_clause",
"recreate_clause : TRIGGER trigger_clause",
"recreate_clause : PACKAGE package_clause",
"recreate_clause : PACKAGE BODY package_body_clause",
"recreate_clause : EXCEPTION exception_clause",
"recreate_clause : GENERATOR generator_clause",
"recreate_clause : SEQUENCE generator_clause",
"create_or_alter : CREATE OR ALTER replace_clause",
"replace_clause : PROCEDURE replace_procedure_clause",
"replace_clause : FUNCTION replace_function_clause",
"replace_clause : TRIGGER replace_trigger_clause",
"replace_clause : PACKAGE replace_package_clause",
"replace_clause : VIEW replace_view_clause",
"replace_clause : EXCEPTION replace_exception_clause",
"replace_clause : GENERATOR replace_sequence_clause",
"replace_clause : SEQUENCE replace_sequence_clause",
"replace_clause : USER replace_user_clause",
"replace_clause : MAPPING $$27 replace_map_clause",
"replace_clause : GLOBAL MAPPING $$28 replace_map_clause",
"exception_clause : symbol_exception_name sql_string",
"replace_exception_clause : symbol_exception_name sql_string",
"alter_exception_clause : symbol_exception_name sql_string",
"unique_opt :",
"unique_opt : UNIQUE",
"index_definition : column_list",
"index_definition : column_parens",
"index_definition : computed_by '(' value ')'",
"$$29 :",
"$$30 :",
"shadow_clause : pos_short_integer manual_auto conditional utf_string first_file_length $$29 $$30 sec_shadow_files",
"manual_auto :",
"manual_auto : MANUAL",
"manual_auto : AUTO",
"conditional :",
"conditional : CONDITIONAL",
"first_file_length :",
"first_file_length : LENGTH equals long_integer page_noise",
"sec_shadow_files :",
"$$31 :",
"sec_shadow_files : $$31 db_file_list",
"db_file_list : db_file",
"db_file_list : db_file_list db_file",
"$$32 :",
"$$33 :",
"domain_clause : symbol_column_name as_opt data_type domain_default_opt $$32 $$33 domain_constraints_opt collate_clause",
"domain_constraints_opt :",
"domain_constraints_opt : $$33 domain_constraints",
"domain_constraints : $$33 domain_constraint",
"$$34 :",
"domain_constraints : domain_constraints $$34 domain_constraint",
"domain_constraint : null_constraint",
"domain_constraint : check_constraint",
"as_opt :",
"as_opt : AS",
"domain_default : DEFAULT default_value",
"domain_default_opt :",
"domain_default_opt : domain_default",
"null_constraint : NOT KW_NULL",
"check_constraint : CHECK '(' search_condition ')'",
"generator_clause : symbol_generator_name start_with_opt step_option",
"start_with_opt :",
"start_with_opt : start_with",
"start_with : START WITH sequence_value",
"step_option :",
"step_option : INCREMENT by_noise signed_long_integer",
"by_noise :",
"by_noise : BY",
"replace_sequence_clause : symbol_generator_name replace_sequence_options step_option",
"replace_sequence_options : RESTART",
"replace_sequence_options : START WITH sequence_value",
"alter_sequence_clause : symbol_generator_name restart_value_opt2 step_option",
"restart_value_opt : RESTART",
"restart_value_opt : RESTART WITH sequence_value",
"restart_value_opt2 :",
"restart_value_opt2 : restart_value_opt",
"set_generator_clause : SET GENERATOR symbol_generator_name TO sequence_value",
"sequence_value : signed_long_integer",
"sequence_value : NUMBER64BIT",
"sequence_value : '-' NUMBER64BIT",
"role_clause : symbol_role_name",
"$$35 :",
"$$36 :",
"$$37 :",
"$$38 :",
"collation_clause : symbol_collation_name FOR symbol_character_set_name $$35 $$36 collation_sequence_definition $$37 collation_attribute_list_opt $$38 collation_specific_attribute_opt",
"collation_sequence_definition :",
"collation_sequence_definition : FROM symbol_collation_name",
"collation_sequence_definition : FROM EXTERNAL '(' utf_string ')'",
"collation_attribute_list_opt :",
"collation_attribute_list_opt : $$36 collation_attribute_list",
"collation_attribute_list : $$36 collation_attribute",
"$$39 :",
"collation_attribute_list : collation_attribute_list $$39 collation_attribute",
"collation_attribute : $$36 collation_pad_attribute",
"collation_attribute : $$36 collation_case_attribute",
"collation_attribute : $$36 collation_accent_attribute",
"collation_pad_attribute : NO PAD",
"collation_pad_attribute : PAD SPACE",
"collation_case_attribute : CASE SENSITIVE",
"collation_case_attribute : CASE INSENSITIVE",
"collation_accent_attribute : ACCENT SENSITIVE",
"collation_accent_attribute : ACCENT INSENSITIVE",
"collation_specific_attribute_opt :",
"collation_specific_attribute_opt : utf_string",
"alter_charset_clause : symbol_character_set_name SET DEFAULT COLLATION symbol_collation_name",
"$$40 :",
"$$41 :",
"$$42 :",
"db_clause : db_name $$40 $$41 db_initial_desc1 $$42 db_rem_desc1",
"equals :",
"equals : '='",
"db_name : utf_string",
"db_initial_desc1 :",
"db_initial_desc1 : $$41 db_initial_desc",
"db_initial_desc : $$41 db_initial_option",
"$$43 :",
"db_initial_desc : db_initial_desc $$43 db_initial_option",
"db_initial_option : KW_PAGE_SIZE equals pos_short_integer",
"db_initial_option : USER symbol_user_name",
"db_initial_option : USER utf_string",
"db_initial_option : ROLE valid_symbol_name",
"db_initial_option : ROLE utf_string",
"db_initial_option : PASSWORD utf_string",
"db_initial_option : SET NAMES utf_string",
"db_initial_option : LENGTH equals long_integer page_noise",
"db_rem_desc1 :",
"db_rem_desc1 : $$41 db_rem_desc",
"db_rem_desc : $$41 db_rem_option",
"db_rem_desc : db_rem_desc $$43 db_rem_option",
"db_rem_option : db_file",
"db_rem_option : DEFAULT CHARACTER SET symbol_character_set_name",
"db_rem_option : DEFAULT CHARACTER SET symbol_character_set_name COLLATION symbol_collation_name",
"db_rem_option : KW_DIFFERENCE KW_FILE utf_string",
"$$44 :",
"$$45 :",
"db_file : KW_FILE utf_string $$44 $$45 file_desc1",
"file_desc1 :",
"file_desc1 : $$45 file_desc",
"file_desc : $$45 file_clause",
"$$46 :",
"file_desc : file_desc $$46 file_clause",
"file_clause : STARTING file_clause_noise long_integer",
"file_clause : LENGTH equals long_integer page_noise",
"file_clause_noise :",
"file_clause_noise : AT",
"file_clause_noise : AT PAGE",
"page_noise :",
"page_noise : PAGE",
"page_noise : PAGES",
"$$47 :",
"$$48 :",
"table_clause : simple_table_name external_file $$47 '(' $$48 table_elements ')'",
"$$49 :",
"gtt_table_clause : simple_table_name $$49 '(' $$48 table_elements ')' gtt_scope",
"gtt_scope :",
"gtt_scope : ON COMMIT KW_DELETE ROWS",
"gtt_scope : ON COMMIT PRESERVE ROWS",
"external_file :",
"external_file : EXTERNAL KW_FILE utf_string",
"external_file : EXTERNAL utf_string",
"$$50 :",
"table_elements : $$50 table_element",
"$$51 :",
"table_elements : table_elements ',' $$51 table_element",
"$$52 :",
"table_element : $$52 column_def",
"table_element : $$52 table_constraint_definition",
"$$53 :",
"$$54 :",
"column_def : symbol_column_name data_type_or_domain domain_default_opt $$53 $$54 column_constraint_clause collate_clause",
"$$55 :",
"column_def : symbol_column_name data_type_or_domain identity_clause $$55 $$54 column_constraint_clause collate_clause",
"column_def : symbol_column_name non_array_type def_computed",
"column_def : symbol_column_name def_computed",
"identity_clause : GENERATED BY DEFAULT AS IDENTITY identity_clause_options",
"identity_clause_options :",
"identity_clause_options : '(' start_with ')'",
"def_computed : computed_clause '(' value ')'",
"computed_clause : computed_by",
"computed_clause : generated_always_clause",
"generated_always_clause : GENERATED ALWAYS AS",
"computed_by : COMPUTED BY",
"computed_by : COMPUTED",
"data_type_or_domain : data_type",
"data_type_or_domain : symbol_column_name",
"collate_clause :",
"collate_clause : COLLATE symbol_collation_name",
"data_type_descriptor : data_type",
"data_type_descriptor : KW_TYPE OF symbol_column_name",
"data_type_descriptor : KW_TYPE OF COLUMN symbol_column_name '.' symbol_column_name",
"data_type_descriptor : symbol_column_name",
"default_value : constant",
"default_value : current_user",
"default_value : current_role",
"default_value : internal_info",
"default_value : null_value",
"default_value : datetime_value_expression",
"column_constraint_clause :",
"$$56 :",
"column_constraint_clause : $$56 column_constraint_list",
"column_constraint_list : $$56 column_constraint_def",
"$$57 :",
"column_constraint_list : column_constraint_list $$57 column_constraint_def",
"column_constraint_def : constraint_name_opt $$57 column_constraint",
"column_constraint : null_constraint",
"column_constraint : check_constraint",
"column_constraint : REFERENCES symbol_table_name column_parens_opt referential_trigger_action constraint_index_opt",
"column_constraint : UNIQUE constraint_index_opt",
"column_constraint : PRIMARY KEY constraint_index_opt",
"$$58 :",
"table_constraint_definition : constraint_name_opt $$58 table_constraint",
"constraint_name_opt :",
"constraint_name_opt : CONSTRAINT symbol_constraint_name",
"table_constraint : UNIQUE column_parens constraint_index_opt",
"table_constraint : PRIMARY KEY column_parens constraint_index_opt",
"table_constraint : FOREIGN KEY column_parens REFERENCES symbol_table_name column_parens_opt referential_trigger_action constraint_index_opt",
"table_constraint : check_constraint",
"constraint_index_opt :",
"constraint_index_opt : USING order_direction INDEX symbol_index_name",
"referential_trigger_action :",
"referential_trigger_action : update_rule",
"referential_trigger_action : delete_rule",
"referential_trigger_action : delete_rule update_rule",
"referential_trigger_action : update_rule delete_rule",
"update_rule : ON UPDATE referential_action",
"delete_rule : ON KW_DELETE referential_action",
"referential_action : CASCADE",
"referential_action : SET DEFAULT",
"referential_action : SET KW_NULL",
"referential_action : NO ACTION",
"procedure_clause : psql_procedure_clause",
"procedure_clause : external_procedure_clause",
"psql_procedure_clause : procedure_clause_start AS local_declaration_list full_proc_block",
"external_procedure_clause : procedure_clause_start external_clause external_body_clause_opt",
"$$59 :",
"$$60 :",
"$$61 :",
"procedure_clause_start : symbol_procedure_name $$59 $$60 input_parameters $$61 output_parameters",
"alter_procedure_clause : procedure_clause",
"replace_procedure_clause : procedure_clause",
"input_parameters :",
"input_parameters : '(' ')'",
"input_parameters : '(' $$21 input_proc_parameters ')'",
"output_parameters :",
"output_parameters : RETURNS '(' $$22 output_proc_parameters ')'",
"input_proc_parameters : $$20 input_proc_parameter",
"input_proc_parameters : input_proc_parameters ',' $$22 input_proc_parameter",
"input_proc_parameter : column_domain_or_non_array_type collate_clause default_par_opt",
"output_proc_parameters : output_proc_parameter",
"output_proc_parameters : output_proc_parameters ',' $$22 output_proc_parameter",
"output_proc_parameter : column_domain_or_non_array_type collate_clause",
"column_domain_or_non_array_type : symbol_column_name domain_or_non_array_type",
"default_par_opt :",
"default_par_opt : DEFAULT default_value",
"default_par_opt : '=' default_value",
"function_clause : psql_function_clause",
"function_clause : external_function_clause",
"psql_function_clause : function_clause_start AS local_declaration_list full_proc_block",
"external_function_clause : function_clause_start external_clause external_body_clause_opt",
"$$62 :",
"function_clause_start : symbol_UDF_name $$62 $$18 input_parameters RETURNS domain_or_non_array_type collate_clause deterministic_opt",
"deterministic_opt :",
"deterministic_opt : NOT DETERMINISTIC",
"deterministic_opt : DETERMINISTIC",
"external_clause : EXTERNAL NAME utf_string ENGINE valid_symbol_name",
"external_clause : EXTERNAL ENGINE valid_symbol_name",
"external_body_clause_opt :",
"external_body_clause_opt : AS utf_string",
"alter_function_clause : function_clause",
"replace_function_clause : function_clause",
"package_clause : symbol_package_name AS BEGIN package_items_opt END",
"package_items_opt : package_items",
"package_items_opt :",
"package_items : package_item",
"package_items : package_items package_item",
"package_item : FUNCTION function_clause_start ';'",
"package_item : PROCEDURE procedure_clause_start ';'",
"alter_package_clause : package_clause",
"replace_package_clause : package_clause",
"package_body_clause : symbol_package_name AS BEGIN package_items package_body_items_opt END",
"package_body_clause : symbol_package_name AS BEGIN package_body_items_opt END",
"package_body_items_opt :",
"package_body_items_opt : package_body_items",
"package_body_items : package_body_item",
"package_body_items : package_body_items package_body_item",
"package_body_item : FUNCTION psql_function_clause",
"package_body_item : FUNCTION external_function_clause ';'",
"package_body_item : PROCEDURE psql_procedure_clause",
"package_body_item : PROCEDURE external_procedure_clause ';'",
"local_declaration_list :",
"local_declaration_list : local_declarations",
"local_declarations : local_declaration",
"local_declarations : local_declarations local_declaration",
"local_declaration : DECLARE var_decl_opt local_declaration_item ';'",
"$$63 :",
"$$64 :",
"$$65 :",
"local_declaration : DECLARE PROCEDURE symbol_procedure_name $$63 $$64 input_parameters $$65 output_parameters AS local_declaration_list full_proc_block",
"$$66 :",
"local_declaration : DECLARE FUNCTION symbol_UDF_name $$66 $$64 input_parameters RETURNS domain_or_non_array_type collate_clause deterministic_opt AS local_declaration_list full_proc_block",
"local_declaration_item : var_declaration_item",
"local_declaration_item : cursor_declaration_item",
"var_declaration_item : column_domain_or_non_array_type collate_clause default_par_opt",
"var_decl_opt :",
"var_decl_opt : VARIABLE",
"cursor_declaration_item : symbol_cursor_name scroll_opt CURSOR FOR '(' select ')'",
"scroll_opt :",
"scroll_opt : NO SCROLL",
"scroll_opt : SCROLL",
"proc_block : proc_statement",
"proc_block : full_proc_block",
"full_proc_block : BEGIN full_proc_block_body END",
"full_proc_block_body :",
"full_proc_block_body : proc_statements",
"full_proc_block_body : proc_statements excp_hndl_statements",
"proc_statements : proc_block",
"proc_statements : proc_statements proc_block",
"proc_statement : simple_proc_statement ';'",
"proc_statement : complex_proc_statement",
"simple_proc_statement : assignment_statement",
"simple_proc_statement : insert",
"simple_proc_statement : merge",
"simple_proc_statement : update",
"simple_proc_statement : update_or_insert",
"simple_proc_statement : delete",
"simple_proc_statement : singleton_select",
"simple_proc_statement : exec_procedure",
"simple_proc_statement : exec_sql",
"simple_proc_statement : exec_into",
"simple_proc_statement : exec_function",
"simple_proc_statement : excp_statement",
"simple_proc_statement : raise_statement",
"simple_proc_statement : post_event",
"simple_proc_statement : cursor_statement",
"simple_proc_statement : breakleave",
"simple_proc_statement : continue",
"simple_proc_statement : SUSPEND",
"simple_proc_statement : EXIT",
"simple_proc_statement : RETURN value",
"assignment_statement : assignment",
"assignment_statement : ':' assignment",
"complex_proc_statement : in_autonomous_transaction",
"complex_proc_statement : if_then_else",
"complex_proc_statement : while",
"complex_proc_statement : for_select",
"complex_proc_statement : for_exec_into",
"in_autonomous_transaction : KW_IN AUTONOMOUS TRANSACTION DO proc_block",
"excp_statement : EXCEPTION symbol_exception_name",
"excp_statement : EXCEPTION symbol_exception_name value",
"excp_statement : EXCEPTION symbol_exception_name USING '(' value_list ')'",
"raise_statement : EXCEPTION",
"$$67 :",
"$$68 :",
"for_select : label_def_opt FOR select $$67 $$68 for_select_into_cursor DO proc_block",
"for_select_into_cursor : into_variable_list cursor_def_opt",
"for_select_into_cursor : into_variable_list_opt cursor_def",
"into_variable_list_opt :",
"into_variable_list_opt : into_variable_list",
"into_variable_list : INTO variable_list",
"$$69 :",
"$$70 :",
"$$71 :",
"exec_sql : EXECUTE STATEMENT $$69 $$70 exec_stmt_inputs $$71 exec_stmt_options",
"exec_into : exec_sql INTO variable_list",
"for_exec_into : label_def_opt FOR exec_into DO proc_block",
"exec_stmt_inputs : value",
"$$72 :",
"exec_stmt_inputs : '(' value ')' '(' $$72 named_params_list ')'",
"exec_stmt_inputs : '(' value ')' '(' $$72 not_named_params_list ')'",
"named_params_list : $$70 named_param",
"named_params_list : named_params_list ',' $$71 named_param",
"named_param : symbol_variable_name BIND_PARAM value",
"not_named_params_list : $$70 not_named_param",
"not_named_params_list : not_named_params_list ',' $$71 not_named_param",
"not_named_param : value",
"exec_stmt_options :",
"exec_stmt_options : $$70 exec_stmt_options_list",
"$$73 :",
"exec_stmt_options_list : exec_stmt_options_list $$73 exec_stmt_option",
"exec_stmt_options_list : $$70 exec_stmt_option",
"exec_stmt_option : ON EXTERNAL DATA SOURCE value",
"exec_stmt_option : ON EXTERNAL value",
"exec_stmt_option : AS USER value",
"exec_stmt_option : PASSWORD value",
"exec_stmt_option : ROLE value",
"exec_stmt_option : WITH AUTONOMOUS TRANSACTION",
"exec_stmt_option : WITH COMMON TRANSACTION",
"exec_stmt_option : WITH CALLER PRIVILEGES",
"if_then_else : IF '(' search_condition ')' THEN proc_block ELSE proc_block",
"if_then_else : IF '(' search_condition ')' THEN proc_block",
"post_event : POST_EVENT value event_argument_opt",
"event_argument_opt :",
"singleton_select : select INTO variable_list",
"variable : ':' symbol_variable_name",
"variable_list : variable",
"variable_list : column_name",
"variable_list : variable_list ',' column_name",
"variable_list : variable_list ',' variable",
"while : label_def_opt WHILE '(' search_condition ')' DO proc_block",
"label_def_opt :",
"label_def_opt : symbol_label_name ':'",
"breakleave : KW_BREAK",
"breakleave : LEAVE label_use_opt",
"continue : CONTINUE label_use_opt",
"label_use_opt :",
"label_use_opt : symbol_label_name",
"cursor_def_opt :",
"cursor_def_opt : cursor_def",
"cursor_def : AS CURSOR symbol_cursor_name",
"excp_hndl_statements : excp_hndl_statement",
"excp_hndl_statements : excp_hndl_statements excp_hndl_statement",
"$$74 :",
"$$75 :",
"excp_hndl_statement : WHEN $$74 $$75 errors DO proc_block",
"$$76 :",
"errors : $$76 err",
"$$77 :",
"errors : errors ',' $$77 err",
"err : SQLCODE signed_short_integer",
"err : SQLSTATE STRING",
"err : GDSCODE symbol_gdscode_name",
"err : EXCEPTION symbol_exception_name",
"err : ANY",
"cursor_statement : open_cursor",
"cursor_statement : fetch_cursor",
"cursor_statement : close_cursor",
"open_cursor : OPEN symbol_cursor_name",
"close_cursor : CLOSE symbol_cursor_name",
"$$78 :",
"$$79 :",
"fetch_cursor : FETCH $$78 $$79 fetch_scroll FROM symbol_cursor_name into_variable_list_opt",
"fetch_cursor : FETCH symbol_cursor_name into_variable_list_opt",
"fetch_scroll : FIRST",
"fetch_scroll : LAST",
"fetch_scroll : PRIOR",
"fetch_scroll : NEXT",
"fetch_scroll : KW_ABSOLUTE value",
"fetch_scroll : KW_RELATIVE value",
"exec_procedure : EXECUTE PROCEDURE symbol_procedure_name proc_inputs proc_outputs_opt",
"exec_procedure : EXECUTE PROCEDURE symbol_package_name '.' symbol_procedure_name proc_inputs proc_outputs_opt",
"proc_inputs :",
"proc_inputs : value_list",
"proc_inputs : '(' value_list ')'",
"proc_outputs_opt :",
"proc_outputs_opt : RETURNING_VALUES variable_list",
"proc_outputs_opt : RETURNING_VALUES '(' variable_list ')'",
"$$80 :",
"exec_block : EXECUTE BLOCK $$80 $$64 block_input_params $$65 output_parameters AS local_declaration_list full_proc_block",
"block_input_params :",
"block_input_params : '(' $$21 block_parameters ')'",
"block_parameters : $$20 block_parameter",
"block_parameters : block_parameters ',' $$22 block_parameter",
"block_parameter : column_domain_or_non_array_type collate_clause '=' parameter",
"view_clause : simple_table_name column_parens_opt AS select_expr check_opt",
"replace_view_clause : view_clause",
"alter_view_clause : view_clause",
"check_opt :",
"check_opt : WITH CHECK OPTION",
"trigger_clause : symbol_trigger_name trigger_active trigger_type trigger_position AS local_declaration_list full_proc_block",
"trigger_clause : symbol_trigger_name trigger_active trigger_type trigger_position external_clause external_body_clause_opt",
"trigger_clause : symbol_trigger_name trigger_active trigger_type trigger_position ON symbol_table_name AS local_declaration_list full_proc_block",
"trigger_clause : symbol_trigger_name trigger_active trigger_type trigger_position ON symbol_table_name external_clause external_body_clause_opt",
"trigger_clause : symbol_trigger_name FOR symbol_table_name trigger_active trigger_type trigger_position AS local_declaration_list full_proc_block",
"trigger_clause : symbol_trigger_name FOR symbol_table_name trigger_active trigger_type trigger_position external_clause external_body_clause_opt",
"replace_trigger_clause : trigger_clause",
"trigger_active : ACTIVE",
"trigger_active : INACTIVE",
"trigger_active :",
"trigger_type : trigger_type_prefix trigger_type_suffix",
"trigger_type : ON trigger_db_type",
"trigger_type : trigger_type_prefix trigger_ddl_type",
"trigger_db_type : CONNECT",
"trigger_db_type : DISCONNECT",
"trigger_db_type : TRANSACTION START",
"trigger_db_type : TRANSACTION COMMIT",
"trigger_db_type : TRANSACTION ROLLBACK",
"trigger_ddl_type : trigger_ddl_type_items",
"trigger_ddl_type : ANY DDL STATEMENT",
"trigger_ddl_type_items : CREATE TABLE",
"trigger_ddl_type_items : ALTER TABLE",
"trigger_ddl_type_items : DROP TABLE",
"trigger_ddl_type_items : CREATE PROCEDURE",
"trigger_ddl_type_items : ALTER PROCEDURE",
"trigger_ddl_type_items : DROP PROCEDURE",
"trigger_ddl_type_items : CREATE FUNCTION",
"trigger_ddl_type_items : ALTER FUNCTION",
"trigger_ddl_type_items : DROP FUNCTION",
"trigger_ddl_type_items : CREATE TRIGGER",
"trigger_ddl_type_items : ALTER TRIGGER",
"trigger_ddl_type_items : DROP TRIGGER",
"trigger_ddl_type_items : CREATE EXCEPTION",
"trigger_ddl_type_items : ALTER EXCEPTION",
"trigger_ddl_type_items : DROP EXCEPTION",
"trigger_ddl_type_items : CREATE VIEW",
"trigger_ddl_type_items : ALTER VIEW",
"trigger_ddl_type_items : DROP VIEW",
"trigger_ddl_type_items : CREATE KW_DOMAIN",
"trigger_ddl_type_items : ALTER KW_DOMAIN",
"trigger_ddl_type_items : DROP KW_DOMAIN",
"trigger_ddl_type_items : CREATE ROLE",
"trigger_ddl_type_items : ALTER ROLE",
"trigger_ddl_type_items : DROP ROLE",
"trigger_ddl_type_items : CREATE SEQUENCE",
"trigger_ddl_type_items : ALTER SEQUENCE",
"trigger_ddl_type_items : DROP SEQUENCE",
"trigger_ddl_type_items : CREATE USER",
"trigger_ddl_type_items : ALTER USER",
"trigger_ddl_type_items : DROP USER",
"trigger_ddl_type_items : CREATE INDEX",
"trigger_ddl_type_items : ALTER INDEX",
"trigger_ddl_type_items : DROP INDEX",
"trigger_ddl_type_items : CREATE COLLATION",
"trigger_ddl_type_items : DROP COLLATION",
"trigger_ddl_type_items : ALTER CHARACTER SET",
"trigger_ddl_type_items : CREATE PACKAGE",
"trigger_ddl_type_items : ALTER PACKAGE",
"trigger_ddl_type_items : DROP PACKAGE",
"trigger_ddl_type_items : CREATE PACKAGE BODY",
"trigger_ddl_type_items : DROP PACKAGE BODY",
"trigger_ddl_type_items : CREATE MAPPING",
"trigger_ddl_type_items : ALTER MAPPING",
"trigger_ddl_type_items : DROP MAPPING",
"trigger_ddl_type_items : trigger_ddl_type OR trigger_ddl_type",
"trigger_type_prefix : BEFORE",
"trigger_type_prefix : AFTER",
"trigger_type_suffix : INSERT",
"trigger_type_suffix : UPDATE",
"trigger_type_suffix : KW_DELETE",
"trigger_type_suffix : INSERT OR UPDATE",
"trigger_type_suffix : INSERT OR KW_DELETE",
"trigger_type_suffix : UPDATE OR INSERT",
"trigger_type_suffix : UPDATE OR KW_DELETE",
"trigger_type_suffix : KW_DELETE OR INSERT",
"trigger_type_suffix : KW_DELETE OR UPDATE",
"trigger_type_suffix : INSERT OR UPDATE OR KW_DELETE",
"trigger_type_suffix : INSERT OR KW_DELETE OR UPDATE",
"trigger_type_suffix : UPDATE OR INSERT OR KW_DELETE",
"trigger_type_suffix : UPDATE OR KW_DELETE OR INSERT",
"trigger_type_suffix : KW_DELETE OR INSERT OR UPDATE",
"trigger_type_suffix : KW_DELETE OR UPDATE OR INSERT",
"trigger_position :",
"trigger_position : POSITION nonneg_short_integer",
"alter : ALTER alter_clause",
"alter : set_generator_clause",
"alter_clause : EXCEPTION alter_exception_clause",
"$$81 :",
"$$82 :",
"alter_clause : TABLE simple_table_name $$81 $$82 alter_ops",
"alter_clause : VIEW alter_view_clause",
"alter_clause : TRIGGER alter_trigger_clause",
"alter_clause : PROCEDURE alter_procedure_clause",
"alter_clause : PACKAGE alter_package_clause",
"$$83 :",
"alter_clause : DATABASE $$83 $$41 alter_db",
"alter_clause : KW_DOMAIN alter_domain",
"alter_clause : INDEX alter_index_clause",
"alter_clause : EXTERNAL FUNCTION alter_udf_clause",
"alter_clause : FUNCTION alter_function_clause",
"alter_clause : ROLE alter_role_clause",
"alter_clause : USER alter_user_clause",
"alter_clause : CURRENT USER alter_cur_user_clause",
"alter_clause : CHARACTER SET alter_charset_clause",
"alter_clause : GENERATOR alter_sequence_clause",
"alter_clause : SEQUENCE alter_sequence_clause",
"alter_clause : MAPPING $$27 alter_map_clause",
"alter_clause : GLOBAL MAPPING $$28 alter_map_clause",
"$$84 :",
"$$85 :",
"alter_domain : keyword_or_column $$84 $$85 alter_domain_ops",
"alter_domain_ops : $$85 alter_domain_op",
"$$86 :",
"alter_domain_ops : alter_domain_ops $$86 alter_domain_op",
"alter_domain_op : SET domain_default",
"alter_domain_op : ADD CONSTRAINT check_constraint",
"alter_domain_op : ADD check_constraint",
"alter_domain_op : DROP DEFAULT",
"alter_domain_op : DROP CONSTRAINT",
"alter_domain_op : DROP NOT KW_NULL",
"alter_domain_op : SET NOT KW_NULL",
"alter_domain_op : TO symbol_column_name",
"alter_domain_op : KW_TYPE non_array_type",
"alter_ops : $$82 alter_op",
"$$87 :",
"alter_ops : alter_ops ',' $$87 alter_op",
"alter_op : DROP symbol_column_name drop_behaviour",
"alter_op : DROP CONSTRAINT symbol_constraint_name",
"alter_op : ADD $$58 column_def",
"alter_op : ADD $$58 table_constraint_definition",
"alter_op : col_opt alter_column_name POSITION pos_short_integer",
"alter_op : col_opt alter_column_name TO symbol_column_name",
"alter_op : col_opt alter_column_name DROP NOT KW_NULL",
"alter_op : col_opt alter_column_name SET NOT KW_NULL",
"alter_op : col_opt symbol_column_name KW_TYPE alter_data_type_or_domain",
"alter_op : col_opt symbol_column_name KW_TYPE non_array_type def_computed",
"alter_op : col_opt symbol_column_name def_computed",
"alter_op : col_opt symbol_column_name SET domain_default",
"alter_op : col_opt symbol_column_name DROP DEFAULT",
"alter_op : col_opt symbol_column_name restart_value_opt",
"alter_column_name : keyword_or_column",
"keyword_or_column : valid_symbol_name",
"keyword_or_column : ADMIN",
"keyword_or_column : COLUMN",
"keyword_or_column : EXTRACT",
"keyword_or_column : YEAR",
"keyword_or_column : MONTH",
"keyword_or_column : DAY",
"keyword_or_column : HOUR",
"keyword_or_column : MINUTE",
"keyword_or_column : SECOND",
"keyword_or_column : TIME",
"keyword_or_column : TIMESTAMP",
"keyword_or_column : CURRENT_DATE",
"keyword_or_column : CURRENT_TIME",
"keyword_or_column : CURRENT_TIMESTAMP",
"keyword_or_column : CURRENT_USER",
"keyword_or_column : CURRENT_ROLE",
"keyword_or_column : RECREATE",
"keyword_or_column : CURRENT_CONNECTION",
"keyword_or_column : CURRENT_TRANSACTION",
"keyword_or_column : BIGINT",
"keyword_or_column : CASE",
"keyword_or_column : RELEASE",
"keyword_or_column : ROW_COUNT",
"keyword_or_column : SAVEPOINT",
"keyword_or_column : OPEN",
"keyword_or_column : CLOSE",
"keyword_or_column : FETCH",
"keyword_or_column : ROWS",
"keyword_or_column : USING",
"keyword_or_column : CROSS",
"keyword_or_column : BIT_LENGTH",
"keyword_or_column : BOTH",
"keyword_or_column : CHAR_LENGTH",
"keyword_or_column : CHARACTER_LENGTH",
"keyword_or_column : COMMENT",
"keyword_or_column : LEADING",
"keyword_or_column : KW_LOWER",
"keyword_or_column : OCTET_LENGTH",
"keyword_or_column : TRAILING",
"keyword_or_column : TRIM",
"keyword_or_column : CONNECT",
"keyword_or_column : DISCONNECT",
"keyword_or_column : GLOBAL",
"keyword_or_column : INSENSITIVE",
"keyword_or_column : RECURSIVE",
"keyword_or_column : SENSITIVE",
"keyword_or_column : START",
"keyword_or_column : SIMILAR",
"keyword_or_column : KW_BOOLEAN",
"keyword_or_column : CORR",
"keyword_or_column : COVAR_POP",
"keyword_or_column : COVAR_SAMP",
"keyword_or_column : DELETING",
"keyword_or_column : DETERMINISTIC",
"keyword_or_column : KW_FALSE",
"keyword_or_column : INSERTING",
"keyword_or_column : OFFSET",
"keyword_or_column : OVER",
"keyword_or_column : REGR_AVGX",
"keyword_or_column : REGR_AVGY",
"keyword_or_column : REGR_COUNT",
"keyword_or_column : REGR_INTERCEPT",
"keyword_or_column : REGR_R2",
"keyword_or_column : REGR_SLOPE",
"keyword_or_column : REGR_SXX",
"keyword_or_column : REGR_SXY",
"keyword_or_column : REGR_SYY",
"keyword_or_column : RETURN",
"keyword_or_column : RDB_RECORD_VERSION",
"keyword_or_column : ROW",
"keyword_or_column : SCROLL",
"keyword_or_column : SQLSTATE",
"keyword_or_column : STDDEV_SAMP",
"keyword_or_column : STDDEV_POP",
"keyword_or_column : KW_TRUE",
"keyword_or_column : UNKNOWN",
"keyword_or_column : UPDATING",
"keyword_or_column : VAR_SAMP",
"keyword_or_column : VAR_POP",
"keyword_or_column : LOCALTIME",
"keyword_or_column : LOCALTIMESTAMP",
"col_opt : ALTER",
"col_opt : ALTER COLUMN",
"alter_data_type_or_domain : non_array_type",
"alter_data_type_or_domain : symbol_column_name",
"drop_behaviour :",
"drop_behaviour : RESTRICT",
"drop_behaviour : CASCADE",
"alter_index_clause : symbol_index_name ACTIVE",
"alter_index_clause : symbol_index_name INACTIVE",
"alter_udf_clause : symbol_UDF_name entry_op module_op",
"entry_op :",
"entry_op : ENTRY_POINT utf_string",
"module_op :",
"module_op : MODULE_NAME utf_string",
"alter_role_clause : symbol_role_name alter_role_enable AUTO ADMIN MAPPING",
"alter_role_enable : SET",
"alter_role_enable : DROP",
"alter_db : $$41 db_alter_clause",
"alter_db : alter_db $$43 db_alter_clause",
"$$88 :",
"db_alter_clause : ADD $$88 db_file_list",
"db_alter_clause : ADD KW_DIFFERENCE KW_FILE utf_string",
"db_alter_clause : DROP KW_DIFFERENCE KW_FILE",
"db_alter_clause : BEGIN BACKUP",
"db_alter_clause : END BACKUP",
"db_alter_clause : SET DEFAULT CHARACTER SET symbol_character_set_name",
"$$89 :",
"db_alter_clause : ENCRYPT WITH valid_symbol_name $$89 crypt_key_clause",
"db_alter_clause : DECRYPT",
"db_alter_clause : SET LINGER TO long_integer",
"db_alter_clause : DROP LINGER",
"crypt_key_clause :",
"crypt_key_clause : KEY valid_symbol_name",
"alter_trigger_clause : symbol_trigger_name trigger_active trigger_type_opt trigger_position AS local_declaration_list full_proc_block",
"alter_trigger_clause : symbol_trigger_name trigger_active trigger_type_opt trigger_position external_clause external_body_clause_opt",
"alter_trigger_clause : symbol_trigger_name trigger_active trigger_type_opt trigger_position",
"trigger_type_opt : trigger_type_prefix trigger_type_suffix",
"trigger_type_opt :",
"drop : DROP drop_clause",
"drop_clause : EXCEPTION symbol_exception_name",
"drop_clause : INDEX symbol_index_name",
"drop_clause : PROCEDURE symbol_procedure_name",
"drop_clause : TABLE symbol_table_name",
"drop_clause : TRIGGER symbol_trigger_name",
"drop_clause : VIEW symbol_view_name",
"drop_clause : FILTER symbol_filter_name",
"drop_clause : KW_DOMAIN symbol_domain_name",
"drop_clause : EXTERNAL FUNCTION symbol_UDF_name",
"drop_clause : FUNCTION symbol_UDF_name",
"drop_clause : SHADOW pos_short_integer opt_no_file_delete",
"drop_clause : ROLE symbol_role_name",
"drop_clause : GENERATOR symbol_generator_name",
"drop_clause : SEQUENCE symbol_generator_name",
"drop_clause : COLLATION symbol_collation_name",
"drop_clause : USER symbol_user_name USING PLUGIN valid_symbol_name",
"drop_clause : USER symbol_user_name",
"drop_clause : PACKAGE symbol_package_name",
"drop_clause : PACKAGE BODY symbol_package_name",
"drop_clause : MAPPING $$27 drop_map_clause",
"drop_clause : GLOBAL MAPPING $$28 drop_map_clause",
"opt_no_file_delete :",
"opt_no_file_delete : PRESERVE KW_FILE",
"opt_no_file_delete : KW_DELETE KW_FILE",
"data_type : non_array_type",
"data_type : array_type",
"domain_or_non_array_type : domain_or_non_array_type_name",
"domain_or_non_array_type : domain_or_non_array_type_name NOT KW_NULL",
"domain_or_non_array_type_name : non_array_type",
"domain_or_non_array_type_name : domain_type",
"domain_type : KW_TYPE OF symbol_column_name",
"domain_type : KW_TYPE OF COLUMN symbol_column_name '.' symbol_column_name",
"domain_type : symbol_column_name",
"non_array_type : simple_type",
"non_array_type : blob_type",
"array_type : non_charset_simple_type '[' array_spec ']'",
"array_type : character_type '[' array_spec ']' charset_clause",
"array_spec : array_range",
"array_spec : array_spec ',' array_range",
"array_range : signed_long_integer",
"array_range : signed_long_integer ':' signed_long_integer",
"simple_type : non_charset_simple_type",
"simple_type : character_type charset_clause",
"non_charset_simple_type : national_character_type",
"non_charset_simple_type : numeric_type",
"non_charset_simple_type : float_type",
"non_charset_simple_type : BIGINT",
"non_charset_simple_type : integer_keyword",
"non_charset_simple_type : SMALLINT",
"non_charset_simple_type : DATE",
"non_charset_simple_type : TIME",
"non_charset_simple_type : TIMESTAMP",
"non_charset_simple_type : KW_BOOLEAN",
"integer_keyword : INTEGER",
"integer_keyword : KW_INT",
"$$90 :",
"$$91 :",
"blob_type : BLOB $$90 $$91 blob_subtype blob_segsize charset_clause",
"blob_type : BLOB '(' unsigned_short_integer ')'",
"blob_type : BLOB '(' unsigned_short_integer ',' signed_short_integer ')'",
"blob_type : BLOB '(' ',' signed_short_integer ')'",
"blob_segsize :",
"blob_segsize : SEGMENT KW_SIZE unsigned_short_integer",
"blob_subtype :",
"blob_subtype : SUB_TYPE signed_short_integer",
"blob_subtype : SUB_TYPE symbol_blob_subtype_name",
"charset_clause :",
"charset_clause : CHARACTER SET symbol_character_set_name",
"national_character_type : national_character_keyword '(' pos_short_integer ')'",
"national_character_type : national_character_keyword",
"national_character_type : national_character_keyword VARYING '(' pos_short_integer ')'",
"character_type : character_keyword '(' pos_short_integer ')'",
"character_type : character_keyword",
"character_type : varying_keyword '(' pos_short_integer ')'",
"varying_keyword : VARCHAR",
"varying_keyword : CHARACTER VARYING",
"varying_keyword : KW_CHAR VARYING",
"character_keyword : CHARACTER",
"character_keyword : KW_CHAR",
"national_character_keyword : NCHAR",
"national_character_keyword : NATIONAL CHARACTER",
"national_character_keyword : NATIONAL KW_CHAR",
"numeric_type : KW_NUMERIC prec_scale",
"numeric_type : decimal_keyword prec_scale",
"prec_scale :",
"prec_scale : '(' signed_long_integer ')'",
"prec_scale : '(' signed_long_integer ',' signed_long_integer ')'",
"decimal_keyword : DECIMAL",
"decimal_keyword : KW_DEC",
"float_type : KW_FLOAT precision_opt",
"float_type : KW_LONG KW_FLOAT precision_opt",
"float_type : REAL",
"float_type : KW_DOUBLE PRECISION",
"precision_opt :",
"precision_opt : '(' nonneg_short_integer ')'",
"savepoint : set_savepoint",
"savepoint : release_savepoint",
"savepoint : undo_savepoint",
"set_savepoint : SAVEPOINT symbol_savepoint_name",
"release_savepoint : RELEASE SAVEPOINT symbol_savepoint_name release_only_opt",
"release_only_opt :",
"release_only_opt : ONLY",
"undo_savepoint : ROLLBACK optional_work TO optional_savepoint symbol_savepoint_name",
"optional_savepoint :",
"optional_savepoint : SAVEPOINT",
"commit : COMMIT optional_work optional_retain",
"rollback : ROLLBACK optional_work optional_retain",
"optional_work :",
"optional_work : WORK",
"optional_retain :",
"optional_retain : RETAIN opt_snapshot",
"opt_snapshot :",
"opt_snapshot : SNAPSHOT",
"$$92 :",
"$$93 :",
"set_transaction : SET TRANSACTION $$92 $$93 tran_option_list_opt",
"set_role : SET ROLE valid_symbol_name",
"set_role : SET TRUSTED ROLE",
"tran_option_list_opt :",
"tran_option_list_opt : $$93 tran_option_list",
"tran_option_list : $$93 tran_option",
"$$94 :",
"tran_option_list : tran_option_list $$94 tran_option",
"tran_option : READ ONLY",
"tran_option : READ WRITE",
"tran_option : WAIT",
"tran_option : NO WAIT",
"tran_option : isolation_mode",
"tran_option : NO AUTO UNDO",
"tran_option : KW_IGNORE LIMBO",
"tran_option : RESTART REQUESTS",
"tran_option : LOCK TIMEOUT nonneg_short_integer",
"$$95 :",
"$$96 :",
"tran_option : RESERVING $$95 $$96 restr_list",
"isolation_mode : ISOLATION LEVEL iso_mode",
"isolation_mode : iso_mode",
"iso_mode : snap_shot",
"iso_mode : READ UNCOMMITTED version_mode",
"iso_mode : READ COMMITTED version_mode",
"snap_shot : SNAPSHOT",
"snap_shot : SNAPSHOT TABLE",
"snap_shot : SNAPSHOT TABLE STABILITY",
"version_mode :",
"version_mode : VERSION",
"version_mode : NO VERSION",
"lock_type :",
"lock_type : KW_SHARED",
"lock_type : PROTECTED",
"lock_mode : READ",
"lock_mode : WRITE",
"restr_list : restr_option",
"restr_list : restr_list ',' restr_option",
"restr_option : table_list table_lock",
"table_lock :",
"table_lock : FOR lock_type lock_mode",
"table_list : symbol_table_name",
"table_list : table_list ',' symbol_table_name",
"set_statistics : SET STATISTICS INDEX symbol_index_name",
"comment : COMMENT ON ddl_type0 IS ddl_desc",
"comment : COMMENT ON ddl_type1 symbol_ddl_name IS ddl_desc",
"comment : COMMENT ON ddl_type2 symbol_ddl_name ddl_subname IS ddl_desc",
"comment : COMMENT ON ddl_type3 ddl_qualified_name ddl_subname IS ddl_desc",
"comment : COMMENT ON ddl_type4 ddl_qualified_name IS ddl_desc",
"comment : COMMENT ON USER symbol_user_name IS ddl_desc",
"ddl_type0 : DATABASE",
"ddl_type1 : KW_DOMAIN",
"ddl_type1 : TABLE",
"ddl_type1 : VIEW",
"ddl_type1 : TRIGGER",
"ddl_type1 : FILTER",
"ddl_type1 : EXCEPTION",
"ddl_type1 : GENERATOR",
"ddl_type1 : SEQUENCE",
"ddl_type1 : INDEX",
"ddl_type1 : ROLE",
"ddl_type1 : CHARACTER SET",
"ddl_type1 : COLLATION",
"ddl_type1 : PACKAGE",
"ddl_type2 : COLUMN",
"ddl_type3 : PARAMETER",
"ddl_type3 : PROCEDURE PARAMETER",
"ddl_type3 : FUNCTION PARAMETER",
"ddl_type4 : PROCEDURE",
"ddl_type4 : EXTERNAL FUNCTION",
"ddl_type4 : FUNCTION",
"ddl_subname : '.' symbol_ddl_name",
"ddl_qualified_name : symbol_ddl_name",
"ddl_qualified_name : symbol_ddl_name '.' symbol_ddl_name",
"ddl_desc : utf_string",
"ddl_desc : KW_NULL",
"select : select_expr for_update_clause lock_clause",
"for_update_clause :",
"for_update_clause : FOR UPDATE for_update_list",
"for_update_list :",
"for_update_list : OF column_list",
"lock_clause :",
"lock_clause : WITH LOCK",
"select_expr : with_clause select_expr_body order_clause rows_clause",
"select_expr : with_clause select_expr_body order_clause result_offset_clause fetch_first_clause",
"with_clause :",
"with_clause : WITH RECURSIVE with_list",
"with_clause : WITH with_list",
"with_list : with_item",
"with_list : with_list ',' with_item",
"with_item : symbol_table_alias_name derived_column_list AS '(' select_expr ')'",
"column_select : select_expr",
"column_singleton : column_select",
"select_expr_body : query_term",
"select_expr_body : select_expr_body UNION distinct_noise query_term",
"select_expr_body : select_expr_body UNION ALL query_term",
"query_term : query_spec",
"query_spec : SELECT limit_clause distinct_clause select_list from_clause where_clause group_clause having_clause plan_clause",
"limit_clause :",
"limit_clause : first_clause skip_clause",
"limit_clause : first_clause",
"limit_clause : skip_clause",
"first_clause : FIRST long_integer",
"first_clause : FIRST '(' value ')'",
"first_clause : FIRST parameter",
"skip_clause : SKIP long_integer",
"skip_clause : SKIP '(' value ')'",
"skip_clause : SKIP parameter",
"distinct_clause : DISTINCT",
"distinct_clause : all_noise",
"select_list : select_items",
"select_list : '*'",
"select_items : select_item",
"select_items : select_items ',' select_item",
"select_item : value_opt_alias",
"select_item : symbol_table_alias_name '.' '*'",
"value_opt_alias : value",
"value_opt_alias : value as_noise symbol_item_alias_name",
"as_noise :",
"as_noise : AS",
"from_clause : FROM from_list",
"from_list : table_reference",
"from_list : from_list ',' table_reference",
"table_reference : joined_table",
"table_reference : table_primary",
"table_primary : table_proc",
"table_primary : derived_table",
"table_primary : '(' joined_table ')'",
"derived_table : '(' select_expr ')' as_noise correlation_name derived_column_list",
"correlation_name :",
"correlation_name : symbol_table_alias_name",
"derived_column_list :",
"derived_column_list : '(' alias_list ')'",
"alias_list : symbol_item_alias_name",
"alias_list : alias_list ',' symbol_item_alias_name",
"joined_table : cross_join",
"joined_table : natural_join",
"joined_table : qualified_join",
"cross_join : table_reference CROSS JOIN table_primary",
"natural_join : table_reference NATURAL join_type JOIN table_primary",
"qualified_join : table_reference join_type JOIN table_reference join_condition",
"qualified_join : table_reference join_type JOIN table_reference named_columns_join",
"join_condition : ON search_condition",
"named_columns_join : USING '(' column_list ')'",
"table_proc : symbol_procedure_name table_proc_inputs as_noise symbol_table_alias_name",
"table_proc : symbol_procedure_name table_proc_inputs",
"table_proc : symbol_package_name '.' symbol_procedure_name table_proc_inputs as_noise symbol_table_alias_name",
"table_proc : symbol_package_name '.' symbol_procedure_name table_proc_inputs",
"table_proc_inputs :",
"table_proc_inputs : '(' value_list ')'",
"table_name : simple_table_name",
"table_name : symbol_table_name as_noise symbol_table_alias_name",
"simple_table_name : symbol_table_name",
"join_type :",
"join_type : INNER",
"join_type : LEFT outer_noise",
"join_type : RIGHT outer_noise",
"join_type : FULL outer_noise",
"outer_noise :",
"outer_noise : OUTER",
"group_clause :",
"group_clause : GROUP BY group_by_list",
"group_by_list : group_by_item",
"group_by_list : group_by_list ',' group_by_item",
"group_by_item : value",
"having_clause :",
"having_clause : HAVING search_condition",
"where_clause :",
"where_clause : WHERE search_condition",
"plan_clause :",
"plan_clause : PLAN plan_expression",
"$$97 :",
"$$98 :",
"plan_expression : plan_type $$97 '(' $$98 plan_item_list ')'",
"plan_type :",
"plan_type : JOIN",
"plan_type : SORT MERGE",
"plan_type : MERGE",
"plan_type : HASH",
"plan_type : SORT",
"plan_item_list : plan_item",
"plan_item_list : plan_item_list ',' plan_item",
"plan_item : table_or_alias_list access_type",
"plan_item : plan_expression",
"table_or_alias_list : symbol_table_name",
"table_or_alias_list : table_or_alias_list symbol_table_name",
"access_type : NATURAL",
"$$99 :",
"$$100 :",
"access_type : INDEX $$99 '(' $$100 index_list ')'",
"$$101 :",
"access_type : ORDER $$101 symbol_index_name $$100 extra_indices_opt",
"index_list : symbol_index_name",
"index_list : index_list ',' symbol_index_name",
"extra_indices_opt :",
"$$102 :",
"extra_indices_opt : INDEX '(' $$102 index_list ')'",
"order_clause :",
"order_clause : ORDER BY order_list",
"order_list : order_item",
"order_list : order_list ',' order_item",
"order_item : value order_direction nulls_clause",
"order_direction :",
"order_direction : ASC",
"order_direction : DESC",
"nulls_clause :",
"nulls_clause : NULLS nulls_placement",
"nulls_placement : FIRST",
"nulls_placement : LAST",
"rows_clause : ROWS value",
"rows_clause : ROWS value TO value",
"rows_clause_optional :",
"rows_clause_optional : rows_clause",
"row_noise : ROW",
"row_noise : ROWS",
"result_offset_clause :",
"result_offset_clause : OFFSET simple_value_spec row_noise",
"first_next_noise : FIRST",
"first_next_noise : NEXT",
"fetch_first_clause :",
"fetch_first_clause : FETCH first_next_noise simple_value_spec row_noise ONLY",
"fetch_first_clause : FETCH first_next_noise row_noise ONLY",
"$$103 :",
"insert : insert_start $$103 ins_column_parens_opt VALUES '(' value_list ')' returning_clause",
"insert : insert_start $$103 ins_column_parens_opt select_expr returning_clause",
"insert : insert_start DEFAULT VALUES returning_clause",
"insert_start : INSERT INTO simple_table_name",
"$$104 :",
"$$105 :",
"merge : MERGE INTO table_name USING table_reference ON search_condition $$104 $$105 merge_when_clause returning_clause",
"merge_when_clause : $$105 merge_when_matched_clause",
"merge_when_clause : $$105 merge_when_not_matched_clause",
"$$106 :",
"merge_when_clause : merge_when_clause $$106 merge_when_matched_clause",
"merge_when_clause : merge_when_clause $$106 merge_when_not_matched_clause",
"$$107 :",
"$$108 :",
"merge_when_matched_clause : WHEN MATCHED $$107 $$108 merge_update_specification",
"$$109 :",
"$$110 :",
"merge_when_not_matched_clause : WHEN NOT MATCHED $$109 $$110 merge_insert_specification",
"merge_update_specification : THEN UPDATE SET assignments",
"merge_update_specification : AND search_condition THEN UPDATE SET assignments",
"merge_update_specification : THEN KW_DELETE",
"merge_update_specification : AND search_condition THEN KW_DELETE",
"$$111 :",
"merge_insert_specification : THEN INSERT $$111 ins_column_parens_opt VALUES '(' value_list ')'",
"$$112 :",
"merge_insert_specification : AND search_condition THEN INSERT $$112 ins_column_parens_opt VALUES '(' value_list ')'",
"delete : delete_searched",
"delete : delete_positioned",
"delete_searched : KW_DELETE FROM table_name where_clause plan_clause order_clause rows_clause_optional returning_clause",
"delete_positioned : KW_DELETE FROM table_name cursor_clause returning_clause",
"update : update_searched",
"update : update_positioned",
"update_searched : UPDATE table_name SET assignments where_clause plan_clause order_clause rows_clause_optional returning_clause",
"update_positioned : UPDATE table_name SET assignments cursor_clause returning_clause",
"$$113 :",
"$$114 :",
"$$115 :",
"update_or_insert : UPDATE OR INSERT INTO simple_table_name $$113 $$114 ins_column_parens_opt VALUES '(' value_list ')' $$115 update_or_insert_matching_opt returning_clause",
"update_or_insert_matching_opt :",
"$$116 :",
"update_or_insert_matching_opt : MATCHING $$116 ins_column_parens",
"returning_clause :",
"returning_clause : RETURNING value_opt_alias_list",
"returning_clause : RETURNING value_opt_alias_list INTO variable_list",
"value_opt_alias_list : value_opt_alias",
"value_opt_alias_list : value_opt_alias_list ',' value_opt_alias",
"cursor_clause : WHERE CURRENT OF symbol_cursor_name",
"assignments : assignment",
"assignments : assignments ',' assignment",
"assignment : update_column_name '=' value",
"exec_function : udf",
"exec_function : non_aggregate_function",
"column_parens_opt :",
"column_parens_opt : column_parens",
"column_parens : '(' column_list ')'",
"column_list : simple_column_name",
"column_list : column_list ',' simple_column_name",
"ins_column_parens_opt :",
"$$117 :",
"ins_column_parens_opt : $$117 ins_column_parens",
"ins_column_parens : '(' $$116 ins_column_list ')'",
"ins_column_list : update_column_name",
"ins_column_list : ins_column_list ',' update_column_name",
"column_name : simple_column_name",
"column_name : symbol_table_alias_name '.' symbol_column_name",
"column_name : ':' symbol_table_alias_name '.' symbol_column_name",
"simple_column_name : symbol_column_name",
"update_column_name : simple_column_name",
"update_column_name : symbol_table_alias_name '.' symbol_column_name",
"search_condition : value",
"boolean_value_expression : predicate",
"boolean_value_expression : value OR value",
"boolean_value_expression : value AND value",
"boolean_value_expression : NOT value",
"boolean_value_expression : '(' boolean_value_expression ')'",
"boolean_value_expression : value IS boolean_literal",
"boolean_value_expression : value IS NOT boolean_literal",
"predicate : comparison_predicate",
"predicate : distinct_predicate",
"predicate : between_predicate",
"predicate : binary_pattern_predicate",
"predicate : ternary_pattern_predicate",
"predicate : in_predicate",
"predicate : null_predicate",
"predicate : quantified_predicate",
"predicate : exists_predicate",
"predicate : singular_predicate",
"predicate : trigger_action_predicate",
"comparison_predicate : value comparison_operator value",
"comparison_operator : '='",
"comparison_operator : '<'",
"comparison_operator : '>'",
"comparison_operator : GEQ",
"comparison_operator : LEQ",
"comparison_operator : NOT_GTR",
"comparison_operator : NOT_LSS",
"comparison_operator : NEQ",
"quantified_predicate : value comparison_operator quantified_flag '(' column_select ')'",
"quantified_flag : ALL",
"quantified_flag : SOME",
"quantified_flag : ANY",
"distinct_predicate : value IS DISTINCT FROM value",
"distinct_predicate : value IS NOT DISTINCT FROM value",
"between_predicate : value BETWEEN value_between AND value_between",
"between_predicate : value NOT BETWEEN value_between AND value_between",
"value_between : value_primary",
"value_between : '(' boolean_value_expression ')'",
"binary_pattern_predicate : value binary_pattern_operator value",
"binary_pattern_predicate : value NOT binary_pattern_operator value",
"binary_pattern_operator : CONTAINING",
"binary_pattern_operator : STARTING",
"binary_pattern_operator : STARTING WITH",
"ternary_pattern_predicate : value LIKE value",
"ternary_pattern_predicate : value LIKE value ESCAPE value",
"ternary_pattern_predicate : value NOT LIKE value",
"ternary_pattern_predicate : value NOT LIKE value ESCAPE value",
"ternary_pattern_predicate : value SIMILAR TO value",
"ternary_pattern_predicate : value SIMILAR TO value ESCAPE value",
"ternary_pattern_predicate : value NOT SIMILAR TO value",
"ternary_pattern_predicate : value NOT SIMILAR TO value ESCAPE value",
"in_predicate : value KW_IN in_predicate_value",
"in_predicate : value NOT KW_IN in_predicate_value",
"exists_predicate : EXISTS '(' select_expr ')'",
"singular_predicate : SINGULAR '(' select_expr ')'",
"trigger_action_predicate : INSERTING",
"trigger_action_predicate : UPDATING",
"trigger_action_predicate : DELETING",
"null_predicate : value IS KW_NULL",
"null_predicate : value IS UNKNOWN",
"null_predicate : value IS NOT KW_NULL",
"null_predicate : value IS NOT UNKNOWN",
"in_predicate_value : table_subquery",
"in_predicate_value : '(' value_list ')'",
"table_subquery : '(' column_select ')'",
"$$118 :",
"$$119 :",
"$$120 :",
"create_user_clause : symbol_user_name $$118 $$119 user_fixed_list_opt $$120 user_var_opts",
"$$121 :",
"alter_user_clause : symbol_user_name set_noise $$121 $$119 user_fixed_list_opt $$120 user_var_opts",
"$$122 :",
"alter_cur_user_clause : set_noise $$122 $$119 user_fixed_list_opt $$120 user_var_opts",
"$$123 :",
"replace_user_clause : symbol_user_name set_noise $$123 $$119 user_fixed_list_opt $$120 user_var_opts",
"set_noise :",
"set_noise : SET",
"user_fixed_list_opt :",
"user_fixed_list_opt : $$119 user_fixed_list",
"user_fixed_list : $$119 user_fixed_option",
"$$124 :",
"user_fixed_list : user_fixed_list $$124 user_fixed_option",
"user_fixed_option : FIRSTNAME utf_string",
"user_fixed_option : MIDDLENAME utf_string",
"user_fixed_option : LASTNAME utf_string",
"user_fixed_option : PASSWORD utf_string",
"user_fixed_option : GRANT ADMIN ROLE",
"user_fixed_option : REVOKE ADMIN ROLE",
"user_fixed_option : ACTIVE",
"user_fixed_option : INACTIVE",
"user_fixed_option : USING PLUGIN valid_symbol_name",
"user_var_opts :",
"user_var_opts : TAGS '(' $$120 user_var_list ')'",
"user_var_list : $$119 user_var_option",
"user_var_list : user_var_list ',' $$120 user_var_option",
"user_var_option : valid_symbol_name '=' utf_string",
"user_var_option : DROP valid_symbol_name",
"$$125 :",
"$$126 :",
"$$127 :",
"create_map_clause : $$125 map_clause $$126 $$127 map_to",
"$$128 :",
"$$129 :",
"alter_map_clause : $$128 map_clause $$129 $$127 map_to",
"$$130 :",
"$$131 :",
"replace_map_clause : $$130 map_clause $$131 $$127 map_to",
"drop_map_clause : map_name",
"$$132 :",
"$$133 :",
"$$134 :",
"map_clause : map_name $$132 USING $$133 map_using FROM $$134 map_from",
"map_name : valid_symbol_name",
"map_from : map_from_symbol_name map_logoninfo",
"map_from : ANY map_from_symbol_name",
"map_from_symbol_name : valid_symbol_name",
"map_from_symbol_name : USER",
"map_from_symbol_name : GROUP",
"map_logoninfo : STRING",
"map_logoninfo : valid_symbol_name",
"map_using : PLUGIN valid_symbol_name map_in",
"map_using : ANY PLUGIN map_in",
"map_using : ANY PLUGIN SERVERWIDE",
"map_using : MAPPING map_in",
"map_using : '*' map_in",
"map_in :",
"map_in : KW_IN valid_symbol_name",
"map_to : TO map_role valid_symbol_name",
"map_to : TO map_role",
"map_role : ROLE",
"map_role : USER",
"value : value_primary",
"value : boolean_value_expression",
"value_primary : nonparenthesized_value",
"value_primary : '(' value_primary ')'",
"simple_value_spec : constant",
"simple_value_spec : variable",
"simple_value_spec : parameter",
"nonparenthesized_value : column_name",
"nonparenthesized_value : array_element",
"nonparenthesized_value : function",
"nonparenthesized_value : u_constant",
"nonparenthesized_value : boolean_literal",
"nonparenthesized_value : parameter",
"nonparenthesized_value : variable",
"nonparenthesized_value : cast_specification",
"nonparenthesized_value : case_expression",
"nonparenthesized_value : next_value_expression",
"nonparenthesized_value : udf",
"nonparenthesized_value : '-' value_primary",
"nonparenthesized_value : '+' value_primary",
"nonparenthesized_value : value_primary '+' value_primary",
"nonparenthesized_value : value_primary CONCATENATE value_primary",
"nonparenthesized_value : value_primary COLLATE symbol_collation_name",
"nonparenthesized_value : value_primary '-' value_primary",
"nonparenthesized_value : value_primary '*' value_primary",
"nonparenthesized_value : value_primary '/' value_primary",
"nonparenthesized_value : '(' column_singleton ')'",
"nonparenthesized_value : current_user",
"nonparenthesized_value : current_role",
"nonparenthesized_value : internal_info",
"nonparenthesized_value : recordKeyType",
"nonparenthesized_value : symbol_table_alias_name '.' recordKeyType",
"nonparenthesized_value : KW_VALUE",
"nonparenthesized_value : datetime_value_expression",
"nonparenthesized_value : null_value",
"recordKeyType : DB_KEY",
"recordKeyType : RDB_RECORD_VERSION",
"datetime_value_expression : CURRENT_DATE",
"datetime_value_expression : CURRENT_TIME time_precision_opt",
"datetime_value_expression : LOCALTIME time_precision_opt",
"datetime_value_expression : CURRENT_TIMESTAMP timestamp_precision_opt",
"datetime_value_expression : LOCALTIMESTAMP timestamp_precision_opt",
"time_precision_opt :",
"time_precision_opt : '(' nonneg_short_integer ')'",
"timestamp_precision_opt :",
"timestamp_precision_opt : '(' nonneg_short_integer ')'",
"array_element : column_name '[' value_list ']'",
"value_list_opt :",
"value_list_opt : value_list",
"value_list : value",
"value_list : value_list ',' value",
"constant : u_constant",
"constant : '-' u_numeric_constant",
"constant : boolean_literal",
"u_numeric_constant : NUMBER",
"u_numeric_constant : FLOAT_NUMBER",
"u_numeric_constant : NUMBER64BIT",
"u_numeric_constant : SCALEDINT",
"u_constant : u_numeric_constant",
"u_constant : sql_string",
"u_constant : DATE STRING",
"u_constant : TIME STRING",
"u_constant : TIMESTAMP STRING",
"boolean_literal : KW_FALSE",
"boolean_literal : KW_TRUE",
"parameter : '?'",
"current_user : USER",
"current_user : CURRENT_USER",
"current_role : CURRENT_ROLE",
"internal_info : CURRENT_CONNECTION",
"internal_info : CURRENT_TRANSACTION",
"internal_info : GDSCODE",
"internal_info : SQLCODE",
"internal_info : SQLSTATE",
"internal_info : ROW_COUNT",
"sql_string : STRING",
"sql_string : INTRODUCER STRING",
"utf_string : sql_string",
"signed_short_integer : nonneg_short_integer",
"signed_short_integer : '-' neg_short_integer",
"nonneg_short_integer : NUMBER",
"neg_short_integer : NUMBER",
"pos_short_integer : nonneg_short_integer",
"unsigned_short_integer : NUMBER",
"signed_long_integer : long_integer",
"signed_long_integer : '-' long_integer",
"long_integer : NUMBER",
"function : aggregate_function",
"function : non_aggregate_function",
"function : over_clause",
"non_aggregate_function : numeric_value_function",
"non_aggregate_function : string_value_function",
"non_aggregate_function : system_function_expression",
"aggregate_function : COUNT '(' '*' ')'",
"aggregate_function : COUNT '(' all_noise value ')'",
"aggregate_function : COUNT '(' DISTINCT value ')'",
"aggregate_function : SUM '(' all_noise value ')'",
"aggregate_function : SUM '(' DISTINCT value ')'",
"aggregate_function : AVG '(' all_noise value ')'",
"aggregate_function : AVG '(' DISTINCT value ')'",
"aggregate_function : MINIMUM '(' all_noise value ')'",
"aggregate_function : MINIMUM '(' DISTINCT value ')'",
"aggregate_function : MAXIMUM '(' all_noise value ')'",
"aggregate_function : MAXIMUM '(' DISTINCT value ')'",
"aggregate_function : LIST '(' all_noise value delimiter_opt ')'",
"aggregate_function : LIST '(' DISTINCT value delimiter_opt ')'",
"aggregate_function : STDDEV_SAMP '(' value ')'",
"aggregate_function : STDDEV_POP '(' value ')'",
"aggregate_function : VAR_SAMP '(' value ')'",
"aggregate_function : VAR_POP '(' value ')'",
"aggregate_function : COVAR_SAMP '(' value ',' value ')'",
"aggregate_function : COVAR_POP '(' value ',' value ')'",
"aggregate_function : CORR '(' value ',' value ')'",
"aggregate_function : REGR_AVGX '(' value ',' value ')'",
"aggregate_function : REGR_AVGY '(' value ',' value ')'",
"aggregate_function : REGR_COUNT '(' value ',' value ')'",
"aggregate_function : REGR_INTERCEPT '(' value ',' value ')'",
"aggregate_function : REGR_R2 '(' value ',' value ')'",
"aggregate_function : REGR_SLOPE '(' value ',' value ')'",
"aggregate_function : REGR_SXX '(' value ',' value ')'",
"aggregate_function : REGR_SXY '(' value ',' value ')'",
"aggregate_function : REGR_SYY '(' value ',' value ')'",
"window_function : DENSE_RANK '(' ')'",
"window_function : RANK '(' ')'",
"window_function : ROW_NUMBER '(' ')'",
"window_function : FIRST_VALUE '(' value ')'",
"window_function : LAST_VALUE '(' value ')'",
"window_function : NTH_VALUE '(' value ',' value ')' nth_from",
"window_function : LAG '(' value ',' value ',' value ')'",
"window_function : LAG '(' value ',' value ')'",
"window_function : LAG '(' value ')'",
"window_function : LEAD '(' value ',' value ',' value ')'",
"window_function : LEAD '(' value ',' value ')'",
"window_function : LEAD '(' value ')'",
"nth_from :",
"nth_from : FROM FIRST",
"nth_from : FROM LAST",
"aggregate_window_function : aggregate_function",
"aggregate_window_function : window_function",
"over_clause : aggregate_window_function OVER '(' window_partition_opt order_clause ')'",
"window_partition_opt :",
"window_partition_opt : PARTITION BY value_list",
"delimiter_opt :",
"delimiter_opt : ',' value",
"numeric_value_function : extract_expression",
"numeric_value_function : length_expression",
"extract_expression : EXTRACT '(' timestamp_part FROM value ')'",
"length_expression : bit_length_expression",
"length_expression : char_length_expression",
"length_expression : octet_length_expression",
"bit_length_expression : BIT_LENGTH '(' value ')'",
"char_length_expression : CHAR_LENGTH '(' value ')'",
"char_length_expression : CHARACTER_LENGTH '(' value ')'",
"octet_length_expression : OCTET_LENGTH '(' value ')'",
"system_function_expression : system_function_std_syntax '(' value_list_opt ')'",
"system_function_expression : system_function_special_syntax",
"system_function_std_syntax : ABS",
"system_function_std_syntax : ACOS",
"system_function_std_syntax : ACOSH",
"system_function_std_syntax : ASCII_CHAR",
"system_function_std_syntax : ASCII_VAL",
"system_function_std_syntax : ASIN",
"system_function_std_syntax : ASINH",
"system_function_std_syntax : ATAN",
"system_function_std_syntax : ATAN2",
"system_function_std_syntax : ATANH",
"system_function_std_syntax : BIN_AND",
"system_function_std_syntax : BIN_NOT",
"system_function_std_syntax : BIN_OR",
"system_function_std_syntax : BIN_SHL",
"system_function_std_syntax : BIN_SHR",
"system_function_std_syntax : BIN_XOR",
"system_function_std_syntax : CEIL",
"system_function_std_syntax : CHAR_TO_UUID",
"system_function_std_syntax : COS",
"system_function_std_syntax : COSH",
"system_function_std_syntax : COT",
"system_function_std_syntax : EXP",
"system_function_std_syntax : FLOOR",
"system_function_std_syntax : GEN_UUID",
"system_function_std_syntax : HASH",
"system_function_std_syntax : LEFT",
"system_function_std_syntax : LN",
"system_function_std_syntax : LOG",
"system_function_std_syntax : LOG10",
"system_function_std_syntax : LPAD",
"system_function_std_syntax : MAXVALUE",
"system_function_std_syntax : MINVALUE",
"system_function_std_syntax : MOD",
"system_function_std_syntax : PI",
"system_function_std_syntax : POWER",
"system_function_std_syntax : RAND",
"system_function_std_syntax : RDB_GET_CONTEXT",
"system_function_std_syntax : RDB_SET_CONTEXT",
"system_function_std_syntax : REPLACE",
"system_function_std_syntax : REVERSE",
"system_function_std_syntax : RIGHT",
"system_function_std_syntax : ROUND",
"system_function_std_syntax : RPAD",
"system_function_std_syntax : SIGN",
"system_function_std_syntax : SIN",
"system_function_std_syntax : SINH",
"system_function_std_syntax : SQRT",
"system_function_std_syntax : TAN",
"system_function_std_syntax : TANH",
"system_function_std_syntax : TRUNC",
"system_function_std_syntax : UUID_TO_CHAR",
"system_function_special_syntax : DATEADD '(' value timestamp_part TO value ')'",
"system_function_special_syntax : DATEADD '(' timestamp_part ',' value ',' value ')'",
"system_function_special_syntax : DATEDIFF '(' timestamp_part FROM value TO value ')'",
"system_function_special_syntax : DATEDIFF '(' timestamp_part ',' value ',' value ')'",
"system_function_special_syntax : OVERLAY '(' value PLACING value FROM value FOR value ')'",
"system_function_special_syntax : OVERLAY '(' value PLACING value FROM value ')'",
"system_function_special_syntax : POSITION '(' value KW_IN value ')'",
"system_function_special_syntax : POSITION '(' value_list_opt ')'",
"string_value_function : substring_function",
"string_value_function : trim_function",
"string_value_function : KW_UPPER '(' value ')'",
"string_value_function : KW_LOWER '(' value ')'",
"substring_function : SUBSTRING '(' value FROM value string_length_opt ')'",
"substring_function : SUBSTRING '(' value SIMILAR value ESCAPE value ')'",
"string_length_opt :",
"string_length_opt : FOR value",
"trim_function : TRIM '(' trim_specification value FROM value ')'",
"trim_function : TRIM '(' value FROM value ')'",
"trim_function : TRIM '(' trim_specification FROM value ')'",
"trim_function : TRIM '(' value ')'",
"trim_specification : BOTH",
"trim_specification : TRAILING",
"trim_specification : LEADING",
"udf : symbol_UDF_call_name '(' value_list ')'",
"udf : symbol_UDF_call_name '(' ')'",
"udf : symbol_package_name '.' symbol_UDF_name '(' value_list ')'",
"udf : symbol_package_name '.' symbol_UDF_name '(' ')'",
"cast_specification : CAST '(' value AS data_type_descriptor ')'",
"case_expression : case_abbreviation",
"case_expression : case_specification",
"case_abbreviation : NULLIF '(' value ',' value ')'",
"case_abbreviation : IIF '(' search_condition ',' value ',' value ')'",
"case_abbreviation : COALESCE '(' value ',' value_list ')'",
"case_abbreviation : DECODE '(' value ',' decode_pairs ')'",
"case_abbreviation : DECODE '(' value ',' decode_pairs ',' value ')'",
"case_specification : simple_case",
"case_specification : searched_case",
"$$135 :",
"$$136 :",
"$$137 :",
"simple_case : CASE case_operand $$135 $$136 $$137 simple_when_clause else_case_result_opt END",
"simple_when_clause : WHEN when_operand THEN case_result",
"simple_when_clause : simple_when_clause WHEN when_operand THEN case_result",
"else_case_result_opt :",
"else_case_result_opt : ELSE case_result",
"searched_case : CASE searched_when_clause END",
"searched_case : CASE searched_when_clause ELSE case_result END",
"searched_when_clause : WHEN search_condition THEN case_result",
"searched_when_clause : searched_when_clause WHEN search_condition THEN case_result",
"when_operand : value",
"case_operand : value",
"case_result : value",
"decode_pairs : value ',' value",
"decode_pairs : decode_pairs ',' value ',' value",
"next_value_expression : NEXT KW_VALUE FOR symbol_generator_name",
"next_value_expression : GEN_ID '(' symbol_generator_name ',' value ')'",
"timestamp_part : YEAR",
"timestamp_part : MONTH",
"timestamp_part : DAY",
"timestamp_part : HOUR",
"timestamp_part : MINUTE",
"timestamp_part : SECOND",
"timestamp_part : MILLISECOND",
"timestamp_part : WEEK",
"timestamp_part : WEEKDAY",
"timestamp_part : YEARDAY",
"all_noise :",
"all_noise : ALL",
"distinct_noise :",
"distinct_noise : DISTINCT",
"null_value : KW_NULL",
"null_value : UNKNOWN",
"symbol_UDF_call_name : SYMBOL",
"symbol_UDF_name : valid_symbol_name",
"symbol_blob_subtype_name : valid_symbol_name",
"symbol_character_set_name : valid_symbol_name",
"symbol_collation_name : valid_symbol_name",
"symbol_column_name : valid_symbol_name",
"symbol_constraint_name : valid_symbol_name",
"symbol_cursor_name : valid_symbol_name",
"symbol_domain_name : valid_symbol_name",
"symbol_exception_name : valid_symbol_name",
"symbol_filter_name : valid_symbol_name",
"symbol_gdscode_name : valid_symbol_name",
"symbol_generator_name : valid_symbol_name",
"symbol_index_name : valid_symbol_name",
"symbol_item_alias_name : valid_symbol_name",
"symbol_label_name : valid_symbol_name",
"symbol_ddl_name : valid_symbol_name",
"symbol_procedure_name : valid_symbol_name",
"symbol_role_name : valid_symbol_name",
"symbol_table_alias_name : valid_symbol_name",
"symbol_table_name : valid_symbol_name",
"symbol_trigger_name : valid_symbol_name",
"symbol_user_name : valid_symbol_name",
"symbol_variable_name : valid_symbol_name",
"symbol_view_name : valid_symbol_name",
"symbol_savepoint_name : valid_symbol_name",
"symbol_package_name : valid_symbol_name",
"valid_symbol_name : SYMBOL",
"valid_symbol_name : non_reserved_word",
"non_reserved_word : ACTION",
"non_reserved_word : CASCADE",
"non_reserved_word : FREE_IT",
"non_reserved_word : RESTRICT",
"non_reserved_word : ROLE",
"non_reserved_word : KW_TYPE",
"non_reserved_word : KW_BREAK",
"non_reserved_word : KW_DESCRIPTOR",
"non_reserved_word : SUBSTRING",
"non_reserved_word : COALESCE",
"non_reserved_word : LAST",
"non_reserved_word : LEAVE",
"non_reserved_word : LOCK",
"non_reserved_word : NULLIF",
"non_reserved_word : NULLS",
"non_reserved_word : STATEMENT",
"non_reserved_word : FIRST",
"non_reserved_word : SKIP",
"non_reserved_word : BLOCK",
"non_reserved_word : BACKUP",
"non_reserved_word : KW_DIFFERENCE",
"non_reserved_word : IIF",
"non_reserved_word : SCALAR_ARRAY",
"non_reserved_word : WEEKDAY",
"non_reserved_word : YEARDAY",
"non_reserved_word : SEQUENCE",
"non_reserved_word : NEXT",
"non_reserved_word : RESTART",
"non_reserved_word : COLLATION",
"non_reserved_word : RETURNING",
"non_reserved_word : KW_IGNORE",
"non_reserved_word : LIMBO",
"non_reserved_word : UNDO",
"non_reserved_word : REQUESTS",
"non_reserved_word : TIMEOUT",
"non_reserved_word : ABS",
"non_reserved_word : ACCENT",
"non_reserved_word : ACOS",
"non_reserved_word : ALWAYS",
"non_reserved_word : ASCII_CHAR",
"non_reserved_word : ASCII_VAL",
"non_reserved_word : ASIN",
"non_reserved_word : ATAN",
"non_reserved_word : ATAN2",
"non_reserved_word : BIN_AND",
"non_reserved_word : BIN_OR",
"non_reserved_word : BIN_SHL",
"non_reserved_word : BIN_SHR",
"non_reserved_word : BIN_XOR",
"non_reserved_word : CEIL",
"non_reserved_word : COS",
"non_reserved_word : COSH",
"non_reserved_word : COT",
"non_reserved_word : DATEADD",
"non_reserved_word : DATEDIFF",
"non_reserved_word : DECODE",
"non_reserved_word : EXP",
"non_reserved_word : FLOOR",
"non_reserved_word : GEN_UUID",
"non_reserved_word : GENERATED",
"non_reserved_word : HASH",
"non_reserved_word : LIST",
"non_reserved_word : LN",
"non_reserved_word : LOG",
"non_reserved_word : LOG10",
"non_reserved_word : LPAD",
"non_reserved_word : MATCHED",
"non_reserved_word : MATCHING",
"non_reserved_word : MAXVALUE",
"non_reserved_word : MILLISECOND",
"non_reserved_word : MINVALUE",
"non_reserved_word : MOD",
"non_reserved_word : OVERLAY",
"non_reserved_word : PAD",
"non_reserved_word : PI",
"non_reserved_word : PLACING",
"non_reserved_word : POWER",
"non_reserved_word : PRESERVE",
"non_reserved_word : RAND",
"non_reserved_word : REPLACE",
"non_reserved_word : REVERSE",
"non_reserved_word : ROUND",
"non_reserved_word : RPAD",
"non_reserved_word : SIGN",
"non_reserved_word : SIN",
"non_reserved_word : SINH",
"non_reserved_word : SPACE",
"non_reserved_word : SQRT",
"non_reserved_word : TAN",
"non_reserved_word : TANH",
"non_reserved_word : TEMPORARY",
"non_reserved_word : TRUNC",
"non_reserved_word : WEEK",
"non_reserved_word : AUTONOMOUS",
"non_reserved_word : CHAR_TO_UUID",
"non_reserved_word : FIRSTNAME",
"non_reserved_word : MIDDLENAME",
"non_reserved_word : LASTNAME",
"non_reserved_word : MAPPING",
"non_reserved_word : OS_NAME",
"non_reserved_word : UUID_TO_CHAR",
"non_reserved_word : GRANTED",
"non_reserved_word : CALLER",
"non_reserved_word : COMMON",
"non_reserved_word : DATA",
"non_reserved_word : SOURCE",
"non_reserved_word : TWO_PHASE",
"non_reserved_word : BIN_NOT",
"non_reserved_word : ACTIVE",
"non_reserved_word : AFTER",
"non_reserved_word : ASC",
"non_reserved_word : AUTO",
"non_reserved_word : BEFORE",
"non_reserved_word : COMMITTED",
"non_reserved_word : COMPUTED",
"non_reserved_word : CONDITIONAL",
"non_reserved_word : CONTAINING",
"non_reserved_word : CSTRING",
"non_reserved_word : DATABASE",
"non_reserved_word : DESC",
"non_reserved_word : DO",
"non_reserved_word : KW_DOMAIN",
"non_reserved_word : ENTRY_POINT",
"non_reserved_word : EXCEPTION",
"non_reserved_word : EXIT",
"non_reserved_word : KW_FILE",
"non_reserved_word : GENERATOR",
"non_reserved_word : GEN_ID",
"non_reserved_word : IF",
"non_reserved_word : INACTIVE",
"non_reserved_word : INPUT_TYPE",
"non_reserved_word : ISOLATION",
"non_reserved_word : KEY",
"non_reserved_word : LENGTH",
"non_reserved_word : LEVEL",
"non_reserved_word : MANUAL",
"non_reserved_word : MODULE_NAME",
"non_reserved_word : NAMES",
"non_reserved_word : OPTION",
"non_reserved_word : OUTPUT_TYPE",
"non_reserved_word : KW_OVERFLOW",
"non_reserved_word : PAGE",
"non_reserved_word : PAGES",
"non_reserved_word : KW_PAGE_SIZE",
"non_reserved_word : PASSWORD",
"non_reserved_word : PRIVILEGES",
"non_reserved_word : PROTECTED",
"non_reserved_word : READ",
"non_reserved_word : RESERVING",
"non_reserved_word : RETAIN",
"non_reserved_word : SEGMENT",
"non_reserved_word : SHADOW",
"non_reserved_word : KW_SHARED",
"non_reserved_word : SINGULAR",
"non_reserved_word : KW_SIZE",
"non_reserved_word : SNAPSHOT",
"non_reserved_word : SORT",
"non_reserved_word : STABILITY",
"non_reserved_word : STARTING",
"non_reserved_word : STATISTICS",
"non_reserved_word : SUB_TYPE",
"non_reserved_word : SUSPEND",
"non_reserved_word : TRANSACTION",
"non_reserved_word : UNCOMMITTED",
"non_reserved_word : WAIT",
"non_reserved_word : WORK",
"non_reserved_word : WRITE",
"non_reserved_word : KW_ABSOLUTE",
"non_reserved_word : ACOSH",
"non_reserved_word : ASINH",
"non_reserved_word : ATANH",
"non_reserved_word : BODY",
"non_reserved_word : CONTINUE",
"non_reserved_word : DDL",
"non_reserved_word : DECRYPT",
"non_reserved_word : ENCRYPT",
"non_reserved_word : ENGINE",
"non_reserved_word : IDENTITY",
"non_reserved_word : NAME",
"non_reserved_word : PACKAGE",
"non_reserved_word : PARTITION",
"non_reserved_word : PRIOR",
"non_reserved_word : RDB_GET_CONTEXT",
"non_reserved_word : RDB_SET_CONTEXT",
"non_reserved_word : KW_RELATIVE",
"non_reserved_word : DENSE_RANK",
"non_reserved_word : FIRST_VALUE",
"non_reserved_word : NTH_VALUE",
"non_reserved_word : LAST_VALUE",
"non_reserved_word : LAG",
"non_reserved_word : LEAD",
"non_reserved_word : RANK",
"non_reserved_word : ROW_NUMBER",
"non_reserved_word : USAGE",
"non_reserved_word : LINGER",
"non_reserved_word : TAGS",
"non_reserved_word : PLUGIN",
"non_reserved_word : SERVERWIDE",
"non_reserved_word : INCREMENT",
"non_reserved_word : TRUSTED",
};
#endif

#ifdef YYREDUCEPOSNFUNC
#define YYCALLREDUCEPOSN(e)   \
	if (reduce_posn) {     \
	  YYREDUCEPOSNFUNC(yyps->pos, &(yyps->psp)[1 - yym], &(yyps->vsp)[1 - yym], \
			   yym, yyps->psp - yyps->ps, yychar, yyposn, e);       \
	  reduce_posn = 0;    \
	}

#ifndef YYCALLREDUCEPOSNARG
#define YYCALLREDUCEPOSNARG yyps->val
#endif


#define YYPOSNARG(n) ((yyps->psp)[1 - yym + (n) - 1])
#define YYPOSNOUT    (yyps->pos)
#endif

// If delete function is not defined by the user, do not deletions.
#ifndef YYDELETEVAL
#define YYDELETEVAL(v, n) 
#endif

// If delete function is not defined by the user, do not deletions.
#ifndef YYDELETEPOSN
#define YYDELETEPOSN(v, n) 
#endif

#define yyclearin (yychar = (-1))

#define yyerrok (yyps->errflag = 0)

#ifndef YYSTACKGROWTH
#define YYSTACKGROWTH 16
#endif

#ifndef YYDEFSTACKSIZE
#define YYDEFSTACKSIZE 12
#endif

#ifdef YYDEBUG
int yydebug;
#endif


#define YYABORT  goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR  goto yyerrlab
#define YYVALID         do { if (yyps->save)          goto yyvalid; } while(0)
#define YYVALID_NESTED  do { if (yyps->save && \
                                 yyps->save->save == 0) goto yyvalid; } while(0)
//
// For use in generated program
//
#define yytrial (yyps->save)
#define yyvsp   (yyps->vsp)
#define yyval   (yyps->val)
#define yydepth (yyps->ssp - yyps->ss)





void Parser::yySCopy(YYSTYPE* to, YYSTYPE* from, int size)
{
  int i;                             
  for (i = size - 1; i >= 0; i--) {
    to[i] = from[i];
  }
}


void Parser::yyPCopy(YYPOSN* to, YYPOSN* from, int size)
{
  int i;                             
  for (i = size - 1; i >= 0; i--) {
    to[i] = from[i];
  }
}


void Parser::yyMoreStack(yyparsestate* yyps)
{
  int p = yyps->ssp - yyps->ss;                               
  Yshort  *tss = yyps->ss;
  YYSTYPE *tvs = yyps->vs;
  YYPOSN  *tps = yyps->ps;
  yyps->ss = FB_NEW_POOL(getPool()) Yshort [yyps->stacksize + YYSTACKGROWTH];   
  yyps->vs = FB_NEW_POOL(getPool()) YYSTYPE[yyps->stacksize + YYSTACKGROWTH];  
  yyps->ps = FB_NEW_POOL(getPool()) YYPOSN [yyps->stacksize + YYSTACKGROWTH];  
  memcpy(yyps->ss, tss, yyps->stacksize * sizeof(Yshort));  
  yySCopy(yyps->vs, tvs, yyps->stacksize);
  yyPCopy(yyps->ps, tps, yyps->stacksize);
  yyps->stacksize += YYSTACKGROWTH;
  delete[] tss;
  delete[] tvs;
  delete[] tps;
  yyps->ssp = yyps->ss + p;                                   
  yyps->vsp = yyps->vs + p;                                   
  yyps->psp = yyps->ps + p;                                   
}


Parser::yyparsestate* Parser::yyNewState(int size)
{
  yyparsestate *p = FB_NEW_POOL(getPool()) yyparsestate;
  p->stacksize = size + 4;
  p->ss = FB_NEW_POOL(getPool()) Yshort [size + 4];
  p->vs = FB_NEW_POOL(getPool()) YYSTYPE[size + 4];
  p->ps = FB_NEW_POOL(getPool()) YYPOSN [size + 4];
  memset(&p->vs[0], 0, (size + 4) * sizeof(YYSTYPE));
  memset(&p->ps[0], 0, (size + 4) * sizeof(YYPOSN));
  return p;
}


void Parser::yyFreeState(Parser::yyparsestate* p)
{
  delete[] p->ss;
  delete[] p->vs;
  delete[] p->ps;
  delete p;
}


int Parser::parseAux()
{
  int yym, yyn, yystate, yychar, yynewerrflag;
  yyparsestate *yyerrctx = NULL;
  int reduce_posn;

#if YYDEBUG
  const char *yys;
  
  if ((yys = getenv("YYDEBUG"))) {
    yyn = *yys;
    if (yyn >= '0' && yyn <= '9')
      yydebug = yyn - '0'; 
  }
#endif
  
  yyps = yyNewState(YYDEFSTACKSIZE);
  yyps->save = 0;
  yynerrs = 0;
  yyps->errflag = 0;
  yychar = (-1);
  
  yyps->ssp = yyps->ss;
  yyps->vsp = yyps->vs;
  yyps->psp = yyps->ps;
  *(yyps->ssp) = yystate = 0;
  

  //
  // Main parsing loop
  //
 yyloop:
  if ((yyn = yydefred[yystate])) {
    goto yyreduce;
  }

  //
  // Read one token
  //
  if (yychar < 0) {
    if ((yychar = yylex1()) < 0) yychar = 0;
#if YYDEBUG
    if (yydebug) {
      yys = 0;
      if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
      if (!yys) yys = "illegal-symbol";
      printf("yydebug[%d,%p]: state %d, reading %d (%s)", 
	     int(yydepth), yytrial, yystate, yychar, yys);
#ifdef YYDBPR
      printf("<");
      YYDBPR(yylval);
      printf(">");
#endif
      printf("\n"); 
    }
#endif
  }

  //
  // Do we have a conflict?
  //
  if ((yyn = yycindex[yystate]) &&
      (yyn += yychar) >= 0 &&
      yyn <= YYTABLESIZE &&
      yycheck[yyn] == yychar) {
    int ctry;

    if (yypath) {
#if YYDEBUG
      if (yydebug) {
        printf("yydebug[%d,%p]: CONFLICT in state %d: following successful trial parse\n", 
	       int(yydepth), yytrial, yystate);
      }
#endif
      // Switch to the next conflict context
      yyparsestate *save = yypath;
      yypath = save->save;
      ctry = save->ctry;
      if (save->state != yystate) 
        goto yyabort;
      yyFreeState(save); 

    } else {

#if YYDEBUG
      if (yydebug) {
        printf("yydebug[%d,%p]: CONFLICT in state %d. ", 
	       int(yydepth), yytrial, yystate);
        if (yyps->save) {
          printf("ALREADY in conflict. Continue trial parse.");
        } else {
          printf("Start trial parse.");
        }
        printf("\n");
      }
#endif
      yyparsestate *save = yyNewState(yyps->ssp - yyps->ss);
      save->save    = yyps->save;
      save->state   = yystate;
      save->errflag = yyps->errflag;
      save->ssp     = save->ss + (yyps->ssp - yyps->ss);
      save->vsp     = save->vs + (yyps->vsp - yyps->vs);
      save->psp     = save->ps + (yyps->psp - yyps->ps);
      memcpy (save->ss, yyps->ss, (yyps->ssp - yyps->ss + 1) * sizeof(Yshort));
      yySCopy(save->vs, yyps->vs, (yyps->ssp - yyps->ss + 1));
      yyPCopy(save->ps, yyps->ps, (yyps->ssp - yyps->ss + 1));
      ctry = yytable[yyn];
      if (yyctable[ctry] == -1) {
#if YYDEBUG
        if (yydebug && yychar >= 0)
          printf("yydebug[%p]: backtracking 1 token\n", yytrial);
#endif
        ctry++; 
      }
      save->ctry = ctry;
      if (!yyps->save) {
        // If this is a first conflict in the stack, start saving lexemes
        if (!yylexemes) {
          yylexemes = FB_NEW_POOL(getPool()) Yshort[YYSTACKGROWTH];
          yylvals = FB_NEW_POOL(getPool()) YYSTYPE[YYSTACKGROWTH];
          yylvlim = yylvals + YYSTACKGROWTH; 
          yylpsns = FB_NEW_POOL(getPool()) YYPOSN[YYSTACKGROWTH];
          yylplim = yylpsns + YYSTACKGROWTH; 
        }
        if (yylvp == yylve) {
          yylvp = yylve = yylvals;
	  yylpp = yylpe = yylpsns;
          yylexp = yylexemes;
          if (yychar >= 0) {
            *yylve++ = yylval;
            *yylpe++ = yyposn;
            *yylexp = yychar;
            yychar = -1; 
          } 
        } 
      }
      if (yychar >= 0) {
        yylvp--, yylpp--, yylexp--;
        yychar = -1; 
      }
      save->lexeme = yylvp - yylvals;
      yyps->save = save; 
    }
    if (yytable[yyn] == ctry) {
#if YYDEBUG
      if (yydebug)
        printf("yydebug[%d,%p]: state %d, shifting to state %d\n",
               int(yydepth), yytrial, yystate, yyctable[ctry]);
#endif
      if (yychar < 0)
        yylvp++, yylpp++, yylexp++;
      yychar = -1;
      if (yyps->errflag > 0) --yyps->errflag;
      yystate = yyctable[ctry];
      goto yyshift; 
    } else {
      yyn = yyctable[ctry];
      goto yyreduce; 
    } 
  }

  //
  // Is action a shift?
  //
  if ((yyn = yysindex[yystate]) &&
      (yyn += yychar) >= 0 &&
      yyn <= YYTABLESIZE &&
      yycheck[yyn] == yychar) {
#if YYDEBUG
    if (yydebug)
      printf("yydebug[%d,%p]: state %d, shifting to state %d\n",
             int(yydepth), yytrial, yystate, yytable[yyn]);
#endif
    yychar = (-1);
    if (yyps->errflag > 0)  --yyps->errflag;
    yystate = yytable[yyn];
  yyshift:
    if (yyps->ssp >= yyps->ss + yyps->stacksize - 1) {
      yyMoreStack(yyps);
    }
    *++(yyps->ssp) = yystate;
    *++(yyps->vsp) = yylval;
    *++(yyps->psp) = yyposn;
    goto yyloop;
  }
  if ((yyn = yyrindex[yystate]) &&
      (yyn += yychar) >= 0 &&
      yyn <= YYTABLESIZE &&
      yycheck[yyn] == yychar) {
    yyn = yytable[yyn];
    goto yyreduce;
  }

  //
  // Action: error
  //
  if (yyps->errflag) goto yyinrecovery;
  yynewerrflag = 1;
  goto yyerrhandler;
yyerrlab:
  yynewerrflag = 0;
yyerrhandler:
  while (yyps->save) { 
    int ctry; 
    yyparsestate *save = yyps->save;
#if YYDEBUG
    if (yydebug)
      printf("yydebug[%d,%p]: ERROR in state %d, CONFLICT BACKTRACKING to state %d, %d tokens\n",
             int(yydepth), yytrial, yystate, yyps->save->state, int(yylvp - yylvals - yyps->save->lexeme));
#endif
    // Memorize most forward-looking error state in case
    // it's really an error.
    if (yyerrctx == NULL || yyerrctx->lexeme < yylvp-yylvals) {
      // Free old saved error context state
      if (yyerrctx) yyFreeState(yyerrctx);
      // Create and fill out new saved error context state
      yyerrctx = yyNewState(yyps->ssp - yyps->ss);
      yyerrctx->save = yyps->save;
      yyerrctx->state = yystate;
      yyerrctx->errflag = yyps->errflag;
      yyerrctx->ssp = yyerrctx->ss + (yyps->ssp - yyps->ss);
      yyerrctx->vsp = yyerrctx->vs + (yyps->vsp - yyps->vs);
      yyerrctx->psp = yyerrctx->ps + (yyps->psp - yyps->ps);
      memcpy (yyerrctx->ss, yyps->ss, (yyps->ssp - yyps->ss + 1) * sizeof(Yshort));
      yySCopy(yyerrctx->vs, yyps->vs, (yyps->ssp - yyps->ss + 1));
      yyPCopy(yyerrctx->ps, yyps->ps, (yyps->ssp - yyps->ss + 1));
      yyerrctx->lexeme = yylvp - yylvals;
    }
    yylvp  = yylvals   + save->lexeme;
    yylpp  = yylpsns   + save->lexeme;
    yylexp = yylexemes + save->lexeme;
    yychar = -1;
    yyps->ssp = yyps->ss + (save->ssp - save->ss);
    yyps->vsp = yyps->vs + (save->vsp - save->vs);
    yyps->psp = yyps->ps + (save->psp - save->ps);
    memcpy (yyps->ss, save->ss, (yyps->ssp - yyps->ss + 1) * sizeof(Yshort));
    yySCopy(yyps->vs, save->vs,  yyps->vsp - yyps->vs + 1);
    yyPCopy(yyps->ps, save->ps,  yyps->psp - yyps->ps + 1);
    ctry = ++save->ctry;
    yystate = save->state;
    // We tried shift, try reduce now
    if ((yyn = yyctable[ctry]) >= 0) {
      goto yyreduce;
    }
    yyps->save = save->save;
    yyFreeState(save);
    //
    // Nothing left on the stack -- error
    //
    if (!yyps->save) {
#if YYDEBUG
      if (yydebug) {
        printf("yydebug[%p]: trial parse FAILED, entering ERROR mode\n", 
	       yytrial);
      }
#endif
      // Restore state as it was in the most forward-advanced error
      yylvp  = yylvals   + yyerrctx->lexeme;
      yylpp  = yylpsns   + yyerrctx->lexeme;
      yylexp = yylexemes + yyerrctx->lexeme;
      yychar = yylexp[-1];
      yylval = yylvp[-1];
      yyposn = yylpp[-1];
      yyps->ssp = yyps->ss + (yyerrctx->ssp - yyerrctx->ss);
      yyps->vsp = yyps->vs + (yyerrctx->vsp - yyerrctx->vs);
      yyps->psp = yyps->ps + (yyerrctx->psp - yyerrctx->ps);
      memcpy (yyps->ss, yyerrctx->ss, (yyps->ssp - yyps->ss + 1) * sizeof(Yshort));
      yySCopy(yyps->vs, yyerrctx->vs,  yyps->vsp - yyps->vs + 1);
      yyPCopy(yyps->ps, yyerrctx->ps,  yyps->psp - yyps->ps + 1);
      yystate = yyerrctx->state;
      yyFreeState(yyerrctx);
      yyerrctx = NULL;
    }
    yynewerrflag = 1; 
  }
  if (yynewerrflag) {
#ifdef YYERROR_DETAILED
    yyerror_detailed("syntax error", yychar, yylval, yyposn);
#else
    yyerror("syntax error");
#endif
  }
  ++yynerrs;
 yyinrecovery:
  if (yyps->errflag < 3) {
    yyps->errflag = 3;
    for (;;) {
      if ((yyn = yysindex[*(yyps->ssp)]) && 
	  (yyn += YYERRCODE) >= 0 &&
          yyn <= YYTABLESIZE && 
	  yycheck[yyn] == YYERRCODE) {
#if YYDEBUG
        if (yydebug)
          printf("yydebug[%d,%p]: state %d, ERROR recovery shifts to state %d\n",
                 int(yydepth), yytrial, *(yyps->ssp), yytable[yyn]);
#endif
        /* Use label yyerrlab, so that compiler does not warn */
        if (yyps->errflag != yyps->errflag) goto yyerrlab;
        yystate = yytable[yyn];
        goto yyshift; 
      } else {
#if YYDEBUG
        if (yydebug)
          printf("yydebug[%d,%p]: ERROR recovery discards state %d\n",
                 int(yydepth), yytrial, *(yyps->ssp));
#endif
        if (yyps->ssp <= yyps->ss) {
	  goto yyabort;
	}
	if (!yytrial) {
	  YYDELETEVAL(yyps->vsp[0], 1);
	  YYDELETEPOSN(yyps->psp[0], 1);
	}
        --(yyps->ssp);
        --(yyps->vsp);
        --(yyps->psp);
      }
    }
  } else {
    if (yychar == 0) goto yyabort;
#if YYDEBUG
    if (yydebug) {
      yys = 0;
      if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
      if (!yys) yys = "illegal-symbol";
      printf("yydebug[%d,%p]: state %d, ERROR recovery discards token %d (%s)\n",
             int(yydepth), yytrial, yystate, yychar, yys); 
    }
#endif
    if (!yytrial) {
      YYDELETEVAL(yylval, 0);
      YYDELETEPOSN(yyposn, 0);
    }
    yychar = (-1);
    goto yyloop;
  }

  //
  // Reduce the rule
  //
yyreduce:
  yym = yylen[yyn];
#if YYDEBUG
  if (yydebug) {
    printf("yydebug[%d,%p]: state %d, reducing by rule %d (%s)",
           int(yydepth), yytrial, yystate, yyn, yyrule[yyn]);
#ifdef YYDBPR
    if (yym) {
      int i;
      printf("<");
      for (i = yym; i > 0; i--) {
        if (i != yym) printf(", ");
        YYDBPR((yyps->vsp)[1 - i]);
      }
      printf(">");
    }
#endif
    printf("\n");
  }
#endif
  if (yyps->ssp + 1 - yym >= yyps->ss + yyps->stacksize) {
    yyMoreStack(yyps);
  }

  /* default action - assign last argument as in standard yacc */
  yyps->val = yyvsp[1 - yym];

  /* default reduced position is NULL -- no position at all.
     no position will be assigned at trial time and if no position handling is present */
  memset(&yyps->pos, 0, sizeof(yyps->pos));

  reduce_posn = TRUE;

  yyposn.firstLine = YYPOSNARG(1).firstLine; 
  yyposn.firstColumn = YYPOSNARG(1).firstColumn; 

  switch (yyn) {

case 1:
  if (!yytrial)
{ DSQL_parse = yyvsp[0].dsqlReq; }
break;
case 2:
  if (!yytrial)
{ DSQL_parse = yyvsp[-1].dsqlReq; }
break;
case 3:
  if (!yytrial)
{ yyval.dsqlReq = newNode<DsqlDmlRequest>(yyvsp[0].stmtNode); }
break;
case 4:
  if (!yytrial)
{ yyval.dsqlReq = newNode<DsqlDdlRequest>(yyvsp[0].ddlNode); }
break;
case 5:
  if (!yytrial)
{ yyval.dsqlReq = newNode<DsqlTransactionRequest>(yyvsp[0].traNode); }
break;
case 6:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].stmtNode; }
break;
case 7:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].storeNode; }
break;
case 8:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].mergeNode; }
break;
case 9:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].stmtNode; }
break;
case 10:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].execBlockNode; }
break;
case 11:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].stmtNode; }
break;
case 12:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].selectNode; }
break;
case 13:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].stmtNode; }
break;
case 14:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].updInsNode; }
break;
case 15:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 16:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 17:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 18:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 19:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 20:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 21:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].grantRevokeNode; }
break;
case 22:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 23:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].grantRevokeNode; }
break;
case 24:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 25:
  if (!yytrial)
{ yyval.traNode = yyvsp[0].setTransactionNode; }
break;
case 26:
  if (!yytrial)
{ yyval.traNode = yyvsp[0].traNode; }
break;
case 27:
  if (!yytrial)
{ yyval.traNode = yyvsp[0].traNode; }
break;
case 28:
  if (!yytrial)
{ yyval.traNode = yyvsp[0].setRoleNode; }
break;
case 29:
  if (!yytrial)
{ yyval.grantRevokeNode = newNode<GrantRevokeNode>(true); }
break;
case 30:
yyval.grantRevokeNode = yyvsp[0].grantRevokeNode;
break;
case 31:
  if (!yytrial)
{ yyval.grantRevokeNode = yyvsp[-2].grantRevokeNode; }
break;
case 32:
yyval.privilegeArray = NOTRIAL(&yyvsp[0].grantRevokeNode->privileges);
break;
case 33:
yyval.granteeArray = NOTRIAL(&yyvsp[-6].grantRevokeNode->users);
break;
case 34:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_relation, *yyvsp[-5].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 35:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_procedure, *yyvsp[-5].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 36:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_udf, *yyvsp[-5].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 37:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_package_header, *yyvsp[-5].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 38:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_exception, *yyvsp[-5].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 39:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_generator, *yyvsp[-5].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 40:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_generator, *yyvsp[-5].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 41:
yyval.granteeArray = NOTRIAL(&yyvsp[-4].grantRevokeNode->users);
break;
case 42:
  if (!yytrial)
{ 
			yyvsp[-8].grantRevokeNode->object = yyvsp[-5].granteeClause;
			yyvsp[-8].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-8].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
			yyvsp[-8].grantRevokeNode->isDdl = true;
		}
break;
case 43:
  if (!yytrial)
{
			yyvsp[-8].grantRevokeNode->object = newNode<GranteeClause>(obj_database, get_object_name(obj_database));
			yyvsp[-8].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-8].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
			yyvsp[-8].grantRevokeNode->isDdl = true;
		}
break;
case 44:
yyval.granteeArray = NOTRIAL(&yyvsp[0].grantRevokeNode->roles);
break;
case 45:
yyval.granteeArray = NOTRIAL(&yyvsp[-3].grantRevokeNode->users);
break;
case 46:
  if (!yytrial)
{
			yyvsp[-7].grantRevokeNode->grantAdminOption = yyvsp[-1].boolVal;
			yyvsp[-7].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 47:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_relations, get_object_name(obj_relations)); }
break;
case 48:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_views, get_object_name(obj_views)); }
break;
case 49:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_procedures, get_object_name(obj_procedures)); }
break;
case 50:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_functions, get_object_name(obj_functions)); }
break;
case 51:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_packages, get_object_name(obj_packages)); }
break;
case 52:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_generators, get_object_name(obj_generators)); }
break;
case 53:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_generators, get_object_name(obj_generators)); }
break;
case 54:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_domains, get_object_name(obj_domains)); }
break;
case 55:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_exceptions, get_object_name(obj_exceptions)); }
break;
case 56:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_roles, get_object_name(obj_roles)); }
break;
case 57:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_charsets, get_object_name(obj_charsets)); }
break;
case 58:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_collations, get_object_name(obj_collations)); }
break;
case 59:
  if (!yytrial)
{ yyval.granteeClause = newNode<GranteeClause>(obj_filters, get_object_name(obj_filters)); }
break;
case 62:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('A', NULL)); }
break;
case 63:
  if (!yytrial)
{ yyvsp[-2].privilegeArray->add(PrivilegeClause('A', NULL)); }
break;
case 64:
yyval.privilegeArray = yyvsp[0].privilegeArray;
break;
case 67:
yyval.privilegeArray = yyvsp[-2].privilegeArray;
break;
case 69:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('X', NULL)); }
break;
case 70:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('G', NULL)); }
break;
case 71:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('S', NULL)); }
break;
case 72:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('I', NULL)); }
break;
case 73:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('D', NULL)); }
break;
case 74:
  if (!yytrial)
{ yyvsp[-2].privilegeArray->add(PrivilegeClause('U', yyvsp[0].valueListNode)); }
break;
case 75:
  if (!yytrial)
{ yyvsp[-2].privilegeArray->add(PrivilegeClause('R', yyvsp[0].valueListNode)); }
break;
case 76:
  if (!yytrial)
{
			yyvsp[-2].privilegeArray->add(PrivilegeClause('C', NULL));
			yyvsp[-2].privilegeArray->add(PrivilegeClause('L', NULL));
			yyvsp[-2].privilegeArray->add(PrivilegeClause('O', NULL));
		}
break;
case 82:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('C', NULL)); }
break;
case 83:
  if (!yytrial)
{ yyvsp[-2].privilegeArray->add(PrivilegeClause('L', NULL)); }
break;
case 84:
  if (!yytrial)
{ yyvsp[-2].privilegeArray->add(PrivilegeClause('O', NULL)); }
break;
case 85:
  if (!yytrial)
{
			yyvsp[-2].privilegeArray->add(PrivilegeClause('L', NULL));
			yyvsp[-2].privilegeArray->add(PrivilegeClause('O', NULL));
		}
break;
case 89:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('C', NULL)); }
break;
case 90:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('L', NULL)); }
break;
case 91:
  if (!yytrial)
{ yyvsp[-1].privilegeArray->add(PrivilegeClause('O', NULL)); }
break;
case 92:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 93:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 94:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 95:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 96:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 97:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 101:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 102:
  if (!yytrial)
{ yyval.grantRevokeNode = newNode<GrantRevokeNode>(false); }
break;
case 103:
  if (!yytrial)
{ yyval.grantRevokeNode = yyvsp[-2].grantRevokeNode; }
break;
case 104:
yyval.privilegeArray = NOTRIAL(&yyvsp[-1].grantRevokeNode->privileges);
break;
case 105:
yyval.granteeArray = NOTRIAL(&yyvsp[-7].grantRevokeNode->users);
break;
case 106:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_relation, *yyvsp[-4].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-9].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 107:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_procedure, *yyvsp[-4].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-9].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 108:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_udf, *yyvsp[-4].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-9].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 109:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_package_header, *yyvsp[-4].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-9].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 110:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_exception, *yyvsp[-4].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-9].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 111:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_generator, *yyvsp[-4].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-9].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 112:
  if (!yytrial)
{
			yyvsp[-10].grantRevokeNode->object = newNode<GranteeClause>(obj_generator, *yyvsp[-4].metaNamePtr);
			yyvsp[-10].grantRevokeNode->grantAdminOption = yyvsp[-9].boolVal;
			yyvsp[-10].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 113:
yyval.granteeArray = NOTRIAL(&yyvsp[-5].grantRevokeNode->users);
break;
case 114:
  if (!yytrial)
{
			yyvsp[-8].grantRevokeNode->object = yyvsp[-4].granteeClause;
			yyvsp[-8].grantRevokeNode->grantAdminOption = yyvsp[-7].boolVal;
			yyvsp[-8].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
			yyvsp[-8].grantRevokeNode->isDdl = true;
		}
break;
case 115:
  if (!yytrial)
{
			yyvsp[-8].grantRevokeNode->object = newNode<GranteeClause>(obj_database, get_object_name(obj_database));
			yyvsp[-8].grantRevokeNode->grantAdminOption = yyvsp[-7].boolVal;
			yyvsp[-8].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
			yyvsp[-8].grantRevokeNode->isDdl = true;
		}
break;
case 116:
yyval.granteeArray = NOTRIAL(&yyvsp[-1].grantRevokeNode->roles);
break;
case 117:
  if (!yytrial)
{
			yyvsp[-7].grantRevokeNode->grantAdminOption = yyvsp[-6].boolVal;
			yyvsp[-7].grantRevokeNode->grantor = yyvsp[0].metaNamePtr;
		}
break;
case 119:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 120:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 121:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 122:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 123:
yyval.granteeArray = yyvsp[0].granteeArray;
break;
case 126:
yyval.granteeArray = yyvsp[-2].granteeArray;
break;
case 129:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_procedure, *yyvsp[0].metaNamePtr)); }
break;
case 130:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_udf, *yyvsp[0].metaNamePtr)); }
break;
case 131:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_package_header, *yyvsp[0].metaNamePtr)); }
break;
case 132:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_trigger, *yyvsp[0].metaNamePtr)); }
break;
case 133:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_view, *yyvsp[0].metaNamePtr)); }
break;
case 134:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_sql_role, *yyvsp[0].metaNamePtr)); }
break;
case 135:
  if (!yytrial)
{ yyvsp[-1].granteeArray->add(GranteeClause(obj_user_or_role, *yyvsp[0].metaNamePtr)); }
break;
case 136:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_user, *yyvsp[0].metaNamePtr)); }
break;
case 137:
  if (!yytrial)
{ yyvsp[-2].granteeArray->add(GranteeClause(obj_user_group, *yyvsp[0].metaNamePtr)); }
break;
case 140:
  if (!yytrial)
{ yyvsp[-1].granteeArray->add(GranteeClause(obj_sql_role, *yyvsp[0].metaNamePtr)); }
break;
case 143:
  if (!yytrial)
{ yyvsp[-1].granteeArray->add(GranteeClause(obj_user, *yyvsp[0].metaNamePtr)); }
break;
case 144:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode;}
break;
case 145:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 146:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterFunctionNode; }
break;
case 147:
  if (!yytrial)
{ yyval.createAlterFunctionNode = newNode<CreateAlterFunctionNode>(*yyvsp[0].metaNamePtr); }
break;
case 148:
yyval.parametersClause = NOTRIAL(&yyvsp[0].createAlterFunctionNode->parameters);
break;
case 149:
yyval.createAlterFunctionNode = yyvsp[-3].createAlterFunctionNode;
break;
case 150:
  if (!yytrial)
{
				yyval.createAlterFunctionNode = yyvsp[-9].createAlterFunctionNode;
				yyval.createAlterFunctionNode->external = newNode<ExternalClause>();
				yyval.createAlterFunctionNode->external->name = *yyvsp[-2].stringPtr;
				yyval.createAlterFunctionNode->external->udfModule = *yyvsp[0].stringPtr;
			}
break;
case 152:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_blob;
			yyval.legacyField->length = sizeof(bid);
		}
break;
case 153:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_cstring;
			yyval.legacyField->charLength = (USHORT) yyvsp[-2].int32Val;
			if (yyvsp[0].metaNamePtr)
				yyval.legacyField->charSet = *yyvsp[0].metaNamePtr;
		}
break;
case 155:
yyval.parametersClause = yyvsp[0].parametersClause;
break;
case 157:
yyval.parametersClause = yyvsp[-1].parametersClause;
break;
case 160:
yyval.parametersClause = yyvsp[-2].parametersClause;
break;
case 162:
  if (!yytrial)
{
			yyvsp[-2].parametersClause->add(newNode<ParameterClause>(yyvsp[-1].legacyField, MetaName()));
			yyvsp[-2].parametersClause->back()->udfMechanism = yyvsp[0].nullableIntVal;
		}
break;
case 163:
  if (!yytrial)
{ yyval.nullableIntVal = Nullable<int>::empty(); }
break;
case 164:
  if (!yytrial)
{ yyval.nullableIntVal = Nullable<int>::val(FUN_descriptor); }
break;
case 165:
  if (!yytrial)
{ yyval.nullableIntVal = Nullable<int>::val(FUN_scalar_array); }
break;
case 166:
  if (!yytrial)
{ yyval.nullableIntVal = Nullable<int>::val(FUN_ref_with_null); }
break;
case 167:
yyval.createAlterFunctionNode = yyvsp[0].createAlterFunctionNode;
break;
case 169:
yyval.createAlterFunctionNode = yyvsp[-1].createAlterFunctionNode;
break;
case 171:
  if (!yytrial)
{
			yyvsp[-2].createAlterFunctionNode->returnType = newNode<ParameterClause>(yyvsp[-1].legacyField, MetaName());
			yyvsp[-2].createAlterFunctionNode->returnType->udfMechanism = yyvsp[0].int32Val;
		}
break;
case 172:
  if (!yytrial)
{ yyvsp[-2].createAlterFunctionNode->udfReturnPos = yyvsp[0].int32Val; }
break;
case 173:
  if (!yytrial)
{ yyval.int32Val = FUN_reference; }
break;
case 174:
  if (!yytrial)
{ yyval.int32Val = FUN_value; }
break;
case 175:
  if (!yytrial)
{ yyval.int32Val = FUN_descriptor; }
break;
case 176:
  if (!yytrial)
{ yyval.int32Val = -1 * FUN_reference; }
break;
case 177:
  if (!yytrial)
{ yyval.int32Val = -1 * FUN_descriptor; }
break;
case 178:
  if (!yytrial)
{
				CreateFilterNode* node = newNode<CreateFilterNode>(*yyvsp[-8].metaNamePtr);
				node->inputFilter = yyvsp[-6].filterNameNumber;
				node->outputFilter = yyvsp[-4].filterNameNumber;
				node->entryPoint = *yyvsp[-2].stringPtr;
				node->moduleName = *yyvsp[0].stringPtr;
				yyval.ddlNode = node;
			}
break;
case 179:
  if (!yytrial)
{ yyval.filterNameNumber = newNode<CreateFilterNode::NameNumber>(*yyvsp[0].metaNamePtr); }
break;
case 180:
  if (!yytrial)
{ yyval.filterNameNumber = newNode<CreateFilterNode::NameNumber>(yyvsp[0].int32Val); }
break;
case 181:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 182:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterExceptionNode; }
break;
case 183:
  if (!yytrial)
{
				CreateIndexNode* node = newNode<CreateIndexNode>(*yyvsp[-2].metaNamePtr);
				node->unique = yyvsp[-5].boolVal;
				node->descending = yyvsp[-4].boolVal;
				node->relation = yyvsp[0].relSourceNode;
				yyval.ddlNode = node;
			}
break;
case 184:
yyval.createIndexNode = static_cast<CreateIndexNode*>(yyvsp[0].ddlNode);
break;
case 185:
  if (!yytrial)
{
				yyval.ddlNode = yyvsp[-2].ddlNode;
			}
break;
case 186:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterFunctionNode; }
break;
case 187:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterProcedureNode; }
break;
case 188:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createRelationNode; }
break;
case 189:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createRelationNode; }
break;
case 190:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterTriggerNode; }
break;
case 191:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterViewNode; }
break;
case 192:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterSequenceNode; }
break;
case 193:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterSequenceNode; }
break;
case 194:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].alterDatabaseNode; }
break;
case 195:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createDomainNode; }
break;
case 196:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createShadowNode; }
break;
case 197:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 198:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createCollationNode; }
break;
case 199:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterUserNode; }
break;
case 200:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterPackageNode; }
break;
case 201:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createPackageBodyNode; }
break;
case 202:
yyval.boolVal = false;
break;
case 203:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 204:
yyval.boolVal = true;
break;
case 205:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 206:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 207:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateProcedureNode>(yyvsp[0].createAlterProcedureNode); }
break;
case 208:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateFunctionNode>(yyvsp[0].createAlterFunctionNode); }
break;
case 209:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateTableNode>(yyvsp[0].createRelationNode); }
break;
case 210:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateTableNode>(yyvsp[0].createRelationNode); }
break;
case 211:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateViewNode>(yyvsp[0].createAlterViewNode); }
break;
case 212:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateTriggerNode>(yyvsp[0].createAlterTriggerNode); }
break;
case 213:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreatePackageNode>(yyvsp[0].createAlterPackageNode); }
break;
case 214:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreatePackageBodyNode>(yyvsp[0].createPackageBodyNode); }
break;
case 215:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateExceptionNode>(yyvsp[0].createAlterExceptionNode); }
break;
case 216:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateSequenceNode>(yyvsp[0].createAlterSequenceNode); }
break;
case 217:
  if (!yytrial)
{ yyval.ddlNode = newNode<RecreateSequenceNode>(yyvsp[0].createAlterSequenceNode); }
break;
case 218:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 219:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterProcedureNode; }
break;
case 220:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterFunctionNode; }
break;
case 221:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterTriggerNode; }
break;
case 222:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterPackageNode; }
break;
case 223:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 224:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterExceptionNode; }
break;
case 225:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterSequenceNode; }
break;
case 226:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterSequenceNode; }
break;
case 227:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterUserNode; }
break;
case 228:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 229:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 230:
  if (!yytrial)
{ yyval.createAlterExceptionNode = newNode<CreateAlterExceptionNode>(*yyvsp[-1].metaNamePtr, yyvsp[0].intlStringPtr->getString()); }
break;
case 231:
  if (!yytrial)
{
			CreateAlterExceptionNode* node = newNode<CreateAlterExceptionNode>(*yyvsp[-1].metaNamePtr, yyvsp[0].intlStringPtr->getString());
			node->alter = true;
			yyval.createAlterExceptionNode = node;
		}
break;
case 232:
  if (!yytrial)
{
			CreateAlterExceptionNode* node = newNode<CreateAlterExceptionNode>(*yyvsp[-1].metaNamePtr, yyvsp[0].intlStringPtr->getString());
			node->create = false;
			node->alter = true;
			yyval.createAlterExceptionNode = node;
		}
break;
case 233:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 234:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 235:
  if (!yytrial)
{ yyvsp[-1].createIndexNode->columns = yyvsp[0].valueListNode; }
break;
case 236:
  if (!yytrial)
{ yyvsp[-1].createIndexNode->columns = yyvsp[0].valueListNode; }
break;
case 237:
  if (!yytrial)
{
 			yyvsp[-4].createIndexNode->computed = newNode<ValueSourceClause>();
			yyvsp[-4].createIndexNode->computed->value = yyvsp[-1].valueExprNode;
			yyvsp[-4].createIndexNode->computed->source = makeParseStr(YYPOSNARG(2), YYPOSNARG(4));
		}
break;
case 238:
  if (!yytrial)
{
	 			yyval.createShadowNode = newNode<CreateShadowNode>(yyvsp[-4].int32Val);
		 		yyval.createShadowNode->manual = yyvsp[-3].boolVal;
		 		yyval.createShadowNode->conditional = yyvsp[-2].boolVal;
		 		yyval.createShadowNode->files.add(newNode<DbFileClause>(*yyvsp[-1].stringPtr));
		 		yyval.createShadowNode->files.front()->length = yyvsp[0].int32Val;
	 		}
break;
case 239:
yyval.dbFilesClause = NOTRIAL(&yyvsp[0].createShadowNode->files);
break;
case 240:
  if (!yytrial)
{ yyval.createShadowNode = yyvsp[-2].createShadowNode; }
break;
case 241:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 242:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 243:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 244:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 245:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 246:
  if (!yytrial)
{ yyval.int32Val = 0; }
break;
case 247:
  if (!yytrial)
{ yyval.int32Val = yyvsp[-1].int32Val; }
break;
case 249:
yyval.dbFilesClause = yyvsp[0].dbFilesClause;
break;
case 251:
  if (!yytrial)
{ yyvsp[-1].dbFilesClause->add(yyvsp[0].dbFileClause); }
break;
case 252:
  if (!yytrial)
{ yyvsp[-2].dbFilesClause->add(yyvsp[0].dbFileClause); }
break;
case 253:
  if (!yytrial)
{
				yyvsp[-1].legacyField->fld_name = *yyvsp[-3].metaNamePtr;
				yyval.createDomainNode = newNode<CreateDomainNode>(
					newNode<ParameterClause>(yyvsp[-1].legacyField, MetaName(), yyvsp[0].valueSourceClause));
			}
break;
case 254:
yyval.createDomainNode = yyvsp[0].createDomainNode;
break;
case 255:
  if (!yytrial)
{
				yyval.createDomainNode = yyvsp[-3].createDomainNode;
				if (yyvsp[0].metaNamePtr)
					yyval.createDomainNode->nameType->type->collate = *yyvsp[0].metaNamePtr;
			}
break;
case 259:
yyval.createDomainNode = yyvsp[-1].createDomainNode;
break;
case 261:
  if (!yytrial)
{ setClause(yyvsp[-1].createDomainNode->notNull, "NOT NULL"); }
break;
case 262:
  if (!yytrial)
{ setClause(yyvsp[-1].createDomainNode->check, "DOMAIN CHECK CONSTRAINT", yyvsp[0].boolSourceClause); }
break;
case 265:
  if (!yytrial)
{
			ValueSourceClause* clause = newNode<ValueSourceClause>();
			clause->value = yyvsp[0].valueExprNode;
			clause->source = makeParseStr(YYPOSNARG(1), YYPOSNARG(2));
			yyval.valueSourceClause = clause;
		}
break;
case 266:
  if (!yytrial)
{ yyval.valueSourceClause = NULL; }
break;
case 269:
  if (!yytrial)
{
			BoolSourceClause* clause = newNode<BoolSourceClause>();
			clause->value = yyvsp[-1].boolExprNode;
			clause->source = makeParseStr(YYPOSNARG(1), YYPOSNARG(4));
			yyval.boolSourceClause = clause;
		}
break;
case 270:
  if (!yytrial)
{ yyval.createAlterSequenceNode = newNode<CreateAlterSequenceNode>(*yyvsp[-2].metaNamePtr, yyvsp[-1].nullableInt64Val, yyvsp[0].nullableInt32Val); }
break;
case 271:
  if (!yytrial)
{ yyval.nullableInt64Val = Nullable<SINT64>::empty(); }
break;
case 272:
  if (!yytrial)
{ yyval.nullableInt64Val = Nullable<SINT64>::val(yyvsp[0].int64Val); }
break;
case 273:
  if (!yytrial)
{ yyval.int64Val = yyvsp[0].int64Val; }
break;
case 274:
  if (!yytrial)
{ yyval.nullableInt32Val = Nullable<SLONG>::empty(); }
break;
case 275:
  if (!yytrial)
{ yyval.nullableInt32Val = Nullable<SLONG>::val(yyvsp[0].int32Val); }
break;
case 278:
  if (!yytrial)
{
			CreateAlterSequenceNode* node = newNode<CreateAlterSequenceNode>(*yyvsp[-2].metaNamePtr, yyvsp[-1].nullableInt64Val, yyvsp[0].nullableInt32Val);
			node->alter = true;
			node->restartSpecified = true;
			yyval.createAlterSequenceNode = node;
		}
break;
case 279:
  if (!yytrial)
{ yyval.nullableInt64Val = Nullable<SINT64>::empty(); }
break;
case 280:
  if (!yytrial)
{ yyval.nullableInt64Val = Nullable<SINT64>::val(yyvsp[0].int64Val); }
break;
case 281:
  if (!yytrial)
{
			if (!yyvsp[-1].nullableNullableInt64Val.specified && !yyvsp[0].nullableInt32Val.specified)
				yyerrorIncompleteCmd();
			CreateAlterSequenceNode* node = newNode<CreateAlterSequenceNode>(*yyvsp[-2].metaNamePtr, yyvsp[-1].nullableNullableInt64Val.value, yyvsp[0].nullableInt32Val);
			node->create = false;
			node->alter = true;
			node->restartSpecified = yyvsp[-1].nullableNullableInt64Val.specified;
			yyval.createAlterSequenceNode = node;
		}
break;
case 282:
  if (!yytrial)
{ yyval.nullableInt64Val = Nullable<SINT64>::empty(); }
break;
case 283:
  if (!yytrial)
{ yyval.nullableInt64Val = Nullable<SINT64>::val(yyvsp[0].int64Val); }
break;
case 284:
  if (!yytrial)
{ yyval.nullableNullableInt64Val = Nullable<BaseNullable<SINT64> >::empty(); }
break;
case 285:
  if (!yytrial)
{ yyval.nullableNullableInt64Val = Nullable<BaseNullable<SINT64> > ::val(yyvsp[0].nullableInt64Val); }
break;
case 286:
  if (!yytrial)
{
			CreateAlterSequenceNode* node = newNode<CreateAlterSequenceNode>(*yyvsp[-2].metaNamePtr,
				Nullable<SINT64>::val(yyvsp[0].int64Val), Nullable<SLONG>::empty());
			node->create = false;
			node->alter = true;
			node->legacy = true;
			node->restartSpecified = true;
			yyval.createAlterSequenceNode = node;
		}
break;
case 287:
  if (!yytrial)
{ yyval.int64Val = yyvsp[0].int32Val; }
break;
case 288:
  if (!yytrial)
{
			SINT64 signedNumber = (SINT64) yyvsp[0].scaledNumber.number;

			if (!yyvsp[0].scaledNumber.hex && yyvsp[0].scaledNumber.number > MAX_SINT64)
			{
				ERRD_post(
					Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
					Arg::Gds(isc_arith_except) <<
					Arg::Gds(isc_numeric_out_of_range));
			}

			yyval.int64Val = signedNumber;
		}
break;
case 289:
  if (!yytrial)
{
			SINT64 signedNumber = (SINT64) yyvsp[0].scaledNumber.number;

			if (yyvsp[0].scaledNumber.hex && signedNumber == MIN_SINT64)
				ERRD_post(Arg::Gds(isc_exception_integer_overflow));

			yyval.int64Val = -signedNumber;
		}
break;
case 290:
  if (!yytrial)
{ yyval.ddlNode = newNode<CreateRoleNode>(*yyvsp[0].metaNamePtr); }
break;
case 291:
  if (!yytrial)
{ yyval.createCollationNode = newNode<CreateCollationNode>(*yyvsp[-2].metaNamePtr, *yyvsp[0].metaNamePtr); }
break;
case 292:
yyval.createCollationNode = yyvsp[0].createCollationNode;
break;
case 293:
yyval.createCollationNode = yyvsp[-2].createCollationNode;
break;
case 294:
yyval.createCollationNode = yyvsp[-4].createCollationNode;
break;
case 295:
  if (!yytrial)
{ yyval.createCollationNode = yyvsp[-6].createCollationNode; }
break;
case 297:
  if (!yytrial)
{ yyvsp[-2].createCollationNode->fromName = *yyvsp[0].metaNamePtr; }
break;
case 298:
  if (!yytrial)
{ yyvsp[-5].createCollationNode->fromExternal = *yyvsp[-1].stringPtr; }
break;
case 302:
yyval.createCollationNode = yyvsp[-1].createCollationNode;
break;
case 307:
  if (!yytrial)
{ yyvsp[-2].createCollationNode->unsetAttribute(TEXTTYPE_ATTR_PAD_SPACE); }
break;
case 308:
  if (!yytrial)
{ yyvsp[-2].createCollationNode->setAttribute(TEXTTYPE_ATTR_PAD_SPACE); }
break;
case 309:
  if (!yytrial)
{ yyvsp[-2].createCollationNode->unsetAttribute(TEXTTYPE_ATTR_CASE_INSENSITIVE); }
break;
case 310:
  if (!yytrial)
{ yyvsp[-2].createCollationNode->setAttribute(TEXTTYPE_ATTR_CASE_INSENSITIVE); }
break;
case 311:
  if (!yytrial)
{ yyvsp[-2].createCollationNode->unsetAttribute(TEXTTYPE_ATTR_ACCENT_INSENSITIVE); }
break;
case 312:
  if (!yytrial)
{ yyvsp[-2].createCollationNode->setAttribute(TEXTTYPE_ATTR_ACCENT_INSENSITIVE); }
break;
case 314:
  if (!yytrial)
{
			const string& s = *yyvsp[0].stringPtr;
			yyvsp[-1].createCollationNode->specificAttributes.clear();
			yyvsp[-1].createCollationNode->specificAttributes.add((const UCHAR*) s.begin(), s.length());
		}
break;
case 315:
  if (!yytrial)
{ yyval.ddlNode = newNode<AlterCharSetNode>(*yyvsp[-4].metaNamePtr, *yyvsp[0].metaNamePtr); }
break;
case 316:
  if (!yytrial)
{
				yyval.alterDatabaseNode = newNode<AlterDatabaseNode>();
				yyval.alterDatabaseNode->create = true;
			}
break;
case 317:
yyval.alterDatabaseNode = yyvsp[0].alterDatabaseNode;
break;
case 318:
yyval.alterDatabaseNode = yyvsp[-2].alterDatabaseNode;
break;
case 319:
  if (!yytrial)
{ yyval.alterDatabaseNode = yyvsp[-4].alterDatabaseNode; }
break;
case 326:
yyval.alterDatabaseNode = yyvsp[-1].alterDatabaseNode;
break;
case 335:
  if (!yytrial)
{ yyvsp[-4].alterDatabaseNode->createLength = yyvsp[-1].int32Val; }
break;
case 340:
  if (!yytrial)
{ yyvsp[-1].alterDatabaseNode->files.add(yyvsp[0].dbFileClause); }
break;
case 341:
  if (!yytrial)
{ yyvsp[-4].alterDatabaseNode->setDefaultCharSet = *yyvsp[0].metaNamePtr; }
break;
case 342:
  if (!yytrial)
{
			yyvsp[-6].alterDatabaseNode->setDefaultCharSet = *yyvsp[-2].metaNamePtr;
			yyvsp[-6].alterDatabaseNode->setDefaultCollation = *yyvsp[0].metaNamePtr;
		}
break;
case 343:
  if (!yytrial)
{ yyvsp[-3].alterDatabaseNode->differenceFile = *yyvsp[0].stringPtr; }
break;
case 344:
  if (!yytrial)
{
				DbFileClause* clause = newNode<DbFileClause>(*yyvsp[0].stringPtr);
				yyval.dbFileClause = clause;
			}
break;
case 345:
yyval.dbFileClause = yyvsp[0].dbFileClause;
break;
case 346:
  if (!yytrial)
{ yyval.dbFileClause = yyvsp[-2].dbFileClause; }
break;
case 350:
yyval.dbFileClause = yyvsp[-1].dbFileClause;
break;
case 352:
  if (!yytrial)
{ yyvsp[-3].dbFileClause->start = yyvsp[0].int32Val; }
break;
case 353:
  if (!yytrial)
{ yyvsp[-4].dbFileClause->length = yyvsp[-1].int32Val; }
break;
case 360:
  if (!yytrial)
{ yyval.createRelationNode = newNode<CreateRelationNode>(yyvsp[-1].relSourceNode, yyvsp[0].stringPtr); }
break;
case 361:
yyval.createRelationNode = yyvsp[-1].createRelationNode;
break;
case 362:
  if (!yytrial)
{ yyval.createRelationNode = yyvsp[-4].createRelationNode; }
break;
case 363:
  if (!yytrial)
{ yyval.createRelationNode = newNode<CreateRelationNode>(yyvsp[0].relSourceNode); }
break;
case 364:
  if (!yytrial)
{
				yyval.createRelationNode = yyvsp[-5].createRelationNode;
				yyval.createRelationNode->relationType = static_cast<rel_t>(yyvsp[0].intVal);
			}
break;
case 365:
  if (!yytrial)
{ yyval.intVal = rel_global_temp_delete; }
break;
case 366:
  if (!yytrial)
{ yyval.intVal = rel_global_temp_delete; }
break;
case 367:
  if (!yytrial)
{ yyval.intVal = rel_global_temp_preserve; }
break;
case 368:
  if (!yytrial)
{ yyval.stringPtr = NULL; }
break;
case 369:
  if (!yytrial)
{ yyval.stringPtr = yyvsp[0].stringPtr; }
break;
case 370:
  if (!yytrial)
{ yyval.stringPtr = yyvsp[0].stringPtr; }
break;
case 371:
yyval.createRelationNode = yyvsp[0].createRelationNode;
break;
case 373:
yyval.createRelationNode = yyvsp[-2].createRelationNode;
break;
case 375:
yyval.relationNode = yyvsp[0].createRelationNode;
break;
case 378:
  if (!yytrial)
{
				RelationNode::AddColumnClause* clause = yyval.addColumnClause =
					newNode<RelationNode::AddColumnClause>();
				clause->field = yyvsp[-1].legacyField;
				clause->field->fld_name = *yyvsp[-2].metaNamePtr;
				clause->defaultValue = yyvsp[0].valueSourceClause;
				yyvsp[-3].relationNode->clauses.add(clause);
			}
break;
case 379:
yyval.addColumnClause = NOTRIAL(yyvsp[0].addColumnClause);
break;
case 380:
  if (!yytrial)
{
				if (yyvsp[0].metaNamePtr)
					yyvsp[-3].addColumnClause->collate = *yyvsp[0].metaNamePtr;
			}
break;
case 381:
  if (!yytrial)
{
				RelationNode::AddColumnClause* clause = yyval.addColumnClause =
					newNode<RelationNode::AddColumnClause>();
				clause->field = yyvsp[-1].legacyField;
				clause->field->fld_name = *yyvsp[-2].metaNamePtr;
				clause->identity = true;
				clause->identityStart = yyvsp[0].int64Val;
				yyvsp[-3].relationNode->clauses.add(clause);
			}
break;
case 382:
  if (!yytrial)
{
				if (yyvsp[0].metaNamePtr)
					yyvsp[-3].addColumnClause->collate = *yyvsp[0].metaNamePtr;
			}
break;
case 383:
  if (!yytrial)
{
			RelationNode::AddColumnClause* clause = newNode<RelationNode::AddColumnClause>();
			clause->field = yyvsp[-1].legacyField;
			clause->field->fld_name = *yyvsp[-2].metaNamePtr;
			clause->computed = yyvsp[0].valueSourceClause;
			yyvsp[-3].relationNode->clauses.add(clause);
			clause->field->flags |= FLD_computed;
		}
break;
case 384:
  if (!yytrial)
{
			RelationNode::AddColumnClause* clause = newNode<RelationNode::AddColumnClause>();
			clause->field = newNode<dsql_fld>();
			clause->field->fld_name = *yyvsp[-1].metaNamePtr;
			clause->computed = yyvsp[0].valueSourceClause;
			yyvsp[-2].relationNode->clauses.add(clause);
			clause->field->flags |= FLD_computed;
		}
break;
case 385:
  if (!yytrial)
{ yyval.int64Val = yyvsp[0].int64Val; }
break;
case 386:
  if (!yytrial)
{ yyval.int64Val = 0; }
break;
case 387:
  if (!yytrial)
{ yyval.int64Val = yyvsp[-1].int64Val; }
break;
case 388:
  if (!yytrial)
{
			ValueSourceClause* clause = newNode<ValueSourceClause>();
			clause->value = yyvsp[-1].valueExprNode;
			clause->source = makeParseStr(YYPOSNARG(2), YYPOSNARG(4));
			yyval.valueSourceClause = clause;
		}
break;
case 395:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
		}
break;
case 396:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 397:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 399:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
		}
break;
case 400:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfTable = *yyvsp[-2].metaNamePtr;
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
		}
break;
case 401:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
			yyval.legacyField->fullDomain = true;
		}
break;
case 402:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 403:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 404:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 405:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 406:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 407:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 409:
yyval.addColumnClause = yyvsp[0].addColumnClause;
break;
case 412:
yyval.addColumnClause = yyvsp[-1].addColumnClause;
break;
case 414:
  if (!yytrial)
{
			if (yyvsp[-2].metaNamePtr)
				yyvsp[-3].addColumnClause->constraints.back()->name = *yyvsp[-2].metaNamePtr;
		}
break;
case 415:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = yyvsp[-1].addColumnClause->constraints.add();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_NOT_NULL;
		}
break;
case 416:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = yyvsp[-1].addColumnClause->constraints.add();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_CHECK;
			constraint.check = yyvsp[0].boolSourceClause;
		}
break;
case 417:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = yyvsp[-5].addColumnClause->constraints.add();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_FK;

			constraint.columns.add(yyvsp[-5].addColumnClause->field->fld_name);
			constraint.refRelation = *yyvsp[-3].metaNamePtr;
			constraint.refAction = yyvsp[-1].refActionClause;

			const ValueListNode* refColumns = yyvsp[-2].valueListNode;
			if (refColumns)
			{
				const NestConst<ValueExprNode>* ptr = refColumns->items.begin();

				for (const NestConst<ValueExprNode>* const end = refColumns->items.end(); ptr != end; ++ptr)
					constraint.refColumns.add((*ptr)->as<FieldNode>()->dsqlName);
			}

			constraint.index = yyvsp[0].indexConstraintClause;
		}
break;
case 418:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = yyvsp[-2].addColumnClause->constraints.add();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_UNIQUE;
			constraint.index = yyvsp[0].indexConstraintClause;
		}
break;
case 419:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = yyvsp[-3].addColumnClause->constraints.add();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_PK;
			constraint.index = yyvsp[0].indexConstraintClause;
		}
break;
case 420:
yyval.relationNode = yyvsp[-1].relationNode;
break;
case 421:
  if (!yytrial)
{
			if (yyvsp[-2].metaNamePtr)
			{
				static_cast<RelationNode::AddConstraintClause*>(
					yyvsp[-3].relationNode->clauses.back().getObject())->name = *yyvsp[-2].metaNamePtr;
			}
		}
break;
case 422:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 423:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 424:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = *newNode<RelationNode::AddConstraintClause>();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_UNIQUE;

			const ValueListNode* columns = yyvsp[-1].valueListNode;
			const NestConst<ValueExprNode>* ptr = columns->items.begin();

			for (const NestConst<ValueExprNode>* const end = columns->items.end(); ptr != end; ++ptr)
				constraint.columns.add((*ptr)->as<FieldNode>()->dsqlName);

			constraint.index = yyvsp[0].indexConstraintClause;

			yyvsp[-3].relationNode->clauses.add(&constraint);
		}
break;
case 425:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = *newNode<RelationNode::AddConstraintClause>();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_PK;

			const ValueListNode* columns = yyvsp[-1].valueListNode;
			const NestConst<ValueExprNode>* ptr = columns->items.begin();

			for (const NestConst<ValueExprNode>* const end = columns->items.end(); ptr != end; ++ptr)
				constraint.columns.add((*ptr)->as<FieldNode>()->dsqlName);

			constraint.index = yyvsp[0].indexConstraintClause;

			yyvsp[-4].relationNode->clauses.add(&constraint);
		}
break;
case 426:
  if (!yytrial)
{
			RelationNode::AddConstraintClause& constraint = *newNode<RelationNode::AddConstraintClause>();
			constraint.constraintType = RelationNode::AddConstraintClause::CTYPE_FK;

			const ValueListNode* columns = yyvsp[-5].valueListNode;
			const NestConst<ValueExprNode>* ptr = columns->items.begin();

			for (const NestConst<ValueExprNode>* const end = columns->items.end(); ptr != end; ++ptr)
				constraint.columns.add((*ptr)->as<FieldNode>()->dsqlName);

			constraint.refRelation = *yyvsp[-3].metaNamePtr;
			constraint.refAction = yyvsp[-1].refActionClause;

			const ValueListNode* refColumns = yyvsp[-2].valueListNode;
			if (refColumns)
			{
				const NestConst<ValueExprNode>* ptr = refColumns->items.begin();

				for (const NestConst<ValueExprNode>* const end = refColumns->items.end(); ptr != end; ++ptr)
					constraint.refColumns.add((*ptr)->as<FieldNode>()->dsqlName);
			}

			constraint.index = yyvsp[0].indexConstraintClause;

			yyvsp[-8].relationNode->clauses.add(&constraint);
		}
break;
case 427:
  if (!yytrial)
{
			RelationNode::AddConstraintClause* constraint = newNode<RelationNode::AddConstraintClause>();
			constraint->constraintType = RelationNode::AddConstraintClause::CTYPE_CHECK;
			constraint->check = yyvsp[0].boolSourceClause;
			yyvsp[-1].relationNode->clauses.add(constraint);
		}
break;
case 428:
  if (!yytrial)
{ yyval.indexConstraintClause = newNode<RelationNode::IndexConstraintClause>(); }
break;
case 429:
  if (!yytrial)
{
			RelationNode::IndexConstraintClause* clause = yyval.indexConstraintClause =
				newNode<RelationNode::IndexConstraintClause>();
			clause->descending = yyvsp[-2].boolVal;
			clause->name = *yyvsp[0].metaNamePtr;
		}
break;
case 430:
  if (!yytrial)
{ yyval.refActionClause = NULL; }
break;
case 431:
  if (!yytrial)
{ yyval.refActionClause = newNode<RelationNode::RefActionClause>(yyvsp[0].uintVal, 0); }
break;
case 432:
  if (!yytrial)
{ yyval.refActionClause = newNode<RelationNode::RefActionClause>(0, yyvsp[0].uintVal); }
break;
case 433:
  if (!yytrial)
{ yyval.refActionClause = newNode<RelationNode::RefActionClause>(yyvsp[0].uintVal, yyvsp[-1].uintVal); }
break;
case 434:
  if (!yytrial)
{ yyval.refActionClause = newNode<RelationNode::RefActionClause>(yyvsp[-1].uintVal, yyvsp[0].uintVal); }
break;
case 435:
  if (!yytrial)
{ yyval.uintVal = yyvsp[0].uintVal;}
break;
case 436:
  if (!yytrial)
{ yyval.uintVal = yyvsp[0].uintVal;}
break;
case 437:
  if (!yytrial)
{ yyval.uintVal = RelationNode::RefActionClause::ACTION_CASCADE; }
break;
case 438:
  if (!yytrial)
{ yyval.uintVal = RelationNode::RefActionClause::ACTION_SET_DEFAULT; }
break;
case 439:
  if (!yytrial)
{ yyval.uintVal = RelationNode::RefActionClause::ACTION_SET_NULL; }
break;
case 440:
  if (!yytrial)
{ yyval.uintVal = RelationNode::RefActionClause::ACTION_NONE; }
break;
case 443:
  if (!yytrial)
{
			yyval.createAlterProcedureNode = yyvsp[-3].createAlterProcedureNode;
			yyval.createAlterProcedureNode->source = makeParseStr(YYPOSNARG(3), YYPOSNARG(4));
			yyval.createAlterProcedureNode->localDeclList = yyvsp[-1].compoundStmtNode;
			yyval.createAlterProcedureNode->body = yyvsp[0].stmtNode;
		}
break;
case 444:
  if (!yytrial)
{
			yyval.createAlterProcedureNode = yyvsp[-2].createAlterProcedureNode;
			yyval.createAlterProcedureNode->external = yyvsp[-1].externalClause;
			if (yyvsp[0].stringPtr)
				yyval.createAlterProcedureNode->source = *yyvsp[0].stringPtr;
		}
break;
case 445:
  if (!yytrial)
{ yyval.createAlterProcedureNode = newNode<CreateAlterProcedureNode>(*yyvsp[0].metaNamePtr); }
break;
case 446:
yyval.parametersClause = NOTRIAL(&yyvsp[0].createAlterProcedureNode->parameters);
break;
case 447:
yyval.parametersClause = NOTRIAL(&yyvsp[-2].createAlterProcedureNode->returns);
break;
case 448:
  if (!yytrial)
{ yyval.createAlterProcedureNode = yyvsp[-4].createAlterProcedureNode; }
break;
case 449:
  if (!yytrial)
{
			yyval.createAlterProcedureNode = yyvsp[0].createAlterProcedureNode;
			yyval.createAlterProcedureNode->alter = true;
			yyval.createAlterProcedureNode->create = false;
		}
break;
case 450:
  if (!yytrial)
{
			yyval.createAlterProcedureNode = yyvsp[0].createAlterProcedureNode;
			yyval.createAlterProcedureNode->alter = true;
		}
break;
case 458:
  if (!yytrial)
{ yyvsp[-3].parametersClause->add(newNode<ParameterClause>(yyvsp[-2].legacyField, optName(yyvsp[-1].metaNamePtr), yyvsp[0].valueSourceClause)); }
break;
case 461:
  if (!yytrial)
{ yyvsp[-2].parametersClause->add(newNode<ParameterClause>(yyvsp[-1].legacyField, optName(yyvsp[0].metaNamePtr))); }
break;
case 462:
  if (!yytrial)
{
			yyval.legacyField = yyvsp[0].legacyField;
			yyval.legacyField->fld_name = *yyvsp[-1].metaNamePtr;
		}
break;
case 463:
  if (!yytrial)
{ yyval.valueSourceClause = NULL; }
break;
case 464:
  if (!yytrial)
{
			ValueSourceClause* clause = newNode<ValueSourceClause>();
			clause->value = yyvsp[0].valueExprNode;
			clause->source = makeParseStr(YYPOSNARG(1), YYPOSNARG(2));
			yyval.valueSourceClause = clause;
		}
break;
case 465:
  if (!yytrial)
{
			ValueSourceClause* clause = newNode<ValueSourceClause>();
			clause->value = yyvsp[0].valueExprNode;
			clause->source = makeParseStr(YYPOSNARG(1), YYPOSNARG(2));
			yyval.valueSourceClause = clause;
		}
break;
case 468:
  if (!yytrial)
{
			yyval.createAlterFunctionNode = yyvsp[-3].createAlterFunctionNode;
			yyval.createAlterFunctionNode->source = makeParseStr(YYPOSNARG(3), YYPOSNARG(4));
			yyval.createAlterFunctionNode->localDeclList = yyvsp[-1].compoundStmtNode;
			yyval.createAlterFunctionNode->body = yyvsp[0].stmtNode;
		}
break;
case 469:
  if (!yytrial)
{
			yyval.createAlterFunctionNode = yyvsp[-2].createAlterFunctionNode;
			yyval.createAlterFunctionNode->external = yyvsp[-1].externalClause;
			if (yyvsp[0].stringPtr)
				yyval.createAlterFunctionNode->source = *yyvsp[0].stringPtr;
		}
break;
case 470:
  if (!yytrial)
{ yyval.createAlterFunctionNode = newNode<CreateAlterFunctionNode>(*yyvsp[0].metaNamePtr); }
break;
case 471:
  if (!yytrial)
{
				yyval.createAlterFunctionNode = yyvsp[-6].createAlterFunctionNode;
				yyval.createAlterFunctionNode->returnType = newNode<ParameterClause>(yyvsp[-2].legacyField, optName(yyvsp[-1].metaNamePtr));
				yyval.createAlterFunctionNode->deterministic = yyvsp[0].boolVal;
			}
break;
case 472:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 473:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 474:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 475:
  if (!yytrial)
{
			yyval.externalClause = newNode<ExternalClause>();
			yyval.externalClause->name = *yyvsp[-2].stringPtr;
			yyval.externalClause->engine = *yyvsp[0].metaNamePtr;
		}
break;
case 476:
  if (!yytrial)
{
			yyval.externalClause = newNode<ExternalClause>();
			yyval.externalClause->engine = *yyvsp[0].metaNamePtr;
		}
break;
case 477:
  if (!yytrial)
{ yyval.stringPtr = NULL; }
break;
case 478:
  if (!yytrial)
{ yyval.stringPtr = yyvsp[0].stringPtr; }
break;
case 479:
  if (!yytrial)
{
			yyval.createAlterFunctionNode = yyvsp[0].createAlterFunctionNode;
			yyval.createAlterFunctionNode->alter = true;
			yyval.createAlterFunctionNode->create = false;
		}
break;
case 480:
  if (!yytrial)
{
			yyval.createAlterFunctionNode = yyvsp[0].createAlterFunctionNode;
			yyval.createAlterFunctionNode->alter = true;
		}
break;
case 481:
  if (!yytrial)
{
			CreateAlterPackageNode* node = newNode<CreateAlterPackageNode>(*yyvsp[-4].metaNamePtr);
			node->source = makeParseStr(YYPOSNARG(3), YYPOSNARG(5));
			node->items = yyvsp[-1].packageItems;
			yyval.createAlterPackageNode = node;
		}
break;
case 483:
  if (!yytrial)
{ yyval.packageItems = newNode<Array<CreateAlterPackageNode::Item> >(); }
break;
case 484:
  if (!yytrial)
{
			yyval.packageItems = newNode<Array<CreateAlterPackageNode::Item> >();
			yyval.packageItems->add(yyvsp[0].packageItem);
		}
break;
case 485:
  if (!yytrial)
{
			yyval.packageItems = yyvsp[-1].packageItems;
			yyval.packageItems->add(yyvsp[0].packageItem);
		}
break;
case 486:
  if (!yytrial)
{ yyval.packageItem = CreateAlterPackageNode::Item::create(yyvsp[-1].createAlterFunctionNode); }
break;
case 487:
  if (!yytrial)
{ yyval.packageItem = CreateAlterPackageNode::Item::create(yyvsp[-1].createAlterProcedureNode); }
break;
case 488:
  if (!yytrial)
{
			yyval.createAlterPackageNode = yyvsp[0].createAlterPackageNode;
			yyval.createAlterPackageNode->alter = true;
			yyval.createAlterPackageNode->create = false;
		}
break;
case 489:
  if (!yytrial)
{
			yyval.createAlterPackageNode = yyvsp[0].createAlterPackageNode;
			yyval.createAlterPackageNode->alter = true;
		}
break;
case 490:
  if (!yytrial)
{
			CreatePackageBodyNode* node = newNode<CreatePackageBodyNode>(*yyvsp[-5].metaNamePtr);
			node->source = makeParseStr(YYPOSNARG(3), YYPOSNARG(6));
			node->declaredItems = yyvsp[-2].packageItems;
			node->items = yyvsp[-1].packageItems;
			yyval.createPackageBodyNode = node;
		}
break;
case 491:
  if (!yytrial)
{
			CreatePackageBodyNode* node = newNode<CreatePackageBodyNode>(*yyvsp[-4].metaNamePtr);
			node->source = makeParseStr(YYPOSNARG(3), YYPOSNARG(5));
			node->items = yyvsp[-1].packageItems;
			yyval.createPackageBodyNode = node;
		}
break;
case 492:
  if (!yytrial)
{ yyval.packageItems = newNode<Array<CreateAlterPackageNode::Item> >(); }
break;
case 494:
  if (!yytrial)
{
			yyval.packageItems = newNode<Array<CreateAlterPackageNode::Item> >();
			yyval.packageItems->add(yyvsp[0].packageItem);
		}
break;
case 495:
  if (!yytrial)
{
			yyval.packageItems = yyvsp[-1].packageItems;
			yyval.packageItems->add(yyvsp[0].packageItem);
		}
break;
case 496:
  if (!yytrial)
{ yyval.packageItem = CreateAlterPackageNode::Item::create(yyvsp[0].createAlterFunctionNode); }
break;
case 497:
  if (!yytrial)
{ yyval.packageItem = CreateAlterPackageNode::Item::create(yyvsp[-1].createAlterFunctionNode); }
break;
case 498:
  if (!yytrial)
{ yyval.packageItem = CreateAlterPackageNode::Item::create(yyvsp[0].createAlterProcedureNode); }
break;
case 499:
  if (!yytrial)
{ yyval.packageItem = CreateAlterPackageNode::Item::create(yyvsp[-1].createAlterProcedureNode); }
break;
case 500:
  if (!yytrial)
{ yyval.compoundStmtNode = NULL; }
break;
case 502:
  if (!yytrial)
{
			yyval.compoundStmtNode = newNode<CompoundStmtNode>();
			yyval.compoundStmtNode->statements.add(yyvsp[0].stmtNode);
		}
break;
case 503:
  if (!yytrial)
{
			yyvsp[-1].compoundStmtNode->statements.add(yyvsp[0].stmtNode);
			yyval.compoundStmtNode = yyvsp[-1].compoundStmtNode;
		}
break;
case 504:
  if (!yytrial)
{
			yyval.stmtNode = yyvsp[-1].stmtNode;
			yyval.stmtNode->line = YYPOSNARG(1).firstLine;
			yyval.stmtNode->column = YYPOSNARG(1).firstColumn;
		}
break;
case 505:
  if (!yytrial)
{ yyval.execBlockNode = newNode<ExecBlockNode>(); }
break;
case 506:
yyval.parametersClause = NOTRIAL(&yyvsp[0].execBlockNode->parameters);
break;
case 507:
yyval.parametersClause = NOTRIAL(&yyvsp[-2].execBlockNode->returns);
break;
case 508:
  if (!yytrial)
{
			DeclareSubProcNode* node = newNode<DeclareSubProcNode>(*yyvsp[-8].metaNamePtr);
			node->dsqlBlock = yyvsp[-7].execBlockNode;
			node->dsqlBlock->localDeclList = yyvsp[-1].compoundStmtNode;
			node->dsqlBlock->body = yyvsp[0].stmtNode;

			for (FB_SIZE_T i = 0; i < node->dsqlBlock->parameters.getCount(); ++i)
				node->dsqlBlock->parameters[i]->parameterExpr = make_parameter();

			yyval.stmtNode = node;
		}
break;
case 509:
  if (!yytrial)
{ yyval.execBlockNode = newNode<ExecBlockNode>(); }
break;
case 510:
  if (!yytrial)
{
			DeclareSubFuncNode* node = newNode<DeclareSubFuncNode>(*yyvsp[-10].metaNamePtr);
			node->dsqlDeterministic = yyvsp[-3].boolVal;
			node->dsqlBlock = yyvsp[-9].execBlockNode;
			node->dsqlBlock->localDeclList = yyvsp[-1].compoundStmtNode;
			node->dsqlBlock->body = yyvsp[0].stmtNode;

			for (FB_SIZE_T i = 0; i < node->dsqlBlock->parameters.getCount(); ++i)
				node->dsqlBlock->parameters[i]->parameterExpr = make_parameter();

			node->dsqlBlock->returns.add(newNode<ParameterClause>(yyvsp[-5].legacyField, optName(yyvsp[-4].metaNamePtr)));

			yyval.stmtNode = node;
		}
break;
case 513:
  if (!yytrial)
{
			DeclareVariableNode* node = newNode<DeclareVariableNode>();
			node->dsqlDef = newNode<ParameterClause>(yyvsp[-2].legacyField, optName(yyvsp[-1].metaNamePtr), yyvsp[0].valueSourceClause);
			yyval.stmtNode = node;
		}
break;
case 516:
  if (!yytrial)
{
			DeclareCursorNode* node = newNode<DeclareCursorNode>(*yyvsp[-6].metaNamePtr,
				DeclareCursorNode::CUR_TYPE_EXPLICIT);
			node->dsqlScroll = yyvsp[-5].boolVal;
			node->dsqlSelect = yyvsp[-1].selectNode;
			yyval.stmtNode = node;
		}
break;
case 517:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 518:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 519:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 522:
  if (!yytrial)
{ yyval.stmtNode = newNode<LineColumnNode>(YYPOSNARG(1).firstLine, YYPOSNARG(1).firstColumn, yyvsp[-1].stmtNode); }
break;
case 523:
  if (!yytrial)
{ yyval.stmtNode = newNode<CompoundStmtNode>(); }
break;
case 524:
  if (!yytrial)
{
			BlockNode* node = newNode<BlockNode>();
			node->action = yyvsp[0].compoundStmtNode;
			yyval.stmtNode = node;
		}
break;
case 525:
  if (!yytrial)
{
			BlockNode* node = newNode<BlockNode>();
			node->action = yyvsp[-1].compoundStmtNode;
			node->handlers = yyvsp[0].compoundStmtNode;
			yyval.stmtNode = node;
		}
break;
case 526:
  if (!yytrial)
{
			yyval.compoundStmtNode = newNode<CompoundStmtNode>();
			yyval.compoundStmtNode->statements.add(yyvsp[0].stmtNode);
		}
break;
case 527:
  if (!yytrial)
{
			yyvsp[-1].compoundStmtNode->statements.add(yyvsp[0].stmtNode);
			yyval.compoundStmtNode = yyvsp[-1].compoundStmtNode;
		}
break;
case 528:
  if (!yytrial)
{ yyval.stmtNode = newNode<LineColumnNode>(YYPOSNARG(1).firstLine, YYPOSNARG(1).firstColumn, yyvsp[-1].stmtNode); }
break;
case 529:
  if (!yytrial)
{ yyval.stmtNode = newNode<LineColumnNode>(YYPOSNARG(1).firstLine, YYPOSNARG(1).firstColumn, yyvsp[0].stmtNode); }
break;
case 531:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].storeNode; }
break;
case 532:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].mergeNode; }
break;
case 534:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].updInsNode; }
break;
case 538:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].execStatementNode; }
break;
case 539:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].execStatementNode; }
break;
case 541:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].stmtNode; }
break;
case 547:
  if (!yytrial)
{ yyval.stmtNode = newNode<SuspendNode>(); }
break;
case 548:
  if (!yytrial)
{ yyval.stmtNode = newNode<ExitNode>(); }
break;
case 549:
  if (!yytrial)
{ yyval.stmtNode = newNode<ReturnNode>(yyvsp[0].valueExprNode); }
break;
case 551:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].stmtNode; }
break;
case 555:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].forNode; }
break;
case 556:
  if (!yytrial)
{ yyval.stmtNode = yyvsp[0].execStatementNode; }
break;
case 557:
  if (!yytrial)
{
			InAutonomousTransactionNode* node = newNode<InAutonomousTransactionNode>();
			node->action = yyvsp[0].stmtNode;
			yyval.stmtNode = node;
		}
break;
case 558:
  if (!yytrial)
{ yyval.stmtNode = newNode<ExceptionNode>(*yyvsp[0].metaNamePtr); }
break;
case 559:
  if (!yytrial)
{ yyval.stmtNode = newNode<ExceptionNode>(*yyvsp[-1].metaNamePtr, yyvsp[0].valueExprNode); }
break;
case 560:
  if (!yytrial)
{ yyval.stmtNode = newNode<ExceptionNode>(*yyvsp[-4].metaNamePtr, (ValueExprNode*) NULL, yyvsp[-1].valueListNode); }
break;
case 561:
  if (!yytrial)
{ yyval.stmtNode = newNode<ExceptionNode>(); }
break;
case 562:
  if (!yytrial)
{
				ForNode* node = newNode<ForNode>();
				node->dsqlLabelName = yyvsp[-2].metaNamePtr;
				node->dsqlSelect = yyvsp[0].selectNode;
				yyval.forNode = node;
			}
break;
case 563:
yyval.forNode = yyvsp[0].forNode;
break;
case 564:
  if (!yytrial)
{
				ForNode* node = yyvsp[-4].forNode;
				node->statement = yyvsp[0].stmtNode;
				yyval.forNode = node;
			}
break;
case 565:
  if (!yytrial)
{
			yyvsp[-2].forNode->dsqlInto = yyvsp[-1].valueListNode;
			yyvsp[-2].forNode->dsqlCursor = yyvsp[0].declCursorNode;
		}
break;
case 566:
  if (!yytrial)
{
			yyvsp[-2].forNode->dsqlInto = yyvsp[-1].valueListNode;
			yyvsp[-2].forNode->dsqlCursor = yyvsp[0].declCursorNode;
		}
break;
case 567:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 569:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 570:
  if (!yytrial)
{ yyval.execStatementNode = newNode<ExecStatementNode>(); }
break;
case 571:
yyval.execStatementNode = yyvsp[0].execStatementNode;
break;
case 572:
yyval.execStatementNode = yyvsp[-2].execStatementNode;
break;
case 573:
  if (!yytrial)
{
			yyval.execStatementNode = yyvsp[-4].execStatementNode;
		}
break;
case 574:
  if (!yytrial)
{
			yyval.execStatementNode = yyvsp[-2].execStatementNode;
			yyval.execStatementNode->outputs = yyvsp[0].valueListNode;
		}
break;
case 575:
  if (!yytrial)
{
			yyval.execStatementNode = yyvsp[-2].execStatementNode;
			yyval.execStatementNode->dsqlLabelName = yyvsp[-4].metaNamePtr;
			yyval.execStatementNode->innerStmt = yyvsp[0].stmtNode;
		}
break;
case 576:
  if (!yytrial)
{ yyvsp[-1].execStatementNode->sql = yyvsp[0].valueExprNode; }
break;
case 577:
yyval.execStatementNode = yyvsp[-4].execStatementNode;
break;
case 578:
  if (!yytrial)
{ yyvsp[-7].execStatementNode->sql = yyvsp[-5].valueExprNode; }
break;
case 579:
  if (!yytrial)
{ yyvsp[-7].execStatementNode->sql = yyvsp[-5].valueExprNode; }
break;
case 582:
  if (!yytrial)
{
			if (!yyvsp[-3].execStatementNode->inputNames)
				yyvsp[-3].execStatementNode->inputNames = FB_NEW_POOL(getPool()) EDS::ParamNames(getPool());

			yyvsp[-3].execStatementNode->inputNames->add(yyvsp[-2].metaNamePtr);

			if (!yyvsp[-3].execStatementNode->inputs)
				yyvsp[-3].execStatementNode->inputs = newNode<ValueListNode>(yyvsp[0].valueExprNode);
			else
				yyvsp[-3].execStatementNode->inputs->add(yyvsp[0].valueExprNode);
		}
break;
case 585:
  if (!yytrial)
{
			if (!yyvsp[-1].execStatementNode->inputs)
				yyvsp[-1].execStatementNode->inputs = newNode<ValueListNode>(yyvsp[0].valueExprNode);
			else
				yyvsp[-1].execStatementNode->inputs->add(yyvsp[0].valueExprNode);
		}
break;
case 588:
yyval.execStatementNode = yyvsp[-1].execStatementNode;
break;
case 591:
  if (!yytrial)
{ setClause(yyvsp[-5].execStatementNode->dataSource, "EXTERNAL DATA SOURCE", yyvsp[0].valueExprNode); }
break;
case 592:
  if (!yytrial)
{ setClause(yyvsp[-3].execStatementNode->dataSource, "EXTERNAL DATA SOURCE", yyvsp[0].valueExprNode); }
break;
case 593:
  if (!yytrial)
{ setClause(yyvsp[-3].execStatementNode->userName, "USER", yyvsp[0].valueExprNode); }
break;
case 594:
  if (!yytrial)
{ setClause(yyvsp[-2].execStatementNode->password, "PASSWORD", yyvsp[0].valueExprNode); }
break;
case 595:
  if (!yytrial)
{ setClause(yyvsp[-2].execStatementNode->role, "ROLE", yyvsp[0].valueExprNode); }
break;
case 596:
  if (!yytrial)
{ setClause(yyvsp[-3].execStatementNode->traScope, "TRANSACTION", EDS::traAutonomous); }
break;
case 597:
  if (!yytrial)
{ setClause(yyvsp[-3].execStatementNode->traScope, "TRANSACTION", EDS::traCommon); }
break;
case 598:
  if (!yytrial)
{ setClause(yyvsp[-3].execStatementNode->useCallerPrivs, "CALLER PRIVILEGES"); }
break;
case 599:
  if (!yytrial)
{
			IfNode* node = newNode<IfNode>();
			node->condition = yyvsp[-5].boolExprNode;
			node->trueAction = yyvsp[-2].stmtNode;
			node->falseAction = yyvsp[0].stmtNode;
			yyval.stmtNode = node;
		}
break;
case 600:
  if (!yytrial)
{
			IfNode* node = newNode<IfNode>();
			node->condition = yyvsp[-3].boolExprNode;
			node->trueAction = yyvsp[0].stmtNode;
			yyval.stmtNode = node;
		}
break;
case 601:
  if (!yytrial)
{
			PostEventNode* node = newNode<PostEventNode>();
			node->event = yyvsp[-1].valueExprNode;
			node->argument = yyvsp[0].valueExprNode;
			yyval.stmtNode = node;
		}
break;
case 602:
  if (!yytrial)
{ yyval.valueExprNode = NULL; }
break;
case 603:
  if (!yytrial)
{
			ForNode* node = newNode<ForNode>();
			node->dsqlSelect = yyvsp[-2].selectNode;
			node->dsqlInto = yyvsp[0].valueListNode;
			yyval.stmtNode = node;
		}
break;
case 604:
  if (!yytrial)
{
			VariableNode* node = newNode<VariableNode>();
			node->dsqlName = *yyvsp[0].metaNamePtr;
			yyval.valueExprNode = node;
		}
break;
case 605:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode); }
break;
case 606:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].fieldNode); }
break;
case 607:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].fieldNode); }
break;
case 608:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].valueExprNode); }
break;
case 609:
  if (!yytrial)
{
			LoopNode* node = newNode<LoopNode>();
			node->dsqlLabelName = yyvsp[-6].metaNamePtr;
			node->dsqlExpr = yyvsp[-3].boolExprNode;
			node->statement = yyvsp[0].stmtNode;
			yyval.stmtNode = node;
		}
break;
case 610:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 611:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[-1].metaNamePtr; }
break;
case 612:
  if (!yytrial)
{ yyval.stmtNode = newNode<ContinueLeaveNode>(blr_leave); }
break;
case 613:
  if (!yytrial)
{
			ContinueLeaveNode* node = newNode<ContinueLeaveNode>(blr_leave);
			node->dsqlLabelName = yyvsp[0].metaNamePtr;
			yyval.stmtNode = node;
		}
break;
case 614:
  if (!yytrial)
{
			ContinueLeaveNode* node = newNode<ContinueLeaveNode>(blr_continue_loop);
			node->dsqlLabelName = yyvsp[0].metaNamePtr;
			yyval.stmtNode = node;
		}
break;
case 615:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 617:
  if (!yytrial)
{ yyval.declCursorNode = NULL; }
break;
case 619:
  if (!yytrial)
{ yyval.declCursorNode = newNode<DeclareCursorNode>(*yyvsp[0].metaNamePtr, DeclareCursorNode::CUR_TYPE_FOR); }
break;
case 620:
  if (!yytrial)
{
			yyval.compoundStmtNode = newNode<CompoundStmtNode>();
			yyval.compoundStmtNode->statements.add(yyvsp[0].stmtNode);
		}
break;
case 621:
  if (!yytrial)
{
			yyvsp[-1].compoundStmtNode->statements.add(yyvsp[0].stmtNode);
			yyval.compoundStmtNode = yyvsp[-1].compoundStmtNode;
		}
break;
case 622:
  if (!yytrial)
{ yyval.errorHandlerNode = newNode<ErrorHandlerNode>(); }
break;
case 623:
yyval.exceptionArray = NOTRIAL(&yyvsp[0].errorHandlerNode->conditions);
break;
case 624:
  if (!yytrial)
{
				ErrorHandlerNode* node = yyvsp[-4].errorHandlerNode;
				node->action = yyvsp[0].stmtNode;
				yyval.stmtNode = node;
			}
break;
case 625:
yyval.exceptionArray = yyvsp[0].exceptionArray;
break;
case 627:
yyval.exceptionArray = yyvsp[-2].exceptionArray;
break;
case 629:
  if (!yytrial)
{
			ExceptionItem& item = yyvsp[-2].exceptionArray->add();
			item.type = ExceptionItem::SQL_CODE;
			item.code = yyvsp[0].int32Val;
		}
break;
case 630:
  if (!yytrial)
{
			ExceptionItem& item = yyvsp[-2].exceptionArray->add();
			item.type = ExceptionItem::SQL_STATE;
			item.name = yyvsp[0].intlStringPtr->getString();
		}
break;
case 631:
  if (!yytrial)
{
			ExceptionItem& item = yyvsp[-2].exceptionArray->add();
			item.type = ExceptionItem::GDS_CODE;
			item.name = yyvsp[0].metaNamePtr->c_str();
		}
break;
case 632:
  if (!yytrial)
{
			ExceptionItem& item = yyvsp[-2].exceptionArray->add();
			item.type = ExceptionItem::XCP_CODE;
			item.name = yyvsp[0].metaNamePtr->c_str();
		}
break;
case 633:
  if (!yytrial)
{
			ExceptionItem& item = yyvsp[-1].exceptionArray->add();
			item.type = ExceptionItem::XCP_DEFAULT;
		}
break;
case 637:
  if (!yytrial)
{ yyval.stmtNode = newNode<CursorStmtNode>(blr_cursor_open, *yyvsp[0].metaNamePtr); }
break;
case 638:
  if (!yytrial)
{ yyval.stmtNode = newNode<CursorStmtNode>(blr_cursor_close, *yyvsp[0].metaNamePtr); }
break;
case 639:
  if (!yytrial)
{ yyval.cursorStmtNode = newNode<CursorStmtNode>(blr_cursor_fetch_scroll); }
break;
case 640:
yyval.cursorStmtNode = yyvsp[0].cursorStmtNode;
break;
case 641:
  if (!yytrial)
{
			CursorStmtNode* cursorStmt = yyvsp[-5].cursorStmtNode;
			cursorStmt->dsqlName = *yyvsp[-1].metaNamePtr;
			cursorStmt->dsqlIntoStmt = yyvsp[0].valueListNode;
			yyval.stmtNode = cursorStmt;
		}
break;
case 642:
  if (!yytrial)
{ yyval.stmtNode = newNode<CursorStmtNode>(blr_cursor_fetch, *yyvsp[-1].metaNamePtr, yyvsp[0].valueListNode); }
break;
case 643:
  if (!yytrial)
{ yyvsp[-1].cursorStmtNode->scrollOp = blr_scroll_bof; }
break;
case 644:
  if (!yytrial)
{ yyvsp[-1].cursorStmtNode->scrollOp = blr_scroll_eof; }
break;
case 645:
  if (!yytrial)
{ yyvsp[-1].cursorStmtNode->scrollOp = blr_scroll_backward; }
break;
case 646:
  if (!yytrial)
{ yyvsp[-1].cursorStmtNode->scrollOp = blr_scroll_forward; }
break;
case 647:
  if (!yytrial)
{
			yyvsp[-2].cursorStmtNode->scrollOp = blr_scroll_absolute;
			yyvsp[-2].cursorStmtNode->scrollExpr = yyvsp[0].valueExprNode;
		}
break;
case 648:
  if (!yytrial)
{
			yyvsp[-2].cursorStmtNode->scrollOp = blr_scroll_relative;
			yyvsp[-2].cursorStmtNode->scrollExpr = yyvsp[0].valueExprNode;
		}
break;
case 649:
  if (!yytrial)
{ yyval.stmtNode = newNode<ExecProcedureNode>(QualifiedName(*yyvsp[-2].metaNamePtr), yyvsp[-1].valueListNode, yyvsp[0].valueListNode); }
break;
case 650:
  if (!yytrial)
{ yyval.stmtNode = newNode<ExecProcedureNode>(QualifiedName(*yyvsp[-2].metaNamePtr, *yyvsp[-4].metaNamePtr), yyvsp[-1].valueListNode, yyvsp[0].valueListNode); }
break;
case 651:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 652:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 653:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-1].valueListNode; }
break;
case 654:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 655:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 656:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-1].valueListNode; }
break;
case 657:
  if (!yytrial)
{ yyval.execBlockNode = newNode<ExecBlockNode>(); }
break;
case 658:
  if (!yytrial)
{
			ExecBlockNode* node = yyvsp[-7].execBlockNode;
			node->localDeclList = yyvsp[-1].compoundStmtNode;
			node->body = yyvsp[0].stmtNode;
			yyval.execBlockNode = node;
		}
break;
case 663:
  if (!yytrial)
{ yyvsp[-4].parametersClause->add(newNode<ParameterClause>(yyvsp[-3].legacyField, optName(yyvsp[-2].metaNamePtr), (ValueSourceClause*) NULL, yyvsp[0].valueExprNode)); }
break;
case 664:
  if (!yytrial)
{
			CreateAlterViewNode* node = newNode<CreateAlterViewNode>(yyvsp[-4].relSourceNode, yyvsp[-3].valueListNode, yyvsp[-1].selectExprNode);
			node->source = makeParseStr(YYPOSNARG(4), YYPOSNARG(5));
			node->withCheckOption = yyvsp[0].boolVal;
			yyval.createAlterViewNode = node;
		}
break;
case 665:
  if (!yytrial)
{
			yyvsp[0].createAlterViewNode->alter = true;
			yyval.ddlNode = yyvsp[0].createAlterViewNode;
		}
break;
case 666:
  if (!yytrial)
{
			yyvsp[0].createAlterViewNode->alter = true;
			yyvsp[0].createAlterViewNode->create = false;
			yyval.ddlNode = yyvsp[0].createAlterViewNode;
		}
break;
case 667:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 668:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 669:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = newNode<CreateAlterTriggerNode>(*yyvsp[-6].metaNamePtr);
			yyval.createAlterTriggerNode->active = yyvsp[-5].nullableBoolVal;
			yyval.createAlterTriggerNode->type = yyvsp[-4].uint64Val;
			yyval.createAlterTriggerNode->position = yyvsp[-3].nullableIntVal;
			yyval.createAlterTriggerNode->source = makeParseStr(YYPOSNARG(5), YYPOSNARG(7));
			yyval.createAlterTriggerNode->localDeclList = yyvsp[-1].compoundStmtNode;
			yyval.createAlterTriggerNode->body = yyvsp[0].stmtNode;
		}
break;
case 670:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = newNode<CreateAlterTriggerNode>(*yyvsp[-5].metaNamePtr);
			yyval.createAlterTriggerNode->active = yyvsp[-4].nullableBoolVal;
			yyval.createAlterTriggerNode->type = yyvsp[-3].uint64Val;
			yyval.createAlterTriggerNode->position = yyvsp[-2].nullableIntVal;
			yyval.createAlterTriggerNode->external = yyvsp[-1].externalClause;
			if (yyvsp[0].stringPtr)
				yyval.createAlterTriggerNode->source = *yyvsp[0].stringPtr;
		}
break;
case 671:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = newNode<CreateAlterTriggerNode>(*yyvsp[-8].metaNamePtr);
			yyval.createAlterTriggerNode->active = yyvsp[-7].nullableBoolVal;
			yyval.createAlterTriggerNode->type = yyvsp[-6].uint64Val;
			yyval.createAlterTriggerNode->position = yyvsp[-5].nullableIntVal;
			yyval.createAlterTriggerNode->relationName = *yyvsp[-3].metaNamePtr;
			yyval.createAlterTriggerNode->source = makeParseStr(YYPOSNARG(7), YYPOSNARG(9));
			yyval.createAlterTriggerNode->localDeclList = yyvsp[-1].compoundStmtNode;
			yyval.createAlterTriggerNode->body = yyvsp[0].stmtNode;
		}
break;
case 672:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = newNode<CreateAlterTriggerNode>(*yyvsp[-7].metaNamePtr);
			yyval.createAlterTriggerNode->active = yyvsp[-6].nullableBoolVal;
			yyval.createAlterTriggerNode->type = yyvsp[-5].uint64Val;
			yyval.createAlterTriggerNode->position = yyvsp[-4].nullableIntVal;
			yyval.createAlterTriggerNode->relationName = *yyvsp[-2].metaNamePtr;
			yyval.createAlterTriggerNode->external = yyvsp[-1].externalClause;
			if (yyvsp[0].stringPtr)
				yyval.createAlterTriggerNode->source = *yyvsp[0].stringPtr;
		}
break;
case 673:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = newNode<CreateAlterTriggerNode>(*yyvsp[-8].metaNamePtr);
			yyval.createAlterTriggerNode->active = yyvsp[-5].nullableBoolVal;
			yyval.createAlterTriggerNode->type = yyvsp[-4].uint64Val;
			yyval.createAlterTriggerNode->position = yyvsp[-3].nullableIntVal;
			yyval.createAlterTriggerNode->relationName = *yyvsp[-6].metaNamePtr;
			yyval.createAlterTriggerNode->source = makeParseStr(YYPOSNARG(7), YYPOSNARG(9));
			yyval.createAlterTriggerNode->localDeclList = yyvsp[-1].compoundStmtNode;
			yyval.createAlterTriggerNode->body = yyvsp[0].stmtNode;
		}
break;
case 674:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = newNode<CreateAlterTriggerNode>(*yyvsp[-7].metaNamePtr);
			yyval.createAlterTriggerNode->active = yyvsp[-4].nullableBoolVal;
			yyval.createAlterTriggerNode->type = yyvsp[-3].uint64Val;
			yyval.createAlterTriggerNode->position = yyvsp[-2].nullableIntVal;
			yyval.createAlterTriggerNode->relationName = *yyvsp[-5].metaNamePtr;
			yyval.createAlterTriggerNode->external = yyvsp[-1].externalClause;
			if (yyvsp[0].stringPtr)
				yyval.createAlterTriggerNode->source = *yyvsp[0].stringPtr;
		}
break;
case 675:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = yyvsp[0].createAlterTriggerNode;
			yyval.createAlterTriggerNode->alter = true;
		}
break;
case 676:
  if (!yytrial)
{ yyval.nullableBoolVal = Nullable<bool>::val(true); }
break;
case 677:
  if (!yytrial)
{ yyval.nullableBoolVal = Nullable<bool>::val(false); }
break;
case 678:
  if (!yytrial)
{ yyval.nullableBoolVal = Nullable<bool>::empty(); }
break;
case 679:
  if (!yytrial)
{ yyval.uint64Val = yyvsp[-1].uint64Val + yyvsp[0].uint64Val - 1; }
break;
case 680:
  if (!yytrial)
{ yyval.uint64Val = yyvsp[0].uint64Val; }
break;
case 681:
  if (!yytrial)
{ yyval.uint64Val = yyvsp[-1].uint64Val + yyvsp[0].uint64Val; }
break;
case 682:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DB | DB_TRIGGER_CONNECT; }
break;
case 683:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DB | DB_TRIGGER_DISCONNECT; }
break;
case 684:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DB | DB_TRIGGER_TRANS_START; }
break;
case 685:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DB | DB_TRIGGER_TRANS_COMMIT; }
break;
case 686:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DB | DB_TRIGGER_TRANS_ROLLBACK; }
break;
case 688:
  if (!yytrial)
{
			yyval.uint64Val = TRIGGER_TYPE_DDL | DDL_TRIGGER_ANY;
		}
break;
case 689:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_TABLE); }
break;
case 690:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_TABLE); }
break;
case 691:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_TABLE); }
break;
case 692:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_PROCEDURE); }
break;
case 693:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_PROCEDURE); }
break;
case 694:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_PROCEDURE); }
break;
case 695:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_FUNCTION); }
break;
case 696:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_FUNCTION); }
break;
case 697:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_FUNCTION); }
break;
case 698:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_TRIGGER); }
break;
case 699:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_TRIGGER); }
break;
case 700:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_TRIGGER); }
break;
case 701:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_EXCEPTION); }
break;
case 702:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_EXCEPTION); }
break;
case 703:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_EXCEPTION); }
break;
case 704:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_VIEW); }
break;
case 705:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_VIEW); }
break;
case 706:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_VIEW); }
break;
case 707:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_DOMAIN); }
break;
case 708:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_DOMAIN); }
break;
case 709:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_DOMAIN); }
break;
case 710:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_ROLE); }
break;
case 711:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_ROLE); }
break;
case 712:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_ROLE); }
break;
case 713:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_SEQUENCE); }
break;
case 714:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_SEQUENCE); }
break;
case 715:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_SEQUENCE); }
break;
case 716:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_USER); }
break;
case 717:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_USER); }
break;
case 718:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_USER); }
break;
case 719:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_INDEX); }
break;
case 720:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_INDEX); }
break;
case 721:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_INDEX); }
break;
case 722:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_COLLATION); }
break;
case 723:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_COLLATION); }
break;
case 724:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_CHARACTER_SET); }
break;
case 725:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_PACKAGE); }
break;
case 726:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_PACKAGE); }
break;
case 727:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_PACKAGE); }
break;
case 728:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_PACKAGE_BODY); }
break;
case 729:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_PACKAGE_BODY); }
break;
case 730:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_CREATE_MAPPING); }
break;
case 731:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_ALTER_MAPPING); }
break;
case 732:
  if (!yytrial)
{ yyval.uint64Val = TRIGGER_TYPE_DDL | (1LL << DDL_TRIGGER_DROP_MAPPING); }
break;
case 733:
  if (!yytrial)
{ yyval.uint64Val = yyvsp[-2].uint64Val | yyvsp[0].uint64Val; }
break;
case 734:
  if (!yytrial)
{ yyval.uint64Val = 0; }
break;
case 735:
  if (!yytrial)
{ yyval.uint64Val = 1; }
break;
case 736:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(1, 0, 0); }
break;
case 737:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(2, 0, 0); }
break;
case 738:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(3, 0, 0); }
break;
case 739:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(1, 2, 0); }
break;
case 740:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(1, 3, 0); }
break;
case 741:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(2, 1, 0); }
break;
case 742:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(2, 3, 0); }
break;
case 743:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(3, 1, 0); }
break;
case 744:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(3, 2, 0); }
break;
case 745:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(1, 2, 3); }
break;
case 746:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(1, 3, 2); }
break;
case 747:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(2, 1, 3); }
break;
case 748:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(2, 3, 1); }
break;
case 749:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(3, 1, 2); }
break;
case 750:
  if (!yytrial)
{ yyval.uint64Val = trigger_type_suffix(3, 2, 1); }
break;
case 751:
  if (!yytrial)
{ yyval.nullableIntVal = Nullable<int>::empty(); }
break;
case 752:
  if (!yytrial)
{ yyval.nullableIntVal = Nullable<int>::val(yyvsp[0].int32Val); }
break;
case 753:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 754:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterSequenceNode; }
break;
case 755:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterExceptionNode; }
break;
case 756:
  if (!yytrial)
{ yyval.ddlNode = newNode<AlterRelationNode>(yyvsp[0].relSourceNode); }
break;
case 757:
yyval.relationNode = yyvsp[0].relationNode;
break;
case 758:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[-2].relationNode; }
break;
case 759:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 760:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterTriggerNode; }
break;
case 761:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterProcedureNode; }
break;
case 762:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterPackageNode; }
break;
case 763:
  if (!yytrial)
{ yyval.alterDatabaseNode = newNode<AlterDatabaseNode>(); }
break;
case 764:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[-2].alterDatabaseNode; }
break;
case 765:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].alterDomainNode; }
break;
case 766:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 767:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 768:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterFunctionNode; }
break;
case 769:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 770:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterUserNode; }
break;
case 771:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterUserNode; }
break;
case 772:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 773:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterSequenceNode; }
break;
case 774:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].createAlterSequenceNode; }
break;
case 775:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 776:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 777:
  if (!yytrial)
{ yyval.alterDomainNode = newNode<AlterDomainNode>(*yyvsp[0].metaNamePtr); }
break;
case 778:
yyval.alterDomainNode = yyvsp[0].alterDomainNode;
break;
case 779:
  if (!yytrial)
{ yyval.alterDomainNode = yyvsp[-2].alterDomainNode; }
break;
case 781:
yyval.alterDomainNode = yyvsp[-1].alterDomainNode;
break;
case 783:
  if (!yytrial)
{ setClause(yyvsp[-2].alterDomainNode->setDefault, "DOMAIN DEFAULT", yyvsp[0].valueSourceClause); }
break;
case 784:
  if (!yytrial)
{ setClause(yyvsp[-3].alterDomainNode->setConstraint, "DOMAIN CONSTRAINT", yyvsp[0].boolSourceClause); }
break;
case 785:
  if (!yytrial)
{ setClause(yyvsp[-2].alterDomainNode->setConstraint, "DOMAIN CONSTRAINT", yyvsp[0].boolSourceClause); }
break;
case 786:
  if (!yytrial)
{ setClause(yyvsp[-2].alterDomainNode->dropDefault, "DOMAIN DROP DEFAULT"); }
break;
case 787:
  if (!yytrial)
{ setClause(yyvsp[-2].alterDomainNode->dropConstraint, "DOMAIN DROP CONSTRAINT"); }
break;
case 788:
  if (!yytrial)
{ setClause(yyvsp[-3].alterDomainNode->notNullFlag, "{SET | DROP} NOT NULL", false); }
break;
case 789:
  if (!yytrial)
{ setClause(yyvsp[-3].alterDomainNode->notNullFlag, "{SET | DROP} NOT NULL", true); }
break;
case 790:
  if (!yytrial)
{ setClause(yyvsp[-2].alterDomainNode->renameTo, "DOMAIN NAME", *yyvsp[0].metaNamePtr); }
break;
case 791:
  if (!yytrial)
{
			/*// FIXME: ALTER DOMAIN doesn't support collations, and altered domain's*/
			/*// collation is always lost.*/
			dsql_fld* type = yyvsp[0].legacyField;
			type->collate = "";
			setClause(yyvsp[-2].alterDomainNode->type, "DOMAIN TYPE", type);
		}
break;
case 793:
yyval.relationNode = yyvsp[-2].relationNode;
break;
case 795:
  if (!yytrial)
{
			RelationNode::DropColumnClause* clause = newNode<RelationNode::DropColumnClause>();
			clause->name = *yyvsp[-1].metaNamePtr;
			clause->cascade = yyvsp[0].boolVal;
			yyvsp[-3].relationNode->clauses.add(clause);
		}
break;
case 796:
  if (!yytrial)
{
			RelationNode::DropConstraintClause* clause = newNode<RelationNode::DropConstraintClause>();
			clause->name = *yyvsp[0].metaNamePtr;
			yyvsp[-3].relationNode->clauses.add(clause);
		}
break;
case 799:
  if (!yytrial)
{
			RelationNode::AlterColPosClause* clause = newNode<RelationNode::AlterColPosClause>();
			clause->name = *yyvsp[-2].metaNamePtr;
			clause->newPos = yyvsp[0].int32Val;
			yyvsp[-4].relationNode->clauses.add(clause);
		}
break;
case 800:
  if (!yytrial)
{
			RelationNode::AlterColNameClause* clause = newNode<RelationNode::AlterColNameClause>();
			clause->fromName = *yyvsp[-2].metaNamePtr;
			clause->toName = *yyvsp[0].metaNamePtr;
			yyvsp[-4].relationNode->clauses.add(clause);
		}
break;
case 801:
  if (!yytrial)
{
			RelationNode::AlterColNullClause* clause = newNode<RelationNode::AlterColNullClause>();
			clause->name = *yyvsp[-3].metaNamePtr;
			clause->notNullFlag = false;
			yyvsp[-5].relationNode->clauses.add(clause);
		}
break;
case 802:
  if (!yytrial)
{
			RelationNode::AlterColNullClause* clause = newNode<RelationNode::AlterColNullClause>();
			clause->name = *yyvsp[-3].metaNamePtr;
			clause->notNullFlag = true;
			yyvsp[-5].relationNode->clauses.add(clause);
		}
break;
case 803:
  if (!yytrial)
{
			RelationNode::AlterColTypeClause* clause = newNode<RelationNode::AlterColTypeClause>();
			clause->field = yyvsp[0].legacyField;
			clause->field->fld_name = *yyvsp[-2].metaNamePtr;
			yyvsp[-4].relationNode->clauses.add(clause);
		}
break;
case 804:
  if (!yytrial)
{
			RelationNode::AlterColTypeClause* clause = newNode<RelationNode::AlterColTypeClause>();
			clause->field = yyvsp[-1].legacyField;
			clause->field->fld_name = *yyvsp[-3].metaNamePtr;
			clause->computed = yyvsp[0].valueSourceClause;
			yyvsp[-5].relationNode->clauses.add(clause);
			clause->field->flags |= FLD_computed;
		}
break;
case 805:
  if (!yytrial)
{
			RelationNode::AlterColTypeClause* clause = newNode<RelationNode::AlterColTypeClause>();
			clause->field = newNode<dsql_fld>();
			clause->field->fld_name = *yyvsp[-1].metaNamePtr;
			clause->computed = yyvsp[0].valueSourceClause;
			yyvsp[-3].relationNode->clauses.add(clause);
			clause->field->flags |= FLD_computed;
		}
break;
case 806:
  if (!yytrial)
{
			RelationNode::AlterColTypeClause* clause = newNode<RelationNode::AlterColTypeClause>();
			clause->field = newNode<dsql_fld>();
			clause->field->fld_name = *yyvsp[-2].metaNamePtr;
			clause->defaultValue = yyvsp[0].valueSourceClause;
			yyvsp[-4].relationNode->clauses.add(clause);
		}
break;
case 807:
  if (!yytrial)
{
			RelationNode::AlterColTypeClause* clause = newNode<RelationNode::AlterColTypeClause>();
			clause->field = newNode<dsql_fld>();
			clause->field->fld_name = *yyvsp[-2].metaNamePtr;
			clause->dropDefault = true;
			yyvsp[-4].relationNode->clauses.add(clause);
		}
break;
case 808:
  if (!yytrial)
{
			RelationNode::AlterColTypeClause* clause = newNode<RelationNode::AlterColTypeClause>();
			clause->field = newNode<dsql_fld>();
			clause->field->fld_name = *yyvsp[-1].metaNamePtr;
			clause->identityRestart = true;
			clause->identityRestartValue = yyvsp[0].nullableInt64Val;
			yyvsp[-3].relationNode->clauses.add(clause);
		}
break;
case 895:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
		}
break;
case 896:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 897:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 898:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 899:
  if (!yytrial)
{ yyval.ddlNode = newNode<AlterIndexNode>(*yyvsp[-1].metaNamePtr, true); }
break;
case 900:
  if (!yytrial)
{ yyval.ddlNode = newNode<AlterIndexNode>(*yyvsp[-1].metaNamePtr, false); }
break;
case 901:
  if (!yytrial)
{
			AlterExternalFunctionNode* node = newNode<AlterExternalFunctionNode>(*yyvsp[-2].metaNamePtr);
			if (yyvsp[-1].stringPtr)
				node->clauses.name = *yyvsp[-1].stringPtr;
			if (yyvsp[0].stringPtr)
				node->clauses.udfModule = *yyvsp[0].stringPtr;
			yyval.ddlNode = node;
		}
break;
case 902:
  if (!yytrial)
{ yyval.stringPtr = NULL; }
break;
case 903:
  if (!yytrial)
{ yyval.stringPtr = yyvsp[0].stringPtr; }
break;
case 904:
  if (!yytrial)
{ yyval.stringPtr = NULL; }
break;
case 905:
  if (!yytrial)
{ yyval.stringPtr = yyvsp[0].stringPtr; }
break;
case 906:
  if (!yytrial)
{
			yyval.mappingNode = newNode<MappingNode>(MappingNode::MAP_RPL, "AutoAdminImplementationMapping");
			yyval.mappingNode->op = yyvsp[-3].boolVal ? MappingNode::MAP_RPL : MappingNode::MAP_DROP;
			yyval.mappingNode->from = newNode<IntlString>(FB_DOMAIN_ANY_RID_ADMINS);
			yyval.mappingNode->fromType = newNode<MetaName>(FB_PREDEFINED_GROUP);
			yyval.mappingNode->mode = 'P';
			yyval.mappingNode->plugin = newNode<MetaName>("Win_Sspi");
			yyval.mappingNode->role = true;
			yyval.mappingNode->to = yyvsp[-4].metaNamePtr;
			yyval.mappingNode->validateAdmin();
		}
break;
case 907:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 908:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 911:
yyval.dbFilesClause = NOTRIAL(&yyvsp[-1].alterDatabaseNode->files);
break;
case 913:
  if (!yytrial)
{ yyvsp[-4].alterDatabaseNode->differenceFile = *yyvsp[0].stringPtr; }
break;
case 914:
  if (!yytrial)
{ yyvsp[-3].alterDatabaseNode->clauses |= AlterDatabaseNode::CLAUSE_DROP_DIFFERENCE; }
break;
case 915:
  if (!yytrial)
{ yyvsp[-2].alterDatabaseNode->clauses |= AlterDatabaseNode::CLAUSE_BEGIN_BACKUP; }
break;
case 916:
  if (!yytrial)
{ yyvsp[-2].alterDatabaseNode->clauses |= AlterDatabaseNode::CLAUSE_END_BACKUP; }
break;
case 917:
  if (!yytrial)
{ yyvsp[-5].alterDatabaseNode->setDefaultCharSet = *yyvsp[0].metaNamePtr; }
break;
case 918:
yyval.alterDatabaseNode = yyvsp[-3].alterDatabaseNode;
break;
case 919:
  if (!yytrial)
{
			setClauseFlag(yyvsp[-5].alterDatabaseNode->clauses, AlterDatabaseNode::CLAUSE_CRYPT, "CRYPT");
			yyvsp[-5].alterDatabaseNode->cryptPlugin = *yyvsp[-2].metaNamePtr;
		}
break;
case 920:
  if (!yytrial)
{ setClauseFlag(yyvsp[-1].alterDatabaseNode->clauses, AlterDatabaseNode::CLAUSE_CRYPT, "CRYPT"); }
break;
case 921:
  if (!yytrial)
{ yyvsp[-4].alterDatabaseNode->linger = yyvsp[0].int32Val; }
break;
case 922:
  if (!yytrial)
{ yyvsp[-2].alterDatabaseNode->linger = 0; }
break;
case 924:
  if (!yytrial)
{ yyvsp[-2].alterDatabaseNode->keyName = *yyvsp[0].metaNamePtr; }
break;
case 925:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = newNode<CreateAlterTriggerNode>(*yyvsp[-6].metaNamePtr);
			yyval.createAlterTriggerNode->alter = true;
			yyval.createAlterTriggerNode->create = false;
			yyval.createAlterTriggerNode->active = yyvsp[-5].nullableBoolVal;
			yyval.createAlterTriggerNode->type = yyvsp[-4].nullableUint64Val;
			yyval.createAlterTriggerNode->position = yyvsp[-3].nullableIntVal;
			yyval.createAlterTriggerNode->source = makeParseStr(YYPOSNARG(5), YYPOSNARG(7));
			yyval.createAlterTriggerNode->localDeclList = yyvsp[-1].compoundStmtNode;
			yyval.createAlterTriggerNode->body = yyvsp[0].stmtNode;
		}
break;
case 926:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = newNode<CreateAlterTriggerNode>(*yyvsp[-5].metaNamePtr);
			yyval.createAlterTriggerNode->alter = true;
			yyval.createAlterTriggerNode->create = false;
			yyval.createAlterTriggerNode->active = yyvsp[-4].nullableBoolVal;
			yyval.createAlterTriggerNode->type = yyvsp[-3].nullableUint64Val;
			yyval.createAlterTriggerNode->position = yyvsp[-2].nullableIntVal;
			yyval.createAlterTriggerNode->external = yyvsp[-1].externalClause;
			if (yyvsp[0].stringPtr)
				yyval.createAlterTriggerNode->source = *yyvsp[0].stringPtr;
		}
break;
case 927:
  if (!yytrial)
{
			yyval.createAlterTriggerNode = newNode<CreateAlterTriggerNode>(*yyvsp[-3].metaNamePtr);
			yyval.createAlterTriggerNode->alter = true;
			yyval.createAlterTriggerNode->create = false;
			yyval.createAlterTriggerNode->active = yyvsp[-2].nullableBoolVal;
			yyval.createAlterTriggerNode->type = yyvsp[-1].nullableUint64Val;
			yyval.createAlterTriggerNode->position = yyvsp[0].nullableIntVal;
		}
break;
case 928:
  if (!yytrial)
{ yyval.nullableUint64Val = Nullable<FB_UINT64>::val(yyvsp[-1].uint64Val + yyvsp[0].uint64Val - 1); }
break;
case 929:
  if (!yytrial)
{ yyval.nullableUint64Val = Nullable<FB_UINT64>::empty(); }
break;
case 930:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].ddlNode; }
break;
case 931:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropExceptionNode>(*yyvsp[0].metaNamePtr); }
break;
case 932:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropIndexNode>(*yyvsp[0].metaNamePtr); }
break;
case 933:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropProcedureNode>(*yyvsp[0].metaNamePtr); }
break;
case 934:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropRelationNode>(*yyvsp[0].metaNamePtr, false); }
break;
case 935:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropTriggerNode>(*yyvsp[0].metaNamePtr); }
break;
case 936:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropRelationNode>(*yyvsp[0].metaNamePtr, true); }
break;
case 937:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropFilterNode>(*yyvsp[0].metaNamePtr); }
break;
case 938:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropDomainNode>(*yyvsp[0].metaNamePtr); }
break;
case 939:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropFunctionNode>(*yyvsp[0].metaNamePtr); }
break;
case 940:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropFunctionNode>(*yyvsp[0].metaNamePtr); }
break;
case 941:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropShadowNode>(yyvsp[-1].int32Val, yyvsp[0].boolVal); }
break;
case 942:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropRoleNode>(*yyvsp[0].metaNamePtr); }
break;
case 943:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropSequenceNode>(*yyvsp[0].metaNamePtr); }
break;
case 944:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropSequenceNode>(*yyvsp[0].metaNamePtr); }
break;
case 945:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropCollationNode>(*yyvsp[0].metaNamePtr); }
break;
case 946:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropUserNode>(*yyvsp[-3].metaNamePtr, yyvsp[0].metaNamePtr); }
break;
case 947:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropUserNode>(*yyvsp[0].metaNamePtr); }
break;
case 948:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropPackageNode>(*yyvsp[0].metaNamePtr); }
break;
case 949:
  if (!yytrial)
{ yyval.ddlNode = newNode<DropPackageBodyNode>(*yyvsp[0].metaNamePtr); }
break;
case 950:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 951:
  if (!yytrial)
{ yyval.ddlNode = yyvsp[0].mappingNode; }
break;
case 952:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 953:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 954:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 958:
  if (!yytrial)
{
			yyval.legacyField = yyvsp[-2].legacyField;
			yyval.legacyField->notNull = true;
		}
break;
case 961:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
		}
break;
case 962:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
			yyval.legacyField->typeOfTable = *yyvsp[-2].metaNamePtr;
		}
break;
case 963:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->typeOfName = *yyvsp[0].metaNamePtr;
			yyval.legacyField->fullDomain = true;
		}
break;
case 966:
  if (!yytrial)
{
			yyvsp[-3].legacyField->ranges = yyvsp[-1].valueListNode;
			yyvsp[-3].legacyField->dimensions = yyvsp[-3].legacyField->ranges->items.getCount() / 2;
			yyvsp[-3].legacyField->elementDtype = yyvsp[-3].legacyField->dtype;
			yyval.legacyField = yyvsp[-3].legacyField;
		}
break;
case 967:
  if (!yytrial)
{
			yyvsp[-4].legacyField->ranges = yyvsp[-2].valueListNode;
			yyvsp[-4].legacyField->dimensions = yyvsp[-4].legacyField->ranges->items.getCount() / 2;
			yyvsp[-4].legacyField->elementDtype = yyvsp[-4].legacyField->dtype;
			if (yyvsp[0].metaNamePtr)
				yyvsp[-4].legacyField->charSet = *yyvsp[0].metaNamePtr;
			yyval.legacyField = yyvsp[-4].legacyField;
		}
break;
case 969:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].valueListNode->items[0])->add(yyvsp[0].valueListNode->items[1]); }
break;
case 970:
  if (!yytrial)
{
			if (yyvsp[0].int32Val < 1)
		 		yyval.valueListNode = newNode<ValueListNode>(MAKE_const_slong(yyvsp[0].int32Val))->add(MAKE_const_slong(1));
			else
		 		yyval.valueListNode = newNode<ValueListNode>(MAKE_const_slong(1))->add(MAKE_const_slong(yyvsp[0].int32Val));
		}
break;
case 971:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(MAKE_const_slong(yyvsp[-2].int32Val))->add(MAKE_const_slong(yyvsp[0].int32Val)); }
break;
case 973:
  if (!yytrial)
{
			yyval.legacyField = yyvsp[-1].legacyField;
			if (yyvsp[0].metaNamePtr)
				yyval.legacyField->charSet = *yyvsp[0].metaNamePtr;
		}
break;
case 977:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();

			if (client_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post (Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
					Arg::Gds(isc_sql_dialect_datatype_unsupport) << Arg::Num(client_dialect) <<
																	Arg::Str("BIGINT"));
			}

			if (db_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post (Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
					Arg::Gds(isc_sql_db_dialect_dtype_unsupport) << Arg::Num(db_dialect) <<
																Arg::Str("BIGINT"));
			}

			yyval.legacyField->dtype = dtype_int64;
			yyval.legacyField->length = sizeof(SINT64);
		}
break;
case 978:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_long;
			yyval.legacyField->length = sizeof(SLONG);
		}
break;
case 979:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_short;
			yyval.legacyField->length = sizeof(SSHORT);
		}
break;
case 980:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			stmt_ambiguous = true;

			if (client_dialect <= SQL_DIALECT_V5)
			{
				/* Post warning saying that DATE is equivalent to TIMESTAMP*/
				ERRD_post_warning(Arg::Warning(isc_sqlwarn) << Arg::Num(301) <<
								  Arg::Warning(isc_dtype_renamed));
				yyval.legacyField->dtype = dtype_timestamp;
				yyval.legacyField->length = sizeof(GDS_TIMESTAMP);
			}
			else if (client_dialect == SQL_DIALECT_V6_TRANSITION)
				yyabandon(-104, isc_transitional_date);
			else
			{
				yyval.legacyField->dtype = dtype_sql_date;
				yyval.legacyField->length = sizeof(ULONG);
			}
		}
break;
case 981:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();

			if (client_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_dialect_datatype_unsupport) << Arg::Num(client_dialect) <<
																		  Arg::Str("TIME"));
			}
			if (db_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_db_dialect_dtype_unsupport) << Arg::Num(db_dialect) <<
																		  Arg::Str("TIME"));
			}
			yyval.legacyField->dtype = dtype_sql_time;
			yyval.legacyField->length = sizeof(SLONG);
		}
break;
case 982:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_timestamp;
			yyval.legacyField->length = sizeof(GDS_TIMESTAMP);
		}
break;
case 983:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_boolean;
			yyval.legacyField->length = sizeof(UCHAR);
		}
break;
case 986:
  if (!yytrial)
{ yyval.legacyField = newNode<dsql_fld>(); }
break;
case 987:
yyval.legacyField = NOTRIAL(yyvsp[0].legacyField);
break;
case 988:
  if (!yytrial)
{
			yyval.legacyField = yyvsp[-4].legacyField;
			yyval.legacyField->dtype = dtype_blob;
			yyval.legacyField->length = sizeof(ISC_QUAD);
			yyval.legacyField->segLength = yyvsp[-1].uintVal;
			if (yyvsp[0].metaNamePtr)
				yyval.legacyField->charSet = *yyvsp[0].metaNamePtr;
		}
break;
case 989:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_blob;
			yyval.legacyField->length = sizeof(ISC_QUAD);
			yyval.legacyField->segLength = (USHORT) yyvsp[-1].int32Val;
			yyval.legacyField->subType = 0;
		}
break;
case 990:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_blob;
			yyval.legacyField->length = sizeof(ISC_QUAD);
			yyval.legacyField->segLength = (USHORT) yyvsp[-3].int32Val;
			yyval.legacyField->subType = (USHORT) yyvsp[-1].int32Val;
		}
break;
case 991:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_blob;
			yyval.legacyField->length = sizeof(ISC_QUAD);
			yyval.legacyField->segLength = 80;
			yyval.legacyField->subType = (USHORT) yyvsp[-1].int32Val;
		}
break;
case 992:
  if (!yytrial)
{ yyval.uintVal = (USHORT) 80; }
break;
case 993:
  if (!yytrial)
{ yyval.uintVal = (USHORT) yyvsp[0].int32Val; }
break;
case 994:
  if (!yytrial)
{ yyvsp[0].legacyField->subType = (USHORT) 0; }
break;
case 995:
  if (!yytrial)
{ yyvsp[-2].legacyField->subType = (USHORT) yyvsp[0].int32Val; }
break;
case 996:
  if (!yytrial)
{ yyvsp[-2].legacyField->subTypeName = *yyvsp[0].metaNamePtr; }
break;
case 997:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 998:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 999:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_text;
			yyval.legacyField->charLength = (USHORT) yyvsp[-1].int32Val;
			yyval.legacyField->flags |= FLD_national;
		}
break;
case 1000:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_text;
			yyval.legacyField->charLength = 1;
			yyval.legacyField->flags |= FLD_national;
		}
break;
case 1001:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_varying;
			yyval.legacyField->charLength = (USHORT) yyvsp[-1].int32Val;
			yyval.legacyField->flags |= FLD_national;
		}
break;
case 1002:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_text;
			yyval.legacyField->charLength = (USHORT) yyvsp[-1].int32Val;
		}
break;
case 1003:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_text;
			yyval.legacyField->charLength = 1;
		}
break;
case 1004:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_varying;
			yyval.legacyField->charLength = (USHORT) yyvsp[-1].int32Val;
		}
break;
case 1013:
  if (!yytrial)
{
			yyval.legacyField = yyvsp[0].legacyField;
			yyval.legacyField->subType = dsc_num_type_numeric;
		}
break;
case 1014:
  if (!yytrial)
{
			yyval.legacyField = yyvsp[0].legacyField;
			yyval.legacyField->subType = dsc_num_type_decimal;

			if (yyval.legacyField->dtype == dtype_short)
			{
				yyval.legacyField->dtype = dtype_long;
				yyval.legacyField->length = sizeof(SLONG);
			}
		}
break;
case 1015:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_long;
			yyval.legacyField->length = sizeof(SLONG);
			yyval.legacyField->precision = 9;
		}
break;
case 1016:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();

			if (yyvsp[-1].int32Val < 1 || yyvsp[-1].int32Val > 18)
				yyabandon(-842, isc_precision_err);	/* Precision must be between 1 and 18.*/

			if (yyvsp[-1].int32Val > 9)
			{
				if ( ( (client_dialect <= SQL_DIALECT_V5) && (db_dialect > SQL_DIALECT_V5) ) ||
					( (client_dialect > SQL_DIALECT_V5) && (db_dialect <= SQL_DIALECT_V5) ) )
				{
					ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-817) <<
							  Arg::Gds(isc_ddl_not_allowed_by_db_sql_dial) << Arg::Num(db_dialect));
				}

				if (client_dialect <= SQL_DIALECT_V5)
				{
					yyval.legacyField->dtype = dtype_double;
					yyval.legacyField->length = sizeof(double);
				}
				else
				{
					if (client_dialect == SQL_DIALECT_V6_TRANSITION)
					{
						ERRD_post_warning(Arg::Warning(isc_dsql_warn_precision_ambiguous));
						ERRD_post_warning(Arg::Warning(isc_dsql_warn_precision_ambiguous1));
						ERRD_post_warning(Arg::Warning(isc_dsql_warn_precision_ambiguous2));
					}

					yyval.legacyField->dtype = dtype_int64;
					yyval.legacyField->length = sizeof(SINT64);
				}
			}
			else
			{
				if (yyvsp[-1].int32Val < 5)
				{
					yyval.legacyField->dtype = dtype_short;
					yyval.legacyField->length = sizeof(SSHORT);
				}
				else
				{
					yyval.legacyField->dtype = dtype_long;
					yyval.legacyField->length = sizeof(SLONG);
				}
			}

			yyval.legacyField->precision = (USHORT) yyvsp[-1].int32Val;
		}
break;
case 1017:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();

			if (yyvsp[-3].int32Val < 1 || yyvsp[-3].int32Val > 18)
				yyabandon (-842, isc_precision_err);	/* Precision should be between 1 and 18*/

			if (yyvsp[-1].int32Val > yyvsp[-3].int32Val || yyvsp[-1].int32Val < 0)
				yyabandon (-842, isc_scale_nogt);	/* Scale must be between 0 and precision*/

			if (yyvsp[-3].int32Val > 9)
			{
				if ( ( (client_dialect <= SQL_DIALECT_V5) && (db_dialect > SQL_DIALECT_V5) ) ||
					( (client_dialect > SQL_DIALECT_V5) && (db_dialect <= SQL_DIALECT_V5) ) )
				{
					ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-817) <<
							  Arg::Gds(isc_ddl_not_allowed_by_db_sql_dial) << Arg::Num(db_dialect));
				}

				if (client_dialect <= SQL_DIALECT_V5)
				{
					yyval.legacyField->dtype = dtype_double;
					yyval.legacyField->length = sizeof(double);
				}
				else
				{
					if (client_dialect == SQL_DIALECT_V6_TRANSITION)
					{
						ERRD_post_warning(Arg::Warning(isc_dsql_warn_precision_ambiguous));
						ERRD_post_warning(Arg::Warning(isc_dsql_warn_precision_ambiguous1));
						ERRD_post_warning(Arg::Warning(isc_dsql_warn_precision_ambiguous2));
					}
					/* client_dialect >= SQL_DIALECT_V6*/
					yyval.legacyField->dtype = dtype_int64;
					yyval.legacyField->length = sizeof(SINT64);
				}
			}
			else
			{
				if (yyvsp[-3].int32Val < 5)
				{
					yyval.legacyField->dtype = dtype_short;
					yyval.legacyField->length = sizeof(SSHORT);
				}
				else
				{
					yyval.legacyField->dtype = dtype_long;
					yyval.legacyField->length = sizeof(SLONG);
				}
			}

			yyval.legacyField->precision = (USHORT) yyvsp[-3].int32Val;
			yyval.legacyField->scale = - (SSHORT) yyvsp[-1].int32Val;
		}
break;
case 1020:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();

			if (yyvsp[0].int32Val > 7)
			{
				yyval.legacyField->dtype = dtype_double;
				yyval.legacyField->length = sizeof(double);
			}
			else
			{
				yyval.legacyField->dtype = dtype_real;
				yyval.legacyField->length = sizeof(float);
			}
		}
break;
case 1021:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_double;
			yyval.legacyField->length = sizeof(double);
		}
break;
case 1022:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_real;
			yyval.legacyField->length = sizeof(float);
		}
break;
case 1023:
  if (!yytrial)
{
			yyval.legacyField = newNode<dsql_fld>();
			yyval.legacyField->dtype = dtype_double;
			yyval.legacyField->length = sizeof(double);
		}
break;
case 1024:
  if (!yytrial)
{ yyval.int32Val = 0; }
break;
case 1025:
  if (!yytrial)
{ yyval.int32Val = yyvsp[-1].int32Val; }
break;
case 1029:
  if (!yytrial)
{
			UserSavepointNode* node = newNode<UserSavepointNode>();
			node->command = UserSavepointNode::CMD_SET;
			node->name = *yyvsp[0].metaNamePtr;
			yyval.stmtNode = node;
		}
break;
case 1030:
  if (!yytrial)
{
			UserSavepointNode* node = newNode<UserSavepointNode>();
			node->command = (yyvsp[0].boolVal ? UserSavepointNode::CMD_RELEASE_ONLY : UserSavepointNode::CMD_RELEASE);
			node->name = *yyvsp[-1].metaNamePtr;
			yyval.stmtNode = node;
		}
break;
case 1031:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1032:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 1033:
  if (!yytrial)
{
			UserSavepointNode* node = newNode<UserSavepointNode>();
			node->command = UserSavepointNode::CMD_ROLLBACK;
			node->name = *yyvsp[0].metaNamePtr;
			yyval.stmtNode = node;
		}
break;
case 1036:
  if (!yytrial)
{ yyval.traNode = newNode<CommitRollbackNode>(CommitRollbackNode::CMD_COMMIT, yyvsp[0].boolVal); }
break;
case 1037:
  if (!yytrial)
{ yyval.traNode = newNode<CommitRollbackNode>(CommitRollbackNode::CMD_ROLLBACK, yyvsp[0].boolVal); }
break;
case 1040:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1041:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 1044:
  if (!yytrial)
{ yyval.setTransactionNode = newNode<SetTransactionNode>(); }
break;
case 1045:
yyval.setTransactionNode = yyvsp[0].setTransactionNode;
break;
case 1046:
  if (!yytrial)
{ yyval.setTransactionNode = yyvsp[-2].setTransactionNode; }
break;
case 1047:
  if (!yytrial)
{ yyval.setRoleNode = newNode<SetRoleNode>(yyvsp[0].metaNamePtr); }
break;
case 1048:
  if (!yytrial)
{ yyval.setRoleNode = newNode<SetRoleNode>(); }
break;
case 1052:
yyval.setTransactionNode = yyvsp[-1].setTransactionNode;
break;
case 1054:
  if (!yytrial)
{ setClause(yyvsp[-2].setTransactionNode->readOnly, "READ {ONLY | WRITE}", true); }
break;
case 1055:
  if (!yytrial)
{ setClause(yyvsp[-2].setTransactionNode->readOnly, "READ {ONLY | WRITE}", false); }
break;
case 1056:
  if (!yytrial)
{ setClause(yyvsp[-1].setTransactionNode->wait, "[NO] WAIT", true); }
break;
case 1057:
  if (!yytrial)
{ setClause(yyvsp[-2].setTransactionNode->wait, "[NO] WAIT", false); }
break;
case 1058:
  if (!yytrial)
{ setClause(yyvsp[-1].setTransactionNode->isoLevel, "ISOLATION LEVEL", yyvsp[0].uintVal); }
break;
case 1059:
  if (!yytrial)
{ setClause(yyvsp[-3].setTransactionNode->noAutoUndo, "NO AUTO UNDO", true); }
break;
case 1060:
  if (!yytrial)
{ setClause(yyvsp[-2].setTransactionNode->ignoreLimbo, "IGNORE LIMBO", true); }
break;
case 1061:
  if (!yytrial)
{ setClause(yyvsp[-2].setTransactionNode->restartRequests, "RESTART REQUESTS", true); }
break;
case 1062:
  if (!yytrial)
{ setClause(yyvsp[-3].setTransactionNode->lockTimeout, "LOCK TIMEOUT", (USHORT) yyvsp[0].int32Val); }
break;
case 1063:
  if (!yytrial)
{ checkDuplicateClause(yyvsp[-1].setTransactionNode->reserveList, "RESERVING"); }
break;
case 1064:
yyval.setTransactionNode = yyvsp[-2].setTransactionNode;
break;
case 1066:
  if (!yytrial)
{ yyval.uintVal = yyvsp[0].uintVal;}
break;
case 1069:
  if (!yytrial)
{ yyval.uintVal = yyvsp[0].uintVal; }
break;
case 1070:
  if (!yytrial)
{ yyval.uintVal = yyvsp[0].uintVal; }
break;
case 1071:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::ISO_LEVEL_CONCURRENCY; }
break;
case 1072:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::ISO_LEVEL_CONSISTENCY; }
break;
case 1073:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::ISO_LEVEL_CONSISTENCY; }
break;
case 1074:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::ISO_LEVEL_READ_COMMITTED_NO_REC_VERSION; }
break;
case 1075:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::ISO_LEVEL_READ_COMMITTED_REC_VERSION; }
break;
case 1076:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::ISO_LEVEL_READ_COMMITTED_NO_REC_VERSION; }
break;
case 1077:
  if (!yytrial)
{ yyval.uintVal = 0; }
break;
case 1078:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::LOCK_MODE_SHARED; }
break;
case 1079:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::LOCK_MODE_PROTECTED; }
break;
case 1080:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::LOCK_MODE_READ; }
break;
case 1081:
  if (!yytrial)
{ yyval.uintVal = SetTransactionNode::LOCK_MODE_WRITE; }
break;
case 1082:
  if (!yytrial)
{ yyvsp[-1].setTransactionNode->reserveList.add(yyvsp[0].setTransactionRestrictionClause); }
break;
case 1083:
  if (!yytrial)
{ yyvsp[-3].setTransactionNode->reserveList.add(yyvsp[0].setTransactionRestrictionClause); }
break;
case 1084:
  if (!yytrial)
{ yyval.setTransactionRestrictionClause = newNode<SetTransactionNode::RestrictionOption>(yyvsp[-1].metaNameArray, yyvsp[0].uintVal); }
break;
case 1085:
  if (!yytrial)
{ yyval.uintVal = 0; }
break;
case 1086:
  if (!yytrial)
{ yyval.uintVal = yyvsp[-1].uintVal | yyvsp[0].uintVal; }
break;
case 1087:
  if (!yytrial)
{
			ObjectsArray<MetaName>* node = newNode<ObjectsArray<MetaName> >();
			node->add(*yyvsp[0].metaNamePtr);
			yyval.metaNameArray = node;
		}
break;
case 1088:
  if (!yytrial)
{
			ObjectsArray<MetaName>* node = yyvsp[-2].metaNameArray;
			node->add(*yyvsp[0].metaNamePtr);
			yyval.metaNameArray = node;
		}
break;
case 1089:
  if (!yytrial)
{ yyval.ddlNode = newNode<SetStatisticsNode>(*yyvsp[0].metaNamePtr); }
break;
case 1090:
  if (!yytrial)
{ yyval.ddlNode = newNode<CommentOnNode>(yyvsp[-2].intVal, QualifiedName(""), "", *yyvsp[0].stringPtr); }
break;
case 1091:
  if (!yytrial)
{ yyval.ddlNode = newNode<CommentOnNode>(yyvsp[-3].intVal, QualifiedName(*yyvsp[-2].metaNamePtr), "", *yyvsp[0].stringPtr); }
break;
case 1092:
  if (!yytrial)
{ yyval.ddlNode = newNode<CommentOnNode>(yyvsp[-4].intVal, QualifiedName(*yyvsp[-3].metaNamePtr), *yyvsp[-2].metaNamePtr, *yyvsp[0].stringPtr); }
break;
case 1093:
  if (!yytrial)
{ yyval.ddlNode = newNode<CommentOnNode>(yyvsp[-4].intVal, *yyvsp[-3].qualifiedNamePtr, *yyvsp[-2].metaNamePtr, *yyvsp[0].stringPtr); }
break;
case 1094:
  if (!yytrial)
{ yyval.ddlNode = newNode<CommentOnNode>(yyvsp[-3].intVal, *yyvsp[-2].qualifiedNamePtr, "", *yyvsp[0].stringPtr); }
break;
case 1095:
  if (!yytrial)
{
			CreateAlterUserNode* node =
				newNode<CreateAlterUserNode>(CreateAlterUserNode::USER_MOD, *yyvsp[-2].metaNamePtr);
			node->comment = yyvsp[0].stringPtr;
			yyval.ddlNode = node;
		}
break;
case 1096:
  if (!yytrial)
{ yyval.intVal = obj_database; }
break;
case 1097:
  if (!yytrial)
{ yyval.intVal = obj_field; }
break;
case 1098:
  if (!yytrial)
{ yyval.intVal = obj_relation; }
break;
case 1099:
  if (!yytrial)
{ yyval.intVal = obj_view; }
break;
case 1100:
  if (!yytrial)
{ yyval.intVal = obj_trigger; }
break;
case 1101:
  if (!yytrial)
{ yyval.intVal = obj_blob_filter; }
break;
case 1102:
  if (!yytrial)
{ yyval.intVal = obj_exception; }
break;
case 1103:
  if (!yytrial)
{ yyval.intVal = obj_generator; }
break;
case 1104:
  if (!yytrial)
{ yyval.intVal = obj_generator; }
break;
case 1105:
  if (!yytrial)
{ yyval.intVal = obj_index; }
break;
case 1106:
  if (!yytrial)
{ yyval.intVal = obj_sql_role; }
break;
case 1107:
  if (!yytrial)
{ yyval.intVal = obj_charset; }
break;
case 1108:
  if (!yytrial)
{ yyval.intVal = obj_collation; }
break;
case 1109:
  if (!yytrial)
{ yyval.intVal = obj_package_header; }
break;
case 1110:
  if (!yytrial)
{ yyval.intVal = obj_relation; }
break;
case 1111:
  if (!yytrial)
{ yyval.intVal = obj_parameter; }
break;
case 1112:
  if (!yytrial)
{ yyval.intVal = obj_procedure; }
break;
case 1113:
  if (!yytrial)
{ yyval.intVal = obj_udf; }
break;
case 1114:
  if (!yytrial)
{ yyval.intVal = obj_procedure; }
break;
case 1115:
  if (!yytrial)
{ yyval.intVal = obj_udf; }
break;
case 1116:
  if (!yytrial)
{ yyval.intVal = obj_udf; }
break;
case 1117:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 1118:
  if (!yytrial)
{ yyval.qualifiedNamePtr = newNode<QualifiedName>(*yyvsp[0].metaNamePtr); }
break;
case 1119:
  if (!yytrial)
{ yyval.qualifiedNamePtr = newNode<QualifiedName>(*yyvsp[0].metaNamePtr, *yyvsp[-2].metaNamePtr); }
break;
case 1120:
  if (!yytrial)
{ yyval.stringPtr = yyvsp[0].stringPtr; }
break;
case 1121:
  if (!yytrial)
{ yyval.stringPtr = newString(""); }
break;
case 1122:
  if (!yytrial)
{
			SelectNode* node = newNode<SelectNode>();
			node->dsqlExpr = yyvsp[-2].selectExprNode;
			node->dsqlForUpdate = yyvsp[-1].boolVal;
			node->dsqlWithLock = yyvsp[0].boolVal;
			yyval.selectNode = node;
		}
break;
case 1123:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1124:
  if (!yytrial)
{ yyval.boolVal = true; /* for_update_list is ignored */ }
break;
case 1125:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1126:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 1127:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1128:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 1129:
  if (!yytrial)
{
			SelectExprNode* node = yyval.selectExprNode = newNode<SelectExprNode>();
			node->querySpec = yyvsp[-2].recSourceNode;
			node->orderClause = yyvsp[-1].valueListNode;
			node->rowsClause = yyvsp[0].rowsClause;
			node->withClause = yyvsp[-3].withClause;
		}
break;
case 1130:
  if (!yytrial)
{
			SelectExprNode* node = yyval.selectExprNode = newNode<SelectExprNode>();
			node->querySpec = yyvsp[-3].recSourceNode;
			node->orderClause = yyvsp[-2].valueListNode;

			if (yyvsp[-1].valueExprNode || yyvsp[0].valueExprNode)
			{
				RowsClause* rowsNode = newNode<RowsClause>();
				rowsNode->skip = yyvsp[-1].valueExprNode;
				rowsNode->length = yyvsp[0].valueExprNode;
				node->rowsClause = rowsNode;
			}

			node->withClause = yyvsp[-4].withClause;
		}
break;
case 1131:
  if (!yytrial)
{ yyval.withClause = NULL; }
break;
case 1132:
  if (!yytrial)
{
			yyval.withClause = yyvsp[0].withClause;
			yyval.withClause->recursive = true;
		}
break;
case 1133:
  if (!yytrial)
{ yyval.withClause = yyvsp[0].withClause; }
break;
case 1134:
  if (!yytrial)
{
			yyval.withClause = newNode<WithClause>();
			yyval.withClause->add(yyvsp[0].selectExprNode);
		}
break;
case 1135:
  if (!yytrial)
{
			yyval.withClause = yyvsp[-2].withClause;
			yyval.withClause->add(yyvsp[0].selectExprNode);
		}
break;
case 1136:
  if (!yytrial)
{
			yyval.selectExprNode = yyvsp[-1].selectExprNode;
			yyval.selectExprNode->dsqlFlags |= RecordSourceNode::DFLAG_DERIVED;
			yyval.selectExprNode->alias = yyvsp[-5].metaNamePtr->c_str();
			yyval.selectExprNode->columns = yyvsp[-4].metaNameArray;
		}
break;
case 1137:
  if (!yytrial)
{
			yyval.selectExprNode = yyvsp[0].selectExprNode;
			yyval.selectExprNode->dsqlFlags |= RecordSourceNode::DFLAG_VALUE;
		}
break;
case 1138:
  if (!yytrial)
{
			yyvsp[0].selectExprNode->dsqlFlags |= RecordSourceNode::DFLAG_SINGLETON;
			yyval.valueExprNode = newNode<SubQueryNode>(blr_via, yyvsp[0].selectExprNode);
		}
break;
case 1139:
  if (!yytrial)
{ yyval.recSourceNode = yyvsp[0].rseNode; }
break;
case 1140:
  if (!yytrial)
{
			UnionSourceNode* node = yyvsp[-3].recSourceNode->as<UnionSourceNode>();
			if (node && !node->dsqlAll)
				node->dsqlClauses->add(yyvsp[0].rseNode);
			else
			{
				node = newNode<UnionSourceNode>();
				node->dsqlClauses = newNode<RecSourceListNode>(yyvsp[-3].recSourceNode)->add(yyvsp[0].rseNode);
			}
			yyval.recSourceNode = node;
		}
break;
case 1141:
  if (!yytrial)
{
			UnionSourceNode* node = yyvsp[-3].recSourceNode->as<UnionSourceNode>();
			if (node && node->dsqlAll)
				node->dsqlClauses->add(yyvsp[0].rseNode);
			else
			{
				node = newNode<UnionSourceNode>();
				node->dsqlAll = true;
				node->dsqlClauses = newNode<RecSourceListNode>(yyvsp[-3].recSourceNode)->add(yyvsp[0].rseNode);
			}
			yyval.recSourceNode = node;
		}
break;
case 1143:
  if (!yytrial)
{
			RseNode* rse = newNode<RseNode>();
			rse->dsqlFirst = yyvsp[-7].valueListNode ? yyvsp[-7].valueListNode->items[1] : NULL;
			rse->dsqlSkip = yyvsp[-7].valueListNode ? yyvsp[-7].valueListNode->items[0] : NULL;
			rse->dsqlDistinct = yyvsp[-6].valueListNode;
			rse->dsqlSelectList = yyvsp[-5].valueListNode;
			rse->dsqlFrom = yyvsp[-4].recSourceListNode;
			rse->dsqlWhere = yyvsp[-3].boolExprNode;
			rse->dsqlGroup = yyvsp[-2].valueListNode;
			rse->dsqlHaving = yyvsp[-1].boolExprNode;
			rse->rse_plan = yyvsp[0].planNode;
			yyval.rseNode = rse;
		}
break;
case 1144:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1145:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode)->add(yyvsp[-1].valueExprNode); }
break;
case 1146:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(1u)->add(yyvsp[0].valueExprNode); }
break;
case 1147:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode)->add(NULL); }
break;
case 1148:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(yyvsp[0].int32Val); }
break;
case 1149:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[-1].valueExprNode; }
break;
case 1150:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1151:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(yyvsp[0].int32Val); }
break;
case 1152:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[-1].valueExprNode; }
break;
case 1153:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1154:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(0); }
break;
case 1155:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1156:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 1157:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1158:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode); }
break;
case 1159:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].valueExprNode); }
break;
case 1161:
  if (!yytrial)
{
			FieldNode* fieldNode = newNode<FieldNode>();
			fieldNode->dsqlQualifier = *yyvsp[-2].metaNamePtr;
			yyval.valueExprNode = fieldNode;
		}
break;
case 1163:
  if (!yytrial)
{ yyval.valueExprNode = newNode<DsqlAliasNode>(*yyvsp[0].metaNamePtr, yyvsp[-2].valueExprNode); }
break;
case 1166:
  if (!yytrial)
{ yyval.recSourceListNode = yyvsp[0].recSourceListNode; }
break;
case 1167:
  if (!yytrial)
{ yyval.recSourceListNode = newNode<RecSourceListNode>(yyvsp[0].recSourceNode); }
break;
case 1168:
  if (!yytrial)
{ yyval.recSourceListNode = yyvsp[-2].recSourceListNode->add(yyvsp[0].recSourceNode); }
break;
case 1172:
  if (!yytrial)
{ yyval.recSourceNode = yyvsp[0].selectExprNode; }
break;
case 1173:
  if (!yytrial)
{ yyval.recSourceNode = yyvsp[-1].recSourceNode; }
break;
case 1174:
  if (!yytrial)
{
			yyval.selectExprNode = yyvsp[-4].selectExprNode;
			yyval.selectExprNode->dsqlFlags |= RecordSourceNode::DFLAG_DERIVED;
			if (yyvsp[-1].metaNamePtr)
				yyval.selectExprNode->alias = yyvsp[-1].metaNamePtr->c_str();
			yyval.selectExprNode->columns = yyvsp[0].metaNameArray;
		}
break;
case 1175:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 1177:
  if (!yytrial)
{ yyval.metaNameArray = NULL; }
break;
case 1178:
  if (!yytrial)
{ yyval.metaNameArray = yyvsp[-1].metaNameArray; }
break;
case 1179:
  if (!yytrial)
{
			ObjectsArray<MetaName>* node = newNode<ObjectsArray<MetaName> >();
			node->add(*yyvsp[0].metaNamePtr);
			yyval.metaNameArray = node;
		}
break;
case 1180:
  if (!yytrial)
{
			ObjectsArray<MetaName>* node = yyvsp[-2].metaNameArray;
			node->add(*yyvsp[0].metaNamePtr);
			yyval.metaNameArray = node;
		}
break;
case 1184:
  if (!yytrial)
{
			RseNode* rse = newNode<RseNode>();
			rse->dsqlExplicitJoin = true;
			rse->rse_jointype = blr_inner;
			rse->dsqlFrom = newNode<RecSourceListNode>(yyvsp[-3].recSourceNode)->add(yyvsp[0].recSourceNode);
			yyval.recSourceNode = rse;
		}
break;
case 1185:
  if (!yytrial)
{
			RseNode* rse = newNode<RseNode>();
			rse->dsqlExplicitJoin = true;
			rse->rse_jointype = yyvsp[-2].blrOp;
			rse->dsqlFrom = newNode<RecSourceListNode>(yyvsp[-4].recSourceNode)->add(yyvsp[0].recSourceNode);
			rse->dsqlJoinUsing = newNode<ValueListNode>(0u);	/* using list with size 0 -> natural*/
			yyval.recSourceNode = rse;
		}
break;
case 1186:
  if (!yytrial)
{
			RseNode* rse = newNode<RseNode>();
			rse->dsqlExplicitJoin = true;
			rse->rse_jointype = yyvsp[-3].blrOp;
			rse->dsqlFrom = newNode<RecSourceListNode>(yyvsp[-4].recSourceNode);
			rse->dsqlFrom->add(yyvsp[-1].recSourceNode);
			rse->dsqlWhere = yyvsp[0].boolExprNode;
			yyval.recSourceNode = rse;
		}
break;
case 1187:
  if (!yytrial)
{
			RseNode* rse = newNode<RseNode>();
			rse->dsqlExplicitJoin = true;
			rse->rse_jointype = yyvsp[-3].blrOp;
			rse->dsqlFrom = newNode<RecSourceListNode>(yyvsp[-4].recSourceNode);
			rse->dsqlFrom->add(yyvsp[-1].recSourceNode);
			rse->dsqlJoinUsing = yyvsp[0].valueListNode;
			yyval.recSourceNode = rse;
		}
break;
case 1188:
  if (!yytrial)
{ yyval.boolExprNode = yyvsp[0].boolExprNode; }
break;
case 1189:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-1].valueListNode; }
break;
case 1190:
  if (!yytrial)
{
			ProcedureSourceNode* node = newNode<ProcedureSourceNode>(QualifiedName(*yyvsp[-3].metaNamePtr));
			node->sourceList = yyvsp[-2].valueListNode;
			node->alias = yyvsp[0].metaNamePtr->c_str();
			yyval.recSourceNode = node;
		}
break;
case 1191:
  if (!yytrial)
{
			ProcedureSourceNode* node = newNode<ProcedureSourceNode>(QualifiedName(*yyvsp[-1].metaNamePtr));
			node->sourceList = yyvsp[0].valueListNode;
			yyval.recSourceNode = node;
		}
break;
case 1192:
  if (!yytrial)
{
			ProcedureSourceNode* node = newNode<ProcedureSourceNode>(
				QualifiedName(*yyvsp[-3].metaNamePtr, *yyvsp[-5].metaNamePtr));
			node->sourceList = yyvsp[-2].valueListNode;
			node->alias = yyvsp[0].metaNamePtr->c_str();
			yyval.recSourceNode = node;
		}
break;
case 1193:
  if (!yytrial)
{
			ProcedureSourceNode* node = newNode<ProcedureSourceNode>(
				QualifiedName(*yyvsp[-1].metaNamePtr, *yyvsp[-3].metaNamePtr));
			node->sourceList = yyvsp[0].valueListNode;
			yyval.recSourceNode = node;
		}
break;
case 1194:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1195:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-1].valueListNode; }
break;
case 1197:
  if (!yytrial)
{
			RelationSourceNode* node = newNode<RelationSourceNode>(*yyvsp[-2].metaNamePtr);
			node->alias = yyvsp[0].metaNamePtr->c_str();
			yyval.relSourceNode = node;
		}
break;
case 1198:
  if (!yytrial)
{ yyval.relSourceNode = newNode<RelationSourceNode>(*yyvsp[0].metaNamePtr); }
break;
case 1199:
  if (!yytrial)
{ yyval.blrOp = blr_inner; }
break;
case 1200:
  if (!yytrial)
{ yyval.blrOp = blr_inner; }
break;
case 1201:
  if (!yytrial)
{ yyval.blrOp = blr_left; }
break;
case 1202:
  if (!yytrial)
{ yyval.blrOp = blr_right; }
break;
case 1203:
  if (!yytrial)
{ yyval.blrOp = blr_full; }
break;
case 1206:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1207:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 1208:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode); }
break;
case 1209:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].valueExprNode); }
break;
case 1211:
  if (!yytrial)
{ yyval.boolExprNode = NULL; }
break;
case 1212:
  if (!yytrial)
{ yyval.boolExprNode = yyvsp[0].boolExprNode; }
break;
case 1213:
  if (!yytrial)
{ yyval.boolExprNode = NULL; }
break;
case 1214:
  if (!yytrial)
{ yyval.boolExprNode = yyvsp[0].boolExprNode; }
break;
case 1215:
  if (!yytrial)
{ yyval.planNode = NULL; }
break;
case 1216:
  if (!yytrial)
{ yyval.planNode = yyvsp[0].planNode; }
break;
case 1217:
  if (!yytrial)
{ yyval.planNode = newNode<PlanNode>(PlanNode::TYPE_RETRIEVE); }
break;
case 1218:
yyval.planNode = yyvsp[-1].planNode;
break;
case 1219:
  if (!yytrial)
{ yyval.planNode = yyvsp[-4].planNode; }
break;
case 1226:
  if (!yytrial)
{ yyvsp[-1].planNode->subNodes.add(yyvsp[0].planNode); }
break;
case 1227:
  if (!yytrial)
{ yyvsp[-3].planNode->subNodes.add(yyvsp[0].planNode); }
break;
case 1228:
  if (!yytrial)
{
			yyval.planNode = newNode<PlanNode>(PlanNode::TYPE_RETRIEVE);
			yyval.planNode->dsqlNames = yyvsp[-1].metaNameArray;
			yyval.planNode->accessType = yyvsp[0].accessType;
		}
break;
case 1230:
  if (!yytrial)
{
			ObjectsArray<MetaName>* node = newNode<ObjectsArray<MetaName> >();
			node->add(*yyvsp[0].metaNamePtr);
			yyval.metaNameArray = node;
		}
break;
case 1231:
  if (!yytrial)
{
			ObjectsArray<MetaName>* node = yyvsp[-1].metaNameArray;
			node->add(*yyvsp[0].metaNamePtr);
			yyval.metaNameArray = node;
		}
break;
case 1232:
  if (!yytrial)
{ yyval.accessType = newNode<PlanNode::AccessType>(PlanNode::AccessType::TYPE_SEQUENTIAL); }
break;
case 1233:
  if (!yytrial)
{ yyval.accessType = newNode<PlanNode::AccessType>(PlanNode::AccessType::TYPE_INDICES); }
break;
case 1234:
yyval.accessType = yyvsp[-1].accessType;
break;
case 1235:
  if (!yytrial)
{ yyval.accessType = yyvsp[-4].accessType; }
break;
case 1236:
  if (!yytrial)
{ yyval.accessType = newNode<PlanNode::AccessType>(PlanNode::AccessType::TYPE_NAVIGATIONAL); }
break;
case 1237:
  if (!yytrial)
{
			yyval.accessType = yyvsp[-3].accessType;
			yyval.accessType->items.insert(0).indexName = *yyvsp[-2].metaNamePtr;
		}
break;
case 1238:
  if (!yytrial)
{
			PlanNode::AccessItem& item = yyvsp[-1].accessType->items.add();
			item.indexName = *yyvsp[0].metaNamePtr;
		}
break;
case 1239:
  if (!yytrial)
{
			PlanNode::AccessItem& item = yyvsp[-3].accessType->items.add();
			item.indexName = *yyvsp[0].metaNamePtr;
		}
break;
case 1241:
yyval.accessType = yyvsp[-2].accessType;
break;
case 1243:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1244:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 1245:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode); }
break;
case 1246:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].valueExprNode); }
break;
case 1247:
  if (!yytrial)
{
			OrderNode* node = newNode<OrderNode>(yyvsp[-2].valueExprNode);
			node->descending = yyvsp[-1].boolVal;
			node->nullsPlacement = yyvsp[0].nullsPlacement;
			yyval.valueExprNode = node;
		}
break;
case 1248:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1249:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1250:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 1251:
  if (!yytrial)
{ yyval.nullsPlacement = OrderNode::NULLS_DEFAULT; }
break;
case 1252:
  if (!yytrial)
{ yyval.nullsPlacement = yyvsp[0].nullsPlacement; }
break;
case 1253:
  if (!yytrial)
{ yyval.nullsPlacement = OrderNode::NULLS_FIRST; }
break;
case 1254:
  if (!yytrial)
{ yyval.nullsPlacement = OrderNode::NULLS_LAST; }
break;
case 1255:
  if (!yytrial)
{
			yyval.rowsClause = newNode<RowsClause>();
			yyval.rowsClause->length = yyvsp[0].valueExprNode;
		}
break;
case 1256:
  if (!yytrial)
{
			yyval.rowsClause = newNode<RowsClause>();
			yyval.rowsClause->skip = newNode<ArithmeticNode>(blr_subtract, true, yyvsp[-2].valueExprNode, MAKE_const_slong(1));
			yyval.rowsClause->length = newNode<ArithmeticNode>(blr_add, true,
				newNode<ArithmeticNode>(blr_subtract, true, yyvsp[0].valueExprNode, yyvsp[-2].valueExprNode), MAKE_const_slong(1));
		}
break;
case 1257:
  if (!yytrial)
{ yyval.rowsClause = NULL; }
break;
case 1261:
  if (!yytrial)
{ yyval.valueExprNode = NULL; }
break;
case 1262:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[-1].valueExprNode; }
break;
case 1265:
  if (!yytrial)
{ yyval.valueExprNode = NULL; }
break;
case 1266:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[-2].valueExprNode; }
break;
case 1267:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(1); }
break;
case 1268:
yyval.nestFieldArray = NOTRIAL(&yyvsp[0].storeNode->dsqlFields);
break;
case 1269:
  if (!yytrial)
{
			StoreNode* node = yyval.storeNode = yyvsp[-7].storeNode;
			node->dsqlValues = yyvsp[-2].valueListNode;
			node->dsqlReturning = yyvsp[0].returningClause;
		}
break;
case 1270:
  if (!yytrial)
{
			StoreNode* node = yyval.storeNode = yyvsp[-4].storeNode;
			node->dsqlRse = yyvsp[-1].selectExprNode;
			node->dsqlReturning = yyvsp[0].returningClause;
			yyval.storeNode = node;
		}
break;
case 1271:
  if (!yytrial)
{
			StoreNode* node = yyval.storeNode = yyvsp[-3].storeNode;
			node->dsqlReturning = yyvsp[0].returningClause;
			yyval.storeNode = node;
		}
break;
case 1272:
  if (!yytrial)
{
			StoreNode* node = newNode<StoreNode>();
			node->dsqlRelation = yyvsp[0].relSourceNode;
			yyval.storeNode = node;
		}
break;
case 1273:
  if (!yytrial)
{
				MergeNode* node = yyval.mergeNode = newNode<MergeNode>();
				node->relation = yyvsp[-4].relSourceNode;
				node->usingClause = yyvsp[-2].recSourceNode;
				node->condition = yyvsp[0].boolExprNode;
			}
break;
case 1274:
yyval.mergeNode = yyvsp[0].mergeNode;
break;
case 1275:
  if (!yytrial)
{
				MergeNode* node = yyval.mergeNode = yyvsp[-3].mergeNode;
				node->returning = yyvsp[0].returningClause;
			}
break;
case 1278:
yyval.mergeNode = yyvsp[-1].mergeNode;
break;
case 1281:
  if (!yytrial)
{ yyval.mergeMatchedClause = &yyvsp[-2].mergeNode->whenMatched.add(); }
break;
case 1282:
yyval.mergeMatchedClause = NOTRIAL(yyvsp[0].mergeMatchedClause);
break;
case 1284:
  if (!yytrial)
{ yyval.mergeNotMatchedClause = &yyvsp[-3].mergeNode->whenNotMatched.add(); }
break;
case 1285:
yyval.mergeNotMatchedClause = NOTRIAL(yyvsp[0].mergeNotMatchedClause);
break;
case 1287:
  if (!yytrial)
{ yyvsp[-4].mergeMatchedClause->assignments = yyvsp[0].compoundStmtNode; }
break;
case 1288:
  if (!yytrial)
{
			yyvsp[-6].mergeMatchedClause->condition = yyvsp[-4].boolExprNode;
			yyvsp[-6].mergeMatchedClause->assignments = yyvsp[0].compoundStmtNode;
		}
break;
case 1290:
  if (!yytrial)
{ yyvsp[-4].mergeMatchedClause->condition = yyvsp[-2].boolExprNode; }
break;
case 1291:
yyval.nestFieldArray = NOTRIAL(&yyvsp[-2].mergeNotMatchedClause->fields);
break;
case 1292:
  if (!yytrial)
{ yyvsp[-8].mergeNotMatchedClause->values = yyvsp[-1].valueListNode; }
break;
case 1293:
yyval.nestFieldArray = NOTRIAL(&yyvsp[-4].mergeNotMatchedClause->fields);
break;
case 1294:
  if (!yytrial)
{
			yyvsp[-10].mergeNotMatchedClause->values = yyvsp[-1].valueListNode;
			yyvsp[-10].mergeNotMatchedClause->condition = yyvsp[-8].boolExprNode;
		}
break;
case 1297:
  if (!yytrial)
{
			EraseNode* node = newNode<EraseNode>();
			node->dsqlRelation = yyvsp[-5].relSourceNode;
			node->dsqlBoolean = yyvsp[-4].boolExprNode;
			node->dsqlPlan = yyvsp[-3].planNode;
			node->dsqlOrder = yyvsp[-2].valueListNode;
			node->dsqlRows = yyvsp[-1].rowsClause;
			node->dsqlReturning = yyvsp[0].returningClause;
			yyval.stmtNode = node;
		}
break;
case 1298:
  if (!yytrial)
{
			EraseNode* node = newNode<EraseNode>();
			node->dsqlRelation = yyvsp[-2].relSourceNode;
			node->dsqlCursorName = *yyvsp[-1].metaNamePtr;
			node->dsqlReturning = yyvsp[0].returningClause;
			yyval.stmtNode = node;
		}
break;
case 1301:
  if (!yytrial)
{
			ModifyNode* node = newNode<ModifyNode>();
			node->dsqlRelation = yyvsp[-7].relSourceNode;
			node->statement = yyvsp[-5].compoundStmtNode;
			node->dsqlBoolean = yyvsp[-4].boolExprNode;
			node->dsqlPlan = yyvsp[-3].planNode;
			node->dsqlOrder = yyvsp[-2].valueListNode;
			node->dsqlRows = yyvsp[-1].rowsClause;
			node->dsqlReturning = yyvsp[0].returningClause;
			yyval.stmtNode = node;
		}
break;
case 1302:
  if (!yytrial)
{
			ModifyNode* node = newNode<ModifyNode>();
			node->dsqlRelation = yyvsp[-4].relSourceNode;
			node->statement = yyvsp[-2].compoundStmtNode;
			node->dsqlCursorName = *yyvsp[-1].metaNamePtr;
			node->dsqlReturning = yyvsp[0].returningClause;
			yyval.stmtNode = node;
		}
break;
case 1303:
  if (!yytrial)
{
				UpdateOrInsertNode* node = yyval.updInsNode = newNode<UpdateOrInsertNode>();
				node->relation = yyvsp[0].relSourceNode;
			}
break;
case 1304:
yyval.nestFieldArray = NOTRIAL(&yyvsp[0].updInsNode->fields);
break;
case 1305:
yyval.nestFieldArray = NOTRIAL(&yyvsp[-6].updInsNode->matching);
break;
case 1306:
  if (!yytrial)
{
				UpdateOrInsertNode* node = yyval.updInsNode = yyvsp[-9].updInsNode;
				node->values = yyvsp[-4].valueListNode;
				node->returning = yyvsp[0].returningClause;
			}
break;
case 1308:
yyval.nestFieldArray = yyvsp[-1].nestFieldArray;
break;
case 1310:
  if (!yytrial)
{ yyval.returningClause = NULL; }
break;
case 1311:
  if (!yytrial)
{
			yyval.returningClause = FB_NEW_POOL(getPool()) ReturningClause(getPool());
			yyval.returningClause->first = yyvsp[0].valueListNode;
		}
break;
case 1312:
  if (!yytrial)
{
			yyval.returningClause = FB_NEW_POOL(getPool()) ReturningClause(getPool());
			yyval.returningClause->first = yyvsp[-2].valueListNode;
			yyval.returningClause->second = yyvsp[0].valueListNode;
		}
break;
case 1313:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode); }
break;
case 1314:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].valueExprNode); }
break;
case 1315:
  if (!yytrial)
{ yyval.metaNamePtr = newNode<MetaName>(*yyvsp[0].metaNamePtr); }
break;
case 1316:
  if (!yytrial)
{
			yyval.compoundStmtNode = newNode<CompoundStmtNode>();
			yyval.compoundStmtNode->statements.add(yyvsp[0].stmtNode);
		}
break;
case 1317:
  if (!yytrial)
{
			yyvsp[-2].compoundStmtNode->statements.add(yyvsp[0].stmtNode);
			yyval.compoundStmtNode = yyvsp[-2].compoundStmtNode;
		}
break;
case 1318:
  if (!yytrial)
{
			AssignmentNode* node = newNode<AssignmentNode>();
			node->asgnTo = yyvsp[-2].fieldNode;
			node->asgnFrom = yyvsp[0].valueExprNode;
			yyval.stmtNode = node;
		}
break;
case 1319:
  if (!yytrial)
{
			AssignmentNode* node = newNode<AssignmentNode>();
			node->asgnTo = newNode<NullNode>();
			node->asgnFrom = yyvsp[0].valueExprNode;
			yyval.stmtNode = node;
		}
break;
case 1320:
  if (!yytrial)
{
			AssignmentNode* node = newNode<AssignmentNode>();
			node->asgnTo = newNode<NullNode>();
			node->asgnFrom = yyvsp[0].valueExprNode;
			yyval.stmtNode = node;
		}
break;
case 1321:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1323:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-1].valueListNode; }
break;
case 1324:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].fieldNode); }
break;
case 1325:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].fieldNode); }
break;
case 1327:
yyval.nestFieldArray = yyvsp[0].nestFieldArray;
break;
case 1330:
  if (!yytrial)
{ yyvsp[-1].nestFieldArray->add(yyvsp[0].fieldNode); }
break;
case 1331:
  if (!yytrial)
{ yyvsp[-3].nestFieldArray->add(yyvsp[0].fieldNode); }
break;
case 1333:
  if (!yytrial)
{
			FieldNode* fieldNode = newNode<FieldNode>();
			fieldNode->dsqlQualifier = *yyvsp[-2].metaNamePtr;
			fieldNode->dsqlName = *yyvsp[0].metaNamePtr;
			yyval.fieldNode = fieldNode;
		}
break;
case 1334:
  if (!yytrial)
{
			FieldNode* fieldNode = newNode<FieldNode>();
			fieldNode->dsqlQualifier = *yyvsp[-2].metaNamePtr;
			fieldNode->dsqlName = *yyvsp[0].metaNamePtr;
			fieldNode->dsqlCursorField = true;
			yyval.fieldNode = fieldNode;
		}
break;
case 1335:
  if (!yytrial)
{
			FieldNode* fieldNode = newNode<FieldNode>();
			fieldNode->dsqlName = *yyvsp[0].metaNamePtr;
			yyval.fieldNode = fieldNode;
		}
break;
case 1337:
  if (!yytrial)
{
			FieldNode* fieldNode = newNode<FieldNode>();
			fieldNode->dsqlQualifier = *yyvsp[-2].metaNamePtr;
			fieldNode->dsqlName = *yyvsp[0].metaNamePtr;
			yyval.fieldNode = fieldNode;
		}
break;
case 1338:
  if (!yytrial)
{ yyval.boolExprNode = valueToBool(yyvsp[0].valueExprNode); }
break;
case 1340:
  if (!yytrial)
{ yyval.boolExprNode = newNode<BinaryBoolNode>(blr_or, valueToBool(yyvsp[-2].valueExprNode), valueToBool(yyvsp[0].valueExprNode)); }
break;
case 1341:
  if (!yytrial)
{ yyval.boolExprNode = newNode<BinaryBoolNode>(blr_and, valueToBool(yyvsp[-2].valueExprNode), valueToBool(yyvsp[0].valueExprNode)); }
break;
case 1342:
  if (!yytrial)
{ yyval.boolExprNode = newNode<NotBoolNode>(valueToBool(yyvsp[0].valueExprNode)); }
break;
case 1343:
  if (!yytrial)
{ yyval.boolExprNode = yyvsp[-1].boolExprNode; }
break;
case 1344:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_eql, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode);
			node->dsqlCheckBoolean = true;
			yyval.boolExprNode = node;
		}
break;
case 1345:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_eql, yyvsp[-3].valueExprNode, yyvsp[0].valueExprNode);
			node->dsqlCheckBoolean = true;
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1357:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(yyvsp[-1].blrOp, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1358:
  if (!yytrial)
{ yyval.blrOp = blr_eql; }
break;
case 1359:
  if (!yytrial)
{ yyval.blrOp = blr_lss; }
break;
case 1360:
  if (!yytrial)
{ yyval.blrOp = blr_gtr; }
break;
case 1361:
  if (!yytrial)
{ yyval.blrOp = blr_geq; }
break;
case 1362:
  if (!yytrial)
{ yyval.blrOp = blr_leq; }
break;
case 1363:
  if (!yytrial)
{ yyval.blrOp = blr_leq; }
break;
case 1364:
  if (!yytrial)
{ yyval.blrOp = blr_geq; }
break;
case 1365:
  if (!yytrial)
{ yyval.blrOp = blr_neq; }
break;
case 1366:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(yyvsp[-4].blrOp, yyvsp[-5].valueExprNode);
			node->dsqlFlag = yyvsp[-3].cmpBoolFlag;
			node->dsqlSpecialArg = yyvsp[-1].selectExprNode;
			yyval.boolExprNode = node;
		}
break;
case 1367:
  if (!yytrial)
{ yyval.cmpBoolFlag = ComparativeBoolNode::DFLAG_ANSI_ALL; }
break;
case 1368:
  if (!yytrial)
{ yyval.cmpBoolFlag = ComparativeBoolNode::DFLAG_ANSI_ANY; }
break;
case 1369:
  if (!yytrial)
{ yyval.cmpBoolFlag = ComparativeBoolNode::DFLAG_ANSI_ANY; }
break;
case 1370:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_equiv, yyvsp[-4].valueExprNode, yyvsp[0].valueExprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1371:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_equiv, yyvsp[-5].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1372:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_between, yyvsp[-4].valueExprNode, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1373:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_between, yyvsp[-5].valueExprNode, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1375:
  if (!yytrial)
{ yyval.valueExprNode = newNode<BoolAsValueNode>(yyvsp[-1].boolExprNode); }
break;
case 1376:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(yyvsp[-1].blrOp, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1377:
  if (!yytrial)
{
			ComparativeBoolNode* cmpNode = newNode<ComparativeBoolNode>(yyvsp[-1].blrOp, yyvsp[-3].valueExprNode, yyvsp[0].valueExprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(cmpNode);
		}
break;
case 1378:
  if (!yytrial)
{ yyval.blrOp = blr_containing; }
break;
case 1379:
  if (!yytrial)
{ yyval.blrOp = blr_starting; }
break;
case 1380:
  if (!yytrial)
{ yyval.blrOp = blr_starting; }
break;
case 1381:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_like, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1382:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_like, yyvsp[-4].valueExprNode, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1383:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_like, yyvsp[-3].valueExprNode, yyvsp[0].valueExprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1384:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_like, yyvsp[-5].valueExprNode, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1385:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_similar, yyvsp[-3].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1386:
  if (!yytrial)
{ yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_similar, yyvsp[-5].valueExprNode, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1387:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_similar, yyvsp[-4].valueExprNode, yyvsp[0].valueExprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1388:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_similar, yyvsp[-6].valueExprNode, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode);
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1389:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_eql, yyvsp[-2].valueExprNode);
			node->dsqlFlag = ComparativeBoolNode::DFLAG_ANSI_ANY;
			node->dsqlSpecialArg = yyvsp[0].exprNode;
			yyval.boolExprNode = node;
		}
break;
case 1390:
  if (!yytrial)
{
			ComparativeBoolNode* node = newNode<ComparativeBoolNode>(blr_eql, yyvsp[-3].valueExprNode);
			node->dsqlFlag = ComparativeBoolNode::DFLAG_ANSI_ANY;
			node->dsqlSpecialArg = yyvsp[0].exprNode;
			yyval.boolExprNode = newNode<NotBoolNode>(node);
		}
break;
case 1391:
  if (!yytrial)
{ yyval.boolExprNode = newNode<RseBoolNode>(blr_any, yyvsp[-1].selectExprNode); }
break;
case 1392:
  if (!yytrial)
{ yyval.boolExprNode = newNode<RseBoolNode>(blr_unique, yyvsp[-1].selectExprNode); }
break;
case 1393:
  if (!yytrial)
{
			yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_eql,
					newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_TRIGGER_ACTION)),
					MAKE_const_slong(1));
		}
break;
case 1394:
  if (!yytrial)
{
			yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_eql,
					newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_TRIGGER_ACTION)),
					MAKE_const_slong(2));
		}
break;
case 1395:
  if (!yytrial)
{
			yyval.boolExprNode = newNode<ComparativeBoolNode>(blr_eql,
					newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_TRIGGER_ACTION)),
					MAKE_const_slong(3));
		}
break;
case 1396:
  if (!yytrial)
{ yyval.boolExprNode = newNode<MissingBoolNode>(yyvsp[-2].valueExprNode); }
break;
case 1397:
  if (!yytrial)
{ yyval.boolExprNode = newNode<MissingBoolNode>(yyvsp[-2].valueExprNode, true); }
break;
case 1398:
  if (!yytrial)
{ yyval.boolExprNode = newNode<NotBoolNode>(newNode<MissingBoolNode>(yyvsp[-3].valueExprNode)); }
break;
case 1399:
  if (!yytrial)
{ yyval.boolExprNode = newNode<NotBoolNode>(newNode<MissingBoolNode>(yyvsp[-3].valueExprNode, true)); }
break;
case 1400:
  if (!yytrial)
{ yyval.exprNode = yyvsp[0].selectExprNode; }
break;
case 1401:
  if (!yytrial)
{ yyval.exprNode = yyvsp[-1].valueListNode; }
break;
case 1402:
  if (!yytrial)
{ yyval.selectExprNode = yyvsp[-1].selectExprNode; }
break;
case 1403:
  if (!yytrial)
{
			yyval.createAlterUserNode = newNode<CreateAlterUserNode>(CreateAlterUserNode::USER_ADD, *yyvsp[0].metaNamePtr);
		}
break;
case 1404:
yyval.createAlterUserNode = yyvsp[0].createAlterUserNode;
break;
case 1405:
yyval.createAlterUserNode = yyvsp[-2].createAlterUserNode;
break;
case 1406:
  if (!yytrial)
{
			yyval.createAlterUserNode = yyvsp[-4].createAlterUserNode;
		}
break;
case 1407:
  if (!yytrial)
{
			yyval.createAlterUserNode = newNode<CreateAlterUserNode>(CreateAlterUserNode::USER_MOD, *yyvsp[-1].metaNamePtr);
		}
break;
case 1408:
  if (!yytrial)
{
			yyval.createAlterUserNode = yyvsp[-4].createAlterUserNode;
		}
break;
case 1409:
  if (!yytrial)
{
			yyval.createAlterUserNode = newNode<CreateAlterUserNode>(CreateAlterUserNode::USER_MOD, "");
		}
break;
case 1410:
  if (!yytrial)
{
			yyval.createAlterUserNode = yyvsp[-4].createAlterUserNode;
		}
break;
case 1411:
  if (!yytrial)
{
			yyval.createAlterUserNode = newNode<CreateAlterUserNode>(CreateAlterUserNode::USER_RPL, *yyvsp[-1].metaNamePtr);
		}
break;
case 1412:
  if (!yytrial)
{
			yyval.createAlterUserNode = yyvsp[-4].createAlterUserNode;
		}
break;
case 1418:
yyval.createAlterUserNode = yyvsp[-1].createAlterUserNode;
break;
case 1420:
  if (!yytrial)
{ setClause(yyvsp[-2].createAlterUserNode->firstName, "FIRSTNAME", yyvsp[0].stringPtr); }
break;
case 1421:
  if (!yytrial)
{ setClause(yyvsp[-2].createAlterUserNode->middleName, "MIDDLENAME", yyvsp[0].stringPtr); }
break;
case 1422:
  if (!yytrial)
{ setClause(yyvsp[-2].createAlterUserNode->lastName, "LASTNAME", yyvsp[0].stringPtr); }
break;
case 1423:
  if (!yytrial)
{ setClause(yyvsp[-2].createAlterUserNode->password, "PASSWORD", yyvsp[0].stringPtr); }
break;
case 1424:
  if (!yytrial)
{ setClause(yyvsp[-3].createAlterUserNode->adminRole, "ADMIN ROLE", Nullable<bool>(true)); }
break;
case 1425:
  if (!yytrial)
{ setClause(yyvsp[-3].createAlterUserNode->adminRole, "ADMIN ROLE", Nullable<bool>(false)); }
break;
case 1426:
  if (!yytrial)
{ setClause(yyvsp[-1].createAlterUserNode->active, "ACTIVE/INACTIVE", Nullable<bool>(true)); }
break;
case 1427:
  if (!yytrial)
{ setClause(yyvsp[-1].createAlterUserNode->active, "ACTIVE/INACTIVE", Nullable<bool>(false)); }
break;
case 1428:
  if (!yytrial)
{ setClause(yyvsp[-3].createAlterUserNode->plugin, "USING PLUGIN", yyvsp[0].metaNamePtr); }
break;
case 1433:
  if (!yytrial)
{
			yyvsp[-3].createAlterUserNode->addProperty(yyvsp[-2].metaNamePtr, yyvsp[0].stringPtr);
		}
break;
case 1434:
  if (!yytrial)
{
			yyvsp[-2].createAlterUserNode->addProperty(yyvsp[0].metaNamePtr);
		}
break;
case 1435:
yyval.mappingOp = MappingNode::MAP_ADD;
break;
case 1436:
  if (!yytrial)
{
				yyval.mappingNode = yyvsp[0].mappingNode;
				yyval.mappingNode->global = yyvsp[-2].boolVal;
			}
break;
case 1437:
yyval.mappingNode = yyvsp[0].mappingNode;
break;
case 1438:
  if (!yytrial)
{
				yyval.mappingNode = yyvsp[-2].mappingNode;
			}
break;
case 1439:
yyval.mappingOp = MappingNode::MAP_MOD;
break;
case 1440:
  if (!yytrial)
{
				yyval.mappingNode = yyvsp[0].mappingNode;
				yyval.mappingNode->global = yyvsp[-2].boolVal;
			}
break;
case 1441:
  if (!yytrial)
{
				yyval.mappingNode = yyvsp[-2].mappingNode;
			}
break;
case 1442:
yyval.mappingOp = MappingNode::MAP_RPL;
break;
case 1443:
  if (!yytrial)
{
				yyval.mappingNode = yyvsp[0].mappingNode;
				yyval.mappingNode->global = yyvsp[-2].boolVal;
			}
break;
case 1444:
  if (!yytrial)
{
				yyval.mappingNode = yyvsp[-2].mappingNode;
			}
break;
case 1445:
  if (!yytrial)
{
 			MappingNode* node = newNode<MappingNode>(MappingNode::MAP_DROP, *yyvsp[0].metaNamePtr);
			node->global = yyvsp[-1].boolVal;
			yyval.mappingNode = node;
		}
break;
case 1446:
  if (!yytrial)
{
				yyval.mappingNode = newNode<MappingNode>(yyvsp[-1].mappingOp, *yyvsp[0].metaNamePtr);
			}
break;
case 1447:
yyval.mappingNode = yyvsp[-1].mappingNode;
break;
case 1448:
yyval.mappingNode = yyvsp[-4].mappingNode;
break;
case 1449:
  if (!yytrial)
{
				yyval.mappingNode = yyvsp[-6].mappingNode;
			}
break;
case 1450:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 1451:
  if (!yytrial)
{
			yyvsp[-2].mappingNode->fromType = yyvsp[-1].metaNamePtr;
			yyvsp[-2].mappingNode->from = yyvsp[0].intlStringPtr;
		}
break;
case 1452:
  if (!yytrial)
{
			yyvsp[-2].mappingNode->fromType = yyvsp[0].metaNamePtr;
			yyvsp[-2].mappingNode->from = newNode<IntlString>("*");
		}
break;
case 1454:
  if (!yytrial)
{ yyval.metaNamePtr = newNode<MetaName>("USER"); }
break;
case 1455:
  if (!yytrial)
{ yyval.metaNamePtr = newNode<MetaName>("GROUP"); }
break;
case 1457:
  if (!yytrial)
{ yyval.intlStringPtr = newNode<IntlString>(yyvsp[0].metaNamePtr->c_str()); }
break;
case 1458:
  if (!yytrial)
{
			yyvsp[-3].mappingNode->mode = 'P';
			yyvsp[-3].mappingNode->plugin = yyvsp[-1].metaNamePtr;
			yyvsp[-3].mappingNode->db = yyvsp[0].metaNamePtr;
		}
break;
case 1459:
  if (!yytrial)
{
			yyvsp[-3].mappingNode->mode = 'P';
			yyvsp[-3].mappingNode->db = yyvsp[0].metaNamePtr;
		}
break;
case 1460:
  if (!yytrial)
{
			yyvsp[-3].mappingNode->mode = 'S';
		}
break;
case 1461:
  if (!yytrial)
{
			yyvsp[-2].mappingNode->mode = 'M';
			yyvsp[-2].mappingNode->db = yyvsp[0].metaNamePtr;
		}
break;
case 1462:
  if (!yytrial)
{
			yyvsp[-2].mappingNode->mode = '*';
			yyvsp[-2].mappingNode->db = yyvsp[0].metaNamePtr;
		}
break;
case 1463:
  if (!yytrial)
{ yyval.metaNamePtr = NULL; }
break;
case 1464:
  if (!yytrial)
{ yyval.metaNamePtr = yyvsp[0].metaNamePtr; }
break;
case 1465:
  if (!yytrial)
{
			yyvsp[-3].mappingNode->role = yyvsp[-1].boolVal;
			yyvsp[-3].mappingNode->to = yyvsp[0].metaNamePtr;
		}
break;
case 1466:
  if (!yytrial)
{
			yyvsp[-2].mappingNode->role = yyvsp[0].boolVal;
		}
break;
case 1467:
  if (!yytrial)
{ yyval.boolVal = true; }
break;
case 1468:
  if (!yytrial)
{ yyval.boolVal = false; }
break;
case 1470:
  if (!yytrial)
{ yyval.valueExprNode = newNode<BoolAsValueNode>(yyvsp[0].boolExprNode); }
break;
case 1472:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[-1].valueExprNode; }
break;
case 1476:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].fieldNode; }
break;
case 1478:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1485:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1486:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1487:
  if (!yytrial)
{ yyval.valueExprNode = newNode<NegateNode>(yyvsp[0].valueExprNode); }
break;
case 1488:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1489:
  if (!yytrial)
{ yyval.valueExprNode = newNode<ArithmeticNode>(blr_add, (client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1490:
  if (!yytrial)
{ yyval.valueExprNode = newNode<ConcatenateNode>(yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1491:
  if (!yytrial)
{ yyval.valueExprNode = newNode<CollateNode>(yyvsp[-2].valueExprNode, *yyvsp[0].metaNamePtr); }
break;
case 1492:
  if (!yytrial)
{ yyval.valueExprNode = newNode<ArithmeticNode>(blr_subtract, (client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1493:
  if (!yytrial)
{ yyval.valueExprNode = newNode<ArithmeticNode>(blr_multiply, (client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1494:
  if (!yytrial)
{ yyval.valueExprNode = newNode<ArithmeticNode>(blr_divide, (client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1495:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[-1].valueExprNode; }
break;
case 1496:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1497:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1498:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1499:
  if (!yytrial)
{ yyval.valueExprNode = newNode<RecordKeyNode>(yyvsp[0].blrOp); }
break;
case 1500:
  if (!yytrial)
{ yyval.valueExprNode = newNode<RecordKeyNode>(yyvsp[0].blrOp, *yyvsp[-2].metaNamePtr); }
break;
case 1501:
  if (!yytrial)
{ yyval.valueExprNode = newNode<DomainValidationNode>(); }
break;
case 1502:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1503:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1504:
  if (!yytrial)
{ yyval.blrOp = blr_dbkey; }
break;
case 1505:
  if (!yytrial)
{ yyval.blrOp = blr_record_version2; }
break;
case 1506:
  if (!yytrial)
{
			if (client_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_dialect_datatype_unsupport) << Arg::Num(client_dialect) <<
						  												  Arg::Str("DATE"));
			}

			if (db_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_db_dialect_dtype_unsupport) << Arg::Num(db_dialect) <<
						  												  Arg::Str("DATE"));
			}

			yyval.valueExprNode = newNode<CurrentDateNode>();
		}
break;
case 1507:
  if (!yytrial)
{
			if (client_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_dialect_datatype_unsupport) << Arg::Num(client_dialect) <<
						  												  Arg::Str("TIME"));
			}

			if (db_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_db_dialect_dtype_unsupport) << Arg::Num(db_dialect) <<
						  												  Arg::Str("TIME"));
			}

			yyval.valueExprNode = newNode<CurrentTimeNode>(yyvsp[0].uintVal);
		}
break;
case 1508:
  if (!yytrial)
{
			if (client_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_dialect_datatype_unsupport) << Arg::Num(client_dialect) <<
						  												  Arg::Str("TIME"));
			}

			if (db_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_db_dialect_dtype_unsupport) << Arg::Num(db_dialect) <<
						  												  Arg::Str("TIME"));
			}

			CurrentTimeNode* node = newNode<CurrentTimeNode>(yyvsp[0].uintVal);
			node->dsqlLocal = true;
			yyval.valueExprNode = node;
		}
break;
case 1509:
  if (!yytrial)
{ yyval.valueExprNode = newNode<CurrentTimeStampNode>(yyvsp[0].uintVal); }
break;
case 1510:
  if (!yytrial)
{
			CurrentTimeStampNode* node = newNode<CurrentTimeStampNode>(yyvsp[0].uintVal);
			node->dsqlLocal = true;
			yyval.valueExprNode = node;
		}
break;
case 1511:
  if (!yytrial)
{ yyval.uintVal = DEFAULT_TIME_PRECISION; }
break;
case 1512:
  if (!yytrial)
{ yyval.uintVal = yyvsp[-1].int32Val; }
break;
case 1513:
  if (!yytrial)
{ yyval.uintVal = DEFAULT_TIMESTAMP_PRECISION; }
break;
case 1514:
  if (!yytrial)
{ yyval.uintVal = yyvsp[-1].int32Val; }
break;
case 1515:
  if (!yytrial)
{
			ArrayNode* node = newNode<ArrayNode>(yyvsp[-3].fieldNode);
			node->field->dsqlIndices = yyvsp[-1].valueListNode;
			yyval.valueExprNode = node;
		}
break;
case 1516:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(0); }
break;
case 1517:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 1518:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(yyvsp[0].valueExprNode); }
break;
case 1519:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-2].valueListNode->add(yyvsp[0].valueExprNode); }
break;
case 1521:
  if (!yytrial)
{ yyval.valueExprNode = newNode<NegateNode>(yyvsp[0].valueExprNode); }
break;
case 1523:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(yyvsp[0].int32Val); }
break;
case 1524:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_constant(yyvsp[0].stringPtr->c_str(), CONSTANT_DOUBLE); }
break;
case 1525:
  if (!yytrial)
{
			SINT64 signedNumber = (SINT64) yyvsp[0].scaledNumber.number;

			if (yyvsp[0].scaledNumber.hex && signedNumber < 0)
				yyval.valueExprNode = newNode<NegateNode>(MAKE_const_sint64(-signedNumber, yyvsp[0].scaledNumber.scale));
			else
				yyval.valueExprNode = MAKE_const_sint64(signedNumber, yyvsp[0].scaledNumber.scale);
		}
break;
case 1526:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_sint64((SINT64) yyvsp[0].scaledNumber.number, yyvsp[0].scaledNumber.scale); }
break;
case 1528:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_str_constant(yyvsp[0].intlStringPtr, lex.att_charset); }
break;
case 1529:
  if (!yytrial)
{
			if (client_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_dialect_datatype_unsupport) << Arg::Num(client_dialect) <<
						  												  Arg::Str("DATE"));
			}
			if (db_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_db_dialect_dtype_unsupport) << Arg::Num(db_dialect) <<
						  												  Arg::Str("DATE"));
			}
			yyval.valueExprNode = MAKE_constant(yyvsp[0].intlStringPtr->getString().c_str(), CONSTANT_DATE);
		}
break;
case 1530:
  if (!yytrial)
{
			if (client_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_dialect_datatype_unsupport) << Arg::Num(client_dialect) <<
						  												  Arg::Str("TIME"));
			}
			if (db_dialect < SQL_DIALECT_V6_TRANSITION)
			{
				ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-104) <<
						  Arg::Gds(isc_sql_db_dialect_dtype_unsupport) << Arg::Num(db_dialect) <<
						  												  Arg::Str("TIME"));
			}
			yyval.valueExprNode = MAKE_constant(yyvsp[0].intlStringPtr->getString().c_str(), CONSTANT_TIME);
		}
break;
case 1531:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_constant(yyvsp[0].intlStringPtr->getString().c_str(), CONSTANT_TIMESTAMP); }
break;
case 1532:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_constant("", CONSTANT_BOOLEAN); }
break;
case 1533:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_constant("1", CONSTANT_BOOLEAN); }
break;
case 1534:
  if (!yytrial)
{ yyval.valueExprNode = make_parameter(); }
break;
case 1535:
  if (!yytrial)
{ yyval.valueExprNode = newNode<CurrentUserNode>(); }
break;
case 1536:
  if (!yytrial)
{ yyval.valueExprNode = newNode<CurrentUserNode>(); }
break;
case 1537:
  if (!yytrial)
{ yyval.valueExprNode = newNode<CurrentRoleNode>(); }
break;
case 1538:
  if (!yytrial)
{ yyval.valueExprNode = newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_CONNECTION_ID)); }
break;
case 1539:
  if (!yytrial)
{ yyval.valueExprNode = newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_TRANSACTION_ID)); }
break;
case 1540:
  if (!yytrial)
{ yyval.valueExprNode = newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_GDSCODE)); }
break;
case 1541:
  if (!yytrial)
{ yyval.valueExprNode = newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_SQLCODE)); }
break;
case 1542:
  if (!yytrial)
{ yyval.valueExprNode = newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_SQLSTATE)); }
break;
case 1543:
  if (!yytrial)
{ yyval.valueExprNode = newNode<InternalInfoNode>(MAKE_const_slong(INFO_TYPE_ROWS_AFFECTED)); }
break;
case 1545:
  if (!yytrial)
{
			yyval.intlStringPtr = yyvsp[0].intlStringPtr;
			yyval.intlStringPtr->setCharSet(*yyvsp[-1].metaNamePtr);

			StrMark* mark = strMarks.get(yyvsp[0].intlStringPtr);

			if (mark)	/* hex string is not in strMarks*/
				mark->introduced = true;
		}
break;
case 1546:
  if (!yytrial)
{ yyval.stringPtr = newString(yyvsp[0].intlStringPtr->toUtf8(scratch)); }
break;
case 1548:
  if (!yytrial)
{ yyval.int32Val = -yyvsp[0].int32Val; }
break;
case 1549:
  if (!yytrial)
{
			if (yyvsp[0].int32Val > SHRT_POS_MAX)
				yyabandon(-842, isc_expec_short);	/* Short integer expected*/

			yyval.int32Val = yyvsp[0].int32Val;
		}
break;
case 1550:
  if (!yytrial)
{
			if (yyvsp[0].int32Val > SHRT_NEG_MAX)
				yyabandon(-842, isc_expec_short);	/* Short integer expected*/

			yyval.int32Val = yyvsp[0].int32Val;
		}
break;
case 1551:
  if (!yytrial)
{
			if (yyvsp[0].int32Val == 0)
				yyabandon(-842, isc_expec_positive);	/* Positive number expected*/

			yyval.int32Val = yyvsp[0].int32Val;
		}
break;
case 1552:
  if (!yytrial)
{
			if (yyvsp[0].int32Val > SHRT_UNSIGNED_MAX)
				yyabandon(-842, isc_expec_ushort);	/* Unsigned short integer expected*/

			yyval.int32Val = yyvsp[0].int32Val;
		}
break;
case 1554:
  if (!yytrial)
{ yyval.int32Val = -yyvsp[0].int32Val; }
break;
case 1555:
  if (!yytrial)
{ yyval.int32Val = yyvsp[0].int32Val;}
break;
case 1556:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].aggNode; }
break;
case 1562:
  if (!yytrial)
{ yyval.aggNode = newNode<CountAggNode>(false, (client_dialect < SQL_DIALECT_V6_TRANSITION)); }
break;
case 1563:
  if (!yytrial)
{ yyval.aggNode = newNode<CountAggNode>(false, (client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-1].valueExprNode); }
break;
case 1564:
  if (!yytrial)
{ yyval.aggNode = newNode<CountAggNode>(true, (client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-1].valueExprNode); }
break;
case 1565:
  if (!yytrial)
{
			yyval.aggNode = newNode<SumAggNode>(false,
				(client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-1].valueExprNode);
		}
break;
case 1566:
  if (!yytrial)
{
			yyval.aggNode = newNode<SumAggNode>(true,
				(client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-1].valueExprNode);
		}
break;
case 1567:
  if (!yytrial)
{
			yyval.aggNode = newNode<AvgAggNode>(false,
				(client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-1].valueExprNode);
		}
break;
case 1568:
  if (!yytrial)
{
			yyval.aggNode = newNode<AvgAggNode>(true,
				(client_dialect < SQL_DIALECT_V6_TRANSITION), yyvsp[-1].valueExprNode);
		}
break;
case 1569:
  if (!yytrial)
{ yyval.aggNode = newNode<MaxMinAggNode>(MaxMinAggNode::TYPE_MIN, yyvsp[-1].valueExprNode); }
break;
case 1570:
  if (!yytrial)
{ yyval.aggNode = newNode<MaxMinAggNode>(MaxMinAggNode::TYPE_MIN, yyvsp[-1].valueExprNode); }
break;
case 1571:
  if (!yytrial)
{ yyval.aggNode = newNode<MaxMinAggNode>(MaxMinAggNode::TYPE_MAX, yyvsp[-1].valueExprNode); }
break;
case 1572:
  if (!yytrial)
{ yyval.aggNode = newNode<MaxMinAggNode>(MaxMinAggNode::TYPE_MAX, yyvsp[-1].valueExprNode); }
break;
case 1573:
  if (!yytrial)
{ yyval.aggNode = newNode<ListAggNode>(false, yyvsp[-2].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1574:
  if (!yytrial)
{ yyval.aggNode = newNode<ListAggNode>(true, yyvsp[-2].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1575:
  if (!yytrial)
{ yyval.aggNode = newNode<StdDevAggNode>(StdDevAggNode::TYPE_STDDEV_SAMP, yyvsp[-1].valueExprNode); }
break;
case 1576:
  if (!yytrial)
{ yyval.aggNode = newNode<StdDevAggNode>(StdDevAggNode::TYPE_STDDEV_POP, yyvsp[-1].valueExprNode); }
break;
case 1577:
  if (!yytrial)
{ yyval.aggNode = newNode<StdDevAggNode>(StdDevAggNode::TYPE_VAR_SAMP, yyvsp[-1].valueExprNode); }
break;
case 1578:
  if (!yytrial)
{ yyval.aggNode = newNode<StdDevAggNode>(StdDevAggNode::TYPE_VAR_POP, yyvsp[-1].valueExprNode); }
break;
case 1579:
  if (!yytrial)
{ yyval.aggNode = newNode<CorrAggNode>(CorrAggNode::TYPE_COVAR_SAMP, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1580:
  if (!yytrial)
{ yyval.aggNode = newNode<CorrAggNode>(CorrAggNode::TYPE_COVAR_POP, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1581:
  if (!yytrial)
{ yyval.aggNode = newNode<CorrAggNode>(CorrAggNode::TYPE_CORR, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1582:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_AVGX, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1583:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_AVGY, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1584:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrCountAggNode>(yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1585:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_INTERCEPT, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1586:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_R2, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1587:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_SLOPE, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1588:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_SXX, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1589:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_SXY, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1590:
  if (!yytrial)
{ yyval.aggNode = newNode<RegrAggNode>(RegrAggNode::TYPE_REGR_SYY, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1591:
  if (!yytrial)
{ yyval.aggNode = newNode<DenseRankWinNode>(); }
break;
case 1592:
  if (!yytrial)
{ yyval.aggNode = newNode<RankWinNode>(); }
break;
case 1593:
  if (!yytrial)
{ yyval.aggNode = newNode<RowNumberWinNode>(); }
break;
case 1594:
  if (!yytrial)
{ yyval.aggNode = newNode<FirstValueWinNode>(yyvsp[-1].valueExprNode); }
break;
case 1595:
  if (!yytrial)
{ yyval.aggNode = newNode<LastValueWinNode>(yyvsp[-1].valueExprNode); }
break;
case 1596:
  if (!yytrial)
{ yyval.aggNode = newNode<NthValueWinNode>(yyvsp[-4].valueExprNode, yyvsp[-2].valueExprNode, yyvsp[0].valueExprNode); }
break;
case 1597:
  if (!yytrial)
{ yyval.aggNode = newNode<LagWinNode>(yyvsp[-5].valueExprNode, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1598:
  if (!yytrial)
{ yyval.aggNode = newNode<LagWinNode>(yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode, newNode<NullNode>()); }
break;
case 1599:
  if (!yytrial)
{ yyval.aggNode = newNode<LagWinNode>(yyvsp[-1].valueExprNode, MAKE_const_slong(1), newNode<NullNode>()); }
break;
case 1600:
  if (!yytrial)
{ yyval.aggNode = newNode<LeadWinNode>(yyvsp[-5].valueExprNode, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1601:
  if (!yytrial)
{ yyval.aggNode = newNode<LeadWinNode>(yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode, newNode<NullNode>()); }
break;
case 1602:
  if (!yytrial)
{ yyval.aggNode = newNode<LeadWinNode>(yyvsp[-1].valueExprNode, MAKE_const_slong(1), newNode<NullNode>()); }
break;
case 1603:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(NthValueWinNode::FROM_FIRST); }
break;
case 1604:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(NthValueWinNode::FROM_FIRST); }
break;
case 1605:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_const_slong(NthValueWinNode::FROM_LAST); }
break;
case 1608:
  if (!yytrial)
{ yyval.valueExprNode = newNode<OverNode>(yyvsp[-5].aggNode, yyvsp[-2].valueListNode, yyvsp[-1].valueListNode); }
break;
case 1609:
  if (!yytrial)
{ yyval.valueListNode = NULL; }
break;
case 1610:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[0].valueListNode; }
break;
case 1611:
  if (!yytrial)
{ yyval.valueExprNode = MAKE_str_constant(newIntlString(","), lex.att_charset); }
break;
case 1612:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1615:
  if (!yytrial)
{ yyval.valueExprNode = newNode<ExtractNode>(yyvsp[-3].blrOp, yyvsp[-1].valueExprNode); }
break;
case 1619:
  if (!yytrial)
{ yyval.valueExprNode = newNode<StrLenNode>(blr_strlen_bit, yyvsp[-1].valueExprNode); }
break;
case 1620:
  if (!yytrial)
{ yyval.valueExprNode = newNode<StrLenNode>(blr_strlen_char, yyvsp[-1].valueExprNode); }
break;
case 1621:
  if (!yytrial)
{ yyval.valueExprNode = newNode<StrLenNode>(blr_strlen_char, yyvsp[-1].valueExprNode); }
break;
case 1622:
  if (!yytrial)
{ yyval.valueExprNode = newNode<StrLenNode>(blr_strlen_octet, yyvsp[-1].valueExprNode); }
break;
case 1623:
  if (!yytrial)
{ yyval.valueExprNode = newNode<SysFuncCallNode>(*yyvsp[-3].metaNamePtr, yyvsp[-1].valueListNode); }
break;
case 1624:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].sysFuncCallNode; }
break;
case 1676:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-6].metaNamePtr,
				newNode<ValueListNode>(yyvsp[-4].valueExprNode)->add(MAKE_const_slong(yyvsp[-3].blrOp))->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1677:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-7].metaNamePtr,
				newNode<ValueListNode>(yyvsp[-3].valueExprNode)->add(MAKE_const_slong(yyvsp[-5].blrOp))->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1678:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-7].metaNamePtr,
				newNode<ValueListNode>(MAKE_const_slong(yyvsp[-5].blrOp))->add(yyvsp[-3].valueExprNode)->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1679:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-7].metaNamePtr,
				newNode<ValueListNode>(MAKE_const_slong(yyvsp[-5].blrOp))->add(yyvsp[-3].valueExprNode)->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1680:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-9].metaNamePtr,
				newNode<ValueListNode>(yyvsp[-7].valueExprNode)->add(yyvsp[-5].valueExprNode)->add(yyvsp[-3].valueExprNode)->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1681:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-7].metaNamePtr,
				newNode<ValueListNode>(yyvsp[-5].valueExprNode)->add(yyvsp[-3].valueExprNode)->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1682:
  if (!yytrial)
{
			yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-5].metaNamePtr, newNode<ValueListNode>(yyvsp[-3].valueExprNode)->add(yyvsp[-1].valueExprNode));
			yyval.sysFuncCallNode->dsqlSpecialSyntax = true;
		}
break;
case 1683:
  if (!yytrial)
{ yyval.sysFuncCallNode = newNode<SysFuncCallNode>(*yyvsp[-3].metaNamePtr, yyvsp[-1].valueListNode); }
break;
case 1686:
  if (!yytrial)
{ yyval.valueExprNode = newNode<StrCaseNode>(blr_upcase, yyvsp[-1].valueExprNode); }
break;
case 1687:
  if (!yytrial)
{ yyval.valueExprNode = newNode<StrCaseNode>(blr_lowcase, yyvsp[-1].valueExprNode); }
break;
case 1688:
  if (!yytrial)
{
			/* SQL spec requires numbering to start with 1,*/
			/* hence we decrement the first parameter to make it*/
			/* compatible with the engine's implementation*/
			ArithmeticNode* subtractNode = newNode<ArithmeticNode>(
				blr_subtract, true, yyvsp[-2].valueExprNode, MAKE_const_slong(1));

			yyval.valueExprNode = newNode<SubstringNode>(yyvsp[-4].valueExprNode, subtractNode, yyvsp[-1].valueExprNode);
		}
break;
case 1689:
  if (!yytrial)
{ yyval.valueExprNode = newNode<SubstringSimilarNode>(yyvsp[-5].valueExprNode, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1690:
  if (!yytrial)
{ yyval.valueExprNode = NULL; }
break;
case 1691:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1692:
  if (!yytrial)
{ yyval.valueExprNode = newNode<TrimNode>(yyvsp[-4].blrOp, yyvsp[-1].valueExprNode, yyvsp[-3].valueExprNode); }
break;
case 1693:
  if (!yytrial)
{ yyval.valueExprNode = newNode<TrimNode>(blr_trim_both, yyvsp[-1].valueExprNode, yyvsp[-3].valueExprNode); }
break;
case 1694:
  if (!yytrial)
{ yyval.valueExprNode = newNode<TrimNode>(yyvsp[-3].blrOp, yyvsp[-1].valueExprNode); }
break;
case 1695:
  if (!yytrial)
{ yyval.valueExprNode = newNode<TrimNode>(blr_trim_both, yyvsp[-1].valueExprNode); }
break;
case 1696:
  if (!yytrial)
{ yyval.blrOp = blr_trim_both; }
break;
case 1697:
  if (!yytrial)
{ yyval.blrOp = blr_trim_trailing; }
break;
case 1698:
  if (!yytrial)
{ yyval.blrOp = blr_trim_leading; }
break;
case 1699:
  if (!yytrial)
{ yyval.valueExprNode = newNode<UdfCallNode>(QualifiedName(*yyvsp[-3].metaNamePtr, ""), yyvsp[-1].valueListNode); }
break;
case 1700:
  if (!yytrial)
{ yyval.valueExprNode = newNode<UdfCallNode>(QualifiedName(*yyvsp[-2].metaNamePtr, ""), newNode<ValueListNode>(0)); }
break;
case 1701:
  if (!yytrial)
{ yyval.valueExprNode = newNode<UdfCallNode>(QualifiedName(*yyvsp[-3].metaNamePtr, *yyvsp[-5].metaNamePtr), yyvsp[-1].valueListNode); }
break;
case 1702:
  if (!yytrial)
{ yyval.valueExprNode = newNode<UdfCallNode>(QualifiedName(*yyvsp[-2].metaNamePtr, *yyvsp[-4].metaNamePtr), newNode<ValueListNode>(0)); }
break;
case 1703:
  if (!yytrial)
{ yyval.valueExprNode = newNode<CastNode>(yyvsp[-3].valueExprNode, yyvsp[-1].legacyField); }
break;
case 1706:
  if (!yytrial)
{
			ComparativeBoolNode* condition = newNode<ComparativeBoolNode>(blr_eql, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode);
			yyval.valueExprNode = newNode<ValueIfNode>(condition, newNode<NullNode>(), yyvsp[-3].valueExprNode);
		}
break;
case 1707:
  if (!yytrial)
{ yyval.valueExprNode = newNode<ValueIfNode>(yyvsp[-5].boolExprNode, yyvsp[-3].valueExprNode, yyvsp[-1].valueExprNode); }
break;
case 1708:
  if (!yytrial)
{ yyval.valueExprNode = newNode<CoalesceNode>(yyvsp[-1].valueListNode->addFront(yyvsp[-3].valueExprNode)); }
break;
case 1709:
  if (!yytrial)
{
			ValueListNode* list = yyvsp[-1].valueListNode;
			ValueListNode* conditions = newNode<ValueListNode>(list->items.getCount() / 2);
			ValueListNode* values = newNode<ValueListNode>(list->items.getCount() / 2);

			for (FB_SIZE_T i = 0; i < list->items.getCount(); i += 2)
			{
				conditions->items[i / 2] = list->items[i];
				values->items[i / 2] = list->items[i + 1];
			}

			yyval.valueExprNode = newNode<DecodeNode>(yyvsp[-3].valueExprNode, conditions, values);
		}
break;
case 1710:
  if (!yytrial)
{
			ValueListNode* list = yyvsp[-3].valueListNode;
			ValueListNode* conditions = newNode<ValueListNode>(list->items.getCount() / 2);
			ValueListNode* values = newNode<ValueListNode>(list->items.getCount() / 2 + 1);

			for (FB_SIZE_T i = 0; i < list->items.getCount(); i += 2)
			{
				conditions->items[i / 2] = list->items[i];
				values->items[i / 2] = list->items[i + 1];
			}

			values->items[list->items.getCount() / 2] = yyvsp[-1].valueExprNode;

			yyval.valueExprNode = newNode<DecodeNode>(yyvsp[-5].valueExprNode, conditions, values);
		}
break;
case 1711:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].decodeNode; }
break;
case 1712:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1713:
  if (!yytrial)
{ yyval.decodeNode = newNode<DecodeNode>(yyvsp[0].valueExprNode, newNode<ValueListNode>(0u), newNode<ValueListNode>(0u)); }
break;
case 1714:
yyval.valueListNode = NOTRIAL(yyvsp[0].decodeNode->conditions);
break;
case 1715:
yyval.valueListNode =  NOTRIAL(yyvsp[-1].decodeNode->values);
break;
case 1716:
  if (!yytrial)
{
				DecodeNode* node = yyval.decodeNode = yyvsp[-5].decodeNode;
				node->label = "CASE";
				if (yyvsp[-1].valueExprNode)
					node->values->add(yyvsp[-1].valueExprNode);
			}
break;
case 1717:
  if (!yytrial)
{
			yyvsp[-5].valueListNode->add(yyvsp[-2].valueExprNode);
			yyvsp[-4].valueListNode->add(yyvsp[0].valueExprNode);
		}
break;
case 1718:
  if (!yytrial)
{
			yyvsp[-6].valueListNode->add(yyvsp[-2].valueExprNode);
			yyvsp[-5].valueListNode->add(yyvsp[0].valueExprNode);
		}
break;
case 1719:
  if (!yytrial)
{ yyval.valueExprNode = NULL; }
break;
case 1720:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[0].valueExprNode; }
break;
case 1721:
  if (!yytrial)
{ yyval.valueExprNode = yyvsp[-1].valueIfNode; }
break;
case 1722:
  if (!yytrial)
{
			ValueIfNode* last = yyvsp[-3].valueIfNode;
			ValueIfNode* next;

			while ((next = last->falseValue->as<ValueIfNode>()))
				last = next;

			fb_assert(last->falseValue->is<NullNode>());

			last->falseValue = yyvsp[-1].valueExprNode;
			yyval.valueExprNode = yyvsp[-3].valueIfNode;
		}
break;
case 1723:
  if (!yytrial)
{ yyval.valueIfNode = newNode<ValueIfNode>(yyvsp[-2].boolExprNode, yyvsp[0].valueExprNode, newNode<NullNode>()); }
break;
case 1724:
  if (!yytrial)
{
			ValueIfNode* cond = newNode<ValueIfNode>(yyvsp[-2].boolExprNode, yyvsp[0].valueExprNode, newNode<NullNode>());
			ValueIfNode* last = yyvsp[-4].valueIfNode;
			ValueIfNode* next;

			while ((next = last->falseValue->as<ValueIfNode>()))
				last = next;

			fb_assert(last->falseValue->is<NullNode>());

			last->falseValue = cond;
			yyval.valueIfNode = yyvsp[-4].valueIfNode;
		}
break;
case 1728:
  if (!yytrial)
{ yyval.valueListNode = newNode<ValueListNode>(0u)->add(yyvsp[-2].valueExprNode)->add(yyvsp[0].valueExprNode); }
break;
case 1729:
  if (!yytrial)
{ yyval.valueListNode = yyvsp[-4].valueListNode->add(yyvsp[-2].valueExprNode)->add(yyvsp[0].valueExprNode); }
break;
case 1730:
  if (!yytrial)
{
			yyval.valueExprNode = newNode<GenIdNode>((client_dialect < SQL_DIALECT_V6_TRANSITION),
				*yyvsp[0].metaNamePtr, ((Jrd::ValueExprNode*) NULL), true, false);
		}
break;
case 1731:
  if (!yytrial)
{
			yyval.valueExprNode = newNode<GenIdNode>((client_dialect < SQL_DIALECT_V6_TRANSITION),
				*yyvsp[-3].metaNamePtr, yyvsp[-1].valueExprNode, false, false);
		}
break;
case 1732:
  if (!yytrial)
{ yyval.blrOp = blr_extract_year; }
break;
case 1733:
  if (!yytrial)
{ yyval.blrOp = blr_extract_month; }
break;
case 1734:
  if (!yytrial)
{ yyval.blrOp = blr_extract_day; }
break;
case 1735:
  if (!yytrial)
{ yyval.blrOp = blr_extract_hour; }
break;
case 1736:
  if (!yytrial)
{ yyval.blrOp = blr_extract_minute; }
break;
case 1737:
  if (!yytrial)
{ yyval.blrOp = blr_extract_second; }
break;
case 1738:
  if (!yytrial)
{ yyval.blrOp = blr_extract_millisecond; }
break;
case 1739:
  if (!yytrial)
{ yyval.blrOp = blr_extract_week; }
break;
case 1740:
  if (!yytrial)
{ yyval.blrOp = blr_extract_weekday; }
break;
case 1741:
  if (!yytrial)
{ yyval.blrOp = blr_extract_yearday; }
break;
case 1746:
  if (!yytrial)
{ yyval.valueExprNode = newNode<NullNode>(); }
break;
case 1747:
  if (!yytrial)
{
			dsql_fld* field = newNode<dsql_fld>();
			field->dtype = dtype_boolean;
			field->length = sizeof(UCHAR);

			CastNode* castNode = newNode<CastNode>(newNode<NullNode>(), field);
			castNode->dsqlAlias = "CONSTANT";
			yyval.valueExprNode = castNode;
		}
break;

  default:
    break;
  }

#if YYDEBUG && defined(YYDBPR)
  if (yydebug) {
    printf("yydebug[%d]: after reduction, result is ", yytrial);
    YYDBPR(yyps->val);
    printf("\n");
  }
#endif

  // Perform user-defined position reduction
#ifdef YYREDUCEPOSNFUNC
  if (!yytrial) {
    YYCALLREDUCEPOSN(YYREDUCEPOSNFUNCARG);
  }
#endif

  yyps->ssp -= yym;
  yystate = *(yyps->ssp);
  yyps->vsp -= yym;
  yyps->psp -= yym;

  yym = yylhs[yyn];
  if (yystate == 0 && yym == 0) {
#if YYDEBUG
    if (yydebug) {
      printf("yydebug[%d,%p]: after reduction, shifting from state 0 to state %d\n", 
	     int(yydepth), yytrial, YYFINAL);
    }
#endif
    yystate = YYFINAL;
    *++(yyps->ssp) = YYFINAL;
    *++(yyps->vsp) = yyps->val;
    yyretlval = yyps->val;	// return value of root non-terminal to yylval
    *++(yyps->psp) = yyps->pos;
    yyretposn = yyps->pos;	// return value of root position to yyposn
    if (yychar < 0) {
      if ((yychar = yylex1()) < 0) {
        yychar = 0;
      }
#if YYDEBUG
      if (yydebug) {
        yys = 0;
        if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
        if (!yys) yys = "illegal-symbol";
        printf("yydebug[%d,%p]: state %d, reading %d (%s)\n", 
	       int(yydepth), yytrial, YYFINAL, yychar, yys); 
      }
#endif
    }
    if (yychar == 0) goto yyaccept;
    goto yyloop;
  }

  if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
      yyn <= YYTABLESIZE && yycheck[yyn] == yystate) {
    yystate = yytable[yyn];
  } else {
    yystate = yydgoto[yym];
  }
#if YYDEBUG
  if (yydebug)
    printf("yydebug[%d,%p]: after reduction, shifting from state %d to state %d\n",
           int(yydepth), yytrial, *(yyps->ssp), yystate);
#endif
  if (yyps->ssp >= yyps->ss + yyps->stacksize - 1) {
    yyMoreStack(yyps);
  }
  *++(yyps->ssp) = yystate;
  *++(yyps->vsp) = yyps->val;
  *++(yyps->psp) = yyps->pos;
  goto yyloop;


  //
  // Reduction declares that this path is valid.
  // Set yypath and do a full parse
  //
yyvalid:
  if (yypath) {
    goto yyabort;
  }
  while (yyps->save) {
    yyparsestate *save = yyps->save;
    yyps->save = save->save;
    save->save = yypath;
    yypath = save;
  }
#if YYDEBUG
  if (yydebug)
    printf("yydebug[%d,%p]: CONFLICT trial successful, backtracking to state %d, %d tokens\n",
           int(yydepth), yytrial, yypath->state, int(yylvp - yylvals - yypath->lexeme));
#endif
  if (yyerrctx) {
    yyFreeState(yyerrctx);
    yyerrctx = NULL;
  }
  yychar = -1;
  yyps->ssp = yyps->ss + (yypath->ssp - yypath->ss);
  yyps->vsp = yyps->vs + (yypath->vsp - yypath->vs);
  yyps->psp = yyps->ps + (yypath->psp - yypath->ps);
  memcpy (yyps->ss, yypath->ss, (yyps->ssp - yyps->ss + 1) * sizeof(Yshort));
  yySCopy(yyps->vs, yypath->vs,  yyps->vsp - yyps->vs + 1);
  yyPCopy(yyps->ps, yypath->ps,  yyps->psp - yyps->ps + 1);
  yylvp = yylvals + yypath->lexeme;
  yylpp = yylpsns + yypath->lexeme;
  yylexp = yylexemes + yypath->lexeme;
  yystate = yypath->state;
  goto yyloop;


yyabort:
  if (yyerrctx) {
    yyFreeState(yyerrctx);
    yyerrctx = NULL;
  }

  YYSTYPE *pv;
  for(pv = yyps->vs; pv < yyps->vsp; pv++) {
    YYDELETEVAL(*pv, 2);
  }

  YYPOSN *pp;
  for(pp = yyps->ps; pp < yyps->psp; pp++) {
    YYDELETEPOSN(*pp, 2);
  }

  while (yyps) {
    yyparsestate *save = yyps;
    yyps = save->save;
    yyFreeState(save);
  }
  while (yypath) {
    yyparsestate *save = yypath;
    yypath = save->save;
    yyFreeState(save); 
  }
  return (1);


yyaccept:
  if (yyps->save) goto yyvalid;
  if (yyerrctx) {
    yyFreeState(yyerrctx);
    yyerrctx = NULL;
  }
  while (yyps) {
    yyparsestate *save = yyps;
    yyps = save->save;
    yyFreeState(save);
  }
  while (yypath) {
    yyparsestate *save = yypath;
    yypath = save->save;
    yyFreeState(save); 
  }
  return (0);
}


int Parser::yylex1()
{
  if (yylvp < yylve) {
    yylval = *yylvp++;
    yyposn = *yylpp++;
    return *yylexp++;
  } else {
    if (yyps->save) {
      if (yylvp == yylvlim) {
	yyexpand();
      }
      *yylexp = yylex();
      *yylvp++ = yylval;
      yylve++;
      *yylpp++ = yyposn;
      yylpe++;
      return *yylexp++;
    } else {
      return yylex();
    }
  }
}


int Parser::yyexpand()
{
  int p = yylvp - yylvals;
  int s = yylvlim - yylvals;
  s += YYSTACKGROWTH;
  { Yshort  *tl = yylexemes; 
    YYSTYPE *tv = yylvals;
    YYPOSN  *tp = yylpsns;
    yylvals = FB_NEW_POOL(getPool()) YYSTYPE[s];
    yylpsns = FB_NEW_POOL(getPool()) YYPOSN[s];
    yylexemes = FB_NEW_POOL(getPool()) Yshort[s];
    memcpy(yylexemes, tl, (s - YYSTACKGROWTH) * sizeof(Yshort));
    yySCopy(yylvals, tv, s - YYSTACKGROWTH);
    yyPCopy(yylpsns, tp, s - YYSTACKGROWTH);
    delete[] tl;
    delete[] tv;
    delete[] tp;
  }
  yylvp = yylve = yylvals + p;
  yylvlim = yylvals + s;
  yylpp = yylpe = yylpsns + p;
  yylplim = yylpsns + s;
  yylexp = yylexemes + p;
  return 0;
}

